/*****************************************************************************
 *
 * File: PADIM_Macros.h
 * Version: 1.0.0
 * Date: 10/18/21
 *
 *****************************************************************************
 * Copyright (c) 2018-2022, FieldComm Group, All Rights Reserved
 *****************************************************************************
 *
 * Description: HART related Macro Definitions for PA-DIM representation
 *
 *
 *****************************************************************************
 */
 
#define _SEMANTIC_MAP_TO_CDD(cdd_entry, dditem)\
SEMANTIC_MAP CddMap_##dditem\
{\
    cdd_entry: dditem\
}


#define _SEMANTIC_MAP_FOR_HART_UNIT(opcuaType, signalName, unitVar) \
SEMANTIC_MAP HartUnitsMap_##signalName\
{\
    opcuaType: unitVar \
    {\
      {  1, "UNIT//UNECE/4601656" /* inH2O */ },\
      {  2, "UNIT//UNECE/4601657" /* inHG */ },\
      {  3, "UNIT//UNECE/4928052" /* ftH2O */ },\
      {  4, "UNIT//UNECE/18512"   /* mmH2O */ },\
      {  5, "UNIT//UNECE/18510"   /* mmHg */ },\
      {  6, "UNIT//UNECE/20563"   /* psi */ },\
      {  7, "UNIT//UNECE/4342098" /* bar */ },\
      {  8, "UNIT//UNECE/5063250" /* mbar */ },\
      {  9, "UNIT//UNECE/12853"   /* g/cm */ },\
      { 10, "UNIT//UNECE/17461"   /* kg/cm */ },\
      { 11, "UNIT//UNECE/5259596" /* Pa */ },\
      { 12, "UNIT//UNECE/4935745" /* kPa */ },\
      { 13, "UNIT//UNECE/21825"   /* Torr */ },\
      { 14, "UNIT//UNECE/4281421" /* atm */ },\
      { 15, "UNIT//UNECE/12876"   /* ft/min */ },\
      { 17, "UNIT//UNECE/19506"   /* l/min */ },\
      { 19, "UNIT//UNECE/5067080" /* m/h */ },\
      { 20, "UNIT//UNECE/18003"   /* ft/s */ },\
      { 21, "UNIT//UNECE/5067859" /* m/s */ },\
      { 24, "UNIT//UNECE/4666673" /* l/s */ },\
      { 26, "UNIT//UNECE/4534583" /* ft/s */ },\
      { 27, "UNIT//UNECE/4928050" /* ft/d */ },\
      { 28, "UNIT//UNECE/5067091" /* m/s */ },\
      { 29, "UNIT//UNECE/4666674" /* m/d */ },\
      { 30, "UNIT//UNECE/4928055" /* gal(UK)/h */ },\
      { 31, "UNIT//UNECE/4928054" /* gal(UK)/d */ },\
      { 32, "UNIT//UNECE/4408652" /* C */ },\
      { 33, "UNIT//UNECE/4604232" /* F */ },\
      { 34, "UNIT//UNECE/4273208" /* R */ },\
      { 35, "UNIT//UNECE/4932940" /* K */ },\
      { 36, "UNIT//UNECE/12890"   /* mv */ },\
      { 37, "UNIT//UNECE/5195853" /* Ohm */ },\
      { 38, "UNIT//UNECE/4740186" /* Hz */ },\
      { 39, "UNIT//UNECE/13387"   /* mA */ },\
      { 40, "UNIT//UNECE/4672588" /* gal(US) */ },\
      { 41, "UNIT//UNECE/5002322" /* l */ },\
      { 42, "UNIT//UNECE/4672585" /* gal(UK) */ },\
      { 43, "UNIT//UNECE/5067857" /* m */ },\
      { 44, "UNIT//UNECE/4607828" /* ft */},\
      { 45, "UNIT//UNECE/5067858" /* m */},\
      { 46, "UNIT//UNECE/4344908" /* bbl */ },\
      { 47, "UNIT//UNECE/4804168" /* in */},\
      { 48, "UNIT//UNECE/4410708" /* cm */ },\
      { 49, "UNIT//UNECE/5066068" /* mm */},\
      { 50, "UNIT//UNECE/5065038" /* min */},\
      { 51, "UNIT//UNECE/5457219" /* s */},\
      { 52, "UNIT//UNECE/4740434" /* h */},\
      { 53, "UNIT//UNECE/4473177" /* d */},\
      { 54, "UNIT//UNECE/13379"   /* cSt */},\
      { 55, "UNIT//UNECE/17207"   /* cP */},\
      { 56, "UNIT//UNECE/4340025" /* S */},\
      { 57, "UNIT//UNECE/20529"   /* % */},\
      { 58, "UNIT//UNECE/5655636" /* V */},\
      { 59, "UNIT//UNECE/4405043" /* ph */},\
      { 60, "UNIT//UNECE/4674125" /* g */},\
      { 61, "UNIT//UNECE/4933453" /* kg */},\
      { 62, "UNIT//UNECE/5525061" /* t */},\
      { 63, "UNIT//UNECE/4997714" /* lb */},\
      { 65, "UNIT//UNECE/5002318" /* ton(UK) */},\
      { 66, "UNIT//UNECE/4732465" /* mS/cm */},\
      { 67, "UNIT//UNECE/4666418" /* S/cm */},\
      { 68, "UNIT//UNECE/5129559" /* N */},\
      { 69, "UNIT//UNECE/4869973" /* J */},\
      { 70, "UNIT//UNECE/4600377" /* g/s */},\
      { 71, "UNIT//UNECE/4600376" /* g/min */},\
      { 72, "UNIT//UNECE/4600375" /* g/h */},\
      { 73, "UNIT//UNECE/4933459" /* kg/s */},\
      { 74, "UNIT//UNECE/19276"   /* kg/m */},\
      { 75, "UNIT//UNECE/4536627" /* kg/h */},\
      { 76, "UNIT//UNECE/4600624" /* kg/d */},\
      { 77, "UNIT//UNECE/4994872" /* t/min */},\
      { 78, "UNIT//UNECE/4534584" /* t/h */},\
      { 79, "UNIT//UNECE/4994865" /* t/d */},\
      { 80, "UNIT//UNECE/4929585" /* lb/s */},\
      { 81, "UNIT//UNECE/4929336" /* lb/min */},\
      { 82, "UNIT//UNECE/13397"   /* lb/h */},\
      { 83, "UNIT//UNECE/4929078" /* lb/d */},\
      { 86, "UNIT//UNECE/4995128" /* ton(US)/d */},\
      { 88, "UNIT//UNECE/4995125" /* ton(UK)/d */},\
      { 91, "UNIT//UNECE/12851"   /* g/cm */},\
      { 92, "UNIT//UNECE/4934993" /* kg/m */},\
      { 93, "UNIT//UNECE/18245"   /* lb/gal (US) */},\
      { 94, "UNIT//UNECE/14391"   /* lb/ft */},\
      { 95, "UNIT//UNECE/18250"   /* g/ml */},\
      { 96, "UNIT//UNECE/4338485" /* kg/l */},\
      { 97, "UNIT//UNECE/18252"   /* g/l */},\
      { 98, "UNIT//UNECE/19521"   /* lb/in */},\
      { 99, "UNIT//UNECE/4995379" /* ton/yd */},\
      {100, "UNIT//UNECE/4862769" /* Tw */},\
      {101, "UNIT//UNECE/4862264" /* Bx */},\
      {102, "UNIT//UNECE/4862261" /* B (US heavy) */},\
      {103, "UNIT//UNECE/4862262" /* B (US light) */},\
      {104, "UNIT//UNECE/4862259" /* API */},\
      {107, "UNIT//UNECE/4862263" /* Balling */},\
      {110, "UNIT//UNECE/4347201" /* bu (US) */},\
      {111, "UNIT//UNECE/5850193" /* yd */},\
      {112, "UNIT//UNECE/4609105" /* ft */},\
      {113, "UNIT//UNECE/4804177" /* in */},\
      {114, "UNIT//UNECE/18773"   /* in/s */},\
      {115, "UNIT//UNECE/5060147" /* in/min */},\
      {116, "UNIT//UNECE/18002"   /* ft/min */},\
      {117, "UNIT//UNECE/4536630" /* /s */},\
      {118, "UNIT//UNECE/5394515" /* rps */},\
      {119, "UNIT//UNECE/5394509" /* rpm */},\
      {120, "UNIT//UNECE/5060144" /* m/h */},\
      {124, "UNIT//UNECE/4344900" /* bbl (US) */},\
      {125, "UNIT//UNECE/5197402" /* oz */},\
      {126, "UNIT//UNECE/14389"   /* ft-lbf */},\
      {127, "UNIT//UNECE/4937556" /* kW */},\
      {128, "UNIT//UNECE/4937544" /* kWh */},\
      {130, "UNIT//UNECE/12875"   /* ft/h */},\
      {131, "UNIT//UNECE/4666675" /* m/min */},\
      {135, "UNIT//UNECE/16945"   /* bbl/d */},\
      {136, "UNIT//UNECE/4666672" /* gal/h */},\
      {137, "UNIT//UNECE/4928056" /* gal(UK)/s */},\
      {138, "UNIT//UNECE/4535090" /* l/h */},\
      {139, "UNIT//UNECE/13625"   /* ppm */},\
      {142, "UNIT//UNECE/12873"   /* Btu/h */},\
      {143, "UNIT//UNECE/17476"   /*  */},\
      {144, "UNIT//UNECE/4405297" /* rad */},\
      {145, "UNIT//UNECE/5124409" /* inH2O (60 F) */},\
      {146, "UNIT//UNECE/4731449" /* g/l */},\
      {147, "UNIT//UNECE/18257"   /* g/m */},\
      {152, "UNIT//UNECE/5124665" /* ft/lb */},\
      {153, "UNIT//UNECE/13396"   /* pF */},\
      {154, "UNIT//UNECE/4993337" /* ml/l */},\
      {155, "UNIT//UNECE/4862774" /* l/l */},\
      {156, "UNIT//UNECE/12878"   /* dB */},\
      {160, "UNIT//UNECE/5262401" /* P */},\
      {163, "UNIT//UNECE/4338745" /* kOhm */},\
      {164, "UNIT//UNECE/13122"   /* MJ */},\
      {165, "UNIT//UNECE/4862777" /* Btu */},\
      {169, "UNIT//UNECE/13873"   /* ppb */},\
      /* 170 - 219 depend on device variable classification */ \
      {235, "UNIT//UNECE/18242"   /* gal/d */},\
      {236, "UNIT//UNECE/4738132" /* hl */},\
      {237, "UNIT//UNECE/5066817" /* MPa */},\
      {238, "UNIT//UNECE/5124408" /* inH2O (4 C) */}\
      /* 240 - 249 manufacturer specific units */ \
    }\
}
 
/*
 * PA-DIM standard process signal definition
 */

#define PROCESS_SIGNAL_MAP(signalName) \
SEMANTIC_MAP AnalogSignalMap_##signalName \
{ \
    "OPC-UA//__PADIM_/AnalogSignalType" : AnalogSignalType_##signalName \
}

#define ANALOG_SIGNAL_TYPE(signalName, refSignalTag) \
  ANALOG_SIGNAL_TYPE_BASE(signalName, refSignalTag, AnalogSignalVariableType_##signalName)

#define ANALOG_SIGNAL_TYPE_BASE(signalName, refSignalTag, refAnalogSignalVariableType) \
COLLECTION AnalogSignalType_##signalName \
{ \
  LABEL #signalName; \
  MEMBERS \
  { \
    /*mandatory property */ \
    __PADIM_SignalTag,             refSignalTag; \
    __PADIM_AnalogSignal,          refAnalogSignalVariableType; \
    \
    /*optional propoerty */ \
    /*__PADIM_ZeroPointAdjustment, << fill this in >>; */ \
  } \
}

//Extending for optional properties, it is possible to extend with predefined names for a certain DD and use "signalName" or "devVarIndex" to concatinated.
//Definition is outside of macro.
#define PROCESS_SIGNAL_COL_BASE_MEMBER(signalName, devVarIndex) \
    /*Mandatory properties */ \
    __UA_Value,                 deviceVariables[devVarIndex].DIGITAL_VALUE; \
    __UA_EURange,               colEURange_##signalName; \
    __UA_EngineeringUnits,      colEngineeringUnit_##signalName; \
    /*Optional properties */ \
    __UA_InstrumentRange,       colInstrumentRange_##signalName; \
    /*__PADIM_ActualValue,      << fill this in >>;*/ \
    /*__PADIM_SimulationValue,  << fill this in >>;*/ \
    __PADIM_SimulationState,    deviceVariables[devVarIndex].SIMULATED; \
    __PADIM_Damping,            deviceVariables[devVarIndex].DAMPING_VALUE;
    
#define PROCESS_SIGNAL_COL(signalName, signalLabel, devVarIndex ) \
SEMANTIC_MAP AnalogSignalVariableTypeMap_##signalName \
{ \
    "OPC-UA//__PADIM_/AnalogSignalVariableType" : AnalogSignalVariableType_##signalName \
}\
COLLECTION AnalogSignalVariableType_##signalName \
{ \
  LABEL signalLabel ; \
  MEMBERS \
  { \
    PROCESS_SIGNAL_COL_BASE_MEMBER(signalName, devVarIndex) \
  } \
}

#define ENGINEERING_UNIT_COL(signalName,devVarIndex) \
/* Implicitly mapped to the __UA_ structure 'EUInformation' */ \
COLLECTION colEngineeringUnit_##signalName \
{ \
  LABEL "Engineering Unit"; \
  MEMBERS \
  { \
    __UA_unitId,   deviceVariables[devVarIndex].DIGITAL_UNITS; \
    /* PA DIM server is responsible to fill other properies based on unit id. */ \
  } \
}\
_SEMANTIC_MAP_FOR_HART_UNIT("OPC-UA//__UA_/MultiStateDictionaryEntryDiscreteBaseType", signalName, deviceVariables[devVarIndex].DIGITAL_UNITS)

#define EURANGE_COL(signalName,refLOW, refHIGH) \
/* Implicitly mapped to the __UA_ structure 'Range' */ \
COLLECTION colEURange_##signalName \
{ \
  LABEL "EU range"; \
  MEMBERS \
  { \
    __UA_low,    refLOW; \
    __UA_high,   refHIGH; \
  } \
} 

#define INSTRUMENT_RANGE_COL(signalName, devVarIndex) \
/* Implicitly mapped to the __UA_ structure 'Range' */ \
COLLECTION colInstrumentRange_##signalName \
{ \
  LABEL "Instrument range"; \
  MEMBERS \
  { \
    __UA_low,   deviceVariables[devVarIndex].LOWER_SENSOR_LIMIT; \
    __UA_high,  deviceVariables[devVarIndex].UPPER_SENSOR_LIMIT; \
  } \
}

#define PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  PROCESS_SIGNAL_MAP(signalName) \
  ANALOG_SIGNAL_TYPE(signalName, refSignalTag) \
  PROCESS_SIGNAL_COL(signalName, signalLabel, devVarIndex) \
  ENGINEERING_UNIT_COL(signalName, devVarIndex) \
  EURANGE_COL(signalName, refLOW, refHIGH) \
  INSTRUMENT_RANGE_COL(signalName, devVarIndex)
  
#define PROCESS_SIGNAL_BASE(signalName, signalLabel, refSignalTag, refAnalogSignalVariableType, devVarIndex, refLOW, refHIGH) \
  PROCESS_SIGNAL_MAP(signalName) \
  ANALOG_SIGNAL_TYPE_BASE(signalName, refSignalTag, refAnalogSignalVariableType) \
  ENGINEERING_UNIT_COL(signalName, devVarIndex) \
  EURANGE_COL(signalName, refLOW, refHIGH) \
  INSTRUMENT_RANGE_COL(signalName, devVarIndex)

/*
Concrete analog signal variable types
*/
#define LevelMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  \
  SEMANTIC_MAP LevelMeasurementMap_##signalName \
  { \
    "OPC-UA//__PADIM_/LevelMeasurementVariableType" : AnalogSignalVariableType_##signalName \
  }
  
#define AnalyticalMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  \
  SEMANTIC_MAP AnalyticalMeasurementMap_##signalName \
  { \
    "OPC-UA//__PADIM_/AnalyticalMeasurementVariableType" : AnalogSignalVariableType_##signalName \
  }
  
#define ActualDensityMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  \
  SEMANTIC_MAP ActualDensityMeasurementMap_##signalName \
  { \
    "OPC-UA//__PADIM_/ActualDensityVariableType" : AnalogSignalVariableType_##signalName \
  }
  
#define PressureMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH) \
  \
  SEMANTIC_MAP PressureMeasurementMap_##signalName \
  { \
    "OPC-UA//__PADIM_/PressureMeasurementVariableType" : AnalogSignalVariableType_##signalName \
  }
  
#define TemperatureMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refSensorType) \
  PROCESS_SIGNAL_BASE(signalName, signalLabel, refSignalTag, TemperatureMeasurementVariableType_##signalName, devVarIndex, refLOW, refHIGH) \
  \
  COLLECTION TemperatureMeasurementVariableType_##signalName \
  { \
    LABEL signalLabel ; \
    MEMBERS \
    { \
      PROCESS_SIGNAL_COL_BASE_MEMBER(signalName, devVarIndex) \
      /*Mandatory properties */ \
      __PADIM_SensorType, refSensorType; \
      \
      /*Optional properties */ \
      /* __PADIM_SensorConnection, locSensorConnection;*/ \
      /* __PADIM_SensorReference, locSensorReference;*/ \
    } \
  } \
  \
  SEMANTIC_MAP TemperatureMeasurementMap_##signalName \
  { \
    "OPC-UA//__PADIM_/TemperatureMeasurementVariableType" : TemperatureMeasurementVariableType_##signalName \
  }
  
#define  FlowMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refLowFlowCutOff) \
  PROCESS_SIGNAL_BASE(signalName, signalLabel, refSignalTag, FlowMeasurementVariableType_##signalName, devVarIndex, refLOW, refHIGH) \
  \
  COLLECTION FlowMeasurementVariableType_##signalName \
  { \
    LABEL signalLabel ; \
    MEMBERS \
    { \
      PROCESS_SIGNAL_COL_BASE_MEMBER(signalName, devVarIndex) \
      /*Mandatory properties */ \
      __PADIM_LowFlowCutOff, refLowFlowCutOff; \
      \
      /*Optional properties */ \
      /* __PADIM_FlowDirection, locFlowDirection;*/ \
    } \
  } \
  \
  SEMANTIC_MAP FlowMeasurementMap_##signalName \
  { \
      "OPC-UA//__PADIM_/FlowMeasurementVariableType" : FlowMeasurementVariableType_##signalName \
  }
  
#define  MassFlowRate_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refLowFlowCutOff) \
  FlowMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refLowFlowCutOff) \
  \
  SEMANTIC_MAP MassFlowRateMap_##signalName \
  { \
    "OPC-UA//__PADIM_/MassFlowRateVariableType" : FlowMeasurementVariableType_##signalName \
  }
  
#define  ActualVolumeFlowRate_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refLowFlowCutOff) \
  FlowMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refLowFlowCutOff) \
  \
  SEMANTIC_MAP ActualVolumeFlowRateMap_##signalName \
  { \
    "OPC-UA//__PADIM_/ActualVolumeFlowRateVariableType" : FlowMeasurementVariableType_##signalName \
  }
  
#define  NormalizedVolumeFlowRate_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refLowFlowCutOff) \
  FlowMeasurement_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refLowFlowCutOff) \
  \
  SEMANTIC_MAP NormalizedVolumeFlowRateMap_##signalName \
  { \
    "OPC-UA//__PADIM_/NormalizedVolumeFlowRateVariableType" : FlowMeasurementVariableType_##signalName \
  }
  
#define  Totalizer_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refPulseValue, refPulseWidth) \
  PROCESS_SIGNAL_BASE(signalName, signalLabel, refSignalTag, TotalizerVariableType_##signalName, devVarIndex, refLOW, refHIGH) \
  \
  COLLECTION TotalizerVariableType_##signalName \
  { \
    LABEL signalLabel ; \
    MEMBERS \
    { \
      PROCESS_SIGNAL_COL_BASE_MEMBER(signalName, devVarIndex) \
      /*Mandatory properties */ \
      __PADIM_PulseValue, refPulseValue; \
      __PADIM_PulseWidth, refPulseWidth; \
    } \
  } \
  \
  SEMANTIC_MAP TotalizerMap_##signalName \
  { \
    "OPC-UA//__PADIM_/TotalizerVariableType" : TotalizerVariableType_##signalName \
  }

#define  Control_PROCESS_SIGNAL(signalName, signalLabel, refSignalTag, devVarIndex, refLOW, refHIGH, refSetpoint, refOperatingDirection, refActuatorType) \
  PROCESS_SIGNAL_BASE(signalName, signalLabel, refSignalTag, ControlVariableType_##signalName, devVarIndex, refLOW, refHIGH) \
  \
  COLLECTION ControlVariableType_##signalName \
  { \
    LABEL signalLabel ; \
    MEMBERS \
    { \
      PROCESS_SIGNAL_COL_BASE_MEMBER(signalName, devVarIndex) \
      /*Mandatory properties */ \
      __PADIM_Setpoint, refSetpoint; \
      __PADIM_OperatingDirection, refOperatingDirection; \
      __PADIM_ActuatorType, refActuatorType; \
    } \
  } \
  \
  SEMANTIC_MAP ControlMap_##signalName \
  { \
    "OPC-UA//__PADIM_/ControlVariableType" : ControlVariableType_##signalName \
  }


#define DeviceHealthAlarms(alarmMessageVar) \
  MENU menuDeviceHealthAlarms \
  { \
    LABEL "DeviceHealthAlarms";\
    STYLE WINDOW;\
    ITEMS \
    { \
      colAlarm_Failure, \
      colAlarm_CheckFunction, \
      colAlarm_OffSpec, \
      colAlarm_MaintenanceRequired \
    } \
  } \
  \
  AlarmCollection(Failure, "OPC-UA//__DI_/FailureAlarmType", alarmMessageVar) \
  AlarmCollection(CheckFunction, "OPC-UA//__DI_/CheckFunctionAlarmType", alarmMessageVar) \
  AlarmCollection(OffSpec, "OPC-UA//__DI_/OffSpecAlarmType", alarmMessageVar) \
  AlarmCollection(MaintenanceRequired, "OPC-UA//__DI_/MaintenanceRequiredAlarmType", alarmMessageVar) \
  \

#define AlarmCollection(alarmName, semantic, alarmMessageVar) \
SEMANTIC_MAP AlarmMap_##alarmName\
{\
  semantic: colAlarm_##alarmName\
}\
COLLECTION colAlarm_##alarmName \
{ \
  LABEL #alarmName; \
  MEMBERS \
  { \
    /* AlarmConditionType */ \
    __UA_ActiveState, locAlarmState_##alarmName; /*True, False - 0 is False, Non Zero is True*/ \
    \
    /* ConditionType */ \
    /*__UA_EnabledState, << fill this in >>;*/ \
  \
    /* BaseEventType */ \
    __UA_Message, alarmMessageVar; /*Static MAP via BIT_ENUMERATION*/ \
  } \
} \
\
VARIABLE locAlarmState_##alarmName \
{ \
    CLASS LOCAL;\
    TYPE ENUMERATED(1) \
    { \
        { 0, "FALSE" }, \
        { 1, "TRUE" } \
    } \
    DEFAULT_VALUE 0; \
} \

