/*****************************************************************************
 *
 * File: Common12_2_Analog_Channels.ddl
 * Version: 12.02.1
 * Date: 1/14/2022
 *
 *****************************************************************************
 * Copyright (c) 2018-2022, FieldComm Group, All Rights Reserved
 *****************************************************************************
 *
 * Description: This file contains the DDL based model of the Subset of items
 *      necessary to implement Analog Channel Commands from the Common Practice
 *      Command Specification Rev.12.0.  There were significant changes in this
 *      model reflecting the increasing focus on the device variable. This
 *      resulted in a hierachal model that matches the one taught in the
 *      Writing Device Description Workshop and the Command Summary
 *      Specification.
 *
 * Source Specifications (These are revision numbers of the protocol spec which
 *                        may differ from the standard DD revision numbers.)
 *      Command Practice Command Specification      Rev.12.0
 *      Command Summary Specification               Rev.10.1
 *      Universal Command Specification             Rev.7.2
 *      Common Tables Specification                 Rev.25.0
 *
 *****************************************************************************
 */

/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
MANUFACTURER  STANDARD, DEVICE_TYPE _COMMON_PRACTICE_ANALOG_CHANNELS, DEVICE_REVISION 12, DD_REVISION 2
 */
MANUFACTURER  STANDARD, DEVICE_TYPE 26, DEVICE_REVISION 12, DD_REVISION 2

/*****************************************************************************
 * Include Files
 *****************************************************************************
 */
#include "macros2.h"
#include "embedded_strings.h"
#include "methods.h"

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */
#define UPDATE_IN_PROGRESS      8
#define DEVICE_BUSY             32
#define MAXIMUM_RESET_RETRIES   10

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.2
 * PV Range Commands.  The commands affect the scaling of the digital value
 * to/from the loop current
 *
 * Commands 35, 36, 37
 */

/*
 *****************************************************************************
 * Write PV Range Values  changes the scaling relationship between the loop
 * current and the PV Digital Value
 *
 *NOTE: As per the Protocol Specification the Engineering Units Can be different
 *  from the PV Units.  The Units sent to the device must not affect the PV
 *  Units Use Command 44 for this purpose
 */
COMMAND write_pv_range_values
{
    NUMBER 35;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.RANGING.RANGE_UNITS (INFO), PV.RANGING.UPPER_RANGE_VALUE, PV.RANGING.LOWER_RANGE_VALUE
        }
        REPLY
        {
            response_code, device_status,
            PV.RANGING.RANGE_UNITS (INFO), PV.RANGING.UPPER_RANGE_VALUE, PV.RANGING.LOWER_RANGE_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
        9,  DATA_ENTRY_ERROR,   lower_range_value_too_high_temp;
//Remove new dictionary references from legacy binary format
        10, DATA_ENTRY_ERROR,   lower_range_value_too_low_temp;
//Remove new dictionary references from legacy binary format
        11, DATA_ENTRY_ERROR,   upper_range_value_too_high_temp;
//Remove new dictionary references from legacy binary format
        12, DATA_ENTRY_ERROR,   upper_range_value_too_low_temp;
//Remove new dictionary references from legacy binary format
        13, DATA_ENTRY_ERROR,   URV_and_LRV_out_of_limits_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, span_too_small_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        18, DATA_ENTRY_ERROR,   invalid_units_temp;
//Remove new dictionary references from legacy binary format
        29, DATA_ENTRY_ERROR,   invalid_span_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

/*
 *****************************************************************************
 * This Command sets the URV to the current conditions.  Sets the 100 Percent
 * Range operating point
 */
COMMAND set_pv_upper_range_value
{
    NUMBER 36;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
        9,  PROCESS_ERROR,      applied_process_too_high_temp;
//Remove new dictionary references from legacy binary format
        10, PROCESS_ERROR,      applied_process_too_low_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, span_too_small_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        29, DATA_ENTRY_ERROR,   invalid_span_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

/*
 *****************************************************************************
 * This Command sets the LRV to the current conditions.  Sets the 0 Percent
 * Range operating point
 */
COMMAND set_pv_lower_range_value
{
    NUMBER 37;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  PROCESS_ERROR,      applied_process_too_high_temp;
//Remove new dictionary references from legacy binary format
        10, PROCESS_ERROR,      applied_process_too_low_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, new_LRV_pushed_URV_over_USL_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        29, DATA_ENTRY_ERROR,   invalid_span_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

METHOD applied_rerange
{
//Remove new dictionary references from legacy binary format
    LABEL Apply_Values_temp;
//Remove new dictionary references from legacy binary format
    HELP  Applied_Values_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long varid[3];
        int choice;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        varid[0] = VARID(PV.DEVICE_VARIABLE.DIGITAL_VALUE);
        varid[1] = VARID(PV.DEVICE_VARIABLE.DIGITAL_UNITS);

        while (1)
        {
            get_dictionary_string(set_the, disp_string1, slen1);
            get_dictionary_string(choice_4_20_exit, disp_string2, slen2);

            switch (SELECT_FROM_LIST(disp_string1, disp_string2))
            {
            case 0:
                get_dictionary_string(apply_new_4ma_input, disp_string1,slen1);
                ACKNOWLEDGE(disp_string1);

                do
                {
                    send_command(1);

                    get_dictionary_string(current_applied_process_val, disp_string1, slen1);
                    get_dictionary_string(set_4ma_read_new_leave, disp_string2, slen2);

                    choice = select_from_list(disp_string1, varid, disp_string2);

                } while (choice == 1);

                if (choice == 0)
                {
                    send(37, status);

                    if (status[STATUS_RESPONSE_CODE])
                    {
                        display_response_status(37, status[STATUS_RESPONSE_CODE]);
                    }

                    send_command(15);
                }
                break;

            case 1:
                get_dictionary_string(apply_new_20ma_input, disp_string1, slen1);
                ACKNOWLEDGE(disp_string1);

                do
                {
                    send_command(1);

                    get_dictionary_string(current_applied_process_val, disp_string1, slen1);
                    get_dictionary_string(set_20ma_read_new_leave, disp_string2, slen2);

                    choice = select_from_list(disp_string1, varid,disp_string2);

                } while (choice == 1);

                if (choice == 0)
                {
                    send(36,status);

                    if (status[STATUS_RESPONSE_CODE])
                    {
                        display_response_status(36, status[STATUS_RESPONSE_CODE]);
                    }

                    send_command(15);
                }
                break;

            case 2:
                process_abort();

            default:
                process_abort();
            }
        }
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.3
 * Loop Current Commands
 *
 * Commands 40, 45, 46
 */

COMMAND enter_exit_fixed_pv_current_mode
{
    NUMBER 40;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR, Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        11, MODE_ERROR,       in_multidrop_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND trim_pv_current_dac_zero
{
    NUMBER 45;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR, Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  MODE_ERROR,       not_in_proper_current_mode_temp;
//Remove new dictionary references from legacy binary format
        11, MODE_ERROR,       in_multidrop_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND trim_pv_current_dac_gain
{
    NUMBER 46;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR, Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  MODE_ERROR,       not_in_proper_current_mode_temp;
//Remove new dictionary references from legacy binary format
        11, MODE_ERROR,       in_multidrop_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

METHOD transmitter_loop_test
{
//Remove new dictionary references from legacy binary format
    LABEL force_loop_current_temp;
//Remove new dictionary references from legacy binary format
    HELP  force_loop_current_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int first_time;
        int choice;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        /*
         * Here comes the standard part for analog outputs, e.g. transmitters
         */
        first_time = 1;

        do
        {
            get_dictionary_string(choose_loop_current_level, disp_string1, slen1);
            get_dictionary_string(choices_4_20_other_end, disp_string2, slen2);

            choice = SELECT_FROM_LIST(disp_string1, disp_string2);

            switch (choice)
            {
            case 0:
                assign_float(PV.DAQ.ANALOG_VALUE,4.0);
                break;

            case 1:
                assign_float(PV.DAQ.ANALOG_VALUE,20.0);
                break;

            case 2:
                get_dictionary_string(loop_output, disp_string1, slen1);
                GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);
                break;

            case 3:
            default:
                /* If loop has been fixed, an abort method has already been
                   added to cause the fixed output to return to normal. */
                process_abort();
                break;
            }

            if (first_time)
            {
               add_abort_method(leave_fixed_current_mode);
               remove_abort_method(return_to_normal);
               first_time = 0;
            }

            send(40, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                /*
                 * Display the resonse code from the error
                 */
                display_response_status(40, status[STATUS_RESPONSE_CODE]);

                /*
                 * If the response code is either 3 or 4, meaning the value
                 * was either too large or too small, do not abort method.
                 * Allow the user to try again.
                 */
                if ((status[STATUS_RESPONSE_CODE] != 3)
                    && (status[STATUS_RESPONSE_CODE] != 4))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
            }
            else
            {
                var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);
                get_dictionary_string(fld_dev_otpt_fixed, disp_string1, slen1);
                acknowledge(disp_string1, var_id);
            }
        } while (choice < 3);

        process_abort();
    }
}

METHOD transmitter_dac_trim
{
//Remove new dictionary references from legacy binary format
    LABEL DAC_Trim_temp;
//Remove new dictionary references from legacy binary format
    HELP  DAC_trim_help_temp;
/*  APPHELP "This type of calibration is usually used to compensate for"
            " drifting or aging output electronics"; */
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(connect_reference_meter, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(setting_fld_dev_4_ma, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        assign_float(PV.DAQ.ANALOG_VALUE,4.0);

        add_abort_method(leave_fixed_current_mode);
        remove_abort_method(return_to_normal);

        send(40, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(40,status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);

        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(45, status);

            assign_float(PV.DAQ.ANALOG_VALUE, 4.0); /* Put the previous value back */

            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_otpt_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        get_dictionary_string(setting_fld_dev_20_ma, disp_string1,slen1);
        ACKNOWLEDGE(disp_string1);

        assign_float(PV.DAQ.ANALOG_VALUE, 20.0);

        send(40, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(40, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(46,status);

            assign_float(PV.DAQ.ANALOG_VALUE, 20.0); /* Put the previous value back */

            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_otpt_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        process_abort();
    }
}

METHOD valve_adc_trim
{
//Remove new dictionary references from legacy binary format
    LABEL AD_trim_temp;
//Remove new dictionary references from legacy binary format
    HELP  AD_trim_help_temp;
/*  APPHELP "This type of calibration is usually used to compensate for"
            " drifting or aging output electronics"; */
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[77];
        char disp_string2[60];
        long var_id[1];
        int retry;
        int slen1;
        int slen2;
        float current;

        slen1 = 77;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(connect_reference_meter, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);

        /*
         * 4 mA test
         */
        current = 4.0;

        send(2, status); /* Test the device to see if all is ok */

        if ((status[STATUS_RESPONSE_CODE] != 0) && (status[STATUS_RESPONSE_CODE] != 8))
        {
            display_response_status(2, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        get_dictionary_string(apply_current_input_to_value, disp_string1, slen1);
        acknowledge(disp_string1, var_id);

        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(45, status);

            assign_float(PV.DAQ.ANALOG_VALUE, 4.0);

            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_current_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        /*
         * 20 mA test
         */
        current = 20.0;

        send(2, status); /* Test the device to see if all is ok */

        if ((status[STATUS_RESPONSE_CODE] != 0) && (status[STATUS_RESPONSE_CODE] != 8))
        {
            display_response_status(2, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        get_dictionary_string(apply_current_input_to_value, disp_string1, slen1);
        acknowledge(disp_string1, var_id);

        do
        {
            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE);

            send(46, status);

            assign_float(PV.DAQ.ANALOG_VALUE, 20.0);

            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_current_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        process_abort();
    }
}

METHOD leave_fixed_current_mode
{
//Remove new dictionary references from legacy binary format
    LABEL Leaving_Fixed_Current_Mode_temp;
//Remove new dictionary references from legacy binary format
    HELP  leaving_fixed_current_mode_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        long var_id[1];
        int slen1;

        slen1 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        assign_float(PV.DAQ.ANALOG_VALUE, 0.0);

        do
        {
            get_dictionary_string(return_device_to_original_output, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(40, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(40, status[STATUS_RESPONSE_CODE]);
            }

        } while (status[STATUS_RESPONSE_CODE]);

        get_dictionary_string(loop_may_be_returned_to_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}


COMMAND write_pv_alarm_code
{
    NUMBER 100;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ALARM_CODE
        }
        REPLY
        {
            response_code, device_status, PV.DAQ.ALARM_CODE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.14
 * Analog channel Commands
 *
 * Commands 60-70
 */

/*
 *****************************************************************************
 * Variable  analog_channel_number_code is used to specify the analog channel
 * for the following commands
 *
 * NOTE - Index = 0 is Primary Variable
 */
VARIABLE analog_channel_number_code
{
//Remove new dictionary references from legacy binary format
    LABEL analog_channel_number_temp;
//Remove new dictionary references from legacy binary format
    HELP  analog_channel_number_help_temp;
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

COMMAND read_analog_output_and_percent_of_range
{
    NUMBER 60;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.PERCENT_RANGE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        8,  MISC_WARNING,     update_failure_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
    }
}

VARIABLE analog_channel_number_code_1
{
//Remove new dictionary references from legacy binary format
    LABEL analog_channel_number_1_temp;
//Remove new dictionary references from legacy binary format
    HELP  analog_channel_number_1_help_temp;
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #0 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

VARIABLE analog_channel_number_code_2
{
//Remove new dictionary references from legacy binary format
    LABEL analog_channel_number_2_temp;
//Remove new dictionary references from legacy binary format
    HELP  analog_channel_number_2_help_temp;
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #1 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

VARIABLE analog_channel_number_code_3
{
//Remove new dictionary references from legacy binary format
    LABEL analog_channel_number_3_temp;
//Remove new dictionary references from legacy binary format
    HELP  analog_channel_number_3_help_temp;
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #2 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

VARIABLE analog_channel_number_code_4
{
//Remove new dictionary references from legacy binary format
    LABEL analog_channel_number_4_temp;
//Remove new dictionary references from legacy binary format
    HELP  analog_channel_number_4_help_temp;
/*  APPHELP "This variable is used to access more variables that relate to"
            " that Analog Output index."
            " NOTE - Index = 0 is not valid"
            " HART_VAR:SLOT #3 ANALOG OUTPUT NUMBER CODE"; */
    CLASS LOCAL;
    TYPE INDEX dynamic_variables;
}

COMMAND read_analog_channels
{
    NUMBER 62;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code_1 (INFO,INDEX),
            analog_channel_number_code_2 (INFO,INDEX),
            analog_channel_number_code_3 (INFO,INDEX),
            analog_channel_number_code_4 (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code_1 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_1].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_1].DAQ.ANALOG_VALUE,
            analog_channel_number_code_2 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_2].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_2].DAQ.ANALOG_VALUE,
            analog_channel_number_code_3 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_3].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_3].DAQ.ANALOG_VALUE,
            analog_channel_number_code_4 (INFO,INDEX),
            dynamic_variables[analog_channel_number_code_4].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code_4].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        8,  MISC_WARNING,     update_failure_temp;
    }
}

COMMAND read_analog_channel_information
{
    NUMBER 63;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ALARM_CODE,
            dynamic_variables[analog_channel_number_code].RANGING.TRANSFER_FUNCTION,
            dynamic_variables[analog_channel_number_code].RANGING.RANGE_UNITS,
            dynamic_variables[analog_channel_number_code].RANGING.UPPER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.LOWER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_DAMPING,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_CHANNEL_FLAGS
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_analog_channel_additional_damping_value
{
    NUMBER 64;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_DAMPING
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_DAMPING
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR,   Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR,   Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

COMMAND write_analog_channel_range_values
{
    NUMBER 65;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.RANGE_UNITS (INFO),
            dynamic_variables[analog_channel_number_code].RANGING.UPPER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.LOWER_RANGE_VALUE
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.RANGE_UNITS (INFO),
            dynamic_variables[analog_channel_number_code].RANGING.UPPER_RANGE_VALUE,
            dynamic_variables[analog_channel_number_code].RANGING.LOWER_RANGE_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
        9,  DATA_ENTRY_ERROR,   lower_range_value_too_high_temp;
//Remove new dictionary references from legacy binary format
        10, DATA_ENTRY_ERROR,   lower_range_value_too_low_temp;
//Remove new dictionary references from legacy binary format
        11, DATA_ENTRY_ERROR,   upper_range_value_too_high_temp;
//Remove new dictionary references from legacy binary format
        12, DATA_ENTRY_ERROR,   upper_range_value_too_low_temp;
//Remove new dictionary references from legacy binary format
        13, DATA_ENTRY_ERROR,   URV_and_LRV_out_of_limits_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, span_too_small_temp;
//Remove new dictionary references from legacy binary format
        15, DATA_ENTRY_ERROR,   invalid_analog_output_number_code_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        28, DATA_ENTRY_ERROR,   invalid_units_temp;
//Remove new dictionary references from legacy binary format
        29, DATA_ENTRY_ERROR,   invalid_span_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

COMMAND enter_exit_fixed_analog_channel_mode
{
    NUMBER 66;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS (INFO),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS (INFO),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR, Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        11, MODE_ERROR,       in_multidrop_mode_temp;
//Remove new dictionary references from legacy binary format
        12, DATA_ENTRY_ERROR, invalid_units_temp;
//Remove new dictionary references from legacy binary format
        15, DATA_ENTRY_ERROR, invalid_analog_output_number_code_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND trim_analog_channel_zero
{
    NUMBER 67;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR, Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  MODE_ERROR,       not_in_proper_anlg_output_mode_temp;
//Remove new dictionary references from legacy binary format
        11, MODE_ERROR,       in_multidrop_mode_temp;
//Remove new dictionary references from legacy binary format
        12, DATA_ENTRY_ERROR, invalid_units_temp;
//Remove new dictionary references from legacy binary format
        15, DATA_ENTRY_ERROR, invalid_analog_output_number_code_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND trim_analog_channel_gain
{
    NUMBER 68;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR, Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  MODE_ERROR,       not_in_proper_anlg_output_mode_temp;
//Remove new dictionary references from legacy binary format
        11, MODE_ERROR,       in_multidrop_mode_temp;
//Remove new dictionary references from legacy binary format
        12, DATA_ENTRY_ERROR, invalid_units_temp;
//Remove new dictionary references from legacy binary format
        15, DATA_ENTRY_ERROR, invalid_analog_output_number_code_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_analog_channel_transfer_function
{
    NUMBER 69;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.TRANSFER_FUNCTION
        }

        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].RANGING.TRANSFER_FUNCTION
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        13, DATA_ENTRY_ERROR, invalid_transfer_function_code_temp;
//Remove new dictionary references from legacy binary format
        15, DATA_ENTRY_ERROR, invalid_analog_output_number_code_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND read_analog_channel_endpoint_values
{
    NUMBER 70;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.UPPER_ENDPOINT_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.LOWER_ENDPOINT_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.UPPER_LIMIT_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.LOWER_LIMIT_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_analog_channel_endpoint_values
{
    NUMBER 535;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.UPPER_ENDPOINT_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.LOWER_ENDPOINT_VALUE
        }
        REPLY
        {
            response_code, device_status,
            analog_channel_number_code (INFO,INDEX),
            dynamic_variables[analog_channel_number_code].DAQ.ANALOG_UNITS,
            dynamic_variables[analog_channel_number_code].DAQ.UPPER_ENDPOINT_VALUE,
            dynamic_variables[analog_channel_number_code].DAQ.LOWER_ENDPOINT_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
	9,  DATA_ENTRY_ERROR,   Lower_endpoint_too_high_temp;
//Remove new dictionary references from legacy binary format
	10, DATA_ENTRY_ERROR,   Lower_endpoint_too_low_temp;
//Remove new dictionary references from legacy binary format
	11, DATA_ENTRY_ERROR,   Upper_endpoint_too_high_temp;
//Remove new dictionary references from legacy binary format
	12, DATA_ENTRY_ERROR,   Upper_endpoint_too_low_temp;
//Remove new dictionary references from legacy binary format
	13, DATA_ENTRY_ERROR,   Upper_and_lower_endpoints_out_of_limits_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, span_too_small_temp;
//Remove new dictionary references from legacy binary format
        15, DATA_ENTRY_ERROR,   invalid_analog_output_number_code_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        28, DATA_ENTRY_ERROR,   invalid_units_temp;
//Remove new dictionary references from legacy binary format
        29, DATA_ENTRY_ERROR,   invalid_span_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,         delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,         delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,         delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,         delayed_response_conflict_temp;
    }
}

METHOD fixed_analog_output
{
//Remove new dictionary references from legacy binary format
    LABEL fix_analog_channel_temp;
//Remove new dictionary references from legacy binary format
    HELP  fix_analog_channel_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[3];
        int use_abort_method;
        int choice;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        use_abort_method = 0;

        get_dictionary_string(select_analog_channel_to_trim, disp_string1, slen1);
        GET_DEV_VAR_VALUE(disp_string1, analog_channel_number_code);

        get_dictionary_string(select_analog_output_units, disp_string1, slen1);
        GET_DEV_VAR_VALUE(disp_string1,dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS);

        var_id[0] = VARID(analog_channel_number_code);
        var_id[2] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS);

        do
        {
            if (ivar_value(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS)== 39)
            {
                get_dictionary_string(choose_loop_current_level, disp_string1, slen1);
                get_dictionary_string(choices_4_20_other_end, disp_string2, slen2);

                choice = SELECT_FROM_LIST(disp_string1, disp_string2);

                switch (choice)
                {
                case 0:
                    assign_float(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE, 4.0);
                    break;

                case 1:
                    assign_float(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE, 20.0);
                    break;

                case 2:
                    get_dictionary_string(enter_desired_analog_output, disp_string1, slen1);
                    GET_DEV_VAR_VALUE(disp_string1,
                                      dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
                    break;

                case 3:
                default:
                    process_abort();
                    break;
                }
            }
            else
            {
                get_dictionary_string(choose_analog_channel_level, disp_string1, slen1);
                get_dictionary_string(new_value_end, disp_string2, slen2);

                choice = SELECT_FROM_LIST(disp_string1, disp_string2);

                switch(choice)
                {
                case 0:
                    get_dictionary_string(enter_desired_analog_output, disp_string1, slen1);
                    GET_DEV_VAR_VALUE(disp_string1,
                                      dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
                    break;
                case 1:
                    process_abort();
                    break;
                }
            }

            send(66,status);

            if (!use_abort_method)
            {
                if ((status[STATUS_RESPONSE_CODE] != 0)
                    && (status[STATUS_RESPONSE_CODE] != 3)
                    && (status[STATUS_RESPONSE_CODE] != 4))
                {
                    display_response_status(66, status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
                if (status[STATUS_RESPONSE_CODE] == 0)
                {
                    use_abort_method = 1;
                    remove_abort_method(return_to_normal);
                    add_abort_method(leave_analog_fixed_current_mode);
                }
            }

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(66, status[STATUS_RESPONSE_CODE]);
                if ((status[STATUS_RESPONSE_CODE] != 3) && (status[STATUS_RESPONSE_CODE] != 4))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
            }
            else
            {
                var_id[1] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
                get_dictionary_string(fld_dev_AO_is_fixed_at, disp_string1, slen1);
                acknowledge(disp_string1, var_id);
            }
        } while (choice < 3);
    }
}

METHOD analog_dac_trim
{
//Remove new dictionary references from legacy binary format
    LABEL Trim_Analog_Output_temp;
//Remove new dictionary references from legacy binary format
    HELP  trim_analog_output_help_temp;
/*  APPHELP "This type of calibration is usually used to compensate for"
            " drifting or aging output electronics"; */
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[3];
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        add_abort_method(return_to_normal);

        get_dictionary_string(connect_reference_meter, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(select_analog_channel_to_trim,disp_string1,slen1);
        GET_DEV_VAR_VALUE(disp_string1, analog_channel_number_code);
        get_dictionary_string(select_analog_output_units, disp_string1, slen1);

        GET_DEV_VAR_VALUE(disp_string1,
                          dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS);

        var_id[0] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);
        var_id[1] = VARID(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_UNITS);

        assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
                   dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.LOWER_ENDPOINT_VALUE);

        get_dictionary_string(about_to_set_fld_dev_output, disp_string1, slen1);
        acknowledge(disp_string1, var_id);

        send(66, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(66, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }
        else
        {
            remove_abort_method(return_to_normal);
            add_abort_method(leave_analog_fixed_current_mode);
        }

        do
        {
            assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
                       dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.LOWER_ENDPOINT_VALUE);

            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1,
                              dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);

            send(67, status);

            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(67, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(67, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_output_eq_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
                   dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.UPPER_ENDPOINT_VALUE);

        get_dictionary_string(about_to_set_fld_dev_output, disp_string1, slen1);
        acknowledge(disp_string1, var_id);

        send(66, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(66, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        do
        {
            assign_var(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,
                       dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.UPPER_ENDPOINT_VALUE);

            get_dictionary_string(enter_meter_value, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1,
                              dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE);

            send(68, status);
            if ((status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(68, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(68, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_output_eq_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        process_abort();
    }
}

METHOD leave_analog_fixed_current_mode
{
//Remove new dictionary references from legacy binary format
    LABEL leaving_fixed_analog_channel_mode_temp;
//Remove new dictionary references from legacy binary format
    HELP  leaving_fixed_analog_channel_mode_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        long var_id[1];
        int slen1;

        slen1 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        assign_float(dynamic_variables[ivar_value(analog_channel_number_code)].DAQ.ANALOG_VALUE,NaN);

        do
        {
            get_dictionary_string(return_device_to_original_output, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(66, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(66, status[STATUS_RESPONSE_CODE]);
            }

        } while(status[STATUS_RESPONSE_CODE]);

        get_dictionary_string(loop_may_be_returned_to_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}
