/*****************************************************************************
 *
 * File: Common12_2_Device_Vars.ddl
 * Version: 12.02.1
 * Date: 01/14/2022
 *
 *****************************************************************************
 * Copyright (c) 2018-2022, FieldComm Group, All Rights Reserved
 *****************************************************************************
 *
 * Description: This file contains the DDL based model of the Subset of items
 *      necessary to implement Device Variable Commands from the the Common Practice
 *      Command Specification Rev.12.0.  
 *
 * Source Specifications (These are revision numbers of the protocol spec which
 *                        may differ from the standard DD revision numbers.)
 *      Command Practice Command Specification      Rev.12.0
 *      Command Summary Specification               Rev.10.1
 *      Universal Command Specification             Rev.7.2
 *      Common Tables Specification                 Rev.25.0
 *
 *****************************************************************************
 */

/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
MANUFACTURER  STANDARD, DEVICE_TYPE _COMMON_PRACTICE_DEVICE_VARIABLES, DEVICE_REVISION 12, DD_REVISION 2
 */
MANUFACTURER  STANDARD, DEVICE_TYPE 25, DEVICE_REVISION 12, DD_REVISION 2

/*****************************************************************************
 * Include Files
 *****************************************************************************
 */
#include "macros2.h"
#include "embedded_strings.h"
#include "methods.h"

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */
#define UPDATE_IN_PROGRESS      8
#define DEVICE_BUSY             32
#define MAXIMUM_RESET_RETRIES   10

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.5
 * Transducer trim Commands
 *
 * Commands 80 81 82 83
 */

VARIABLE device_variable_trim_code
{
//Remove new dictionary references from legacy binary format
    LABEL device_variable_trim_code_temp;
//Remove new dictionary references from legacy binary format
    HELP  Device_Variable_Trim_Code_help_temp;
    CLASS LOCAL;
    TYPE INDEX deviceVariables;
}

COMMAND read_device_variable_trim_point
{
    NUMBER 80;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_trim_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            device_variable_trim_code (INFO,INDEX),
            deviceVariables[device_variable_trim_code].TRIM_POINT_UNITS,
            deviceVariables[device_variable_trim_code].LOWER_TRIM_POINT,
            deviceVariables[device_variable_trim_code].UPPER_TRIM_POINT
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR, invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        19, DATA_ENTRY_ERROR, device_variable_not_allowed_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND read_device_variable_trim_guidelines
{
    NUMBER 81;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_trim_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            device_variable_trim_code (INFO,INDEX),
            deviceVariables[device_variable_trim_code].TRIM_POINT_SUPPORT,
            deviceVariables[device_variable_trim_code].TRIM_POINT_UNITS,
            deviceVariables[device_variable_trim_code].MINIMUM_LOWER_TRIM_POINT,
            deviceVariables[device_variable_trim_code].MAXIMUM_LOWER_TRIM_POINT,
            deviceVariables[device_variable_trim_code].MINIMUM_UPPER_TRIM_POINT,
            deviceVariables[device_variable_trim_code].MAXIMUM_UPPER_TRIM_POINT,
            deviceVariables[device_variable_trim_code].MINIMUM_TRIM_DIFFERENTIAL
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR, invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        19, DATA_ENTRY_ERROR, device_variable_not_allowed_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_device_variable_trim_point
{
    NUMBER 82;
    OPERATION COMMAND; // A veriable trim should not be confused with writing a device variable with command 79
    TRANSACTION
    {
        REQUEST
        {
            device_variable_trim_code (INFO,INDEX),
            deviceVariables[device_variable_trim_code].TRIM_POINT_SUPPORT (INFO),
            deviceVariables[device_variable_trim_code].TRIM_POINT_UNITS,
            deviceVariables[device_variable_trim_code].DIGITAL_VALUE
        }
        REPLY
        {
            response_code, device_status,
            device_variable_trim_code (INFO,INDEX),
            deviceVariables[device_variable_trim_code].TRIM_POINT_SUPPORT (INFO),
            deviceVariables[device_variable_trim_code].TRIM_POINT_UNITS,
            deviceVariables[device_variable_trim_code].DIGITAL_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR,   Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR,   Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  PROCESS_ERROR,      applied_process_too_high_temp;
//Remove new dictionary references from legacy binary format
        10, PROCESS_ERROR,      applied_process_too_low_temp;
//Remove new dictionary references from legacy binary format
        11, MISC_ERROR,         excess_correction_attempted_temp;
//Remove new dictionary references from legacy binary format
        13, MISC_ERROR,         computational_error_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, span_too_small_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR,   invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        18, DATA_ENTRY_ERROR,   invalid_units_temp;
//Remove new dictionary references from legacy binary format
        19, DATA_ENTRY_ERROR,   device_variable_not_allowed_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,         delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,         delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,         delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,         delayed_response_conflict_temp;
    }
}

COMMAND reset_device_variable_trim
{
    NUMBER 83;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_trim_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            device_variable_trim_code (INFO,INDEX)
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR, invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        19, DATA_ENTRY_ERROR, device_variable_not_allowed_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}

METHOD zero_trim
{
//Remove new dictionary references from legacy binary format
    LABEL Zero_Trim_temp;
//Remove new dictionary references from legacy binary format
    HELP  Zero_Trim_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        add_abort_method(return_to_normal);

        get_dictionary_string(this_will_affect_snsr_cal, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        do
        {
            get_dictionary_string(apply_0_input_to_snsr, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);

            get_dictionary_string(snsr_input_stabilizing, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(43, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(43, status[STATUS_RESPONSE_CODE]);

                if ((status[STATUS_RESPONSE_CODE] != 9)
                    && (status[STATUS_RESPONSE_CODE] != 10))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }

                get_dictionary_string(zero_the_input_and_try_again, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = SELECT_FROM_LIST(disp_string1, disp_string2);
            }
            else
            {
                get_dictionary_string(snsr_zero_succeeded, disp_string1, slen1);
                DELAY(3, disp_string1);
                process_abort();
            }
        } while (!retry);

        process_abort();
    }
}

METHOD return_to_normal
{
//Remove new dictionary references from legacy binary format
    LABEL Aborting_Return_to_Normal_temp;
    DEFINITION
    {
        char disp_string1[60];
        int slen1;

        slen1 = 60;

        get_dictionary_string(loop_may_be_returned_to_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}

METHOD warning_message
{
//Remove new dictionary references from legacy binary format
    LABEL Set_Loop_to_Manual_temp;
    DEFINITION
    {
        char disp_string1[125];
        int slen1;

        slen1 = 125;

        get_dictionary_string(remove_from_auto_befor_send, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}

METHOD device_variable_zero_trim
{
//Remove new dictionary references from legacy binary format
    LABEL device_variable_zero_trim_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_variable_zero_trim_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        int tv;
        int retry;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        add_abort_method(return_to_normal);

        get_dictionary_string(this_will_affect_snsr_cal, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(select_fld_dev_var_snsr_trim, disp_string1, slen1);
        GET_DEV_VAR_VALUE(disp_string1, device_variable_code);

        do
        {
            get_dictionary_string(apply_0_input_to_snsr, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);

            get_dictionary_string(snsr_input_stabilizing, disp_string1, slen1);
            DELAY(3, disp_string1);

            send(52, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(52, status[STATUS_RESPONSE_CODE]);

                if ((status[STATUS_RESPONSE_CODE] != 2)
                    && (status[STATUS_RESPONSE_CODE] != 9)
                    && (status[STATUS_RESPONSE_CODE] != 10))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }

                get_dictionary_string(zero_the_input_and_try_again, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);

                retry = SELECT_FROM_LIST(disp_string1, disp_string2);

                if (!retry)
                {
                    if (status[STATUS_RESPONSE_CODE] == 2)
                    {
                        do
                        {
                            tv = ivar_value(device_variable_code);
                            get_dictionary_string(select_fld_dev_var_snsr_trim, disp_string1, slen1);

                            GET_DEV_VAR_VALUE(disp_string1, device_variable_code);

                            if (tv == ivar_value(device_variable_code))
                            {
                                get_dictionary_string(invalid_value_try_again, disp_string1, slen1);
                                DELAY(2, disp_string1);
                            }
                        } while (tv == ivar_value(device_variable_code));
                    }
                }
            }
            else
            {
                get_dictionary_string(snsr_zero_succeeded, disp_string1, slen1);
                DELAY(3, disp_string1);
                process_abort();
            }
        } while (!retry);

        process_abort();
    }
}

METHOD device_variable_trim
{
//Remove new dictionary references from legacy binary format
    LABEL Device_Variable_Trim_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_variable_trim_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[135];
        char disp_string2[60];
        long var_id[2];
        int trim_code;
        int doneLowTrim;
        int retry;
        int slen1;
        int slen2;
        float trim_point;
        float trim_min;
        float trim_max;

        slen1 = 135;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(this_will_affect_snsr_cal, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(connect_reference_meter_trim_dv, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        do
        {
            doneLowTrim = 0; /* Low trim has not been performed on this device variable */

            do
            {
                retry = 1; /* Device variable has not been found */

                /*
                 * Get the device variable "to trim" from the user
                 */
                get_dictionary_string(select_fld_dev_var_snsr_trim, disp_string1, slen1);
                GET_DEV_VAR_VALUE(disp_string1, device_variable_trim_code);

                /*
                 * Issue Command 81 (Read Device Variable Trim Guidelines) to determine the number of
                 * applicable trim points and their acceptable limits (if any) for this device variable
                 */
                send(81, status);

                if (status[STATUS_RESPONSE_CODE])
                {
                    display_response_status(81, status[STATUS_RESPONSE_CODE]);

                    if ((status[STATUS_RESPONSE_CODE] != 17) && (status[STATUS_RESPONSE_CODE] != 19))
                    {
                        get_dictionary_string(abort_last_error, disp_string1, slen1);
                        DELAY(2, disp_string1);
                        process_abort();
                    }
                }
                else
                {
                    retry = 0; /* A valid device variable to trim has been found */
                }
            } while (retry);

            var_id[0] = VARID(deviceVariables[ivar_value(device_variable_trim_code)].DIGITAL_VALUE);
            var_id[1] = VARID(deviceVariables[ivar_value(device_variable_trim_code)].DIGITAL_UNITS);

            /*
             *  Find out what trim points this device variable supports
             */
            trim_code = ivar_value(deviceVariables[ivar_value(device_variable_trim_code)].TRIM_POINT_SUPPORT);

            /*
             * Do the lower trim (Single or two trim point supported)
             */
            do
            {
                /*
                 * Issue Command 80 (Read Device Variable Trim Points) to retrieve the last trim points
                 * used.  These should be used as default values for a new trim operation.  If the trim
                 * code supports a two point trim, them perform the low trim first.  If the code supports
                 * a single trim (lower trim or upper trim), perform only the lower trim.
                 */
                send(80, status);

                if (status[STATUS_RESPONSE_CODE])
                {
                    display_response_status(80, status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }

                /*
                 * Prompt the user to set the variable input to a value within acceptable limits for the
                 * lower trim point
                 */
                trim_point = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].LOWER_TRIM_POINT);
                trim_min = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].MINIMUM_LOWER_TRIM_POINT);
                trim_max = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].MAXIMUM_LOWER_TRIM_POINT);

                get_dictionary_string(apply_lower_trim_input, disp_string1, slen1);
                acknowledge(disp_string1, var_id);

                /*
                 * First get PV to use for an editing starting point.  This should be very close to the
                 * trim_point value retrieved from the Command 80 above
                 */
                get_dictionary_string(enter_meter_value, disp_string1, slen1);
                GET_DEV_VAR_VALUE(disp_string1, deviceVariables[ivar_value(device_variable_trim_code)].DIGITAL_VALUE);

                /*
                 * We are triming the lower point value so make sure the code = 1
                 * See Table 22
                 */
                if (trim_code == 3)
                {
                    iassign(deviceVariables[ivar_value(device_variable_trim_code)].TRIM_POINT_SUPPORT, 1); /* Lower point value */
                }

                /*
                 * Write the trim point
                 */
                send(82, status);

                if (status[STATUS_RESPONSE_CODE])
                {
                    display_response_status(82, status[STATUS_RESPONSE_CODE]);

                    if (status[STATUS_RESPONSE_CODE]  == 3 || status[STATUS_RESPONSE_CODE]  == 4 ||
                        status[STATUS_RESPONSE_CODE]  == 9 || status[STATUS_RESPONSE_CODE]  == 10)
                    {
                        /*
                         * Value out of range
                         */
                        retry = 1; /* Let's try this again */
                    }
                    else
                    {
                        get_dictionary_string(abort_last_error, disp_string1, slen1);
                        DELAY(2, disp_string1);
                        process_abort();
                    }
                }
                else /* Success */
                {
                    /*
                     * If the upper trim needs to be peformed and user aborts after the lower
                     * trim, show a warning message to the user
                     */
                    if (!doneLowTrim && trim_code == 3)
                    {
                        remove_abort_method(return_to_normal);
                        add_abort_method(trim_warning);
                        add_abort_method(return_to_normal);
                    }

                    doneLowTrim = 1; /* Indicate that the lower (or single) trim has been performed */

                    get_dictionary_string(applied_process_equal_ref_meter, disp_string1, slen1);
                    get_dictionary_string(yes_no, disp_string2, slen2);
                    retry = select_from_list(disp_string1, var_id, disp_string2);
                }
            } while (retry);

            /*
             * Now, do the upper trim (if two trim points are supported)
             */
            if (trim_code == 3)
            {
                do
                {
                    /*
                     * Issue Command 80 (Read Device Variable Trim Points) to retrieve the last trim points
                     * used.  These should be used as default values for a new trim operation.  If the value
                     * supports a two point trim, perfrom the upper trim after the lower trim is performed.
                     */
                    send(80, status);

                    if (status[STATUS_RESPONSE_CODE])
                    {
                        display_response_status(80, status[STATUS_RESPONSE_CODE]);
                        get_dictionary_string(abort_last_error, disp_string1, slen1);
                        DELAY(2, disp_string1);
                        process_abort();
                    }

                    /*
                     * Prompt the user to set the variable input to a value within acceptable limits for the
                     * upper trim point
                     */
                    trim_point = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].UPPER_TRIM_POINT);
                    trim_min = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].MINIMUM_UPPER_TRIM_POINT);
                    trim_max = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].MAXIMUM_UPPER_TRIM_POINT);

                    get_dictionary_string(apply_upper_trim_input, disp_string1, slen1);
                    acknowledge(disp_string1, var_id);

                    /*
                     * First get PV to use for an editing starting point.  This should be very close to the
                     * trim_point value retrieved from the Command 80 above
                     */
                    get_dictionary_string(enter_meter_value, disp_string1, slen1);
                    GET_DEV_VAR_VALUE(disp_string1, deviceVariables[ivar_value(device_variable_trim_code)].DIGITAL_VALUE);

                    /*
                     * We are triming the upper point value so make sure the code = 2
                     * See Table 22
                     */
                    iassign(deviceVariables[ivar_value(device_variable_trim_code)].TRIM_POINT_SUPPORT, 2); /* Upper point value */

                    /*
                     * Write the trim point
                     */
                    send(82, status);

                    if (status[STATUS_RESPONSE_CODE])
                    {
                        display_response_status(82, status[STATUS_RESPONSE_CODE]);

                        if (status[STATUS_RESPONSE_CODE]  == 3 || status[STATUS_RESPONSE_CODE]  == 4 ||
                            status[STATUS_RESPONSE_CODE]  == 9 || status[STATUS_RESPONSE_CODE]  == 10)
                        {
                            /*
                             * Value out of range
                             */
                            retry = 1; /* Let's try this again */
                        }
                        else
                        {
                            get_dictionary_string(abort_last_error, disp_string1, slen1);
                            DELAY(2, disp_string1);
                            process_abort();
                        }
                    }
                    else /* Success */
                    {
                        remove_abort_method(trim_warning);
                        get_dictionary_string(applied_process_equal_ref_meter, disp_string1, slen1);
                        get_dictionary_string(yes_no, disp_string2, slen2);
                        retry = select_from_list(disp_string1, var_id, disp_string2);
                    }
                } while (retry);
            }

            /*
             *  Send command 80 to re-read the lower trim point and/or
             *  upper trim point to update it as a result of sending
             *  command 82
             */
            send(80, status);

            get_dictionary_string(trim_another_device_variable, disp_string1, slen1);
            get_dictionary_string(yes_no, disp_string2, slen2);
            retry = SELECT_FROM_LIST(disp_string1, disp_string2);
        } while (!retry);

        process_abort();
    }
}

METHOD trim_warning
{
//Remove new dictionary references from legacy binary format
    LABEL Trim_Warning_temp;
    DEFINITION
    {
        char disp_string1[90];
        int slen1;

        slen1 = 90;

        get_dictionary_string(trim_not_complete, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}

METHOD device_variable_trim_reset
{
//Remove new dictionary references from legacy binary format
    LABEL Device_Variable_Trim_Reset_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_variable_trim_reset_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[100];
        char disp_string2[60];
        char ack;
        long var_id[2];
        int slen1;
        int slen2;
        float lower_trim_point;
        float upper_trim_point;

        slen1 = 100;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(this_will_affect_snsr_cal, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        /*
         * Get the device variable "to trim" from the user
         */
        get_dictionary_string(select_fld_dev_var_snsr_trim, disp_string1, slen1);
        GET_DEV_VAR_VALUE(disp_string1, device_variable_trim_code);

        /*
         * Display current trim points
         */
        send(80, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(80, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        /*
         * Current trim point values
         */
        lower_trim_point = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].LOWER_TRIM_POINT);
        upper_trim_point = fvar_value(deviceVariables[ivar_value(device_variable_trim_code)].UPPER_TRIM_POINT);

        /*
         * Units
         */
        var_id[0] = VARID(deviceVariables[ivar_value(device_variable_trim_code)].TRIM_POINT_UNITS);

        /* Ask for acknowledgement */
        get_dictionary_string(current_trimpoints, disp_string1, slen1);
        get_dictionary_string(yes_no, disp_string2, slen2);
        ack = select_from_list(disp_string1, var_id, disp_string2);

        /*
         * If yes, then the user wants to reset the device variable trim points
         */
        if(ack == 0)
        {
            /*
             * Issue Command 83 (Reset Device Variable Trim) to reset the device variable to the
             * default factory trim
             */
            send(83,status);

            if(status[STATUS_RESPONSE_CODE])
            {
                display_response_status(83, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
            }

            /*
             *  Send command 80 to re-read the lower trim point and/or
             *  upper trim point to update it as a result of sending
             *  command 83
             */
            send(80, status);
        }
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.6
 * Device Variable Mapping Commands
 *
 * Commands 50 51
 */

VARIABLE primary_variable_code
{
//Remove new dictionary references from legacy binary format
    LABEL primary_variable_code_temp;
//Remove new dictionary references from legacy binary format
    HELP  primary_variable_code_help_temp;
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

VARIABLE secondary_variable_code
{
//Remove new dictionary references from legacy binary format
    LABEL secondary_variable_code_temp;
//Remove new dictionary references from legacy binary format
    HELP  secondary_variable_code_help_temp;
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

VARIABLE tertiary_variable_code
{
//Remove new dictionary references from legacy binary format
    LABEL tertiary_variable_code_temp;
//Remove new dictionary references from legacy binary format
    HELP  tertiary_variable_code_help_temp;
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

VARIABLE quaternary_variable_code
{
//Remove new dictionary references from legacy binary format
    LABEL quaternary_variable_code_temp;
//Remove new dictionary references from legacy binary format
    HELP  quaternary_variable_code_help_temp;
    CLASS HART;
    TYPE INDEX deviceVariables;
    POST_EDIT_ACTIONS
    {
        warning_message
    }
}

COMMAND read_device_variable_assignments
{
    NUMBER 50;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            primary_variable_code, secondary_variable_code,
            tertiary_variable_code, quaternary_variable_code
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
    }
}

COMMAND write_device_variable_assignments
{
    NUMBER 51;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            primary_variable_code, secondary_variable_code,
            tertiary_variable_code, quaternary_variable_code
        }
        REPLY
        {
            response_code, device_status,
            primary_variable_code, secondary_variable_code,
            tertiary_variable_code, quaternary_variable_code
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, Burst_configuration_update_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.7
 * PV Commands
 *
 * Commands 34 43 44 47 49 100
 */

COMMAND write_pv_damping_value
{
    NUMBER 34;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DEVICE_VARIABLE.DAMPING_VALUE
        }
        REPLY
        {
            response_code, device_status, PV.DEVICE_VARIABLE.DAMPING_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR,   Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR,   Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

COMMAND set_pv_zero
{
    NUMBER 43;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,       no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,    xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,    in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  PROCESS_ERROR, applied_process_too_high_temp;
//Remove new dictionary references from legacy binary format
        10, PROCESS_ERROR, applied_process_too_low_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,    access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,    busy_temp;
    }
}

COMMAND write_pv_units
{
    NUMBER 44;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DEVICE_VARIABLE.CONFIG_UNITS
        }
        REPLY
        {
            response_code, device_status, PV.DEVICE_VARIABLE.CONFIG_UNITS
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_pv_transfer_function
{
    NUMBER 47;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.RANGING.TRANSFER_FUNCTION
        }
        REPLY
        {
            response_code, device_status, PV.RANGING.TRANSFER_FUNCTION
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_pv_sensor_serial_number
{
    NUMBER 49;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DEVICE_VARIABLE.SENSOR_SERIAL_NUMBER
        }
        REPLY
        {
            response_code, device_status, PV.DEVICE_VARIABLE.SENSOR_SERIAL_NUMBER
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.8
 * Device Variable Commands.  The commands allow access to the Device
 * Variables in the Field Device
 *
 * Commands 33, 52-56, 79, 113, 114
 *
 */

COMMAND read_device_variables
{
    NUMBER 33;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code_1 (INFO,INDEX),
            device_variable_code_2 (INFO,INDEX),
            device_variable_code_3 (INFO,INDEX),
            device_variable_code_4 (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            device_variable_code_1 (INFO,INDEX),
            deviceVariables[device_variable_code_1].DIGITAL_UNITS,
            deviceVariables[device_variable_code_1].DIGITAL_VALUE,
            device_variable_code_2 (INFO,INDEX),
            deviceVariables[device_variable_code_2].DIGITAL_UNITS,
            deviceVariables[device_variable_code_2].DIGITAL_VALUE,
            device_variable_code_3 (INFO,INDEX),
            deviceVariables[device_variable_code_3].DIGITAL_UNITS,
            deviceVariables[device_variable_code_3].DIGITAL_VALUE,
            device_variable_code_4 (INFO,INDEX),
            deviceVariables[device_variable_code_4].DIGITAL_UNITS,
            deviceVariables[device_variable_code_4].DIGITAL_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        8,  MISC_WARNING,     update_failure_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
    }
}

VARIABLE device_variable_code
{
//Remove new dictionary references from legacy binary format
    LABEL device_variable_code_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_variable_code_help_temp;
/*  APPHELP "This variable can represent measured or calculated type variables."
            "The numeric assigned to a Field Device variable can never be"
            "changed. If it is changed this would be a violation of the Forward"
            "Compatibility rules."
            "HART_VAR:XMTR VAR CODE"; */
    CLASS LOCAL;
    TYPE INDEX deviceVariables;
}

COMMAND set_device_variable_zero
{
    NUMBER 52;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO)
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO)
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  PROCESS_ERROR,    applied_process_too_high_temp;
//Remove new dictionary references from legacy binary format
        10, PROCESS_ERROR,    applied_process_too_low_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_device_variable_units
{
    NUMBER 53;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].CONFIG_UNITS
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].CONFIG_UNITS
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        11, DATA_ENTRY_ERROR, invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        12, DATA_ENTRY_ERROR, invalid_units_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND read_device_variable_information
{
    NUMBER 54;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SENSOR_SERIAL_NUMBER,
            deviceVariables[device_variable_code].CONFIG_UNITS,
            deviceVariables[device_variable_code].UPPER_SENSOR_LIMIT,
            deviceVariables[device_variable_code].LOWER_SENSOR_LIMIT,
            deviceVariables[device_variable_code].DAMPING_VALUE,
            deviceVariables[device_variable_code].MINIMUM_SPAN,
            deviceVariables[device_variable_code].CLASSIFICATION,
            deviceVariables[device_variable_code].DEVICE_FAMILY,
            deviceVariables[device_variable_code].UPDATE_TIME_PERIOD,
            deviceVariables[device_variable_code].PROPERTIES
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

COMMAND write_device_variable_damping_value
{
    NUMBER 55;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].DAMPING_VALUE
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].DAMPING_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR,   Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR,   Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

COMMAND write_device_variable_sensor_serial_number
{
    NUMBER 56;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SENSOR_SERIAL_NUMBER
        }
        REPLY
        {
            response_code, device_status, device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SENSOR_SERIAL_NUMBER
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}


/*  APPHELP "Command 79 can be used in 2 different ways."
            "1 - to simulate the value of a variable that is normally calculated by the field device."
            "2 - to set the value of a device variable that is NOT calculated by the field device (e.g. change a setpoint)."
            "In either case, the process of modifying a device variable must be done in a method, as it is not as simple as just modifying the value on a menu."
            "For the above reasons, this command is declared as operation COMMAND (not WRITE)."*/
COMMAND write_device_variable
{
    NUMBER 79;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO, INDEX),
            deviceVariables[device_variable_code].SIMULATED,
            deviceVariables[device_variable_code].DIGITAL_UNITS (INFO),
            deviceVariables[device_variable_code].DIGITAL_VALUE,
            deviceVariables[device_variable_code].DATA_QUALITY <0xC0>,
            deviceVariables[device_variable_code].LIMIT_STATUS <0x30>,
            deviceVariables[device_variable_code].DEVICE_FAMILY_STATUS <0x0F>

        }
        REPLY
        {
            response_code, device_status,
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SIMULATED,
            deviceVariables[device_variable_code].DIGITAL_UNITS (INFO),
            deviceVariables[device_variable_code].DIGITAL_VALUE,
            deviceVariables[device_variable_code].DATA_QUALITY <0xC0>,
            deviceVariables[device_variable_code].LIMIT_STATUS <0x30>,
            deviceVariables[device_variable_code].DEVICE_FAMILY_STATUS <0x0F>
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, device_family_status_bits_not_set_temp;
//Remove new dictionary references from legacy binary format
        10, DATA_ENTRY_ERROR,   invalid_write_device_variable_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, rate_of_change_enforced_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR,   invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        18, DATA_ENTRY_ERROR,   invalid_units_temp;
//Remove new dictionary references from legacy binary format
        19, DATA_ENTRY_ERROR,   device_variable_not_allowed_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,         delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,         delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,         delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,         delayed_response_conflict_temp;
    }
}

COMMAND read_device_variable_command_code
{
    NUMBER 534;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO, INDEX)
        }
        REPLY
        {
            response_code, device_status,
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].SIMULATED,
            deviceVariables[device_variable_code].DIGITAL_UNITS (INFO),
            deviceVariables[device_variable_code].SIMULATED_DIGITAL_VALUE,
            deviceVariables[device_variable_code].SIMULATED_DATA_QUALITY <0xC0>,
            deviceVariables[device_variable_code].SIMULATED_LIMIT_STATUS <0x30>,
            deviceVariables[device_variable_code].SIMULATED_DEVICE_FAMILY_STATUS <0x0F>
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        14, DATA_ENTRY_WARNING, rate_of_change_enforced_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR,   invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,         delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,         delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,         delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,         delayed_response_conflict_temp;
    }
}

VARIABLE destination_device_variable_code
{
//Remove new dictionary references from legacy binary format
    LABEL destination_device_variable_code_temp;
//Remove new dictionary references from legacy binary format
    HELP  destination_device_variable_code_help_temp;
    CLASS LOCAL;
    TYPE INDEX deviceVariables;
}

COMMAND catch_device_variable
{
    NUMBER 113;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            destination_device_variable_code (INFO,INDEX),
            deviceVariables[destination_device_variable_code].CAPTURE_MODE,
            deviceVariables[destination_device_variable_code].SOURCE_DEVICE_TYPE,
            deviceVariables[destination_device_variable_code].SOURCE_DEVICE_ID,
            0x1F,
            deviceVariables[destination_device_variable_code].SOURCE_SLOT_NUMBER,
            deviceVariables[destination_device_variable_code].SHED_TIME,
            deviceVariables[destination_device_variable_code].SOURCE_COMMAND_NUMBER
        }
        REPLY
        {
            response_code, device_status,
            destination_device_variable_code (INFO,INDEX),
            deviceVariables[destination_device_variable_code].CAPTURE_MODE,
            deviceVariables[destination_device_variable_code].SOURCE_DEVICE_TYPE,
            deviceVariables[destination_device_variable_code].SOURCE_DEVICE_ID,
            0x1F,
            deviceVariables[destination_device_variable_code].SOURCE_SLOT_NUMBER,
            deviceVariables[destination_device_variable_code].SHED_TIME,
            deviceVariables[destination_device_variable_code].SOURCE_COMMAND_NUMBER
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, capture_mode_set_to_back_temp;
//Remove new dictionary references from legacy binary format
        10, DATA_ENTRY_ERROR,   invalid_capture_mode_code_temp;
//Remove new dictionary references from legacy binary format
        11, DATA_ENTRY_ERROR,   invalid_slot_number_temp;
//Remove new dictionary references from legacy binary format
        12, DATA_ENTRY_ERROR,   command_number_not_supported_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR,   invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        19, DATA_ENTRY_ERROR,   device_variable_not_allowed_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,         delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,         delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,         delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,         delayed_response_conflict_temp;

    }
}

VARIABLE source_command_number_byte
{
//Remove new dictionary references from legacy binary format
    LABEL command_number_temp;
//Remove new dictionary references from legacy binary format
    HELP  command_number_help_temp;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

COMMAND read_caught_device_variable
{
    NUMBER 114;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            destination_device_variable_code (INFO,INDEX)
        }
        REPLY
        {
            response_code, device_status,
            destination_device_variable_code (INFO,INDEX),
            deviceVariables[destination_device_variable_code].CAPTURE_MODE,
            deviceVariables[destination_device_variable_code].SOURCE_DEVICE_TYPE,
            deviceVariables[destination_device_variable_code].SOURCE_DEVICE_ID,
            source_command_number_byte,
            deviceVariables[destination_device_variable_code].SOURCE_SLOT_NUMBER,
            deviceVariables[destination_device_variable_code].SHED_TIME,
            deviceVariables[destination_device_variable_code].SOURCE_COMMAND_NUMBER
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        17, DATA_ENTRY_ERROR, invalid_device_variable_temp;
//Remove new dictionary references from legacy binary format
        19, DATA_ENTRY_ERROR, device_variable_not_allowed_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}

METHOD catchDeviceVariable
{
//Remove new dictionary references from legacy binary format
    LABEL Catch_Device_Variable_temp;
//Remove new dictionary references from legacy binary format
    HELP  catch_device_variable_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[140];
        char disp_string2[60];
        long var_id[8];
        int mode;
        int choice;
        int slot;
        int retry;
        int slen1;
        int slen2;
        float shed;

        slen1 = 140;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        /*
         * Choose device variable
         *
         * NOTE: This method assumes that only one device variable needs to catch data
         * The method will need to be executed again for the mapping of any additional
         * device variable.
         */
        do
        {
            retry = 1; /* Device variable has not been found */

            /*
             * Get the device variable "to catch" from the user
             */
            get_dictionary_string(select_device_variable_catch, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, destination_device_variable_code);

            /*
             * Send Command 114 (Read Caught Device Variable) to determine if the specified variable
             * supports catching
             */
            send(114, status);

            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(114, status[STATUS_RESPONSE_CODE]);

                if ((status[STATUS_RESPONSE_CODE] != 17) && (status[STATUS_RESPONSE_CODE] != 19))
                {
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(5, disp_string1);
                    process_abort();
                }
            }
            else
            {
                retry = 0; /* A valid device variable to catch has been found */
            }
        } while (retry);

        /*
         * A valid device variable to catch has been found
         */
        var_id[0] = VARID(destination_device_variable_code);

        /*
         * Get the current capture mode of the device variable
         */
        mode = ivar_value(deviceVariables[ivar_value(destination_device_variable_code)].CAPTURE_MODE);

        /*
         * Set catch info
         */
        do
        {
            retry = 1; /* New values have not been set */
            /*
             * Get Capture mode
             */
            get_dictionary_string(select_catch_dv_capture_mode, disp_string1, slen1);
            GET_DEV_VAR_VALUE(disp_string1, deviceVariables[ivar_value(destination_device_variable_code)].CAPTURE_MODE);

            choice = ivar_value(deviceVariables[ivar_value(destination_device_variable_code)].CAPTURE_MODE);

            if (choice > 0) /* No need to ask more if catching will be disabled */
            {
                if (choice == 1) /* Only ask for address for address match mode */
                {
                    var_id[1] = 0;  //Manufacturer is removed because device type below will be 2 byte expaneded device type.
                    /*
                     * Get Extended Device Type
                     * NOTE: deviceVariables[destination_device_variable_code].SOURCE_DEVICE_TYPE must (at a minimum) reference an
                     * UNSIGNED_INTEGER variable of size 2.
                     */
                    get_dictionary_string(select_src_device_type, disp_string1, slen1);
                    GET_DEV_VAR_VALUE(disp_string1, deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_DEVICE_TYPE);

                    var_id[2] = VARID(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_DEVICE_TYPE);

                    /*
                     * Get Device Id
                     * NOTE: deviceVariables[destination_device_variable_code].SOURCE_DEVICE_ID must (at a minimum) reference an
                     * UNSIGNED_INTEGER variable of size 3.
                     */
                    get_dictionary_string(any_device_matching_mfr_dev_type, disp_string1, slen1);
                    get_dictionary_string(yes_no, disp_string2, slen2);

                    if (!select_from_list(disp_string1, var_id, disp_string2))
                    {
                        /*
                         * Set to zero to indicate any field device matching the Manufacturer ID and Device Type
                         * is to act as a data source
                         */
                        iassign(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_DEVICE_ID, 0);
                    }
                    else
                    {
                        /*
                         * The user chooses the device id
                         */
                        get_dictionary_string(choose_src_device_id, disp_string1, slen1);
                        GET_DEV_VAR_VALUE(disp_string1, deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_DEVICE_ID);
                    }

                    var_id[3] = VARID(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_DEVICE_ID);
                }
                else if (choice == 2)
                {
                    /*
                     * Set the address to zero since the data source is a bursting slave
                     */
                    iassign(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_DEVICE_TYPE, 0);
                    iassign(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_DEVICE_ID, 0);
                }
                else
                {
                    /*
                     * Not a valid selection (negative enteries not accepted)
                     */
                    get_dictionary_string(invalid_selection, disp_string1, slen1);
                    DELAY(5, disp_string1);
                    process_abort();
                }

                /*
                 * Get the command number
                 *
                 * NOTE: deviceVariables[destination_device_variable_code].SOURCE_COMMAND_NUMBER must reference a variable in your DD
                 * based on same Command choices as those specified in variable burst_command_number.  It should be an ENUMERATED
                 * variable of size 2.
                 */
                get_dictionary_string(select_src_command, disp_string1, slen1);
                GET_DEV_VAR_VALUE(disp_string1, deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_COMMAND_NUMBER);

                var_id[4] = VARID(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_COMMAND_NUMBER);

                /*
                 * Get data slot from user based on the command number
                 */
                get_dictionary_string(select_dv_catch_data, disp_string1, slen1);

                switch (ivar_value(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_COMMAND_NUMBER))
                {
                case 1: /* Command 1 */
                    slot = 0;
                    /*
                     * Source slot is always PV
                     */
                    break;

                case 2: /* Command 2 */
                    get_dictionary_string(choice_current_percent_range, disp_string2, slen2);
                    slot = SELECT_FROM_LIST(disp_string1,disp_string2);
                    break;

                case 3: /* Command 3 */
                    get_dictionary_string(choice_dynamic_variables, disp_string2, slen2);
                    slot = SELECT_FROM_LIST(disp_string1,disp_string2);
                    break;

                case 9: /* Command 9 */
                case 33: /* Command 33 */
                    get_dictionary_string(choice_device_variables, disp_string2, slen2);
                    slot = SELECT_FROM_LIST(disp_string1,disp_string2);
                    break;

                default:
                    /*
                     * Not a valid selection
                     */
                    get_dictionary_string(invalid_selection, disp_string1, slen1);
                    DELAY(5, disp_string1);
                    process_abort();
                    break;
                }

                /*
                 * Set the slot value based upon the choices above (see Table 2 in Command 113 - Common Practice Spec 8.0)
                 */
                iassign(deviceVariables[ivar_value(destination_device_variable_code)].SOURCE_SLOT_NUMBER, slot+1);

                /*
                 * Get the shed time
                 */
                get_dictionary_string(choose_shed_time, disp_string1, slen1);
                GET_DEV_VAR_VALUE(disp_string1, deviceVariables[ivar_value(destination_device_variable_code)].SHED_TIME);

                /*
                 * Confirm new setting
                 */
                if(choice == 1) /* Address match mode */
                {
                    get_dictionary_string(enable_catch_data_using_specified_device, disp_string1, slen1);

                }
                else /* Catch burst */
                {
                    get_dictionary_string(enable_catch_data_using_burst_device, disp_string1, slen1);
                }

                /*
                 * Confirm enabling
                 */
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
            else if (choice == 0) /* Choice = 0 */
            {
                /*
                 * Determine if it is already disabled
                 */
                if (mode != 0)
                {
                    /*
                     * Confirm disabling
                     */
                    get_dictionary_string(disable_catch_data, disp_string1, slen1);
                    get_dictionary_string(yes_no, disp_string2, slen2);
                    retry = select_from_list(disp_string1, var_id, disp_string2);
                }
            }
            else
            {
                /*
                 * Not a valid selection (only values 0, 1, or 2 are accepted)
                 */
                get_dictionary_string(invalid_selection, disp_string1, slen1);
                DELAY(5, disp_string1);
                process_abort();
            }
        } while (retry);

        /*
         * Send Command 113 (Catch Device Variable)
         * Only act when new settings are confirmed from enabling or disabling above
         */
        send(113, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(113, status[STATUS_RESPONSE_CODE]);

            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(5, disp_string1);
            process_abort();
        }
    }
}

/*
 *****************************************************************************
 * The following commands have no section 6 heading in the Common Practice Command Spec
 * Volumetric Flow Commands
 *
 * Commands 522
 */
COMMAND write_volumetric_flow_classification
{
    NUMBER 522;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].CLASSIFICATION
        }
        REPLY
        {
            response_code, device_status,
            device_variable_code (INFO,INDEX),
            deviceVariables[device_variable_code].CLASSIFICATION
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}


/*****************************************************************************
 * The following commands are NOT RECOMMENDED FOR NEW DESIGNS
 */

COMMAND read_dynamic_variables_and_pv_analog_output
{
    NUMBER 61;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            PV.DAQ.ANALOG_UNITS, PV.DAQ.ANALOG_VALUE,
            PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE,
            SV.DEVICE_VARIABLE.DIGITAL_UNITS, SV.DEVICE_VARIABLE.DIGITAL_VALUE,
            TV.DEVICE_VARIABLE.DIGITAL_UNITS, TV.DEVICE_VARIABLE.DIGITAL_VALUE,
            QV.DEVICE_VARIABLE.DIGITAL_UNITS, QV.DEVICE_VARIABLE.DIGITAL_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,      no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,   xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        8,  MISC_WARNING, update_failure_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,   access_restricted_temp;
    }
}

COMMAND read_all_dynamic_variables
{
    NUMBER 110;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE,
            SV.DEVICE_VARIABLE.DIGITAL_UNITS, SV.DEVICE_VARIABLE.DIGITAL_VALUE,
            TV.DEVICE_VARIABLE.DIGITAL_UNITS, TV.DEVICE_VARIABLE.DIGITAL_VALUE,
            QV.DEVICE_VARIABLE.DIGITAL_UNITS, QV.DEVICE_VARIABLE.DIGITAL_VALUE
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,      no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,   xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        8,  MISC_WARNING, update_failure_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,   access_restricted_temp;
    }
}
