/*****************************************************************************
 *
 * File: Common12_2_Misc.ddl
 * Version: 12.02.1
 * Date: 1/14/2022
 *
 *****************************************************************************
 * Copyright (c) 2018-2022, FieldComm Group, All Rights Reserved
 *****************************************************************************
 *
 * Description: This file contains the DDL based model of the Subset of items
 *      necessary to implement <insert list here> from the Common Practice
 *      Command Specification Rev.12.0.  
 *      Specification.
 *
 * Source Specifications (These are revision numbers of the protocol spec which
 *                        may differ from the standard DD revision numbers.)
 *      Command Practice Command Specification      Rev.12.0
 *      Command Summary Specification               Rev.10.1
 *      Universal Command Specification             Rev.7.2
 *      Common Tables Specification                 Rev.25.0
 *
 *****************************************************************************
 */

/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
MANUFACTURER  STANDARD, DEVICE_TYPE _COMMON_PRACTICE_MISC, DEVICE_REVISION 12, DD_REVISION 2
 */
MANUFACTURER  STANDARD, DEVICE_TYPE 32, DEVICE_REVISION 12, DD_REVISION 2

/*****************************************************************************
 * Include Files
 *****************************************************************************
 */
#include "macros2.h"
#include "embedded_strings.h"
#include "methods.h"

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */
#define UPDATE_IN_PROGRESS      8
#define DEVICE_BUSY             32
#define MAXIMUM_RESET_RETRIES   10

/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.1
 * Data Link Layer Commands.  The commands affect the operation of the Data
 * Layer or Enhances its capabilities
 *
 * Commands 59 72 73
 */

COMMAND write_number_of_response_preambles
{
    NUMBER 59;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            response_preambles
        }
        REPLY
        {
            response_code, device_status, response_preambles
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,            no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR,   Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR,   Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,         xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,         in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        8,  DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,         access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,         busy_temp;
    }
}

/*
 *****************************************************************************
 * This Command causes the Field Device to visually or mechanically indicate
 * the reception of the command.  It assists in troubleshooting wiring or in
 * the identification of the field device
 */
COMMAND squawk
{
    NUMBER 72;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            squawk_control (INFO)
        }
        REPLY
        {
            response_code, device_status,
            squawk_control (INFO)
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR,   invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        9,  MODE_ERROR, unable_to_squawk_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR, busy_temp;
    }
}

METHOD makeItSquawk
{
//Remove new dictionary references from legacy binary format
    LABEL squawk_temp;
//Remove new dictionary references from legacy binary format
    HELP  squawk_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        int  count, // how many time to send the squawk command
        exit,       // flags when it is time to leave
        i;          // for loop counter
        int  slen1;
        int  slen2;
        int  choice;

        exit  = 1;
        count = 5;
        slen1 = 60;
        slen2 = 60;

        while (exit)
        {
            get_dictionary_string(begin_squawk, disp_string1, slen1);
            get_dictionary_string(choices_number_squawk_exit, disp_string2, slen2);

            squawk_control = 2; // Just squawk 1 time, for each time the command 72 is sent

            choice = SELECT_FROM_LIST(disp_string1, disp_string2);

            switch (choice)
            {
            case 0:
                get_dictionary_string(change_number_of_squawks, disp_string1, slen1);
                GET_LOCAL_VAR_VALUE(disp_string1, count);
                break;
            case 1:
                get_dictionary_string(squawk_in_process, disp_string1, slen1);
                for (i = 0; i < count; i++)
                {
                    send(72, status);
                    if (status[STATUS_RESPONSE_CODE])
                    {
                        display_response_status(72, status[STATUS_RESPONSE_CODE]);
                        break;  //Don't keep looping if the device returns an error response code.
                    }

                    DELAY(1, disp_string1);
                }
		/* Most device implementations should not need to have the squawk_control written back to zero
		   after a squawk once operation, but doing it here wouldn't hurt, and might be a good idea just
		   to ensure the device is not left in an unexpected state after the method ends. */
                squawk_control = 0; //Tell device that it can stop squawking
                send_command(72);
                break;
            case 2:
                exit = 0;
                break;
            }
        }
    }
}

/*
 *****************************************************************************
 * This Command allows for mechanical identification of a field device
 * Command 73 is used inherently by host applications to identify devices.
   Even though this is declared as operation COMMAND, a corresponding METHOD is not included
   for this command, because it does not make sense to do this within the DD itself.
   A host application would have to identify a transmitter first, before connecting to it
   using a DD. */
COMMAND findDevice
{
    NUMBER 73;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            IDENTIFY_REPLY_7
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
       38,  MODE_ERROR, Device_not_armed_temp;
    }
}


/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.4
 * Device management Commands
 *
 * Commands 41, 42, 71, 76, 89, 90, 95, 512, 513
 */

COMMAND perform_self_test /* Took out transmitter */
{
    NUMBER 41;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR, busy_temp;
    }
}

COMMAND perform_device_reset
{
    NUMBER 42;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR, busy_temp;
    }
}


COMMAND lock_device
{
    NUMBER 71;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            lock_device_code
        }
        REPLY
        {
            response_code, device_status, lock_device_code
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        10, DATA_ENTRY_ERROR, invalid_lock_code_temp;
//Remove new dictionary references from legacy binary format
        11, MODE_ERROR,       cannot_lock_device_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}

COMMAND read_lock_device_state
{
    NUMBER 76;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, lock_device_status_code
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR, busy_temp;
    }
}

/*
 * Methods for lock device, self test, device reset
 */

METHOD lock_unlock_device
{
//Remove new dictionary references from legacy binary format
    LABEL Lock_Unlock_Device_temp;
//Remove new dictionary references from legacy binary format
    HELP  lock_unlock_device_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int use_abort_method;
        int choice;
        int result;
        int locked;
        int slen1;
        int slen2;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        /*
         * Send command 76 to read the lock device state
         */
        send(76, status);

        if(status[STATUS_RESPONSE_CODE])
        {
            display_response_status(76, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        do
        {
            locked = (ivar_value(lock_device_status_code) & 0x01);

            if (locked)
            {
                /*  locked = 1 */
                get_dictionary_string(device_locked, disp_string1, slen1);
            }
            else
            {
                /* locked = 0 */
                get_dictionary_string(device_unlocked, disp_string1, slen1);
            }

            get_dictionary_string(yes_no, disp_string2, slen2);

            choice = SELECT_FROM_LIST(disp_string1, disp_string2);

            if (choice == 0) /* If yes, user wants to change the lock state */
            {
                if (locked == 0) /* If it is not locked */
                {
                    /* Lock the device */

                    /*
                     * Determine if user wants to lock the device, Temporarly or Permanently
                     */
                    get_dictionary_string(make_permanent, disp_string1, slen1);
                    result = SELECT_FROM_LIST(disp_string1, disp_string2);

                    if (result == 0) /* If yes */
                    {
                        slen1 = 90;
                        get_dictionary_string(prevent_config_change, disp_string1, slen1);
                        result = SELECT_FROM_LIST(disp_string1, disp_string2);

                        if (result == 0)      /* If yes */
                        {
                            /*
                             * Lock Configuration to all masters
                             */
                            iassign (lock_device_code, 3);
                        }
                        else
                        {
                            /*
                             * Lock it Permanently to other masters
                             */
                            iassign (lock_device_code, 2);
                        }
                    }
                    else
                    {
                        /*
                         * Lock it Temporarly
                         */
                        iassign(lock_device_code, 1);
                    }
                }
                else
                {
                    /*
                     * Unlock the device
                     */
                    iassign(lock_device_code, 0);
                }

                /*
                 * Send command 71 to lock or Unlock the device
                 */
                send(71, status);

                if(status[STATUS_RESPONSE_CODE])
                {
                    display_response_status(71, status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }

                /*
                 * Send command 76 to read the lock device state to update it as a result of
                 * sending command 71
                 */
                send(76, status);

                if(status[STATUS_RESPONSE_CODE])
                {
                    display_response_status(76, status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
            }
        } while(!choice);
    }
}

METHOD device_self_test
{
//Remove new dictionary references from legacy binary format
    LABEL self_test_temp;
//Remove new dictionary references from legacy binary format
    HELP  selftest_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char xmtr_status[MAX_XMTR_STATUS_LEN];
        char disp_string1[60];
        long x;
        int number_of_tries;
        int slen1;

        slen1 = 60;

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(about_to_perform_limited_selftest, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(performing_selftest, disp_string1, slen1);
        PUT_MESSAGE(disp_string1);

        send(41, status);

        if(status[STATUS_RESPONSE_CODE])
        {
            display_response_status(41, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        RETRY_ON_NO_DEVICE();
        number_of_tries = 0;

        do
        {
            get_more_status(status, xmtr_status);
        } while (((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                  || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
                 && (number_of_tries++ < MAXIMUM_RESET_RETRIES));

        if ((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
            || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
        {
            get_dictionary_string(device_stuck_in_self_test, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
        else if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(48, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
        else if (xmtr_status[0])
        {
            for (x = 1; x < 0x100; x <<= 1)
            {
                if (xmtr_status[0] & x)
                {
                    display_xmtr_status(device_specific_status_0, x);
                }
            }
        }
        else
        {
            get_dictionary_string(self_test_ok, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }

        process_abort(); /* This will call return_to_normal */
    }
}

METHOD device_master_reset
{
//Remove new dictionary references from legacy binary format
    LABEL Device_Reset_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_reset_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char xmtr_status[MAX_XMTR_STATUS_LEN];
        char disp_string1[60];
        long x;
        int number_of_tries;
        int slen1;

        slen1 = 60;

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(about_to_perform_device_reset, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(device_reset_in_progress, disp_string1, slen1);
        PUT_MESSAGE(disp_string1);

        send(42, status);

        if(status[STATUS_RESPONSE_CODE])
        {
            display_response_status(42, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        RETRY_ON_NO_DEVICE();
        number_of_tries = 0;

        do
        {
            get_more_status(status, xmtr_status);
        } while (((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                  || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
                 && (number_of_tries++ < MAXIMUM_RESET_RETRIES));

        if ((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
            || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
        {
            get_dictionary_string(device_stuck_in_device_reset, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
        else if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(48, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
        else if (xmtr_status[0])
        {
            for (x = 1; x < 0x100; x <<= 1)
            {
                if (xmtr_status[0] & x)
                {
                    display_xmtr_status(device_specific_status_0, x);
                }
            }
        }
        else
        {
            get_dictionary_string(device_reset_ok, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }

        process_abort(); /* This will call return_to_normal */
    }
}

#if __TOKVER__ >= 800
// Real time clock commands 89 and 90

VARIABLE current_date
{
//Remove new dictionary references from legacy binary format
    LABEL current_date_temp;
//Remove new dictionary references from legacy binary format
    HELP  current_date_help_temp;
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE DATE;
}

VARIABLE current_time
{
//Remove new dictionary references from legacy binary format
    LABEL current_time_temp;
//Remove new dictionary references from legacy binary format
    HELP  current_time_help_temp;
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE TIME_VALUE;
}

VARIABLE last_clock_date
{
//Remove new dictionary references from legacy binary format
    LABEL set_clock_date_temp;
//Remove new dictionary references from legacy binary format
    HELP  set_clock_date_help_temp;
    CLASS DEVICE;
    HANDLING READ;
    TYPE DATE;
}

VARIABLE last_clock_time
{
//Remove new dictionary references from legacy binary format
    LABEL set_clock_time_temp;
//Remove new dictionary references from legacy binary format
    HELP  set_clock_time_help_temp;
    CLASS DEVICE;
    HANDLING READ;
    TYPE TIME_VALUE;
}

COMMAND set_real_time_clock
{
    NUMBER 89;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            time_set (INFO), last_clock_date, last_clock_time, 0, 0
        }
        REPLY
        {
            response_code, device_status,
            time_set (INFO), last_clock_date, last_clock_time
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  DATA_ENTRY_ERROR, invalid_date_detected_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
    }
}

COMMAND read_real_time_clock
{
    NUMBER 90;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            current_date, current_time, last_clock_date,
            last_clock_time, real_time_clock_flag
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
    }
}


METHOD Set_Real_Time_Clock
{
//Remove new dictionary references from legacy binary format
    LABEL set_real_time_clock_temp;
//Remove new dictionary references from legacy binary format
    HELP set_real_time_clock_help_temp;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[200];
        char disp_string2[60];
        DD_STRING disp_enter_date;
        DD_STRING disp_enter_time;
        int choice;
        int slen1;
        int slen2;

        slen1 = 200;
        slen2 = 60;

        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(time_set_information, disp_string1, slen1);
        get_dictionary_string(yes_no_choices, disp_string2, slen2);

        choice = SELECT_FROM_LIST(disp_string1,disp_string2);

        switch(choice)
        {
           case 0:
              break;

           default:
              process_abort();
        }

        send(90, status);

        if ( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status(90,status[STATUS_RESPONSE_CODE]);
            process_abort();
        }

        // Update last set values with current values for user input convenience
        last_clock_date = current_date;
        last_clock_time = current_time;

        disp_enter_date = dictionary_string(enter_date);
        disp_enter_time = dictionary_string(enter_time);

        GET_DEV_VAR_VALUE(disp_enter_date, last_clock_date);
        GET_DEV_VAR_VALUE(disp_enter_time, last_clock_time);

        time_set = 1;  // Write Date and Time

        send(89,status);

        if ( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status(89,status[STATUS_RESPONSE_CODE]);
            process_abort();
        }

        send(90, status);

        get_dictionary_string(show_date_time, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
    }
}
#endif //__TOKVER__ >=  800

// Device Communication Statistics Command 95

VARIABLE STX_count
{
//Remove new dictionary references from legacy binary format
    LABEL stx_count_temp;
//Remove new dictionary references from legacy binary format
    HELP  stx_count_help_temp;
    CLASS HART & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}

VARIABLE ACK_count
{
//Remove new dictionary references from legacy binary format
    LABEL ack_count_temp;
//Remove new dictionary references from legacy binary format
    HELP  ack_count_help_temp;
    CLASS HART & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}

VARIABLE BACK_count
{
//Remove new dictionary references from legacy binary format
    LABEL back_count_temp;
//Remove new dictionary references from legacy binary format
    HELP  back_count_help_temp;
    CLASS HART & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}

COMMAND read_device_communication_statistics
{
    NUMBER 95;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            STX_count, ACK_count, BACK_count
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
    }
}

/**********************
* Country Code Commands
*/

COMMAND read_country_code
{
    NUMBER 512;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            country_code, si_control // See common tables DD
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
    }
}

COMMAND write_country_code
{
    NUMBER 513;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            country_code, si_control // See common tables DD
        }
        REPLY
        {
            response_code, device_status,
            country_code, si_control
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        9,  MODE_ERROR,       non_si_compliant_units_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}


 /* Command 106 is used inherently by host applications to manage delayed response buffers.
   Even though this is declared as operation COMMAND, a corresponding METHOD is not included
   for this command, because it does not make sense to do this within the DD itself.
   A host application would have to manage delayed responses outside of a DD method. */
COMMAND flush_delayed_responses
{
    NUMBER 106;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,      no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,   xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        8,  MISC_WARNING, all_but_running_flushed_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,   access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,   busy_temp;
    }
}


/*
 *****************************************************************************
 * Common Practice Command Spec
 * Section 6.13
 * Synchronized Device Action Commands
 *
 * Command 96 Read Synchronous Action
 * Command 97 Configure Synchronous Action
 * Command 98 Read Command Action
 * Command 99 Configure Command Action
 *
 * not implemented at this time.
 */



/*
 *****************************************************************************
 * The following commands have no section 6 heading in the Common Practice Command Spec
 * Device Location Commands
 *
 * Commands 516 - 521
 */
VARIABLE device_latitude
{
//Remove new dictionary references from legacy binary format
    LABEL device_latitude_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_latitude_help_temp;
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE  -90.0;
        MAX_VALUE  90.0;
        DISPLAY_FORMAT "010.5f";
        EDIT_FORMAT "010.5f";
    }
    CONSTANT_UNIT [degrees];
}

VARIABLE device_longitude
{
//Remove new dictionary references from legacy binary format
    LABEL device_longitude_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_longitude_help_temp;
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE  -180.0;
        MAX_VALUE  180.0;
        DISPLAY_FORMAT "010.5f";
        EDIT_FORMAT "010.5f";
    }
    CONSTANT_UNIT [degrees];
}

VARIABLE device_altitude
{
//Remove new dictionary references from legacy binary format
    LABEL device_altitude_temp;
//Remove new dictionary references from legacy binary format
    HELP  device_altitude_help_temp;
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE FLOAT;
    CONSTANT_UNIT [meter];
}

VARIABLE device_location_description
{
//Remove new dictionary references from legacy binary format
    LABEL location_description_temp;
//Remove new dictionary references from legacy binary format
    HELP  location_description_help_temp;
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE ASCII(32);
}

VARIABLE process_unit_tag
{
//Remove new dictionary references from legacy binary format
    LABEL process_unit_tag_temp;
//Remove new dictionary references from legacy binary format
    HELP  process_unit_tag_help_temp;
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE ASCII(32);
}

COMMAND read_device_location
{
    NUMBER 516;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST { }
        REPLY
        {
            response_code, device_status,
            device_latitude,
            device_longitude,
            location_mechanism,
            device_altitude
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR, busy_temp;
    }
}

COMMAND write_device_location
{
    NUMBER 517;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_latitude,
            device_longitude,
            location_mechanism,
            device_altitude
        }
        REPLY
        {
            response_code, device_status,
            device_latitude,
            device_longitude,
            location_mechanism,
            device_altitude
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        3,  DATA_ENTRY_ERROR, Parameter_too_large_temp;
//Remove new dictionary references from legacy binary format
        4,  DATA_ENTRY_ERROR, Parameter_too_small_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}

COMMAND read_location_description
{
    NUMBER 518;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST { }
        REPLY
        {
            response_code, device_status,
            device_location_description
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR, busy_temp;
    }
}

COMMAND write_location_description
{
    NUMBER 519;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            device_location_description
        }
        REPLY
        {
            response_code, device_status,
            device_location_description
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}

COMMAND read_process_unit_tag
{
    NUMBER 520;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST { }
        REPLY
        {
            response_code, device_status,
            process_unit_tag
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR, busy_temp;
    }
}

COMMAND write_process_unit_tag
{
    NUMBER 521;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            process_unit_tag
        }
        REPLY
        {
            response_code, device_status,
            process_unit_tag
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
//Remove new dictionary references from legacy binary format
        33, MODE_ERROR,       delayed_response_initiated_temp;
//Remove new dictionary references from legacy binary format
        34, MODE_ERROR,       delayed_response_running_temp;
//Remove new dictionary references from legacy binary format
        35, MODE_ERROR,       delayed_response_dead_temp;
//Remove new dictionary references from legacy binary format
        36, MODE_ERROR,       delayed_response_conflict_temp;
    }
}



/*
 *****************************************************************************
 *The Following commands are obsolete and NOT RECOMMENDED for new Designs
 *
 */

VARIABLE eeprom_select
{
//Remove new dictionary references from legacy binary format
    LABEL EEPROM_select_temp;
//Remove new dictionary references from legacy binary format
    HELP  eeprom_select_help_temp;
/*  APPHELP "For Burn requests, the Burn may not begin until the response that"
            " acknowledges the receipt of the variable has been sent back to the"
            " Master."
            " HART_VAR:EEPROM CONTROL"; */
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, burn_eeprom_temp, "help" },                              //Standard Dictionary
//Remove new dictionary references from legacy binary format
        { 1, Restore_Shadow_Ram_temp, "help" }                        //Standard Dictionary
    }
}

COMMAND eeprom_control
{
    NUMBER 39;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            eeprom_select
        }
        REPLY
        {
            response_code, device_status, eeprom_select
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        2,  DATA_ENTRY_ERROR, invalid_selection_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}

METHOD burn_eeprom
{
//Remove new dictionary references from legacy binary format
    LABEL burn_eeprom_temp;
//Remove new dictionary references from legacy binary format
    HELP  Burn_EEPROM_help_temp;
    DEFINITION
    {
        char status[3];
        char disp_string1[60];
        int slen1;

        slen1 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        assign_int(eeprom_select,0);

        get_dictionary_string(making_changes_permanent, disp_string1, slen1);
        DELAY(3, disp_string1);

        send(39, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(39, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(previous_err_seen_brn_eeprom, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
    }
}

VARIABLE unit_tag
{
//Remove new dictionary references from legacy binary format
    LABEL unit_tag_temp;
//Remove new dictionary references from legacy binary format
    HELP  unit_tag_help_temp;
/*  APPHELP "A Field Device may have sub applications that require being"
            " addressed individually. For this addressing the Tag variable"
            " associated with Command #11 should be used. This Tag is used to"
            " identify the application that controls the sub applications. This"
            " Text can be used by the user in any way. A recommended use is as"
            " a unique label to a plant that correlates to a Field Device label:"
            " a plant drawing, or on a Control System."
            " NOTE - HART does not recommend using this variable. If you have an"
            " application that controls the sub applications, all of these"
            " applications should be addressed using the HART Frame itself."
            " HART_VAR:UNIT TAG"; */
    CLASS DEVICE;
    TYPE PACKED_ASCII(8);
}

VARIABLE unit_descriptor
{
//Remove new dictionary references from legacy binary format
    LABEL unit_descriptor_temp;
//Remove new dictionary references from legacy binary format
    HELP  unit_descriptor_help_temp;
/*  APPHELP "A Field Device may have sub applications that require being"
            " addressed individually. This Descriptor"
            " is used to identify the application that controls the sub"
            " applications. This text can be used by the user in any way. There is"
            " no recommended use.
            " NOTE - HART does not recommend using this variable. If you have an"
            " application that controls the sub applications, all of these"
            " applications should be addressed using the HART Frame itself."
            " HART_VAR:UNIT DESCR"; */
    CLASS DEVICE;
    TYPE PACKED_ASCII(16);
}

VARIABLE unit_date
{
//Remove new dictionary references from legacy binary format
    LABEL unit_date_temp;
//Remove new dictionary references from legacy binary format
    HELP  unit_date_help_temp;
/*  APPHELP "A Field Device may have sub applications that require being"
            " addressed individually. This Date is used to"
            " identify the application that controls the sub applications."
            " This text can be used by the user in any way. There is"
            " no recommended use.
            " Format: Byte Closest to the Delimiter: Day, Next Byte: Month,
            " Byte Furthest from the Delimiter: Year-1900.
            " NOTE - HART does not recommend using this variable. If you have an"
            " application that controls the sub applications, all of these"
            " applications should be addressed using the HART Frame itself."
            " HART_VAR:UNIT DATE"; */
    CLASS DEVICE;
    TYPE DATE;
}

COMMAND read_unit_tag_descriptor_date
{
    NUMBER 57;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            unit_tag, unit_descriptor, unit_date
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,    no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR, xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR, access_restricted_temp;
    }
}

COMMAND write_unit_tag_descriptor_date
{
    NUMBER 58;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            unit_tag, unit_descriptor, unit_date
        }
        REPLY
        {
            response_code, device_status,
            unit_tag, unit_descriptor, unit_date
        }
    }
    RESPONSE_CODES
    {
//Remove new dictionary references from legacy binary format
        0,  SUCCESS,          no_command_specific_errors_temp;
//Remove new dictionary references from legacy binary format
        5,  DATA_ENTRY_ERROR, too_few_data_bytes_recieved_temp;
//Remove new dictionary references from legacy binary format
        6,  MISC_ERROR,       xmtr_specific_command_error_temp;
//Remove new dictionary references from legacy binary format
        7,  MODE_ERROR,       in_write_protect_mode_temp;
//Remove new dictionary references from legacy binary format
        16, MODE_ERROR,       access_restricted_temp;
//Remove new dictionary references from legacy binary format
        32, MODE_ERROR,       busy_temp;
    }
}
