/*****************************************************************************
 *
 * File:  PV1_3.ddl
 * Version: 1.3.0
 * Date: 01/14/2022
 *
 *****************************************************************************
 * Copyright (c) 2016-2022, FieldComm Group, All Rights Reserved
 *****************************************************************************
 *
 * Description: Defines the standard collections used by most DD's
 *
 *****************************************************************************
 */

/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
 */
MANUFACTURER STANDARD, DEVICE_TYPE _PV, DEVICE_REVISION 1, DD_REVISION 3


/*****************************************************************************
 * Include Files
 *****************************************************************************
 */
#include "macros2.h"
#include "embedded_strings.h"
#include "methods.h"

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */

/*****************************************************************************
 * Analog channel variables and collection
 */

VARIABLE loopCurrent
{
//Remove new dictionary references from legacy binary format
    LABEL Loop_Current_temp;
//Remove new dictionary references from legacy binary format
    HELP loop_current_help_temp;
    CLASS ANALOG_CHANNEL & DYNAMIC;
    HANDLING READ;
//Remove new dictionary references from legacy binary format
    CONSTANT_UNIT mA_temp;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        EDIT_FORMAT "8.3f";
    }
}

COLLECTION OF VARIABLE analog_io
{
//Remove new dictionary references from legacy binary format
    LABEL analog_input_temp;
//Remove new dictionary references from legacy binary format
    HELP analog_input_block_help_temp;
    MEMBERS
    {
        /* Analog Information */
//Remove new dictionary references from legacy binary format
        ANALOG_VALUE,           loopCurrent,        Loop_Current_temp,         loop_current_help_temp;
//Remove new dictionary references from legacy binary format
        ALARM_CODE,             loop_alarm_code,    loop_alarm_code_temp,      loop_alarm_code_help_temp;
//Remove new dictionary references from legacy binary format
        ANALOG_CHANNEL_FLAGS,   loop_flags,         loop_flags_temp,           loop_flags_help_temp;
    }
}


/*****************************************************************************
 * Range/ scaling variables and collection
 */

VARIABLE percentRange
{
//Remove new dictionary references from legacy binary format
    LABEL Percent_Range_temp;
//Remove new dictionary references from legacy binary format
    HELP percent_of_range_help_temp;
    CLASS RANGE & DYNAMIC;
    HANDLING READ;
//Remove new dictionary references from legacy binary format
    CONSTANT_UNIT Percent_temp;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE upperRange_value
{
//Remove new dictionary references from legacy binary format
    LABEL Upper_Range_Value_temp;
//Remove new dictionary references from legacy binary format
    HELP upper_range_value_help_temp;
    CLASS RANGE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        EDIT_FORMAT "8.3f";
    }
}

VARIABLE lowerRange_value
{
//Remove new dictionary references from legacy binary format
    LABEL Lower_Range_Value_temp;
//Remove new dictionary references from legacy binary format
    HELP lower_range_value_help_temp;
    CLASS RANGE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        EDIT_FORMAT "8.3f";
    }
}

COLLECTION OF VARIABLE scaling
{
//Remove new dictionary references from legacy binary format
    LABEL analog_input_temp;
//Remove new dictionary references from legacy binary format
    HELP analog_input_block_help_temp;
    MEMBERS
    {
        /* Range Information */
//Remove new dictionary references from legacy binary format
        PERCENT_RANGE,          percentRange,                                         Percent_Range_temp,        percent_of_range_help_temp;
//Remove new dictionary references from legacy binary format
        TRANSFER_FUNCTION,      transfer_function,                                    Transfer_Function_temp,    transfer_function_help_temp;
//Remove new dictionary references from legacy binary format
        UPPER_RANGE_VALUE,      upperRange_value,                                     Upper_Range_Value_temp,    upper_range_value_help_temp;
//Remove new dictionary references from legacy binary format
        LOWER_RANGE_VALUE,      lowerRange_value,                                     Lower_Range_Value_temp,    lower_range_value_help_temp;
//Remove new dictionary references from legacy binary format
        RANGE_UNITS,            deviceVariables[primary_variable_code].CONFIG_UNITS,  range_unit_temp,           [range_units_help];
    }
}

/*****************************************************************************
 * Now to declare the dynamic variables, and the associated collections
 *
 * All primary_variable collections must contain the following members
 */
COLLECTION OF COLLECTION primary_variable
{
    MEMBERS
    {
        DEVICE_VARIABLE,        deviceVariables[primary_variable_code];
        RANGING,                scaling;
        DAQ,                    analog_io;
    }
}

COLLECTION OF COLLECTION secondary_variable
{
    MEMBERS
    {
        DEVICE_VARIABLE,        deviceVariables[secondary_variable_code];
    }
}

COLLECTION OF COLLECTION tertiary_variable
{
    MEMBERS
    {
        DEVICE_VARIABLE,        deviceVariables[tertiary_variable_code];
    }
}

COLLECTION OF COLLECTION quaternary_variable
{
    MEMBERS
    {
        DEVICE_VARIABLE,        deviceVariables[quaternary_variable_code];
    }
}

/* This type of definition is needed in every device */
ARRAY OF COLLECTION dynamic_variables
{
    ELEMENTS
    {
//Remove new dictionary references from legacy binary format
        PRIMARY,        primary_variable,       pv_temp;
//Remove new dictionary references from legacy binary format
        SECONDARY,      secondary_variable,     sv_temp;
//Remove new dictionary references from legacy binary format
        TERTIARY,       tertiary_variable,      tv_temp;
//Remove new dictionary references from legacy binary format
        QUATERNARY,     quaternary_variable,    Qv_temp;
    }
}

/*
 * Relationships
 */

UNIT scaling_units_relation
{
    deviceVariables[primary_variable_code].CONFIG_UNITS:
        upperRange_value,
        lowerRange_value
}

WRITE_AS_ONE scaling_wao
{
    upperRange_value,
    lowerRange_value
}

