/*****************************************************************************
 *
 * File: Macros2.h
 * Version: 5.0.2
 * Date: 01/01/22
 *
 *****************************************************************************
 * Copyright (c) 2018-2022, FieldComm Group, All Rights Reserved
 *****************************************************************************
 *
 * Description: HART Device Description Macro Definitions
 *
 * Note - Macros2.h is now used instead of Macros.h as of March 2021 to be used
 *        with the following HARt standard DD revisions:
 *
 *        Tables          2801 or later
 *        Universal       0503, 0603, 0703 or later
 *        PV              0102, 0201 or later
 *        Common practice 0705, 0803, 0903, 0B01 or later
 *        Wireless        0103 or later.
 *
 *  The older version of the file (macros.h) remains only for the purpose of
 * rebuilding older DDs designed to use HART standard DD versions prior to
 * thos listed above.
 *
 *****************************************************************************
 */

#define USER_INTERFACE LOCAL
#define LOCAL_INTERFACE LOCAL_DISPLAY
#define ANALOG_CHANNEL ANALOG_OUTPUT
#define RANGE INPUT

#define I_NO_COMMAND_SPECIFIC_ERRORS     0
#define I_INVALID_SELECTION              2
#define I_PARAMETER_TOO_LARGE            3
#define I_PARAMETER_TOO_SMALL            4
#define I_TOO_FEW_DATA_BYTES             5
#define I_TRANSMITTER_SPECIFIC_ERROR     6
#define I_IN_WRITE_PROTECT_MODE          7
#define I_ACCESS_RESTRICTED             16


/*
 * Strings for the single definition response codes.
 */
//Remove new dictionary references from legacy binary format
#define S_NO_COMMAND_SPECIFIC_ERRORS no_command_specific_errors_temp
//Remove new dictionary references from legacy binary format
#define S_INVALID_SELECTION invalid_selection_temp
#define S_PARAMETER_TOO_LARGE [passed_parameter_too_large]
#define S_PARAMETER_TOO_SMALL [passed_parameter_too_small]
//Remove new dictionary references from legacy binary format
#define S_TOO_FEW_DATA_BYTES too_few_data_bytes_recieved_temp
//Remove new dictionary references from legacy binary format
#define S_TRANSMITTER_SPECIFIC_ERROR xmtr_specific_command_error_temp
//Remove new dictionary references from legacy binary format
#define S_IN_WRITE_PROTECT_MODE in_write_protect_mode_temp
//Remove new dictionary references from legacy binary format
#define S_ACCESS_RESTRICTED access_restricted_temp

#define PRIMARY         0
#define SECONDARY       1
#define TERTIARY        2
#define QUATERNARY      3

#define PV  dynamic_variables[PRIMARY]
#define SV  dynamic_variables[SECONDARY]
#define TV  dynamic_variables[TERTIARY]
#define QV  dynamic_variables[QUATERNARY]


/*
 * Common table references.
 */
#define ROSEMOUNT_MODEL_CODE(N) { N, rosemount_model_code(N) }
#define ROSEMOUNT_ANALYTICAL_MODEL_CODE(N) { N, rosemount_analytical_model_code(N) }
#define KAYRAY_MODEL_CODE(N) { N, kayray_model_code(N) }
#define MICRO_MOTION_MODEL_CODE(N) { N, micro_motion_model_code(N) }
#define UNITS_CODE(N) { N, units_code(N) }
#define TRANSFER_FUNCTION_CODE(N) { N, transfer_function_code(N) }
#define MATERIAL_CODE(N) { N, material_code(N) }
#define ALARM_SELECTION_CODE(N) { N, alarm_selection_code(N) }
#define WRITE_PROTECT_CODE(N) { N, write_protect_code(N) }
#define COMPANY_IDENTIFICATION_CODE(N) { N, company_identification_code(N) }
#define DEVICE_TYPE_CODE(N) { N, device_type_code(N) }
#define BURST_MODE_CONTROL_CODE(N) { N, burst_mode_control_code(N) }
#define PHYSICAL_SIGNALING_CODE(N) { N, physical_signaling_codes(N) }
#define FLAG_ASSIGNMENT(N) { N, std_flag_assignment(N) }
#define OPERATING_MODE_CODE(N) { N, std_operating_mode_code(N) }
#define ANALOG_OUTPUT_NUMBERS_CODE(N) { N, analog_output_numbers_code(N) }
#define LOOP_CURRENT_MODE_CODE(N)  { N, loop_current_mode_codes(N) }
#define EXTENDED_DEVICE_STATUS_CODE(N)  { N, std_extended_device_status_codes(N)}
#define TRIM_POINT_CODE(N)   { N, trim_point_codes(N) }
#define DEVICE_VARIABLE_FAMILY_CODE(N)  { N, device_variable_family_codes(N) }
#define DEVICE_VARIABLE_CLASSIFICATION_CODE(N)  { N, device_variable_classification_codes(N) }
#define WRITE_DEVICE_VARIABLE_CODE(N) { N, write_device_variable_codes(N) }
#define CAPTURE_MODE_CODE(N) { N, capture_mode_codes(N) }
#define LOCK_DEVICE_CODE(N) { N, lock_device_codes(N) }
#define ANALOG_CHANNEL_FLAG(N) { N, std_analog_channel_flags(N) }
#define UNITS_CODE_TEMPERATURE(n) { n, units_code_temperature(n) }
#define UNITS_CODE_PRESSURE(n) { n, units_code_pressure(n) }
#define UNITS_CODE_VOLUMETRIC_FLOW(n) { n, units_code_volumetric_flow(n) }
#define UNITS_CODE_VELOCITY(n) { n, units_code_velocity(n) }
#define UNITS_CODE_VOLUME(n) { n, units_code_volume(n) }
#define UNITS_CODE_LENGTH(n) { n, units_code_length(n) }
#define UNITS_CODE_TIME(n) { n, units_code_time(n) }
#define UNITS_CODE_MASS(n) { n, units_code_mass(n) }
#define UNITS_CODE_MASS_FLOW(n) { n, units_code_mass_flow(n) }
#define UNITS_CODE_MASS_PER_VOLUME(n) { n, units_code_mass_per_volume(n) }
#define UNITS_CODE_VISCOSITY(n) { n, units_code_viscosity(n) }
#define UNITS_CODE_ANGULAR_VELOCITY(n) { n, units_code_angular_velocity(n) }
#define UNITS_CODE_ENERGY(n) { n, units_code_energy(n) }
#define UNITS_CODE_FORCE(n) { n, units_code_force(n) }
#define UNITS_CODE_POWER(n) { n, units_code_power(n) }
#define UNITS_CODE_FREQUENCY(n) { n, units_code_frequency(n) }
#define UNITS_CODE_ANALYTICAL(n) { n, units_code_analytical(n) }
#define UNITS_CODE_CAPACITANCE(n) { n, units_code_capacitance(n) }
#define UNITS_CODE_EMF(n) { n, units_code_emf(n) }
#define UNITS_CODE_CURRENT(n) { n, units_code_current(n) }
#define UNITS_CODE_RESISTANCE(n) { n, units_code_resistance(n) }
#define UNITS_CODE_ANGLE(n) { n, units_code_angle(n) }
#define UNITS_CODE_VOLUME_PER_VOLUME(n) { n, units_code_volume_per_volume(n) }
#define UNITS_CODE_VOLUME_PER_MASS(n) { n, units_code_volume_per_mass(n) }
#define UNITS_CODE_CONCENTRATION(n) { n, units_code_concentration(n) }

#define PROCESS_DATA_STATUS_CODES \
        {0 , process_data_status(0)}, \
        {1 , process_data_status(1)}, \
        {2 , process_data_status(2)}, \
        {3 , process_data_status(3)}

#define LIMIT_STATUS_CODES \
        {0 , limit_status(0)}, \
        {1 , limit_status(1)}, \
        {2 , limit_status(2)}, \
        {3 , limit_status(3)}

#define SET_DEFAULT(x) REDEFINE DEFAULT_VALUE x
/*
 * The reply message of an identify request is defined here as a macro because
 * the same reply is returned by command 0,11,21,73 and 75 (the identify commands).
 */

#define IDENTIFY_REPLY_7 \
    response_code, device_status, 254, device_type, \
    request_preambles, universal_revision, transmitter_revision, \
    software_revision, hardware_revision <0xf8>, physical_signaling_code <0x07>, \
    device_flags, device_id, response_preambles, max_num_device_variables, \
    config_change_counter, extended_fld_device_status, manufacturer_id, \
    private_label_distributor, device_profile

#define IDENTIFY_REPLY_6 \
    response_code, device_status, 254, manufacturer_id, device_type, \
    request_preambles, universal_revision, transmitter_revision, \
    software_revision, hardware_revision <0xf8>, physical_signaling_code <0x07>, \
    device_flags, device_id, response_preambles, max_num_device_variables, \
    config_change_counter, extended_fld_device_status

#define IDENTIFY_REPLY_5 \
    response_code, device_status, 254, manufacturer_id, device_type, \
    request_preambles, universal_revision, transmitter_revision, \
    software_revision, hardware_revision <0xf8>, physical_signaling_code <0x07>, \
    device_flags, device_id

/* Need to break up the device type macros into multiple groups
   to avoid fatal tokenizer error "Work buffer overflow */
#define IMPORT_TABLES_DEVICE_TYPES_GROUP1() \
VARIABLE Ametek_model_code; \
VARIABLE Brooks_Instrument_model_code; \
VARIABLE Daniel_Industries_model_code; \
VARIABLE Delta_model_code; \
VARIABLE Endress_Hauser_model_code; \
VARIABLE Fischer_Porter_model_code; \
VARIABLE Fisher_Controls_model_code; \
VARIABLE SchneiderElectric_Foxboro_model_code; \
VARIABLE Fuji_model_code; \
VARIABLE Hartmann_Braun_model_code; \
VARIABLE Honeywell_model_code; \
VARIABLE kayray_model_code; \
VARIABLE ABB_model_code; \
VARIABLE M_System_Co_model_code; \
VARIABLE micro_motion_model_code; \
VARIABLE Moore_Industries_model_code; \
VARIABLE Moore_Products_model_code; \
VARIABLE Ronan_model_code; \
VARIABLE rosemount_model_code; \
VARIABLE Peek_Measurement_model_code; \
VARIABLE Actaris_Neptune_model_code; \
VARIABLE Siemens_model_code; \
VARIABLE Toshiba_model_code; \
VARIABLE rosemount_analytical_model_code; \
VARIABLE Valmet_Automation_model_code; \
VARIABLE Flowserve_model_code; \
VARIABLE Viatran_model_code; \
VARIABLE Azbil_Corporation_model_code; \
VARIABLE Yokogawa_model_code; \
VARIABLE Solartron_Mobrey_model_code; \
VARIABLE Princo_model_code; \
VARIABLE Smar_model_code; \
VARIABLE SchneiderElectric_Eckardt_model_code; \
VARIABLE Measurement_Technology_model_code; \
VARIABLE Samson_model_code; \
VARIABLE Sparling_Instruments_model_code; \
VARIABLE Krohne_model_code; \
VARIABLE Druck_model_code; \
VARIABLE SOR_model_code; \
VARIABLE Elcon_Instruments_model_code; \
VARIABLE Westlock_Controls_model_code; \
VARIABLE Drexelbrook_model_code; \
VARIABLE Rosemount_Tank_Radar_model_code; \
VARIABLE KTEK_model_code; \
VARIABLE Draeger_model_code; \
VARIABLE Meridian_model_code; \
VARIABLE BTG_model_code; \
VARIABLE Magnetrol_model_code; \
VARIABLE Neles_Finland_Oy_model_code; \
VARIABLE Milltronics_model_code; \
VARIABLE Anderson_Instrument_Company_model_code; \
VARIABLE Inor_model_code; \
VARIABLE Robertshaw_model_code; \
VARIABLE PEPPERL_FUCHS_model_code; \
VARIABLE Accutech_model_code; \
VARIABLE Baumer_model_code; \
VARIABLE Knick_model_code; \
VARIABLE VEGA_model_code; \
VARIABLE MTS_SYS_CORPS_model_code; \
VARIABLE Oval_model_code; \
VARIABLE GE_Masoneilan_model_code; \
VARIABLE Besta_model_code; \
VARIABLE Ohmart_model_code; \
VARIABLE Harold_Beck_and_Sons_model_code; \
VARIABLE Wika_model_code; \
VARIABLE Bopp_Reuther_Heinrichs_model_code; \
VARIABLE PR_Electronics_model_code; \
VARIABLE Rotork_Process_Controls_model_code; \
VARIABLE Apparatebau_Hundsbach_model_code; \
VARIABLE Dynisco_model_code; \
VARIABLE Direct_Measurement_model_code; \
VARIABLE Klay_model_code; \
VARIABLE CiDRA_CORP_model_code; \
VARIABLE Buerkert_Fluid_Control_Systems_model_code; \
VARIABLE Venture_Measurement_model_code; \
VARIABLE GLI_model_code; \
VARIABLE Paper_Machine_Components_model_code; \
VARIABLE Labom_model_code; \
VARIABLE Danfoss_model_code; \
VARIABLE Turbo_model_code; \
VARIABLE Tokyo_Keiso_model_code; \
VARIABLE SMC_model_code; \
VARIABLE Status_Instruments_model_code; \
VARIABLE Huakong_model_code; \
VARIABLE Autrol_Duon_model_code; \
VARIABLE Vortek_Instruments_model_code; \
VARIABLE Action_Instruments_model_code; \
VARIABLE Magtech_model_code; \
VARIABLE Rueger_model_code; \
VARIABLE Mettler_Toledo_model_code; \
VARIABLE Det_Tronics_model_code; \
VARIABLE Thermo_MeasureTech_model_code; \
VARIABLE Dezurik_model_code; \
VARIABLE Phase_Dynamics_model_code; \
VARIABLE Welltech_model_code; \
VARIABLE ENRAF_model_code; \
VARIABLE Metran_model_code; \
VARIABLE Milton_Roy_model_code; \
VARIABLE PMV_model_code; \
VARIABLE Turck_model_code; \
VARIABLE Panametrics_model_code; \
VARIABLE Stahl_model_code; \
VARIABLE Analytical_Technology_Inc_model_code; \
VARIABLE FINT_model_code; \
VARIABLE BERTHOLD_model_code; \
VARIABLE InterCorr_model_code; \
VARIABLE Sierra_Instruments_model_code; \
VARIABLE Fluid_Components_model_code; \
VARIABLE Meriam_Instrument_model_code; \
VARIABLE Invensys_model_code; \
VARIABLE CRANE_model_code; \
VARIABLE JTec_Associates_model_code; \
VARIABLE Tracerco_model_code; \
VARIABLE AGAR_model_code; \
VARIABLE Phoenix_Contact_model_code; \
VARIABLE Amer_Level_Instr_model_code; \
VARIABLE Hawk_model_code; \
VARIABLE Rotork_YTC_model_code; \
VARIABLE Pyromation_Inc_model_code; \
VARIABLE Satron_Instruments_model_code; \
VARIABLE BIFFI_model_code; \
VARIABLE BD_Sensors_model_code; \
VARIABLE APLISENS_model_code; \
VARIABLE Badger_Meter_model_code; \
VARIABLE HIMA_model_code; \
VARIABLE GP_50_model_code; \
VARIABLE Kongsberg_Maritime_model_code; \
VARIABLE ASA_SpA_model_code; \
VARIABLE Hengesbach_model_code; \
VARIABLE Lanlian_Instruments_model_code; \
VARIABLE Spectrum_Controls_model_code; \
VARIABLE Kajaani_Process_Measurements_model_code; \
VARIABLE FAFNIR_model_code; \
VARIABLE SICK_model_code; \
VARIABLE Spirax_Sarco_SRL_model_code; \
VARIABLE Tecfluid_SA_model_code; \
VARIABLE Roost_model_code; \
VARIABLE KOSO_model_code; \
VARIABLE Honeywell_Analytics_model_code; \
VARIABLE HEINRICHS_model_code; \
VARIABLE SIC_model_code; \
VARIABLE HACH_LANGE_model_code; \
VARIABLE Manometr_Kharkiv_model_code; \
VARIABLE Spectrex_model_code; \
VARIABLE REXA_model_code; \
VARIABLE General_Monitors_MSA_model_code; \
VARIABLE FCG_model_code; \
VARIABLE ExSaf_model_code; \
VARIABLE SEOJIN_INSTECH_model_code; \
VARIABLE TASI_FLOW_model_code; \
VARIABLE Power_Genex_Ltd_model_code;


#define IMPORT_TABLES_DEVICE_TYPES_GROUP2() \
VARIABLE APM_model_code; \
VARIABLE BIFOLD_ORANGE_model_code; \
VARIABLE BARTEC_model_code; \
VARIABLE Detcon_model_code; \
VARIABLE MSA_model_code; \
VARIABLE METROVAL_model_code; \
VARIABLE Etalon_Rus_model_code; \
VARIABLE JOGLER_model_code; \
VARIABLE KSB_model_code; \
VARIABLE Richter_CT_model_code; \
VARIABLE NET_SAFETY_model_code; \
VARIABLE SECanada_model_code; \
VARIABLE SUPCON_model_code; \
VARIABLE DKK_TOA_model_code; \
VARIABLE Dwyer_Instruments_model_code; \
VARIABLE FineTek_model_code; \
VARIABLE TopWorx_model_code; \
VARIABLE HOFFER_FLOW_CNTRLS_model_code; \
VARIABLE DUST_NETWORKS_model_code; \
VARIABLE Forbes_Marshall_model_code; \
VARIABLE All_Measures_Ltd_model_code; \
VARIABLE Pepperl_Fuchs_model_code; \
VARIABLE CSI_model_code; \
VARIABLE TC_Fluid_Control_model_code; \
VARIABLE Rohrback_Cosasco_model_code; \
VARIABLE AirSprite_model_code; \
VARIABLE Microcyber_model_code; \
VARIABLE TIG_model_code; \
VARIABLE ifm_prover_Gmbh_model_code; \
VARIABLE FLEXIM_model_code; \
VARIABLE TOKYO_KEIKI_model_code; \
VARIABLE SBEM_model_code; \
VARIABLE SkoFlo_Industries_model_code; \
VARIABLE StoneL_Corporation_model_code; \
VARIABLE EUREKA_FLOW_model_code; \
VARIABLE BEKA_associates_model_code; \
VARIABLE CAPSTAR_AUTOMATION_model_code; \
VARIABLE Pulsar_model_code; \
VARIABLE ELEMER_model_code; \
VARIABLE Soft_Tech_Group_model_code; \
VARIABLE FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code; \
VARIABLE ENOTEC_GmbH_model_code; \
VARIABLE Val_Controls_AS_model_code; \
VARIABLE Kurz_Instruments_model_code; \
VARIABLE E_Senza_Technologies_model_code; \
VARIABLE Crowcon_model_code; \
VARIABLE GEORGIN_model_code; \
VARIABLE Eldridge_Products_model_code; \
VARIABLE DDTOP_model_code; \
VARIABLE KSR_Kuebler_model_code; \
VARIABLE Armstrong_Intl_model_code; \
VARIABLE Hitachi_High_Tech_model_code; \
VARIABLE AMFLOW_model_code; \
VARIABLE Fluidwell_model_code; \
VARIABLE Mecon_GmbH_model_code; \
VARIABLE Fox_Thermal_Inst_model_code; \
VARIABLE EJMConsulting_model_code; \
VARIABLE Anton_Paar_model_code; \
VARIABLE Dinel_sro_model_code; \
VARIABLE Simtronics_ASA_model_code; \
VARIABLE INTEC_model_code; \
VARIABLE Softing_model_code; \
VARIABLE Fire_Sentry_Corp_model_code; \
VARIABLE FUTURE_INSTRUMENT_model_code; \
VARIABLE MOTOYAMA_model_code; \
VARIABLE ifak_system_model_code; \
VARIABLE MORGAN_KOREA_model_code; \
VARIABLE AWIATECH_model_code; \
VARIABLE Chentronics_model_code; \
VARIABLE United_Electric_model_code; \
VARIABLE Shanghai_Sinoto_Instrument_Co_Ltd_model_code; \
VARIABLE MATSUSHIMA_model_code; \
VARIABLE Westcontrol_AS_model_code; \
VARIABLE Oldham_model_code; \
VARIABLE WooriTG_model_code; \
VARIABLE trianglemicrosol_model_code; \
VARIABLE Nivis_LLC_model_code; \
VARIABLE SKF_USA_model_code; \
VARIABLE ESP_Safety_model_code; \
VARIABLE HollySys_model_code; \
VARIABLE GF_Signet_model_code; \
VARIABLE Sierra_Monitor_model_code; \
VARIABLE Schneider_Electric_model_code; \
VARIABLE Emerson_model_code; \
VARIABLE ABM_Sensor_Tech_model_code; \
VARIABLE RIKEN_KEIKI_model_code; \
VARIABLE EEPL_India_model_code; \
VARIABLE Thermal_Inst_model_code; \
VARIABLE HYDAC_ELECTRONIC_model_code; \
VARIABLE GNEUSS_model_code; \
VARIABLE Actuation_Technologies_model_code; \
VARIABLE Roxar_model_code; \
VARIABLE GASTRON_model_code; \
VARIABLE HAMILTON_Bonaduz_model_code; \
VARIABLE Nobel_Weighing_Systems_model_code; \
VARIABLE ECD_model_code; \
VARIABLE Sensor_Electronics_model_code; \
VARIABLE Senscient_model_code; \
VARIABLE Shanghai_Kent_Instrument_Co_Ltd_model_code; \
VARIABLE Scott_Safety_model_code; \
VARIABLE Welkin_Co_Ltd_model_code; \
VARIABLE Teplopribor_model_code; \
VARIABLE Auburn_Systems_model_code; \
VARIABLE Therm_O_Disc_model_code; \
VARIABLE SWISA_INSTRUMENT_model_code; \
VARIABLE Sage_Metering_model_code; \
VARIABLE Cooper_Bussmann_model_code; \
VARIABLE FSUE_VNIIA_model_code; \
VARIABLE Groveley_Detection_model_code; \
VARIABLE Hengyang_No_Photoelectric_Info_Tech_Co_model_code; \
VARIABLE Imtex_Controls_Ltd_model_code; \
VARIABLE AUMA_model_code; \
VARIABLE Novus_Automation_model_code; \
VARIABLE MISTRAS_Group_model_code; \
VARIABLE Gefran_SpA_model_code; \
VARIABLE Lumasense_Technologies_model_code; \
VARIABLE Three_S_model_code; \
VARIABLE Euromag_International_model_code; \
VARIABLE Isoil_Hemina_model_code; \
VARIABLE Walsn_model_code; \
VARIABLE JUMO_GmbH_Co_KG_model_code; \
VARIABLE NEW_COSMOS_ELECTRIC_model_code; \
VARIABLE Nuoan_model_code; \
VARIABLE SPA_ANALITPRIBOR_model_code; \
VARIABLE Gasensor_model_code; \
VARIABLE Flowline_model_code; \
VARIABLE SWAN_Analytical_Instruments_model_code; \
VARIABLE Sensia_model_code; \
VARIABLE Hoerbiger_model_code; \
VARIABLE Shanghai_Yinuo_model_code; \
VARIABLE LIMACO_model_code; \
VARIABLE Rhosonics_Analytical_BV_model_code; \
VARIABLE WISE_Sensing_model_code; \
VARIABLE EMIS_model_code; \
VARIABLE VALTEK_SULAMERIC_model_code; \
VARIABLE Elimko_Ltd_Sti_model_code; \
VARIABLE GODA_Instruments_model_code; \
VARIABLE DVG_Automation_model_code; \
VARIABLE OTIS_Instruments_model_code; \
VARIABLE KANEKO_SANGYO_model_code; \
VARIABLE Barksdale_Control_Products_model_code; \
VARIABLE NIVUS_model_code; \
VARIABLE Guangzhou_Xitai_model_code; \
VARIABLE VZLJOT_model_code; \
VARIABLE ROCKSENSOR_model_code; \
VARIABLE DELTA_CONTROLS_model_code; \
VARIABLE WISE_CONTROL_INC_model_code; \
VARIABLE CURTO_model_code; \
VARIABLE Uniphos_Envirotronics_model_code; \
VARIABLE Three_Sci_model_code; \
VARIABLE SHENYANG_LANSHEN_INSTRUMENT_CO_LTD_model_code; \
VARIABLE SENKO_model_code; \
VARIABLE Toshniwal_Industries_Private_Limited_model_code; \
VARIABLE JSC_EPF_SIBNA_model_code; \
VARIABLE Diyuan_model_code; \
VARIABLE New_Power_model_code; \
VARIABLE Hanwei_Electronics_Group_Corporation_model_code; \
VARIABLE SEG_model_code; \
VARIABLE BALDOTA_CONTROL_model_code; \
VARIABLE GASDNA_CO_LTD_model_code; \
VARIABLE BONRAY_model_code;

#define IMPORT_TABLES_BASE() \
    VARIABLE device_status; \
    VARIABLE physical_signaling_code; \
    VARIABLE device_flags; \
    VARIABLE write_protect; \
    VARIABLE loop_alarm_code; \
    VARIABLE transfer_function;

#define IMPORT_TABLES_HART(u) \
    TABLE_HART_##u

#define TABLE_HART_7 \
    VARIABLE manufacturer_id; \
    VARIABLE private_label_distributor; \
    VARIABLE device_type; \
    VARIABLE device_profile; \
    VARIABLE loop_current_mode; \
    VARIABLE loop_flags;

#define TABLE_HART_6 \
    VARIABLE loop_current_mode; \
    VARIABLE loop_flags;

#define TABLE_HART_5 /*Noting mandatory for Univ rev 5*/

#define IMPORT_TABLES_DEVELOPER() \
    VARIABLE device_variable_code_codes; \
    VARIABLE material_code; \
    VARIABLE write_device_variable_codes; \
    VARIABLE device_variable_family_codes; \
    VARIABLE device_variable_classification_codes; \
    VARIABLE trim_point_codes; \
    VARIABLE capture_mode_codes; \
    VARIABLE dev_var_property_codes; \
    VARIABLE process_data_status; \
    VARIABLE limit_status; \
    VARIABLE std_device_family_status;

#define IMPORT_TABLES_BURST_VAL_ARRAY(u) \
    TABLE_BURST_VAL_ARRAY_##u
    
#define TABLE_BURST_VAL_ARRAY_5 \
    VARIABLE burst_mode_select;

#define TABLE_BURST_VAL_ARRAY_6 \
    TABLE_BURST_VAL_ARRAY_5

#define TABLE_BURST_VAL_ARRAY_7 \
    TABLE_BURST_VAL_ARRAY_6 \
    VARIABLE burst_trigger_units; \
    VARIABLE burst_trigger_classification; \
    VARIABLE burst_message_trigger_mode;

#define IMPORT_TABLES_BURST_REF_ARRAY(u,b) \
    TABLE_BURST_REF_ARRAY_##u##_##b
    
#define TABLE_BURST_REF_ARRAY_5_1 \
    VARIABLE burst_mode_select_0;

#define TABLE_BURST_REF_ARRAY_6_1 \
    TABLE_BURST_REF_ARRAY_5_1

#define TABLE_BURST_REF_ARRAY_7_1 \
    VARIABLE burst_trigger_units; \
    VARIABLE burst_trigger_classification; \
    VARIABLE burst_message_trigger_mode; \
    VARIABLE burst_mode_select_0;

#define TABLE_BURST_REF_ARRAY_7_2 \
    TABLE_BURST_REF_ARRAY_7_1 \
    VARIABLE burst_trigger_units_1; \
    VARIABLE burst_trigger_class_1; \
    VARIABLE burst_trigger_mode_1; \
    VARIABLE burst_mode_select_1;

#define TABLE_BURST_REF_ARRAY_7_3 \
    TABLE_BURST_REF_ARRAY_7_2 \
    VARIABLE burst_trigger_units_2; \
    VARIABLE burst_trigger_class_2; \
    VARIABLE burst_trigger_mode_2; \
    VARIABLE burst_mode_select_2;

#define TABLE_BURST_REF_ARRAY_7_4 \
    TABLE_BURST_REF_ARRAY_7_3 \
    VARIABLE burst_trigger_units_3; \
    VARIABLE burst_trigger_class_3; \
    VARIABLE burst_trigger_mode_3; \
    VARIABLE burst_mode_select_3;

#define IMPORT_TABLES_EVENT_VAL_ARRAY() \
    VARIABLE event_status; \
    VARIABLE device_status_mask; \
    VARIABLE extended_fld_device_status_mask; \
    VARIABLE dev_operating_mode_mask; \
    VARIABLE standardized_status_0_mask; \
    VARIABLE standardized_status_1_mask; \
    VARIABLE standardized_status_2_mask; \
    VARIABLE standardized_status_3_mask; \
    VARIABLE analog_channel_saturated1_mask; \
    VARIABLE analog_channel_fixed1_mask; \
    VARIABLE event_notification_control_n; \
    VARIABLE device_status_latched_value; \
    VARIABLE extended_fld_device_status_latched_value; \
    VARIABLE std_operating_mode_latched_value; \
    VARIABLE standardized_status_0_latched_value; \
    VARIABLE standardized_status_1_latched_value; \
    VARIABLE standardized_status_2_latched_value; \
    VARIABLE standardized_status_3_latched_value; \
    VARIABLE analog_channel_saturated1_latched_value; \
    VARIABLE analog_channel_fixed1_latched_value;

#define IMPORT_TABLES_EVENT_REF_ARRAY(e) \
    TABLE_EVENT_REF_ARRAY_##e

#define TABLE_EVENT_REF_ARRAY_1 \
    VARIABLE event_status; \
    VARIABLE device_status_mask; \
    VARIABLE extended_fld_device_status_mask; \
    VARIABLE dev_operating_mode_mask; \
    VARIABLE standardized_status_0_mask; \
    VARIABLE standardized_status_1_mask; \
    VARIABLE standardized_status_2_mask; \
    VARIABLE standardized_status_3_mask; \
    VARIABLE analog_channel_saturated1_mask; \
    VARIABLE analog_channel_fixed1_mask; \
    VARIABLE event_notification_control_n; \
    VARIABLE device_status_latched_value; \
    VARIABLE extended_fld_device_status_latched_value; \
    VARIABLE std_operating_mode_latched_value; \
    VARIABLE standardized_status_0_latched_value; \
    VARIABLE standardized_status_1_latched_value; \
    VARIABLE standardized_status_2_latched_value; \
    VARIABLE standardized_status_3_latched_value; \
    VARIABLE analog_channel_saturated1_latched_value; \
    VARIABLE analog_channel_fixed1_latched_value;

#define TABLE_EVENT_REF_ARRAY_2 \
    TABLE_EVENT_REF_ARRAY_1 \
    VARIABLE event_status_1; \
    VARIABLE device_status_mask_1; \
    VARIABLE extended_fld_device_status_mask_1; \
    VARIABLE dev_operating_mode_mask_1; \
    VARIABLE standardized_status_0_mask_1; \
    VARIABLE standardized_status_1_mask_1; \
    VARIABLE standardized_status_2_mask_1; \
    VARIABLE standardized_status_3_mask_1; \
    VARIABLE analog_channel_saturated1_mask_1; \
    VARIABLE analog_channel_fixed1_mask_1; \
    VARIABLE event_notification_control_n_1; \
    VARIABLE device_status_latched_value_1; \
    VARIABLE extended_fld_device_status_latched_value_1; \
    VARIABLE dev_operating_mode_latched_value_1; \
    VARIABLE standardized_status_0_latched_value_1; \
    VARIABLE standardized_status_1_latched_value_1; \
    VARIABLE standardized_status_2_latched_value_1; \
    VARIABLE standardized_status_3_latched_value_1; \
    VARIABLE analog_channel_saturated1_latched_value_1; \
    VARIABLE analog_channel_fixed1_latched_value_1;

#define IMPORT_TABLES_EVENT_MANAGER() \
    VARIABLE event_manager_registration_control; \
    VARIABLE event_manager_registration_status;

#define IMPORT_TABLES_IO_ADAPTER_V9() \
    VARIABLE master_mode; \
    VARIABLE subdev_expanded_device_type; \
    VARIABLE subdev_manufacturer_id;

#define IMPORT_TABLES_IO_ADAPTER_V11() \
    IMPORT_TABLES_IO_ADAPTER_V9() \
    VARIABLE subdev_distributor; \
    VARIABLE subdev_dev_profile;

#define IMPORT_TABLES_IO_ASSIGN() \
    VARIABLE subdev_assign_status; \
    VARIABLE subdevice_transfer_code;

#define IMPORT_TABLES_LOCATION() \
    VARIABLE location_mechanism;

#define IMPORT_TABLES_LOCK() \
    VARIABLE lock_device_code; \
    VARIABLE lock_device_status_code;

#define IMPORT_TABLES_SI_UNIT() \
    VARIABLE country_code; \
    VARIABLE si_control;

#define IMPORT_TABLES_SQUAWK() \
    VARIABLE squawk_control;

#define IMPORT_TABLES_STANDARD_STATUS(u, n) \
    TABLE_STATUS_##u##_##n

#define TABLE_STATUS_5_1 //None
#define TABLE_STATUS_5_2 TABLE_STATUS_5_1
#define TABLE_STATUS_5_3 TABLE_STATUS_5_2
#define TABLE_STATUS_5_4 TABLE_STATUS_5_3
#define TABLE_STATUS_5_5 TABLE_STATUS_5_4
#define TABLE_STATUS_5_6 TABLE_STATUS_5_5
#define TABLE_STATUS_5_7 TABLE_STATUS_5_6
#define TABLE_STATUS_5_8 TABLE_STATUS_5_7
#define TABLE_STATUS_5_9 TABLE_STATUS_5_8
#define TABLE_STATUS_5_10 TABLE_STATUS_5_9

#define TABLE_STATUS_5_11 \
    TABLE_STATUS_5_10 \
    VARIABLE analog_channel_saturated1;

#define TABLE_STATUS_5_12 TABLE_STATUS_5_11
#define TABLE_STATUS_5_13 TABLE_STATUS_5_12

#define TABLE_STATUS_5_14 \
    TABLE_STATUS_5_13 \
    VARIABLE analog_channel_fixed1;    

#define TABLE_STATUS_5_15 TABLE_STATUS_5_14
#define TABLE_STATUS_5_16 TABLE_STATUS_5_15
#define TABLE_STATUS_5_17 TABLE_STATUS_5_16
#define TABLE_STATUS_5_18 TABLE_STATUS_5_17
#define TABLE_STATUS_5_19 TABLE_STATUS_5_18
#define TABLE_STATUS_5_20 TABLE_STATUS_5_19
#define TABLE_STATUS_5_21 TABLE_STATUS_5_20
#define TABLE_STATUS_5_22 TABLE_STATUS_5_21
#define TABLE_STATUS_5_23 TABLE_STATUS_5_22
#define TABLE_STATUS_5_24 TABLE_STATUS_5_23
#define TABLE_STATUS_5_25 TABLE_STATUS_5_24


#define TABLE_STATUS_6_1 //None
#define TABLE_STATUS_6_2 TABLE_STATUS_6_1
#define TABLE_STATUS_6_3 TABLE_STATUS_6_2
#define TABLE_STATUS_6_4 TABLE_STATUS_6_3
#define TABLE_STATUS_6_5 TABLE_STATUS_6_4
#define TABLE_STATUS_6_6 TABLE_STATUS_6_5
#define TABLE_STATUS_6_7 \
    TABLE_STATUS_6_6 \
    VARIABLE extended_fld_device_status;

#define TABLE_STATUS_6_8 TABLE_STATUS_6_7
#define TABLE_STATUS_6_9 TABLE_STATUS_6_8
#define TABLE_STATUS_6_10 TABLE_STATUS_6_9

#define TABLE_STATUS_6_11 \
    TABLE_STATUS_6_10 \
    VARIABLE analog_channel_saturated1;

#define TABLE_STATUS_6_12 TABLE_STATUS_6_11
#define TABLE_STATUS_6_13 TABLE_STATUS_6_12

#define TABLE_STATUS_6_14 \
    TABLE_STATUS_6_13 \
    VARIABLE analog_channel_fixed1;    

#define TABLE_STATUS_6_15 TABLE_STATUS_6_14
#define TABLE_STATUS_6_16 TABLE_STATUS_6_15
#define TABLE_STATUS_6_17 TABLE_STATUS_6_16
#define TABLE_STATUS_6_18 TABLE_STATUS_6_17
#define TABLE_STATUS_6_19 TABLE_STATUS_6_18
#define TABLE_STATUS_6_20 TABLE_STATUS_6_19
#define TABLE_STATUS_6_21 TABLE_STATUS_6_20
#define TABLE_STATUS_6_22 TABLE_STATUS_6_21
#define TABLE_STATUS_6_23 TABLE_STATUS_6_22
#define TABLE_STATUS_6_24 TABLE_STATUS_6_23
#define TABLE_STATUS_6_25 TABLE_STATUS_6_24


#define TABLE_STATUS_7_1 //None
#define TABLE_STATUS_7_2 TABLE_STATUS_7_1
#define TABLE_STATUS_7_3 TABLE_STATUS_7_2
#define TABLE_STATUS_7_4 TABLE_STATUS_7_3
#define TABLE_STATUS_7_5 TABLE_STATUS_7_4
#define TABLE_STATUS_7_6 TABLE_STATUS_7_5
#define TABLE_STATUS_7_7 \
    TABLE_STATUS_7_6 \
    VARIABLE extended_fld_device_status;

#define TABLE_STATUS_7_8 TABLE_STATUS_7_7

#define TABLE_STATUS_7_9 \
    TABLE_STATUS_7_8 \
    VARIABLE standardized_status_0;

#define TABLE_STATUS_7_10 \
    TABLE_STATUS_7_9 \
    VARIABLE standardized_status_1;

#define TABLE_STATUS_7_11 \
    TABLE_STATUS_7_10 \
    VARIABLE analog_channel_saturated1;

#define TABLE_STATUS_7_12 \
    TABLE_STATUS_7_11 \
    VARIABLE standardized_status_2;

#define TABLE_STATUS_7_13 \
    TABLE_STATUS_7_12 \
    VARIABLE standardized_status_3;

#define TABLE_STATUS_7_14 \
    TABLE_STATUS_7_13 \
    VARIABLE analog_channel_fixed1;

#define TABLE_STATUS_7_15 TABLE_STATUS_7_14
#define TABLE_STATUS_7_16 TABLE_STATUS_7_15
#define TABLE_STATUS_7_17 TABLE_STATUS_7_16
#define TABLE_STATUS_7_18 TABLE_STATUS_7_17
#define TABLE_STATUS_7_19 TABLE_STATUS_7_18
#define TABLE_STATUS_7_20 TABLE_STATUS_7_19
#define TABLE_STATUS_7_21 TABLE_STATUS_7_20
#define TABLE_STATUS_7_22 TABLE_STATUS_7_21
#define TABLE_STATUS_7_23 TABLE_STATUS_7_22
#define TABLE_STATUS_7_24 TABLE_STATUS_7_23
#define TABLE_STATUS_7_25 TABLE_STATUS_7_24


#define IMPORT_TABLES_READ_TIME() \
    VARIABLE real_time_clock_flag;

#define IMPORT_TABLES_READ_WRITE_TIME() \
    VARIABLE time_set; \
    VARIABLE real_time_clock_flag;

#define IMPORT_TABLES_TREND_REF_ARRAY() \
    VARIABLE trend_control; \
    VARIABLE trend_digital_units; \
    VARIABLE trend_classification; \
    VARIABLE trend_value_data_quality; \
    VARIABLE std_trend_value_0_data_quality_1; \
    VARIABLE std_trend_value_0_data_quality_2; \
    VARIABLE std_trend_value_0_data_quality_3; \
    VARIABLE std_trend_value_0_data_quality_4; \
    VARIABLE std_trend_value_0_data_quality_5; \
    VARIABLE std_trend_value_0_data_quality_6; \
    VARIABLE std_trend_value_0_data_quality_7; \
    VARIABLE std_trend_value_0_data_quality_8; \
    VARIABLE std_trend_value_0_data_quality_9; \
    VARIABLE std_trend_value_0_data_quality_10; \
    VARIABLE std_trend_value_0_data_quality_11; \
    VARIABLE trend_value_limit_status; \
    VARIABLE std_trend_value_0_limit_status_1; \
    VARIABLE std_trend_value_0_limit_status_2; \
    VARIABLE std_trend_value_0_limit_status_3; \
    VARIABLE std_trend_value_0_limit_status_4; \
    VARIABLE std_trend_value_0_limit_status_5; \
    VARIABLE std_trend_value_0_limit_status_6; \
    VARIABLE std_trend_value_0_limit_status_7; \
    VARIABLE std_trend_value_0_limit_status_8; \
    VARIABLE std_trend_value_0_limit_status_9; \
    VARIABLE std_trend_value_0_limit_status_10; \
    VARIABLE std_trend_value_0_limit_status_11; \
    VARIABLE std_trend_value_device_family_status; \
    VARIABLE std_trend_value_0_device_family_status_1; \
    VARIABLE std_trend_value_0_device_family_status_2; \
    VARIABLE std_trend_value_0_device_family_status_3; \
    VARIABLE std_trend_value_0_device_family_status_4; \
    VARIABLE std_trend_value_0_device_family_status_5; \
    VARIABLE std_trend_value_0_device_family_status_6; \
    VARIABLE std_trend_value_0_device_family_status_7; \
    VARIABLE std_trend_value_0_device_family_status_8; \
    VARIABLE std_trend_value_0_device_family_status_9; \
    VARIABLE std_trend_value_0_device_family_status_10; \
    VARIABLE std_trend_value_0_device_family_status_11;

#define IMPORT_TABLES_TREND_VAL_ARRAY() \
    VARIABLE trend_control; \
    VARIABLE trend_digital_units; \
    VARIABLE trend_classification; \
    VARIABLE trend_value_data_quality; \
    VARIABLE trend_value_limit_status; \
    VARIABLE std_trend_value_device_family_status;

#define IMPORT_TABLES_WIRELESS() \
    VARIABLE wireless_mode; \
    VARIABLE join_status; \
    VARIABLE join_mode; \
    VARIABLE wireless_module_device_type; \
    VARIABLE wireless_module_manufacturer_id;

#define IMPORT_TABLES_WIRELESS_DEVICE_CAPABILITIES() \
    VARIABLE wireless_capability; \
    VARIABLE power_source;

#define IMPORT_TABLES_WIRELESS_NETWORK_ACCESS() \
    VARIABLE network_access_mode;

#define IMPORT_TABLES_MULTI_AO(u) \
    MULTI_AO_##u

#define MULTI_AO_5 \
    VARIABLE SVanalogChannelAlarmCode; \
    VARIABLE TVanalogChannelAlarmCode; \
    VARIABLE QVanalogChannelAlarmCode; \
    VARIABLE SVtransferFunction; \
    VARIABLE TVtransferFunction; \
    VARIABLE QVtransferFunction;

#define MULTI_AO_6 \
    MULTI_AO_5 \
    VARIABLE SVanalogChannelFlags; \
    VARIABLE TVanalogChannelFlags; \
    VARIABLE QVanalogChannelFlags;

#define MULTI_AO_7 \
    MULTI_AO_6

#define IMPORT_UNIVERSAL_BASE(u) \
    UNIVERSAL_BASE_##u
    
#define UNIVERSAL_BASE_5 \
    VARIABLE manufacturer_id; \
    VARIABLE private_label_distributor; \
    VARIABLE device_type; \
    VARIABLE response_code; \
    VARIABLE comm_status; \
    VARIABLE request_preambles; \
    VARIABLE universal_revision; \
    VARIABLE transmitter_revision; \
    VARIABLE software_revision; \
    VARIABLE hardware_revision; \
    VARIABLE device_id; \
    VARIABLE tag; \
    VARIABLE polling_address; \
    VARIABLE message; \
    VARIABLE descriptor; \
    VARIABLE date; \
    VARIABLE final_assembly_number; \
    COMMAND read_unique_identifier; \
    COMMAND read_unique_identifier_with_tag; \
    COMMAND read_pv; \
    COMMAND read_pv_current_and_percent_range; \
    COMMAND read_dynamic_variables_and_pv_current; \
    COMMAND write_polling_address; \
    COMMAND read_pv_sensor_info; \
    COMMAND read_pv_output_info; \
    COMMAND read_message; \
    COMMAND write_message; \
    COMMAND read_tag_descriptor_date; \
    COMMAND write_tag_descriptor_date; \
    COMMAND read_final_assembly_number; \
    COMMAND write_final_assembly_number;

#define UNIVERSAL_BASE_6 \
    VARIABLE manufacturer_id; \
    VARIABLE private_label_distributor; \
    VARIABLE device_type; \
    VARIABLE response_preambles; \
    VARIABLE max_num_device_variables; \
    VARIABLE config_change_counter; \
    VARIABLE longTag; \
    VARIABLE device_variable_code_1; \
    VARIABLE device_variable_code_2; \
    VARIABLE device_variable_code_3; \
    VARIABLE device_variable_code_4; \
    COMMAND read_unique_identifier_with_long_tag; \
    COMMAND read_device_variables_and_status; \
    COMMAND read_loop_configuration; \
    COMMAND read_dynamic_variable_classification; \
    COMMAND read_long_tag; \
    COMMAND write_long_tag; \
    VARIABLE response_code; \
    VARIABLE comm_status; \
    VARIABLE request_preambles; \
    VARIABLE universal_revision; \
    VARIABLE transmitter_revision; \
    VARIABLE software_revision; \
    VARIABLE hardware_revision; \
    VARIABLE device_id; \
    VARIABLE tag; \
    VARIABLE polling_address; \
    VARIABLE message; \
    VARIABLE descriptor; \
    VARIABLE date; \
    VARIABLE final_assembly_number; \
    COMMAND read_unique_identifier; \
    COMMAND read_unique_identifier_with_tag; \
    COMMAND read_pv; \
    COMMAND read_pv_current_and_percent_range; \
    COMMAND read_dynamic_variables_and_pv_current; \
    COMMAND write_polling_address; \
    COMMAND read_pv_sensor_info; \
    COMMAND read_pv_output_info; \
    COMMAND read_message; \
    COMMAND write_message; \
    COMMAND read_tag_descriptor_date; \
    COMMAND write_tag_descriptor_date; \
    COMMAND read_final_assembly_number; \
    COMMAND write_final_assembly_number;

#define UNIVERSAL_BASE_7 \
    VARIABLE device_variable_code_5; \
    VARIABLE device_variable_code_6; \
    VARIABLE device_variable_code_7; \
    VARIABLE device_variable_code_8; \
    VARIABLE time_stamp; \
    VARIABLE response_preambles; \
    VARIABLE max_num_device_variables; \
    VARIABLE config_change_counter; \
    VARIABLE longTag; \
    VARIABLE device_variable_code_1; \
    VARIABLE device_variable_code_2; \
    VARIABLE device_variable_code_3; \
    VARIABLE device_variable_code_4; \
    COMMAND read_unique_identifier_with_long_tag; \
    COMMAND read_device_variables_and_status; \
    COMMAND read_loop_configuration; \
    COMMAND read_dynamic_variable_classification; \
    COMMAND read_long_tag; \
    COMMAND write_long_tag; \
    VARIABLE response_code; \
    VARIABLE comm_status; \
    VARIABLE request_preambles; \
    VARIABLE universal_revision; \
    VARIABLE transmitter_revision; \
    VARIABLE software_revision; \
    VARIABLE hardware_revision; \
    VARIABLE device_id; \
    VARIABLE tag; \
    VARIABLE polling_address; \
    VARIABLE message; \
    VARIABLE descriptor; \
    VARIABLE date; \
    VARIABLE final_assembly_number; \
    COMMAND read_unique_identifier; \
    COMMAND read_unique_identifier_with_tag; \
    COMMAND read_pv; \
    COMMAND read_pv_current_and_percent_range; \
    COMMAND read_dynamic_variables_and_pv_current; \
    COMMAND write_polling_address; \
    COMMAND read_pv_sensor_info; \
    COMMAND read_pv_output_info; \
    COMMAND read_message; \
    COMMAND write_message; \
    COMMAND read_tag_descriptor_date; \
    COMMAND write_tag_descriptor_date; \
    COMMAND read_final_assembly_number; \
    COMMAND write_final_assembly_number; \
    COMMAND reset_configuration_change_flag; \
    COMMAND read_additional_device_status;

#define REDEFINE_COMMAND_3_TRANSACTIONS(d) \
    REDEFINE TRANSACTION \
    { \
        REQUEST {} \
        REPLY \
        { \
            response_code, device_status, \
            COMMAND_3_VARS_##d \
        } \
    }

#define COMMAND_3_VARS_1      \
    PV.DAQ.ANALOG_VALUE,      \
    PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE

#define COMMAND_3_VARS_2      \
    COMMAND_3_VARS_1,         \
    SV.DEVICE_VARIABLE.DIGITAL_UNITS, SV.DEVICE_VARIABLE.DIGITAL_VALUE

#define COMMAND_3_VARS_3      \
    COMMAND_3_VARS_2,         \
    TV.DEVICE_VARIABLE.DIGITAL_UNITS, TV.DEVICE_VARIABLE.DIGITAL_VALUE

#define COMMAND_3_VARS_4      \
    COMMAND_3_VARS_3,         \
    QV.DEVICE_VARIABLE.DIGITAL_UNITS, QV.DEVICE_VARIABLE.DIGITAL_VALUE


#define REDEFINE_COMMAND_8_TRANSACTIONS(d) \
    REDEFINE TRANSACTION \
    { \
        REQUEST {} \
        REPLY \
        { \
            response_code, device_status, \
            COMMAND_8_VARS_##d \
        } \
    }

#define COMMAND_8_VARS_1      \
    PV.DEVICE_VARIABLE.CLASSIFICATION, \
    0,                                 \
    0,                                 \
    0

#define COMMAND_8_VARS_2      \
    PV.DEVICE_VARIABLE.CLASSIFICATION, \
    SV.DEVICE_VARIABLE.CLASSIFICATION, \
    0,                                 \
    0

#define COMMAND_8_VARS_3      \
    PV.DEVICE_VARIABLE.CLASSIFICATION, \
    SV.DEVICE_VARIABLE.CLASSIFICATION, \
    TV.DEVICE_VARIABLE.CLASSIFICATION, \
    0

#define COMMAND_8_VARS_4      \
    PV.DEVICE_VARIABLE.CLASSIFICATION, \
    SV.DEVICE_VARIABLE.CLASSIFICATION, \
    TV.DEVICE_VARIABLE.CLASSIFICATION, \
    QV.DEVICE_VARIABLE.CLASSIFICATION



#define REDEFINE_COMMAND_9_TRANSACTIONS(s) \
    REDEFINE TRANSACTION \
    { \
        REQUEST \
        { \
            COMMAND_9_RQST_##s \
        } \
        REPLY \
        { \
            response_code, device_status, \
            COMMAND_9_RESP_##s, \
            time_stamp \
        } \
    }

#define COMMAND_9_RQST_4      \
    device_variable_code_1 (INFO,INDEX), \
    device_variable_code_2 (INFO,INDEX), \
    device_variable_code_3 (INFO,INDEX), \
    device_variable_code_4 (INFO,INDEX)

#define COMMAND_9_RQST_5      \
    COMMAND_9_RQST_4,         \
    device_variable_code_5 (INFO,INDEX)

#define COMMAND_9_RQST_6      \
    COMMAND_9_RQST_5,         \
    device_variable_code_6 (INFO,INDEX)

#define COMMAND_9_RQST_7      \
    COMMAND_9_RQST_6,         \
    device_variable_code_7 (INFO,INDEX)

#define COMMAND_9_RQST_8      \
    COMMAND_9_RQST_7,         \
    device_variable_code_8 (INFO,INDEX)


#define COMMAND_9_RESP_4      \
    extended_fld_device_status, \
    device_variable_code_1 (INFO,INDEX), \
    deviceVariables[device_variable_code_1].CLASSIFICATION, \
    deviceVariables[device_variable_code_1].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_1].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_1].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_1].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_1].DEVICE_FAMILY_STATUS <0x0F>, \
 \
    device_variable_code_2 (INFO,INDEX), \
    deviceVariables[device_variable_code_2].CLASSIFICATION, \
    deviceVariables[device_variable_code_2].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_2].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_2].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_2].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_2].DEVICE_FAMILY_STATUS <0x0F>, \
 \
    device_variable_code_3 (INFO,INDEX), \
    deviceVariables[device_variable_code_3].CLASSIFICATION, \
    deviceVariables[device_variable_code_3].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_3].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_3].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_3].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_3].DEVICE_FAMILY_STATUS <0x0F>, \
 \
    device_variable_code_4 (INFO,INDEX), \
    deviceVariables[device_variable_code_4].CLASSIFICATION, \
    deviceVariables[device_variable_code_4].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_4].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_4].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_4].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_4].DEVICE_FAMILY_STATUS <0x0F>

#define COMMAND_9_RESP_5      \
    COMMAND_9_RESP_4,         \
    device_variable_code_5 (INFO,INDEX), \
    deviceVariables[device_variable_code_5].CLASSIFICATION, \
    deviceVariables[device_variable_code_5].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_5].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_5].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_5].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_5].DEVICE_FAMILY_STATUS <0x0F>

#define COMMAND_9_RESP_6      \
    COMMAND_9_RESP_5,         \
    device_variable_code_6 (INFO,INDEX), \
    deviceVariables[device_variable_code_6].CLASSIFICATION, \
    deviceVariables[device_variable_code_6].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_6].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_6].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_6].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_6].DEVICE_FAMILY_STATUS <0x0F>


#define COMMAND_9_RESP_7      \
    COMMAND_9_RESP_6,         \
    device_variable_code_7 (INFO,INDEX), \
    deviceVariables[device_variable_code_7].CLASSIFICATION, \
    deviceVariables[device_variable_code_7].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_7].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_7].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_7].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_7].DEVICE_FAMILY_STATUS <0x0F>

#define COMMAND_9_RESP_8      \
    COMMAND_9_RESP_7,         \
    device_variable_code_8 (INFO,INDEX), \
    deviceVariables[device_variable_code_8].CLASSIFICATION, \
    deviceVariables[device_variable_code_8].DIGITAL_UNITS, \
    deviceVariables[device_variable_code_8].DIGITAL_VALUE, \
    deviceVariables[device_variable_code_8].DATA_QUALITY <0xC0>, \
    deviceVariables[device_variable_code_8].LIMIT_STATUS <0x30>, \
    deviceVariables[device_variable_code_8].DEVICE_FAMILY_STATUS <0x0F>


#define REDEFINE_H5_COMMAND_48_TRANSACTIONS(n) \
    REDEFINE TRANSACTION \
    { \
        REQUEST {} \
        REPLY \
        { \
            response_code, device_status, \
            COMMAND_H5_48_BYTES_##n \
        } \
    } \

#define COMMAND_H5_48_BYTES_1     \
    device_specific_status_0

#define COMMAND_H5_48_BYTES_2     \
    COMMAND_H5_48_BYTES_1,        \
    device_specific_status_1

#define COMMAND_H5_48_BYTES_3     \
    COMMAND_H5_48_BYTES_2,        \
    device_specific_status_2

#define COMMAND_H5_48_BYTES_4     \
    COMMAND_H5_48_BYTES_3,        \
    device_specific_status_3

#define COMMAND_H5_48_BYTES_5     \
    COMMAND_H5_48_BYTES_4,        \
    device_specific_status_4

#define COMMAND_H5_48_BYTES_6     \
    COMMAND_H5_48_BYTES_5,        \
    device_specific_status_5

#define COMMAND_H5_48_BYTES_7     \
    COMMAND_H5_48_BYTES_6,        \
    0x00

#define COMMAND_H5_48_BYTES_8     \
    COMMAND_H5_48_BYTES_7,        \
    0x00

#define COMMAND_H5_48_BYTES_9     \
    COMMAND_H5_48_BYTES_8,        \
    0x00

#define COMMAND_H5_48_BYTES_10    \
    COMMAND_H5_48_BYTES_9,        \
    0x00

#define COMMAND_H5_48_BYTES_11    \
    COMMAND_H5_48_BYTES_10,       \
    analog_channel_saturated1

#define COMMAND_H5_48_BYTES_12    \
    COMMAND_H5_48_BYTES_11,       \
    0x00

#define COMMAND_H5_48_BYTES_13    \
    COMMAND_H5_48_BYTES_12,       \
    0x00

#define COMMAND_H5_48_BYTES_14    \
    COMMAND_H5_48_BYTES_13,       \
    analog_channel_fixed1

#define COMMAND_H5_48_BYTES_15    \
    COMMAND_H5_48_BYTES_14,       \
    device_specific_status_14

#define COMMAND_H5_48_BYTES_16    \
    COMMAND_H5_48_BYTES_15,       \
    device_specific_status_15

#define COMMAND_H5_48_BYTES_17    \
    COMMAND_H5_48_BYTES_16,       \
    device_specific_status_16

#define COMMAND_H5_48_BYTES_18    \
    COMMAND_H5_48_BYTES_17,       \
    device_specific_status_17

#define COMMAND_H5_48_BYTES_19    \
    COMMAND_H5_48_BYTES_18,       \
    device_specific_status_18

#define COMMAND_H5_48_BYTES_20    \
    COMMAND_H5_48_BYTES_19,       \
    device_specific_status_19

#define COMMAND_H5_48_BYTES_21    \
    COMMAND_H5_48_BYTES_20,       \
    device_specific_status_20

#define COMMAND_H5_48_BYTES_22    \
    COMMAND_H5_48_BYTES_21,       \
    device_specific_status_21

#define COMMAND_H5_48_BYTES_23    \
    COMMAND_H5_48_BYTES_22,       \
    device_specific_status_22

#define COMMAND_H5_48_BYTES_24    \
    COMMAND_H5_48_BYTES_23,       \
    device_specific_status_23

#define COMMAND_H5_48_BYTES_25    \
    COMMAND_H5_48_BYTES_24,       \
    device_specific_status_24

#define REDEFINE_H6_COMMAND_48_TRANSACTIONS(n) \
    REDEFINE TRANSACTION \
    { \
        REQUEST {} \
        REPLY \
        { \
            response_code, device_status, \
            COMMAND_H6_48_BYTES_##n \
        } \
    } \

#define COMMAND_H6_48_BYTES_1     \
    device_specific_status_0

#define COMMAND_H6_48_BYTES_2     \
    COMMAND_H6_48_BYTES_1,        \
    device_specific_status_1

#define COMMAND_H6_48_BYTES_3     \
    COMMAND_H6_48_BYTES_2,        \
    device_specific_status_2

#define COMMAND_H6_48_BYTES_4     \
    COMMAND_H6_48_BYTES_3,        \
    device_specific_status_3

#define COMMAND_H6_48_BYTES_5     \
    COMMAND_H6_48_BYTES_4,        \
    device_specific_status_4

#define COMMAND_H6_48_BYTES_6     \
    COMMAND_H6_48_BYTES_5,        \
    device_specific_status_5

#define COMMAND_H6_48_BYTES_7     \
    COMMAND_H6_48_BYTES_6,        \
    extended_fld_device_status

#define COMMAND_H6_48_BYTES_8     \
    COMMAND_H6_48_BYTES_7,        \
    0x00

#define COMMAND_H6_48_BYTES_9     \
    COMMAND_H6_48_BYTES_8,        \
    0x00

#define COMMAND_H6_48_BYTES_10    \
    COMMAND_H6_48_BYTES_9,        \
    0x00

#define COMMAND_H6_48_BYTES_11    \
    COMMAND_H6_48_BYTES_10,       \
    analog_channel_saturated1

#define COMMAND_H6_48_BYTES_12    \
    COMMAND_H6_48_BYTES_11,       \
    0x00

#define COMMAND_H6_48_BYTES_13    \
    COMMAND_H6_48_BYTES_12,       \
    0x00

#define COMMAND_H6_48_BYTES_14    \
    COMMAND_H6_48_BYTES_13,       \
    analog_channel_fixed1

#define COMMAND_H6_48_BYTES_15    \
    COMMAND_H6_48_BYTES_14,       \
    device_specific_status_14

#define COMMAND_H6_48_BYTES_16    \
    COMMAND_H6_48_BYTES_15,       \
    device_specific_status_15

#define COMMAND_H6_48_BYTES_17    \
    COMMAND_H6_48_BYTES_16,       \
    device_specific_status_16

#define COMMAND_H6_48_BYTES_18    \
    COMMAND_H6_48_BYTES_17,       \
    device_specific_status_17

#define COMMAND_H6_48_BYTES_19    \
    COMMAND_H6_48_BYTES_18,       \
    device_specific_status_18

#define COMMAND_H6_48_BYTES_20    \
    COMMAND_H6_48_BYTES_19,       \
    device_specific_status_19

#define COMMAND_H6_48_BYTES_21    \
    COMMAND_H6_48_BYTES_20,       \
    device_specific_status_20

#define COMMAND_H6_48_BYTES_22    \
    COMMAND_H6_48_BYTES_21,       \
    device_specific_status_21

#define COMMAND_H6_48_BYTES_23    \
    COMMAND_H6_48_BYTES_22,       \
    device_specific_status_22

#define COMMAND_H6_48_BYTES_24    \
    COMMAND_H6_48_BYTES_23,       \
    device_specific_status_23

#define COMMAND_H6_48_BYTES_25    \
    COMMAND_H6_48_BYTES_24,       \
    device_specific_status_24

#define REDEFINE_H7_COMMAND_48_TRANSACTIONS(n) \
    REDEFINE TRANSACTION 0 \
    { \
        REQUEST {} \
        REPLY \
        { \
            response_code, device_status, \
            COMMAND_48_BYTES_##n \
        } \
    } \
    REDEFINE TRANSACTION 1 \
    { \
        REQUEST \
        { \
            COMMAND_48_INFO_BYTES_##n \
        } \
        REPLY \
        { \
            response_code, device_status, \
            COMMAND_48_INFO_BYTES_##n \
        } \
    }

#define COMMAND_48_BYTES_1     \
    device_specific_status_0

#define COMMAND_48_BYTES_2     \
    COMMAND_48_BYTES_1,        \
    device_specific_status_1

#define COMMAND_48_BYTES_3     \
    COMMAND_48_BYTES_2,        \
    device_specific_status_2

#define COMMAND_48_BYTES_4     \
    COMMAND_48_BYTES_3,        \
    device_specific_status_3

#define COMMAND_48_BYTES_5     \
    COMMAND_48_BYTES_4,        \
    device_specific_status_4

#define COMMAND_48_BYTES_6     \
    COMMAND_48_BYTES_5,        \
    device_specific_status_5

#define COMMAND_48_BYTES_7     \
    COMMAND_48_BYTES_6,        \
    extended_fld_device_status

#define COMMAND_48_BYTES_8     \
    COMMAND_48_BYTES_7,        \
    0x00

#define COMMAND_48_BYTES_9     \
    COMMAND_48_BYTES_8,        \
    standardized_status_0

#define COMMAND_48_BYTES_10    \
    COMMAND_48_BYTES_9,        \
    standardized_status_1

#define COMMAND_48_BYTES_11    \
    COMMAND_48_BYTES_10,       \
    analog_channel_saturated1

#define COMMAND_48_BYTES_12    \
    COMMAND_48_BYTES_11,       \
    standardized_status_2

#define COMMAND_48_BYTES_13    \
    COMMAND_48_BYTES_12,       \
    standardized_status_3

#define COMMAND_48_BYTES_14    \
    COMMAND_48_BYTES_13,       \
    analog_channel_fixed1

#define COMMAND_48_BYTES_15    \
    COMMAND_48_BYTES_14,       \
    device_specific_status_14

#define COMMAND_48_BYTES_16    \
    COMMAND_48_BYTES_15,       \
    device_specific_status_15

#define COMMAND_48_BYTES_17    \
    COMMAND_48_BYTES_16,       \
    device_specific_status_16

#define COMMAND_48_BYTES_18    \
    COMMAND_48_BYTES_17,       \
    device_specific_status_17

#define COMMAND_48_BYTES_19    \
    COMMAND_48_BYTES_18,       \
    device_specific_status_18

#define COMMAND_48_BYTES_20    \
    COMMAND_48_BYTES_19,       \
    device_specific_status_19

#define COMMAND_48_BYTES_21    \
    COMMAND_48_BYTES_20,       \
    device_specific_status_20

#define COMMAND_48_BYTES_22    \
    COMMAND_48_BYTES_21,       \
    device_specific_status_21

#define COMMAND_48_BYTES_23    \
    COMMAND_48_BYTES_22,       \
    device_specific_status_22

#define COMMAND_48_BYTES_24    \
    COMMAND_48_BYTES_23,       \
    device_specific_status_23

#define COMMAND_48_BYTES_25    \
    COMMAND_48_BYTES_24,       \
    device_specific_status_24

#define COMMAND_48_INFO_BYTES_1     \
    device_specific_status_0 (INFO)

#define COMMAND_48_INFO_BYTES_2     \
    COMMAND_48_INFO_BYTES_1,        \
    device_specific_status_1 (INFO)

#define COMMAND_48_INFO_BYTES_3     \
    COMMAND_48_INFO_BYTES_2,        \
    device_specific_status_2 (INFO)

#define COMMAND_48_INFO_BYTES_4     \
    COMMAND_48_INFO_BYTES_3,        \
    device_specific_status_3 (INFO)

#define COMMAND_48_INFO_BYTES_5     \
    COMMAND_48_INFO_BYTES_4,        \
    device_specific_status_4 (INFO)

#define COMMAND_48_INFO_BYTES_6     \
    COMMAND_48_INFO_BYTES_5,        \
    device_specific_status_5 (INFO)

#define COMMAND_48_INFO_BYTES_7     \
    COMMAND_48_INFO_BYTES_6,        \
    extended_fld_device_status (INFO)

#define COMMAND_48_INFO_BYTES_8     \
    COMMAND_48_INFO_BYTES_7,        \
    0x00

#define COMMAND_48_INFO_BYTES_9     \
    COMMAND_48_INFO_BYTES_8,        \
    standardized_status_0 (INFO)

#define COMMAND_48_INFO_BYTES_10    \
    COMMAND_48_INFO_BYTES_9,        \
    standardized_status_1 (INFO)

#define COMMAND_48_INFO_BYTES_11    \
    COMMAND_48_INFO_BYTES_10,       \
    analog_channel_saturated1 (INFO)

#define COMMAND_48_INFO_BYTES_12    \
    COMMAND_48_INFO_BYTES_11,       \
    standardized_status_2 (INFO)

#define COMMAND_48_INFO_BYTES_13    \
    COMMAND_48_INFO_BYTES_12,       \
    standardized_status_3 (INFO)

#define COMMAND_48_INFO_BYTES_14    \
    COMMAND_48_INFO_BYTES_13,       \
    analog_channel_fixed1 (INFO)

#define COMMAND_48_INFO_BYTES_15    \
    COMMAND_48_INFO_BYTES_14,       \
    device_specific_status_14 (INFO)

#define COMMAND_48_INFO_BYTES_16    \
    COMMAND_48_INFO_BYTES_15,       \
    device_specific_status_15 (INFO)

#define COMMAND_48_INFO_BYTES_17    \
    COMMAND_48_INFO_BYTES_16,       \
    device_specific_status_16 (INFO)

#define COMMAND_48_INFO_BYTES_18    \
    COMMAND_48_INFO_BYTES_17,       \
    device_specific_status_17 (INFO)

#define COMMAND_48_INFO_BYTES_19    \
    COMMAND_48_INFO_BYTES_18,       \
    device_specific_status_18 (INFO)

#define COMMAND_48_INFO_BYTES_20    \
    COMMAND_48_INFO_BYTES_19,       \
    device_specific_status_19 (INFO)

#define COMMAND_48_INFO_BYTES_21    \
    COMMAND_48_INFO_BYTES_20,       \
    device_specific_status_20 (INFO)

#define COMMAND_48_INFO_BYTES_22    \
    COMMAND_48_INFO_BYTES_21,       \
    device_specific_status_21 (INFO)

#define COMMAND_48_INFO_BYTES_23    \
    COMMAND_48_INFO_BYTES_22,       \
    device_specific_status_22 (INFO)

#define COMMAND_48_INFO_BYTES_24    \
    COMMAND_48_INFO_BYTES_23,       \
    device_specific_status_23 (INFO)

#define COMMAND_48_INFO_BYTES_25    \
    COMMAND_48_INFO_BYTES_24,       \
    device_specific_status_24 (INFO)


#define IMPORT_UNIVERSAL_DEVICE_STATUS(n) \
    UNIVERSAL_DEVICE_STATUS_##n

  
#define UNIVERSAL_DEVICE_STATUS_1 \
    VARIABLE device_specific_status_0;
    
#define UNIVERSAL_DEVICE_STATUS_2 \
    UNIVERSAL_DEVICE_STATUS_1 \
    VARIABLE device_specific_status_1;
    
#define UNIVERSAL_DEVICE_STATUS_3 \
    UNIVERSAL_DEVICE_STATUS_2 \
    VARIABLE device_specific_status_2;
    
#define UNIVERSAL_DEVICE_STATUS_4 \
    UNIVERSAL_DEVICE_STATUS_3 \
    VARIABLE device_specific_status_3;
    
#define UNIVERSAL_DEVICE_STATUS_5 \
    UNIVERSAL_DEVICE_STATUS_4 \
    VARIABLE device_specific_status_4;
    
#define UNIVERSAL_DEVICE_STATUS_6 \
    UNIVERSAL_DEVICE_STATUS_5 \
    VARIABLE device_specific_status_5;

#define UNIVERSAL_DEVICE_STATUS_7 \
    UNIVERSAL_DEVICE_STATUS_6

#define UNIVERSAL_DEVICE_STATUS_8 \
    UNIVERSAL_DEVICE_STATUS_7

#define UNIVERSAL_DEVICE_STATUS_9 \
    UNIVERSAL_DEVICE_STATUS_8

#define UNIVERSAL_DEVICE_STATUS_10 \
    UNIVERSAL_DEVICE_STATUS_9

#define UNIVERSAL_DEVICE_STATUS_11 \
    UNIVERSAL_DEVICE_STATUS_10

#define UNIVERSAL_DEVICE_STATUS_12 \
    UNIVERSAL_DEVICE_STATUS_11

#define UNIVERSAL_DEVICE_STATUS_13 \
    UNIVERSAL_DEVICE_STATUS_12

#define UNIVERSAL_DEVICE_STATUS_14 \
    UNIVERSAL_DEVICE_STATUS_13

#define UNIVERSAL_DEVICE_STATUS_15 \
    UNIVERSAL_DEVICE_STATUS_14 \
    VARIABLE device_specific_status_14;
    
#define UNIVERSAL_DEVICE_STATUS_16 \
    UNIVERSAL_DEVICE_STATUS_15 \
    VARIABLE device_specific_status_15;

#define UNIVERSAL_DEVICE_STATUS_17 \
    UNIVERSAL_DEVICE_STATUS_16 \
    VARIABLE device_specific_status_16;

#define UNIVERSAL_DEVICE_STATUS_18 \
    UNIVERSAL_DEVICE_STATUS_17 \
    VARIABLE device_specific_status_17;

#define UNIVERSAL_DEVICE_STATUS_19 \
    UNIVERSAL_DEVICE_STATUS_18 \
    VARIABLE device_specific_status_18;

#define UNIVERSAL_DEVICE_STATUS_20 \
    UNIVERSAL_DEVICE_STATUS_19 \
    VARIABLE device_specific_status_19;

#define UNIVERSAL_DEVICE_STATUS_21 \
    UNIVERSAL_DEVICE_STATUS_20 \
    VARIABLE device_specific_status_20;

#define UNIVERSAL_DEVICE_STATUS_22 \
    UNIVERSAL_DEVICE_STATUS_21 \
    VARIABLE device_specific_status_21;

#define UNIVERSAL_DEVICE_STATUS_23 \
    UNIVERSAL_DEVICE_STATUS_22 \
    VARIABLE device_specific_status_22;

#define UNIVERSAL_DEVICE_STATUS_24 \
    UNIVERSAL_DEVICE_STATUS_23 \
    VARIABLE device_specific_status_23;

#define UNIVERSAL_DEVICE_STATUS_25 \
    UNIVERSAL_DEVICE_STATUS_24 \
    VARIABLE device_specific_status_24;


#define IMPORT_UNIVERSAL_EVENT_REF_ARRAY(e,n) \
    UNIVERSAL_EVENT_REF_ARRAY_##e##_##n
    
#define UNIVERSAL_EVENT_REF_ARRAY_1_1 \
    VARIABLE device_specific_status_0_latched_value; \
    VARIABLE device_specific_status_0_mask; \
    VARIABLE config_change_counter_latched_value;
    
#define UNIVERSAL_EVENT_REF_ARRAY_1_2 \
    UNIVERSAL_EVENT_REF_ARRAY_1_1 \
    VARIABLE device_specific_status_1_latched_value; \
    VARIABLE device_specific_status_1_mask;
    
#define UNIVERSAL_EVENT_REF_ARRAY_1_3 \
    UNIVERSAL_EVENT_REF_ARRAY_1_2 \
    VARIABLE device_specific_status_2_latched_value; \
    VARIABLE device_specific_status_2_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_4 \
    UNIVERSAL_EVENT_REF_ARRAY_1_3 \
    VARIABLE device_specific_status_3_latched_value; \
    VARIABLE device_specific_status_3_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_5 \
    UNIVERSAL_EVENT_REF_ARRAY_1_4 \
    VARIABLE device_specific_status_4_latched_value; \
    VARIABLE device_specific_status_4_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_6 \
    UNIVERSAL_EVENT_REF_ARRAY_1_5 \
    VARIABLE device_specific_status_5_latched_value; \
    VARIABLE device_specific_status_5_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_7 \
    UNIVERSAL_EVENT_REF_ARRAY_1_6

#define UNIVERSAL_EVENT_REF_ARRAY_1_8 \
    UNIVERSAL_EVENT_REF_ARRAY_1_7

#define UNIVERSAL_EVENT_REF_ARRAY_1_9 \
    UNIVERSAL_EVENT_REF_ARRAY_1_8

#define UNIVERSAL_EVENT_REF_ARRAY_1_10 \
    UNIVERSAL_EVENT_REF_ARRAY_1_9

#define UNIVERSAL_EVENT_REF_ARRAY_1_11 \
    UNIVERSAL_EVENT_REF_ARRAY_1_10

#define UNIVERSAL_EVENT_REF_ARRAY_1_12 \
    UNIVERSAL_EVENT_REF_ARRAY_1_11

#define UNIVERSAL_EVENT_REF_ARRAY_1_13 \
    UNIVERSAL_EVENT_REF_ARRAY_1_12

#define UNIVERSAL_EVENT_REF_ARRAY_1_14 \
    UNIVERSAL_EVENT_REF_ARRAY_1_13

#define UNIVERSAL_EVENT_REF_ARRAY_1_15 \
    UNIVERSAL_EVENT_REF_ARRAY_1_14 \
    VARIABLE device_specific_status_14_latched_value; \
    VARIABLE device_specific_status_14_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_16 \
    UNIVERSAL_EVENT_REF_ARRAY_1_15 \
    VARIABLE device_specific_status_15_latched_value; \
    VARIABLE device_specific_status_15_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_17 \
    UNIVERSAL_EVENT_REF_ARRAY_1_16 \
    VARIABLE device_specific_status_16_latched_value; \
    VARIABLE device_specific_status_16_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_18 \
    UNIVERSAL_EVENT_REF_ARRAY_1_17 \
    VARIABLE device_specific_status_17_latched_value; \
    VARIABLE device_specific_status_17_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_19 \
    UNIVERSAL_EVENT_REF_ARRAY_1_18 \
    VARIABLE device_specific_status_18_latched_value; \
    VARIABLE device_specific_status_18_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_20 \
    UNIVERSAL_EVENT_REF_ARRAY_1_19 \
    VARIABLE device_specific_status_19_latched_value; \
    VARIABLE device_specific_status_19_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_21 \
    UNIVERSAL_EVENT_REF_ARRAY_1_20 \
    VARIABLE device_specific_status_20_latched_value; \
    VARIABLE device_specific_status_20_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_22 \
    UNIVERSAL_EVENT_REF_ARRAY_1_21 \
    VARIABLE device_specific_status_21_latched_value; \
    VARIABLE device_specific_status_21_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_23 \
    UNIVERSAL_EVENT_REF_ARRAY_1_22 \
    VARIABLE device_specific_status_22_latched_value; \
    VARIABLE device_specific_status_22_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_24 \
    UNIVERSAL_EVENT_REF_ARRAY_1_23 \
    VARIABLE device_specific_status_23_latched_value; \
    VARIABLE device_specific_status_23_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_1_25 \
    UNIVERSAL_EVENT_REF_ARRAY_1_24 \
    VARIABLE device_specific_status_24_latched_value; \
    VARIABLE device_specific_status_24_mask;

#define UNIVERSAL_EVENT_REF_ARRAY_2_1 \
    VARIABLE device_specific_status_0_latched_value; \
    VARIABLE device_specific_status_0_mask; \
    VARIABLE config_change_counter_latched_value; \
    VARIABLE device_specific_status_0_latched_value_1; \
    VARIABLE device_specific_status_0_mask_1; \
    VARIABLE config_change_counter_latched_value_1;
    
#define UNIVERSAL_EVENT_REF_ARRAY_2_2 \
    UNIVERSAL_EVENT_REF_ARRAY_2_1 \
    VARIABLE device_specific_status_1_latched_value; \
    VARIABLE device_specific_status_1_mask; \
    VARIABLE device_specific_status_1_latched_value_1; \
    VARIABLE device_specific_status_1_mask_1;
    
#define UNIVERSAL_EVENT_REF_ARRAY_2_3 \
    UNIVERSAL_EVENT_REF_ARRAY_2_2 \
    VARIABLE device_specific_status_2_latched_value; \
    VARIABLE device_specific_status_2_mask; \
    VARIABLE device_specific_status_2_latched_value_1; \
    VARIABLE device_specific_status_2_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_4 \
    UNIVERSAL_EVENT_REF_ARRAY_2_3 \
    VARIABLE device_specific_status_3_latched_value; \
    VARIABLE device_specific_status_3_mask; \
    VARIABLE device_specific_status_3_latched_value_1; \
    VARIABLE device_specific_status_3_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_5 \
    UNIVERSAL_EVENT_REF_ARRAY_2_4 \
    VARIABLE device_specific_status_4_latched_value; \
    VARIABLE device_specific_status_4_mask; \
    VARIABLE device_specific_status_4_latched_value_1; \
    VARIABLE device_specific_status_4_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_6 \
    UNIVERSAL_EVENT_REF_ARRAY_2_5 \
    VARIABLE device_specific_status_5_latched_value; \
    VARIABLE device_specific_status_5_mask; \
    VARIABLE device_specific_status_5_latched_value_1; \
    VARIABLE device_specific_status_5_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_7 \
    UNIVERSAL_EVENT_REF_ARRAY_2_6

#define UNIVERSAL_EVENT_REF_ARRAY_2_8 \
    UNIVERSAL_EVENT_REF_ARRAY_2_7

#define UNIVERSAL_EVENT_REF_ARRAY_2_9 \
    UNIVERSAL_EVENT_REF_ARRAY_2_8

#define UNIVERSAL_EVENT_REF_ARRAY_2_10 \
    UNIVERSAL_EVENT_REF_ARRAY_2_9

#define UNIVERSAL_EVENT_REF_ARRAY_2_11 \
    UNIVERSAL_EVENT_REF_ARRAY_2_10

#define UNIVERSAL_EVENT_REF_ARRAY_2_12 \
    UNIVERSAL_EVENT_REF_ARRAY_2_11

#define UNIVERSAL_EVENT_REF_ARRAY_2_13 \
    UNIVERSAL_EVENT_REF_ARRAY_2_12

#define UNIVERSAL_EVENT_REF_ARRAY_2_14 \
    UNIVERSAL_EVENT_REF_ARRAY_2_13

#define UNIVERSAL_EVENT_REF_ARRAY_2_15 \
    UNIVERSAL_EVENT_REF_ARRAY_2_14 \
    VARIABLE device_specific_status_14_latched_value; \
    VARIABLE device_specific_status_14_mask; \
    VARIABLE device_specific_status_14_latched_value_1; \
    VARIABLE device_specific_status_14_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_16 \
    UNIVERSAL_EVENT_REF_ARRAY_2_15 \
    VARIABLE device_specific_status_15_latched_value; \
    VARIABLE device_specific_status_15_mask; \
    VARIABLE device_specific_status_15_latched_value_1; \
    VARIABLE device_specific_status_15_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_17 \
    UNIVERSAL_EVENT_REF_ARRAY_2_16 \
    VARIABLE device_specific_status_16_latched_value; \
    VARIABLE device_specific_status_16_mask; \
    VARIABLE device_specific_status_16_latched_value_1; \
    VARIABLE device_specific_status_16_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_18 \
    UNIVERSAL_EVENT_REF_ARRAY_2_17 \
    VARIABLE device_specific_status_17_latched_value; \
    VARIABLE device_specific_status_17_mask; \
    VARIABLE device_specific_status_17_latched_value_1; \
    VARIABLE device_specific_status_17_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_19 \
    UNIVERSAL_EVENT_REF_ARRAY_2_18 \
    VARIABLE device_specific_status_18_latched_value; \
    VARIABLE device_specific_status_18_mask; \
    VARIABLE device_specific_status_18_latched_value_1; \
    VARIABLE device_specific_status_18_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_20 \
    UNIVERSAL_EVENT_REF_ARRAY_2_19 \
    VARIABLE device_specific_status_19_latched_value; \
    VARIABLE device_specific_status_19_mask; \
    VARIABLE device_specific_status_19_latched_value_1; \
    VARIABLE device_specific_status_19_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_21 \
    UNIVERSAL_EVENT_REF_ARRAY_2_20 \
    VARIABLE device_specific_status_20_latched_value; \
    VARIABLE device_specific_status_20_mask; \
    VARIABLE device_specific_status_20_latched_value_1; \
    VARIABLE device_specific_status_20_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_22 \
    UNIVERSAL_EVENT_REF_ARRAY_2_21 \
    VARIABLE device_specific_status_21_latched_value; \
    VARIABLE device_specific_status_21_mask; \
    VARIABLE device_specific_status_21_latched_value_1; \
    VARIABLE device_specific_status_21_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_23 \
    UNIVERSAL_EVENT_REF_ARRAY_2_22 \
    VARIABLE device_specific_status_22_latched_value; \
    VARIABLE device_specific_status_22_mask; \
    VARIABLE device_specific_status_22_latched_value_1; \
    VARIABLE device_specific_status_22_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_24 \
    UNIVERSAL_EVENT_REF_ARRAY_2_23 \
    VARIABLE device_specific_status_23_latched_value; \
    VARIABLE device_specific_status_23_mask; \
    VARIABLE device_specific_status_23_latched_value_1; \
    VARIABLE device_specific_status_23_mask_1;

#define UNIVERSAL_EVENT_REF_ARRAY_2_25 \
    UNIVERSAL_EVENT_REF_ARRAY_2_24 \
    VARIABLE device_specific_status_24_latched_value; \
    VARIABLE device_specific_status_24_mask; \
    VARIABLE device_specific_status_24_latched_value_1; \
    VARIABLE device_specific_status_24_mask_1;


#define IMPORT_UNIVERSAL_EVENT_VAL_ARRAY(n) \
    UNIVERSAL_EVENT_VAL_ARRAY_##n
    
#define UNIVERSAL_EVENT_VAL_ARRAY_1 \
    VARIABLE device_specific_status_0_latched_value; \
    VARIABLE device_specific_status_0_mask; \
    VARIABLE config_change_counter_latched_value;

#define UNIVERSAL_EVENT_VAL_ARRAY_2 \
    UNIVERSAL_EVENT_VAL_ARRAY_1 \
    VARIABLE device_specific_status_1_latched_value; \
    VARIABLE device_specific_status_1_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_3 \
    UNIVERSAL_EVENT_VAL_ARRAY_2 \
    VARIABLE device_specific_status_2_latched_value; \
    VARIABLE device_specific_status_2_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_4 \
    UNIVERSAL_EVENT_VAL_ARRAY_3 \
    VARIABLE device_specific_status_3_latched_value; \
    VARIABLE device_specific_status_3_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_5 \
    UNIVERSAL_EVENT_VAL_ARRAY_4 \
    VARIABLE device_specific_status_4_latched_value; \
    VARIABLE device_specific_status_4_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_6 \
    UNIVERSAL_EVENT_VAL_ARRAY_5 \
    VARIABLE device_specific_status_5_latched_value; \
    VARIABLE device_specific_status_5_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_7 \
    UNIVERSAL_EVENT_VAL_ARRAY_6

#define UNIVERSAL_EVENT_VAL_ARRAY_8 \
    UNIVERSAL_EVENT_VAL_ARRAY_7

#define UNIVERSAL_EVENT_VAL_ARRAY_9 \
    UNIVERSAL_EVENT_VAL_ARRAY_8

#define UNIVERSAL_EVENT_VAL_ARRAY_10 \
    UNIVERSAL_EVENT_VAL_ARRAY_9

#define UNIVERSAL_EVENT_VAL_ARRAY_11 \
    UNIVERSAL_EVENT_VAL_ARRAY_10

#define UNIVERSAL_EVENT_VAL_ARRAY_12 \
    UNIVERSAL_EVENT_VAL_ARRAY_11

#define UNIVERSAL_EVENT_VAL_ARRAY_13\
    UNIVERSAL_EVENT_VAL_ARRAY_12

#define UNIVERSAL_EVENT_VAL_ARRAY_14 \
    UNIVERSAL_EVENT_VAL_ARRAY_13

#define UNIVERSAL_EVENT_VAL_ARRAY_15 \
    UNIVERSAL_EVENT_VAL_ARRAY_14 \
    VARIABLE device_specific_status_14_latched_value; \
    VARIABLE device_specific_status_14_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_16 \
    UNIVERSAL_EVENT_VAL_ARRAY_15 \
    VARIABLE device_specific_status_15_latched_value; \
    VARIABLE device_specific_status_15_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_17 \
    UNIVERSAL_EVENT_VAL_ARRAY_16 \
    VARIABLE device_specific_status_16_latched_value; \
    VARIABLE device_specific_status_16_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_18 \
    UNIVERSAL_EVENT_VAL_ARRAY_17 \
    VARIABLE device_specific_status_17_latched_value; \
    VARIABLE device_specific_status_17_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_19 \
    UNIVERSAL_EVENT_VAL_ARRAY_18 \
    VARIABLE device_specific_status_18_latched_value; \
    VARIABLE device_specific_status_18_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_20 \
    UNIVERSAL_EVENT_VAL_ARRAY_19 \
    VARIABLE device_specific_status_19_latched_value; \
    VARIABLE device_specific_status_19_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_21 \
    UNIVERSAL_EVENT_VAL_ARRAY_20 \
    VARIABLE device_specific_status_20_latched_value; \
    VARIABLE device_specific_status_20_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_22 \
    UNIVERSAL_EVENT_VAL_ARRAY_21 \
    VARIABLE device_specific_status_21_latched_value; \
    VARIABLE device_specific_status_21_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_23 \
    UNIVERSAL_EVENT_VAL_ARRAY_22 \
    VARIABLE device_specific_status_22_latched_value; \
    VARIABLE device_specific_status_22_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_24 \
    UNIVERSAL_EVENT_VAL_ARRAY_23 \
    VARIABLE device_specific_status_23_latched_value; \
    VARIABLE device_specific_status_23_mask;

#define UNIVERSAL_EVENT_VAL_ARRAY_25 \
    UNIVERSAL_EVENT_VAL_ARRAY_24 \
    VARIABLE device_specific_status_24_latched_value; \
    VARIABLE device_specific_status_24_mask;

#define IMPORT_PV1_BASE(d) \
    PV1_BASE_##d
    
#define PV1_BASE_1 \
    VARIABLE loopCurrent; \
    VARIABLE percentRange; \
    VARIABLE upperRange_value; \
    VARIABLE lowerRange_value; \
    ARRAY OF COLLECTION dynamic_variables; \
    COLLECTION primary_variable; \
    COLLECTION analog_io; \
    COLLECTION scaling; \
    UNIT scaling_units_relation;

#define PV1_BASE_2 \
    PV1_BASE_1 \
    COLLECTION secondary_variable;

#define PV1_BASE_3 \
    PV1_BASE_2 \
    COLLECTION tertiary_variable;

#define PV1_BASE_4 \
    PV1_BASE_3 \
    COLLECTION quaternary_variable;

#define REDEFINE_DYNAMIC_VAR_SIZE(d)           \
        REMOVE_DYNAMIC_VARIABLE_##d

#define REMOVE_DYNAMIC_VARIABLE_3              \
    DELETE QUATERNARY;

#define REMOVE_DYNAMIC_VARIABLE_2              \
    REMOVE_DYNAMIC_VARIABLE_3                  \
    DELETE TERTIARY;

#define REMOVE_DYNAMIC_VARIABLE_1              \
    REMOVE_DYNAMIC_VARIABLE_2                  \
    DELETE SECONDARY;

#define IMPORT_PV2_1_BASE() \
    VARIABLE loopCurrent; \
    VARIABLE percentRange; \
    VARIABLE upperRange_value; \
    VARIABLE lowerRange_value; \
    VARIABLE PVloopUnits; \
    VARIABLE PVloopDamping; \
    VARIABLE PVloopCurrentUpperEndPoint; \
    VARIABLE PVloopCurrentLowerEndPoint; \
    VARIABLE SVanalogChannelValue; \
    VARIABLE SVanalogChannelUnits; \
    VARIABLE SVanalogChannelDamping; \
    VARIABLE SVanalogChannelUpperEndPoint; \
    VARIABLE SVanalogChannelLowerEndPoint; \
    VARIABLE SVpercentRange; \
    VARIABLE SVupperRange_value; \
    VARIABLE SVlowerRange_value; \
    VARIABLE TVanalogChannelValue; \
    VARIABLE TVanalogChannelUnits; \
    VARIABLE TVanalogChannelDamping; \
    VARIABLE TVanalogChannelUpperEndPoint; \
    VARIABLE TVanalogChannelLowerEndPoint; \
    VARIABLE TVpercentRange; \
    VARIABLE TVupperRange_value; \
    VARIABLE TVlowerRange_value; \
    VARIABLE QVanalogChannelValue; \
    VARIABLE QVanalogChannelUnits; \
    VARIABLE QVanalogChannelDamping; \
    VARIABLE QVanalogChannelUpperEndPoint; \
    VARIABLE QVanalogChannelLowerEndPoint; \
    VARIABLE QVpercentRange; \
    VARIABLE QVupperRange_value; \
    VARIABLE QVlowerRange_value; \
    ARRAY OF COLLECTION dynamic_variables; \
    COLLECTION primary_variable; \
    COLLECTION secondary_variable; \
    COLLECTION tertiary_variable; \
    COLLECTION quaternary_variable; \
    COLLECTION analog_io; \
    COLLECTION SVanalog_io; \
    COLLECTION TVanalog_io; \
    COLLECTION QVanalog_io; \
    COLLECTION scaling; \
    COLLECTION SVscaling; \
    COLLECTION TVscaling; \
    COLLECTION QVscaling; \
    UNIT scaling_units_relation; \
    UNIT SVscaling_units_relation; \
    UNIT TVscaling_units_relation; \
    UNIT QVscaling_units_relation;

//Imports are the same weather importing PV2_1 or PV2_3
#define IMPORT_PV2_3_BASE() IMPORT_PV2_1_BASE()

#define IMPORT_PV2_2_BASE() \
    IMPORT_PV2_1_BASE() \
    VARIABLE PVloopCurrentUpperLimit; \
    VARIABLE PVloopCurrentLowerLimit; \
    VARIABLE SVanalogChannelUpperLimit; \
    VARIABLE SVanalogChannelLowerLimit; \
    VARIABLE TVanalogChannelUpperLimit; \
    VARIABLE TVanalogChannelLowerLimit; \
    VARIABLE QVanalogChannelUpperLimit; \
    VARIABLE QVanalogChannelLowerLimit;

//Imports are the same weather importing PV2_2 or PV2_4
#define IMPORT_PV2_4_BASE() IMPORT_PV2_2_BASE()

#define IMPORT_DEV_VARS_BASE() \
    COMMAND read_device_variable_information; \
    VARIABLE device_variable_code;

#define IMPORT_DEV_VARS_CATCH_V8() \
    VARIABLE destination_device_variable_code; \
    COMMAND catch_device_variable; \
    COMMAND read_caught_device_variable;

#define IMPORT_DEV_VARS_CATCH_V11() \
    IMPORT_DEV_VARS_CATCH_V8() \
    VARIABLE source_command_number_byte;

#define IMPORT_DEV_VARS_MAPPING_READ()           \
    VARIABLE primary_variable_code; \
    VARIABLE secondary_variable_code; \
    VARIABLE tertiary_variable_code; \
    VARIABLE quaternary_variable_code; \
    COMMAND read_device_variable_assignments; \
    METHOD warning_message;

#define IMPORT_DEV_VARS_MAPPING_READ_WRITE()           \
    VARIABLE primary_variable_code; \
    VARIABLE secondary_variable_code; \
    VARIABLE tertiary_variable_code; \
    VARIABLE quaternary_variable_code; \
    COMMAND read_device_variable_assignments; \
    COMMAND write_device_variable_assignments; \
    METHOD warning_message;


#define IMPORT_DEV_VARS_TRIM() \
    VARIABLE device_variable_trim_code; \
    COMMAND read_device_variable_trim_point; \
    COMMAND read_device_variable_trim_guidelines; \
    COMMAND write_device_variable_trim_point; \
    COMMAND reset_device_variable_trim;

#define IMPORT_BURST_VAL_ARRAY_BASE(u) \
    BURST_VAL_ARRAY_BASE_##u
    
#define BURST_VAL_ARRAY_BASE_5 \
    VARIABLE burst_command_number; \
    VARIABLE burst_variable_code; \
    VARIABLE dummy_variable; \
    VARIABLE dummy_variable_float; \
    VARIABLE dummy_variable_unit; \
    VARIABLE dummy_variable_unsigned_3; \
    ARRAY OF COLLECTION burstDeviceVariables; \
    ARRAY burst_variable_codes; \
    ARRAY burst_messages; \
    COLLECTION std_burst_message_collection; \
    COLLECTION dummy_collection; \
    COMMAND write_burst_device_variables; \
    COMMAND write_burst_command_number; \
    COMMAND burst_mode_control;

#define BURST_VAL_ARRAY_BASE_6 \
    BURST_VAL_ARRAY_BASE_5 \
    COMMAND read_burst_mode_configuration;

#define BURST_VAL_ARRAY_BASE_7 \
    BURST_VAL_ARRAY_BASE_6 \
    VARIABLE update_period; \
    VARIABLE max_update_period; \
    VARIABLE dummy_variable_time_int; \
    VARIABLE trigger_level; \
    VARIABLE burst_message_number; \
    VARIABLE total_number_burst_messages; \
    COMMAND write_burst_period; \
    COMMAND write_burst_trigger;

#define REDEFINE_BURST_VAL_ARRAY_SIZE(b) \
    REDEFINE NUMBER_OF_ELEMENTS b;

#define REDEFINE_BURST_REF_ARRAY_SIZE(b) \
    ELEMENTS \
    { \
        REMOVE_BURST_MESSAGE_##b \
    }

#define REMOVE_BURST_MESSAGE_3 \
    DELETE 3;

#define REMOVE_BURST_MESSAGE_2 \
    REMOVE_BURST_MESSAGE_3 \
    DELETE 2;

#define REMOVE_BURST_MESSAGE_1 \
    REMOVE_BURST_MESSAGE_2 \
    DELETE 1;

#define IMPORT_BURST_VAL_ARRAY_EVENT() \
    VARIABLE number_events_supported; \
    VARIABLE time_of_first_unack_event; \
    VARIABLE event_notification_retry_time_event; \
    VARIABLE max_update_time_event; \
    VARIABLE event_debounce_interval_event; \
    VARIABLE eventNumber; \
    ARRAY events; \
    COLLECTION event; \
    COLLECTION event_control; \
    COLLECTION event_mask; \
    COLLECTION event_report; \
    COMMAND read_event_notification_summary; \
    COMMAND write_event_notification_bit_mask; \
    COMMAND write_event_notification_timing; \
    COMMAND event_notification_control; \
    COMMAND acknowledge_event_notification;
/*
    VARIABLE time_of_first_unack_event_int; \
    VARIABLE event_notification_retry_time_event_int; \
    VARIABLE max_update_time_event_int; \
    VARIABLE event_debounce_interval_event_int; \
*/

#define REDEFINE_EVENT_VAL_ARRAY_SIZE(e) \
    REDEFINE NUMBER_OF_ELEMENTS e;

#define REDEFINE_EVENT_REF_ARRAY_SIZE(e) \
    ELEMENTS \
    { \
        REMOVE_EVENT_MESSAGE_##e \
    }

#define REMOVE_EVENT_MESSAGE_1 \
    DELETE 1;

#define REDEFINE_COMMAND_115_VAL_ARRAY_TRANSACTIONS(n) \
    REDEFINE TRANSACTION \
    { \
        REQUEST {eventNumber (INFO, INDEX)} \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            number_events_supported, \
            events[eventNumber].EVENT_CONTROL.EVENT_STATUS<0xf0>, \
            events[eventNumber].EVENT_CONTROL.EVENT_NOTIFICATION_CONTROL<0x0f>, \
            events[eventNumber].EVENT_CONTROL.TIME_OF_FIRST_UNACK_EVENT, \
            events[eventNumber].EVENT_CONTROL.EVENT_NOTIFICATION_RETRY_TIME, \
            events[eventNumber].EVENT_CONTROL.MAX_UPDATE_TIME, \
            events[eventNumber].EVENT_CONTROL.EVENT_DEBOUNCE_INTERVAL, \
            events[eventNumber].EVENT_MASK.DEVICE_STATUS_MASK, \
            COMMAND_115_VAL_ARRAY_BYTES_##n \
        } \
    }

#define REDEFINE_COMMAND_116_VAL_ARRAY_TRANSACTIONS(n) \
    REDEFINE TRANSACTION \
    { \
        REQUEST \
        { \
            eventNumber (INFO, INDEX), \
            events[eventNumber].EVENT_MASK.DEVICE_STATUS_MASK, \
            COMMAND_115_VAL_ARRAY_BYTES_##n \
        } \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            events[eventNumber].EVENT_MASK.DEVICE_STATUS_MASK, \
            COMMAND_115_VAL_ARRAY_BYTES_##n \
        } \
    }

#define COMMAND_115_VAL_ARRAY_BYTES_1     \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_0_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_2     \
    COMMAND_115_VAL_ARRAY_BYTES_1,        \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_1_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_3     \
    COMMAND_115_VAL_ARRAY_BYTES_2,        \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_2_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_4     \
    COMMAND_115_VAL_ARRAY_BYTES_3,        \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_3_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_5     \
    COMMAND_115_VAL_ARRAY_BYTES_4,        \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_4_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_6     \
    COMMAND_115_VAL_ARRAY_BYTES_5,        \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_5_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_7     \
    COMMAND_115_VAL_ARRAY_BYTES_6,        \
    events[eventNumber].EVENT_MASK.EXTENDED_FLD_DEVICE_STATUS_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_8     \
    COMMAND_115_VAL_ARRAY_BYTES_7,        \
    events[eventNumber].EVENT_MASK.DEV_OPERATING_MODE_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_9     \
    COMMAND_115_VAL_ARRAY_BYTES_8,        \
    events[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_0_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_10    \
    COMMAND_115_VAL_ARRAY_BYTES_9,        \
    events[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_1_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_11    \
    COMMAND_115_VAL_ARRAY_BYTES_10,       \
    events[eventNumber].EVENT_MASK.ANALOG_CHANNEL_SATURATED1_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_12    \
    COMMAND_115_VAL_ARRAY_BYTES_11,       \
    events[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_2_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_13    \
    COMMAND_115_VAL_ARRAY_BYTES_12,       \
    events[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_3_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_14    \
    COMMAND_115_VAL_ARRAY_BYTES_13,       \
    events[eventNumber].EVENT_MASK.ANALOG_CHANNEL_FIXED1_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_15    \
    COMMAND_115_VAL_ARRAY_BYTES_14,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_14_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_16    \
    COMMAND_115_VAL_ARRAY_BYTES_15,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_15_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_17    \
    COMMAND_115_VAL_ARRAY_BYTES_16,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_16_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_18    \
    COMMAND_115_VAL_ARRAY_BYTES_17,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_17_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_19    \
    COMMAND_115_VAL_ARRAY_BYTES_18,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_18_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_20    \
    COMMAND_115_VAL_ARRAY_BYTES_19,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_19_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_21    \
    COMMAND_115_VAL_ARRAY_BYTES_20,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_20_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_22    \
    COMMAND_115_VAL_ARRAY_BYTES_21,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_21_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_23    \
    COMMAND_115_VAL_ARRAY_BYTES_22,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_22_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_24    \
    COMMAND_115_VAL_ARRAY_BYTES_23,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_23_MASK

#define COMMAND_115_VAL_ARRAY_BYTES_25    \
    COMMAND_115_VAL_ARRAY_BYTES_24,       \
    events[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_24_MASK


#define REDEFINE_COMMAND_119_VAL_ARRAY_TRANSACTIONS(n) \
    REDEFINE TRANSACTION 0\
    { \
        REQUEST \
        { \
            eventNumber (INFO, INDEX) \
        } \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            events[eventNumber].EVENT_REPORT.DEVICE_STATUS_LATCHED_VALUE, \
            COMMAND_119_VAL_ARRAY_BYTES_##n \
        } \
    } \
    REDEFINE TRANSACTION 1\
    { \
        REQUEST \
        { \
            eventNumber (INFO, INDEX), \
            events[eventNumber].EVENT_REPORT.DEVICE_STATUS_LATCHED_VALUE (INFO), \
            COMMAND_119_VAL_ARRAY_INFO_BYTES_##n \
        } \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            events[eventNumber].EVENT_REPORT.DEVICE_STATUS_LATCHED_VALUE (INFO), \
            COMMAND_119_VAL_ARRAY_INFO_BYTES_##n \
        } \
    }

#define COMMAND_119_VAL_ARRAY_BYTES_1     \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_0_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_2     \
    COMMAND_119_VAL_ARRAY_BYTES_1,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_1_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_3     \
    COMMAND_119_VAL_ARRAY_BYTES_2,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_2_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_4     \
    COMMAND_119_VAL_ARRAY_BYTES_3,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_3_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_5     \
    COMMAND_119_VAL_ARRAY_BYTES_4,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_4_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_6     \
    COMMAND_119_VAL_ARRAY_BYTES_5,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_5_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_7     \
    COMMAND_119_VAL_ARRAY_BYTES_6,        \
    events[eventNumber].EVENT_REPORT.EXTENDED_FLD_DEVICE_STATUS_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_8     \
    COMMAND_119_VAL_ARRAY_BYTES_7,        \
    events[eventNumber].EVENT_REPORT.DEV_OPERATING_MODE_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_9     \
    COMMAND_119_VAL_ARRAY_BYTES_8,        \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_0_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_10    \
    COMMAND_119_VAL_ARRAY_BYTES_9,        \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_1_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_11    \
    COMMAND_119_VAL_ARRAY_BYTES_10,       \
    events[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_SATURATED1_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_12    \
    COMMAND_119_VAL_ARRAY_BYTES_11,       \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_2_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_13    \
    COMMAND_119_VAL_ARRAY_BYTES_12,       \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_3_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_14    \
    COMMAND_119_VAL_ARRAY_BYTES_13,       \
    events[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_FIXED1_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_15    \
    COMMAND_119_VAL_ARRAY_BYTES_14,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_14_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_16    \
    COMMAND_119_VAL_ARRAY_BYTES_15,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_15_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_17    \
    COMMAND_119_VAL_ARRAY_BYTES_16,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_16_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_18    \
    COMMAND_119_VAL_ARRAY_BYTES_17,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_17_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_19    \
    COMMAND_119_VAL_ARRAY_BYTES_18,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_18_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_20    \
    COMMAND_119_VAL_ARRAY_BYTES_19,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_19_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_21    \
    COMMAND_119_VAL_ARRAY_BYTES_20,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_20_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_22    \
    COMMAND_119_VAL_ARRAY_BYTES_21,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_21_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_23    \
    COMMAND_119_VAL_ARRAY_BYTES_22,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_22_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_24    \
    COMMAND_119_VAL_ARRAY_BYTES_23,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_23_LATCHED_VALUE

#define COMMAND_119_VAL_ARRAY_BYTES_25    \
    COMMAND_119_VAL_ARRAY_BYTES_24,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_24_LATCHED_VALUE


#define COMMAND_119_VAL_ARRAY_INFO_BYTES_1     \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_0_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_2     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_1,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_1_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_3     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_2,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_2_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_4     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_3,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_3_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_5     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_4,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_4_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_6     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_5,        \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_5_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_7     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_6,        \
    events[eventNumber].EVENT_REPORT.EXTENDED_FLD_DEVICE_STATUS_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_8     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_7,        \
    events[eventNumber].EVENT_REPORT.DEV_OPERATING_MODE_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_9     \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_8,        \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_0_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_10    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_9,        \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_1_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_11    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_10,       \
    events[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_SATURATED1_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_12    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_11,       \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_2_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_13    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_12,       \
    events[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_3_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_14    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_13,       \
    events[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_FIXED1_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_15    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_14,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_14_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_16    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_15,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_15_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_17    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_16,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_16_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_18    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_17,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_17_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_19    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_18,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_18_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_20    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_19,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_19_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_21    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_20,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_20_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_22    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_21,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_21_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_23    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_22,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_22_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_24    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_23,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_23_LATCHED_VALUE (INFO)

#define COMMAND_119_VAL_ARRAY_INFO_BYTES_25    \
    COMMAND_119_VAL_ARRAY_INFO_BYTES_24,       \
    events[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_24_LATCHED_VALUE (INFO)


#define REDEFINE_TREND_VAL_ARRAY_SIZE(t) \
    REDEFINE NUMBER_OF_ELEMENTS t;

#define IMPORT_BURST_VAL_ARRAY_EVENT_REGISTER() \
    COMMAND register_event_manager; \
    COMMAND read_event_manager_registration_status;

#define IMPORT_BURST_VAL_ARRAY_SUBDEVICE() \
    VARIABLE subdevice_missing; \
    VARIABLE event_subdevice_index; \
    VARIABLE burst_subdevice_index; \
    VARIABLE constOne; \
    VARIABLE constZero; \
    COMMAND read_subdevice_burst_map; \
    COMMAND write_subdevice_burst_map;

#define IMPORT_BURST_VAL_ARRAY_TREND() \
    VARIABLE trend_sample_interval; \
    VARIABLE trend_0_time_stamp; \
    VARIABLE trend_0_date_stamp; \
    VARIABLE trend_value; \
    VARIABLE trend_number; \
    VARIABLE number_trends_supported; \
    VARIABLE trend_device_variable_code; \
    ARRAY trend_values; \
    ARRAY trend_array; \
    COLLECTION std_trend_collection; \
    COLLECTION trend_data; \
    COMMAND read_trend_configuration; \
    COMMAND write_trend_configuration; \
    COMMAND read_trend; \
    METHOD readTrendFromDevice;

#define IMPORT_BURST_REF_ARRAY_BASE(u,b) \
    BURST_REF_ARRAY_BASE_##u##_##b
    
#define BURST_REF_ARRAY_BASE_5_1 \
    VARIABLE burst_command_number; \
    VARIABLE burst_variable_code_1; \
    VARIABLE burst_variable_code_2; \
    VARIABLE burst_variable_code_3; \
    VARIABLE burst_variable_code_4; \
    VARIABLE dummy_variable; \
    VARIABLE dummy_variable_float; \
    VARIABLE dummy_variable_unit; \
    VARIABLE dummy_variable_unsigned_3; \
    ARRAY OF ARRAY burst_variables; \
    ARRAY OF COLLECTION burstDeviceVariables; \
    ARRAY OF COLLECTION burst_message_array; \
    ARRAY OF VARIABLE burst_variables_0; \
    COLLECTION std_burst_message_collection; \
    COLLECTION dummy_collection; \
    COMMAND write_burst_device_variables; \
    COMMAND write_burst_command_number; \
    COMMAND burst_mode_control;

#define BURST_REF_ARRAY_BASE_6_1 \
    BURST_REF_ARRAY_BASE_5_1 \
    COMMAND read_burst_mode_configuration;

#define BURST_REF_ARRAY_BASE_7_1 \
    BURST_REF_ARRAY_BASE_6_1 \
    UNIT burst_trigger_0_units_relation; \
    REFRESH burst_message_0_relation; \
    VARIABLE total_number_burst_messages; \
    VARIABLE update_period; \
    VARIABLE max_update_period; \
    VARIABLE burst_variable_code_5; \
    VARIABLE burst_variable_code_6; \
    VARIABLE burst_variable_code_7; \
    VARIABLE burst_variable_code_8; \
    VARIABLE dummy_variable_time_int; \
    VARIABLE trigger_level; \
    VARIABLE burst_message_number; \
    COMMAND write_burst_period; \
    COMMAND write_burst_trigger;

#define BURST_REF_ARRAY_BASE_7_2 \
    BURST_REF_ARRAY_BASE_7_1 \
    VARIABLE burst_command_1; \
    VARIABLE burst_trigger_level_1; \
    VARIABLE update_period_1; \
    VARIABLE max_update_period_1; \
    VARIABLE burst_variable_code_0_message_1; \
    VARIABLE burst_variable_code_1_message_1; \
    VARIABLE burst_variable_code_2_message_1; \
    VARIABLE burst_variable_code_3_message_1; \
    VARIABLE burst_variable_code_4_message_1; \
    VARIABLE burst_variable_code_5_message_1; \
    VARIABLE burst_variable_code_6_message_1; \
    VARIABLE burst_variable_code_7_message_1; \
    ARRAY OF VARIABLE burst_variables_1; \
    COLLECTION burst_message_1; \
    UNIT burst_trigger_1_units_relation; \
    REFRESH burst_message_1_relation;

#define BURST_REF_ARRAY_BASE_7_3 \
    BURST_REF_ARRAY_BASE_7_2 \
    VARIABLE burst_command_2; \
    VARIABLE burst_trigger_level_2; \
    VARIABLE update_period_2; \
    VARIABLE max_update_period_2; \
    VARIABLE burst_variable_code_0_message_2; \
    VARIABLE burst_variable_code_1_message_2; \
    VARIABLE burst_variable_code_2_message_2; \
    VARIABLE burst_variable_code_3_message_2; \
    VARIABLE burst_variable_code_4_message_2; \
    VARIABLE burst_variable_code_5_message_2; \
    VARIABLE burst_variable_code_6_message_2; \
    VARIABLE burst_variable_code_7_message_2; \
    ARRAY OF VARIABLE burst_variables_2; \
    COLLECTION burst_message_2; \
    UNIT burst_trigger_2_units_relation; \
    REFRESH burst_message_2_relation;

#define BURST_REF_ARRAY_BASE_7_4 \
    BURST_REF_ARRAY_BASE_7_3 \
    VARIABLE burst_command_3; \
    VARIABLE burst_trigger_level_3; \
    VARIABLE update_period_3; \
    VARIABLE max_update_period_3; \
    VARIABLE burst_variable_code_0_message_3; \
    VARIABLE burst_variable_code_1_message_3; \
    VARIABLE burst_variable_code_2_message_3; \
    VARIABLE burst_variable_code_3_message_3; \
    VARIABLE burst_variable_code_4_message_3; \
    VARIABLE burst_variable_code_5_message_3; \
    VARIABLE burst_variable_code_6_message_3; \
    VARIABLE burst_variable_code_7_message_3; \
    ARRAY OF VARIABLE burst_variables_3; \
    COLLECTION burst_message_3; \
    UNIT burst_trigger_3_units_relation; \
    REFRESH burst_message_3_relation;

#define IMPORT_BURST_REF_ARRAY_EVENT(e) \
    BURST_REF_ARRAY_EVENT_##e
    
#define BURST_REF_ARRAY_EVENT_1 \
    VARIABLE number_events_supported; \
    VARIABLE time_of_first_unack_event; \
    VARIABLE event_notification_retry_time_event; \
    VARIABLE max_update_time_event; \
    VARIABLE event_debounce_interval_event; \
    VARIABLE eventNumber; \
    ARRAY OF COLLECTION event_array; \
    COLLECTION event; \
    COLLECTION event_control; \
    COLLECTION event_mask; \
    COLLECTION event_report; \
    COMMAND read_event_notification_summary; \
    COMMAND write_event_notification_bit_mask; \
    COMMAND write_event_notification_timing; \
    COMMAND event_notification_control; \
    COMMAND acknowledge_event_notification;

#define BURST_REF_ARRAY_EVENT_2 \
    BURST_REF_ARRAY_EVENT_1 \
    VARIABLE time_of_first_unack_event_1; \
    VARIABLE event_notification_retry_time_event_1; \
    VARIABLE max_update_time_event_1; \
    VARIABLE event_debounce_interval_event_1; \
    COLLECTION event1; \
    COLLECTION event_control_1; \
    COLLECTION event_mask_1; \
    COLLECTION event_report_1;


#define REDEFINE_EVENT_MASK_SIZE(n)            \
        REMOVE_EVENT_MASK_##n

#define REMOVE_EVENT_MASK_24                   \
    DELETE DEVICE_SPECIFIC_STATUS_24_MASK;

#define REMOVE_EVENT_MASK_23                   \
    REMOVE_EVENT_MASK_24                       \
    DELETE DEVICE_SPECIFIC_STATUS_23_MASK;

#define REMOVE_EVENT_MASK_22                   \
    REMOVE_EVENT_MASK_23                       \
    DELETE DEVICE_SPECIFIC_STATUS_22_MASK;

#define REMOVE_EVENT_MASK_21                   \
    REMOVE_EVENT_MASK_22                       \
    DELETE DEVICE_SPECIFIC_STATUS_21_MASK;

#define REMOVE_EVENT_MASK_20                   \
    REMOVE_EVENT_MASK_21                       \
    DELETE DEVICE_SPECIFIC_STATUS_20_MASK;

#define REMOVE_EVENT_MASK_19                   \
    REMOVE_EVENT_MASK_20                       \
    DELETE DEVICE_SPECIFIC_STATUS_19_MASK;

#define REMOVE_EVENT_MASK_18                   \
    REMOVE_EVENT_MASK_19                       \
    DELETE DEVICE_SPECIFIC_STATUS_18_MASK;

#define REMOVE_EVENT_MASK_17                   \
    REMOVE_EVENT_MASK_18                       \
    DELETE DEVICE_SPECIFIC_STATUS_17_MASK;

#define REMOVE_EVENT_MASK_16                   \
    REMOVE_EVENT_MASK_17                       \
    DELETE DEVICE_SPECIFIC_STATUS_16_MASK;

#define REMOVE_EVENT_MASK_15                   \
    REMOVE_EVENT_MASK_16                       \
    DELETE DEVICE_SPECIFIC_STATUS_15_MASK;

#define REMOVE_EVENT_MASK_14                   \
    REMOVE_EVENT_MASK_15                       \
    DELETE DEVICE_SPECIFIC_STATUS_14_MASK;

#define REMOVE_EVENT_MASK_13                   \
    REMOVE_EVENT_MASK_14                       \
    DELETE ANALOG_CHANNEL_FIXED1_MASK;

#define REMOVE_EVENT_MASK_12                   \
    REMOVE_EVENT_MASK_13                       \
    DELETE STANDARDIZED_STATUS_3_MASK;

#define REMOVE_EVENT_MASK_11                   \
    REMOVE_EVENT_MASK_12                       \
    DELETE STANDARDIZED_STATUS_2_MASK;

#define REMOVE_EVENT_MASK_10                   \
    REMOVE_EVENT_MASK_11                       \
    DELETE ANALOG_CHANNEL_SATURATED1_MASK;

#define REMOVE_EVENT_MASK_9                    \
    REMOVE_EVENT_MASK_10                       \
    DELETE STANDARDIZED_STATUS_1_MASK;

#define REMOVE_EVENT_MASK_8                    \
    REMOVE_EVENT_MASK_9                        \
    DELETE STANDARDIZED_STATUS_0_MASK;

#define REMOVE_EVENT_MASK_7                    \
    REMOVE_EVENT_MASK_8                        \
    DELETE DEV_OPERATING_MODE_MASK;

#define REMOVE_EVENT_MASK_6                    \
    REMOVE_EVENT_MASK_7                        \
    DELETE EXTENDED_FLD_DEVICE_STATUS_MASK;

#define REMOVE_EVENT_MASK_5                    \
    REMOVE_EVENT_MASK_6                        \
    DELETE DEVICE_SPECIFIC_STATUS_5_MASK;

#define REMOVE_EVENT_MASK_4                    \
    REMOVE_EVENT_MASK_5                        \
    DELETE DEVICE_SPECIFIC_STATUS_4_MASK;

#define REMOVE_EVENT_MASK_3                    \
    REMOVE_EVENT_MASK_4                        \
    DELETE DEVICE_SPECIFIC_STATUS_3_MASK;

#define REMOVE_EVENT_MASK_2                    \
    REMOVE_EVENT_MASK_3                        \
    DELETE DEVICE_SPECIFIC_STATUS_2_MASK;

#define REMOVE_EVENT_MASK_1                    \
    REMOVE_EVENT_MASK_2                        \
    DELETE DEVICE_SPECIFIC_STATUS_1_MASK;


#define REDEFINE_EVENT_REPORT_SIZE(n)               \
        REMOVE_EVENT_REPORT_##n

#define REMOVE_EVENT_REPORT_24                      \
    DELETE DEVICE_SPECIFIC_STATUS_24_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_23                      \
    REMOVE_EVENT_REPORT_24                          \
    DELETE DEVICE_SPECIFIC_STATUS_23_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_22                      \
    REMOVE_EVENT_REPORT_23                          \
    DELETE DEVICE_SPECIFIC_STATUS_22_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_21                      \
    REMOVE_EVENT_REPORT_22                          \
    DELETE DEVICE_SPECIFIC_STATUS_21_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_20                      \
    REMOVE_EVENT_REPORT_21                          \
    DELETE DEVICE_SPECIFIC_STATUS_20_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_19                      \
    REMOVE_EVENT_REPORT_20                          \
    DELETE DEVICE_SPECIFIC_STATUS_19_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_18                      \
    REMOVE_EVENT_REPORT_19                          \
    DELETE DEVICE_SPECIFIC_STATUS_18_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_17                      \
    REMOVE_EVENT_REPORT_18                          \
    DELETE DEVICE_SPECIFIC_STATUS_17_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_16                      \
    REMOVE_EVENT_REPORT_17                          \
    DELETE DEVICE_SPECIFIC_STATUS_16_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_15                      \
    REMOVE_EVENT_REPORT_16                          \
    DELETE DEVICE_SPECIFIC_STATUS_15_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_14                      \
    REMOVE_EVENT_REPORT_15                          \
    DELETE DEVICE_SPECIFIC_STATUS_14_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_13                      \
    REMOVE_EVENT_REPORT_14                          \
    DELETE ANALOG_CHANNEL_FIXED1_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_12                      \
    REMOVE_EVENT_REPORT_13                          \
    DELETE STANDARDIZED_STATUS_3_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_11                      \
    REMOVE_EVENT_REPORT_12                          \
    DELETE STANDARDIZED_STATUS_2_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_10                      \
    REMOVE_EVENT_REPORT_11                          \
    DELETE ANALOG_CHANNEL_SATURATED1_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_9                       \
    REMOVE_EVENT_REPORT_10                          \
    DELETE STANDARDIZED_STATUS_1_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_8                       \
    REMOVE_EVENT_REPORT_9                           \
    DELETE STANDARDIZED_STATUS_0_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_7                       \
    REMOVE_EVENT_REPORT_8                           \
    DELETE DEV_OPERATING_MODE_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_6                       \
    REMOVE_EVENT_REPORT_7                           \
    DELETE EXTENDED_FLD_DEVICE_STATUS_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_5                       \
    REMOVE_EVENT_REPORT_6                           \
    DELETE DEVICE_SPECIFIC_STATUS_5_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_4                       \
    REMOVE_EVENT_REPORT_5                           \
    DELETE DEVICE_SPECIFIC_STATUS_4_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_3                       \
    REMOVE_EVENT_REPORT_4                           \
    DELETE DEVICE_SPECIFIC_STATUS_3_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_2                       \
    REMOVE_EVENT_REPORT_3                           \
    DELETE DEVICE_SPECIFIC_STATUS_2_LATCHED_VALUE;

#define REMOVE_EVENT_REPORT_1                       \
    REMOVE_EVENT_REPORT_2                           \
    DELETE DEVICE_SPECIFIC_STATUS_1_LATCHED_VALUE;

#define REDEFINE_COMMAND_115_REF_ARRAY_TRANSACTIONS(n) \
    REDEFINE TRANSACTION \
    { \
        REQUEST {eventNumber (INFO, INDEX)} \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            number_events_supported, \
            event_array[eventNumber].EVENT_CONTROL.EVENT_STATUS<0xf0>, \
            event_array[eventNumber].EVENT_CONTROL.EVENT_NOTIFICATION_CONTROL<0x0f>, \
            event_array[eventNumber].EVENT_CONTROL.TIME_OF_FIRST_UNACK_EVENT, \
            event_array[eventNumber].EVENT_CONTROL.EVENT_NOTIFICATION_RETRY_TIME, \
            event_array[eventNumber].EVENT_CONTROL.MAX_UPDATE_TIME, \
            event_array[eventNumber].EVENT_CONTROL.EVENT_DEBOUNCE_INTERVAL, \
            event_array[eventNumber].EVENT_MASK.DEVICE_STATUS_MASK, \
            COMMAND_115_REF_ARRAY_BYTES_##n \
        } \
    }

#define REDEFINE_COMMAND_116_REF_ARRAY_TRANSACTIONS(n) \
    REDEFINE TRANSACTION \
    { \
        REQUEST \
        { \
            eventNumber (INFO, INDEX), \
            event_array[eventNumber].EVENT_MASK.DEVICE_STATUS_MASK, \
            COMMAND_115_REF_ARRAY_BYTES_##n \
        } \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            event_array[eventNumber].EVENT_MASK.DEVICE_STATUS_MASK, \
            COMMAND_115_REF_ARRAY_BYTES_##n \
        } \
    }

#define COMMAND_115_REF_ARRAY_BYTES_1     \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_0_MASK

#define COMMAND_115_REF_ARRAY_BYTES_2     \
    COMMAND_115_REF_ARRAY_BYTES_1,        \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_1_MASK

#define COMMAND_115_REF_ARRAY_BYTES_3     \
    COMMAND_115_REF_ARRAY_BYTES_2,        \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_2_MASK

#define COMMAND_115_REF_ARRAY_BYTES_4     \
    COMMAND_115_REF_ARRAY_BYTES_3,        \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_3_MASK

#define COMMAND_115_REF_ARRAY_BYTES_5     \
    COMMAND_115_REF_ARRAY_BYTES_4,        \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_4_MASK

#define COMMAND_115_REF_ARRAY_BYTES_6     \
    COMMAND_115_REF_ARRAY_BYTES_5,        \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_5_MASK

#define COMMAND_115_REF_ARRAY_BYTES_7     \
    COMMAND_115_REF_ARRAY_BYTES_6,        \
    event_array[eventNumber].EVENT_MASK.EXTENDED_FLD_DEVICE_STATUS_MASK

#define COMMAND_115_REF_ARRAY_BYTES_8     \
    COMMAND_115_REF_ARRAY_BYTES_7,        \
    event_array[eventNumber].EVENT_MASK.DEV_OPERATING_MODE_MASK

#define COMMAND_115_REF_ARRAY_BYTES_9     \
    COMMAND_115_REF_ARRAY_BYTES_8,        \
    event_array[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_0_MASK

#define COMMAND_115_REF_ARRAY_BYTES_10    \
    COMMAND_115_REF_ARRAY_BYTES_9,        \
    event_array[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_1_MASK

#define COMMAND_115_REF_ARRAY_BYTES_11    \
    COMMAND_115_REF_ARRAY_BYTES_10,       \
    event_array[eventNumber].EVENT_MASK.ANALOG_CHANNEL_SATURATED1_MASK

#define COMMAND_115_REF_ARRAY_BYTES_12    \
    COMMAND_115_REF_ARRAY_BYTES_11,       \
    event_array[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_2_MASK

#define COMMAND_115_REF_ARRAY_BYTES_13    \
    COMMAND_115_REF_ARRAY_BYTES_12,       \
    event_array[eventNumber].EVENT_MASK.STANDARDIZED_STATUS_3_MASK

#define COMMAND_115_REF_ARRAY_BYTES_14    \
    COMMAND_115_REF_ARRAY_BYTES_13,       \
    event_array[eventNumber].EVENT_MASK.ANALOG_CHANNEL_FIXED1_MASK

#define COMMAND_115_REF_ARRAY_BYTES_15    \
    COMMAND_115_REF_ARRAY_BYTES_14,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_14_MASK

#define COMMAND_115_REF_ARRAY_BYTES_16    \
    COMMAND_115_REF_ARRAY_BYTES_15,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_15_MASK

#define COMMAND_115_REF_ARRAY_BYTES_17    \
    COMMAND_115_REF_ARRAY_BYTES_16,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_16_MASK

#define COMMAND_115_REF_ARRAY_BYTES_18    \
    COMMAND_115_REF_ARRAY_BYTES_17,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_17_MASK

#define COMMAND_115_REF_ARRAY_BYTES_19    \
    COMMAND_115_REF_ARRAY_BYTES_18,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_18_MASK

#define COMMAND_115_REF_ARRAY_BYTES_20    \
    COMMAND_115_REF_ARRAY_BYTES_19,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_19_MASK

#define COMMAND_115_REF_ARRAY_BYTES_21    \
    COMMAND_115_REF_ARRAY_BYTES_20,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_20_MASK

#define COMMAND_115_REF_ARRAY_BYTES_22    \
    COMMAND_115_REF_ARRAY_BYTES_21,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_21_MASK

#define COMMAND_115_REF_ARRAY_BYTES_23    \
    COMMAND_115_REF_ARRAY_BYTES_22,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_22_MASK

#define COMMAND_115_REF_ARRAY_BYTES_24    \
    COMMAND_115_REF_ARRAY_BYTES_23,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_23_MASK

#define COMMAND_115_REF_ARRAY_BYTES_25    \
    COMMAND_115_REF_ARRAY_BYTES_24,       \
    event_array[eventNumber].EVENT_MASK.DEVICE_SPECIFIC_STATUS_24_MASK


#define REDEFINE_COMMAND_119_REF_ARRAY_TRANSACTIONS(n) \
    REDEFINE TRANSACTION 0\
    { \
        REQUEST \
        { \
            eventNumber (INFO, INDEX) \
        } \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            event_array[eventNumber].EVENT_REPORT.DEVICE_STATUS_LATCHED_VALUE, \
            COMMAND_119_REF_ARRAY_BYTES_##n \
        } \
    } \
    REDEFINE TRANSACTION 1\
    { \
        REQUEST \
        { \
            eventNumber (INFO, INDEX), \
            event_array[eventNumber].EVENT_REPORT.DEVICE_STATUS_LATCHED_VALUE (INFO), \
            COMMAND_119_REF_ARRAY_INFO_BYTES_##n \
        } \
        REPLY \
        { \
            response_code, device_status, \
            eventNumber (INFO, INDEX), \
            event_array[eventNumber].EVENT_REPORT.DEVICE_STATUS_LATCHED_VALUE (INFO), \
            COMMAND_119_REF_ARRAY_INFO_BYTES_##n \
        } \
    }

#define COMMAND_119_REF_ARRAY_BYTES_1     \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_0_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_2     \
    COMMAND_119_REF_ARRAY_BYTES_1,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_1_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_3     \
    COMMAND_119_REF_ARRAY_BYTES_2,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_2_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_4     \
    COMMAND_119_REF_ARRAY_BYTES_3,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_3_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_5     \
    COMMAND_119_REF_ARRAY_BYTES_4,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_4_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_6     \
    COMMAND_119_REF_ARRAY_BYTES_5,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_5_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_7     \
    COMMAND_119_REF_ARRAY_BYTES_6,        \
    event_array[eventNumber].EVENT_REPORT.EXTENDED_FLD_DEVICE_STATUS_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_8     \
    COMMAND_119_REF_ARRAY_BYTES_7,        \
    event_array[eventNumber].EVENT_REPORT.DEV_OPERATING_MODE_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_9     \
    COMMAND_119_REF_ARRAY_BYTES_8,        \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_0_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_10    \
    COMMAND_119_REF_ARRAY_BYTES_9,        \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_1_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_11    \
    COMMAND_119_REF_ARRAY_BYTES_10,       \
    event_array[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_SATURATED1_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_12    \
    COMMAND_119_REF_ARRAY_BYTES_11,       \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_2_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_13    \
    COMMAND_119_REF_ARRAY_BYTES_12,       \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_3_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_14    \
    COMMAND_119_REF_ARRAY_BYTES_13,       \
    event_array[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_FIXED1_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_15    \
    COMMAND_119_REF_ARRAY_BYTES_14,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_14_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_16    \
    COMMAND_119_REF_ARRAY_BYTES_15,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_15_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_17    \
    COMMAND_119_REF_ARRAY_BYTES_16,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_16_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_18    \
    COMMAND_119_REF_ARRAY_BYTES_17,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_17_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_19    \
    COMMAND_119_REF_ARRAY_BYTES_18,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_18_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_20    \
    COMMAND_119_REF_ARRAY_BYTES_19,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_19_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_21    \
    COMMAND_119_REF_ARRAY_BYTES_20,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_20_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_22    \
    COMMAND_119_REF_ARRAY_BYTES_21,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_21_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_23    \
    COMMAND_119_REF_ARRAY_BYTES_22,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_22_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_24    \
    COMMAND_119_REF_ARRAY_BYTES_23,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_23_LATCHED_VALUE

#define COMMAND_119_REF_ARRAY_BYTES_25    \
    COMMAND_119_REF_ARRAY_BYTES_24,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_24_LATCHED_VALUE


#define COMMAND_119_REF_ARRAY_INFO_BYTES_1     \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_0_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_2     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_1,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_1_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_3     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_2,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_2_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_4     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_3,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_3_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_5     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_4,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_4_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_6     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_5,        \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_5_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_7     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_6,        \
    event_array[eventNumber].EVENT_REPORT.EXTENDED_FLD_DEVICE_STATUS_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_8     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_7,        \
    event_array[eventNumber].EVENT_REPORT.DEV_OPERATING_MODE_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_9     \
    COMMAND_119_REF_ARRAY_INFO_BYTES_8,        \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_0_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_10    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_9,        \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_1_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_11    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_10,       \
    event_array[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_SATURATED1_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_12    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_11,       \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_2_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_13    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_12,       \
    event_array[eventNumber].EVENT_REPORT.STANDARDIZED_STATUS_3_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_14    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_13,       \
    event_array[eventNumber].EVENT_REPORT.ANALOG_CHANNEL_FIXED1_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_15    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_14,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_14_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_16    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_15,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_15_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_17    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_16,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_16_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_18    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_17,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_17_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_19    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_18,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_18_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_20    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_19,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_19_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_21    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_20,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_20_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_22    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_21,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_21_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_23    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_22,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_22_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_24    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_23,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_23_LATCHED_VALUE (INFO)

#define COMMAND_119_REF_ARRAY_INFO_BYTES_25    \
    COMMAND_119_REF_ARRAY_INFO_BYTES_24,       \
    event_array[eventNumber].EVENT_REPORT.DEVICE_SPECIFIC_STATUS_24_LATCHED_VALUE (INFO)


#define IMPORT_BURST_REF_ARRAY_EVENT_REGISTER() \
    COMMAND register_event_manager; \
    COMMAND read_event_manager_registration_status;

#define IMPORT_BURST_REF_ARRAY_SUBDEVICE_BURST(b) \
    BURST_REF_ARRAY_SUBDEVICE_BURST_##b
    
#define BURST_REF_ARRAY_SUBDEVICE_BURST_1 \
    VARIABLE subdevice_missing; \
    VARIABLE burst_subdevice_index_int; \
    VARIABLE constOne; \
    VARIABLE constZero; \
    COMMAND read_subdevice_burst_map; \
    COMMAND write_subdevice_burst_map;

#define BURST_REF_ARRAY_SUBDEVICE_BURST_2 \
    BURST_REF_ARRAY_SUBDEVICE_BURST_1 \
    VARIABLE subdevice_missing_1; \
    VARIABLE burst_subdevice_index_1;

#define BURST_REF_ARRAY_SUBDEVICE_BURST_3 \
    BURST_REF_ARRAY_SUBDEVICE_BURST_2 \
    VARIABLE subdevice_missing_2; \
    VARIABLE burst_subdevice_index_2;

#define BURST_REF_ARRAY_SUBDEVICE_BURST_4 \
    BURST_REF_ARRAY_SUBDEVICE_BURST_3 \
    VARIABLE subdevice_missing_3; \
    VARIABLE burst_subdevice_index_3;

#define IMPORT_BURST_REF_ARRAY_SUBDEVICE_EVENT(e) \
    BURST_REF_ARRAY_SUBDEVICE_EVENT_##e
    
#define BURST_REF_ARRAY_SUBDEVICE_EVENT_1 \
    VARIABLE event_subdevice_index_int;

#define BURST_REF_ARRAY_SUBDEVICE_EVENT_2 \
    BURST_REF_ARRAY_SUBDEVICE_EVENT_1 \
    VARIABLE event_subdevice_index_int_1;

#define IMPORT_BURST_REF_ARRAY_TREND() \
    VARIABLE trend_sample_interval; \
    VARIABLE trend_0_time_stamp; \
    VARIABLE trend_0_date_stamp; \
    VARIABLE trend_value; \
    VARIABLE std_trend_value_0_1; \
    VARIABLE std_trend_value_0_2; \
    VARIABLE std_trend_value_0_3; \
    VARIABLE std_trend_value_0_4; \
    VARIABLE std_trend_value_0_5; \
    VARIABLE std_trend_value_0_6; \
    VARIABLE std_trend_value_0_7; \
    VARIABLE std_trend_value_0_8; \
    VARIABLE std_trend_value_0_9; \
    VARIABLE std_trend_value_0_10; \
    VARIABLE std_trend_value_0_11; \
    VARIABLE trend_number; \
    VARIABLE number_trends_supported; \
    VARIABLE trend_device_variable_code; \
    ARRAY OF COLLECTION std_trend_rarray; \
    ARRAY OF COLLECTION std_trend_data_array; \
    COLLECTION std_trend_0; \
    COLLECTION std_trend_data_0; \
    COLLECTION std_trend_data_1; \
    COLLECTION std_trend_data_2; \
    COLLECTION std_trend_data_3; \
    COLLECTION std_trend_data_4; \
    COLLECTION std_trend_data_5; \
    COLLECTION std_trend_data_6; \
    COLLECTION std_trend_data_7; \
    COLLECTION std_trend_data_8; \
    COLLECTION std_trend_data_9; \
    COLLECTION std_trend_data_10; \
    COLLECTION std_trend_data_11; \
    UNIT std_trend_0_units_relation; \
    COMMAND read_trend_configuration; \
    COMMAND write_trend_configuration; \
    COMMAND read_trend; \
    METHOD readTrendFromDevice;

#define REDEFINE_CARD_CHANNEL_VAL_ARRAY_SIZE(c) \
    REDEFINE NUMBER_OF_ELEMENTS c;

#define IMPORT_WIRELESS_BASE() \
    VARIABLE join_key_1; \
    VARIABLE join_key_2; \
    VARIABLE join_key_3; \
    VARIABLE join_key_4; \
    VARIABLE neighbor_count; \
    VARIABLE advertisements; \
    VARIABLE join_attempts; \
    VARIABLE join_retry_timer; \
    VARIABLE network_search_timer; \
    VARIABLE reserved_bits; \
    VARIABLE join_shed_time; \
    VARIABLE network_id; \
    VARIABLE wireless_module_transmitter_revision; \
    VARIABLE wireless_module_software_revision; \
    VARIABLE wireless_module_hardware_revision; \
    VARIABLE radio_power_output; \
    COMMAND write_join_key; \
    COMMAND read_join_status; \
    COMMAND force_join_mode; \
    COMMAND read_join_mode_configuration; \
    COMMAND write_network_id; \
    COMMAND read_network_id; \
    COMMAND read_wireless_module_revision; \
    COMMAND read_radio_output_power;

#define IMPORT_WIRELESS_BASE_V2() \
    IMPORT_WIRELESS_BASE() \
    VARIABLE current_network_id; \
    VARIABLE max_join_retries;

#define IMPORT_WIRELESS_DEVICE_ADVERTISING()  \
    VARIABLE advertised_neighbors_number;     \
    VARIABLE advertising_period;              \
    VARIABLE shed_time;                       \
    COMMAND request_active_advertising;

#define IMPORT_WIRELESS_DEVICE_CAPABILITIES() \
    VARIABLE keep_alive_min_time;             \
    VARIABLE max_neighbors;                   \
    VARIABLE max_packet_buffers;              \
    VARIABLE peak_packet_load;                \
    VARIABLE peak_packets_per_second;         \
    VARIABLE receive_signal_level;            \
    VARIABLE recover_time;                    \
    COMMAND read_wireless_device_capabilities;

#define IMPORT_WIRELESS_NETWORK_ACCESS()      \
    COMMAND read_network_access_mode;         \
    COMMAND write_network_access_mode;

#define IMPORT_WIRELESS_NETWORK_TAG()         \
    VARIABLE network_tag;                     \
    COMMAND read_network_tag;                 \
    COMMAND write_network_tag;

#define IMPORT_WIRELESS_NICKNAME()            \
    VARIABLE device_nickname_address;         \
    COMMAND read_device_nickname_address;

#define IMPORT_IO_LIST_BASE_V9() \
    VARIABLE devices_detected; \
    VARIABLE max_channels_per_io_card; \
    VARIABLE max_dr; \
    VARIABLE max_io_cards; \
    VARIABLE max_sub_dev_per_channel; \
    VARIABLE retry_count; \
    VARIABLE subdevice_number; \
    VARIABLE subdev_IO_card; \
    VARIABLE subdev_channel; \
    VARIABLE subdev_device_id; \
    VARIABLE subdev_universal_command_revision; \
    VARIABLE subdev_long_tag; \
    VARIABLE stx_sent_to_subdev; \
    VARIABLE ack_received_from_subdev; \
    VARIABLE back_received_from_subdev; \
    VARIABLE io_channel_stx_sent; \
    VARIABLE io_channel_ack_received; \
    VARIABLE io_channel_ostx_received; \
    VARIABLE io_channel_oack_received; \
    VARIABLE io_channel_back_received; \
    VARIABLE IO_channel_number; \
    LIST subdevice_list; \
    ARRAY OF ARRAY io_cards; \
    ARRAY card0_channels; \
    COLLECTION sub_device; \
    COLLECTION subdev_identity; \
    COLLECTION subdev_stats; \
    COLLECTION io_stats; \
    COMMAND read_io_system_capabilities; \
    COMMAND read_subdev_id_summary; \
    COMMAND read_io_channel_stats; \
    COMMAND read_subdevice_statistics; \
    COMMAND write_io_system_master_mode; \
    COMMAND write_io_system_retry_count;

#define IMPORT_IO_LIST_BASE_V11() \
    IMPORT_IO_LIST_BASE_V9() \
    VARIABLE subdev_dev_rev;

#define IMPORT_IO_LIST_ASSIGN() \
    VARIABLE subdevs_assigned; \
    VARIABLE max_subdevs_assigned; \
    VARIABLE num_devices_transferred; \
    LIST assignment_list; \
    COLLECTION subdev_assignment; \
    COMMAND read_subdevice_assignment_list_info; \
    COMMAND read_subdevice_assignment; \
    COMMAND write_subdevice_assignment; \
    COMMAND transfer_live_to_assignment_list;

#define IMPORT_IO_LIST_SYSTEM_COMM_STATS() \
    VARIABLE messages_received; \
    VARIABLE messages_returned; \
    VARIABLE requests_forwarded; \
    VARIABLE responses_returned; \
    COMMAND read_io_client_side_communication_statistics;

#define IMPORT_IO_REF_ARRAY_BASE_V9() \
    VARIABLE devices_detected; \
    VARIABLE max_channels_per_io_card; \
    VARIABLE max_dr; \
    VARIABLE max_io_cards; \
    VARIABLE max_sub_dev_per_channel; \
    VARIABLE retry_count; \
    VARIABLE subdev_channel; \
    VARIABLE subdev_device_id; \
    VARIABLE subdev_IO_card; \
    VARIABLE subdev_universal_command_revision; \
    VARIABLE subdevice_number_int; \
    VARIABLE subdev_long_tag; \
    VARIABLE stx_sent_to_subdev; \
    VARIABLE ack_received_from_subdev; \
    VARIABLE back_received_from_subdev; \
    VARIABLE io_channel_stx_sent; \
    VARIABLE io_channel_ack_received; \
    VARIABLE io_channel_ostx_received; \
    VARIABLE io_channel_oack_received; \
    VARIABLE io_channel_back_received; \
    VARIABLE IO_channel_number; \
    ARRAY OF COLLECTION IO_card0; \
    COLLECTION sub_device; \
    COLLECTION subdev_identity; \
    COLLECTION subdev_stats; \
    COLLECTION io_stats; \
    COMMAND read_io_system_capabilities; \
    COMMAND read_subdev_id_summary; \
    COMMAND read_io_channel_stats; \
    COMMAND read_subdevice_statistics; \
    COMMAND write_io_system_master_mode; \
    COMMAND write_io_system_retry_count;

#define IMPORT_IO_REF_ARRAY_BASE_V11() \
    IMPORT_IO_REF_ARRAY_BASE_V9() \
    VARIABLE subdev_dev_rev;

#define IMPORT_IO_REF_ARRAY_ASSIGN() \
    VARIABLE subdevs_assigned; \
    VARIABLE max_subdevs_assigned; \
    VARIABLE num_devices_transferred; \
    VARIABLE subdevice_number; \
    LIST assignment_list; \
    COLLECTION subdev_assignment; \
    COMMAND read_subdevice_assignment_list_info; \
    COMMAND read_subdevice_assignment; \
    COMMAND write_subdevice_assignment; \
    COMMAND transfer_live_to_assignment_list;

#define IMPORT_IO_REF_ARRAY_SYSTEM_COMM_STATS() \
    VARIABLE messages_received; \
    VARIABLE messages_returned; \
    VARIABLE requests_forwarded; \
    VARIABLE responses_returned; \
    COMMAND read_io_client_side_communication_statistics;

#define IMPORT_ANALOG_CHANNELS_MULTI_AO() \
    VARIABLE analog_channel_number_code; \
    VARIABLE analog_channel_number_code_1; \
    VARIABLE analog_channel_number_code_2; \
    VARIABLE analog_channel_number_code_3; \
    VARIABLE analog_channel_number_code_4; \
    COMMAND read_analog_output_and_percent_of_range; \
    COMMAND read_analog_channels; \
    COMMAND read_analog_channel_information; \
    COMMAND write_analog_channel_additional_damping_value; \
    COMMAND write_analog_channel_range_values; \
    COMMAND enter_exit_fixed_analog_channel_mode; \
    COMMAND trim_analog_channel_zero; \
    COMMAND trim_analog_channel_gain; \
    COMMAND write_analog_channel_transfer_function; \
    COMMAND read_analog_channel_endpoint_values;

#define IMPORT_MISC_COMM_STATS() \
    VARIABLE STX_count; \
    VARIABLE ACK_count; \
    VARIABLE BACK_count; \
    COMMAND read_device_communication_statistics;

#define IMPORT_MISC_LOCATION() \
    VARIABLE device_latitude; \
    VARIABLE device_longitude; \
    VARIABLE device_altitude; \
    VARIABLE device_location_description; \
    COMMAND read_device_location; \
    COMMAND write_device_location; \
    COMMAND read_location_description; \
    COMMAND write_location_description;

#define IMPORT_MISC_LOCK() \
    COMMAND lock_device; \
    COMMAND read_lock_device_state;

#define IMPORT_MISC_PROCESS_UNIT_TAG() \
    VARIABLE process_unit_tag; \
    COMMAND read_process_unit_tag; \
    COMMAND write_process_unit_tag;

#define IMPORT_MISC_READ_TIME() \
    VARIABLE current_date; \
    VARIABLE current_time; \
    VARIABLE last_clock_date; \
    VARIABLE last_clock_time; \
    COMMAND read_real_time_clock;

#define IMPORT_MISC_READ_WRITE_TIME() \
    VARIABLE current_date; \
    VARIABLE current_time; \
    VARIABLE last_clock_date; \
    VARIABLE last_clock_time; \
    COMMAND read_real_time_clock; \
    COMMAND set_real_time_clock;

#define IMPORT_MISC_SI_UNIT() \
    COMMAND read_country_code; \
    COMMAND write_country_code;

#define IMPORT_MISC_SQUAWK() \
    COMMAND squawk;

#define IMPORT_CONDENSED_STATUS_BASE() \
    VARIABLE simulate_bit_index; \
    COMMAND read_condensed_status_map; \
    COMMAND write_condensed_status_map; \
    COMMAND reset_condensed_status_map; \
    COMMAND write_status_simulation_mode; \
    COMMAND simulate_status_bit; \
    METHOD Reset_Condensed_Status_Map;

#define IMPORT_TABLES_CONDENSED_STATUS(n) \
    TABLE_CONDENSED_STATUS_##n
    
#define TABLE_CONDENSED_STATUS_10 \
    VARIABLE status_simulate_mode; \
    VARIABLE simulate_bit_value; \
    VARIABLE statusmap_variable_000; \
    VARIABLE statusmap_variable_001; \
    VARIABLE statusmap_variable_002; \
    VARIABLE statusmap_variable_003; \
    VARIABLE statusmap_variable_004; \
    VARIABLE statusmap_variable_005; \
    VARIABLE statusmap_variable_006; \
    VARIABLE statusmap_variable_007; \
    VARIABLE statusmap_variable_008; \
    VARIABLE statusmap_variable_009; \
    VARIABLE statusmap_variable_010; \
    VARIABLE statusmap_variable_011; \
    VARIABLE statusmap_variable_012; \
    VARIABLE statusmap_variable_013; \
    VARIABLE statusmap_variable_014; \
    VARIABLE statusmap_variable_015; \
    VARIABLE statusmap_variable_016; \
    VARIABLE statusmap_variable_017; \
    VARIABLE statusmap_variable_018; \
    VARIABLE statusmap_variable_019; \
    VARIABLE statusmap_variable_020; \
    VARIABLE statusmap_variable_021; \
    VARIABLE statusmap_variable_022; \
    VARIABLE statusmap_variable_023; \
    VARIABLE statusmap_variable_024; \
    VARIABLE statusmap_variable_025; \
    VARIABLE statusmap_variable_026; \
    VARIABLE statusmap_variable_027; \
    VARIABLE statusmap_variable_028; \
    VARIABLE statusmap_variable_029; \
    VARIABLE statusmap_variable_030; \
    VARIABLE statusmap_variable_031; \
    VARIABLE statusmap_variable_032; \
    VARIABLE statusmap_variable_033; \
    VARIABLE statusmap_variable_034; \
    VARIABLE statusmap_variable_035; \
    VARIABLE statusmap_variable_036; \
    VARIABLE statusmap_variable_037; \
    VARIABLE statusmap_variable_038; \
    VARIABLE statusmap_variable_039; \
    VARIABLE statusmap_variable_040; \
    VARIABLE statusmap_variable_041; \
    VARIABLE statusmap_variable_042; \
    VARIABLE statusmap_variable_043; \
    VARIABLE statusmap_variable_044; \
    VARIABLE statusmap_variable_045; \
    VARIABLE statusmap_variable_046; \
    VARIABLE statusmap_variable_047; \
    VARIABLE statusmap_variable_048; \
    VARIABLE statusmap_variable_049; \
    VARIABLE statusmap_variable_050; \
    VARIABLE statusmap_variable_051; \
    VARIABLE statusmap_variable_052; \
    VARIABLE statusmap_variable_053; \
    VARIABLE statusmap_variable_054; \
    VARIABLE statusmap_variable_055; \
    VARIABLE statusmap_variable_056; \
    VARIABLE statusmap_variable_057; \
    VARIABLE statusmap_variable_058; \
    VARIABLE statusmap_variable_059; \
    VARIABLE statusmap_variable_060; \
    VARIABLE statusmap_variable_061; \
    VARIABLE statusmap_variable_062; \
    VARIABLE statusmap_variable_063; \
    VARIABLE statusmap_variable_064; \
    VARIABLE statusmap_variable_065; \
    VARIABLE statusmap_variable_066; \
    VARIABLE statusmap_variable_067; \
    VARIABLE statusmap_variable_068; \
    VARIABLE statusmap_variable_069; \
    VARIABLE statusmap_variable_070; \
    VARIABLE statusmap_variable_071; \
    VARIABLE statusmap_variable_072; \
    VARIABLE statusmap_variable_073; \
    VARIABLE statusmap_variable_074; \
    VARIABLE statusmap_variable_075; \
    VARIABLE statusmap_variable_076; \
    VARIABLE statusmap_variable_077; \
    VARIABLE statusmap_variable_078; \
    VARIABLE statusmap_variable_079; \
    VARIABLE statusmap_variable_080; \
    VARIABLE statusmap_variable_081; \
    VARIABLE statusmap_variable_082; \
    VARIABLE statusmap_variable_083; \
    VARIABLE statusmap_variable_084; \
    VARIABLE statusmap_variable_085; \
    VARIABLE statusmap_variable_086; \
    VARIABLE statusmap_variable_087;

#define TABLE_CONDENSED_STATUS_11 \
    TABLE_CONDENSED_STATUS_10 \
    VARIABLE statusmap_variable_088; \
    VARIABLE statusmap_variable_089; \
    VARIABLE statusmap_variable_090; \
    VARIABLE statusmap_variable_091; \
    VARIABLE statusmap_variable_092; \
    VARIABLE statusmap_variable_093; \
    VARIABLE statusmap_variable_094; \
    VARIABLE statusmap_variable_095;

#define TABLE_CONDENSED_STATUS_12 \
    TABLE_CONDENSED_STATUS_11 \
    VARIABLE statusmap_variable_096; \
    VARIABLE statusmap_variable_097; \
    VARIABLE statusmap_variable_098; \
    VARIABLE statusmap_variable_099; \
    VARIABLE statusmap_variable_100; \
    VARIABLE statusmap_variable_101; \
    VARIABLE statusmap_variable_102; \
    VARIABLE statusmap_variable_103;

#define TABLE_CONDENSED_STATUS_13 \
    TABLE_CONDENSED_STATUS_12 \
    VARIABLE statusmap_variable_104; \
    VARIABLE statusmap_variable_105; \
    VARIABLE statusmap_variable_106; \
    VARIABLE statusmap_variable_107; \
    VARIABLE statusmap_variable_108; \
    VARIABLE statusmap_variable_109; \
    VARIABLE statusmap_variable_110; \
    VARIABLE statusmap_variable_111;

#define TABLE_CONDENSED_STATUS_14 \
    TABLE_CONDENSED_STATUS_13 \
    VARIABLE statusmap_variable_112; \
    VARIABLE statusmap_variable_113; \
    VARIABLE statusmap_variable_114; \
    VARIABLE statusmap_variable_115; \
    VARIABLE statusmap_variable_116; \
    VARIABLE statusmap_variable_117; \
    VARIABLE statusmap_variable_118; \
    VARIABLE statusmap_variable_119;

#define TABLE_CONDENSED_STATUS_15 \
    TABLE_CONDENSED_STATUS_14 \
    VARIABLE statusmap_variable_120; \
    VARIABLE statusmap_variable_121; \
    VARIABLE statusmap_variable_122; \
    VARIABLE statusmap_variable_123; \
    VARIABLE statusmap_variable_124; \
    VARIABLE statusmap_variable_125; \
    VARIABLE statusmap_variable_126; \
    VARIABLE statusmap_variable_127;

#define TABLE_CONDENSED_STATUS_16 \
    TABLE_CONDENSED_STATUS_15 \
    VARIABLE statusmap_variable_128; \
    VARIABLE statusmap_variable_129; \
    VARIABLE statusmap_variable_130; \
    VARIABLE statusmap_variable_131; \
    VARIABLE statusmap_variable_132; \
    VARIABLE statusmap_variable_133; \
    VARIABLE statusmap_variable_134; \
    VARIABLE statusmap_variable_135;

#define TABLE_CONDENSED_STATUS_17 \
    TABLE_CONDENSED_STATUS_16 \
    VARIABLE statusmap_variable_136; \
    VARIABLE statusmap_variable_137; \
    VARIABLE statusmap_variable_138; \
    VARIABLE statusmap_variable_139; \
    VARIABLE statusmap_variable_140; \
    VARIABLE statusmap_variable_141; \
    VARIABLE statusmap_variable_142; \
    VARIABLE statusmap_variable_143;

#define TABLE_CONDENSED_STATUS_18 \
    TABLE_CONDENSED_STATUS_17 \
    VARIABLE statusmap_variable_144; \
    VARIABLE statusmap_variable_145; \
    VARIABLE statusmap_variable_146; \
    VARIABLE statusmap_variable_147; \
    VARIABLE statusmap_variable_148; \
    VARIABLE statusmap_variable_149; \
    VARIABLE statusmap_variable_150; \
    VARIABLE statusmap_variable_151;

#define TABLE_CONDENSED_STATUS_19 \
    TABLE_CONDENSED_STATUS_18 \
    VARIABLE statusmap_variable_152; \
    VARIABLE statusmap_variable_153; \
    VARIABLE statusmap_variable_154; \
    VARIABLE statusmap_variable_155; \
    VARIABLE statusmap_variable_156; \
    VARIABLE statusmap_variable_157; \
    VARIABLE statusmap_variable_158; \
    VARIABLE statusmap_variable_159;

#define TABLE_CONDENSED_STATUS_20 \
    TABLE_CONDENSED_STATUS_19 \
    VARIABLE statusmap_variable_160; \
    VARIABLE statusmap_variable_161; \
    VARIABLE statusmap_variable_162; \
    VARIABLE statusmap_variable_163; \
    VARIABLE statusmap_variable_164; \
    VARIABLE statusmap_variable_165; \
    VARIABLE statusmap_variable_166; \
    VARIABLE statusmap_variable_167;

#define TABLE_CONDENSED_STATUS_21 \
    TABLE_CONDENSED_STATUS_20 \
    VARIABLE statusmap_variable_168; \
    VARIABLE statusmap_variable_169; \
    VARIABLE statusmap_variable_170; \
    VARIABLE statusmap_variable_171; \
    VARIABLE statusmap_variable_172; \
    VARIABLE statusmap_variable_173; \
    VARIABLE statusmap_variable_174; \
    VARIABLE statusmap_variable_175;

#define TABLE_CONDENSED_STATUS_22 \
    TABLE_CONDENSED_STATUS_21 \
    VARIABLE statusmap_variable_176; \
    VARIABLE statusmap_variable_177; \
    VARIABLE statusmap_variable_178; \
    VARIABLE statusmap_variable_179; \
    VARIABLE statusmap_variable_180; \
    VARIABLE statusmap_variable_181; \
    VARIABLE statusmap_variable_182; \
    VARIABLE statusmap_variable_183;

#define TABLE_CONDENSED_STATUS_23 \
    TABLE_CONDENSED_STATUS_22 \
    VARIABLE statusmap_variable_184; \
    VARIABLE statusmap_variable_185; \
    VARIABLE statusmap_variable_186; \
    VARIABLE statusmap_variable_187; \
    VARIABLE statusmap_variable_188; \
    VARIABLE statusmap_variable_189; \
    VARIABLE statusmap_variable_190; \
    VARIABLE statusmap_variable_191;

#define TABLE_CONDENSED_STATUS_24 \
    TABLE_CONDENSED_STATUS_23 \
    VARIABLE statusmap_variable_192; \
    VARIABLE statusmap_variable_193; \
    VARIABLE statusmap_variable_194; \
    VARIABLE statusmap_variable_195; \
    VARIABLE statusmap_variable_196; \
    VARIABLE statusmap_variable_197; \
    VARIABLE statusmap_variable_198; \
    VARIABLE statusmap_variable_199;

#define TABLE_CONDENSED_STATUS_25 \
    TABLE_CONDENSED_STATUS_24 \
    VARIABLE statusmap_variable_200; \
    VARIABLE statusmap_variable_201; \
    VARIABLE statusmap_variable_202; \
    VARIABLE statusmap_variable_203; \
    VARIABLE statusmap_variable_204; \
    VARIABLE statusmap_variable_205; \
    VARIABLE statusmap_variable_206; \
    VARIABLE statusmap_variable_207;

#define IMPORT_CONDENSED_STATUS_METHOD_SIMULATE() \
    VARIABLE status_simulation_byte; \
    VARIABLE userdefined_label_matches; \
    ARRAY OF VARIABLE simulation_items; \
    METHOD set_status_simulation; \
    METHOD abort_disable_status_simulation;

#define REDEFINE_CONDENSED_STATUS_TRANSACTIONS(n) \
    REDEFINE COMMAND read_condensed_status_map \
    { \
        NUMBER 523; \
        OPERATION READ; \
        CONDENSED_STATUS_READ_TRANSACTIONS_##n \
        RESPONSE_CODES \
        { \
/*Remove new dictionary references from legacy binary format*/ \
            0, SUCCESS,            no_command_specific_errors_temp; \
/*Remove new dictionary references from legacy binary format*/ \
            5, DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp; \
/*Remove new dictionary references from legacy binary format*/ \
            8, DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp; \
        } \
    } \
    \
    REDEFINE COMMAND write_condensed_status_map \
    { \
        NUMBER 524; \
        OPERATION WRITE; \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_##n \
        RESPONSE_CODES \
        { \
/*Remove new dictionary references from legacy binary format*/ \
                0, SUCCESS,            no_command_specific_errors_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                2, DATA_ENTRY_ERROR,   invalid_selection_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                5, DATA_ENTRY_ERROR,   too_few_data_bytes_recieved_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                6, MISC_ERROR,         xmtr_specific_command_error_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                7, MODE_ERROR,         in_write_protect_mode_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                8, DATA_ENTRY_WARNING, set_to_nearest_possible_value_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                9, DATA_ENTRY_ERROR,   starting_index_must_be_even_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                16, MODE_ERROR,        access_restricted_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                32, MODE_ERROR,        busy_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                33, MODE_ERROR,        delayed_response_initiated_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                34, MODE_ERROR,        delayed_response_running_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                35, MODE_ERROR,        delayed_response_dead_temp; \
/*Remove new dictionary references from legacy binary format*/ \
                36, MODE_ERROR,        delayed_response_conflict_temp; \
        } \
    }
    
#define CONDENSED_STATUS_READ_TRANSACTIONS_10 \
        TRANSACTION 0 \
        { \
            REQUEST \
            { \
                0, \
                48 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_0_ITEMS \
            } \
        } \
        TRANSACTION 1 \
        { \
            REQUEST \
            { \
                48, \
                40 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_1_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_11 \
        CONDENSED_STATUS_READ_TRANSACTIONS_10 \
        TRANSACTION 2 \
        { \
            REQUEST \
            { \
                88, \
                8 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_2_11_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_12 \
        CONDENSED_STATUS_READ_TRANSACTIONS_10 \
        TRANSACTION 2 \
        { \
            REQUEST \
            { \
                88, \
                16 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_2_12_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_13 \
        CONDENSED_STATUS_READ_TRANSACTIONS_10 \
        TRANSACTION 2 \
        { \
            REQUEST \
            { \
                88, \
                24 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_2_13_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_14 \
        CONDENSED_STATUS_READ_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                112, \
                8 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_14_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_15 \
        CONDENSED_STATUS_READ_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                112, \
                16 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_15_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_16 \
        CONDENSED_STATUS_READ_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                112, \
                24 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_16_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_17 \
        CONDENSED_STATUS_READ_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                112, \
                32 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_17_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_18 \
        CONDENSED_STATUS_READ_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                112, \
                40 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_18_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_19 \
        CONDENSED_STATUS_READ_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                152, \
                8 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_19_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_20 \
        CONDENSED_STATUS_READ_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                152, \
                16 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_20_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_21 \
        CONDENSED_STATUS_READ_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                152, \
                24 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_21_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_22 \
        CONDENSED_STATUS_READ_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                152, \
                32 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_22_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_23 \
        CONDENSED_STATUS_READ_TRANSACTIONS_22 \
        TRANSACTION 5 \
        { \
            REQUEST \
            { \
                184, \
                8 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_5_23_ITEMS \
            } \
        }
        
#define CONDENSED_STATUS_READ_TRANSACTIONS_24 \
        CONDENSED_STATUS_READ_TRANSACTIONS_22 \
        TRANSACTION 5 \
        { \
            REQUEST \
            { \
                184, \
                16 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_5_24_ITEMS \
            } \
        }

#define CONDENSED_STATUS_READ_TRANSACTIONS_25 \
        CONDENSED_STATUS_READ_TRANSACTIONS_22 \
        TRANSACTION 5 \
        { \
            REQUEST \
            { \
                184, \
                24 \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_5_25_ITEMS \
            } \
        }



#define CONDENSED_STATUS_WRITE_TRANSACTIONS_10 \
        TRANSACTION 0 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_0_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_0_ITEMS \
            } \
        } \
        TRANSACTION 1 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_1_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_1_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_11 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_10 \
        TRANSACTION 2 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_2_11_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_2_11_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_12 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_10 \
        TRANSACTION 2 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_2_12_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_2_12_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_13 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_10 \
        TRANSACTION 2 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_2_13_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_2_13_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_14 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_3_14_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_14_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_15 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_3_15_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_15_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_16 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_3_16_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_16_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_17 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_3_17_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_17_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_18 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_13 \
        TRANSACTION 3 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_3_18_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_3_18_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_19 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_4_19_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_19_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_20 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_4_20_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_20_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_21 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_4_21_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_21_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_22 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_18 \
        TRANSACTION 4 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_4_22_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_4_22_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_23 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_22 \
        TRANSACTION 5 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_5_23_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_5_23_ITEMS \
            } \
        }
        
#define CONDENSED_STATUS_WRITE_TRANSACTIONS_24 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_22 \
        TRANSACTION 5 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_5_24_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_5_24_ITEMS \
            } \
        }

#define CONDENSED_STATUS_WRITE_TRANSACTIONS_25 \
        CONDENSED_STATUS_WRITE_TRANSACTIONS_22 \
        TRANSACTION 5 \
        { \
            REQUEST \
            { \
                CONDENSED_STATUS_TRANSACTION_5_25_ITEMS \
            } \
            REPLY \
            { \
                response_code, device_status, \
                CONDENSED_STATUS_TRANSACTION_5_25_ITEMS \
            } \
        }


#define CONDENSED_STATUS_TRANSACTION_0_ITEMS \
                    0, \
                    48, \
                    statusmap_variable_001 <0xF0>, statusmap_variable_000 <0x0F>, \
                    statusmap_variable_003 <0xF0>, statusmap_variable_002 <0x0F>, \
                    statusmap_variable_005 <0xF0>, statusmap_variable_004 <0x0F>, \
                    statusmap_variable_007 <0xF0>, statusmap_variable_006 <0x0F>, \
                    \
                    statusmap_variable_009 <0xF0>, statusmap_variable_008 <0x0F>, \
                    statusmap_variable_011 <0xF0>, statusmap_variable_010 <0x0F>, \
                    statusmap_variable_013 <0xF0>, statusmap_variable_012 <0x0F>, \
                    statusmap_variable_015 <0xF0>, statusmap_variable_014 <0x0F>, \
                    \
                    statusmap_variable_017 <0xF0>, statusmap_variable_016 <0x0F>, \
                    statusmap_variable_019 <0xF0>, statusmap_variable_018 <0x0F>, \
                    statusmap_variable_021 <0xF0>, statusmap_variable_020 <0x0F>, \
                    statusmap_variable_023 <0xF0>, statusmap_variable_022 <0x0F>, \
                    \
                    statusmap_variable_025 <0xF0>, statusmap_variable_024 <0x0F>, \
                    statusmap_variable_027 <0xF0>, statusmap_variable_026 <0x0F>, \
                    statusmap_variable_029 <0xF0>, statusmap_variable_028 <0x0F>, \
                    statusmap_variable_031 <0xF0>, statusmap_variable_030 <0x0F>, \
                    \
                    statusmap_variable_033 <0xF0>, statusmap_variable_032 <0x0F>, \
                    statusmap_variable_035 <0xF0>, statusmap_variable_034 <0x0F>, \
                    statusmap_variable_037 <0xF0>, statusmap_variable_036 <0x0F>, \
                    statusmap_variable_039 <0xF0>, statusmap_variable_038 <0x0F>, \
                    \
                    statusmap_variable_041 <0xF0>, statusmap_variable_040 <0x0F>, \
                    statusmap_variable_043 <0xF0>, statusmap_variable_042 <0x0F>, \
                    statusmap_variable_045 <0xF0>, statusmap_variable_044 <0x0F>, \
                    statusmap_variable_047 <0xF0>, statusmap_variable_046 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_1_ITEMS \
                    48, \
                    40, \
                    statusmap_variable_049 <0xF0>, statusmap_variable_048 <0x0F>, \
                    statusmap_variable_051 <0xF0>, statusmap_variable_050 <0x0F>, \
                    statusmap_variable_053 <0xF0>, statusmap_variable_052 <0x0F>, \
                    statusmap_variable_055 <0xF0>, statusmap_variable_054 <0x0F>, \
                    \
                    statusmap_variable_057 <0xF0>, statusmap_variable_056 <0x0F>, \
                    statusmap_variable_059 <0xF0>, statusmap_variable_058 <0x0F>, \
                    statusmap_variable_061 <0xF0>, statusmap_variable_060 <0x0F>, \
                    statusmap_variable_063 <0xF0>, statusmap_variable_062 <0x0F>, \
                    \
                    statusmap_variable_065 <0xF0>, statusmap_variable_064 <0x0F>, \
                    statusmap_variable_067 <0xF0>, statusmap_variable_066 <0x0F>, \
                    statusmap_variable_069 <0xF0>, statusmap_variable_068 <0x0F>, \
                    statusmap_variable_071 <0xF0>, statusmap_variable_070 <0x0F>, \
                    \
                    statusmap_variable_073 <0xF0>, statusmap_variable_072 <0x0F>, \
                    statusmap_variable_075 <0xF0>, statusmap_variable_074 <0x0F>, \
                    statusmap_variable_077 <0xF0>, statusmap_variable_076 <0x0F>, \
                    statusmap_variable_079 <0xF0>, statusmap_variable_078 <0x0F>, \
                    \
                    statusmap_variable_081 <0xF0>, statusmap_variable_080 <0x0F>, \
                    statusmap_variable_083 <0xF0>, statusmap_variable_082 <0x0F>, \
                    statusmap_variable_085 <0xF0>, statusmap_variable_084 <0x0F>, \
                    statusmap_variable_087 <0xF0>, statusmap_variable_086 <0x0F>


#define CONDENSED_STATUS_TRANSACTION_2_11_ITEMS \
                    88, \
                    8, \
                    statusmap_variable_089 <0xF0>, statusmap_variable_088 <0x0F>, \
                    statusmap_variable_091 <0xF0>, statusmap_variable_090 <0x0F>, \
                    statusmap_variable_093 <0xF0>, statusmap_variable_092 <0x0F>, \
                    statusmap_variable_095 <0xF0>, statusmap_variable_094 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_2_12_ITEMS \
                    88, \
                    16, \
                    statusmap_variable_089 <0xF0>, statusmap_variable_088 <0x0F>, \
                    statusmap_variable_091 <0xF0>, statusmap_variable_090 <0x0F>, \
                    statusmap_variable_093 <0xF0>, statusmap_variable_092 <0x0F>, \
                    statusmap_variable_095 <0xF0>, statusmap_variable_094 <0x0F>, \
                    \
                    statusmap_variable_097 <0xF0>, statusmap_variable_096 <0x0F>, \
                    statusmap_variable_099 <0xF0>, statusmap_variable_098 <0x0F>, \
                    statusmap_variable_101 <0xF0>, statusmap_variable_100 <0x0F>, \
                    statusmap_variable_103 <0xF0>, statusmap_variable_102 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_2_13_ITEMS \
                    88, \
                    24, \
                    statusmap_variable_089 <0xF0>, statusmap_variable_088 <0x0F>, \
                    statusmap_variable_091 <0xF0>, statusmap_variable_090 <0x0F>, \
                    statusmap_variable_093 <0xF0>, statusmap_variable_092 <0x0F>, \
                    statusmap_variable_095 <0xF0>, statusmap_variable_094 <0x0F>, \
                    \
                    statusmap_variable_097 <0xF0>, statusmap_variable_096 <0x0F>, \
                    statusmap_variable_099 <0xF0>, statusmap_variable_098 <0x0F>, \
                    statusmap_variable_101 <0xF0>, statusmap_variable_100 <0x0F>, \
                    statusmap_variable_103 <0xF0>, statusmap_variable_102 <0x0F>, \
                    \
                    statusmap_variable_105 <0xF0>, statusmap_variable_104 <0x0F>, \
                    statusmap_variable_107 <0xF0>, statusmap_variable_106 <0x0F>, \
                    statusmap_variable_109 <0xF0>, statusmap_variable_108 <0x0F>, \
                    statusmap_variable_111 <0xF0>, statusmap_variable_110 <0x0F>


#define CONDENSED_STATUS_TRANSACTION_3_14_ITEMS \
                    112, \
                    8, \
                    statusmap_variable_113 <0xF0>, statusmap_variable_112 <0x0F>, \
                    statusmap_variable_115 <0xF0>, statusmap_variable_114 <0x0F>, \
                    statusmap_variable_117 <0xF0>, statusmap_variable_116 <0x0F>, \
                    statusmap_variable_119 <0xF0>, statusmap_variable_118 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_3_15_ITEMS \
                    112, \
                    16, \
                    statusmap_variable_113 <0xF0>, statusmap_variable_112 <0x0F>, \
                    statusmap_variable_115 <0xF0>, statusmap_variable_114 <0x0F>, \
                    statusmap_variable_117 <0xF0>, statusmap_variable_116 <0x0F>, \
                    statusmap_variable_119 <0xF0>, statusmap_variable_118 <0x0F>, \
                    \
                    statusmap_variable_121 <0xF0>, statusmap_variable_120 <0x0F>, \
                    statusmap_variable_123 <0xF0>, statusmap_variable_122 <0x0F>, \
                    statusmap_variable_125 <0xF0>, statusmap_variable_124 <0x0F>, \
                    statusmap_variable_127 <0xF0>, statusmap_variable_126 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_3_16_ITEMS \
                    112, \
                    24, \
                    statusmap_variable_113 <0xF0>, statusmap_variable_112 <0x0F>, \
                    statusmap_variable_115 <0xF0>, statusmap_variable_114 <0x0F>, \
                    statusmap_variable_117 <0xF0>, statusmap_variable_116 <0x0F>, \
                    statusmap_variable_119 <0xF0>, statusmap_variable_118 <0x0F>, \
                    \
                    statusmap_variable_121 <0xF0>, statusmap_variable_120 <0x0F>, \
                    statusmap_variable_123 <0xF0>, statusmap_variable_122 <0x0F>, \
                    statusmap_variable_125 <0xF0>, statusmap_variable_124 <0x0F>, \
                    statusmap_variable_127 <0xF0>, statusmap_variable_126 <0x0F>, \
                    \
                    statusmap_variable_129 <0xF0>, statusmap_variable_128 <0x0F>, \
                    statusmap_variable_131 <0xF0>, statusmap_variable_130 <0x0F>, \
                    statusmap_variable_133 <0xF0>, statusmap_variable_132 <0x0F>, \
                    statusmap_variable_135 <0xF0>, statusmap_variable_134 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_3_17_ITEMS \
                    112, \
                    32, \
                    statusmap_variable_113 <0xF0>, statusmap_variable_112 <0x0F>, \
                    statusmap_variable_115 <0xF0>, statusmap_variable_114 <0x0F>, \
                    statusmap_variable_117 <0xF0>, statusmap_variable_116 <0x0F>, \
                    statusmap_variable_119 <0xF0>, statusmap_variable_118 <0x0F>, \
                    \
                    statusmap_variable_121 <0xF0>, statusmap_variable_120 <0x0F>, \
                    statusmap_variable_123 <0xF0>, statusmap_variable_122 <0x0F>, \
                    statusmap_variable_125 <0xF0>, statusmap_variable_124 <0x0F>, \
                    statusmap_variable_127 <0xF0>, statusmap_variable_126 <0x0F>, \
                    \
                    statusmap_variable_129 <0xF0>, statusmap_variable_128 <0x0F>, \
                    statusmap_variable_131 <0xF0>, statusmap_variable_130 <0x0F>, \
                    statusmap_variable_133 <0xF0>, statusmap_variable_132 <0x0F>, \
                    statusmap_variable_135 <0xF0>, statusmap_variable_134 <0x0F>, \
                    \
                    statusmap_variable_137 <0xF0>, statusmap_variable_136 <0x0F>, \
                    statusmap_variable_139 <0xF0>, statusmap_variable_138 <0x0F>, \
                    statusmap_variable_141 <0xF0>, statusmap_variable_140 <0x0F>, \
                    statusmap_variable_143 <0xF0>, statusmap_variable_142 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_3_18_ITEMS \
                    112, \
                    40, \
                    statusmap_variable_113 <0xF0>, statusmap_variable_112 <0x0F>, \
                    statusmap_variable_115 <0xF0>, statusmap_variable_114 <0x0F>, \
                    statusmap_variable_117 <0xF0>, statusmap_variable_116 <0x0F>, \
                    statusmap_variable_119 <0xF0>, statusmap_variable_118 <0x0F>, \
                    \
                    statusmap_variable_121 <0xF0>, statusmap_variable_120 <0x0F>, \
                    statusmap_variable_123 <0xF0>, statusmap_variable_122 <0x0F>, \
                    statusmap_variable_125 <0xF0>, statusmap_variable_124 <0x0F>, \
                    statusmap_variable_127 <0xF0>, statusmap_variable_126 <0x0F>, \
                    \
                    statusmap_variable_129 <0xF0>, statusmap_variable_128 <0x0F>, \
                    statusmap_variable_131 <0xF0>, statusmap_variable_130 <0x0F>, \
                    statusmap_variable_133 <0xF0>, statusmap_variable_132 <0x0F>, \
                    statusmap_variable_135 <0xF0>, statusmap_variable_134 <0x0F>, \
                    \
                    statusmap_variable_137 <0xF0>, statusmap_variable_136 <0x0F>, \
                    statusmap_variable_139 <0xF0>, statusmap_variable_138 <0x0F>, \
                    statusmap_variable_141 <0xF0>, statusmap_variable_140 <0x0F>, \
                    statusmap_variable_143 <0xF0>, statusmap_variable_142 <0x0F>, \
                    \
                    statusmap_variable_145 <0xF0>, statusmap_variable_144 <0x0F>, \
                    statusmap_variable_147 <0xF0>, statusmap_variable_146 <0x0F>, \
                    statusmap_variable_149 <0xF0>, statusmap_variable_148 <0x0F>, \
                    statusmap_variable_151 <0xF0>, statusmap_variable_150 <0x0F>


#define CONDENSED_STATUS_TRANSACTION_4_19_ITEMS \
                    152, \
                    8, \
                    statusmap_variable_153 <0xF0>, statusmap_variable_152 <0x0F>, \
                    statusmap_variable_155 <0xF0>, statusmap_variable_154 <0x0F>, \
                    statusmap_variable_157 <0xF0>, statusmap_variable_156 <0x0F>, \
                    statusmap_variable_159 <0xF0>, statusmap_variable_158 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_4_20_ITEMS \
                    152, \
                    16, \
                    statusmap_variable_153 <0xF0>, statusmap_variable_152 <0x0F>, \
                    statusmap_variable_155 <0xF0>, statusmap_variable_154 <0x0F>, \
                    statusmap_variable_157 <0xF0>, statusmap_variable_156 <0x0F>, \
                    statusmap_variable_159 <0xF0>, statusmap_variable_158 <0x0F>, \
                    \
                    statusmap_variable_161 <0xF0>, statusmap_variable_160 <0x0F>, \
                    statusmap_variable_163 <0xF0>, statusmap_variable_162 <0x0F>, \
                    statusmap_variable_165 <0xF0>, statusmap_variable_164 <0x0F>, \
                    statusmap_variable_167 <0xF0>, statusmap_variable_166 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_4_21_ITEMS \
                    152, \
                    24, \
                    statusmap_variable_153 <0xF0>, statusmap_variable_152 <0x0F>, \
                    statusmap_variable_155 <0xF0>, statusmap_variable_154 <0x0F>, \
                    statusmap_variable_157 <0xF0>, statusmap_variable_156 <0x0F>, \
                    statusmap_variable_159 <0xF0>, statusmap_variable_158 <0x0F>, \
                    \
                    statusmap_variable_161 <0xF0>, statusmap_variable_160 <0x0F>, \
                    statusmap_variable_163 <0xF0>, statusmap_variable_162 <0x0F>, \
                    statusmap_variable_165 <0xF0>, statusmap_variable_164 <0x0F>, \
                    statusmap_variable_167 <0xF0>, statusmap_variable_166 <0x0F>, \
                    \
                    statusmap_variable_169 <0xF0>, statusmap_variable_168 <0x0F>, \
                    statusmap_variable_171 <0xF0>, statusmap_variable_170 <0x0F>, \
                    statusmap_variable_173 <0xF0>, statusmap_variable_172 <0x0F>, \
                    statusmap_variable_175 <0xF0>, statusmap_variable_174 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_4_22_ITEMS \
                    152, \
                    32, \
                    statusmap_variable_153 <0xF0>, statusmap_variable_152 <0x0F>, \
                    statusmap_variable_155 <0xF0>, statusmap_variable_154 <0x0F>, \
                    statusmap_variable_157 <0xF0>, statusmap_variable_156 <0x0F>, \
                    statusmap_variable_159 <0xF0>, statusmap_variable_158 <0x0F>, \
                    \
                    statusmap_variable_161 <0xF0>, statusmap_variable_160 <0x0F>, \
                    statusmap_variable_163 <0xF0>, statusmap_variable_162 <0x0F>, \
                    statusmap_variable_165 <0xF0>, statusmap_variable_164 <0x0F>, \
                    statusmap_variable_167 <0xF0>, statusmap_variable_166 <0x0F>, \
                    \
                    statusmap_variable_169 <0xF0>, statusmap_variable_168 <0x0F>, \
                    statusmap_variable_171 <0xF0>, statusmap_variable_170 <0x0F>, \
                    statusmap_variable_173 <0xF0>, statusmap_variable_172 <0x0F>, \
                    statusmap_variable_175 <0xF0>, statusmap_variable_174 <0x0F>, \
                    \
                    statusmap_variable_177 <0xF0>, statusmap_variable_176 <0x0F>, \
                    statusmap_variable_179 <0xF0>, statusmap_variable_178 <0x0F>, \
                    statusmap_variable_181 <0xF0>, statusmap_variable_180 <0x0F>, \
                    statusmap_variable_183 <0xF0>, statusmap_variable_182 <0x0F>


#define CONDENSED_STATUS_TRANSACTION_5_23_ITEMS \
                    184, \
                    8, \
                    statusmap_variable_185 <0xF0>, statusmap_variable_184 <0x0F>, \
                    statusmap_variable_187 <0xF0>, statusmap_variable_186 <0x0F>, \
                    statusmap_variable_189 <0xF0>, statusmap_variable_188 <0x0F>, \
                    statusmap_variable_191 <0xF0>, statusmap_variable_190 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_5_24_ITEMS \
                    184, \
                    16, \
                    statusmap_variable_185 <0xF0>, statusmap_variable_184 <0x0F>, \
                    statusmap_variable_187 <0xF0>, statusmap_variable_186 <0x0F>, \
                    statusmap_variable_189 <0xF0>, statusmap_variable_188 <0x0F>, \
                    statusmap_variable_191 <0xF0>, statusmap_variable_190 <0x0F>, \
                    \
                    statusmap_variable_193 <0xF0>, statusmap_variable_192 <0x0F>, \
                    statusmap_variable_195 <0xF0>, statusmap_variable_194 <0x0F>, \
                    statusmap_variable_197 <0xF0>, statusmap_variable_196 <0x0F>, \
                    statusmap_variable_199 <0xF0>, statusmap_variable_198 <0x0F>

#define CONDENSED_STATUS_TRANSACTION_5_25_ITEMS \
                    184, \
                    24, \
                    statusmap_variable_185 <0xF0>, statusmap_variable_184 <0x0F>, \
                    statusmap_variable_187 <0xF0>, statusmap_variable_186 <0x0F>, \
                    statusmap_variable_189 <0xF0>, statusmap_variable_188 <0x0F>, \
                    statusmap_variable_191 <0xF0>, statusmap_variable_190 <0x0F>, \
                    \
                    statusmap_variable_193 <0xF0>, statusmap_variable_192 <0x0F>, \
                    statusmap_variable_195 <0xF0>, statusmap_variable_194 <0x0F>, \
                    statusmap_variable_197 <0xF0>, statusmap_variable_196 <0x0F>, \
                    statusmap_variable_199 <0xF0>, statusmap_variable_198 <0x0F>, \
                    \
                    statusmap_variable_201 <0xF0>, statusmap_variable_200 <0x0F>, \
                    statusmap_variable_203 <0xF0>, statusmap_variable_202 <0x0F>, \
                    statusmap_variable_205 <0xF0>, statusmap_variable_204 <0x0F>, \
                    statusmap_variable_207 <0xF0>, statusmap_variable_206 <0x0F>


#define REDEFINE_SIMULATION_ITEM_SIZE(n)                        \
        ELEMENTS \
        { \
            REMOVE_STATUS_ARRAY_ELEMENT_##n                \
        }

#define REMOVE_STATUS_ARRAY_ELEMENT_24                       \
    DELETE 25;

#define REMOVE_STATUS_ARRAY_ELEMENT_23                       \
    REMOVE_STATUS_ARRAY_ELEMENT_24                           \
    DELETE 24;

#define REMOVE_STATUS_ARRAY_ELEMENT_22                       \
    REMOVE_STATUS_ARRAY_ELEMENT_23                           \
    DELETE 23;

#define REMOVE_STATUS_ARRAY_ELEMENT_21                       \
    REMOVE_STATUS_ARRAY_ELEMENT_22                           \
    DELETE 22;

#define REMOVE_STATUS_ARRAY_ELEMENT_20                       \
    REMOVE_STATUS_ARRAY_ELEMENT_21                           \
    DELETE 21;

#define REMOVE_STATUS_ARRAY_ELEMENT_19                       \
    REMOVE_STATUS_ARRAY_ELEMENT_20                           \
    DELETE 20;

#define REMOVE_STATUS_ARRAY_ELEMENT_18                       \
    REMOVE_STATUS_ARRAY_ELEMENT_19                           \
    DELETE 19;

#define REMOVE_STATUS_ARRAY_ELEMENT_17                       \
    REMOVE_STATUS_ARRAY_ELEMENT_18                           \
    DELETE 18;

#define REMOVE_STATUS_ARRAY_ELEMENT_16                       \
    REMOVE_STATUS_ARRAY_ELEMENT_17                           \
    DELETE 17;

#define REMOVE_STATUS_ARRAY_ELEMENT_15                       \
    REMOVE_STATUS_ARRAY_ELEMENT_16                           \
    DELETE 16;

#define REMOVE_STATUS_ARRAY_ELEMENT_14                       \
    REMOVE_STATUS_ARRAY_ELEMENT_15                           \
    DELETE 15;

#define REMOVE_STATUS_ARRAY_ELEMENT_13                       \
    REMOVE_STATUS_ARRAY_ELEMENT_14                           \
    DELETE 14;

#define REMOVE_STATUS_ARRAY_ELEMENT_12                       \
    REMOVE_STATUS_ARRAY_ELEMENT_13                           \
    DELETE 13;

#define REMOVE_STATUS_ARRAY_ELEMENT_11                       \
    REMOVE_STATUS_ARRAY_ELEMENT_12                           \
    DELETE 12;

#define REMOVE_STATUS_ARRAY_ELEMENT_10                       \
    REMOVE_STATUS_ARRAY_ELEMENT_11                           \
    DELETE 11;


/* 
 * Define all the Engineering units
 */
#define ALL_UNITS  \
        UNITS_CODE(1), \
        UNITS_CODE(2), \
        UNITS_CODE(3), \
        UNITS_CODE(4), \
        UNITS_CODE(5), \
        UNITS_CODE(6), \
        UNITS_CODE(7), \
        UNITS_CODE(8), \
        UNITS_CODE(9), \
        UNITS_CODE(10), \
        UNITS_CODE(11), \
        UNITS_CODE(12), \
        UNITS_CODE(13), \
        UNITS_CODE(14), \
        UNITS_CODE(15), \
        UNITS_CODE(16), \
        UNITS_CODE(17), \
        UNITS_CODE(18), \
        UNITS_CODE(19), \
        UNITS_CODE(20), \
        UNITS_CODE(21), \
        UNITS_CODE(22), \
        UNITS_CODE(23), \
        UNITS_CODE(24), \
        UNITS_CODE(25), \
        UNITS_CODE(26), \
        UNITS_CODE(27), \
        UNITS_CODE(28), \
        UNITS_CODE(29), \
        UNITS_CODE(30), \
        UNITS_CODE(31), \
        UNITS_CODE(32), \
        UNITS_CODE(33), \
        UNITS_CODE(34), \
        UNITS_CODE(35), \
        UNITS_CODE(36), \
        UNITS_CODE(37), \
        UNITS_CODE(38), \
        UNITS_CODE(39), \
        UNITS_CODE(40), \
        UNITS_CODE(41), \
        UNITS_CODE(42), \
        UNITS_CODE(43), \
        UNITS_CODE(44), \
        UNITS_CODE(45), \
        UNITS_CODE(46), \
        UNITS_CODE(47), \
        UNITS_CODE(48), \
        UNITS_CODE(49), \
        UNITS_CODE(50), \
        UNITS_CODE(51), \
        UNITS_CODE(52), \
        UNITS_CODE(53), \
        UNITS_CODE(54), \
        UNITS_CODE(55), \
        UNITS_CODE(56), \
        UNITS_CODE(57), \
        UNITS_CODE(58), \
        UNITS_CODE(59), \
        UNITS_CODE(60), \
        UNITS_CODE(61), \
        UNITS_CODE(62), \
        UNITS_CODE(63), \
        UNITS_CODE(64), \
        UNITS_CODE(65), \
        UNITS_CODE(66), \
        UNITS_CODE(67), \
        UNITS_CODE(68), \
        UNITS_CODE(69), \
        UNITS_CODE(70), \
        UNITS_CODE(71), \
        UNITS_CODE(72), \
        UNITS_CODE(73), \
        UNITS_CODE(74), \
        UNITS_CODE(75), \
        UNITS_CODE(76), \
        UNITS_CODE(77), \
        UNITS_CODE(78), \
        UNITS_CODE(79), \
        UNITS_CODE(80), \
        UNITS_CODE(81), \
        UNITS_CODE(82), \
        UNITS_CODE(83), \
        UNITS_CODE(84), \
        UNITS_CODE(85), \
        UNITS_CODE(86), \
        UNITS_CODE(87), \
        UNITS_CODE(88), \
        UNITS_CODE(89), \
        UNITS_CODE(90), \
        UNITS_CODE(91), \
        UNITS_CODE(92), \
        UNITS_CODE(93), \
        UNITS_CODE(94), \
        UNITS_CODE(95), \
        UNITS_CODE(96), \
        UNITS_CODE(97), \
        UNITS_CODE(98), \
        UNITS_CODE(99), \
        UNITS_CODE(100), \
        UNITS_CODE(101), \
        UNITS_CODE(102), \
        UNITS_CODE(103), \
        UNITS_CODE(104), \
        UNITS_CODE(105), \
        UNITS_CODE(106), \
        UNITS_CODE(107), \
        UNITS_CODE(108), \
        UNITS_CODE(109), \
        UNITS_CODE(110), \
        UNITS_CODE(111), \
        UNITS_CODE(112), \
        UNITS_CODE(113), \
        UNITS_CODE(114), \
        UNITS_CODE(115), \
        UNITS_CODE(116), \
        UNITS_CODE(117), \
        UNITS_CODE(118), \
        UNITS_CODE(119), \
        UNITS_CODE(120), \
        UNITS_CODE(121), \
        UNITS_CODE(122), \
        UNITS_CODE(123), \
        UNITS_CODE(124), \
        UNITS_CODE(130), \
        UNITS_CODE(131), \
        UNITS_CODE(132), \
        UNITS_CODE(133), \
        UNITS_CODE(134), \
        UNITS_CODE(135), \
        UNITS_CODE(136), \
        UNITS_CODE(137), \
        UNITS_CODE(138), \
        UNITS_CODE(139), \
        UNITS_CODE(140), \
        UNITS_CODE(141), \
        UNITS_CODE(142), \
        UNITS_CODE(143), \
        UNITS_CODE(144), \
        UNITS_CODE(145), \
        UNITS_CODE(146), \
        UNITS_CODE(147), \
        UNITS_CODE(148), \
        UNITS_CODE(149), \
        UNITS_CODE(150), \
        UNITS_CODE(151), \
        UNITS_CODE(152), \
        UNITS_CODE(153), \
        UNITS_CODE(154), \
        UNITS_CODE(155), \
        UNITS_CODE(160), \
        UNITS_CODE(161), \
        UNITS_CODE(162), \
        UNITS_CODE(163), \
        UNITS_CODE(164), \
        UNITS_CODE(165), \
        UNITS_CODE(166), \
        UNITS_CODE(167), \
        UNITS_CODE(168), \
        UNITS_CODE(169), \
        UNITS_CODE(235), \
        UNITS_CODE(236), \
        UNITS_CODE(237), \
        UNITS_CODE(238), \
        UNITS_CODE(239), \
        UNITS_CODE(250), \
        UNITS_CODE(251), \
        UNITS_CODE(252), \
        UNITS_CODE(253)

/* 
 * Define device variable classifications 
 */
#define ALL_CLASSIFICATIONS \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(0),  \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(64), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(65), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(66), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(67), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(68), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(69), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(70), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(71), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(72), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(73), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(74), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(75), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(76), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(77), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(78), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(79), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(80), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(81), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(82), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(83), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(84), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(85), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(86), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(87), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(88), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(89), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(90), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(91)

#define ALL_FAMILIES \
        DEVICE_VARIABLE_FAMILY_CODE(4), \
        DEVICE_VARIABLE_FAMILY_CODE(5), \
        DEVICE_VARIABLE_FAMILY_CODE(6), \
        DEVICE_VARIABLE_FAMILY_CODE(7), \
        DEVICE_VARIABLE_FAMILY_CODE(8), \
        DEVICE_VARIABLE_FAMILY_CODE(9), \
        DEVICE_VARIABLE_FAMILY_CODE(10), \
        DEVICE_VARIABLE_FAMILY_CODE(250)


/*
 * Defined Colors.
 */
#define BLACK   0x000000                        
#define SILVER  0xC0C0C0                        
#define GRAY    0x808080                        
#define WHITE   0xFFFFFF                        
#define MAROON  0x800000                        
#define RED     0xFF0000                        
#define ORANGE  0xFFA500                        
#define PURPLE  0x800080                        
#define FUCHSIA 0xFF00FF                                
#define GREEN   0x008000
#define LIME    0x00FF00
#define OLIVE   0x808000
#define YELLOW  0xFFFF00
#define NAVY    0x000080
#define BLUE    0x0000FF
#define TEAL    0x008080
#define AQUA    0x00FFFF


#ifdef __GNUC__
#define LOCALVAR( x) #x     /*this is used by the get_local_var commands*/
#else
#define LOCALVAR( x) "x"
#endif

#define STRCMP_EQUAL 0 
