/*************************************************************************************
 *
 * File: Tabl25_2.ddl
 * Version: 25.2.0
 * Date: 01/14/2022
 *
 *************************************************************************************
 * Copyright (c) 2022, FieldComm Group, All Rights Reserved
 *************************************************************************************
 *
 * Description:  Specification DD that models the Common Tables Specification
 *
 * Source Specifications (These are revision numbers of the protocol specs which
 *                        may differ from the standard DD revision numbers.)
 *      Command Practice Command Specification      Rev.12.0
 *      Command Summary Specification               Rev.10.1
 *      Universal Command Specification             Rev.7.2
 *      Common Tables Specification                 Rev.25.0
 *
 *************************************************************************************
 */

/*************************************************************************************
 * The manufacturer line
 *************************************************************************************
 */
MANUFACTURER STANDARD, DEVICE_TYPE _TABLES, DEVICE_REVISION 25, DD_REVISION 2

/*************************************************************************************
 * Include Files
 *************************************************************************************
 */
#include "macros2.h"
#include "embedded_strings.h"

/*************************************************************************************
 * Standard Device Type Codes
 */
VARIABLE Standard_model_code    /* 0x0000 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, "Common Tables"                    ,   "Interoperable *" },
        { 0x0002, "Universal"                        ,   "Interoperable *" },
        { 0x0003, "Common Practice"                  ,   "Interoperable *" },
        { 0x0007, "Pressure Universal"               ,   "Interoperable *" },
        { 0x0008, "Pressure Common"                  ,   "Interoperable *" },
        { 0x0010, "Menu"                             ,   "Interoperable *" },
        { 0x0011, "Rosemount Magflow"                ,   "Interoperable *" },
        { 0x0012, "Rosemount Pressure"               ,   "Interoperable *" },
        { 0x0013, "Moore Products"                   ,   "Interoperable *" },
        { 0x0014, "Siemens Pressure"                 ,   "Interoperable *" },
        { 0x0015, "P2 Common"                        ,   "Interoperable *" },
        { 0x0016, "PV"                               ,   "Interoperable *" },
        { 0x0018, "Wireless"                         ,   "Interoperable *" },
        { 0x0019, "Common Device Variables"          ,   "Interoperable *" },
        { 0x001A, "Common Analog Channels"           ,   "Interoperable *" },
        { 0x001B, "Common Burst Value Array"         ,   "Interoperable *" },
        { 0x001C, "Common Burst Reference Array"     ,   "Interoperable *" },
        { 0x001D, "Common Condensed Status"          ,   "Interoperable *" },
        { 0x001E, "Common IO System List"            ,   "Interoperable *" },
        { 0x001F, "Common IO System Reference Array" ,   "Interoperable *" },
        { 0x0020, "Common Miscellaneous"             ,   "Interoperable *" },
        { 0x0063, "Munge"                            ,   "Interoperable *" }
    }
}

/*************************************************************************************
 * Ametek Device Type Codes
 */
VARIABLE Ametek_model_code    /* 0x0003 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0004, [newthermox] ,   "Analytical    Oxygen/Combustion/Methane/CO" },
        { 0x0304, [newthermox] ,   "Analytical    Oxygen/Combustion/Methane/CO" }
    }
}

/*************************************************************************************
 * Brooks Instrument Device Type Codes
 */
VARIABLE Brooks_Instrument_model_code    /* 0x000A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [tri20]          ,   "Flow          Totalizer - Rate Indicator" },
        { 0x0002, [brooks38xxva]   ,   "Flow          Variable Area" },
        { 0x0003, [brooks99xxoval] },
        { 0x0004, [quantim]        ,   "Flow          Coriolis Mass" },
        { 0x0A01, [tri20]          ,   "Flow          Totalizer - Rate Indicator" },
        { 0x0A02, [brooks38xxva]   ,   "Flow          Variable Area" },
        { 0x0A03, [brooks99xxoval] },
        { 0x0A04, [quantim]        ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * Daniel Industries Device Type Codes
 */
VARIABLE Daniel_Industries_model_code    /* 0x000D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0014, [gas_usm]                ,   "Flow          Ultrasonic - Gas" },
        { 0x0015, [series_3410_gas_usm]    ,   "Flow          Ultrasonic - Gas" },
        { 0x0028, [liquid_usm]             ,   "Flow          Ultrasonic - Liquid" },
        { 0x0029, [series_3810_liquid_usm] ,   "Flow          Ultrasonic - Liquid" },
        { 0x0D14, [gas_usm]                ,   "Flow          Ultrasonic - Gas" },
        { 0x0D15, [series_3410_gas_usm]    ,   "Flow          Ultrasonic - Gas" },
        { 0x0D28, [liquid_usm]             ,   "Flow          Ultrasonic - Liquid" },
        { 0x0D29, [series_3810_liquid_usm] ,   "Flow          Ultrasonic - Liquid" }
    }
}

/*************************************************************************************
 * Delta Device Type Codes
 */
VARIABLE Delta_model_code    /* 0x000E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [ht] },
        { 0x0E01, [ht] }
    }
}

/*************************************************************************************
 * Endress & Hauser Device Type Codes
 */
VARIABLE Endress_Hauser_model_code    /* 0x0011 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0003, [fmu860]                  ,   "Level         Ultrasonic" },
        { 0x0004, [fmu861]                  ,   "Level         Ultrasonic" },
        { 0x0005, [fmu862]                  ,   "Level         Ultrasonic" },
        { 0x0006, [fmr130]                  ,   "Level         Radar - Microwave" },
        { 0x0007, [cerabar_s]               ,   "Pressure      Absolute" },
        { 0x0008, [fec12]                   ,   "Level         Capacitance" },
        { 0x0009, [deltabar_s]              ,   "Pressure      Differential" },
        { 0x000A, [fmu231]                  ,   "Level         Ultrasonic" },
        { 0x000B, [deltapilot]              ,   "Level         Hydrostatic" },
        { 0x000C, [fmr23x]                  ,   "Level         Ultrasonic" },
        { 0x000D, [fmp200]                  ,   "Level" },
        { 0x000E, [cerabar_m]               ,   "Pressure      Absolute/Relative" },
        { 0x000F, [fmr2xx]                  ,   "Level         Microwave" },
        { 0x0010, [fmr53x]                  ,   "Level         Microwave" },
        { 0x0011, [fmu4x]                   ,   "Level         Ultrasonic" },
        { 0x0012, [fmp4x]                   ,   "Level         Radar" },
        { 0x0013, [fmg60]                   ,   "Level         Density - Nuclear" },
        { 0x0014, [nrf590]                  },
        { 0x0017, [deltabars]               ,   "Pressure      Differential" },
        { 0x0018, [cerabars]                ,   "Pressure      Absolute/Relative" },
        { 0x0019, [cerabar_m_5x]            ,   "Pressure      Absolute/Relative" },
        { 0x001A, [deltapilots]             ,   "Level         Hydrostatic" },
        { 0x001B, [fmu9x]                   ,   "Level         Ultrasonic" },
        { 0x001D, [fmi5x]                   ,   "Level" },
        { 0x001E, [fmr25x]                  ,   "Level         Microwave" },
        { 0x001F, [fmr54x]                  ,   "Level         Microwave" },
        { 0x0020, [fmp4x_i]                 ,   "Level" },
        { 0x0021, [deltabar_m_5x]           ,   "Pressure      Differential" },
        { 0x0022, [levelflex_fmp5x]         ,   "Level         Radar" },
        { 0x0023, [deltapilot_m_5x]         ,   "Level         Hydrostatic" },
        { 0x0024, [waterpilot_2x]           ,   "Level         Hydrostatic" },
        { 0x0027, [deltabar_fmd71x]         ,   "Pressure      Differential" },
        { 0x0028, [micropilot_5x]           ,   "Level         Radar" },
        { 0x002A, "Cerabar 5xB/7xB"         ,   "Pressure" },
        { 0x002B, [micropilot_6x]           ,   "Level" },
        { 0x002D, "Proservo NMS8x"          ,   "Level" },
        { 0x002E, "Micropilot NMR8x"        ,   "Level" },
        { 0x002F, "Tank Side Monitor NRF8x" ,   "Level" },
        { 0x0030, [gammapilot_5x]           ,   "Level         Radiometric" },
        { 0x0031, "Deltabar 5xB/7xB"        ,   "Pressure" },
        { 0x0032, [promag33]                ,   "Flow          Electromagnetic" },
        { 0x0033, [prowirl70]               ,   "Flow          Vortex" },
        { 0x0034, [promass63]               ,   "Flow          Coriolis Mass" },
        { 0x0035, [promag39]                ,   "Flow          Magmeter - Rack Version" },
        { 0x0036, [promag35s]               ,   "Flow          High Performance Magmeter" },
        { 0x0037, [prowirl77]               ,   "Flow          Vortex" },
        { 0x0038, [prowirl_200]             ,   "Flow          2-wire Vortex" },
        { 0x0039, [promass60]               ,   "Flow          Mass" },
        { 0x003A, [promag_100]              ,   "Flow          Electromagnetic" },
        { 0x003B, [promass_300_500]         ,   "Flow          Coriolis Mass" },
        { 0x003C, [promag_300_500]          ,   "Flow" },
        { 0x0040, [proson_f]                ,   "Flow          Ultrasonic" },
        { 0x0041, [promag50]                ,   "Flow          Magnetic" },
        { 0x0042, [promag53]                ,   "Flow          Magnetic" },
        { 0x0043, [promag51]                ,   "Flow          Magnetic - Custody Transfer" },
        { 0x0044, [promag55]                ,   "Flow          Magnetic" },
        { 0x0045, [promag10]                ,   "Flow          Magnetic" },
        { 0x0046, [promag23]                ,   "Flow          2-wire Magnetic" },
        { 0x0048, [promag_200]              ,   "Flow          Electromagnetic" },
        { 0x0049, [cubemass_dci]            ,   "Flow          Coriolis Mass" },
        { 0x004A, [promass_100]             ,   "Flow          Coriolis Mass" },
        { 0x0050, [promass80]               ,   "Flow          Coriolis Mass" },
        { 0x0051, [promass83]               ,   "Flow          Coriolis Mass" },
        { 0x0052, [promass84]               ,   "Flow          Coriolis Mass - Custody Transfer" },
        { 0x0053, [promass40]               ,   "Flow          Coriolis Mass" },
        { 0x0054, [promass_200]             ,   "Flow          2-wire Coriolis" },
        { 0x0055, [cngmass_dci]             ,   "Flow          Coriolis Mass" },
        { 0x0056, [prowirl72]               ,   "Flow          2-wire Vortex" },
        { 0x0057, [prowirl73]               ,   "Flow          2-wire Vortex" },
        { 0x0058, [pflow90]                 ,   "Flow          Ultrasonic" },
        { 0x0059, [pflow93]                 ,   "Flow          Ultrasonic" },
        { 0x005A, [prosonic_flow_b_200]     ,   "Flow          Ultrasonic" },
        { 0x005C, [prosonic_flow_e100]      ,   "Flow          Ultrasonic" },
        { 0x005D, [prosonic_flow_300_500]   ,   "Flow          Ultrasonic" },
        { 0x0060, "t-mass 300 500"          ,   "Flow          Thermal" },
        { 0x0061, [pflow92]                 ,   "Flow          Ultrasonic" },
        { 0x0062, [pflow91]                 ,   "Flow          Ultrasonic" },
        { 0x0064, [at70]                    ,   "Flow          Thermal" },
        { 0x0065, [t_mass65]                ,   "Flow          Thermal" },
        { 0x0066, [t_mass_ab_150]           ,   "Flow          Thermal" },
        { 0x0067, [promag_400]              ,   "Flow          Electromagnetic" },
        { 0x0068, [t_mass_l_t_150]          ,   "Flow          Thermal" },
        { 0x0069, [promag_400]              ,   "Flow          Electromagnetic" },
        { 0x0071, "Promag 10"               ,   "Flow" },
        { 0x0072, "Promass 10"              ,   "Flow" },
        { 0x0078, [tmd832]                  ,   "Temperature   Head Mounted" },
        { 0x0079, [tmd833]                  ,   "Temperature   Compact" },
        { 0x007C, [tmd842]                  ,   "Temperature   DIN Rail Mounted" },
        { 0x008C, [mypro_ph]                ,   "Analytical    pH" },
        { 0x008D, [mypro_lfc]               ,   "Analytical    Conductivity - Conductive" },
        { 0x008E, [mypro_lfi]               ,   "Analytical    Conductivity - Inductive" },
        { 0x009C, [liquiline_cm44x]         ,   "Analytical    Multivariable" },
        { 0x009D, [liquistation_csfxx]      ,   "Analytical    Liquid Sampling" },
        { 0x009F, "Liquiline System CA80xx" ,   "Analytical    Analysis Transmitter" },
        { 0x00A3, [liquiline_compact_cm82]  ,   "Analytical    Multiparameter" },
        { 0x00B5, [nmt530]                  ,   "Temperature   Sensor" },
        { 0x00B6, [nms530]                  ,   "Level         Servo" },
        { 0x00C8, [tmt182]                  ,   "Temperature   Head Mounted" },
        { 0x00C9, [tmt122]                  ,   "Temperature   DIN Rail" },
        { 0x00CA, [tmt162]                  ,   "Temperature" },
        { 0x00CB, [tmt142]                  ,   "Temperature" },
        { 0x00CC, [itemp_tmt82]             ,   "Temperature" },
        { 0x00CE, [itemp_tmt162]            ,   "Temperature" },
        { 0x00CF, [trustsens_tm371_tm372]   ,   "Temperature" },
        { 0x00D0, [itemp_tmt72]             ,   "Temperature" },
        { 0x00D1, "iTEMP TMT142"            ,   "Temperature" },
        { 0x00EE, [crocus_p]                ,   "Pressure      Absolute/Relative" },
        { 0x00EF, [crocus_d]                ,   "Pressure      Differential" },
        { 0x00F0, [swa70]                   ,   "WirelessHART  Adapter" },
        { 0x00F1, [swg70]                   ,   "WirelessHART  Adapter" },
        { 0x1103, [fmu860]                  ,   "Level         Ultrasonic" },
        { 0x1104, [fmu861]                  ,   "Level         Ultrasonic" },
        { 0x1105, [fmu862]                  ,   "Level         Ultrasonic" },
        { 0x1106, [fmr130]                  ,   "Level         Radar - Microwave" },
        { 0x1107, [cerabar_s]               ,   "Pressure      Absolute" },
        { 0x1108, [fec12]                   ,   "Level         Capacitance" },
        { 0x1109, [deltabar_s]              ,   "Pressure      Differential" },
        { 0x110A, [fmu231]                  ,   "Level         Ultrasonic" },
        { 0x110B, [deltapilot]              ,   "Level         Hydrostatic" },
        { 0x110C, [fmr23x]                  ,   "Level         Ultrasonic" },
        { 0x110D, [fmp200]                  ,   "Level" },
        { 0x110E, [cerabar_m]               ,   "Pressure      Absolute/Relative" },
        { 0x110F, [fmr2xx]                  ,   "Level         Microwave" },
        { 0x1110, [fmr53x]                  ,   "Level         Microwave" },
        { 0x1111, [fmu4x]                   ,   "Level         Ultrasonic" },
        { 0x1112, [fmp4x]                   ,   "Level         Radar" },
        { 0x1113, [fmg60]                   ,   "Level         Density - Nuclear" },
        { 0x1114, [nrf590]                  },
        { 0x1117, [deltabars]               ,   "Pressure      Differential" },
        { 0x1118, [cerabars]                ,   "Pressure      Absolute/Relative" },
        { 0x1119, [cerabar_m_5x]            ,   "Pressure      Absolute/Relative" },
        { 0x111A, [deltapilots]             ,   "Level         Hydrostatic" },
        { 0x111B, [fmu9x]                   ,   "Level         Ultrasonic" },
        { 0x111D, [fmi5x]                   ,   "Level" },
        { 0x111E, [fmr25x]                  ,   "Level         Microwave" },
        { 0x111F, [fmr54x]                  ,   "Level         Microwave" },
        { 0x1120, [fmp4x_i]                 ,   "Level" },
        { 0x1121, [deltabar_m_5x]           ,   "Pressure      Differential" },
        { 0x1122, [levelflex_fmp5x]         ,   "Level         Radar" },
        { 0x1123, [deltapilot_m_5x]         ,   "Level         Hydrostatic" },
        { 0x1124, [waterpilot_2x]           ,   "Level         Hydrostatic" },
        { 0x1127, [deltabar_fmd71x]         ,   "Pressure      Differential" },
        { 0x1128, [micropilot_5x]           ,   "Level         Radar" },
        { 0x112A, "Cerabar 5xB/7xB"         ,   "Pressure" },
        { 0x112B, [micropilot_6x]           ,   "Level" },
        { 0x112D, "Proservo NMS8x"          ,   "Level" },
        { 0x112E, "Micropilot NMR8x"        ,   "Level" },
        { 0x112F, "Tank Side Monitor NRF8x" ,   "Level" },
        { 0x1130, [gammapilot_5x]           ,   "Level         Radiometric" },
        { 0x1131, "Deltabar 5xB/7xB"        ,   "Pressure" },
        { 0x1132, [promag33]                ,   "Flow          Electromagnetic" },
        { 0x1133, [prowirl70]               ,   "Flow          Vortex" },
        { 0x1134, [promass63]               ,   "Flow          Coriolis Mass" },
        { 0x1135, [promag39]                ,   "Flow          Magmeter - Rack Version" },
        { 0x1136, [promag35s]               ,   "Flow          High Performance Magmeter" },
        { 0x1137, [prowirl77]               ,   "Flow          Vortex" },
        { 0x1138, [prowirl_200]             ,   "Flow          2-wire Vortex" },
        { 0x1139, [promass60]               ,   "Flow          Mass" },
        { 0x113A, [promag_100]              ,   "Flow          Electromagnetic" },
        { 0x113B, [promass_300_500]         ,   "Flow          Coriolis Mass" },
        { 0x113C, [promag_300_500]          ,   "Flow" },
        { 0x1140, [proson_f]                ,   "Flow          Ultrasonic" },
        { 0x1141, [promag50]                ,   "Flow          Magnetic" },
        { 0x1142, [promag53]                ,   "Flow          Magnetic" },
        { 0x1143, [promag51]                ,   "Flow          Magnetic - Custody Transfer" },
        { 0x1144, [promag55]                ,   "Flow          Magnetic" },
        { 0x1145, [promag10]                ,   "Flow          Magnetic" },
        { 0x1146, [promag23]                ,   "Flow          2-wire Magnetic" },
        { 0x1148, [promag_200]              ,   "Flow          Electromagnetic" },
        { 0x1149, [cubemass_dci]            ,   "Flow          Coriolis Mass" },
        { 0x114A, [promass_100]             ,   "Flow          Coriolis Mass" },
        { 0x1150, [promass80]               ,   "Flow          Coriolis Mass" },
        { 0x1151, [promass83]               ,   "Flow          Coriolis Mass" },
        { 0x1152, [promass84]               ,   "Flow          Coriolis Mass - Custody Transfer" },
        { 0x1153, [promass40]               ,   "Flow          Coriolis Mass" },
        { 0x1154, [promass_200]             ,   "Flow          2-wire Coriolis" },
        { 0x1155, [cngmass_dci]             ,   "Flow          Coriolis Mass" },
        { 0x1156, [prowirl72]               ,   "Flow          2-wire Vortex" },
        { 0x1157, [prowirl73]               ,   "Flow          2-wire Vortex" },
        { 0x1158, [pflow90]                 ,   "Flow          Ultrasonic" },
        { 0x1159, [pflow93]                 ,   "Flow          Ultrasonic" },
        { 0x115A, [prosonic_flow_b_200]     ,   "Flow          Ultrasonic" },
        { 0x115C, [prosonic_flow_e100]      ,   "Flow          Ultrasonic" },
        { 0x115D, [prosonic_flow_300_500]   ,   "Flow          Ultrasonic" },
        { 0x1160, "t-mass 300 500"          ,   "Flow          Thermal" },
        { 0x1161, [pflow92]                 ,   "Flow          Ultrasonic" },
        { 0x1162, [pflow91]                 ,   "Flow          Ultrasonic" },
        { 0x1164, [at70]                    ,   "Flow          Thermal" },
        { 0x1165, [t_mass65]                ,   "Flow          Thermal" },
        { 0x1166, [t_mass_ab_150]           ,   "Flow          Thermal" },
        { 0x1167, [promag_400]              ,   "Flow          Electromagnetic" },
        { 0x1168, [t_mass_l_t_150]          ,   "Flow          Thermal" },
        { 0x1169, [promag_400]              ,   "Flow          Electromagnetic" },
        { 0x1171, "Promag 10"               ,   "Flow" },
        { 0x1172, "Promass 10"              ,   "Flow" },
        { 0x1178, [tmd832]                  ,   "Temperature   Head Mounted" },
        { 0x1179, [tmd833]                  ,   "Temperature   Compact" },
        { 0x117C, [tmd842]                  ,   "Temperature   DIN Rail Mounted" },
        { 0x118C, [mypro_ph]                ,   "Analytical    pH" },
        { 0x118D, [mypro_lfc]               ,   "Analytical    Conductivity - Conductive" },
        { 0x118E, [mypro_lfi]               ,   "Analytical    Conductivity - Inductive" },
        { 0x119C, [liquiline_cm44x]         ,   "Analytical    Multivariable" },
        { 0x119D, [liquistation_csfxx]      ,   "Analytical    Liquid Sampling" },
        { 0x119F, "Liquiline System CA80xx" ,   "Analytical    Analysis Transmitter" },
        { 0x11A3, [liquiline_compact_cm82]  ,   "Analytical    Multiparameter" },
        { 0x11B5, [nmt530]                  ,   "Temperature   Sensor" },
        { 0x11B6, [nms530]                  ,   "Level         Servo" },
        { 0x11C8, [tmt182]                  ,   "Temperature   Head Mounted" },
        { 0x11C9, [tmt122]                  ,   "Temperature   DIN Rail" },
        { 0x11CA, [tmt162]                  ,   "Temperature" },
        { 0x11CB, [tmt142]                  ,   "Temperature" },
        { 0x11CC, [itemp_tmt82]             ,   "Temperature" },
        { 0x11CE, [itemp_tmt162]            ,   "Temperature" },
        { 0x11CF, [trustsens_tm371_tm372]   ,   "Temperature" },
        { 0x11D0, [itemp_tmt72]             ,   "Temperature" },
        { 0x11D1, "iTEMP TMT142"            ,   "Temperature" },
        { 0x11EE, [crocus_p]                ,   "Pressure      Absolute/Relative" },
        { 0x11EF, [crocus_d]                ,   "Pressure      Differential" },
        { 0x11F0, [swa70]                   ,   "WirelessHART  Adapter" },
        { 0x11F1, [swg70]                   ,   "WirelessHART  Adapter" }
    }
}

/*************************************************************************************
 * ABB Device Type Codes
 *     Note: was Fischer and Porter
 */
VARIABLE Fischer_Porter_model_code    /* 0x0012 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0007, [xm2000]     ,   "Flow          Magnetic" },
        { 0x0008, [xe4000]     ,   "Flow          Magnetic" },
        { 0x000E, [vt1000]     ,   "Flow          Vortex" },
        { 0x000F, [vm1000]     ,   "Flow          Vortex" },
        { 0x0019, [xm1000]     ,   "Flow          Magnetic" },
        { 0x001A, [sm1000]     ,   "Flow          Magnetic" },
        { 0x0042, [pth]        ,   "Pressure      Differential" },
        { 0x0050, [tb82ph]     ,   "Analytical    pH" },
        { 0x0051, [tb82orp]    ,   "Analytical    ORP" },
        { 0x0052, [tb82pion]   ,   "Analytical    pION" },
        { 0x0053, [tb82iconc]  ,   "Analytical    IConc" },
        { 0x0054, [tb82cond]   ,   "Analytical    Electrode Conductivity" },
        { 0x0055, [tb82conc]   ,   "Analytical    Electrode Conductivity" },
        { 0x0056, [tb82tecond] ,   "Analytical    Electrode Conductivity" },
        { 0x0057, [tb82teconc] ,   "Analytical    Electrode Conductivity" },
        { 0x0058, [tb82tccond] ,   "Analytical    Toroidal Conductivity" },
        { 0x0059, [tb82tcconc] ,   "Analytical    Toroidal Conductivity" },
        { 0x1207, [xm2000]     ,   "Flow          Magnetic" },
        { 0x1208, [xe4000]     ,   "Flow          Magnetic" },
        { 0x120E, [vt1000]     ,   "Flow          Vortex" },
        { 0x120F, [vm1000]     ,   "Flow          Vortex" },
        { 0x1219, [xm1000]     ,   "Flow          Magnetic" },
        { 0x121A, [sm1000]     ,   "Flow          Magnetic" },
        { 0x1242, [pth]        ,   "Pressure      Differential" },
        { 0x1250, [tb82ph]     ,   "Analytical    pH" },
        { 0x1251, [tb82orp]    ,   "Analytical    ORP" },
        { 0x1252, [tb82pion]   ,   "Analytical    pION" },
        { 0x1253, [tb82iconc]  ,   "Analytical    IConc" },
        { 0x1254, [tb82cond]   ,   "Analytical    Electrode Conductivity" },
        { 0x1255, [tb82conc]   ,   "Analytical    Electrode Conductivity" },
        { 0x1256, [tb82tecond] ,   "Analytical    Electrode Conductivity" },
        { 0x1257, [tb82teconc] ,   "Analytical    Electrode Conductivity" },
        { 0x1258, [tb82tccond] ,   "Analytical    Toroidal Conductivity" },
        { 0x1259, [tb82tcconc] ,   "Analytical    Toroidal Conductivity" }
    }
}

/*************************************************************************************
 * Fisher Controls Intl LLC Device Type Codes
 */
VARIABLE Fisher_Controls_model_code    /* 0x0013 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0000, [position_xmitter]      ,   "Valve         Positioner Transmitter" },
        { 0x0003, [dvc6000]               ,   "Valve         Positioner" },
        { 0x0004, [dlc3010]               ,   "Valve         Positioner" },
        { 0x0005, [dvc2000]               ,   "Valve         Controller" },
        { 0x0008, [position_monitor_4320] },
        { 0x0009, [dvc6200]               ,   "Valve         Digital Positioner" },
        { 0x000A, [dvc6200_sis]           ,   "Valve         Digital Controller" },
        { 0x000B, [dvc6000_hw2]           ,   "Valve         Digital Controller" },
        { 0x000F, "DLC3100"               ,   "Level" },
        { 0x1300, [position_xmitter]      ,   "Valve         Positioner Transmitter" },
        { 0x1303, [dvc6000]               ,   "Valve         Positioner" },
        { 0x1304, [dlc3010]               ,   "Valve         Positioner" },
        { 0x1305, [dvc2000]               ,   "Valve         Controller" },
        { 0x1308, [position_monitor_4320] },
        { 0x1309, [dvc6200]               ,   "Valve         Digital Positioner" },
        { 0x130A, [dvc6200_sis]           ,   "Valve         Digital Controller" },
        { 0x130B, [dvc6000_hw2]           ,   "Valve         Digital Controller" },
        { 0x130F, "DLC3100"               ,   "Level" }
    }
}

/*************************************************************************************
 * SchneiderElectric/Foxboro Device Type Codes
 *     Note: was Foxboro 
 */
VARIABLE SchneiderElectric_Foxboro_model_code    /* 0x0014 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [mag2]                  ,   "Flow          Magnetic" },
        { 0x0002, [iaspt_premium_2]       ,   "Pressure" },
        { 0x0003, [cr876]                 },
        { 0x0004, [ec876]                 },
        { 0x0005, [ph876]                 ,   "Analytical    pH/ORP/ISE" },
        { 0x0007, [rtt30]                 ,   "Temperature" },
        { 0x0008, [ecs_pressure]          ,   "Pressure" },
        { 0x0009, [cft51]                 ,   "Flow" },
        { 0x000A, [rtt80]                 ,   "Temperature" },
        { 0x000B, [fox_876ph_s]           ,   "Analytical    Electrochemical" },
        { 0x0019, "CFT34A"                ,   "Flow          Coriolis Mass" },
        { 0x001B, [imt31a]                ,   "Flow          Electromagnetic" },
        { 0x001C, [imt33a]                ,   "Flow          Electromagnetic" },
        { 0x001D, [imv31]                 ,   "Pressure      Multivariable" },
        { 0x001E, [vortex_84]             ,   "Flow          Vortex" },
        { 0x001F, [vortex_84c]            ,   "Flow          Vortex" },
        { 0x0028, [imt96]                 ,   "Flow          Magnetic" },
        { 0x0029, [imt25]                 ,   "Flow          Magnetic" },
        { 0x002E, [ia_pressure]           ,   "Pressure" },
        { 0x002F, [imv_25]                ,   "Pressure      Multivariable" },
        { 0x0030, [fox_875ph]             ,   "Analytical    pH/ORP" },
        { 0x0033, [itvortex]              ,   "Flow          Vortex" },
        { 0x0034, [cft50]                 ,   "Flow          Coriolis Mass" },
        { 0x0035, [fox_875cr]             ,   "Analytical    Conductivity" },
        { 0x0036, [fox_875ec]             ,   "Analytical    Conductivity" },
        { 0x0038, [rtt15]                 ,   "Temperature" },
        { 0x0040, [ia_pressure_s]         ,   "Pressure" },
        { 0x0041, [rtt15_h]               ,   "Temperature" },
        { 0x0042, [rtt15s]                ,   "Temperature" },
        { 0x0043, [absolute_gauge_diff]   ,   "Pressure" },
        { 0x0044, [free_space_radar_lrxx] ,   "Level         Radar" },
        { 0x007D, [cft30x]                ,   "Flow          Coriolis Mass" },
        { 0x1401, [mag2]                  ,   "Flow          Magnetic" },
        { 0x1402, [iaspt_premium_2]       ,   "Pressure" },
        { 0x1403, [cr876]                 },
        { 0x1404, [ec876]                 },
        { 0x1405, [ph876]                 ,   "Analytical    pH/ORP/ISE" },
        { 0x1407, [rtt30]                 ,   "Temperature" },
        { 0x1408, [ecs_pressure]          ,   "Pressure" },
        { 0x1409, [cft51]                 ,   "Flow" },
        { 0x140A, [rtt80]                 ,   "Temperature" },
        { 0x140B, [fox_876ph_s]           ,   "Analytical    Electrochemical" },
        { 0x1419, "CFT34A"                ,   "Flow          Coriolis Mass" },
        { 0x141B, [imt31a]                ,   "Flow          Electromagnetic" },
        { 0x141C, [imt33a]                ,   "Flow          Electromagnetic" },
        { 0x141D, [imv31]                 ,   "Pressure      Multivariable" },
        { 0x141E, [vortex_84]             ,   "Flow          Vortex" },
        { 0x141F, [vortex_84c]            ,   "Flow          Vortex" },
        { 0x1428, [imt96]                 ,   "Flow          Magnetic" },
        { 0x1429, [imt25]                 ,   "Flow          Magnetic" },
        { 0x142E, [ia_pressure]           ,   "Pressure" },
        { 0x142F, [imv_25]                ,   "Pressure      Multivariable" },
        { 0x1430, [fox_875ph]             ,   "Analytical    pH/ORP" },
        { 0x1433, [itvortex]              ,   "Flow          Vortex" },
        { 0x1434, [cft50]                 ,   "Flow          Coriolis Mass" },
        { 0x1435, [fox_875cr]             ,   "Analytical    Conductivity" },
        { 0x1436, [fox_875ec]             ,   "Analytical    Conductivity" },
        { 0x1438, [rtt15]                 ,   "Temperature" },
        { 0x1440, [ia_pressure_s]         ,   "Pressure" },
        { 0x1441, [rtt15_h]               ,   "Temperature" },
        { 0x1442, [rtt15s]                ,   "Temperature" },
        { 0x1443, [absolute_gauge_diff]   ,   "Pressure" },
        { 0x1444, [free_space_radar_lrxx] ,   "Level         Radar" },
        { 0x147D, [cft30x]                ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * Fuji Device Type Codes
 */
VARIABLE Fuji_model_code    /* 0x0015 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [fcx_ac]   ,   "Pressure" },
        { 0x0002, [fcx_a2c2] ,   "Pressure" },
        { 0x0004, [fcx_a3s]  ,   "Pressure" },
        { 0x0005, "FCX-A4"   ,   "Pressure" },
        { 0x0040, [fst]      ,   "Flow          Ultrasonic" },
        { 0x0081, [frc]      ,   "Temperature" },
        { 0x1501, [fcx_ac]   ,   "Pressure" },
        { 0x1502, [fcx_a2c2] ,   "Pressure" },
        { 0x1504, [fcx_a3s]  ,   "Pressure" },
        { 0x1505, "FCX-A4"   ,   "Pressure" },
        { 0x1540, [fst]      ,   "Flow          Ultrasonic" },
        { 0x1581, [frc]      ,   "Temperature" }
    }
}

/*************************************************************************************
 * ABB Device Type Codes
 *     Note: was Hartmann and Braun
 */
VARIABLE Hartmann_Braun_model_code    /* 0x0016 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0003, [teu471]  },
        { 0x0004, [teu421]  },
        { 0x0005, [teu211]  ,   "Temperature" },
        { 0x0006, [ts11]    ,   "Temperature" },
        { 0x0008, [th02]    ,   "Temperature" },
        { 0x0040, [tzid]    ,   "Valve         Positioner/Pneumatic" },
        { 0x0041, [tzidc]   ,   "Valve         Positioner/Pneumatic" },
        { 0x0085, [as800]   ,   "Pressure" },
        { 0x0087, [contrac] ,   "Valve         Actuator - Electrical" },
        { 0x1603, [teu471]  },
        { 0x1604, [teu421]  },
        { 0x1605, [teu211]  ,   "Temperature" },
        { 0x1606, [ts11]    ,   "Temperature" },
        { 0x1608, [th02]    ,   "Temperature" },
        { 0x1640, [tzid]    ,   "Valve         Positioner/Pneumatic" },
        { 0x1641, [tzidc]   ,   "Valve         Positioner/Pneumatic" },
        { 0x1685, [as800]   ,   "Pressure" },
        { 0x1687, [contrac] ,   "Valve         Actuator - Electrical" }
    }
}

/*************************************************************************************
 * Honeywell Device Type Codes
 */
VARIABLE Honeywell_model_code    /* 0x0017 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [st3000]               ,   "Pressure" },
        { 0x0002, [stt25t]               ,   "Temperature   Dual Input" },
        { 0x0003, [hwflow]               ,   "Flow          Magnetic" },
        { 0x0004, [stt25h]               ,   "Temperature" },
        { 0x0005, [herculine]            ,   "Valve         Actuator" },
        { 0x0006, [smartcet]             ,   "Analytical    Corrosion" },
        { 0x0007, [stt17h]               ,   "Temperature" },
        { 0x0008, [stt2500]              },
        { 0x0009, [stt25s]               ,   "Temperature" },
        { 0x000A, [smartcet5500]         },
        { 0x000B, [stt25h6]              ,   "Temperature" },
        { 0x000C, [stt25t6]              ,   "Temperature" },
        { 0x0020, [st800]                ,   "Pressure" },
        { 0x0021, [st700]                ,   "Pressure" },
        { 0x0022, [stt850]               ,   "Temperature" },
        { 0x0024, [smv800]               ,   "Pressure      Multivariable" },
        { 0x0026, [slg700]               ,   "Level" },
        { 0x0027, "SLN 700"              ,   "Level" },
        { 0x0029, [xnx]                  ,   "Analytical    Gas Detector" },
        { 0x002A, [stt750]               ,   "Temperature" },
        { 0x002B, [stt700_smartline]     ,   "Temperature" },
        { 0x002C, [rmx40_x50]            ,   "Level         Radar" },
        { 0x0040, [stt650]               ,   "Temperature" },
        { 0x0050, [twm9000]              ,   "Flow          Electromagnetic" },
        { 0x0051, [twc9000]              ,   "Flow          Coriolis Mass" },
        { 0x0054, [twv9000]              ,   "Flow          Vortex" },
        { 0x0055, [twm1000]              ,   "Flow          Electromagnetic" },
        { 0x0056, [smartline_radar]      ,   "Level         Radar" },
        { 0x0057, [smartline_tdr]        ,   "Level" },
        { 0x0058, [smartline_rm76]       ,   "Level" },
        { 0x0059, [smartline_rm77]       ,   "Level         Radar" },
        { 0x005A, [raeguard_3]           ,   "Analytical    Gas Detector" },
        { 0x005B, "VersaFlow SONIC 1000" ,   "Level         Ultrasonic" },
        { 0x1701, [st3000]               ,   "Pressure" },
        { 0x1702, [stt25t]               ,   "Temperature   Dual Input" },
        { 0x1703, [hwflow]               ,   "Flow          Magnetic" },
        { 0x1704, [stt25h]               ,   "Temperature" },
        { 0x1705, [herculine]            ,   "Valve         Actuator" },
        { 0x1706, [smartcet]             ,   "Analytical    Corrosion" },
        { 0x1707, [stt17h]               ,   "Temperature" },
        { 0x1708, [stt2500]              },
        { 0x1709, [stt25s]               ,   "Temperature" },
        { 0x170A, [smartcet5500]         },
        { 0x170B, [stt25h6]              ,   "Temperature" },
        { 0x170C, [stt25t6]              ,   "Temperature" },
        { 0x1720, [st800]                ,   "Pressure" },
        { 0x1721, [st700]                ,   "Pressure" },
        { 0x1722, [stt850]               ,   "Temperature" },
        { 0x1724, [smv800]               ,   "Pressure      Multivariable" },
        { 0x1726, [slg700]               ,   "Level" },
        { 0x1727, "SLN 700"              ,   "Level" },
        { 0x1729, [xnx]                  ,   "Analytical    Gas Detector" },
        { 0x172A, [stt750]               ,   "Temperature" },
        { 0x172B, [stt700_smartline]     ,   "Temperature" },
        { 0x172C, [rmx40_x50]            ,   "Level         Radar" },
        { 0x1740, [stt650]               ,   "Temperature" },
        { 0x1750, [twm9000]              ,   "Flow          Electromagnetic" },
        { 0x1751, [twc9000]              ,   "Flow          Coriolis Mass" },
        { 0x1754, [twv9000]              ,   "Flow          Vortex" },
        { 0x1755, [twm1000]              ,   "Flow          Electromagnetic" },
        { 0x1756, [smartline_radar]      ,   "Level         Radar" },
        { 0x1757, [smartline_tdr]        ,   "Level" },
        { 0x1758, [smartline_rm76]       ,   "Level" },
        { 0x1759, [smartline_rm77]       ,   "Level         Radar" },
        { 0x175A, [raeguard_3]           ,   "Analytical    Gas Detector" },
        { 0x175B, "VersaFlow SONIC 1000" ,   "Level         Ultrasonic" }
    }
}

/*************************************************************************************
 * Thermo MeasureTech
 *     Note: was KayRay
 */
VARIABLE kayray_model_code    /* 0x0019 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0008, [level_4050]         },
        { 0x0009, [density_3680]       ,   "Analytical    Density - Nuclear" },
        { 0x000A, [kayray4790]         ,   "Level         Continuous Gamma" },
        { 0x000B, [accu_pulse_pro]     ,   "Level         Laser Ranging" },
        { 0x000F, [steam_quality_3280] },
        { 0x1908, [level_4050]         },
        { 0x1909, [density_3680]       ,   "Analytical    Density - Nuclear" },
        { 0x190A, [kayray4790]         ,   "Level         Continuous Gamma" },
        { 0x190B, [accu_pulse_pro]     ,   "Level         Laser Ranging" },
        { 0x190F, [steam_quality_3280] }
    }
}

/*************************************************************************************
 * ABB Device Type Codes
 *     Note: was Kent
 */
VARIABLE ABB_model_code    /* 0x001A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [ksx]                     ,   "Pressure" },
        { 0x0002, [abb_600t]                ,   "Pressure      Differential" },
        { 0x0003, [abb_2600t_268]           ,   "Pressure      Safety" },
        { 0x0004, [abb_2600t_262_264]       ,   "Pressure" },
        { 0x0005, [abb_2600t_364]           ,   "Pressure" },
        { 0x0007, [abb_2600t_266]           ,   "Pressure      Absolute/Differential" },
        { 0x0008, [abb_kst]                 ,   "Temperature" },
        { 0x0009, [th01]                    ,   "Temperature" },
        { 0x000A, [abb_658t]                ,   "Temperature" },
        { 0x000B, [ttx300_series]           ,   "Temperature" },
        { 0x000C, [abb_652]                 ,   "Temperature" },
        { 0x000D, "TTx200 family"           ,   "Temperature" },
        { 0x000F, [abb_665]                 ,   "Valve         Positioner" },
        { 0x0010, [deltapi_kt]              ,   "Valve         Positioner" },
        { 0x0018, [magmaster]               ,   "Flow          Electromagnetic" },
        { 0x001A, [fvs4000]                 ,   "Flow          Vortex/Swirl" },
        { 0x001B, [fam540]                  ,   "Flow          Variable Area" },
        { 0x001C, [fmt500ig]                ,   "Flow" },
        { 0x001D, [fsm4000]                 ,   "Flow          Electromagnetic" },
        { 0x001E, [fex300]                  ,   "Flow          Electromagnetic" },
        { 0x001F, [fex100]                  ,   "Flow          Electromagnetic" },
        { 0x0021, [abb_2600ph]              ,   "Analytical    pH" },
        { 0x0022, [awt210]                  ,   "Analytical" },
        { 0x0023, [tb82ph_comb]             ,   "Analytical    pH" },
        { 0x0024, [tb82ec_comb]             ,   "Analytical    4 Electrode Conductivity" },
        { 0x0025, [tb82te_comb]             ,   "Analytical    2 Electrode Conductivity" },
        { 0x0026, [tb82tc_comb]             ,   "Analytical    Troidal Conductivity" },
        { 0x0029, [ufte_pa]                 },
        { 0x002A, [ufte_ff]                 },
        { 0x0030, [apa592ph]                ,   "Analytical    pH/ORP/pIon/Ion Conc" },
        { 0x0031, [aca592ec]                ,   "Analytical    Conductivity/Concentration" },
        { 0x0032, [aca592te]                ,   "Analytical    Conductivity/Concentration" },
        { 0x0033, [aca592tc]                ,   "Analytical    Conductivity/Concentration" },
        { 0x0034, [az200]                   ,   "Analytical    Oxygen" },
        { 0x0035, [awt420]                  ,   "Analytical" },
        { 0x0063, [cmxx_controlmaster]      },
        { 0x0089, [abb_263_265_2000t]       ,   "Pressure      P, dP + Multivariable" },
        { 0x008A, [abb_2600t_267]           ,   "Pressure" },
        { 0x008C, [abb_261]                 ,   "Pressure      Absolute" },
        { 0x008D, [edp300]                  ,   "Valve         Positioner/Electropneumatic" },
        { 0x008E, [abb_2600t_266_mv]        ,   "Pressure      P, dP + Multivariable" },
        { 0x0091, "Pxx100"                  ,   "Pressure      Absolute/Gauge" },
        { 0x0099, [nhu200_wl]               ,   "WirelessHART  Adapter" },
        { 0x009B, [tsp3xx_energyharvester]  },
        { 0x009C, [lst400_ultrasonic_level] ,   "Level         Ultrasonic" },
        { 0x009D, [lmt_level]               ,   "Level" },
        { 0x009E, [lwt_level]               ,   "Level         Radar" },
        { 0x00A0, [fcx_xxx]                 ,   "Flow" },
        { 0x00A1, [lst300_compact_level]    ,   "Level         Ultrasonic" },
        { 0x00A3, [fsx400]                  ,   "Flow          Mass" },
        { 0x00A4, [few5xx_fex6xx]           ,   "Flow" },
        { 0x1A01, [ksx]                     ,   "Pressure" },
        { 0x1A02, [abb_600t]                ,   "Pressure      Differential" },
        { 0x1A03, [abb_2600t_268]           ,   "Pressure      Safety" },
        { 0x1A04, [abb_2600t_262_264]       ,   "Pressure" },
        { 0x1A05, [abb_2600t_364]           ,   "Pressure" },
        { 0x1A07, [abb_2600t_266]           ,   "Pressure      Absolute/Differential" },
        { 0x1A08, [abb_kst]                 ,   "Temperature" },
        { 0x1A09, [th01]                    ,   "Temperature" },
        { 0x1A0A, [abb_658t]                ,   "Temperature" },
        { 0x1A0B, [ttx300_series]           ,   "Temperature" },
        { 0x1A0C, [abb_652]                 ,   "Temperature" },
        { 0x1A0D, "TTx200 family"           ,   "Temperature" },
        { 0x1A0F, [abb_665]                 ,   "Valve         Positioner" },
        { 0x1A10, [deltapi_kt]              ,   "Valve         Positioner" },
        { 0x1A18, [magmaster]               ,   "Flow          Electromagnetic" },
        { 0x1A1A, [fvs4000]                 ,   "Flow          Vortex/Swirl" },
        { 0x1A1B, [fam540]                  ,   "Flow          Variable Area" },
        { 0x1A1C, [fmt500ig]                ,   "Flow" },
        { 0x1A1D, [fsm4000]                 ,   "Flow          Electromagnetic" },
        { 0x1A1E, [fex300]                  ,   "Flow          Electromagnetic" },
        { 0x1A1F, [fex100]                  ,   "Flow          Electromagnetic" },
        { 0x1A21, [abb_2600ph]              ,   "Analytical    pH" },
        { 0x1A22, [awt210]                  ,   "Analytical" },
        { 0x1A23, [tb82ph_comb]             ,   "Analytical    pH" },
        { 0x1A24, [tb82ec_comb]             ,   "Analytical    4 Electrode Conductivity" },
        { 0x1A25, [tb82te_comb]             ,   "Analytical    2 Electrode Conductivity" },
        { 0x1A26, [tb82tc_comb]             ,   "Analytical    Troidal Conductivity" },
        { 0x1A29, [ufte_pa]                 },
        { 0x1A2A, [ufte_ff]                 },
        { 0x1A30, [apa592ph]                ,   "Analytical    pH/ORP/pIon/Ion Conc" },
        { 0x1A31, [aca592ec]                ,   "Analytical    Conductivity/Concentration" },
        { 0x1A32, [aca592te]                ,   "Analytical    Conductivity/Concentration" },
        { 0x1A33, [aca592tc]                ,   "Analytical    Conductivity/Concentration" },
        { 0x1A34, [az200]                   ,   "Analytical    Oxygen" },
        { 0x1A35, [awt420]                  ,   "Analytical" },
        { 0x1A63, [cmxx_controlmaster]      },
        { 0x1A89, [abb_263_265_2000t]       ,   "Pressure      P, dP + Multivariable" },
        { 0x1A8A, [abb_2600t_267]           ,   "Pressure" },
        { 0x1A8C, [abb_261]                 ,   "Pressure      Absolute" },
        { 0x1A8D, [edp300]                  ,   "Valve         Positioner/Electropneumatic" },
        { 0x1A8E, [abb_2600t_266_mv]        ,   "Pressure      P, dP + Multivariable" },
        { 0x1A91, "Pxx100"                  ,   "Pressure      Absolute/Gauge" },
        { 0x1A99, [nhu200_wl]               ,   "WirelessHART  Adapter" },
        { 0x1A9B, [tsp3xx_energyharvester]  },
        { 0x1A9C, [lst400_ultrasonic_level] ,   "Level         Ultrasonic" },
        { 0x1A9D, [lmt_level]               ,   "Level" },
        { 0x1A9E, [lwt_level]               ,   "Level         Radar" },
        { 0x1AA0, [fcx_xxx]                 ,   "Flow" },
        { 0x1AA1, [lst300_compact_level]    ,   "Level         Ultrasonic" },
        { 0x1AA3, [fsx400]                  ,   "Flow          Mass" },
        { 0x1AA4, [few5xx_fex6xx]           ,   "Flow" }
    }
}

/*************************************************************************************
 * M-System Co Device Type Codes
 */
VARIABLE M_System_Co_model_code    /* 0x001D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [b6u]           ,   "Temperature   2-wire" },
        { 0x0002, [b3hu]          ,   "Temperature" },
        { 0x0003, [m_system_27hu] ,   "Temperature   Head Mounted" },
        { 0x0004, [b3hu2]         ,   "Temperature" },
        { 0x1D01, [b6u]           ,   "Temperature   2-wire" },
        { 0x1D02, [b3hu]          ,   "Temperature" },
        { 0x1D03, [m_system_27hu] ,   "Temperature   Head Mounted" },
        { 0x1D04, [b3hu2]         ,   "Temperature" }
    }
}

/*************************************************************************************
 * Micro Motion Device Type Codes
 */
VARIABLE micro_motion_model_code    /* 0x001F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0007, [mass_flow_9712]              ,   "Flow          Coriolis Mass" },
        { 0x0014, [mass_flow_9720]              },
        { 0x0015, [mass_flow_9739]              ,   "Flow          Coriolis Mass" },
        { 0x0016, [mvd_9739]                    ,   "Flow          Coriolis Mass" },
        { 0x0017, [k_series]                    ,   "Flow          Coriolis Mass" },
        { 0x001E, [mass_flow_9701]              ,   "Flow          Coriolis Mass" },
        { 0x0024, [mvd_2000_io]                 ,   "Flow          Coriolis Mass" },
        { 0x0025, [mvd_1000_io]                 ,   "Flow          Coriolis Mass" },
        { 0x0026, [mvd_2000_is]                 ,   "Flow          Coriolis Mass" },
        { 0x0027, [mvd_1000_is]                 ,   "Flow          Coriolis Mass" },
        { 0x0029, [mvd_1000_ao]                 ,   "Flow          Coriolis Mass" },
        { 0x002A, [mvd_2000_ao]                 ,   "Flow          Coriolis Mass" },
        { 0x0034, [mass_flow_2400s]             ,   "Flow          Coriolis Mass" },
        { 0x003A, [analog_2200s]                ,   "Flow          Coriolis Mass" },
        { 0x003D, [density_gas_viscosity_meter] ,   "Analytical    Viscosity" },
        { 0x0041, [mvd_3000]                    ,   "Flow          Coriolis Mass" },
        { 0x0046, [mass_flow_5700]              ,   "Flow          Mass" },
        { 0x0047, [mass_flow_5700_is]           ,   "Flow          Coriolis Mass" },
        { 0x004B, [flow_4200]                   ,   "Flow          Coriolis Mass" },
        { 0x1F07, [mass_flow_9712]              ,   "Flow          Coriolis Mass" },
        { 0x1F14, [mass_flow_9720]              },
        { 0x1F15, [mass_flow_9739]              ,   "Flow          Coriolis Mass" },
        { 0x1F16, [mvd_9739]                    ,   "Flow          Coriolis Mass" },
        { 0x1F17, [k_series]                    ,   "Flow          Coriolis Mass" },
        { 0x1F1E, [mass_flow_9701]              ,   "Flow          Coriolis Mass" },
        { 0x1F24, [mvd_2000_io]                 ,   "Flow          Coriolis Mass" },
        { 0x1F25, [mvd_1000_io]                 ,   "Flow          Coriolis Mass" },
        { 0x1F26, [mvd_2000_is]                 ,   "Flow          Coriolis Mass" },
        { 0x1F27, [mvd_1000_is]                 ,   "Flow          Coriolis Mass" },
        { 0x1F29, [mvd_1000_ao]                 ,   "Flow          Coriolis Mass" },
        { 0x1F2A, [mvd_2000_ao]                 ,   "Flow          Coriolis Mass" },
        { 0x1F34, [mass_flow_2400s]             ,   "Flow          Coriolis Mass" },
        { 0x1F3A, [analog_2200s]                ,   "Flow          Coriolis Mass" },
        { 0x1F3D, [density_gas_viscosity_meter] ,   "Analytical    Viscosity" },
        { 0x1F41, [mvd_3000]                    ,   "Flow          Coriolis Mass" },
        { 0x1F46, [mass_flow_5700]              ,   "Flow          Mass" },
        { 0x1F47, [mass_flow_5700_is]           ,   "Flow          Coriolis Mass" },
        { 0x1F4B, [flow_4200]                   ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * Moore Industries Device Type Codes
 */
VARIABLE Moore_Industries_model_code    /* 0x0020 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [trz]             ,   "Temperature" },
        { 0x0003, [thz]             ,   "Temperature" },
        { 0x0004, [thz2]            ,   "Temperature" },
        { 0x0005, [htz]             ,   "Temperature" },
        { 0x0006, [tcm]             ,   "Temperature" },
        { 0x0007, [temperature_888] ,   "Temperature" },
        { 0x0008, [thz3_tdz3]       ,   "Temperature" },
        { 0x0009, [stz]             ,   "Temperature" },
        { 0x2001, [trz]             ,   "Temperature" },
        { 0x2003, [thz]             ,   "Temperature" },
        { 0x2004, [thz2]            ,   "Temperature" },
        { 0x2005, [htz]             ,   "Temperature" },
        { 0x2006, [tcm]             ,   "Temperature" },
        { 0x2007, [temperature_888] ,   "Temperature" },
        { 0x2008, [thz3_tdz3]       ,   "Temperature" },
        { 0x2009, [stz]             ,   "Temperature" }
    }
}

/*************************************************************************************
 * PRIME Measurement Products Device Type Codes
 *     Note: was Moore Products
 */
VARIABLE Moore_Products_model_code    /* 0x0021 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [moore340a]  },
        { 0x0002, [moore340b]  },
        { 0x0003, [moore344]   },
        { 0x0005, [prime330]   ,   "Pressure" },
        { 0x0006, [moore340a2] ,   "Pressure      w/ PID" },
        { 0x0008, [moore340b2] },
        { 0x0009, [moore3442]  },
        { 0x000A, [moore340s]  ,   "Pressure      Saturated Steam" },
        { 0x000B, [moore343]   ,   "Temperature" },
        { 0x000C, [moore760d]  },
        { 0x000D, [moore345]   ,   "Pressure" },
        { 0x2101, [moore340a]  },
        { 0x2102, [moore340b]  },
        { 0x2103, [moore344]   },
        { 0x2105, [prime330]   ,   "Pressure" },
        { 0x2106, [moore340a2] ,   "Pressure      w/ PID" },
        { 0x2108, [moore340b2] },
        { 0x2109, [moore3442]  },
        { 0x210A, [moore340s]  ,   "Pressure      Saturated Steam" },
        { 0x210B, [moore343]   ,   "Temperature" },
        { 0x210C, [moore760d]  },
        { 0x210D, [moore345]   ,   "Pressure" }
    }
}

/*************************************************************************************
 * Ronan Device Type Codes
 */
VARIABLE Ronan_model_code    /* 0x0025 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0002, [x96_d]    ,   "Analytical    Density - Gamma Ray" },
        { 0x0003, [x96_l]    ,   "Level         Gamma Ray" },
        { 0x0004, [x96_w]    ,   "Analytical    Weight - Gamma Ray" },
        { 0x0005, [x96_ld]   ,   "Level         w/ Density - Gamma Ray" },
        { 0x0006, [x99_md]   ,   "Analytical    Motorized Density" },
        { 0x0013, [x96si_l]  ,   "Level         Nuclear" },
        { 0x0015, [x96si_ld] ,   "Level         Nuclear w/ Density" },
        { 0x2502, [x96_d]    ,   "Analytical    Density - Gamma Ray" },
        { 0x2503, [x96_l]    ,   "Level         Gamma Ray" },
        { 0x2504, [x96_w]    ,   "Analytical    Weight - Gamma Ray" },
        { 0x2505, [x96_ld]   ,   "Level         w/ Density - Gamma Ray" },
        { 0x2506, [x99_md]   ,   "Analytical    Motorized Density" },
        { 0x2513, [x96si_l]  ,   "Level         Nuclear" },
        { 0x2515, [x96si_ld] ,   "Level         Nuclear w/ Density" }
    }
}

/*************************************************************************************
 * Rosemount Device Type Codes
 */
VARIABLE rosemount_model_code    /* 0x0026 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [pressure_3051b]                  },
        { 0x0002, [temperature_3044]                ,   "Temperature" },
        { 0x0003, [pressure_1151s]                  ,   "Pressure" },
        { 0x0004, [mag_flow_8712]                   ,   "Flow          Magnetic" },
        { 0x0005, [pressure_2001]                   },
        { 0x0006, [pressure_3051]                   ,   "Pressure" },
        { 0x000B, [pressure_3001l]                  },
        { 0x000C, [mag_flow_8712H]                  ,   "Flow          Magnetic" },
        { 0x000D, [temperature_3044C]               ,   "Temperature" },
        { 0x000E, [pressure_3001c]                  ,   "Pressure" },
        { 0x000F, [pressure_3051lp]                 ,   "Pressure" },
        { 0x0010, [vortex_flow_8800]                ,   "Flow          Vortex" },
        { 0x0011, [htg_3201]                        ,   "Level         Hydrostatic" },
        { 0x0012, [pressure_1152s]                  },
        { 0x0013, [htg_3202]                        ,   "Level         Hydrostatic" },
        { 0x0014, [pressure_3001s]                  ,   "Pressure" },
        { 0x0015, [multi_3095FT]                    },
        { 0x0016, [mv_3095]                         ,   "Flow          Multivariable Mass" },
        { 0x0017, [temperature_544]                 },
        { 0x0018, [temperature_644]                 ,   "Temperature" },
        { 0x0019, [temperature_3144]                ,   "Temperature" },
        { 0x001A, [temperature_3244]                ,   "Temperature   Multivariable" },
        { 0x001B, [level_3700]                      },
        { 0x001C, [level_3750]                      ,   "Level" },
        { 0x001D, [TRI_LOOP]                        ,   "Specialty     HART-to-Analog Signal Conv" },
        { 0x001E, [c_3095]                          ,   "Level" },
        { 0x001F, [mm_3095]                         },
        { 0x0020, [tank_3701]                       },
        { 0x0021, [level_3300]                      ,   "Level         Radar" },
        { 0x0022, [asu_3702]                        },
        { 0x0023, [pressure_2088s]                  ,   "Pressure" },
        { 0x0024, [probar]                          ,   "Flow          dP" },
        { 0x0025, [massprobar]                      },
        { 0x0026, [prov]                            },
        { 0x0027, [pressure_2090s]                  ,   "Pressure" },
        { 0x0028, [probar_uc]                       ,   "Flow          dP" },
        { 0x0029, [d_2055]                          },
        { 0x002A, [pv_3095]                         },
        { 0x002D, [sentry]                          },
        { 0x002F, [proplate_uc]                     ,   "Flow          dP" },
        { 0x0036, [pressure_1810]                   ,   "Pressure" },
        { 0x0037, [mag_flow_8712d]                  ,   "Flow          Magnetic" },
        { 0x0038, [pressure_4600]                   ,   "Pressure" },
        { 0x003A, [vortex_flow_8800d]               ,   "Flow          Vortex" },
        { 0x003B, [temperature_248]                 ,   "Temperature" },
        { 0x003C, [mag_flow_8732]                   ,   "Flow          Magnetic" },
        { 0x003D, "8782 Slurry Magnetic Flow Meter" ,   "Flow          Magnetic" },
        { 0x003F, [vortex_flow_8600d]               ,   "Flow          Vortex" },
        { 0x0040, [pressure_2088lp]                 ,   "Pressure      Low Power" },
        { 0x0041, [level_5600]                      },
        { 0x0042, [presssure_951]                   ,   "Pressure" },
        { 0x0043, [level_5400]                      ,   "Level         Radar" },
        { 0x0044, [pressure_3051s_sis]              ,   "Pressure      Certified SIL2" },
        { 0x0045, [temperature_3144p_sis]           ,   "Temperature" },
        { 0x0047, [pressure_4500]                   ,   "Pressure" },
        { 0x0049, [pressure_3051smv_fcmef]          ,   "Flow" },
        { 0x004A, [pressure_3051s_diag]             ,   "Pressure" },
        { 0x004B, [pressure_3051smv_dpvo]           ,   "Pressure" },
        { 0x004E, [wireless_gateway]                },
        { 0x004F, [thum_775_wa]                     },
        { 0x0050, [level_3100]                      ,   "Level         Ultrasonic" },
        { 0x0051, [level_5300]                      ,   "Level         Radar" },
        { 0x0053, [wireless_848t]                   ,   "Temperature   WirelessHART" },
        { 0x0055, [pressure_2051]                   ,   "Pressure" },
        { 0x0056, [pressure_2051lp]                 ,   "Pressure" },
        { 0x0057, [mag_flow_8712e]                  ,   "Flow          Magnetic" },
        { 0x0058, [wireless_648]                    },
        { 0x0059, [wireless_3051s]                  },
        { 0x005A, [wireless_702]                    ,   "Discrete      WirelessHART" },
        { 0x005D, [level_2160]                      ,   "Level         WirelessHART" },
        { 0x005E, "3051S ERS"                       ,   "Pressure" },
        { 0x0062, [metran_150]                      ,   "Pressure" },
        { 0x0064, [wireless_2051]                   },
        { 0x006A, [wireless_3051]                   },
        { 0x006B, [density_3440]                    ,   "Analytical    Density" },
        { 0x006D, [metran_75]                       ,   "Pressure" },
        { 0x006E, [wireless_3308a]                  ,   "Level         Radar - WirelessHART" },
        { 0x006F, [flow_8712em_8732em_hr7]          ,   "Flow" },
        { 0x0070, [pressure_520]                    ,   "Pressure" },
        { 0x0073, [mass_flow_8800d]                 ,   "Flow          Mass" },
        { 0x0076, [wireless_248x]                   ,   "Temperature" },
        { 0x007A, [level_5408_hr7]                  ,   "Level         Radar" },
        { 0x0080, [generic]                         ,   "Interoperable" },
        { 0x0083, [level_5408_hr6]                  ,   "Level         Radar" },
        { 0x0085, "248 Rail Mount Temp Transmitter",   "Temperature" },
        { 0x0087, [level_2140]                      ,   "Level" },
        { 0x008A, [smart_pressure_gauge]            ,   "Pressure      Gauge" },
        { 0x008D, "975"                             ,   "Analytical    Flame Detector" },
        { 0x008E, "4600"                            ,   "Pressure" },
        { 0x0091, "644 Rail Mount Temp Transmitter" ,   "Temperature" },
        { 0x2601, [pressure_3051b]                  },
        { 0x2602, [temperature_3044]                ,   "Temperature" },
        { 0x2603, [pressure_1151s]                  ,   "Pressure" },
        { 0x2604, [mag_flow_8712]                   ,   "Flow          Magnetic" },
        { 0x2605, [pressure_2001]                   },
        { 0x2606, [pressure_3051]                   ,   "Pressure" },
        { 0x260B, [pressure_3001l]                  },
        { 0x260C, [mag_flow_8712H]                  ,   "Flow          Magnetic" },
        { 0x260D, [temperature_3044C]               ,   "Temperature" },
        { 0x260E, [pressure_3001c]                  ,   "Pressure" },
        { 0x260F, [pressure_3051lp]                 ,   "Pressure" },
        { 0x2610, [vortex_flow_8800]                ,   "Flow          Vortex" },
        { 0x2611, [htg_3201]                        ,   "Level         Hydrostatic" },
        { 0x2612, [pressure_1152s]                  },
        { 0x2613, [htg_3202]                        ,   "Level         Hydrostatic" },
        { 0x2614, [pressure_3001s]                  ,   "Pressure" },
        { 0x2615, [multi_3095FT]                    },
        { 0x2616, [mv_3095]                         ,   "Flow          Multivariable Mass" },
        { 0x2617, [temperature_544]                 },
        { 0x2618, [temperature_644]                 ,   "Temperature" },
        { 0x2619, [temperature_3144]                ,   "Temperature" },
        { 0x261A, [temperature_3244]                ,   "Temperature   Multivariable" },
        { 0x261B, [level_3700]                      },
        { 0x261C, [level_3750]                      ,   "Level" },
        { 0x261D, [TRI_LOOP]                        ,   "Specialty     HART-to-Analog Signal Conv" },
        { 0x261E, [c_3095]                          ,   "Level" },
        { 0x261F, [mm_3095]                         },
        { 0x2620, [tank_3701]                       },
        { 0x2621, [level_3300]                      ,   "Level         Radar" },
        { 0x2622, [asu_3702]                        },
        { 0x2623, [pressure_2088s]                  ,   "Pressure" },
        { 0x2624, [probar]                          ,   "Flow          dP" },
        { 0x2625, [massprobar]                      },
        { 0x2626, [prov]                            },
        { 0x2627, [pressure_2090s]                  ,   "Pressure" },
        { 0x2628, [probar_uc]                       ,   "Flow          dP" },
        { 0x2629, [d_2055]                          },
        { 0x262A, [pv_3095]                         },
        { 0x262D, [sentry]                          },
        { 0x262F, [proplate_uc]                     ,   "Flow          dP" },
        { 0x2636, [pressure_1810]                   ,   "Pressure" },
        { 0x2637, [mag_flow_8712d]                  ,   "Flow          Magnetic" },
        { 0x2638, [pressure_4600]                   ,   "Pressure" },
        { 0x263A, [vortex_flow_8800d]               ,   "Flow          Vortex" },
        { 0x263B, [temperature_248]                 ,   "Temperature" },
        { 0x263C, [mag_flow_8732]                   ,   "Flow          Magnetic" },
        { 0x263D, "8782 Slurry Magnetic Flow Meter" ,   "Flow          Magnetic" },
        { 0x263F, [vortex_flow_8600d]               ,   "Flow          Vortex" },
        { 0x2640, [pressure_2088lp]                 ,   "Pressure      Low Power" },
        { 0x2641, [level_5600]                      },
        { 0x2642, [presssure_951]                   ,   "Pressure" },
        { 0x2643, [level_5400]                      ,   "Level         Radar" },
        { 0x2644, [pressure_3051s_sis]              ,   "Pressure      Certified SIL2" },
        { 0x2645, [temperature_3144p_sis]           ,   "Temperature" },
        { 0x2647, [pressure_4500]                   ,   "Pressure" },
        { 0x2649, [pressure_3051smv_fcmef]          ,   "Flow" },
        { 0x264A, [pressure_3051s_diag]             ,   "Pressure" },
        { 0x264B, [pressure_3051smv_dpvo]           ,   "Pressure" },
        { 0x264E, [wireless_gateway]                },
        { 0x264F, [thum_775_wa]                     },
        { 0x2650, [level_3100]                      ,   "Level         Ultrasonic" },
        { 0x2651, [level_5300]                      ,   "Level         Radar" },
        { 0x2653, [wireless_848t]                   ,   "Temperature   WirelessHART" },
        { 0x2655, [pressure_2051]                   ,   "Pressure" },
        { 0x2656, [pressure_2051lp]                 ,   "Pressure" },
        { 0x2657, [mag_flow_8712e]                  ,   "Flow          Magnetic" },
        { 0x2658, [wireless_648]                    },
        { 0x2659, [wireless_3051s]                  },
        { 0x265A, [wireless_702]                    ,   "Discrete      WirelessHART" },
        { 0x265D, [level_2160]                      ,   "Level         WirelessHART" },
        { 0x265E, "3051S ERS"                       ,   "Pressure" },
        { 0x2662, [metran_150]                      ,   "Pressure" },
        { 0x2664, [wireless_2051]                   },
        { 0x266A, [wireless_3051]                   },
        { 0x266B, [density_3440]                    ,   "Analytical    Density" },
        { 0x266D, [metran_75]                       ,   "Pressure" },
        { 0x266E, [wireless_3308a]                  ,   "Level         Radar - WirelessHART" },
        { 0x266F, [flow_8712em_8732em_hr7]          ,   "Flow" },
        { 0x2670, [pressure_520]                    ,   "Pressure" },
        { 0x2673, [mass_flow_8800d]                 ,   "Flow          Mass" },
        { 0x2676, [wireless_248x]                   ,   "Temperature" },
        { 0x267A, [level_5408_hr7]                  ,   "Level         Radar" },
        { 0x2680, [generic]                         ,   "Interoperable" },
        { 0x2683, [level_5408_hr6]                  ,   "Level         Radar" },
        { 0x2685, "248 Rail Mount Temp Transmitter" ,   "Temperature" },
        { 0x2687, [level_2140]                      ,   "Level" },
        { 0x268A, [smart_pressure_gauge]            ,   "Pressure      Gauge" },
        { 0x268D, "975"                             ,   "Analytical    Flame Detector" },
        { 0x268E, "4600"                            ,   "Pressure" },
        { 0x2691, "644 Rail Mount Temp Transmitter" ,   "Temperature" }
    }
}

/*************************************************************************************
 * Peek Measurement  Device Type Codes
 *     Note: was Sarasota
 */
VARIABLE Peek_Measurement_model_code    /* 0x0027 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0000, [peek900densitometer] ,   "Analytical    Density" },
        { 0x2700, [peek900densitometer] ,   "Analytical    Density" }
    }
}

/*************************************************************************************
 * Actaris Neptune Device Type Codes
 */
VARIABLE Actaris_Neptune_model_code    /* 0x0028 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [nexgen] ,   "Flow          Coriolis Mass" },
        { 0x2801, [nexgen] ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * Siemens Device Type Codes
 */
VARIABLE Siemens_model_code    /* 0x002A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [micro_k]            },
        { 0x0002, [sitrans_l]          },
        { 0x0003, [sipan_ph]           },
        { 0x0004, [sitrans_fus]        ,   "Flow          Ultrasonic" },
        { 0x0005, [sipan_lf]           },
        { 0x0006, [sipan_o2]           },
        { 0x0007, [sitrans_lr]         ,   "Level         Radar" },
        { 0x0008, [km35]               ,   "Pressure" },
        { 0x000A, [sitrans_phs]        ,   "Pressure" },
        { 0x000B, [sitrans_pds]        ,   "Pressure" },
        { 0x000C, [sitrans_pes]        ,   "Pressure" },
        { 0x000D, [sitrans_pms]        ,   "Pressure" },
        { 0x0010, [sitrans_tw]         ,   "Temperature" },
        { 0x0012, [sitrans_tkh]        ,   "Temperature" },
        { 0x0013, [sitrans_th300]      ,   "Temperature" },
        { 0x0015, [sipart_ps2]         ,   "Valve         Positioner/Electropneumatic" },
        { 0x0016, [sitrans_fm]         ,   "Flow          Electromagnetic" },
        { 0x0017, [sitrans_fva]        },
        { 0x0018, [sitrans_fm_magflo]  ,   "Flow          Magnetic Inductive" },
        { 0x0019, [sitrans_fc_massflo] ,   "Flow          Coriolis Mass" },
        { 0x001A, [sitrans_fm_it2_tm2] ,   "Flow" },
        { 0x001D, [sitrans_p300]       ,   "Pressure" },
        { 0x001F, [sitrans_tr300]      ,   "Temperature" },
        { 0x0020, [sitrans_vp300]      ,   "Valve         Positioner" },
        { 0x0022, [sitrans_fc430]      ,   "Flow          Coriolis Mass" },
        { 0x0023, [sitrans_p500]       ,   "Pressure" },
        { 0x0024, [sitrans_lr560]      ,   "Level         Radar" },
        { 0x0028, "SIPART PS100"       ,   "Valve         Actuator" },
        { 0x002D, [sitrans_lu240]      ,   "Level         Ultrasonic" },
        { 0x0034, [sitrans_lut400]     ,   "Level         Ultrasonic" },
        { 0x0037, [sipart_ps2]         ,   "Valve         Positioner/Pneumatic" },
        { 0x003D, [sitrans_p320]       ,   "Pressure" },
        { 0x003E, [sitrans_p420]       ,   "Pressure" },
        { 0x2A01, [micro_k]            },
        { 0x2A02, [sitrans_l]          },
        { 0x2A03, [sipan_ph]           },
        { 0x2A04, [sitrans_fus]        ,   "Flow          Ultrasonic" },
        { 0x2A05, [sipan_lf]           },
        { 0x2A06, [sipan_o2]           },
        { 0x2A07, [sitrans_lr]         ,   "Level         Radar" },
        { 0x2A08, [km35]               ,   "Pressure" },
        { 0x2A0A, [sitrans_phs]        ,   "Pressure" },
        { 0x2A0B, [sitrans_pds]        ,   "Pressure" },
        { 0x2A0C, [sitrans_pes]        ,   "Pressure" },
        { 0x2A0D, [sitrans_pms]        ,   "Pressure" },
        { 0x2A10, [sitrans_tw]         ,   "Temperature" },
        { 0x2A12, [sitrans_tkh]        ,   "Temperature" },
        { 0x2A13, [sitrans_th300]      ,   "Temperature" },
        { 0x2A15, [sipart_ps2]         ,   "Valve         Positioner/Electropneumatic" },
        { 0x2A16, [sitrans_fm]         ,   "Flow          Electromagnetic" },
        { 0x2A17, [sitrans_fva]        },
        { 0x2A18, [sitrans_fm_magflo]  ,   "Flow          Magnetic Inductive" },
        { 0x2A19, [sitrans_fc_massflo] ,   "Flow          Coriolis Mass" },
        { 0x2A1A, [sitrans_fm_it2_tm2] ,   "Flow" },
        { 0x2A1D, [sitrans_p300]       ,   "Pressure" },
        { 0x2A1F, [sitrans_tr300]      ,   "Temperature" },
        { 0x2A20, [sitrans_vp300]      ,   "Valve         Positioner" },
        { 0x2A22, [sitrans_fc430]      ,   "Flow          Coriolis Mass" },
        { 0x2A23, [sitrans_p500]       ,   "Pressure" },
        { 0x2A24, [sitrans_lr560]      ,   "Level         Radar" },
        { 0x2A28, "SIPART PS100"       ,   "Valve         Actuator" },
        { 0x2A2D, [sitrans_lu240]      ,   "Level         Ultrasonic" },
        { 0x2A34, [sitrans_lut400]     ,   "Level         Ultrasonic" },
        { 0x2A37, [sipart_ps2]         ,   "Valve         Positioner/Pneumatic" },
        { 0x2A3D, [sitrans_p320]       ,   "Pressure" },
        { 0x2A3E, [sitrans_p420]       ,   "Pressure" }
    }
}

/*************************************************************************************
 * Toshiba Device Type Codes
 */
VARIABLE Toshiba_model_code    /* 0x002C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0002, [ap3100]    ,   "Pressure" },
        { 0x0003, [ap3110]    },
        { 0x0004, [ap3120]    },
        { 0x0005, [ap3140]    },
        { 0x0006, [ap3150]    },
        { 0x0007, [ap3170]    },
        { 0x0008, [ap3180]    },
        { 0x0009, [ap3190]    },
        { 0x000A, [lf_series] ,   "Flow          Electromagnetic" },
        { 0x2C02, [ap3100]    ,   "Pressure" },
        { 0x2C03, [ap3110]    },
        { 0x2C04, [ap3120]    },
        { 0x2C05, [ap3140]    },
        { 0x2C06, [ap3150]    },
        { 0x2C07, [ap3170]    },
        { 0x2C08, [ap3180]    },
        { 0x2C09, [ap3190]    },
        { 0x2C0A, [lf_series] ,   "Flow          Electromagnetic" }
    }
}

/*************************************************************************************
 * Rosemount Analytical Device Type Codes
 */
VARIABLE rosemount_analytical_model_code    /* 0x002E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0006, [ph_1181]                     ,   "Analytical    pH" },
        { 0x0007, [conductivity_1181]           ,   "Analytical    Conductivity" },
        { 0x000A, [ph_1054]                     },
        { 0x000B, [conductivity_1054]           },
        { 0x000C, [oxymitter_4000]              ,   "Analytical    Oxygen" },
        { 0x000D, [wc_3000]                     ,   "Analytical" },
        { 0x000E, [ra3081fg]                    ,   "Analytical" },
        { 0x000F, [opm_2000r]                   ,   "Analytical    Opacity Monitor" },
        { 0x0010, [ocx_4000]                    ,   "Analytical    Oxygen/Combustion" },
        { 0x0014, [ph_3081]                     ,   "Analytical    pH" },
        { 0x0015, [c_3081]                      ,   "Analytical    2-wire Conductivity" },
        { 0x0016, [ra3081_81con]                ,   "Analytical    2-wire Conductivity w/ pure H20" },
        { 0x0017, [ra3081_81t]                  ,   "Analytical    2-wire Conductivity w/ % conc" },
        { 0x0018, [ocx_cl_oz_5081a]             ,   "Analytical    2-wire Oxygen/Chlorine/Ozone" },
        { 0x0019, [conductivity_5081c_t]        ,   "Analytical    2-wire Conductivity" },
        { 0x001A, [ph_5081]                     ,   "Analytical    2-wire pH/ORP" },
        { 0x001B, [xmt_a]                       ,   "Analytical    2-wire Oxygen/Chlorine/Ozone" },
        { 0x001C, [xmt_c_t]                     ,   "Analytical    2-wire Contacting/Toroidal" },
        { 0x001D, [xmt_ph]                      ,   "Analytical    2-wire pH/ORP" },
        { 0x001E, [oxygen_5081fg]               ,   "Analytical    Oxygen - High Temperature" },
        { 0x001F, [ph_orp_6081_p]               },
        { 0x0020, [conductivity_6081c_wireless] ,   "Analytical    Conductivity" },
        { 0x0021, [ra1066]                      ,   "Analytical    Multiparameter" },
        { 0x002B, [xs_o2]                       ,   "Analytical    Oxygen" },
        { 0x003A, [o2_6888]                     ,   "Analytical    Oxygen" },
        { 0x0050, [ph_orp_54]                   ,   "Analytical    pH/ORP" },
        { 0x0051, [c_54]                        ,   "Analytical    Conductivity w/ % conc" },
        { 0x0052, [ra52eph_orp]                 ,   "Analytical    pH/ORP/ISE" },
        { 0x0053, [ra54ea]                      ,   "Analytical    Amperometric Analyzer" },
        { 0x0055, [di_analyzer_1056]            ,   "Analytical    Dual Input Analyzer" },
        { 0x0056, [model_56]                    ,   "Analytical    Dual Input Analyzer" },
        { 0x2E06, [ph_1181]                     ,   "Analytical    pH" },
        { 0x2E07, [conductivity_1181]           ,   "Analytical    Conductivity" },
        { 0x2E0A, [ph_1054]                     },
        { 0x2E0B, [conductivity_1054]           },
        { 0x2E0C, [oxymitter_4000]              ,   "Analytical    Oxygen" },
        { 0x2E0D, [wc_3000]                     ,   "Analytical" },
        { 0x2E0E, [ra3081fg]                    ,   "Analytical" },
        { 0x2E0F, [opm_2000r]                   ,   "Analytical    Opacity Monitor" },
        { 0x2E10, [ocx_4000]                    ,   "Analytical    Oxygen/Combustion" },
        { 0x2E14, [ph_3081]                     ,   "Analytical    pH" },
        { 0x2E15, [c_3081]                      ,   "Analytical    2-wire Conductivity" },
        { 0x2E16, [ra3081_81con]                ,   "Analytical    2-wire Conductivity w/ pure H20" },
        { 0x2E17, [ra3081_81t]                  ,   "Analytical    2-wire Conductivity w/ % conc" },
        { 0x2E18, [ocx_cl_oz_5081a]             ,   "Analytical    2-wire Oxygen/Chlorine/Ozone" },
        { 0x2E19, [conductivity_5081c_t]        ,   "Analytical    2-wire Conductivity" },
        { 0x2E1A, [ph_5081]                     ,   "Analytical    2-wire pH/ORP" },
        { 0x2E1B, [xmt_a]                       ,   "Analytical    2-wire Oxygen/Chlorine/Ozone" },
        { 0x2E1C, [xmt_c_t]                     ,   "Analytical    2-wire Contacting/Toroidal" },
        { 0x2E1D, [xmt_ph]                      ,   "Analytical    2-wire pH/ORP" },
        { 0x2E1E, [oxygen_5081fg]               ,   "Analytical    Oxygen - High Temperature" },
        { 0x2E1F, [ph_orp_6081_p]               },
        { 0x2E20, [conductivity_6081c_wireless] ,   "Analytical    Conductivity" },
        { 0x2E21, [ra1066]                      ,   "Analytical    Multiparameter" },
        { 0x2E2B, [xs_o2]                       ,   "Analytical    Oxygen" },
        { 0x2E3A, [o2_6888]                     ,   "Analytical    Oxygen" },
        { 0x2E50, [ph_orp_54]                   ,   "Analytical    pH/ORP" },
        { 0x2E51, [c_54]                        ,   "Analytical    Conductivity w/ % conc" },
        { 0x2E52, [ra52eph_orp]                 ,   "Analytical    pH/ORP/ISE" },
        { 0x2E53, [ra54ea]                      ,   "Analytical    Amperometric Analyzer" },
        { 0x2E55, [di_analyzer_1056]            ,   "Analytical    Dual Input Analyzer" },
        { 0x2E56, [model_56]                    ,   "Analytical    Dual Input Analyzer" }
    }
}

/*************************************************************************************
 * Metso Flow Control Inc Device Type Codes
 *     Note: was Valmet Automation
 */
VARIABLE Valmet_Automation_model_code    /* 0x002F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [psmart]    ,   "Pressure      Pressure/Level" },
        { 0x003C, [smartpulp] ,   "Analytical    Consistency" },
        { 0x003D, [smartlx]   ,   "Analytical    Consistency" },
        { 0x003E, [mcai]      ,   "Analytical    Microwave-Pulp Consistency" },
        { 0x003F, [smartlc]   ,   "Analytical    Consistency" },
        { 0x0040, [mca]       ,   "Analytical    Microwave Consistency" },
        { 0x2F01, [psmart]    ,   "Pressure      Pressure/Level" },
        { 0x2F3C, [smartpulp] ,   "Analytical    Consistency" },
        { 0x2F3D, [smartlx]   ,   "Analytical    Consistency" },
        { 0x2F3E, [mcai]      ,   "Analytical    Microwave-Pulp Consistency" },
        { 0x2F3F, [smartlc]   ,   "Analytical    Consistency" },
        { 0x2F40, [mca]       ,   "Analytical    Microwave Consistency" }
    }
}

/*************************************************************************************
 * Flowserve Device Type Codes
 */
VARIABLE Flowserve_model_code    /* 0x0030 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [logix_12xx]       ,   "Valve         Digital Positioner" },
        { 0x0002, [kemmer_c2100]     ,   "Valve         Positioner" },
        { 0x0003, [logix_520]        ,   "Valve         Digital Positioner" },
        { 0x0004, [Logix_3200iq]     ,   "Valve         Digital Positioner" },
        { 0x0005, [logix_3200md]     ,   "Valve         Positioner" },
        { 0x0006, [logix_520md]      ,   "Valve         Positioner" },
        { 0x0007, [logix_md_plus]    ,   "Valve         Positioner" },
        { 0x0008, [limitorque_mx_qx] ,   "Valve         Actuator - Electrical" },
        { 0x0009, [logix_420]        ,   "Valve         Positioner" },
        { 0x000A, [logix_3820]       ,   "Valve         Positioner" },
        { 0x000B, "Logix 3820e"      ,   "Valve         Positioner" },
        { 0x3001, [logix_12xx]       ,   "Valve         Digital Positioner" },
        { 0x3002, [kemmer_c2100]     ,   "Valve         Positioner" },
        { 0x3003, [logix_520]        ,   "Valve         Digital Positioner" },
        { 0x3004, [Logix_3200iq]     ,   "Valve         Digital Positioner" },
        { 0x3005, [logix_3200md]     ,   "Valve         Positioner" },
        { 0x3006, [logix_520md]      ,   "Valve         Positioner" },
        { 0x3007, [logix_md_plus]    ,   "Valve         Positioner" },
        { 0x3008, [limitorque_mx_qx] ,   "Valve         Actuator - Electrical" },
        { 0x3009, [logix_420]        ,   "Valve         Positioner" },
        { 0x300A, [logix_3820]       ,   "Valve         Positioner" },
        { 0x300B, "Logix 3820e"      ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Viatran Device Type Codes
 */
VARIABLE Viatran_model_code    /* 0x0032 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [viatran970]          ,   "Pressure" },
        { 0x0002, [trim]                ,   "Pressure" },
        { 0x002E, [viatran_ia_pressure] ,   "Pressure" },
        { 0x3201, [viatran970]          ,   "Pressure" },
        { 0x3202, [trim]                ,   "Pressure" },
        { 0x322E, [viatran_ia_pressure] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Azbil Corporation Device Type Codes
 *     Note: was Yamatake Corporation
 */
VARIABLE Azbil_Corporation_model_code    /* 0x0036 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [magnew]     ,   "Flow          Magnetic" },
        { 0x0002, [st3000]     ,   "Pressure" },
        { 0x0003, [svp]        ,   "Valve         Smart Valve Positioner" },
        { 0x0004, [thermoplus] ,   "Temperature" },
        { 0x0005, [ptg]        ,   "Pressure" },
        { 0x0008, [magnew_2w]  ,   "Flow          Electromagnetic" },
        { 0x000C, [magnew4w]   ,   "Flow          Magnetic" },
        { 0x000D, [at9000]     ,   "Pressure" },
        { 0x000E, [gascvd]     ,   "Analytical    Natural Gas Calorimeter" },
        { 0x000F, [avp700]     ,   "Valve         Positioner" },
        { 0x0010, [avp700_sis] ,   "Valve         Positioner" },
        { 0x0011, [avp307]     ,   "Valve         Positioner" },
        { 0x0014, [svp_v2]     ,   "Valve         Positioner" },
        { 0x0015, [att082]     ,   "Temperature" },
        { 0x0016, [att162]     ,   "Temperature" },
        { 0x3601, [magnew]     ,   "Flow          Magnetic" },
        { 0x3602, [st3000]     ,   "Pressure" },
        { 0x3603, [svp]        ,   "Valve         Smart Valve Positioner" },
        { 0x3604, [thermoplus] ,   "Temperature" },
        { 0x3605, [ptg]        ,   "Pressure" },
        { 0x3608, [magnew_2w]  ,   "Flow          Electromagnetic" },
        { 0x360C, [magnew4w]   ,   "Flow          Magnetic" },
        { 0x360D, [at9000]     ,   "Pressure" },
        { 0x360E, [gascvd]     ,   "Analytical    Natural Gas Calorimeter" },
        { 0x360F, [avp700]     ,   "Valve         Positioner" },
        { 0x3610, [avp700_sis] ,   "Valve         Positioner" },
        { 0x3611, [avp307]     ,   "Valve         Positioner" },
        { 0x3614, [svp_v2]     ,   "Valve         Positioner" },
        { 0x3615, [att082]     ,   "Temperature" },
        { 0x3616, [att162]     ,   "Temperature" }
    }
}

/*************************************************************************************
 * Yokogawa Device Type Codes
 */
VARIABLE Yokogawa_model_code    /* 0x0037 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [yewflo]         ,   "Flow          Vortex" },
        { 0x0002, [yt200]          ,   "Temperature   Differential" },
        { 0x0003, [unicom]         ,   "Pressure" },
        { 0x0004, [eja]            ,   "Pressure      Differential" },
        { 0x0005, [admag_ae]       ,   "Flow          Magnetic" },
        { 0x0006, [am11]           ,   "Flow" },
        { 0x0008, [admag_se]       ,   "Flow" },
        { 0x0009, [yta]            ,   "Temperature" },
        { 0x000A, [yta70e]         ,   "Temperature" },
        { 0x000B, [dyf]            ,   "Flow          Vortex" },
        { 0x000C, [zr202]          ,   "Analytical    Oxygen" },
        { 0x000D, [zr402]          ,   "Analytical    Oxygen" },
        { 0x0011, [yta610]         ,   "Temperature" },
        { 0x0012, [flxa402]        ,   "Analytical    Multivariable" },
        { 0x0014, [isc202]         ,   "Analytical    Conductivity - Inductive" },
        { 0x0015, [ph202]          ,   "Analytical    pH" },
        { 0x0016, [sc202]          ,   "Analytical    Conductivity" },
        { 0x0018, [do202]          ,   "Analytical    Dissolved Oxygen" },
        { 0x001A, [axg4a]          ,   "Flow          Magnetic" },
        { 0x001B, [axw4a]          ,   "Flow          Magnetic" },
        { 0x001C, [axg1a]          ,   "Flow" },
        { 0x0020, "SF14A"          ,   "Flow" },
        { 0x0021, "ZR802"          ,   "Analytical    Oxygen Analyzer" },
        { 0x0027, [ph150]          ,   "Analytical    pH Controller" },
        { 0x0028, [sc150]          ,   "Analytical    Conductivity/Resistivity Controller" },
        { 0x002B, [ph450]          ,   "Analytical    pH" },
        { 0x002C, [sc450]          ,   "Analytical    Conductivity/Resistivity" },
        { 0x002D, [isc450]         ,   "Analytical    Conductivity - Inductive" },
        { 0x003C, [yta70e_z]       ,   "Temperature" },
        { 0x003D, [yta70_j]        ,   "Temperature" },
        { 0x003E, [tdls8000]       ,   "Analytical    Gas Detector" },
        { 0x003F, [yta70p]         ,   "Temperature" },
        { 0x0040, [rotamass]       ,   "Flow          Mass" },
        { 0x0041, [ramc]           ,   "Flow          Variable Area" },
        { 0x0042, [rcct_f3]        ,   "Flow          Mass" },
        { 0x0050, [axfa11]         ,   "Flow          Magnetic" },
        { 0x0051, [ejx]            ,   "Pressure" },
        { 0x0052, [axfa14]         ,   "Flow          Magnetic" },
        { 0x0053, [av550g]         ,   "Analytical    Oxygen" },
        { 0x0054, [ejx_mv]         ,   "Pressure      Multivariable" },
        { 0x0055, [ejx_drs]        ,   "Pressure      Differential" },
        { 0x0057, [axr]            ,   "Flow          Electromagnetic" },
        { 0x0058, [flxa21_ph]      ,   "Analytical    pH/ORP" },
        { 0x0059, [flxa21_202_sc]  ,   "Analytical    Conductivity" },
        { 0x005A, [flxa21_202_isc] ,   "Analytical    Conductivity" },
        { 0x005B, [flxa21_do]      ,   "Analytical    Dissolved Oxygen" },
        { 0x005C, [eja_next]       ,   "Pressure      Differential" },
        { 0x005D, [eja_next_lp]    ,   "Pressure      Differential" },
        { 0x005F, [yta710]         ,   "Temperature" },
        { 0x0060, [rotamass_ti]    ,   "Flow          Coriolis Mass" },
        { 0x3701, [yewflo]         ,   "Flow          Vortex" },
        { 0x3702, [yt200]          ,   "Temperature   Differential" },
        { 0x3703, [unicom]         ,   "Pressure" },
        { 0x3704, [eja]            ,   "Pressure      Differential" },
        { 0x3705, [admag_ae]       ,   "Flow          Magnetic" },
        { 0x3706, [am11]           ,   "Flow" },
        { 0x3708, [admag_se]       ,   "Flow" },
        { 0x3709, [yta]            ,   "Temperature" },
        { 0x370A, [yta70e]         ,   "Temperature" },
        { 0x370B, [dyf]            ,   "Flow          Vortex" },
        { 0x370C, [zr202]          ,   "Analytical    Oxygen" },
        { 0x370D, [zr402]          ,   "Analytical    Oxygen" },
        { 0x3711, [yta610]         ,   "Temperature" },
        { 0x3712, [flxa402]        ,   "Analytical    Multivariable" },
        { 0x3714, [isc202]         ,   "Analytical    Conductivity - Inductive" },
        { 0x3715, [ph202]          ,   "Analytical    pH" },
        { 0x3716, [sc202]          ,   "Analytical    Conductivity" },
        { 0x3718, [do202]          ,   "Analytical    Dissolved Oxygen" },
        { 0x371A, [axg4a]          ,   "Flow          Magnetic" },
        { 0x371B, [axw4a]          ,   "Flow          Magnetic" },
        { 0x371C, [axg1a]          ,   "Flow" },
        { 0x3720, "SF14A"          ,   "Flow" },
        { 0x3721, "ZR802"          ,   "Analytical    Oxygen Analyzer" },
        { 0x3727, [ph150]          ,   "Analytical    pH Controller" },
        { 0x3728, [sc150]          ,   "Analytical    Conductivity/Resistivity Controller" },
        { 0x372B, [ph450]          ,   "Analytical    pH" },
        { 0x372C, [sc450]          ,   "Analytical    Conductivity/Resistivity" },
        { 0x372D, [isc450]         ,   "Analytical    Conductivity - Inductive" },
        { 0x373C, [yta70e_z]       ,   "Temperature" },
        { 0x373D, [yta70_j]        ,   "Temperature" },
        { 0x373E, [tdls8000]       ,   "Analytical    Gas Detector" },
        { 0x373F, [yta70p]         ,   "Temperature" },
        { 0x3740, [rotamass]       ,   "Flow          Mass" },
        { 0x3741, [ramc]           ,   "Flow          Variable Area" },
        { 0x3742, [rcct_f3]        ,   "Flow          Mass" },
        { 0x3750, [axfa11]         ,   "Flow          Magnetic" },
        { 0x3751, [ejx]            ,   "Pressure" },
        { 0x3752, [axfa14]         ,   "Flow          Magnetic" },
        { 0x3753, [av550g]         ,   "Analytical    Oxygen" },
        { 0x3754, [ejx_mv]         ,   "Pressure      Multivariable" },
        { 0x3755, [ejx_drs]        ,   "Pressure      Differential" },
        { 0x3757, [axr]            ,   "Flow          Electromagnetic" },
        { 0x3758, [flxa21_ph]      ,   "Analytical    pH/ORP" },
        { 0x3759, [flxa21_202_sc]  ,   "Analytical    Conductivity" },
        { 0x375A, [flxa21_202_isc] ,   "Analytical    Conductivity" },
        { 0x375B, [flxa21_do]      ,   "Analytical    Dissolved Oxygen" },
        { 0x375C, [eja_next]       ,   "Pressure      Differential" },
        { 0x375D, [eja_next_lp]    ,   "Pressure      Differential" },
        { 0x375F, [yta710]         ,   "Temperature" },
        { 0x3760, [rotamass_ti]    ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * Mobrey Device Type Codes
 *     Note: was Solartron Mobrey
 */
VARIABLE Solartron_Mobrey_model_code    /* 0x003B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0013, [solartron4301] ,   "Pressure      w/ PID" },
        { 0x0014, [solartron3301] },
        { 0x0015, [msp100]        ,   "Level" },
        { 0x0021, [level_3300]    ,   "Level         Radar" },
        { 0x0029, [mlt100]        ,   "Level         Displacement" },
        { 0x002A, [solartron4291] },
        { 0x002B, [msm400]        ,   "Analytical    Density" },
        { 0x002C, [msl600]        },
        { 0x002D, [mcu900]        },
        { 0x002E, [msp900_400]    ,   "Level         Ultrasonic" },
        { 0x002F, [mrl700]        ,   "Level         Radar" },
        { 0x0030, [mrl800]        },
        { 0x0031, [mrl850]        },
        { 0x3B13, [solartron4301] ,   "Pressure      w/ PID" },
        { 0x3B14, [solartron3301] },
        { 0x3B15, [msp100]        ,   "Level" },
        { 0x3B21, [level_3300]    ,   "Level         Radar" },
        { 0x3B29, [mlt100]        ,   "Level         Displacement" },
        { 0x3B2A, [solartron4291] },
        { 0x3B2B, [msm400]        ,   "Analytical    Density" },
        { 0x3B2C, [msl600]        },
        { 0x3B2D, [mcu900]        },
        { 0x3B2E, [msp900_400]    ,   "Level         Ultrasonic" },
        { 0x3B2F, [mrl700]        ,   "Level         Radar" },
        { 0x3B30, [mrl800]        },
        { 0x3B31, [mrl850]        }
    }
}

/*************************************************************************************
 * Princo Device Type Codes
 */
VARIABLE Princo_model_code    /* 0x003D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [l4610] },
        { 0x3D01, [l4610] }
    }
}

/*************************************************************************************
 * Smar Device Type Codes
 */
VARIABLE Smar_model_code    /* 0x003E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [ld301] ,   "Pressure      w/ PID" },
        { 0x0002, [tt301] ,   "Temperature   w/ PID" },
        { 0x0003, [fy301] ,   "Valve         Positioner/Electropneumatic" },
        { 0x0004, [ld291] ,   "Pressure" },
        { 0x0005, [tp301] ,   "Valve         Positioner" },
        { 0x0006, [dt301] ,   "Analytical    Density" },
        { 0x0009, [tt400] ,   "Temperature" },
        { 0x000A, [ld400] ,   "Pressure" },
        { 0x3E01, [ld301] ,   "Pressure      w/ PID" },
        { 0x3E02, [tt301] ,   "Temperature   w/ PID" },
        { 0x3E03, [fy301] ,   "Valve         Positioner/Electropneumatic" },
        { 0x3E04, [ld291] ,   "Pressure" },
        { 0x3E05, [tp301] ,   "Valve         Positioner" },
        { 0x3E06, [dt301] ,   "Analytical    Density" },
        { 0x3E09, [tt400] ,   "Temperature" },
        { 0x3E0A, [ld400] ,   "Pressure" }
    }
}

/*************************************************************************************
 * SchneiderElectric/Eckardt Device Type Codes
 *     Note: was Foxboro Eckardt 
 */
VARIABLE SchneiderElectric_Eckardt_model_code    /* 0x003F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [tsv175]    ,   "Temperature" },
        { 0x0002, [dmu130]    ,   "Pressure" },
        { 0x0003, [ti_rtt20]  ,   "Temperature" },
        { 0x0004, [srd991]    ,   "Valve         Positioner" },
        { 0x0005, [dmu140]    ,   "Pressure" },
        { 0x0006, [srd960]    ,   "Valve         Universal Positioner" },
        { 0x0007, [eck_244ld] ,   "Level" },
        { 0x0008, [srd998]    ,   "Valve         Positioner" },
        { 0x00D0, [lr01]      ,   "Level         Radar" },
        { 0x00D7, [lg01]      ,   "Level" },
        { 0x3F01, [tsv175]    ,   "Temperature" },
        { 0x3F02, [dmu130]    ,   "Pressure" },
        { 0x3F03, [ti_rtt20]  ,   "Temperature" },
        { 0x3F04, [srd991]    ,   "Valve         Positioner" },
        { 0x3F05, [dmu140]    ,   "Pressure" },
        { 0x3F06, [srd960]    ,   "Valve         Universal Positioner" },
        { 0x3F07, [eck_244ld] ,   "Level" },
        { 0x3F08, [srd998]    ,   "Valve         Positioner" },
        { 0x3FD0, [lr01]      ,   "Level         Radar" },
        { 0x3FD7, [lg01]      ,   "Level" }
    }
}

/*************************************************************************************
 * Measurement Technology Device Type Codes
 */
VARIABLE Measurement_Technology_model_code    /* 0x0040 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0015, [mtl4841] },
        { 0x0022, [mtl8512] },
        { 0x4015, [mtl4841] },
        { 0x4022, [mtl8512] }
    }
}

/*************************************************************************************
 * Samson Device Type Codes
 */
VARIABLE Samson_model_code    /* 0x0042 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EB, [trovis_3730_3]    ,   "Valve         Positioner" },
        { 0x00EC, [trovis_safe_3793] ,   "Valve         Positioner/Electropneumatic" },
        { 0x00ED, [trovis_3793]      ,   "Valve         Positioner" },
        { 0x00EE, [samson373x_6]     ,   "Valve         Positioner" },
        { 0x00EF, [samson373x_3]     ,   "Valve         Positioner" },
        { 0x00F9, [samson3780]       ,   "Valve         Controller" },
        { 0x42EB, [trovis_3730_3]    ,   "Valve         Positioner" },
        { 0x42EC, [trovis_safe_3793] ,   "Valve         Positioner/Electropneumatic" },
        { 0x42ED, [trovis_3793]      ,   "Valve         Positioner" },
        { 0x42EE, [samson373x_6]     ,   "Valve         Positioner" },
        { 0x42EF, [samson373x_3]     ,   "Valve         Positioner" },
        { 0x42F9, [samson3780]       ,   "Valve         Controller" }
    }
}

/*************************************************************************************
 * Sparling Instruments Device Type Codes
 */
VARIABLE Sparling_Instruments_model_code    /* 0x0043 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EE, [fm6xx] ,   "Flow          Magnetic" },
        { 0x43EE, [fm6xx] ,   "Flow          Magnetic" }
    }
}

/*************************************************************************************
 * Krohne Device Type Codes
 */
VARIABLE Krohne_model_code    /* 0x0045 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00AD, "OPTISYS IND 8100"             ,   "Analytical    Conductivity - Inductive" },
        { 0x00AE, [optiflex_3200_6200_7200_8200] ,   "Level         Radar" },
        { 0x00AF, "UFC300 V2"                    ,   "Flow          Ultrasonic" },
        { 0x00B9, [optiwave_x400_x500]           ,   "Level         Radar" },
        { 0x00BA, [esk3x]                        ,   "Flow          Variable Area" },
        { 0x00BB, [mfc_400_s]                    ,   "Flow          Coriolis Mass" },
        { 0x00BC, [optibar_5060_sil]             ,   "Pressure" },
        { 0x00BD, [optibar_7060_sil]             ,   "Pressure      Differential" },
        { 0x00BE, [esk4a]                        ,   "Flow          Variable Area" },
        { 0x00BF, [optiwave_1010]                ,   "Level         Radar" },
        { 0x00C2, [smartpat_cond]                ,   "Analytical    Conductivity" },
        { 0x00C3, [smartsens_orp]                ,   "Analytical    ORP" },
        { 0x00C4, [optibar_5060]                 ,   "Pressure" },
        { 0x00C5, [optibar_dp_7060_c]            ,   "Pressure      Differential" },
        { 0x00C6, [smartsens_ph]                 ,   "Analytical    pH" },
        { 0x00C7, [tt53_cr_ex]                   ,   "Temperature" },
        { 0x00C8, [optibar_dp_70xx_f]            },
        { 0x00C9, [optibar_p_50_60xx_f]          },
        { 0x00CA, [tfc_100]                      },
        { 0x00CB, [ifc_050]                      },
        { 0x00CC, [mfc_400]                      ,   "Flow          Coriolis Mass" },
        { 0x00CD, "VFC 200"                      ,   "Flow          Vortex" },
        { 0x00CE, [optilevel_1100]               },
        { 0x00CF, [mac100]                       ,   "Analytical    Multivariable" },
        { 0x00D0, [optiwave_5200_cf]             ,   "Level         Radar" },
        { 0x00D1, [optibar_p_x050_c]             },
        { 0x00D2, [ufc400]                       ,   "Flow          Ultrasonic" },
        { 0x00D3, [tt51cr]                       ,   "Temperature" },
        { 0x00D4, [gfc_300_steam]                },
        { 0x00D5, [gfc_300_gas]                  },
        { 0x00D6, [esk4]                         ,   "Flow          Variable Area" },
        { 0x00D7, [optiflex_2200]                ,   "Level         Radar" },
        { 0x00D9, [ifc100]                       ,   "Flow          Electromagnetic" },
        { 0x00DB, [tt50cr]                       ,   "Temperature" },
        { 0x00DD, [mfc300]                       ,   "Flow          Coriolis Mass" },
        { 0x00DE, [vfc070]                       ,   "Flow          Vortex" },
        { 0x00E2, [esk2a]                        ,   "Flow          Variable Area" },
        { 0x00E3, [ifc300]                       ,   "Flow          Electromagnetic" },
        { 0x00E4, [optiflex1300c]                ,   "Level         Radar" },
        { 0x00E5, [optiwave7300c]                ,   "Level         Radar" },
        { 0x00E6, [m8e]                          ,   "Flow          Variable Area" },
        { 0x00E7, [ufc030]                       ,   "Flow          Ultrasonic" },
        { 0x00E8, [mfc05x]                       ,   "Flow          Coriolis Mass" },
        { 0x00E9, [ifc040]                       ,   "Flow          Electromagnetic" },
        { 0x00EA, [m10]                          ,   "Flow          Variable Area" },
        { 0x00EB, [ifc210]                       },
        { 0x00EC, [bm102]                        ,   "Level         Radar - Reflex" },
        { 0x00ED, [vfm31]                        ,   "Flow          Vortex" },
        { 0x00EE, [bm100]                        ,   "Level         Radar - Reflex" },
        { 0x00EF, [ifc020]                       },
        { 0x00F0, [vfm1091]                      },
        { 0x00F2, [eskii]                        ,   "Flow          Variable Area" },
        { 0x00F3, [ifc110]                       ,   "Flow          Magnetic" },
        { 0x00F4, [ifc090]                       ,   "Flow          Magnetic" },
        { 0x00F5, [ufc500]                       ,   "Flow          Ultrasonic" },
        { 0x00F6, [ifc010]                       },
        { 0x00F7, [mfc08x]                       ,   "Flow          Coriolis Mass" },
        { 0x00F8, [ifc080]                       ,   "Flow          Magnetic" },
        { 0x00F9, [bm70]                         ,   "Level         Radar" },
        { 0x45AD, "OPTISYS IND 8100"             ,   "Analytical    Conductivity - Inductive" },
        { 0x45AE, [optiflex_3200_6200_7200_8200] ,   "Level         Radar" },
        { 0x45AF, "UFC300 V2"                    ,   "Flow          Ultrasonic" },
        { 0x45B9, [optiwave_x400_x500]           ,   "Level         Radar" },
        { 0x45BA, [esk3x]                        ,   "Flow          Variable Area" },
        { 0x45BB, [mfc_400_s]                    ,   "Flow          Coriolis Mass" },
        { 0x45BC, [optibar_5060_sil]             ,   "Pressure" },
        { 0x45BD, [optibar_7060_sil]             ,   "Pressure      Differential" },
        { 0x45BE, [esk4a]                        ,   "Flow          Variable Area" },
        { 0x45BF, [optiwave_1010]                ,   "Level         Radar" },
        { 0x45C2, [smartpat_cond]                ,   "Analytical    Conductivity" },
        { 0x45C3, [smartsens_orp]                ,   "Analytical    ORP" },
        { 0x45C4, [optibar_5060]                 ,   "Pressure" },
        { 0x45C5, [optibar_dp_7060_c]            ,   "Pressure      Differential" },
        { 0x45C6, [smartsens_ph]                 ,   "Analytical    pH" },
        { 0x45C7, [tt53_cr_ex]                   ,   "Temperature" },
        { 0x45C8, [optibar_dp_70xx_f]            },
        { 0x45C9, [optibar_p_50_60xx_f]          },
        { 0x45CA, [tfc_100]                      },
        { 0x45CB, [ifc_050]                      },
        { 0x45CC, [mfc_400]                      ,   "Flow          Coriolis Mass" },
        { 0x45CD, "VFC 200"                      ,   "Flow          Vortex" },
        { 0x45CE, [optilevel_1100]               },
        { 0x45CF, [mac100]                       ,   "Analytical    Multivariable" },
        { 0x45D0, [optiwave_5200_cf]             ,   "Level         Radar" },
        { 0x45D1, [optibar_p_x050_c]             },
        { 0x45D2, [ufc400]                       ,   "Flow          Ultrasonic" },
        { 0x45D3, [tt51cr]                       ,   "Temperature" },
        { 0x45D4, [gfc_300_steam]                },
        { 0x45D5, [gfc_300_gas]                  },
        { 0x45D6, [esk4]                         ,   "Flow          Variable Area" },
        { 0x45D7, [optiflex_2200]                ,   "Level         Radar" },
        { 0x45D9, [ifc100]                       ,   "Flow          Electromagnetic" },
        { 0x45DB, [tt50cr]                       ,   "Temperature" },
        { 0x45DD, [mfc300]                       ,   "Flow          Coriolis Mass" },
        { 0x45DE, [vfc070]                       ,   "Flow          Vortex" },
        { 0x45E2, [esk2a]                        ,   "Flow          Variable Area" },
        { 0x45E3, [ifc300]                       ,   "Flow          Electromagnetic" },
        { 0x45E4, [optiflex1300c]                ,   "Level         Radar" },
        { 0x45E5, [optiwave7300c]                ,   "Level         Radar" },
        { 0x45E6, [m8e]                          ,   "Flow          Variable Area" },
        { 0x45E7, [ufc030]                       ,   "Flow          Ultrasonic" },
        { 0x45E8, [mfc05x]                       ,   "Flow          Coriolis Mass" },
        { 0x45E9, [ifc040]                       ,   "Flow          Electromagnetic" },
        { 0x45EA, [m10]                          ,   "Flow          Variable Area" },
        { 0x45EB, [ifc210]                       },
        { 0x45EC, [bm102]                        ,   "Level         Radar - Reflex" },
        { 0x45ED, [vfm31]                        ,   "Flow          Vortex" },
        { 0x45EE, [bm100]                        ,   "Level         Radar - Reflex" },
        { 0x45EF, [ifc020]                       },
        { 0x45F0, [vfm1091]                      },
        { 0x45F2, [eskii]                        ,   "Flow          Variable Area" },
        { 0x45F3, [ifc110]                       ,   "Flow          Magnetic" },
        { 0x45F4, [ifc090]                       ,   "Flow          Magnetic" },
        { 0x45F5, [ufc500]                       ,   "Flow          Ultrasonic" },
        { 0x45F6, [ifc010]                       },
        { 0x45F7, [mfc08x]                       ,   "Flow          Coriolis Mass" },
        { 0x45F8, [ifc080]                       ,   "Flow          Magnetic" },
        { 0x45F9, [bm70]                         ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * Druck Device Type Codes
 */
VARIABLE Druck_model_code    /* 0x0047 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00E9, [rtx1000h_gauge] ,   "Pressure      Gauge" },
        { 0x00EC, [rtx1000h_sg]    ,   "Pressure      Sealed Gauge" },
        { 0x00EF, [rtx1000h_abs]   ,   "Pressure      Absolute" },
        { 0x47E9, [rtx1000h_gauge] ,   "Pressure      Gauge" },
        { 0x47EC, [rtx1000h_sg]    ,   "Pressure      Sealed Gauge" },
        { 0x47EF, [rtx1000h_abs]   ,   "Pressure      Absolute" }
    }
}

/*************************************************************************************
 * SOR Device Type Codes
 */
VARIABLE SOR_model_code    /* 0x0048 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0000, [series_815] ,   "Pressure" },
        { 0x4800, [series_815] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Elcon Instruments Device Type Codes
 */
VARIABLE Elcon_Instruments_model_code    /* 0x0049 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EB, [ao895]      },
        { 0x00EC, [ai895]      },
        { 0x00ED, [mux_2700_f] },
        { 0x00EE, [mux_2700_e] },
        { 0x49EB, [ao895]      },
        { 0x49EC, [ai895]      },
        { 0x49ED, [mux_2700_f] },
        { 0x49EE, [mux_2700_e] }
    }
}

/*************************************************************************************
 * Westlock Controls Device Type Codes
 */
VARIABLE Westlock_Controls_model_code    /* 0x004D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [icot]              ,   "Valve         Digital Positioner w/ PID" },
        { 0x0002, [smartcal]          ,   "Valve         Intelligent Positioner" },
        { 0x0003, [d_epic]            ,   "Valve         Digital Positioner" },
        { 0x0004, [d_epic_esd]        ,   "Valve         Digital Positioner" },
        { 0x0005, [digital_epic_2]    ,   "Valve         Positioner Transmitter" },
        { 0x0006, "Digital Epic D200" ,   "Valve         Positioner Transmitter" },
        { 0x4D01, [icot]              ,   "Valve         Digital Positioner w/ PID" },
        { 0x4D02, [smartcal]          ,   "Valve         Intelligent Positioner" },
        { 0x4D03, [d_epic]            ,   "Valve         Digital Positioner" },
        { 0x4D04, [d_epic_esd]        ,   "Valve         Digital Positioner" },
        { 0x4D05, [digital_epic_2]    ,   "Valve         Positioner Transmitter" },
        { 0x4D06, "Digital Epic D200" ,   "Valve         Positioner Transmitter" }
    }
}

/*********************************************************************************
 * AMETEK Drexelbrook Device Type Codes
 *     Note: Drexelbrook
 */
VARIABLE Drexelbrook_model_code    /* 0x004E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0002, [universal_iii]       ,   "Level" },
        { 0x00EA, [radar_drx400_drx500] ,   "Level         Radar" },
        { 0x00EB, [usonic]              ,   "Level         Ultrasonic" },
        { 0x00EC, [dr2000]              ,   "Level         Radar" },
        { 0x00ED, [dr5200]              ,   "Level         Radar" },
        { 0x4E02, [universal_iii]       ,   "Level" },
        { 0x4EEA, [radar_drx400_drx500] ,   "Level         Radar" },
        { 0x4EEB, [usonic]              ,   "Level         Ultrasonic" },
        { 0x4EEC, [dr2000]              ,   "Level         Radar" },
        { 0x4EED, [dr5200]              ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * Rosemount Tank Radar Device Type Codes
 *     Note: was Saab Tank Control
 */
VARIABLE Rosemount_Tank_Radar_model_code    /* 0x004F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00ED, [saab_5600]    ,   "Level         Radar" },
        { 0x00EF, [tankradarpro] ,   "Level         Radar" },
        { 0x4FED, [saab_5600]    ,   "Level         Radar" },
        { 0x4FEF, [tankradarpro] ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * K-TEK Device Type Codes
 */
VARIABLE KTEK_model_code    /* 0x0050 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0067, [spm100]           },
        { 0x0072, [at100_vol]        ,   "Level         Magnetorestrictive" },
        { 0x0073, [at100_2_vol]      ,   "Level         Magnetorestrictive" },
        { 0x0076, [at100_temp_vol]   ,   "Level         Magnetorestrictive" },
        { 0x0077, [at100_2_temp_vol] ,   "Level         Magnetorestrictive" },
        { 0x0078, [at100]            ,   "Level         Magnetorestrictive" },
        { 0x007A, [at100_lcd]        ,   "Level         Magnetorestrictive" },
        { 0x007B, [at100_2_lcd]      ,   "Level         Magnetorestrictive" },
        { 0x007E, [at100_temp_lcd]   ,   "Level         Magnetorestrictive" },
        { 0x007F, [at100_2_temp_lcd] ,   "Level         Magnetorestrictive" },
        { 0x0098, [mt2000]           },
        { 0x00AA, [mt5000]           ,   "Level         Radar" },
        { 0x00AB, [mt5100]           ,   "Level         Radar" },
        { 0x00BA, [mt5200]           ,   "Level         Radar" },
        { 0x00C2, [ss140xp]          ,   "Level         Radar" },
        { 0x00CB, [spm200]           },
        { 0x5067, [spm100]           },
        { 0x5072, [at100_vol]        ,   "Level         Magnetorestrictive" },
        { 0x5073, [at100_2_vol]      ,   "Level         Magnetorestrictive" },
        { 0x5076, [at100_temp_vol]   ,   "Level         Magnetorestrictive" },
        { 0x5077, [at100_2_temp_vol] ,   "Level         Magnetorestrictive" },
        { 0x5078, [at100]            ,   "Level         Magnetorestrictive" },
        { 0x507A, [at100_lcd]        ,   "Level         Magnetorestrictive" },
        { 0x507B, [at100_2_lcd]      ,   "Level         Magnetorestrictive" },
        { 0x507E, [at100_temp_lcd]   ,   "Level         Magnetorestrictive" },
        { 0x507F, [at100_2_temp_lcd] ,   "Level         Magnetorestrictive" },
        { 0x5098, [mt2000]           },
        { 0x50AA, [mt5000]           ,   "Level         Radar" },
        { 0x50AB, [mt5100]           ,   "Level         Radar" },
        { 0x50BA, [mt5200]           ,   "Level         Radar" },
        { 0x50C2, [ss140xp]          ,   "Level         Radar" },
        { 0x50CB, [spm200]           }
    }
}

/*************************************************************************************
 * Draeger Device Type Codes
 */
VARIABLE Draeger_model_code    /* 0x0052 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00E8, [polytron_8000] ,   "Analytical    Gas Detector" },
        { 0x00E9, [pir_7000]      ,   "Analytical    Gas Detector" },
        { 0x00EB, [polytron7000]  ,   "Analytical    Gas Detector" },
        { 0x00ED, [polytron2_ir]  ,   "Analytical    Infrared Gas Detector" },
        { 0x00EE, "PIR 7x00"      ,   "Analytical" },
        { 0x00EF, [pulsar_7x00]   ,   "Analytical    Gas Detector" },
        { 0x00F1, "Polytron PID"  ,   "Analytical    Gas Detector" },
        { 0x00F7, [polytron2]     ,   "Analytical    Electrochemical Gas Detector" },
        { 0x52E8, [polytron_8000] ,   "Analytical    Gas Detector" },
        { 0x52E9, [pir_7000]      ,   "Analytical    Gas Detector" },
        { 0x52EB, [polytron7000]  ,   "Analytical    Gas Detector" },
        { 0x52ED, [polytron2_ir]  ,   "Analytical    Infrared Gas Detector" },
        { 0x52EE, "PIR 7x00"      ,   "Analytical" },
        { 0x52EF, [pulsar_7x00]   ,   "Analytical    Gas Detector" },
        { 0x52F1, "Polytron PID"  ,   "Analytical    Gas Detector" },
        { 0x52F7, [polytron2]     ,   "Analytical    Electrochemical Gas Detector" }
    }
}

/*************************************************************************************
 * Siemens Milltronics PI Device Type Codes
 *     Note: was Meridian 
 */
VARIABLE Meridian_model_code    /* 0x0054 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00F8, [msp2002] ,   "Level         Capacitance" },
        { 0x00F9, [mst9500] ,   "Level         Capacitance" },
        { 0x54F8, [msp2002] ,   "Level         Capacitance" },
        { 0x54F9, [mst9500] ,   "Level         Capacitance" }
    }
}

/*************************************************************************************
 * BTG Device Type Codes
 */
VARIABLE BTG_model_code    /* 0x0055 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00DE, [hcm_8000] ,   "Analytical    Measuring Instruments" },
        { 0x55DE, [hcm_8000] ,   "Analytical    Measuring Instruments" }
    }
}

/*************************************************************************************
 * Magnetrol Device Type Codes
 */
VARIABLE Magnetrol_model_code    /* 0x0056 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00DC, [model_700]         ,   "Level" },
        { 0x00DD, [model_r86]         ,   "Level         Radar - Pulse" },
        { 0x00DE, [model_r96]         ,   "Level         Radar - Pulse" },
        { 0x00DF, [jupiter_model_jm4] ,   "Level         Magnostrictive" },
        { 0x00E0, [model_706]         ,   "Level         Radar" },
        { 0x00E1, [model_ta2_2x]      ,   "Flow" },
        { 0x00E2, [model_r82]         ,   "Level         Radar" },
        { 0x00E3, [e3_modulevel]      ,   "Level" },
        { 0x00E4, [model_355]         ,   "Level         Ultrasonic" },
        { 0x00E5, [model_705_3x]      ,   "Level" },
        { 0x00E6, [model_rx5]         ,   "Level" },
        { 0x00E7, [model_704]         ,   "Level" },
        { 0x00E8, [jupiter]           ,   "Level         Magnostrictive" },
        { 0x00E9, [model_ta2]         ,   "Flow          Mass" },
        { 0x00EA, [model_707]         ,   "Level" },
        { 0x00EB, [model_705_2x]      ,   "Level" },
        { 0x00EC, [model_708]         ,   "Level" },
        { 0x00ED, [model_805]         ,   "Level" },
        { 0x00EE, [model_705]         ,   "Level" },
        { 0x00EF, [smartez]           ,   "Level" },
        { 0x56DC, [model_700]         ,   "Level" },
        { 0x56DD, [model_r86]         ,   "Level         Radar - Pulse" },
        { 0x56DE, [model_r96]         ,   "Level         Radar - Pulse" },
        { 0x56DF, [jupiter_model_jm4] ,   "Level         Magnostrictive" },
        { 0x56E0, [model_706]         ,   "Level         Radar" },
        { 0x56E1, [model_ta2_2x]      ,   "Flow" },
        { 0x56E2, [model_r82]         ,   "Level         Radar" },
        { 0x56E3, [e3_modulevel]      ,   "Level" },
        { 0x56E4, [model_355]         ,   "Level         Ultrasonic" },
        { 0x56E5, [model_705_3x]      ,   "Level" },
        { 0x56E6, [model_rx5]         ,   "Level" },
        { 0x56E7, [model_704]         ,   "Level" },
        { 0x56E8, [jupiter]           ,   "Level         Magnostrictive" },
        { 0x56E9, [model_ta2]         ,   "Flow          Mass" },
        { 0x56EA, [model_707]         ,   "Level" },
        { 0x56EB, [model_705_2x]      ,   "Level" },
        { 0x56EC, [model_708]         ,   "Level" },
        { 0x56ED, [model_805]         ,   "Level" },
        { 0x56EE, [model_705]         ,   "Level" },
        { 0x56EF, [smartez]           ,   "Level" }
    }
}

/*************************************************************************************
 * Neles Finland Oy Device Type Codes
 *     Note: was Metso Flow Control Inc
 */
VARIABLE Neles_Finland_Oy_model_code    /* 0x0057 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00A2, [ndx_h6]    ,   "Valve         Controller" },
        { 0x00A3, [ndx_h7]    ,   "Valve         Controller" },
        { 0x00D5, [sg9000h]   ,   "Valve         On/Off Valve Controller" },
        { 0x00D9, [vg9000h]   ,   "Valve         Positioner" },
        { 0x00DE, [nd7000h]   ,   "Valve         Controller" },
        { 0x00E4, [nd9100ht]  ,   "Valve         Positioner" },
        { 0x00E5, [nd9100h]   ,   "Valve         Positioner" },
        { 0x00EB, [valvguard] ,   "Valve         On/Off Valve Controller" },
        { 0x00EE, [nd800_pt]  ,   "Valve         Positioner/Pneumatic" },
        { 0x00EF, [nd800]     ,   "Valve         Positioner/Pneumatic" },
        { 0x00F1, "VG9000H H7" ,   "Valve         Positioner" },
        { 0x57A2, [ndx_h6]    ,   "Valve         Controller" },
        { 0x57A3, [ndx_h7]    ,   "Valve         Controller" },
        { 0x57D5, [sg9000h]   ,   "Valve         On/Off Valve Controller" },
        { 0x57D9, [vg9000h]   ,   "Valve         Positioner" },
        { 0x57DE, [nd7000h]   ,   "Valve         Controller" },
        { 0x57E4, [nd9100ht]  ,   "Valve         Positioner" },
        { 0x57E5, [nd9100h]   ,   "Valve         Positioner" },
        { 0x57EB, [valvguard] ,   "Valve         On/Off Valve Controller" },
        { 0x57EE, [nd800_pt]  ,   "Valve         Positioner/Pneumatic" },
        { 0x57EF, [nd800]     ,   "Valve         Positioner/Pneumatic" },
        { 0x57F1, "VG9000H H7" ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Siemens Milltronics PI Device Type Codes
 */
VARIABLE Milltronics_model_code    /* 0x0058 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00C8, [probe_86x]     ,   "Level         Ultrasonic" },
        { 0x00C9, [sitrans_lr300] ,   "Level         Radar - Pulse" },
        { 0x00CA, [lr_200]        ,   "Level         Radar" },
        { 0x00CC, [sitrans_lr400] ,   "Level         Radar" },
        { 0x00CD, [probe_lu6m]    ,   "Level         Ultrasonic" },
        { 0x00CE, [probe_lu12m]   ,   "Level         Ultrasonic" },
        { 0x00CF, [probe_lr]      ,   "Level         Radar" },
        { 0x00D1, [sitrans_lr250] ,   "Level         Radar - Pulse" },
        { 0x00D3, [sitrans_lr460] ,   "Level         Radar" },
        { 0x00D4, [sitrans_lr260] ,   "Level         Radar - Pulse" },
        { 0x00D6, [sitrans_lr200] ,   "Level         Radar - Pulse" },
        { 0x58C8, [probe_86x]     ,   "Level         Ultrasonic" },
        { 0x58C9, [sitrans_lr300] ,   "Level         Radar - Pulse" },
        { 0x58CA, [lr_200]        ,   "Level         Radar" },
        { 0x58CC, [sitrans_lr400] ,   "Level         Radar" },
        { 0x58CD, [probe_lu6m]    ,   "Level         Ultrasonic" },
        { 0x58CE, [probe_lu12m]   ,   "Level         Ultrasonic" },
        { 0x58CF, [probe_lr]      ,   "Level         Radar" },
        { 0x58D1, [sitrans_lr250] ,   "Level         Radar - Pulse" },
        { 0x58D3, [sitrans_lr460] ,   "Level         Radar" },
        { 0x58D4, [sitrans_lr260] ,   "Level         Radar - Pulse" },
        { 0x58D6, [sitrans_lr200] ,   "Level         Radar - Pulse" }
    }
}

/*************************************************************************************
 * Anderson Instrument Company Device Type Codes
 */
VARIABLE Anderson_Instrument_Company_model_code    /* 0x005A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00C8, [andrsn1] ,   "Pressure" },
        { 0x00CC, [l3]      ,   "Level" },
        { 0x5AC8, [andrsn1] ,   "Pressure" },
        { 0x5ACC, [l3]      ,   "Level" }
    }
}

/*************************************************************************************
 * Inor Device Type Codes
 */
VARIABLE Inor_model_code    /* 0x005B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EA, [ipaq_c530_r530] ,   "Temperature" },
        { 0x00EB, [ipaq_c520_r520] ,   "Temperature" },
        { 0x00EF, [meso]           ,   "Temperature" },
        { 0x5BEA, [ipaq_c530_r530] ,   "Temperature" },
        { 0x5BEB, [ipaq_c520_r520] ,   "Temperature" },
        { 0x5BEF, [meso]           ,   "Temperature" }
    }
}

/*************************************************************************************
 * Robertshaw Device Type Codes
 */
VARIABLE Robertshaw_model_code    /* 0x005C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00C8, [excalibur_7000] ,   "Level" },
        { 0x5CC8, [excalibur_7000] ,   "Level" }
    }
}

/*************************************************************************************
 * Pepperl+Fuchs Device Type Codes
 */
VARIABLE PEPPERL_FUCHS_model_code    /* 0x005D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00E4, [corrtran]     ,   "Analytical    Corrosion" },
        { 0x00E5, [luc_m]        },
        { 0x00E6, [mux2700_g]    },
        { 0x00E7, [rsd_uo_ex8h]  },
        { 0x00E8, [rsd_ci_ex8h]  },
        { 0x00E9, [ksd2_gw2_pro] },
        { 0x00EA, [barcon_m]     },
        { 0x00EB, [luc_plus]     },
        { 0x00EC, [pulscon_plus] },
        { 0x00ED, [pulscon]      },
        { 0x00EE, [barcon]       },
        { 0x00EF, [kfd2_hmm_16]  },
        { 0x5DE4, [corrtran]     ,   "Analytical    Corrosion" },
        { 0x5DE5, [luc_m]        },
        { 0x5DE6, [mux2700_g]    },
        { 0x5DE7, [rsd_uo_ex8h]  },
        { 0x5DE8, [rsd_ci_ex8h]  },
        { 0x5DE9, [ksd2_gw2_pro] },
        { 0x5DEA, [barcon_m]     },
        { 0x5DEB, [luc_plus]     },
        { 0x5DEC, [pulscon_plus] },
        { 0x5DED, [pulscon]      },
        { 0x5DEE, [barcon]       },
        { 0x5DEF, [kfd2_hmm_16]  }
    }
}

/*************************************************************************************
 * Accutech Device Type Codes
 */
VARIABLE Accutech_model_code    /* 0x005E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EE, [press_dp_xmtr] ,   "Pressure" },
        { 0x00EF, [ai1500]        ,   "Temperature" },
        { 0x5EEE, [press_dp_xmtr] ,   "Pressure" },
        { 0x5EEF, [ai1500]        ,   "Temperature" }
    }
}

/*************************************************************************************
 * Baumer Device Type Codes
 *     Note: was Bourdon-Haenni
 */
VARIABLE Baumer_model_code    /* 0x0060 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EB, [flextop_2222]       ,   "Temperature" },
        { 0x00EC, [afi4_afi5_combilyz] ,   "Analytical    Conductivity" },
        { 0x00ED, [flexbar_3501]       ,   "Pressure" },
        { 0x00EE, [flexbar_hrt]        ,   "Pressure" },
        { 0x00EF, [flextop_hrt]        ,   "Temperature" },
        { 0x60EB, [flextop_2222]       ,   "Temperature" },
        { 0x60EC, [afi4_afi5_combilyz] ,   "Analytical    Conductivity" },
        { 0x60ED, [flexbar_3501]       ,   "Pressure" },
        { 0x60EE, [flexbar_hrt]        ,   "Pressure" },
        { 0x60EF, [flextop_hrt]        ,   "Temperature" }
    }
}

/*************************************************************************************
 * Knick Device Type Codes
 */
VARIABLE Knick_model_code    /* 0x0061 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00CD, "Stratos Multi E401" ,   "Analytical" },
        { 0x00CE, "MemoTrans MT201"    ,   "Analytical" },
        { 0x00E0, [condi_a411]         ,   "Analytical    Conductivity" },
        { 0x00E1, [cond_a411]          ,   "Analytical    Conductivity" },
        { 0x00E2, [oxy_a411]           ,   "Analytical    Dissolved Oxygen" },
        { 0x00E3, [ph_a411]            ,   "Analytical    pH" },
        { 0x00E4, [condi_a211]         ,   "Analytical    Conductivity" },
        { 0x00E5, [cond_a211]          ,   "Analytical    Conductivity" },
        { 0x00E6, [oxy_a211]           ,   "Analytical    Dissolved Oxygen" },
        { 0x00E7, [ph_a211]            ,   "Analytical    pH" },
        { 0x00E8, [condi_2211]         ,   "Analytical    Conductivity" },
        { 0x00E9, [oxy_2211]           ,   "Analytical    Dissolved Oxygen" },
        { 0x00EA, [cond_2211]          ,   "Analytical    Conductivity" },
        { 0x00EB, [ph_2211]            ,   "Analytical    pH" },
        { 0x61CD, "Stratos Multi E401" ,   "Analytical" },
        { 0x61CE, "MemoTrans MT201"    ,   "Analytical" },
        { 0x61E0, [condi_a411]         ,   "Analytical    Conductivity" },
        { 0x61E1, [cond_a411]          ,   "Analytical    Conductivity" },
        { 0x61E2, [oxy_a411]           ,   "Analytical    Dissolved Oxygen" },
        { 0x61E3, [ph_a411]            ,   "Analytical    pH" },
        { 0x61E4, [condi_a211]         ,   "Analytical    Conductivity" },
        { 0x61E5, [cond_a211]          ,   "Analytical    Conductivity" },
        { 0x61E6, [oxy_a211]           ,   "Analytical    Dissolved Oxygen" },
        { 0x61E7, [ph_a211]            ,   "Analytical    pH" },
        { 0x61E8, [condi_2211]         ,   "Analytical    Conductivity" },
        { 0x61E9, [oxy_2211]           ,   "Analytical    Dissolved Oxygen" },
        { 0x61EA, [cond_2211]          ,   "Analytical    Conductivity" },
        { 0x61EB, [ph_2211]            ,   "Analytical    pH" }
    }
}

/*************************************************************************************
 * VEGA Grieshaber KG Device Type Codes
 */
VARIABLE VEGA_model_code    /* 0x0062 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00BC, "VEGAPULS 20/30 series"  ,   "Level         Radar" },
        { 0x00BD, "VEGAPULS C 20 series"   ,   "Level         Radar" },
        { 0x00BE, [vegapuls_64]            ,   "Level         Radar" },
        { 0x00BF, [vegadif_80_series_sil]  ,   "Pressure      Differential" },
        { 0x00C0, [vegadif_80_series]      ,   "Pressure      Differential" },
        { 0x00C1, [vegapuls_69]            ,   "Level         Radar" },
        { 0x00C2, [vegabar_80_series_sil]  ,   "Pressure      Pressure/Level" },
        { 0x00C3, [vegabar_80_series]      ,   "Pressure" },
        { 0x00C4, [pointrac_31_sil]        },
        { 0x00C5, [pointrac_31]            ,   "Level         Radiation" },
        { 0x00C6, [vegapuls_sr_68]         ,   "Level         Radar" },
        { 0x00C7, [vegapuls_wl_61]         ,   "Level         Radar" },
        { 0x00C8, [fibertrac_31]           ,   "Level         Radiation" },
        { 0x00C9, [fibertrac_32]           ,   "Level         Radiation" },
        { 0x00CA, [solitrac_31]            ,   "Level         Radiation" },
        { 0x00CB, [minitrac_31]            ,   "Analytical    Radiation Density" },
        { 0x00CC, [minitrac_32]            ,   "Analytical    Radiation Density" },
        { 0x00CD, [minitrac_33]            ,   "Analytical    Radiation Density" },
        { 0x00CE, [weightrac_31]           ,   "Flow          Radiation" },
        { 0x00CF, [weightrac_32]           ,   "Flow          Radiation" },
        { 0x00D0, [minitrac_33_sil]        },
        { 0x00D1, [minitrac_32_sil]        },
        { 0x00D2, [minitrac_31_sil]        },
        { 0x00D3, [solitrac_31_sil]        },
        { 0x00D4, [vegaflex_80_series_sil] ,   "Level         Radar" },
        { 0x00D5, [vegaflex_80_series]     ,   "Level" },
        { 0x00D6, [vegapuls_68]            ,   "Level         Radar" },
        { 0x00D7, [vegapuls_67]            ,   "Level         Radar" },
        { 0x00D8, [vegapuls_66]            ,   "Level         Radar" },
        { 0x00D9, [vegapuls_65]            ,   "Level         Radar" },
        { 0x00DA, [vegapuls_63]            ,   "Level         Radar" },
        { 0x00DB, [vegapuls_62]            ,   "Level         Radar" },
        { 0x00DC, [vegapuls_61]            ,   "Level         Radar" },
        { 0x00DE, [fibertrac_32_sil]       },
        { 0x00DF, [fibertrac_31_sil]       },
        { 0x00E1, [vegadif_65]             ,   "Pressure      Differential" },
        { 0x00E2, [vegabar_5x6x]           ,   "Pressure" },
        { 0x62BC, "VEGAPULS 20/30 series"  ,   "Level         Radar" },
        { 0x62BD, "VEGAPULS C 20 series"   ,   "Level         Radar" },
        { 0x62BE, [vegapuls_64]            ,   "Level         Radar" },
        { 0x62BF, [vegadif_80_series_sil]  ,   "Pressure      Differential" },
        { 0x62C0, [vegadif_80_series]      ,   "Pressure      Differential" },
        { 0x62C1, [vegapuls_69]            ,   "Level         Radar" },
        { 0x62C2, [vegabar_80_series_sil]  ,   "Pressure      Pressure/Level" },
        { 0x62C3, [vegabar_80_series]      ,   "Pressure" },
        { 0x62C4, [pointrac_31_sil]        },
        { 0x62C5, [pointrac_31]            ,   "Level         Radiation" },
        { 0x62C6, [vegapuls_sr_68]         ,   "Level         Radar" },
        { 0x62C7, [vegapuls_wl_61]         ,   "Level         Radar" },
        { 0x62C8, [fibertrac_31]           ,   "Level         Radiation" },
        { 0x62C9, [fibertrac_32]           ,   "Level         Radiation" },
        { 0x62CA, [solitrac_31]            ,   "Level         Radiation" },
        { 0x62CB, [minitrac_31]            ,   "Analytical    Radiation Density" },
        { 0x62CC, [minitrac_32]            ,   "Analytical    Radiation Density" },
        { 0x62CD, [minitrac_33]            ,   "Analytical    Radiation Density" },
        { 0x62CE, [weightrac_31]           ,   "Flow          Radiation" },
        { 0x62CF, [weightrac_32]           ,   "Flow          Radiation" },
        { 0x62D0, [minitrac_33_sil]        },
        { 0x62D1, [minitrac_32_sil]        },
        { 0x62D2, [minitrac_31_sil]        },
        { 0x62D3, [solitrac_31_sil]        },
        { 0x62D4, [vegaflex_80_series_sil] ,   "Level         Radar" },
        { 0x62D5, [vegaflex_80_series]     ,   "Level" },
        { 0x62D6, [vegapuls_68]            ,   "Level         Radar" },
        { 0x62D7, [vegapuls_67]            ,   "Level         Radar" },
        { 0x62D8, [vegapuls_66]            ,   "Level         Radar" },
        { 0x62D9, [vegapuls_65]            ,   "Level         Radar" },
        { 0x62DA, [vegapuls_63]            ,   "Level         Radar" },
        { 0x62DB, [vegapuls_62]            ,   "Level         Radar" },
        { 0x62DC, [vegapuls_61]            ,   "Level         Radar" },
        { 0x62DE, [fibertrac_32_sil]       },
        { 0x62DF, [fibertrac_31_sil]       },
        { 0x62E1, [vegadif_65]             ,   "Pressure      Differential" },
        { 0x62E2, [vegabar_5x6x]           ,   "Pressure" }
    }
}

/*************************************************************************************
 * MTS Systems Corp. Device Type Codes
 */
VARIABLE MTS_SYS_CORPS_model_code    /* 0x0063 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00ED, [m_series]  ,   "Level" },
        { 0x00EE, [lpsil]     ,   "Level" },
        { 0x00EF, [lp_series] ,   "Level" },
        { 0x63ED, [m_series]  ,   "Level" },
        { 0x63EE, [lpsil]     ,   "Level" },
        { 0x63EF, [lp_series] ,   "Level" }
    }
}

/*************************************************************************************
 * Oval Device Type Codes
 */
VARIABLE Oval_model_code    /* 0x0064 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0064, [ultraoval]     ,   "Flow" },
        { 0x0065, [exdelta]       ,   "Flow          Vortex" },
        { 0x006E, [mass_flo_9401] ,   "Flow          Coriolis Mass" },
        { 0x006F, [mass_flo_9801] ,   "Flow          Coriolis Mass" },
        { 0x0070, [mass_flo_9201] ,   "Flow          Coriolis Mass" },
        { 0x007B, [mass_flo_9431] ,   "Flow          Coriolis Mass" },
        { 0x0082, [altimass]      ,   "Flow          Coriolis Mass" },
        { 0x0084, [psonic_l4]     ,   "Flow          Ultrasonic" },
        { 0x6464, [ultraoval]     ,   "Flow" },
        { 0x6465, [exdelta]       ,   "Flow          Vortex" },
        { 0x646E, [mass_flo_9401] ,   "Flow          Coriolis Mass" },
        { 0x646F, [mass_flo_9801] ,   "Flow          Coriolis Mass" },
        { 0x6470, [mass_flo_9201] ,   "Flow          Coriolis Mass" },
        { 0x647B, [mass_flo_9431] ,   "Flow          Coriolis Mass" },
        { 0x6482, [altimass]      ,   "Flow          Coriolis Mass" },
        { 0x6484, [psonic_l4]     ,   "Flow          Ultrasonic" }
    }
}

/*************************************************************************************
 * Masoneilan-Dresser Device Type Codes
 */
VARIABLE GE_Masoneilan_model_code    /* 0x0065 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0064, [hdlt]          ,   "Level" },
        { 0x0065, [dlt12400]      ,   "Level" },
        { 0x00AA, "SVI3"          ,   "Valve         Positioner" },
        { 0x00C8, [svi]           ,   "Valve         Positioner" },
        { 0x00C9, [svi2]          ,   "Valve         Positioner" },
        { 0x00CA, [svi2a]         ,   "Valve         Positioner" },
        { 0x00CB, [svi2esd]       ,   "Valve         Controller" },
        { 0x00CC, [svi1000]       ,   "Valve         Positioner" },
        { 0x00CD, [vector]        ,   "WirelessHART  Adapter" },
        { 0x00CE, [svi_ii_ap_h6]  ,   "Valve         Positioner" },
        { 0x00CF, [svi_ii_esd_h6] },
        { 0x00EE, [svi_ii_ap_h7]  ,   "Valve         Positioner" },
        { 0x00EF, [svi1000_h_7]   ,   "Valve         Positioner" },
        { 0x6564, [hdlt]          ,   "Level" },
        { 0x6565, [dlt12400]      ,   "Level" },
        { 0x65AA, "SVI3"          ,   "Valve         Positioner" },
        { 0x65C8, [svi]           ,   "Valve         Positioner" },
        { 0x65C9, [svi2]          ,   "Valve         Positioner" },
        { 0x65CA, [svi2a]         ,   "Valve         Positioner" },
        { 0x65CB, [svi2esd]       ,   "Valve         Controller" },
        { 0x65CC, [svi1000]       ,   "Valve         Positioner" },
        { 0x65CD, [vector]        ,   "WirelessHART  Adapter" },
        { 0x65CE, [svi_ii_ap_h6]  ,   "Valve         Positioner" },
        { 0x65CF, [svi_ii_esd_h6] },
        { 0x65EE, [svi_ii_ap_h7]  ,   "Valve         Positioner" },
        { 0x65EF, [svi1000_h_7]   ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Besta Device Type Codes
 */
VARIABLE Besta_model_code    /* 0x0066 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [cls_4_20] ,   "Level         Continuous Capactive" },
        { 0x66EF, [cls_4_20] ,   "Level         Continuous Capactive" }
    }
}

/*************************************************************************************
 * Ohmart Device Type Codes
 */
VARIABLE Ohmart_model_code    /* 0x0067 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00E0, [dsth_lsth]      ,   "Level         Density - Nuclear" },
        { 0x00E1, [dsth_lsth_comp] ,   "Level         Density - Nuclear" },
        { 0x67E0, [dsth_lsth]      ,   "Level         Density - Nuclear" },
        { 0x67E1, [dsth_lsth_comp] ,   "Level         Density - Nuclear" }
    }
}

/*************************************************************************************
 * Harold Beck and Sons Device Type Codes
 */
VARIABLE Harold_Beck_and_Sons_model_code    /* 0x0068 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [esr_d]    ,   "Valve         Damper Actuator" },
        { 0x000A, [beck_dcm] ,   "Valve         Electric Damper Actuator" },
        { 0x00EF, [beck_mk2] ,   "Valve         Damper Actuator" },
        { 0x6801, [esr_d]    ,   "Valve         Damper Actuator" },
        { 0x680A, [beck_dcm] ,   "Valve         Electric Damper Actuator" },
        { 0x68EF, [beck_mk2] ,   "Valve         Damper Actuator" }
    }
}

/*************************************************************************************
 * Wika Device Type Codes
 */
VARIABLE Wika_model_code    /* 0x006B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00E4, "DPT-20 SIL2"    ,   "Pressure      Differential" },
        { 0x00E5, "DPT-20"         ,   "Pressure      Differential" },
        { 0x00EB, [dpt]            ,   "Pressure      Differential" },
        { 0x00EC, [ipt]            ,   "Pressure      Absolute/Relative" },
        { 0x00EE, [unitrans]       ,   "Pressure" },
        { 0x00EF, [t32]            ,   "Temperature" },
        { 0x00F0, [ipt_cpt_2x]     ,   "Pressure" },
        { 0x00F1, [ipt_cpt_2x_sil] ,   "Pressure" },
        { 0x6BE4, "DPT-20 SIL2"    ,   "Pressure      Differential" },
        { 0x6BE5, "DPT-20"         ,   "Pressure      Differential" },
        { 0x6BEB, [dpt]            ,   "Pressure      Differential" },
        { 0x6BEC, [ipt]            ,   "Pressure      Absolute/Relative" },
        { 0x6BEE, [unitrans]       ,   "Pressure" },
        { 0x6BEF, [t32]            ,   "Temperature" },
        { 0x6BF0, [ipt_cpt_2x]     ,   "Pressure" },
        { 0x6BF1, [ipt_cpt_2x_sil] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Bopp & Reuther Messtechnik Device Type Codes
 *     Note: was Bopp & Reuther Heinrichs
 */
VARIABLE Bopp_Reuther_Heinrichs_model_code    /* 0x006C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EB, [umc2] ,   "Flow          Mass" },
        { 0x00EC, [es]   ,   "Flow          Electronic" },
        { 0x00ED, [vtx]  ,   "Flow          Vortex" },
        { 0x00EE, [dimf] ,   "Analytical    Density" },
        { 0x00EF, [ust1] ,   "Flow" },
        { 0x6CEB, [umc2] ,   "Flow          Mass" },
        { 0x6CEC, [es]   ,   "Flow          Electronic" },
        { 0x6CED, [vtx]  ,   "Flow          Vortex" },
        { 0x6CEE, [dimf] ,   "Analytical    Density" },
        { 0x6CEF, [ust1] ,   "Flow" }
    }
}

/*************************************************************************************
 * PR Electronics Device Type Codes
 */
VARIABLE PR_Electronics_model_code    /* 0x006D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00DF, [t55]            ,   "Temperature" },
        { 0x00E6, [pr5437]         ,   "Temperature" },
        { 0x00E7, [pr_7501h7]      ,   "Temperature" },
        { 0x00E8, [pr3113]         ,   "Temperature" },
        { 0x00E9, [pr3337]         ,   "Temperature" },
        { 0x00EA, [pr_6337]        ,   "Temperature" },
        { 0x00EB, [pr_5337]        ,   "Temperature" },
        { 0x00EE, [pretrans_6335]  ,   "Temperature" },
        { 0x00EF, [pr_5335_7501h5] ,   "Temperature" },
        { 0x6DDF, [t55]            ,   "Temperature" },
        { 0x6DE6, [pr5437]         ,   "Temperature" },
        { 0x6DE7, [pr_7501h7]      ,   "Temperature" },
        { 0x6DE8, [pr3113]         ,   "Temperature" },
        { 0x6DE9, [pr3337]         ,   "Temperature" },
        { 0x6DEA, [pr_6337]        ,   "Temperature" },
        { 0x6DEB, [pr_5337]        ,   "Temperature" },
        { 0x6DEE, [pretrans_6335]  ,   "Temperature" },
        { 0x6DEF, [pr_5335_7501h5] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Rotork Process Controls Device Type Codes
 */
VARIABLE Rotork_Process_Controls_model_code    /* 0x006E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00DD, [rpc_actuator] ,   "Valve         Actuator" },
        { 0x6EDD, [rpc_actuator] ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * Apparatebau Hundsbach Device Type Codes
 */
VARIABLE Apparatebau_Hundsbach_model_code    /* 0x0071 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EE, [mt115] ,   "Temperature" },
        { 0x71EE, [mt115] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Dynisco Device Type Codes
 */
VARIABLE Dynisco_model_code    /* 0x0072 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00DE, [spx]   ,   "Pressure" },
        { 0x00DF, [ipxii] ,   "Pressure      Multivariable" },
        { 0x00E0, [ipx]   ,   "Pressure      Multivariable" },
        { 0x72DE, [spx]   ,   "Pressure" },
        { 0x72DF, [ipxii] ,   "Pressure      Multivariable" },
        { 0x72E0, [ipx]   ,   "Pressure      Multivariable" }
    }
}

/*************************************************************************************
 * Direct Measurement Device Type Codes
 */
VARIABLE Direct_Measurement_model_code    /* 0x0074 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [dmc_r1a] ,   "Flow          Coriolis Mass" },
        { 0x74EF, [dmc_r1a] ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * Klay Device Type Codes
 */
VARIABLE Klay_model_code    /* 0x0075 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [klay_2000] ,   "Pressure" },
        { 0x75EF, [klay_2000] ,   "Pressure" }
    }
}

/*************************************************************************************
 * CiDRA CORP. Device Type Codes
 */
VARIABLE CiDRA_CORP_model_code    /* 0x0076 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00ED, [vf_100]    ,   "Flow" },
        { 0x00EE, [gvf_100]   ,   "Flow" },
        { 0x00EF, [vfgvf_100] ,   "Flow" },
        { 0x76ED, [vf_100]    ,   "Flow" },
        { 0x76EE, [gvf_100]   ,   "Flow" },
        { 0x76EF, [vfgvf_100] ,   "Flow" }
    }
}

/*************************************************************************************
 * Buerkert Fluid Control Systems Device Type Codes
 */
VARIABLE Buerkert_Fluid_Control_Systems_model_code    /* 0x0078 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [burkert8630] },
        { 0x78EF, [burkert8630] }
    }
}

/*************************************************************************************
 * Venture Measurement Device Type Codes
 */
VARIABLE Venture_Measurement_model_code    /* 0x0079 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [forcemeter] ,   "Flow" },
        { 0x79EF, [forcemeter] ,   "Flow" }
    }
}

/*************************************************************************************
 * GLI Device Type Codes
 */
VARIABLE GLI_model_code    /* 0x007C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EE, [m33_53p] },
        { 0x00EF, [m33_53c] },
        { 0x7CEE, [m33_53p] },
        { 0x7CEF, [m33_53c] }
    }
}

/*************************************************************************************
 * Paper Machine Components Device Type Codes
 */
VARIABLE Paper_Machine_Components_model_code    /* 0x007E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00E0, [smt_el]    ,   "Pressure" },
        { 0x00E1, [smt_el_ii] },
        { 0x7EE0, [smt_el]    ,   "Pressure" },
        { 0x7EE1, [smt_el_ii] }
    }
}

/*************************************************************************************
 * Labom Device Type Codes
 */
VARIABLE Labom_model_code    /* 0x007F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x003C, [pascal_ci]  ,   "Pressure" },
        { 0x00E0, [ipx]        ,   "Pressure      Multivariable" },
        { 0x00EC, [pascal_ci4] },
        { 0x00EF, [pascal_cv]  ,   "Pressure" },
        { 0x7F3C, [pascal_ci]  ,   "Pressure" },
        { 0x7FE0, [ipx]        ,   "Pressure      Multivariable" },
        { 0x7FEC, [pascal_ci4] },
        { 0x7FEF, [pascal_cv]  ,   "Pressure" }
    }
}

/*************************************************************************************
 * Danfoss Device Type Codes
 */
VARIABLE Danfoss_model_code    /* 0x0080 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0003, [magflo] ,   "Flow          Magnetic" },
        { 0x8003, [magflo] ,   "Flow          Magnetic" }
    }
}

/*************************************************************************************
 * Turbo Device Type Codes
 */
VARIABLE Turbo_model_code    /* 0x0081 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [intermag_transmag] ,   "Flow          Magnetic" },
        { 0x817F, [intermag_transmag] ,   "Flow          Magnetic" }
    }
}

/*************************************************************************************
 * TOKYO KEISO Device Type Codes
 */
VARIABLE Tokyo_Keiso_model_code    /* 0x0082 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0054, [tlrx400x500] ,   "Level         Radar" },
        { 0x0057, [mmc400]      ,   "Flow          Coriolis Mass" },
        { 0x0058, "FGY"         ,   "Level" },
        { 0x0059, [am3_h]       ,   "Flow" },
        { 0x005B, "VFC200"      ,   "Flow          Vortex" },
        { 0x005C, [ulc_400]     ,   "Flow          Ultrasonic" },
        { 0x0060, [nlz_h]       ,   "Flow" },
        { 0x0061, [fst4000]     ,   "Level" },
        { 0x0062, [ume1000]     ,   "Flow          Ultrasonic" },
        { 0x0063, [am_t]        ,   "Flow          Multivariable" },
        { 0x0065, [am_h]        ,   "Flow          Variable Area" },
        { 0x0067, [vfc_070]     ,   "Flow          Vortex" },
        { 0x0069, "UL6300V2"    ,   "Flow          Ultrasonic" },
        { 0x007B, [fw_9000]     ,   "Level" },
        { 0x007F, [fst_3000]    ,   "Level" },
        { 0x8254, [tlrx400x500] ,   "Level         Radar" },
        { 0x8257, [mmc400]      ,   "Flow          Coriolis Mass" },
        { 0x8258, "FGY"         ,   "Level" },
        { 0x8259, [am3_h]       ,   "Flow" },
        { 0x825B, "VFC200"      ,   "Flow          Vortex" },
        { 0x825C, [ulc_400]     ,   "Flow          Ultrasonic" },
        { 0x8260, [nlz_h]       ,   "Flow" },
        { 0x8261, [fst4000]     ,   "Level" },
        { 0x8262, [ume1000]     ,   "Flow          Ultrasonic" },
        { 0x8263, [am_t]        ,   "Flow          Multivariable" },
        { 0x8265, [am_h]        ,   "Flow          Variable Area" },
        { 0x8267, [vfc_070]     ,   "Flow          Vortex" },
        { 0x8269, "UL6300V2"    ,   "Flow          Ultrasonic" },
        { 0x827B, [fw_9000]     ,   "Level" },
        { 0x827F, [fst_3000]    ,   "Level" }
    }
}

/*************************************************************************************
 * SMC Device Type Codes
 */
VARIABLE SMC_model_code    /* 0x0083 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007D, [ip8001]    ,   "Valve         Positioner/Electropneumatic" },
        { 0x007E, [ip8101]    ,   "Valve         Positioner/Electropneumatic" },
        { 0x007F, [in_777]    ,   "Valve         Positioner" },
        { 0x00EF, [f793_e701] ,   "Valve         Positioner/Electropneumatic" },
        { 0x837D, [ip8001]    ,   "Valve         Positioner/Electropneumatic" },
        { 0x837E, [ip8101]    ,   "Valve         Positioner/Electropneumatic" },
        { 0x837F, [in_777]    ,   "Valve         Positioner" },
        { 0x83EF, [f793_e701] ,   "Valve         Positioner/Electropneumatic" }
    }
}

/*************************************************************************************
 * Status Instruments Device Type Codes
 */
VARIABLE Status_Instruments_model_code    /* 0x0084 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [sem_300] ,   "Temperature" },
        { 0x84EF, [sem_300] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Huakong Device Type Codes
 */
VARIABLE Huakong_model_code    /* 0x0085 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [hk_tt01] ,   "Temperature" },
        { 0x857F, [hk_tt01] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Autrol by DUON System Co., Ltd. Device Type Codes
 *     Note: was Duon System
 */
VARIABLE Autrol_Duon_model_code    /* 0x0086 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0061, [apt3200]  ,   "Pressure" },
        { 0x0067, [apt3700n] ,   "Pressure      Absolute/Differential" },
        { 0x006B, [apt3100]  ,   "Pressure" },
        { 0x0073, [att2x00]  ,   "Temperature" },
        { 0x0075, [att2100s] ,   "Temperature" },
        { 0x0076, [apt3800n] ,   "Pressure" },
        { 0x0078, [apt3500w] ,   "Pressure" },
        { 0x007B, [apt3500]  ,   "Pressure" },
        { 0x007E, [alt6100]  ,   "Level         Radar" },
        { 0x8661, [apt3200]  ,   "Pressure" },
        { 0x8667, [apt3700n] ,   "Pressure      Absolute/Differential" },
        { 0x866B, [apt3100]  ,   "Pressure" },
        { 0x8673, [att2x00]  ,   "Temperature" },
        { 0x8675, [att2100s] ,   "Temperature" },
        { 0x8676, [apt3800n] ,   "Pressure" },
        { 0x8678, [apt3500w] ,   "Pressure" },
        { 0x867B, [apt3500]  ,   "Pressure" },
        { 0x867E, [alt6100]  ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * Vortek Instruments, LLC Device Type Codes
 */
VARIABLE Vortek_Instruments_model_code    /* 0x0087 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [pro_v] ,   "Flow          Vortex - Multiparameter Mass" },
        { 0x8701, [pro_v] ,   "Flow          Vortex - Multiparameter Mass" }
    }
}

/*************************************************************************************
 * Action Instruments Device Type Codes
 */
VARIABLE Action_Instruments_model_code    /* 0x0089 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [t798] ,   "Temperature" },
        { 0x89EF, [t798] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Magtech Device Type Codes
 */
VARIABLE Magtech_model_code    /* 0x008C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0088, [ltm_100] ,   "Level         Magnostrictive" },
        { 0x0089, [ltm_300] ,   "Level" },
        { 0x008A, [ltm_350] ,   "Level         Magnostrictive" },
        { 0x8C88, [ltm_100] ,   "Level         Magnostrictive" },
        { 0x8C89, [ltm_300] ,   "Level" },
        { 0x8C8A, [ltm_350] ,   "Level         Magnostrictive" }
    }
}

/*************************************************************************************
 * Rueger Device Type Codes
 */
VARIABLE Rueger_model_code    /* 0x008D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [s95_hrt]  ,   "Temperature" },
        { 0x007F, [s95_3u_2] ,   "Temperature" },
        { 0x8D01, [s95_hrt]  ,   "Temperature" },
        { 0x8D7F, [s95_3u_2] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Mettler Toledo Device Type Codes
 */
VARIABLE Mettler_Toledo_model_code    /* 0x008E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0070, [m400_4_wire] ,   "Analytical    Multiparameter" },
        { 0x0074, [condi_m420]  ,   "Analytical    Conductivity" },
        { 0x0075, [cond_m420]   ,   "Analytical    Conductivity" },
        { 0x0076, [o2_m420]     ,   "Analytical    Dissolved Oxygen" },
        { 0x0077, [ph_m420]     ,   "Analytical    pH" },
        { 0x0078, [condi7100]   ,   "Analytical    Conductivity" },
        { 0x0079, [o2_4100e]    ,   "Analytical    Dissolved Oxygen" },
        { 0x007A, [cond7100]    ,   "Analytical    Conductivity" },
        { 0x007B, [ph_2100]     ,   "Analytical    pH" },
        { 0x007C, [m400]        },
        { 0x007D, [m100]        },
        { 0x8E70, [m400_4_wire] ,   "Analytical    Multiparameter" },
        { 0x8E74, [condi_m420]  ,   "Analytical    Conductivity" },
        { 0x8E75, [cond_m420]   ,   "Analytical    Conductivity" },
        { 0x8E76, [o2_m420]     ,   "Analytical    Dissolved Oxygen" },
        { 0x8E77, [ph_m420]     ,   "Analytical    pH" },
        { 0x8E78, [condi7100]   ,   "Analytical    Conductivity" },
        { 0x8E79, [o2_4100e]    ,   "Analytical    Dissolved Oxygen" },
        { 0x8E7A, [cond7100]    ,   "Analytical    Conductivity" },
        { 0x8E7B, [ph_2100]     ,   "Analytical    pH" },
        { 0x8E7C, [m400]        },
        { 0x8E7D, [m100]        }
    }
}

/*************************************************************************************
 * Det-Tronics Device Type Codes
 */
VARIABLE Det_Tronics_model_code    /* 0x008F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0073, [ls2000]      ,   "Analytical    Gas Detector" },
        { 0x0074, [atx10]       ,   "Analytical    Acoustic Gas Detector" },
        { 0x0075, [eclipse_co2] ,   "Analytical    Infrared CO2 Gas Detector" },
        { 0x0076, [x2200]       ,   "Analytical    UV Flame Detector" },
        { 0x0077, [x5200]       ,   "Analytical    UVIR Flame Detector" },
        { 0x0078, [x9800]       ,   "Analytical    IR Flame Detector" },
        { 0x0079, [x3302]       ,   "Analytical    Fire Detector" },
        { 0x007A, [ud10]        ,   "Analytical    Gas Detector Display Interface" },
        { 0x007B, [gt3000]      ,   "Analytical    Toxic Gas Detector" },
        { 0x007C, [x3301]       ,   "Analytical    Flame Detector" },
        { 0x007E, [opecl_rx]    ,   "Analytical    Infrared Gas Detector" },
        { 0x007F, [eclipse]     ,   "Analytical    Infrared Gas Detector" },
        { 0x8F73, [ls2000]      ,   "Analytical    Gas Detector" },
        { 0x8F74, [atx10]       ,   "Analytical    Acoustic Gas Detector" },
        { 0x8F75, [eclipse_co2] ,   "Analytical    Infrared CO2 Gas Detector" },
        { 0x8F76, [x2200]       ,   "Analytical    UV Flame Detector" },
        { 0x8F77, [x5200]       ,   "Analytical    UVIR Flame Detector" },
        { 0x8F78, [x9800]       ,   "Analytical    IR Flame Detector" },
        { 0x8F79, [x3302]       ,   "Analytical    Fire Detector" },
        { 0x8F7A, [ud10]        ,   "Analytical    Gas Detector Display Interface" },
        { 0x8F7B, [gt3000]      ,   "Analytical    Toxic Gas Detector" },
        { 0x8F7C, [x3301]       ,   "Analytical    Flame Detector" },
        { 0x8F7E, [opecl_rx]    ,   "Analytical    Infrared Gas Detector" },
        { 0x8F7F, [eclipse]     ,   "Analytical    Infrared Gas Detector" }
    }
}

/*************************************************************************************
 * Thermo MeasureTech Device Type Codes
 */
VARIABLE Thermo_MeasureTech_model_code    /* 0x0090 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007B, [autoxp]    ,   "Analytical    Multivariable" },
        { 0x007C, [ms2011]    ,   "Level         w/ Density" },
        { 0x007D, [ndmi]      ,   "Analytical    Density - Nuclear" },
        { 0x007E, [accu_wave] ,   "Level         Radar" },
        { 0x007F, [ncmi]      ,   "Level         Nuclear" },
        { 0x907B, [autoxp]    ,   "Analytical    Multivariable" },
        { 0x907C, [ms2011]    ,   "Level         w/ Density" },
        { 0x907D, [ndmi]      ,   "Analytical    Density - Nuclear" },
        { 0x907E, [accu_wave] ,   "Level         Radar" },
        { 0x907F, [ncmi]      ,   "Level         Nuclear" }
    }
}

/*************************************************************************************
 * Dezurik Device Type Codes
 */
VARIABLE Dezurik_model_code    /* 0x0091 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [dezurik_positioner] ,   "Valve         Positioner/Electropneumatic" },
        { 0x91EF, [dezurik_positioner] ,   "Valve         Positioner/Electropneumatic" }
    }
}

/*************************************************************************************
 * Phase Dynamics Device Type Codes
 */
VARIABLE Phase_Dynamics_model_code    /* 0x0092 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [analyzer] ,   "Analytical    Microwave" },
        { 0x927F, [analyzer] ,   "Analytical    Microwave" }
    }
}

/*************************************************************************************
 * Welltech Device Type Codes
 */
VARIABLE Welltech_model_code    /* 0x0093 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007A, [wt3600] },
        { 0x007B, [wt3000] },
        { 0x007C, [wt2000] ,   "Pressure" },
        { 0x937A, [wt3600] },
        { 0x937B, [wt3000] },
        { 0x937C, [wt2000] ,   "Pressure" }
    }
}

/*************************************************************************************
 * ENRAF Device Type Codes
 */
VARIABLE ENRAF_model_code    /* 0x0094 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [smartservo_flexline_954] ,   "Level         Servo" },
        { 0x9480, [smartservo_flexline_954] ,   "Level         Servo" }
    }
}

/*************************************************************************************
 * Metran Device Type Codes
 */
VARIABLE Metran_model_code    /* 0x0099 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0077, [metran_303pr]  ,   "Flow" },
        { 0x0079, [metran_150]    ,   "Pressure" },
        { 0x007B, [metran_390]    },
        { 0x007C, [metran_300prm] },
        { 0x007D, [metran_49]     },
        { 0x007E, [metran_280]    ,   "Temperature" },
        { 0x007F, [metran_100]    ,   "Pressure" },
        { 0x9977, [metran_303pr]  ,   "Flow" },
        { 0x9979, [metran_150]    ,   "Pressure" },
        { 0x997B, [metran_390]    },
        { 0x997C, [metran_300prm] },
        { 0x997D, [metran_49]     },
        { 0x997E, [metran_280]    ,   "Temperature" },
        { 0x997F, [metran_100]    ,   "Pressure" }
    }
}

/*************************************************************************************
 * Milton Roy Co. Device Type Codes
 */
VARIABLE Milton_Roy_model_code    /* 0x009A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [p_series] },
        { 0x9A7F, [p_series] }
    }
}

/*************************************************************************************
 * PMV Device Type Codes
 */
VARIABLE PMV_model_code    /* 0x009B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00D3, [d3] ,   "Valve         Positioner" },
        { 0x9BD3, [d3] ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Turck Device Type Codes
 */
VARIABLE Turck_model_code    /* 0x009C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [kmu_hli] ,   "Temperature" },
        { 0x9C01, [kmu_hli] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Panametrics Device Type Codes
 */
VARIABLE Panametrics_model_code    /* 0x009D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0070, [x868]        ,   "Flow          Ultrasonic" },
        { 0x0077, [xmt900]      },
        { 0x0079, [xmt1000_ufm] ,   "Flow          Ultrasonic" },
        { 0x9D70, [x868]        ,   "Flow          Ultrasonic" },
        { 0x9D77, [xmt900]      },
        { 0x9D79, [xmt1000_ufm] ,   "Flow          Ultrasonic" }
    }
}

/*************************************************************************************
 * Stahl Device Type Codes
 */
VARIABLE Stahl_model_code    /* 0x009E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [ics_mux]          },
        { 0x0002, [ics_1000]         },
        { 0x0003, [vos_200]          },
        { 0x0004, [ics_9148]         },
        { 0x0005, [is_remote_io]     },
        { 0x0006, [is_pac_9192_16ch] },
        { 0x0007, [is_pac_9192_32ch] },
        { 0x9E01, [ics_mux]          },
        { 0x9E02, [ics_1000]         },
        { 0x9E03, [vos_200]          },
        { 0x9E04, [ics_9148]         },
        { 0x9E05, [is_remote_io]     },
        { 0x9E06, [is_pac_9192_16ch] },
        { 0x9E07, [is_pac_9192_32ch] }
    }
}

/*************************************************************************************
 * Analytical Technology Inc. Device Type Codes
 */
VARIABLE Analytical_Technology_Inc_model_code    /* 0x009F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [D12] ,   "Analytical    Gas Detector" },
        { 0x9F7F, [D12] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * FINT Device Type Codes
 */
VARIABLE FINT_model_code    /* 0x00A0 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0002, [pir_2000] ,   "Analytical    Gas Detector" },
        { 0x00ED, [oxi5000]  ,   "Analytical    Oxygen" },
        { 0xA002, [pir_2000] ,   "Analytical    Gas Detector" },
        { 0xA0ED, [oxi5000]  ,   "Analytical    Oxygen" }
    }
}

/*************************************************************************************
 * BERTHOLD Device Type Codes
 */
VARIABLE BERTHOLD_model_code    /* 0x00A1 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0075, [lb414]             ,   "Analytical    Density - Radiometric" },
        { 0x0079, [lb480_density]     ,   "Analytical    Density - Radiometric" },
        { 0x007A, [lb480_level]       ,   "Level" },
        { 0x007B, [lb480_levelswitch] ,   "Level         Radiometric" },
        { 0x007C, [lb466]             },
        { 0x007D, [lb491]             ,   "Analytical    Density - Radiometric" },
        { 0x007F, [lb490]             ,   "Level         Radiometric" },
        { 0xA175, [lb414]             ,   "Analytical    Density - Radiometric" },
        { 0xA179, [lb480_density]     ,   "Analytical    Density - Radiometric" },
        { 0xA17A, [lb480_level]       ,   "Level" },
        { 0xA17B, [lb480_levelswitch] ,   "Level         Radiometric" },
        { 0xA17C, [lb466]             },
        { 0xA17D, [lb491]             ,   "Analytical    Density - Radiometric" },
        { 0xA17F, [lb490]             ,   "Level         Radiometric" }
    }
}

/*************************************************************************************
 * InterCorr Device Type Codes
 */
VARIABLE InterCorr_model_code    /* 0x00A2 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [smartcet_18c] ,   "Analytical    Corrosion" },
        { 0xA2EF, [smartcet_18c] ,   "Analytical    Corrosion" }
    }
}

/*************************************************************************************
 * Sierra Instruments Device Type Codes
 */
VARIABLE Sierra_Instruments_model_code    /* 0x00A5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [sierra_i_series] ,   "Flow          Mass" },
        { 0xA57F, [sierra_i_series] ,   "Flow          Mass" }
    }
}

/*************************************************************************************
 * Fluid Components Device Type Codes
 */
VARIABLE Fluid_Components_model_code    /* 0x00A6 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0077, [st80_st100a] ,   "Flow          Mass" },
        { 0x0078, [st98]        ,   "Flow" },
        { 0x0079, [mt100]       ,   "Flow" },
        { 0x007E, [st50]        ,   "Flow" },
        { 0x007F, [st100]       ,   "Flow          Thermal" },
        { 0xA677, [st80_st100a] ,   "Flow          Mass" },
        { 0xA678, [st98]        ,   "Flow" },
        { 0xA679, [mt100]       ,   "Flow" },
        { 0xA67E, [st50]        ,   "Flow" },
        { 0xA67F, [st100]       ,   "Flow          Thermal" }
    }
}

/*************************************************************************************
 * Meriam Instrument Device Type Codes
 */
VARIABLE Meriam_Instrument_model_code    /* 0x00A8 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [m3500] ,   "Pressure      Multivariable" },
        { 0xA87F, [m3500] ,   "Pressure      Multivariable" }
    }
}

/*************************************************************************************
 * Invensys Device Type Codes
 */
VARIABLE Invensys_model_code    /* 0x00A9 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x002E, [inv_ia_pressure] ,   "Pressure" },
        { 0x0034, [cft50]           },
        { 0xA92E, [inv_ia_pressure] ,   "Pressure" },
        { 0xA934, [cft50]           }
    }
}

/*************************************************************************************
 * CRANE Device Type Codes
 *     Note: was Pentair Valves & Controls
 */
VARIABLE CRANE_model_code    /* 0x00AB */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [smartcal] ,   "Valve         Digital Positioner w/ PID" },
        { 0xAB01, [smartcal] ,   "Valve         Digital Positioner w/ PID" }
    }
}

/*************************************************************************************
 * J-Tec Associates Device Type Codes
 */
VARIABLE JTec_Associates_model_code    /* 0x00AD */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0000, [jtec_gasflowmeter]    ,   "Analytical    Gas/Steam" },
        { 0x0001, [jtec_liquidflowmeter] },
        { 0xAD00, [jtec_gasflowmeter]    ,   "Analytical    Gas/Steam" },
        { 0xAD01, [jtec_liquidflowmeter] }
    }
}

/*************************************************************************************
 * Tracerco Device Type Codes
 */
VARIABLE Tracerco_model_code    /* 0x00AE */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007A, [t215]                   },
        { 0x007B, [t251]                   },
        { 0x007E, [pri_160]                },
        { 0x007F, [pri_150]                ,   "Level         Nucleonic" },
        { 0x009F, [level_combination_unit] },
        { 0x00A0, "T251-X-5"               ,   "Level         w/ Density" },
        { 0xAE7A, [t215]                   },
        { 0xAE7B, [t251]                   },
        { 0xAE7E, [pri_160]                },
        { 0xAE7F, [pri_150]                ,   "Level         Nucleonic" },
        { 0xAE9F, [level_combination_unit] },
        { 0xAEA0, "T251-X-5"               ,   "Level         w/ Density" }
    }
}

/*************************************************************************************
 * AGAR Device Type Codes
 */
VARIABLE AGAR_model_code    /* 0x00AF */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007E, [id200] ,   "Analytical    % water" },
        { 0xAF7E, [id200] ,   "Analytical    % water" }
    }
}

/*************************************************************************************
 * Phoenix Contact Device Type Codes
 */
VARIABLE Phoenix_Contact_model_code    /* 0x00B0 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [mcr_ts_lp]           ,   "Temperature   DIN Rail Mounted" },
        { 0x0002, [mcr_ht_ts]           ,   "Temperature   Head Mounted" },
        { 0x0003, [macx_mcr_s_mux_16ch] },
        { 0x0004, [macx_mcr_s_mux_32ch] },
        { 0x0005, [mcr_ts_i_olp]        ,   "Temperature" },
        { 0x0006, "MCR-()-1TS-I-OLP"    ,   "Temperature" },
        { 0x0010, [rad_whg_wlan_xd]     },
        { 0x0011, [rad_wha]             },
        { 0x0012, [gw_pl_eth_basic_bus] ,   "IOSystem/Gateway" },
        { 0x0013, [gw_pl_eth_uni_bus]   ,   "IOSystem/Gateway" },
        { 0xB001, [mcr_ts_lp]           ,   "Temperature   DIN Rail Mounted" },
        { 0xB002, [mcr_ht_ts]           ,   "Temperature   Head Mounted" },
        { 0xB003, [macx_mcr_s_mux_16ch] },
        { 0xB004, [macx_mcr_s_mux_32ch] },
        { 0xB005, [mcr_ts_i_olp]        ,   "Temperature" },
        { 0xB006, "MCR-()-1TS-I-OLP"    ,   "Temperature" },
        { 0xB010, [rad_whg_wlan_xd]     },
        { 0xB011, [rad_wha]             },
        { 0xB012, [gw_pl_eth_basic_bus] ,   "IOSystem/Gateway" },
        { 0xB013, [gw_pl_eth_uni_bus]   ,   "IOSystem/Gateway" }
    }
}

/*************************************************************************************
 * Amer. Level Instr. Device Type Codes
 */
VARIABLE Amer_Level_Instr_model_code    /* 0x00B2 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [model_32e] ,   "Level         Capacitance" },
        { 0xB27F, [model_32e] ,   "Level         Capacitance" }
    }
}

/*************************************************************************************
 * Hawk Device Type Codes
 */
VARIABLE Hawk_model_code    /* 0x00B3 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [centurion_guided_radar] ,   "Level         Radar" },
        { 0x0002, [senator_24_80_radar]    ,   "Level         Radar" },
        { 0xB301, [centurion_guided_radar] ,   "Level         Radar" },
        { 0xB302, [senator_24_80_radar]    ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * Rotork YTC Device Type Codes
 *     Note: was YTC 
 */
VARIABLE Rotork_YTC_model_code    /* 0x00B4 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [yt_2300]     ,   "Valve         Positioner" },
        { 0x0002, [yt_2400]     ,   "Valve         Positioner" },
        { 0x0003, [yt_2500]     },
        { 0x0004, [yt_2600]     },
        { 0x0005, [yt_2700]     },
        { 0x0007, [yt_3400]     ,   "Valve         Positioner" },
        { 0x0009, [yt_3300]     ,   "Valve         Positioner" },
        { 0x000A, [sptm]        ,   "Valve         Positioner Transmitter" },
        { 0x000B, [yt_3300_pro] ,   "Valve         Positioner" },
        { 0x007F, [c330]        ,   "Valve         Positioner" },
        { 0xB401, [yt_2300]     ,   "Valve         Positioner" },
        { 0xB402, [yt_2400]     ,   "Valve         Positioner" },
        { 0xB403, [yt_2500]     },
        { 0xB404, [yt_2600]     },
        { 0xB405, [yt_2700]     },
        { 0xB407, [yt_3400]     ,   "Valve         Positioner" },
        { 0xB409, [yt_3300]     ,   "Valve         Positioner" },
        { 0xB40A, [sptm]        ,   "Valve         Positioner Transmitter" },
        { 0xB40B, [yt_3300_pro] ,   "Valve         Positioner" },
        { 0xB47F, [c330]        ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Pyromation Inc. Device Type Codes
 */
VARIABLE Pyromation_Inc_model_code    /* 0x00B5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [series442] ,   "Temperature" },
        { 0x0080, [series642] ,   "Temperature" },
        { 0x0081, [t82]       ,   "Temperature" },
        { 0x0082, [series662] ,   "Temperature" },
        { 0xB57F, [series442] ,   "Temperature" },
        { 0xB580, [series642] ,   "Temperature" },
        { 0xB581, [t82]       ,   "Temperature" },
        { 0xB582, [series662] ,   "Temperature" }
    }
}

/*************************************************************************************
 * Satron Instruments Device Type Codes
 */
VARIABLE Satron_Instruments_model_code    /* 0x00B6 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [v_series] ,   "Pressure" },
        { 0xB67F, [v_series] ,   "Pressure" }
    }
}

/*************************************************************************************
 * BIFFI Device Type Codes
 */
VARIABLE BIFFI_model_code    /* 0x00B7 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007C, [hrt_imvs2000v2] ,   "Valve         Actuator" },
        { 0x007D, [hrtimvs2000]    ,   "Valve         Positioner" },
        { 0x007E, [hrt2000v4]      ,   "Valve         Actuator" },
        { 0xB77C, [hrt_imvs2000v2] ,   "Valve         Actuator" },
        { 0xB77D, [hrtimvs2000]    ,   "Valve         Positioner" },
        { 0xB77E, [hrt2000v4]      ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * BD Sensors Device Type Codes
 */
VARIABLE BD_Sensors_model_code    /* 0x00B9 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007F, [dmd_331_a] ,   "Pressure" },
        { 0xB97F, [dmd_331_a] ,   "Pressure" }
    }
}

/*************************************************************************************
 * APLISENS Device Type Codes
 */
VARIABLE APLISENS_model_code    /* 0x00BC */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007A, [li_24]       ,   "Temperature" },
        { 0x007B, [apc_2000alw] ,   "Pressure" },
        { 0xBC7A, [li_24]       ,   "Temperature" },
        { 0xBC7B, [apc_2000alw] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Badger Meter Device Type Codes
 */
VARIABLE Badger_Meter_model_code    /* 0x00BD */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [primo_advanced] ,   "Flow          Electromagnetic" },
        { 0x0002, [m2000]          ,   "Flow          Magnetic" },
        { 0xBD01, [primo_advanced] ,   "Flow          Electromagnetic" },
        { 0xBD02, [m2000]          ,   "Flow          Magnetic" }
    }
}

/*************************************************************************************
 * HIMA Device Type Codes
 */
VARIABLE HIMA_model_code    /* 0x00BE */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007E, [himax_safety_system] },
        { 0x007F, [h_6210_safety_mux]   },
        { 0xBE7E, [himax_safety_system] },
        { 0xBE7F, [h_6210_safety_mux]   }
    }
}

/*************************************************************************************
 * GP:50 Device Type Codes
 */
VARIABLE GP_50_model_code    /* 0x00BF */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x007C, [mdl_400] ,   "Pressure" },
        { 0xBF7C, [mdl_400] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Kongsberg Maritime Device Type Codes
 */
VARIABLE Kongsberg_Maritime_model_code    /* 0x00C0 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00EF, [gt400] ,   "Pressure" },
        { 0xC0EF, [gt400] ,   "Pressure" }
    }
}

/*************************************************************************************
 * ASA S.p.A. Device Type Codes
 */
VARIABLE ASA_SpA_model_code    /* 0x00C1 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [cg_asameter] },
        { 0xC101, [cg_asameter] }
    }
}

/*************************************************************************************
 * Hengesbach Device Type Codes
 */
VARIABLE Hengesbach_model_code    /* 0x00C2 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0021, [te_52] ,   "Temperature   Head Mounted" },
        { 0xC221, [te_52] ,   "Temperature   Head Mounted" }
    }
}

/*************************************************************************************
 * Lanlian Instruments Device Type Codes
 */
VARIABLE Lanlian_Instruments_model_code    /* 0x00C3 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0074, [lsiii_pressure] ,   "Pressure" },
        { 0xC374, [lsiii_pressure] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Spectrum Controls Device Type Codes
 */
VARIABLE Spectrum_Controls_model_code    /* 0x00C4 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0001, [spectrum_io_module] },
        { 0xC401, [spectrum_io_module] }
    }
}

/*************************************************************************************
 * Kajaani Process Measurements Device Type Codes
 */
VARIABLE Kajaani_Process_Measurements_model_code    /* 0x00C5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [kc3] ,   "Analytical    Blade Consistency" },
        { 0x0081, [kc5] ,   "Analytical    Rotary Consistency" },
        { 0xC580, [kc3] ,   "Analytical    Blade Consistency" },
        { 0xC581, [kc5] ,   "Analytical    Rotary Consistency" }
    }
}

/*************************************************************************************
 * FAFNIR Device Type Codes
 */
VARIABLE FAFNIR_model_code    /* 0x00C6 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [torrix] },
        { 0xC680, [torrix] }
    }
}

/*************************************************************************************
 * SICK Engineering GmbH Device Type Codes
 *     Note: was SICK-MAIHAK
 */
VARIABLE SICK_model_code    /* 0x00C7 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [flowsic]     ,   "Flow          Ultrasonic" },
        { 0x0081, [flowsic_100] ,   "Flow          Ultrasonic" },
        { 0x0082, [flowsic_30]  ,   "Flow          Ultrasonic - Gas" },
        { 0x0083, [flowsic_iu]  ,   "Flow          Ultrasonic - Gas" },
        { 0x0084, "LFR SicWave" ,   "Level         Radar" },
        { 0x0085, "LBR SicWave" ,   "Level         Radar" },
        { 0xC780, [flowsic]     ,   "Flow          Ultrasonic" },
        { 0xC781, [flowsic_100] ,   "Flow          Ultrasonic" },
        { 0xC782, [flowsic_30]  ,   "Flow          Ultrasonic - Gas" },
        { 0xC783, [flowsic_iu]  ,   "Flow          Ultrasonic - Gas" },
        { 0xC784, "LFR SicWave" ,   "Level         Radar" },
        { 0xC785, "LBR SicWave" ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * Spirax Sarco SRL Device Type Codes
 *     Note: was Spirax Sarco Plc 
 */
VARIABLE Spirax_Sarco_SRL_model_code    /* 0x00CA */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [sp301]        ,   "Valve         Smart Valve Positioner" },
        { 0x0081, [sp500]        },
        { 0x0082, [wireless_teg] },
        { 0x0090, [eft10]        },
        { 0xCA80, [sp301]        ,   "Valve         Smart Valve Positioner" },
        { 0xCA81, [sp500]        },
        { 0xCA82, [wireless_teg] },
        { 0xCA90, [eft10]        }
    }
}

/*************************************************************************************
 * Tecfluid SA Device Type Codes
 */
VARIABLE Tecfluid_SA_model_code    /* 0x00CC */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [ch420l] },
        { 0x0081, [ch420r] },
        { 0xCC80, [ch420l] },
        { 0xCC81, [ch420r] }
    }
}

/*************************************************************************************
 * Roost Device Type Codes
 */
VARIABLE Roost_model_code    /* 0x00CE */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [epp300] ,   "Valve         Positioner/Electropneumatic" },
        { 0xCE80, [epp300] ,   "Valve         Positioner/Electropneumatic" }
    }
}

/*************************************************************************************
 * KOSO Device Type Codes
 */
VARIABLE KOSO_model_code    /* 0x00CF */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [ep1000]  ,   "Valve         Positioner/Electropneumatic" },
        { 0x0081, [ep1001]  ,   "Valve         Positioner/Electropneumatic" },
        { 0x0083, [kgp5000] ,   "Valve         Positioner" },
        { 0xCF80, [ep1000]  ,   "Valve         Positioner/Electropneumatic" },
        { 0xCF81, [ep1001]  ,   "Valve         Positioner/Electropneumatic" },
        { 0xCF83, [kgp5000] ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Honeywell Analytics Device Type Codes
 */
VARIABLE Honeywell_Analytics_model_code    /* 0x00D2 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [xnx]              ,   "Analytical    Gas Detector" },
        { 0x0081, [Optima_plus]      ,   "Analytical    Gas Detector" },
        { 0x0082, [fsxhcom]          },
        { 0x0083, [searchzone_sonik] ,   "Analytical    Gas Detector" },
        { 0xD280, [xnx]              ,   "Analytical    Gas Detector" },
        { 0xD281, [Optima_plus]      ,   "Analytical    Gas Detector" },
        { 0xD282, [fsxhcom]          },
        { 0xD283, [searchzone_sonik] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * HEINRICHS Device Type Codes
 */
VARIABLE HEINRICHS_model_code    /* 0x00D3 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [pad]   ,   "Pressure      Absolute/Differential" },
        { 0x0081, [pas]   ,   "Pressure      Absolute" },
        { 0x0082, [pad_f] ,   "Pressure      Differential" },
        { 0x00EA, [umf3]  ,   "Flow          Magnetic Inductive" },
        { 0xD380, [pad]   ,   "Pressure      Absolute/Differential" },
        { 0xD381, [pas]   ,   "Pressure      Absolute" },
        { 0xD382, [pad_f] ,   "Pressure      Differential" },
        { 0xD3EA, [umf3]  ,   "Flow          Magnetic Inductive" }
    }
}

/*************************************************************************************
 * SIC Device Type Codes
 */
VARIABLE SIC_model_code    /* 0x00D4 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [pes]         ,   "Pressure" },
        { 0x0081, [pds]         ,   "Pressure" },
        { 0x0082, [hvp]         ,   "Positioner" },
        { 0x0085, [hvp1114]     ,   "Valve         Positioner/Pneumatic" },
        { 0x0086, [flow_master] ,   "Flow          Electromagnetic" },
        { 0x0087, [tts]         ,   "Temperature" },
        { 0x0090, [mps]         ,   "Level         Radar" },
        { 0xD480, [pes]         ,   "Pressure" },
        { 0xD481, [pds]         ,   "Pressure" },
        { 0xD482, [hvp]         ,   "Positioner" },
        { 0xD485, [hvp1114]     ,   "Valve         Positioner/Pneumatic" },
        { 0xD486, [flow_master] ,   "Flow          Electromagnetic" },
        { 0xD487, [tts]         ,   "Temperature" },
        { 0xD490, [mps]         ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * HACH LANGE Device Type Codes
 */
VARIABLE HACH_LANGE_model_code    /* 0x00D5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [si792p]                          ,   "Analytical    pH" },
        { 0x0081, [si792c]                          ,   "Analytical    Conductivity" },
        { 0x0082, [si792t]                          ,   "Analytical    Conductivity" },
        { 0x0083, [si792e]                          ,   "Analytical    Conductivity" },
        { 0x0084, [si792d]                          ,   "Analytical    Dissolved Oxygen" },
        { 0x0086, [sc200_ph_orp]                    ,   "Analytical    pH/ORP" },
        { 0x0087, [ldo]                             ,   "Analytical    Dissolved Oxygen" },
        { 0x0088, [sc200_cnt_cond]                  ,   "Analytical    Conductivity - Contacting" },
        { 0x0089, [sc200_ind_cond]                  ,   "Analytical    Conductivity - Inductive" },
        { 0x008A, [sc200_do]                        ,   "Analytical    Dissolved Oxygen" },
        { 0x008B, [sc200_flow]                      ,   "Flow          Ultrasonic" },
        { 0x008C, [ss7_turbidimeter]                ,   "Analytical    Turbidity" },
        { 0x008D, [sc200_analog_input]              ,   "Analytical    4-20mA Input " },
        { 0x008E, [sc_ph_orp]                       ,   "Analytical    pH/ORP" },
        { 0x008F, [fp360sc]                         ,   "Analytical    Oil in Water" },
        { 0x0090, [tsssc]                           ,   "Analytical    Suspended Solids" },
        { 0x0091, [sc1720e_turbidimeter]            ,   "Analytical    Turbidity" },
        { 0x00AF, [ldo_2]                           ,   "Analytical    Dissolved Oxygen" },
        { 0x00B0, [sc200_conductivity_cco]          ,   "Analytical    Conductivity" },
        { 0x00B1, [sc200_conductivity_ico]          ,   "Analytical    Conductivity - Inductive" },
        { 0x00B2, [sc200_ph]                        ,   "Analytical    pH" },
        { 0x00B3, [sc200_amperometric]              ,   "Analytical    Oxygen" },
        { 0x00B4, [sc200_amperometric_o2_scavenger] ,   "Analytical    Dissolved Oxygen" },
        { 0x00B5, [nax600sc_sodium]                 ,   "Analytical" },
        { 0xD580, [si792p]                          ,   "Analytical    pH" },
        { 0xD581, [si792c]                          ,   "Analytical    Conductivity" },
        { 0xD582, [si792t]                          ,   "Analytical    Conductivity" },
        { 0xD583, [si792e]                          ,   "Analytical    Conductivity" },
        { 0xD584, [si792d]                          ,   "Analytical    Dissolved Oxygen" },
        { 0xD586, [sc200_ph_orp]                    ,   "Analytical    pH/ORP" },
        { 0xD587, [ldo]                             ,   "Analytical    Dissolved Oxygen" },
        { 0xD588, [sc200_cnt_cond]                  ,   "Analytical    Conductivity - Contacting" },
        { 0xD589, [sc200_ind_cond]                  ,   "Analytical    Conductivity - Inductive" },
        { 0xD58A, [sc200_do]                        ,   "Analytical    Dissolved Oxygen" },
        { 0xD58B, [sc200_flow]                      ,   "Flow          Ultrasonic" },
        { 0xD58C, [ss7_turbidimeter]                ,   "Analytical    Turbidity" },
        { 0xD58D, [sc200_analog_input]              ,   "Analytical    4-20mA Input " },
        { 0xD58E, [sc_ph_orp]                       ,   "Analytical    pH/ORP" },
        { 0xD58F, [fp360sc]                         ,   "Analytical    Oil in Water" },
        { 0xD590, [tsssc]                           ,   "Analytical    Suspended Solids" },
        { 0xD591, [sc1720e_turbidimeter]            ,   "Analytical    Turbidity" },
        { 0xD5AF, [ldo_2]                           ,   "Analytical    Dissolved Oxygen" },
        { 0xD5B0, [sc200_conductivity_cco]          ,   "Analytical    Conductivity" },
        { 0xD5B1, [sc200_conductivity_ico]          ,   "Analytical    Conductivity - Inductive" },
        { 0xD5B2, [sc200_ph]                        ,   "Analytical    pH" },
        { 0xD5B3, [sc200_amperometric]              ,   "Analytical    Oxygen" },
        { 0xD5B4, [sc200_amperometric_o2_scavenger] ,   "Analytical    Dissolved Oxygen" },
        { 0xD5B5, [nax600sc_sodium]                 ,   "Analytical" }
    }
}

/*************************************************************************************
 * Manometr-Kharkiv Device Type Codes
 */
VARIABLE Manometr_Kharkiv_model_code    /* 0x00D9 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [safir] ,   "Pressure" },
        { 0xD980, [safir] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Spectrex Device Type Codes
 */
VARIABLE Spectrex_model_code    /* 0x00DB */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [sharpeye]                ,   "Analytical    Flame Detector" },
        { 0x0081, [safeye]                  ,   "Analytical    Optical Open Path Gas Detector" },
        { 0x0083, "Spectrex SharpEye 40/40" ,   "Analytical    Flame Detector" },
        { 0xDB80, [sharpeye]                ,   "Analytical    Flame Detector" },
        { 0xDB81, [safeye]                  ,   "Analytical    Optical Open Path Gas Detector" },
        { 0xDB83, "Spectrex SharpEye 40/40" ,   "Analytical    Flame Detector" }
    }
}

/*************************************************************************************
 * REXA Device Type Codes
 */
VARIABLE REXA_model_code    /* 0x00DE */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0081, "X3" ,   "Valve         Actuator" },
        { 0xDE81, "X3" ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * General Monitors Device Type Codes
 */
VARIABLE General_Monitors_MSA_model_code    /* 0x00DF */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [fl4000]               ,   "Analytical    Flame Detector" },
        { 0x0081, [s4000ch]              ,   "Analytical    Combustible Gas Detector" },
        { 0x0082, [s4000th]              ,   "Analytical    Toxic Gas Detector" },
        { 0x0083, [ir400]                ,   "Analytical    Point Infrared Gas Detector" },
        { 0x0084, [ir4000]               ,   "Analytical    Multi-Detector Monitor System" },
        { 0x0085, [s4100ch]              ,   "Analytical    Gas Detector" },
        { 0x0087, [ir5500_ultima_opir_5] ,   "Analytical    Infrared Open Path Gas Detector" },
        { 0x0088, [ir700]                ,   "Analytical    Infrared Point Detector" },
        { 0x0089, [ts4000h]              ,   "Analytical    Toxic Gas Detector" },
        { 0x0090, [fl310xh]              ,   "Analytical    Flame Detector" },
        { 0x0093, [fl500]                ,   "Analytical    Flame Detector" },
        { 0x0095, [obs_h_ultr_ex_5]      ,   "Analytical    Ultrasonic Gas Detector" },
        { 0x0096, [observer_i]           ,   "Analytical    Ultrasonic Gas Detector" },
        { 0xDF80, [fl4000]               ,   "Analytical    Flame Detector" },
        { 0xDF81, [s4000ch]              ,   "Analytical    Combustible Gas Detector" },
        { 0xDF82, [s4000th]              ,   "Analytical    Toxic Gas Detector" },
        { 0xDF83, [ir400]                ,   "Analytical    Point Infrared Gas Detector" },
        { 0xDF84, [ir4000]               ,   "Analytical    Multi-Detector Monitor System" },
        { 0xDF85, [s4100ch]              ,   "Analytical    Gas Detector" },
        { 0xDF87, [ir5500_ultima_opir_5] ,   "Analytical    Infrared Open Path Gas Detector" },
        { 0xDF88, [ir700]                ,   "Analytical    Infrared Point Detector" },
        { 0xDF89, [ts4000h]              ,   "Analytical    Toxic Gas Detector" },
        { 0xDF90, [fl310xh]              ,   "Analytical    Flame Detector" },
        { 0xDF93, [fl500]                ,   "Analytical    Flame Detector" },
        { 0xDF95, [obs_h_ultr_ex_5]      ,   "Analytical    Ultrasonic Gas Detector" },
        { 0xDF96, [observer_i]           ,   "Analytical    Ultrasonic Gas Detector" }
    }
}

/*************************************************************************************
 * Manufacturer Expansion Device Type Codes
 */
VARIABLE manufacturer_expansion_model_code    /* 0x00E0 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [esd3000]               },
        { 0x00EC, [est3051t]              },
        { 0x00ED, [est3051s]              },
        { 0x0082, [sdt420]                ,   "Level" },
        { 0x00B4, [stf]                   ,   "Flow          Thermal" },
        { 0x00B5, [smf]                   ,   "Flow          Electromagnetic" },
        { 0x0083, [rt_ex15]               },
        { 0x0085, [apm_3dlevelscanner_m]  },
        { 0x0086, [apm_3dlevelscanner_s]  },
        { 0x0087, [apm_3dlevelscanner_l]  },
        { 0x0088, [ex200h_positioner]     ,   "Valve         Positioner/Electro-hydraulic" },
        { 0x0089, [is200h_positioner]     },
        { 0x008A, [r3x]                   },
        { 0x008B, [detcon_700_bridge]     ,   "Analytical    Gas Detector" },
        { 0x008C, [ultima_xl_xt]          ,   "Analytical    Gas Detector" },
        { 0x009F, [ultimaxeh]             ,   "Analytical    Gas Detector" },
        { 0x00BC, [flamegard]             },
        { 0x00F2, [safeye]                },
        { 0x00F3, [primax]                ,   "Analytical    Gas Detector" },
        { 0x00F6, [primax_ir]             ,   "Analytical    Gas Detector" },
        { 0x008D, [cmm_01_mass_flow]      ,   "Flow          Coriolis Mass" },
        { 0x00C8, [cvm_01]                },
        { 0x00C9, [tdm_01]                },
        { 0x0081, [sensor_don]            },
        { 0x008E, [jlt_6000]              },
        { 0x008F, [smartronic_ma]         },
        { 0x0090, [cms_standard]          },
        { 0x0091, [millennium_2]          },
        { 0x009E, [millennium_2_basic]    ,   "Analytical    Gas Detector" },
        { 0x00DF, [jb_mphf_series]        ,   "Analytical    Fire Detector" },
        { 0x0092, [him1008]               },
        { 0x0093, "CXT_CJT"               ,   "Pressure" },
        { 0x0094, [cjt]                   },
        { 0x0095, [hbm_165_h]             ,   "Analytical    pH/ORP" },
        { 0x0096, [hbm_167_h]             },
        { 0x0097, [wbm_165_h]             ,   "Analytical    Conductivity" },
        { 0x0098, [obm_165_h]             },
        { 0x0099, [obm_167_h]             },
        { 0x009A, [mbm_165_h]             },
        { 0x009B, [mbm_167_h]             },
        { 0x009C, [mark_series]           },
        { 0x009D, [tr]                    },
        { 0x00CE, [valve_monitor_4310]    },
        { 0x00EB, [valve_controller_4350] ,   "Valve         Controller" },
        { 0x00A1, [hrt1]                  ,   "Flow          Totalizer - Rate Indicator" },
        { 0x00F1, [freq_ma]               ,   "Flow          Totalizer - Rate Indicator" },
        { 0x00A2, [dn2510_smartmesh]      },
        { 0x00A3, [m2510_smartmesh]       },
        { 0x00A4, [pm2510_smartmesh]      },
        { 0x00A5, [vfm_4097]              },
        { 0x00A6, [ifc_031]               },
        { 0x00A7, [ifc_021]               },
        { 0x00A9, [smartpro_ph_8966]      },
        { 0x00AB, [am_2000]               },
        { 0x00AC, [mactek_wa]             ,   "WirelessHART  Adapter" },
        { 0x00AD, [csi_9420]              },
        { 0x00AE, [kms_2]                 },
        { 0x00AF, [mwt_3905]              },
        { 0x00B0, [uwt_1000]              },
        { 0x00B1, [cwt_9020]              },
        { 0x00F8, [mwt_we]                },
        { 0x00B2, [a_1000_h]              },
        { 0x00B3, [ncs_pt105_ii]          },
        { 0x00CF, [ncs_pt105ii_sp]        },
        { 0x00D0, [ncs_tt105]             },
        { 0x00B6, [ty_tds_p]              },
        { 0x00B7, [ty_trans_f]            },
        { 0x00B8, [ty_trans_l]            },
        { 0x00B9, [ty_trans_ps]           },
        { 0x00BA, [ty_trans_a]            },
        { 0x00BB, [pmh]                   },
        { 0x00BD, [fluxus]                ,   "Flow          Ultrasonic" },
        { 0x00BE, [mrg_10]                },
        { 0x00BF, [mrf_10]                },
        { 0x00C0, [ultramate_136_ult]     },
        { 0x00D8, [microguide_132_tdr]    },
        { 0x00D9, [micromate_138_plr]     },
        { 0x00C1, [flow_control_system]   },
        { 0x00C2, [axiom]                 ,   "Valve         Monitor" },
        { 0x00C3, [eumag]                 },
        { 0x00C5, [mtrt]                  },
        { 0x00C4, [ba47x_67x_series]      ,   "Temperature" },
        { 0x00C6, [cap_30xx]              },
        { 0x00C7, [nemesis_1000]          },
        { 0x00CB, [air_20h]               ,   "Pressure" },
        { 0x00E8, [elemer_100]            },
        { 0x00CD, [frankenstein]          },
        { 0x00D1, [wideplus_ba]           },
        { 0x00D2, [wideplus_a]            },
        { 0x00D3, [oxitec_5000]           ,   "Analytical    Oxygen" },
        { 0x00D4, [comtec_6000]           ,   "Analytical    Gas Detector" },
        { 0x00D5, [ihp]                   },
        { 0x00D6, [idp]                   },
        { 0x00D7, [mft_b_series]          ,   "Flow          Thermal" },
        { 0x00DB, [sgh350]                },
        { 0x00DC, [irmax]                 },
        { 0x00DD, [ir_display]            },
        { 0x00FC, [xgardiq]               ,   "Analytical    Gas Detector" },
        { 0x00DE, [tixo3]                 ,   "Temperature" },
        { 0x00E0, [master_touch]          },
        { 0x00E1, [dlt9000]               ,   "Level         Displacement" },
        { 0x00E2, [ffg_p_level]           ,   "Level" },
        { 0x00E3, [st_5700_steam]         ,   "Analytical    Steam Trap" },
        { 0x00E4, [ad_5000_acousitc]      ,   "Valve         Safety Relief Monitor" },
        { 0x00E5, [td_5100_temp]          },
        { 0x00E6, [n7]                    ,   "Pressure" },
        { 0x00E9, [a2_4_20ma_hrt]         },
        { 0x00EA, [f018p]                 ,   "Flow          Totalizer" },
        { 0x00EE, [mag_flux_m]            },
        { 0x00EF, [fox_ft3]               },
        { 0x00F0, [prototype_unit]        },
        { 0x00F4, [l_dens_427t]           },
        { 0x00F5, [ulm_70]                },
        { 0x00F7, [gd10]                  ,   "Analytical    Infrared Gas Detector" },
        { 0x00F9, [spiroster_07]          },
        { 0x00FF, [caliperay]             ,   "Analytical" }
    }
}

/*************************************************************************************
 * Manufacturer Expansion Device Type Codes
 */
VARIABLE manufacturer_expansion2_model_code    /* 0x00E1 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00B4, [rt_30]                  },
        { 0x00D2, [ex210h]                 },
        { 0x0096, [vcm_01]                 },
        { 0x00E3, [banshee_343]            },
        { 0x0091, [sf10e]                  },
        { 0x0094, [position_xmitter]       },
        { 0x00D3, [dn2600_ltc5800_whm]     },
        { 0x008E, [mwt_qs]                 },
        { 0x0085, [ncs_ft105]              },
        { 0x0086, [ncs_lt105]              },
        { 0x0087, [ncs_at105]              },
        { 0x0088, [ncs_mh105]              },
        { 0x0089, [ncs_pos105]             },
        { 0x00E9, [krg_10]                 ,   "Level         Radar" },
        { 0x008F, [elmag_151]              },
        { 0x00C5, [dbi]                    },
        { 0x00F1, "AIR-10SH"               ,   "Pressure" },
        { 0x00D8, [wideplus_rd]            },
        { 0x00D9, [wp_emf]                 },
        { 0x00DA, [wideplus_zf]            },
        { 0x00DB, [wideplus_um]            },
        { 0x00CF, [ihp24_a]                },
        { 0x0095, [sb350]                  },
        { 0x00CB, [tia3]                   },
        { 0x00AC, [eftn]                   },
        { 0x00AD, [dtu100]                 ,   "Level" },
        { 0x00AE, [mtl100]                 },
        { 0x00AF, [dlc3010]                },
        { 0x00B0, [flow100]                },
        { 0x00B7, [dpu100]                 },
        { 0x00B8, [dlu100]                 },
        { 0x00B9, [dfu100]                 },
        { 0x00A9, [dm_tv6_df_tv7]          },
        { 0x00D5, [dm_dg_gas]              },
        { 0x00A4, [rpw01]                  },
        { 0x00A5, [x_matik]                },
        { 0x00A6, [xi]                     },
        { 0x0092, [wd_h]                   },
        { 0x0080, [fschcom]                },
        { 0x0081, [magnetostrictive_lvl]   },
        { 0x0082, [rf_level_transmitter]   },
        { 0x0083, [radar_lvl_transmitter]  },
        { 0x00CC, [rf_cap_level_switch]    },
        { 0x00CD, [fork_level_switch]      },
        { 0x0084, [ea10s]                  ,   "Valve         Positioner" },
        { 0x00C7, [isnet_line]             },
        { 0x008A, [smart_actuator]         },
        { 0x008B, [warrior_sensor]         },
        { 0x008C, [iscan2]                 },
        { 0x008D, [tx200h]                 ,   "Pressure" },
        { 0x0090, [fst_3000]               ,   "Level" },
        { 0x0093, [mwlm_pr26]              },
        { 0x0097, [ptc_vr_sensor]          },
        { 0x00BE, [ptc_vr_sensor_wireless] },
        { 0x0098, [olct_200]               ,   "Analytical    Gas Detector" },
        { 0x00EA, [itrans_2]               ,   "Analytical    Gas Detector" },
        { 0x009D, [operafcs]               },
        { 0x009E, [tms_hps]                },
        { 0x00A0, [vl10]                   },
        { 0x00A1, [vs220]                  },
        { 0x00A2, [cmwa_8800]              },
        { 0x009C, [sgoes]                  ,   "Analytical    Combustible Gas Analyzer" },
        { 0x00AB, [vector_fcu]             ,   "Analytical    Gas Detector" },
        { 0x00DD, [tgaes_open_path]        },
        { 0x00A3, [at2000]                 },
        { 0x00D6, [at3000]                 },
        { 0x00D7, [at3600]                 },
        { 0x00B3, [h_comm]                 },
        { 0x00AA, [it_series]              ,   "Analytical    Gas Detector" },
        { 0x00B1, [hrm_v1_00]              },
        { 0x00B2, [wireless_interface]     },
        { 0x00B6, [abm200_xxxrxch]         },
        { 0x00B5, [sd_1]                   ,   "Analytical    Gas Detector" },
        { 0x00BA, [elmag200m]              },
        { 0x00BB, [tico_flow_meter]        },
        { 0x00BC, [hda4000]                },
        { 0x00E7, [bis_1000_apt]           },
        { 0x00BD, [daix]                   },
        { 0x00BF, [eim_cam06]              },
        { 0x00C1, [eim_cam206]             ,   "Valve         Actuator - Electrical" },
        { 0x00C2, [eim_cam06_h5]           },
        { 0x00C0, [corrlog_wireless]       },
        { 0x00C3, [gtd2000]                ,   "Analytical    Gas Detector" },
        { 0x00C6, [gir_3000]               ,   "Analytical    Gas Detector" },
        { 0x00C4, [visipro_do]             ,   "Analytical    Dissolved Oxygen" },
        { 0x00C8, [lcamp_1xx]              },
        { 0x00C9, [t80_transmitter]        },
        { 0x00CE, [sec5000irevolution]     },
        { 0x00D0, [elds_1000_2000]         },
        { 0x00D1, [kef]                    },
        { 0x00D4, [meridian_gas_wired]     ,   "Analytical    Gas Detector" },
        { 0x00DC, [model2010]              ,   "Flow" },
        { 0x00DE, [crocus]                 },
        { 0x00DF, [tribodsp3400h]          },
        { 0x00E0, [Wireless_50wm]          },
        { 0x00E1, [mat_matd_mats]          ,   "Level" },
        { 0x00E2, [sage_prime_rio]         ,   "Flow          Thermal" },
        { 0x00E4, [wg_315]                 },
        { 0x00E5, [tjiu406_m100]           },
        { 0x00E6, [gdu_02_incus]           },
        { 0x00E8, [zpzd31]                 },
        { 0x00EC, [st_4312]                ,   "Valve         Positioner Transmitter" },
        { 0x00FD, [aumatic_ac_acex]        ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * Manufacturer Expansion Device Type Codes
 */
VARIABLE manufacturer_expansion3_model_code    /* 0x00E2 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x00A1, [ss235]              ,   "Valve         Smart Valve Positioner" },
        { 0x00B3, [smartpoz_8400s]     ,   "Valve         Positioner" },
        { 0x00A6, [l_mag]              ,   "Flow          Magnetic" },
        { 0x00FA, [idc24]              ,   "Valve         Positioner" },
        { 0x00E7, [n8]                 ,   "Pressure" },
        { 0x00C6, [acm_2_hrt]          ,   "Valve         Positioner" },
        { 0x00CD, [e018p]              ,   "Flow" },
        { 0x00E8, [d58_dc]             ,   "Analytical    Gas Detector" },
        { 0x00E9, [gd_88]              ,   "Analytical    Gas Detector" },
        { 0x008B, [cam16]              ,   "Valve         Actuator" },
        { 0x008C, [cam216]             ,   "Valve         Actuator" },
        { 0x00B5, [drehmo_i_matic]     ,   "Valve         Actuator" },
        { 0x00ED, [txisorail_hrt]      ,   "Temperature" },
        { 0x0085, [h_pres_transmitter] ,   "Pressure" },
        { 0x0086, [e2t_pulsar_4]       ,   "Temperature" },
        { 0x008D, [mp100]              ,   "Valve         Positioner" },
        { 0x009B, [mc608]              ,   "Flow" },
        { 0x00AD, [ml210_me101]        ,   "Flow          Electromagnetic" },
        { 0x00A2, [emf_1000_series]    ,   "Flow          Electromagnetic" },
        { 0x00A3, [cmf_2000_series]    ,   "Flow          Coriolis Mass" },
        { 0x00B8, "SNE4100B"           ,   "Analytical    Gas Detector" },
        { 0x00BA, [dat_m]              ,   "Analytical    Gas Detector" },
        { 0x00BB, "DAH-M"              ,   "Analytical    Gas Detector" },
        { 0x00BF, [gt_ct8900]          ,   "Analytical    Gas Detector" },
        { 0x00C0, [gq_ce8900]          ,   "Analytical    Gas Detector" },
        { 0x00C1, [echopro_lrxx]       ,   "Level" }
    }
}

/*************************************************************************************
 * Manufacturer Expansion Device Type Codes
 */
VARIABLE manufacturer_expansion4_model_code    /* 0x00E3 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0094, [position_xmtr]                           ,   "Pressure" },
        { 0x000E, [ip_0304_m1_h]                            ,   "Temperature" },
        { 0x0030, [air_30m]                                 ,   "Pressure" },
        { 0x0032, "BRIZ TM-2Ex"                             ,   "Temperature" },
        { 0x0008, "TCD50"                                   ,   "Analytical    Gas Detector" },
        { 0x0006, [d58_ac]                                  ,   "Analytical    Gas Detector" },
        { 0x0013, [txisoblock_hrt]                          ,   "Temperature" },
        { 0x009C, [mvx10]                                   ,   "Flow          Electromagnetic" },
        { 0x0089, [jumo_dtrans_t07]                         ,   "Temperature" },
        { 0x00E3, [jumo_dtrans_t08_37]                      ,   "Temperature" },
        { 0x00E4, [jumo_dtrans_t08_13]                      ,   "Temperature" },
        { 0x002F, [kd_12]                                   ,   "Analytical    Gas Detector" },
        { 0x00A8, [ultra_ir800]                             ,   "Analytical    Gas Detector" },
        { 0x00A9, [ultra_fl800]                             ,   "Analytical    IR Flame Detector" },
        { 0x0028, [ami_ph_redox]                            ,   "Analytical    pH/Redox" },
        { 0x002C, [g3_lefm]                                 ,   "Flow" },
        { 0x003D, [camcor]                                  ,   "Flow          Mass" },
        { 0x0034, [trivax]                                  ,   "Valve         Actuator" },
        { 0x0033, [mass_flow_shanghai_yinuo]                ,   "Flow          Mass" },
        { 0x005A, [ulm]                                     ,   "Level         Radar" },
        { 0x005D, [series_analyzer_9d]                      ,   "Analytical" },
        { 0x008E, [gtx_1100]                                ,   "Analytical    Gas Detector" },
        { 0x00E5, "EMIS-BAR"                                ,   "Pressure" },
        { 0x008D, [chronos_idp7600]                         ,   "Valve         Positioner" },
        { 0x008F, [kc_200_h]                                ,   "Temperature" },
        { 0x00BD, [sdcu_20]                                 ,   "Valve         Actuator" },
        { 0x007C, [otis_7543_6_w_hrt]                       ,   "Analytical    Gas Detector" },
        { 0x0063, [aposa]                                   ,   "Valve         On/Off Valve Controller" },
        { 0x00A6, "BiT (Barksdale Intelligent Transmitter)" ,   "Pressure" },
        { 0x00B7, [nivuflow]                                ,   "Flow" },
        { 0x00BB, [xth300i]                                 ,   "Temperature" },
        { 0x00C4, [rp1000]                                  ,   "Pressure" },
        { 0x0005, "WD/BS Series"                            ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Manufacturer Expansion Device Type Codes
 */
VARIABLE manufacturer_expansion5_model_code    /* 0x00E4 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0035, "DX PST with HRT 7" ,   "Valve         Partial Stroke" },
        { 0x000A, [ncs_tt106]         ,   "Temperature" },
        { 0x007A, "PT0304 VT"         ,   "Temperature" },
        { 0x00B8, "REM"               ,   "Flow" },
        { 0x00C2, "RV"                ,   "Flow" },
        { 0x00D5, "IP 0304/M3-H"      ,   "Temperature" },
        { 0x00B6, "TRG80XX"           ,   "Level" },
        { 0x004F, "IPES-IR3"          ,   "Analytical    Flame Detector" },
        { 0x0050, [tgaes_rx_a]        ,   "Analytical    Gas Detector" },
        { 0x0081, "SD-3"              ,   "Analytical    Gas Detector" },
        { 0x004D, "VisiFerm mA"       ,   "Analytical    Dissolved Oxygen" },
        { 0x004E, "VisiTrace mA"      ,   "Analytical    Dissolved Oxygen" },
        { 0x00DE, "PID901"            ,   "Analytical    Gas Detector" },
        { 0x0088, "MC Synergy"        ,   "Flow" },
        { 0x0029, [emis_vihr_200]     ,   "Flow          Vortex" },
        { 0x007B, "EM-260"            ,   "Flow          Coriolis Mass" },
        { 0x00BF, "GDRD8X"            ,   "Level" },
        { 0x0005, [ter]               ,   "Flow          Magnetic" },
        { 0x002C, "MR"                ,   "Flow" },
        { 0x00A9, "RU"                ,   "Level         Radar" },
        { 0x0006, [hir]               ,   "Temperature" },
        { 0x0008, [smt_200x]          ,   "Pressure" },
        { 0x001B, [curto_cu_dqd_0235] ,   "Valve         Positioner" },
        { 0x0065, [ct_pt4550]         ,   "Temperature" },
        { 0x0067, [ct_pms6030]        ,   "Pressure" },
        { 0x001A, [uniphos_501_dt]    ,   "Analytical    Gas Detector" },
        { 0x0057, "Wi-Corr Bond"      ,   "Analytical" },
        { 0x0068, [leon_meter]        ,   "Flow          Electromagnetic" },
        { 0x0069, "SI-100"            ,   "Analytical    Gas Detector" },
        { 0x006B, [tp2000_series]     ,   "Pressure" },
        { 0x006C, [drg_m_drs]         ,   "Flow" },
        { 0x008A, "YYDG"              ,   "Flow          Electromagnetic" },
        { 0x0094, "NTM8"              ,   "Temperature" },
        { 0x009F, "SP4060"            ,   "Valve         Positioner" },
        { 0x00A6, "TLK-TT306H"        ,   "Temperature" },
        { 0x00A7, "TLK-TT305H"        ,   "Temperature" },
        { 0x00A8, "TLK-TT306H-R"      ,   "Temperature" }
        { 0x00D8, "DA-500"            ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Manufacturer Expansion Device Type Codes
 */
VARIABLE manufacturer_expansion6_model_code    /* 0x00E5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0037, "ACTUATOR" ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * FieldComm Group Device Type Codes
 */
VARIABLE FCG_model_code    /* 0x00F9 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0080, [wirelesshart_net_mgr] },
        { 0x0081, [wirelesshart_gateway] },
        { 0x0082, [generic]              ,   "Interoperable" },
        { 0x0083, [sdc625]               },
        { 0x0084, [analys]               },
        { 0xF980, [wirelesshart_net_mgr] },
        { 0xF981, [wirelesshart_gateway] },
        { 0xF982, [generic]              ,   "Interoperable" },
        { 0xF983, [sdc625]               },
        { 0xF984, [analys]               }
    }
}

/*************************************************************************************
 * ExSaf Device Type Codes
 */
VARIABLE ExSaf_model_code    /* 0x6000 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE080, [esd3000]  },
        { 0xE0EC, [est3051t] },
        { 0xE0ED, [est3051s] }
    }
}

/*************************************************************************************
 * SEOJIN INSTECH Device Type Codes
 */
VARIABLE SEOJIN_INSTECH_model_code    /* 0x6001 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE082, [sdt420] ,   "Level" },
        { 0xE0B4, [stf]    ,   "Flow          Thermal" },
        { 0xE0B5, [smf]    ,   "Flow          Electromagnetic" }
    }
}

/*************************************************************************************
 * TASI FLOW Device Type Codes
 */
VARIABLE TASI_FLOW_model_code    /* 0x6002 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE083, [rt_ex15] },
        { 0xE1B4, [rt_30]   }
    }
}

/*************************************************************************************
 * Power-Genex Ltd. Device Type Codes
 *     Note: was Daehan Control
 */
VARIABLE Power_Genex_Ltd_model_code    /* 0x6003 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2A1, [ss235] ,   "Valve         Smart Valve Positioner" }
    }
}

/*************************************************************************************
 * APM Device Type Codes
 */
VARIABLE APM_model_code    /* 0x6004 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE085, [apm_3dlevelscanner_m] },
        { 0xE086, [apm_3dlevelscanner_s] },
        { 0xE087, [apm_3dlevelscanner_l] }
    }
}

/*************************************************************************************
 * BIFOLD ORANGE Device Type Codes
 *     Note: was ORANGE INST. UK
 */
VARIABLE BIFOLD_ORANGE_model_code    /* 0x6005 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE088, [ex200h_positioner] ,   "Valve         Positioner/Electro-hydraulic" },
        { 0xE089, [is200h_positioner] },
        { 0xE1D2, [ex210h]            }
    }
}

/*************************************************************************************
 * BARTEC Device Type Codes
 */
VARIABLE BARTEC_model_code    /* 0x6006 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE08A, [r3x] }
    }
}

/*************************************************************************************
 * Detcon Device Type Codes
 */
VARIABLE Detcon_model_code    /* 0x6007 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE08B, [detcon_700_bridge] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * MSA Device Type Codes
 */
VARIABLE MSA_model_code    /* 0x6008 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE08C, [ultima_xl_xt] ,   "Analytical    Gas Detector" },
        { 0xE09F, [ultimaxeh]    ,   "Analytical    Gas Detector" },
        { 0xE0BC, [flamegard]    },
        { 0xE0F2, [safeye]       },
        { 0xE0F3, [primax]       ,   "Analytical    Gas Detector" },
        { 0xE0F6, [primax_ir]    ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * METROVAL Device Type Codes
 */
VARIABLE METROVAL_model_code    /* 0x6009 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE08D, [cmm_01_mass_flow] ,   "Flow          Coriolis Mass" },
        { 0xE0C8, [cvm_01]           },
        { 0xE0C9, [tdm_01]           },
        { 0xE196, [vcm_01]           }
    }
}

/*************************************************************************************
 * Etalon Rus Device Type Codes
 */
VARIABLE Etalon_Rus_model_code    /* 0x600A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE081, [sensor_don] }
    }
}

/*************************************************************************************
 * JOGLER Device Type Codes
 */
VARIABLE JOGLER_model_code    /* 0x600B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE08E, [jlt_6000] }
    }
}

/*************************************************************************************
 * KSB Device Type Codes
 */
VARIABLE KSB_model_code    /* 0x600C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE08F, [smartronic_ma] }
    }
}

/*************************************************************************************
 * Richter CT Device Type Codes
 */
VARIABLE Richter_CT_model_code    /* 0x600D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE090, [cms_standard] }
    }
}

/*************************************************************************************
 * NET SAFETY Device Type Codes
 */
VARIABLE NET_SAFETY_model_code    /* 0x600E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE091, [millennium_2]       },
        { 0xE09E, [millennium_2_basic] ,   "Analytical    Gas Detector" },
        { 0xE0DF, [jb_mphf_series]     ,   "Analytical    Fire Detector" },
        { 0xE1E3, [banshee_343]        }
    }
}

/*************************************************************************************
 * SECanada Device Type Codes
 */
VARIABLE SECanada_model_code    /* 0x600F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE092, [him1008] }
    }
}

/*************************************************************************************
 * SUPCON Device Type Codes
 */
VARIABLE SUPCON_model_code    /* 0x6010 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE093, "CXT_CJT" ,   "Pressure" },
        { 0xE094, [cjt]     },
        { 0xE191, [sf10e]   }
    }
}

/*************************************************************************************
 * DKK - TOA Device Type Codes
 */
VARIABLE DKK_TOA_model_code    /* 0x6011 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE095, [hbm_165_h] ,   "Analytical    pH/ORP" },
        { 0xE096, [hbm_167_h] },
        { 0xE097, [wbm_165_h] ,   "Analytical    Conductivity" },
        { 0xE098, [obm_165_h] },
        { 0xE099, [obm_167_h] },
        { 0xE09A, [mbm_165_h] },
        { 0xE09B, [mbm_167_h] }
    }
}

/*************************************************************************************
 * Dwyer Instruments Device Type Codes
 */
VARIABLE Dwyer_Instruments_model_code    /* 0x6012 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE09C, [mark_series] }
    }
}

/*************************************************************************************
 * FineTek Device Type Codes
 */
VARIABLE FineTek_model_code    /* 0x6013 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE09D, [tr] }
    }
}

/*************************************************************************************
 * TopWorx Inc. Device Type Codes
 */
VARIABLE TopWorx_model_code    /* 0x6014 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0CE, [valve_monitor_4310]    },
        { 0xE0EB, [valve_controller_4350] ,   "Valve         Controller" },
        { 0xE194, [position_xmitter]      },
        { 0xE394, [position_xmtr]         ,   "Pressure" },
        { 0xE435, "DX PST with HRT 7"     ,   "Valve         Partial Stroke" }
    }
}

/*************************************************************************************
 * HOFFER FLOW CNTRLS Device Type Codes
 */
VARIABLE HOFFER_FLOW_CNTRLS_model_code    /* 0x6015 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0A1, [hrt1]    ,   "Flow          Totalizer - Rate Indicator" },
        { 0xE0F1, [freq_ma] ,   "Flow          Totalizer - Rate Indicator" }
    }
}

/*************************************************************************************
 * DUST NETWORKS Device Type Codes
 */
VARIABLE DUST_NETWORKS_model_code    /* 0x6016 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0A2, [dn2510_smartmesh]   },
        { 0xE0A3, [m2510_smartmesh]    },
        { 0xE0A4, [pm2510_smartmesh]   },
        { 0xE1D3, [dn2600_ltc5800_whm] }
    }
}

/*************************************************************************************
 * Forbes Marshall Device Type Codes
 */
VARIABLE Forbes_Marshall_model_code    /* 0x6017 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0A5, [vfm_4097]         },
        { 0xE0A6, [ifc_031]          },
        { 0xE0A7, [ifc_021]          },
        { 0xE0A9, [smartpro_ph_8966] },
        { 0xE2B3, [smartpoz_8400s]   ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * All Measures, Ltd. Device Type Codes
 */
VARIABLE All_Measures_Ltd_model_code    /* 0x6018 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0AB, [am_2000] }
    }
}

/*************************************************************************************
 * Pepperl+Fuchs Device Type Codes
 *     Note: was MACTek
 */
VARIABLE Pepperl_Fuchs_model_code    /* 0x6019 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0AC, [mactek_wa] ,   "WirelessHART  Adapter" }
    }
}

/*************************************************************************************
 * CSI Device Type Codes
 */
VARIABLE CSI_model_code    /* 0x601A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0AD, [csi_9420] }
    }
}

/*************************************************************************************
 * TC Fluid Control Device Type Codes
 */
VARIABLE TC_Fluid_Control_model_code    /* 0x601B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0AE, [kms_2] }
    }
}

/*************************************************************************************
 * Rohrback Cosasco Device Type Codes
 */
VARIABLE Rohrback_Cosasco_model_code    /* 0x601C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0AF, [mwt_3905] },
        { 0xE0B0, [uwt_1000] },
        { 0xE0B1, [cwt_9020] },
        { 0xE0F8, [mwt_we]   },
        { 0xE18E, [mwt_qs]   }
    }
}

/*************************************************************************************
 * AirSprite Device Type Codes
 */
VARIABLE AirSprite_model_code    /* 0x601D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0B2, [a_1000_h] }
    }
}

/*************************************************************************************
 * Microcyber Inc. Device Type Codes
 */
VARIABLE Microcyber_model_code    /* 0x601E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0B3, [ncs_pt105_ii]   },
        { 0xE0CF, [ncs_pt105ii_sp] },
        { 0xE0D0, [ncs_tt105]      },
        { 0xE185, [ncs_ft105]      },
        { 0xE186, [ncs_lt105]      },
        { 0xE187, [ncs_at105]      },
        { 0xE188, [ncs_mh105]      },
        { 0xE189, [ncs_pos105]     },
        { 0xE2A6, [l_mag]          ,   "Flow          Magnetic" },
        { 0xE40A, [ncs_tt106]      ,   "Temperature" }
    }
}

/*************************************************************************************
 * TIG Device Type Codes
 */
VARIABLE TIG_model_code    /* 0x601F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0B6, [ty_tds_p]    },
        { 0xE0B7, [ty_trans_f]  },
        { 0xE0B8, [ty_trans_l]  },
        { 0xE0B9, [ty_trans_ps] },
        { 0xE0BA, [ty_trans_a]  }
    }
}

/*************************************************************************************
 * ifm prover Gmbh Device Type Codes
 */
VARIABLE ifm_prover_Gmbh_model_code    /* 0x6020 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0BB, [pmh] }
    }
}

/*************************************************************************************
 * FLEXIM Device Type Codes
 */
VARIABLE FLEXIM_model_code    /* 0x6021 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0BD, [fluxus] ,   "Flow          Ultrasonic" }
    }
}

/*************************************************************************************
 * TOKYO KEIKI Device Type Codes
 */
VARIABLE TOKYO_KEIKI_model_code    /* 0x6022 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0BE, [mrg_10] },
        { 0xE0BF, [mrf_10] },
        { 0xE1E9, [krg_10] ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * SBEM Device Type Codes
 */
VARIABLE SBEM_model_code    /* 0x6023 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0C0, [ultramate_136_ult]  },
        { 0xE0D8, [microguide_132_tdr] },
        { 0xE0D9, [micromate_138_plr]  },
        { 0xE18F, [elmag_151]          }
    }
}

/*************************************************************************************
 * SkoFlo Industries, Inc. Device Type Codes
 */
VARIABLE SkoFlo_Industries_model_code    /* 0x6024 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0C1, [flow_control_system] }
    }
}

/*************************************************************************************
 * StoneL Corporation Device Type Codes
 */
VARIABLE StoneL_Corporation_model_code    /* 0x6025 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0C2, [axiom] ,   "Valve         Monitor" }
    }
}

/*************************************************************************************
 * EUREKA FLOW Device Type Codes
 */
VARIABLE EUREKA_FLOW_model_code    /* 0x6026 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0C3, [eumag] },
        { 0xE0C5, [mtrt]  }
    }
}

/*************************************************************************************
 * BEKA associates Device Type Codes
 */
VARIABLE BEKA_associates_model_code    /* 0x6027 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0C4, [ba47x_67x_series] ,   "Temperature" }
    }
}

/*************************************************************************************
 * CAPSTAR AUTOMATION Device Type Codes
 */
VARIABLE CAPSTAR_AUTOMATION_model_code    /* 0x6028 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0C6, [cap_30xx] }
    }
}

/*************************************************************************************
 * Pulsar Device Type Codes
 */
VARIABLE Pulsar_model_code    /* 0x6029 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0C7, [nemesis_1000] },
        { 0xE1C5, [dbi]          }
    }
}

/*************************************************************************************
 * ELEMER Device Type Codes
 */
VARIABLE ELEMER_model_code    /* 0x602A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0CB, [air_20h]      ,   "Pressure" },
        { 0xE0E8, [elemer_100]   },
        { 0xE1F1, "AIR-10SH"     ,   "Pressure" },
        { 0xE30E, [ip_0304_m1_h] ,   "Temperature" },
        { 0xE330, [air_30m]      ,   "Pressure" },
        { 0xE332, "BRIZ TM-2Ex"  ,   "Temperature" },
        { 0xE47A, "PT0304 VT"    ,   "Temperature" },
        { 0xE4B8, "REM"          ,   "Flow" }
        { 0xE4C2, "RV"           ,   "Flow" },
        { 0xE4D5, "IP 0304/M3-H" ,   "Temperature" }
    }
}

/*************************************************************************************
 * Soft Tech Group Device Type Codes
 */
VARIABLE Soft_Tech_Group_model_code    /* 0x602B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0CD, [frankenstein] }
    }
}

/*************************************************************************************
 * FUJIAN WIDEPLUS PRECISION INSTRUMENTS CO., LTD Device Type Codes
 */
VARIABLE FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code    /* 0x602C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0D1, [wideplus_ba] },
        { 0xE0D2, [wideplus_a]  },
        { 0xE1D8, [wideplus_rd] },
        { 0xE1D9, [wp_emf]      },
        { 0xE1DA, [wideplus_zf] },
        { 0xE1DB, [wideplus_um] }
    }
}

/*************************************************************************************
 * ENOTEC GmbH Device Type Codes
 */
VARIABLE ENOTEC_GmbH_model_code    /* 0x602D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0D3, [oxitec_5000] ,   "Analytical    Oxygen" },
        { 0xE0D4, [comtec_6000] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Val Controls A/S Device Type Codes
 */
VARIABLE Val_Controls_AS_model_code    /* 0x602E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0D5, [ihp]     },
        { 0xE0D6, [idp]     },
        { 0xE1CF, [ihp24_a] },
        { 0xE2FA, [idc24]   ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Kurz Instruments Device Type Codes
 */
VARIABLE Kurz_Instruments_model_code    /* 0x602F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0D7, [mft_b_series] ,   "Flow          Thermal" }
    }
}

/*************************************************************************************
 * E-Senza Technologies Device Type Codes
 */
VARIABLE E_Senza_Technologies_model_code    /* 0x6030 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0DB, [sgh350] },
        { 0xE195, [sb350]  }
    }
}

/*************************************************************************************
 * Crowcon Device Type Codes
 */
VARIABLE Crowcon_model_code    /* 0x6031 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0DC, [irmax]      },
        { 0xE0DD, [ir_display] },
        { 0xE0FC, [xgardiq]    ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * GEORGIN Device Type Codes
 */
VARIABLE GEORGIN_model_code    /* 0x6032 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0DE, [tixo3] ,   "Temperature" },
        { 0xE1CB, [tia3]  }
    }
}

/*************************************************************************************
 * Eldridge Products Device Type Codes
 */
VARIABLE Eldridge_Products_model_code    /* 0x6033 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0E0, [master_touch] }
    }
}

/*************************************************************************************
 * DDTOP Device Type Codes
 */
VARIABLE DDTOP_model_code    /* 0x6034 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0E1, [dlt9000] ,   "Level         Displacement" },
        { 0xE1AC, [eftn]    },
        { 0xE1AD, [dtu100]  ,   "Level" },
        { 0xE1AE, [mtl100]  },
        { 0xE1AF, [dlc3010] },
        { 0xE1B0, [flow100] },
        { 0xE1B7, [dpu100]  },
        { 0xE1B8, [dlu100]  },
        { 0xE1B9, [dfu100]  },
        { 0xE4B6, "TRG80XX" ,   "Level" }
    }
}

/*************************************************************************************
 * KSR Kuebler Device Type Codes
 */
VARIABLE KSR_Kuebler_model_code    /* 0x6035 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0E2, [ffg_p_level] ,   "Level" }
    }
}

/*************************************************************************************
 * Armstrong Intl Device Type Codes
 */
VARIABLE Armstrong_Intl_model_code    /* 0x6036 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0E3, [st_5700_steam]    ,   "Analytical    Steam Trap" },
        { 0xE0E4, [ad_5000_acousitc] ,   "Valve         Safety Relief Monitor" },
        { 0xE0E5, [td_5100_temp]     }
    }
}

/*************************************************************************************
 * Hitachi High-Tech Device Type Codes
 */
VARIABLE Hitachi_High_Tech_model_code    /* 0x6037 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0E6, [n7] ,   "Pressure" },
        { 0xE2E7, [n8] ,   "Pressure" }
    }
}

/*************************************************************************************
 * AMFLOW Device Type Codes
 */
VARIABLE AMFLOW_model_code    /* 0x6038 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0E9, [a2_4_20ma_hrt] },
        { 0xE2C6, [acm_2_hrt]     ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Fluidwell Device Type Codes
 */
VARIABLE Fluidwell_model_code    /* 0x6039 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0EA, [f018p] ,   "Flow          Totalizer" },
        { 0xE2CD, [e018p] ,   "Flow" }
    }
}

/*************************************************************************************
 * Mecon GmbH Device Type Codes
 */
VARIABLE Mecon_GmbH_model_code    /* 0x603A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0EE, [mag_flux_m] }
    }
}

/*************************************************************************************
 * Fox Thermal Inst. Device Type Codes
 */
VARIABLE Fox_Thermal_Inst_model_code    /* 0x603B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0EF, [fox_ft3] }
    }
}

/*************************************************************************************
 * EJMConsulting Device Type Codes
 */
VARIABLE EJMConsulting_model_code    /* 0x603C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0F0, [prototype_unit] }
    }
}

/*************************************************************************************
 * Anton Paar Device Type Codes
 */
VARIABLE Anton_Paar_model_code    /* 0x603D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0F4, [l_dens_427t] }
    }
}

/*************************************************************************************
 * Dinel, s.r.o. Device Type Codes
 */
VARIABLE Dinel_sro_model_code    /* 0x603E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0F5, [ulm_70] }
    }
}

/*************************************************************************************
 * Simtronics ASA Device Type Codes
 */
VARIABLE Simtronics_ASA_model_code    /* 0x603F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0F7, [gd10]          ,   "Analytical    Infrared Gas Detector" },
        { 0xE1A9, [dm_tv6_df_tv7] },
        { 0xE1D5, [dm_dg_gas]     }
    }
}

/*************************************************************************************
 * INTEC Device Type Codes
 */
VARIABLE INTEC_model_code    /* 0x6040 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0F9, [spiroster_07] },
        { 0xE1A4, [rpw01]        },
        { 0xE1A5, [x_matik]      },
        { 0xE1A6, [xi]           }
    }
}

/*************************************************************************************
 * Softing Device Type Codes
 */
VARIABLE Softing_model_code    /* 0x6041 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE192, [wd_h] }
    }
}

/*************************************************************************************
 * Fire Sentry Corp Device Type Codes
 */
VARIABLE Fire_Sentry_Corp_model_code    /* 0x6042 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE180, [fschcom] }
    }
}

/*************************************************************************************
 * FUTURE INSTRUMENT Device Type Codes
 */
VARIABLE FUTURE_INSTRUMENT_model_code    /* 0x6043 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE181, [magnetostrictive_lvl]  },
        { 0xE182, [rf_level_transmitter]  },
        { 0xE183, [radar_lvl_transmitter] },
        { 0xE1CC, [rf_cap_level_switch]   },
        { 0xE1CD, [fork_level_switch]     }
    }
}

/*************************************************************************************
 * MOTOYAMA Device Type Codes
 */
VARIABLE MOTOYAMA_model_code    /* 0x6044 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE184, [ea10s] ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * ifak system Device Type Codes
 */
VARIABLE ifak_system_model_code    /* 0x6045 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1C7, [isnet_line] }
    }
}

/*************************************************************************************
 * MORGAN KOREA Device Type Codes
 */
VARIABLE MORGAN_KOREA_model_code    /* 0x6046 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE18A, [smart_actuator] }
    }
}

/*************************************************************************************
 * AWIATECH Device Type Codes
 */
VARIABLE AWIATECH_model_code    /* 0x6047 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE18B, [warrior_sensor] }
    }
}

/*************************************************************************************
 * Chentronics Device Type Codes
 */
VARIABLE Chentronics_model_code    /* 0x6048 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE18C, [iscan2] }
    }
}

/*************************************************************************************
 * United Electric Device Type Codes
 */
VARIABLE United_Electric_model_code    /* 0x6049 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE18D, [tx200h] ,   "Pressure" },
        { 0xE308, "TCD50"  ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Shanghai Sinoto Instrument Co Ltd Device Type Codes
 */
VARIABLE Shanghai_Sinoto_Instrument_Co_Ltd_model_code    /* 0x604A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE190, [fst_3000] ,   "Level" }
    }
}

/*************************************************************************************
 * MATSUSHIMA Device Type Codes
 */
VARIABLE MATSUSHIMA_model_code    /* 0x604B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE193, [mwlm_pr26] }
    }
}

/*************************************************************************************
 * Westcontrol AS Device Type Codes
 */
VARIABLE Westcontrol_AS_model_code    /* 0x604C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE197, [ptc_vr_sensor]          },
        { 0xE1BE, [ptc_vr_sensor_wireless] }
    }
}

/*************************************************************************************
 * Oldham Device Type Codes
 */
VARIABLE Oldham_model_code    /* 0x604D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE198, [olct_200] ,   "Analytical    Gas Detector" },
        { 0xE1EA, [itrans_2] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * WooriTG Device Type Codes
 */
VARIABLE WooriTG_model_code    /* 0x604E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE19D, [operafcs] }
    }
}

/*************************************************************************************
 * trianglemicrosol Device Type Codes
 */
VARIABLE trianglemicrosol_model_code    /* 0x604F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE19E, [tms_hps] }
    }
}

/*************************************************************************************
 * Nivis LLC Device Type Codes
 */
VARIABLE Nivis_LLC_model_code    /* 0x6051 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1A0, [vl10]  },
        { 0xE1A1, [vs220] }
    }
}

/*************************************************************************************
 * SKF USA Device Type Codes
 */
VARIABLE SKF_USA_model_code    /* 0x6052 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1A2, [cmwa_8800] }
    }
}

/*************************************************************************************
 * ESP Safety Device Type Codes
 */
VARIABLE ESP_Safety_model_code    /* 0x6053 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE19C, [sgoes]           ,   "Analytical    Combustible Gas Analyzer" },
        { 0xE1AB, [vector_fcu]      ,   "Analytical    Gas Detector" },
        { 0xE1DD, [tgaes_open_path] },
        { 0xE44F, "IPES-IR3"        ,   "Analytical    Flame Detector" },
        { 0xE450, [tgaes_rx_a]      ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * HollySys Device Type Codes
 */
VARIABLE HollySys_model_code    /* 0x6054 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1A3, [at2000] },
        { 0xE1D6, [at3000] },
        { 0xE1D7, [at3600] }
    }
}

/*************************************************************************************
 * GF Signet Device Type Codes
 */
VARIABLE GF_Signet_model_code    /* 0x6055 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1B3, [h_comm] }
    }
}

/*************************************************************************************
 * Sierra Monitor Device Type Codes
 */
VARIABLE Sierra_Monitor_model_code    /* 0x6057 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1AA, [it_series] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Schneider Electric Device Type Codes
 */
VARIABLE Schneider_Electric_model_code    /* 0x6058 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1B1, [hrm_v1_00] }
    }
}

/*************************************************************************************
 * Emerson Device Type Codes
 */
VARIABLE Emerson_model_code    /* 0x6059 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1B2, [wireless_interface] }
    }
}

/*************************************************************************************
 * ABM Sensor Tech Device Type Codes
 */
VARIABLE ABM_Sensor_Tech_model_code    /* 0x605A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1B6, [abm200_xxxrxch] }
    }
}

/*************************************************************************************
 * RIKEN KEIKI Device Type Codes
 */
VARIABLE RIKEN_KEIKI_model_code    /* 0x605B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1B5, [sd_1]   ,   "Analytical    Gas Detector" },
        { 0xE2E8, [d58_dc] ,   "Analytical    Gas Detector" },
        { 0xE2E9, [gd_88]  ,   "Analytical    Gas Detector" },
        { 0xE306, [d58_ac] ,   "Analytical    Gas Detector" },
        { 0xE481, "SD-3"   ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * EEPL India Device Type Codes
 */
VARIABLE EEPL_India_model_code    /* 0x605C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1BA, [elmag200m] }
    }
}

/*************************************************************************************
 * Thermal Inst. Co. Device Type Codes
 */
VARIABLE Thermal_Inst_model_code    /* 0x605D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1BB, [tico_flow_meter] }
    }
}

/*************************************************************************************
 * HYDAC ELECTRONIC Device Type Codes
 */
VARIABLE HYDAC_ELECTRONIC_model_code    /* 0x605E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1BC, [hda4000]      },
        { 0xE1E7, [bis_1000_apt] }
    }
}

/*************************************************************************************
 * GNEUSS Device Type Codes
 */
VARIABLE GNEUSS_model_code    /* 0x605F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1BD, [daix] }
    }
}

/*************************************************************************************
 * Actuation Technologies Device Type Codes
 *     Note: was Valve Automation
 */
VARIABLE Actuation_Technologies_model_code    /* 0x6060 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1BF, [eim_cam06]    },
        { 0xE1C1, [eim_cam206]   ,   "Valve         Actuator - Electrical" },
        { 0xE1C2, [eim_cam06_h5] },
        { 0xE28B, [cam16]        ,   "Valve         Actuator" },
        { 0xE28C, [cam216]       ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * Roxar Device Type Codes
 */
VARIABLE Roxar_model_code    /* 0x6061 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1C0, [corrlog_wireless] }
    }
}

/*************************************************************************************
 * GASTRON CO.,LTD Device Type Codes
 */
VARIABLE GASTRON_model_code    /* 0x6062 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1C3, [gtd2000]  ,   "Analytical    Gas Detector" },
        { 0xE1C6, [gir_3000] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * HAMILTON Bonaduz Device Type Codes
 */
VARIABLE HAMILTON_Bonaduz_model_code    /* 0x6063 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1C4, [visipro_do]   ,   "Analytical    Dissolved Oxygen" },
        { 0xE44D, "VisiFerm mA"  ,   "Analytical    Dissolved Oxygen" },
        { 0xE44E, "VisiTrace mA" ,   "Analytical    Dissolved Oxygen" }
    }
}

/*************************************************************************************
 * Nobel Weighing Systems Device Type Codes
 */
VARIABLE Nobel_Weighing_Systems_model_code    /* 0x6064 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1C8, [lcamp_1xx] }
    }
}

/*************************************************************************************
 * ECD Device Type Codes
 */
VARIABLE ECD_model_code    /* 0x6065 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1C9, [t80_transmitter] }
    }
}

/*************************************************************************************
 * Sensor Electronics Device Type Codes
 */
VARIABLE Sensor_Electronics_model_code    /* 0x6066 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1CE, [sec5000irevolution] }
    }
}

/*************************************************************************************
 * Senscient Device Type Codes
 */
VARIABLE Senscient_model_code    /* 0x6067 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1D0, [elds_1000_2000] }
    }
}

/*************************************************************************************
 * Shanghai Kent Instrument Co.,Ltd. Device Type Codes
 */
VARIABLE Shanghai_Kent_Instrument_Co_Ltd_model_code    /* 0x6068 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1D1, [kef] }
    }
}

/*************************************************************************************
 * Scott Safety Device Type Codes
 */
VARIABLE Scott_Safety_model_code    /* 0x6069 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1D4, [meridian_gas_wired] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Welkin Co., Ltd Device Type Codes
 */
VARIABLE Welkin_Co_Ltd_model_code    /* 0x606A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1DC, [model2010] ,   "Flow" }
    }
}

/*************************************************************************************
 * Teplopribor Device Type Codes
 */
VARIABLE Teplopribor_model_code    /* 0x606B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1DE, [crocus] }
    }
}

/*************************************************************************************
 * Auburn Systems Device Type Codes
 */
VARIABLE Auburn_Systems_model_code    /* 0x606C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1DF, [tribodsp3400h] }
    }
}

/*************************************************************************************
 * Therm-O-Disc Device Type Codes
 */
VARIABLE Therm_O_Disc_model_code    /* 0x606D */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1E0, [Wireless_50wm] }
    }
}

/*************************************************************************************
 * SWISA INSTRUMENT Device Type Codes
 */
VARIABLE SWISA_INSTRUMENT_model_code    /* 0x606E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1E1, [mat_matd_mats] ,   "Level" }
    }
}

/*************************************************************************************
 * Sage Metering, Inc. Device Type Codes
 */
VARIABLE Sage_Metering_model_code    /* 0x606F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1E2, [sage_prime_rio] ,   "Flow          Thermal" }
    }
}

/*************************************************************************************
 * Cooper Bussmann Device Type Codes
 */
VARIABLE Cooper_Bussmann_model_code    /* 0x6070 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1E4, [wg_315] }
    }
}

/*************************************************************************************
 * FSUE VNIIA Device Type Codes
 */
VARIABLE FSUE_VNIIA_model_code    /* 0x6071 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1E5, [tjiu406_m100] }
    }
}

/*************************************************************************************
 * Groveley Detection Device Type Codes
 */
VARIABLE Groveley_Detection_model_code    /* 0x6072 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1E6, [gdu_02_incus] }
    }
}

/*************************************************************************************
 * Hengyang No. Photoelectric Info Tech Co Device Type Codes
 */
VARIABLE Hengyang_No_Photoelectric_Info_Tech_Co_model_code    /* 0x6073 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1E8, [zpzd31] }
    }
}

/*************************************************************************************
 * Imtex Controls Ltd Device Type Codes
 */
VARIABLE Imtex_Controls_Ltd_model_code    /* 0x6075 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1EC, [st_4312] ,   "Valve         Positioner Transmitter" }
    }
}

/*************************************************************************************
 * AUMA Device Type Codes
 */
VARIABLE AUMA_model_code    /* 0x607C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE1FD, [aumatic_ac_acex] ,   "Valve         Actuator" },
        { 0xE2B5, [drehmo_i_matic]  ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * Novus Automation Device Type Codes
 */
VARIABLE Novus_Automation_model_code    /* 0x607E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2ED, [txisorail_hrt]  ,   "Temperature" },
        { 0xE313, [txisoblock_hrt] ,   "Temperature" }
    }
}

/*************************************************************************************
 * MISTRAS Group, Inc. Device Type Codes
 */
VARIABLE MISTRAS_Group_model_code    /* 0x6080 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE0FF, [caliperay] ,   "Analytical" }
    }
}

/*************************************************************************************
 * Gefran S.p.A. Device Type Codes
 */
VARIABLE Gefran_SpA_model_code    /* 0x6081 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE285, [h_pres_transmitter] ,   "Pressure" }
    }
}

/*************************************************************************************
 * Lumasense Technologies Device Type Codes
 */
VARIABLE Lumasense_Technologies_model_code    /* 0x6082 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE286, [e2t_pulsar_4] ,   "Temperature" }
    }
}

/*************************************************************************************
 * 3S Device Type Codes
 */
VARIABLE Three_S_model_code    /* 0x6084 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE28D, [mp100] ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Euromag International S.r.l. Device Type Codes
 */
VARIABLE Euromag_International_model_code    /* 0x6086 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE29B, [mc608] ,   "Flow" }
    }
}

/*************************************************************************************
 * Isoil-Hemina Device Type Codes
 */
VARIABLE Isoil_Hemina_model_code    /* 0x6088 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2AD, [ml210_me101] ,   "Flow          Electromagnetic" },
        { 0xE39C, [mvx10]       ,   "Flow          Electromagnetic" }
    }
}

/*************************************************************************************
 * Walsn Device Type Codes
 */
VARIABLE Walsn_model_code    /* 0x6089 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2A2, [emf_1000_series] ,   "Flow          Electromagnetic" },
        { 0xE2A3, [cmf_2000_series] ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * JUMO GmbH & Co KG Device Type Codes
 */
VARIABLE JUMO_GmbH_Co_KG_model_code    /* 0x608C */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE389, [jumo_dtrans_t07]    ,   "Temperature" },
        { 0xE3E3, [jumo_dtrans_t08_37] ,   "Temperature" },
        { 0xE3E4, [jumo_dtrans_t08_13] ,   "Temperature" }
    }
}

/*************************************************************************************
 * NEW COSMOS ELECTRIC Device Type Codes
 */
VARIABLE NEW_COSMOS_ELECTRIC_model_code    /* 0x608F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE32F, [kd_12] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Nuoan Device Type Codes
 */
VARIABLE Nuoan_model_code    /* 0x6092 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2B8, "SNE4100B" ,   "Analytical    Gas Detector" },
        { 0xE4DE, "PID901"   ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * SPA ANALITPRIBOR Device Type Codes
 */
VARIABLE SPA_ANALITPRIBOR_model_code    /* 0x6093 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2BA, [dat_m] ,   "Analytical    Gas Detector" },
        { 0xE2BB, "DAH-M" ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Gasensor Device Type Codes
 */
VARIABLE Gasensor_model_code    /* 0x6099 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2BF, [gt_ct8900]   ,   "Analytical    Gas Detector" },
        { 0xE2C0, [gq_ce8900]   ,   "Analytical    Gas Detector" },
        { 0xE3A8, [ultra_ir800] ,   "Analytical    Gas Detector" },
        { 0xE3A9, [ultra_fl800] ,   "Analytical    IR Flame Detector" }
    }
}

/*************************************************************************************
 * Flowline Analytical Instruments Device Type Codes
 */
VARIABLE Flowline_model_code    /* 0x609A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE2C1, [echopro_lrxx] ,   "Level" }
    }
}

/*************************************************************************************
 * SWAN Analytical Instruments Device Type Codes
 */
VARIABLE SWAN_Analytical_Instruments_model_code    /* 0x60B5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE328, [ami_ph_redox] ,   "Analytical    pH/Redox" }
    }
}

/*************************************************************************************
 * Sensia Device Type Codes
 *     Note: was Cameron
 */
VARIABLE Sensia_model_code    /* 0x60B7 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE32C, [g3_lefm]    ,   "Flow" },
        { 0xE33D, [camcor]     ,   "Flow          Mass" },
        { 0xE488, "MC Synergy" ,   "Flow" }
    }
}

/*************************************************************************************
 * Hoerbiger Device Type Codes
 */
VARIABLE Hoerbiger_model_code    /* 0x60B8 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE334, [trivax] ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * Shanghai Yinuo Device Type Codes
 */
VARIABLE Shanghai_Yinuo_model_code    /* 0x60B9 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE333, [mass_flow_shanghai_yinuo] ,   "Flow          Mass" }
    }
}

/*************************************************************************************
 * LIMACO Device Type Codes
 */
VARIABLE LIMACO_model_code    /* 0x60C1 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE35A, [ulm] ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * Rhosonics Analytical BV Device Type Codes
 */
VARIABLE Rhosonics_Analytical_BV_model_code    /* 0x60C2 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE35D, [series_analyzer_9d] ,   "Analytical" }
    }
}

/*************************************************************************************
 * WISE Device Type Codes
 */
VARIABLE WISE_Sensing_model_code    /* 0x60C3 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE38E, [gtx_1100] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * EMIS Device Type Codes
 */
VARIABLE EMIS_model_code    /* 0x60C5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE3E5, "EMIS-BAR"      ,   "Pressure" },
        { 0xE429, [emis_vihr_200] ,   "Flow          Vortex" },
        { 0xE47B, "EM-260"        ,   "Flow          Coriolis Mass" }
    }
}

/*************************************************************************************
 * VALTEK_SULAMERIC Device Type Codes
 */
VARIABLE VALTEK_SULAMERIC_model_code    /* 0x60CA */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE38D, [chronos_idp7600] ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * Elimko Ltd. Sti. Device Type Codes
 */
VARIABLE Elimko_Ltd_Sti_model_code    /* 0x60CB */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE38F, [kc_200_h] ,   "Temperature" }
    }
}

/*************************************************************************************
 * GODA Instruments Device Type Codes
 */
VARIABLE GODA_Instruments_model_code    /* 0x60CD */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE4BF, "GDRD8X" ,   "Level" }
    }
}

/*************************************************************************************
 * DVG Automation Device Type Codes
 */
VARIABLE DVG_Automation_model_code    /* 0x60D5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE3BD, [sdcu_20] ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * Otis Instruments Device Type Codes
 */
VARIABLE OTIS_Instruments_model_code     /* 0x60DB */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE37C, [otis_7543_6_w_hrt] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * KANEKO SANGYO Device Type Codes
 */
VARIABLE KANEKO_SANGYO_model_code    /* 0x60DC */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE363, [aposa] ,   "Valve         On/Off Valve Controller" }
    }
}

/*************************************************************************************
 * Barksdale Control Products Device Type Codes
 */
VARIABLE Barksdale_Control_Products_model_code    /* 0x60E7 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE3A6, "BiT (Barksdale Intelligent Transmitter)" ,   "Pressure" }
    }
}

/*************************************************************************************
 * NIVUS Device Type Codes
 */
VARIABLE NIVUS_model_code    /* 0x60EB */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE3B7, [nivuflow] ,   "Flow" }
    }
}

/*************************************************************************************
 * Guangzhou Xitai Device Type Codes
 */
VARIABLE Guangzhou_Xitai_model_code    /* 0x60EE */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE3BB, [xth300i] ,   "Temperature" }
    }
}

/*************************************************************************************
 * VZLJOT Device Type Codes
 */
VARIABLE VZLJOT_model_code    /* 0x60F4 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE405, [ter] ,   "Flow          Magnetic" },
        { 0xE42C, "MR"  ,   "Flow" },
        { 0xE4A9, "RU"  ,   "Level         Radar" }
    }
}

/*************************************************************************************
 * ROCKSENSOR Device Type Codes
 */
VARIABLE ROCKSENSOR_model_code    /* 0x60F5 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE3C4, [rp1000] ,   "Pressure" }
    }
}

/*************************************************************************************
 * DELTA CONTROLS Device Type Codes
 */
VARIABLE DELTA_CONTROLS_model_code    /* 0x610A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE406, [hir] ,   "Temperature" }
    }
}

/*************************************************************************************
 * WISE CONTROL INC Device Type Codes
 */
VARIABLE WISE_CONTROL_INC_model_code    /* 0x610B */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE408, [smt_200x] ,   "Pressure" }
    }
}

/*************************************************************************************
 * CURTO Device Type Codes
 */
VARIABLE CURTO_model_code    /* 0x610E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE41B, [curto_cu_dqd_0235] ,   "Valve         Positioner" },
        { 0xE465, [ct_pt4550]         ,   "Temperature" },
        { 0xE467, [ct_pms6030]        ,   "Pressure" }
    }
}

/*************************************************************************************
 * Uniphos Envirotronics Device Type Codes
 */
VARIABLE Uniphos_Envirotronics_model_code    /* 0x6110 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE41A, [uniphos_501_dt] ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * 3-Sci Device Type Codes
 */
VARIABLE Three_Sci_model_code    /* 0x611A */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE457, "Wi-Corr Bond" ,   "Analytical" }
    }
}

/*************************************************************************************
 * SHENYANG LANSHEN INSTRUMENT CO., LTD Device Type Codes
 */
VARIABLE SHENYANG_LANSHEN_INSTRUMENT_CO_LTD_model_code    /* 0x611E */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE468, [leon_meter] ,   "Flow          Electromagnetic" }
    }
}

/*************************************************************************************
 * SENKO Device Type Codes
 */
VARIABLE SENKO_model_code    /* 0x6120 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE469, "SI-100" ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * Toshniwal Industries Private Limited Device Type Codes
 */
VARIABLE Toshniwal_Industries_Private_Limited_model_code    /* 0x6124 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE46B, [tp2000_series] ,   "Pressure" }
    }
}

/*************************************************************************************
 * JSC EPF SIBNA Device Type Codes
 */
VARIABLE JSC_EPF_SIBNA_model_code    /* 0x6125 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE46C, [drg_m_drs] ,   "Flow" }
    }
}

/*************************************************************************************
 * Diyuan Device Type Codes
 */
VARIABLE Diyuan_model_code    /* 0x6130 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE48A, "YYDG" ,   "Flow          Electromagnetic" }
    }
}

/*************************************************************************************
 * New Power Device Type Codes
 */
VARIABLE New_Power_model_code    /* 0x6131 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE494, "NTM8" ,   "Temperature" }
    }
}

/*************************************************************************************
 * Hanwei Electronics Group Corporation Device Type Codes
 */
VARIABLE Hanwei_Electronics_Group_Corporation_model_code    /* 0x6135 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE305, "WD/BS Series" ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * SEG Device Type Codes
 */
VARIABLE SEG_model_code    /* 0x6134 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE49F, "SP4060" ,   "Valve         Positioner" }
    }
}

/*************************************************************************************
 * BALDOTA CONTROL Device Type Codes
 */
VARIABLE BALDOTA_CONTROL_model_code    /* 0x6137 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE4A6, "TLK-TT306H"   ,   "Temperature" },
        { 0xE4A7, "TLK-TT305H"   ,   "Temperature" },
        { 0xE4A8, "TLK-TT306H-R" ,   "Temperature" }
    }
}

/*************************************************************************************
 * GASDNA CO., LTD Device Type Codes
 */
VARIABLE GASDNA_CO_LTD_model_code    /* 0x6143 */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE4D8, "DA-500" ,   "Analytical    Gas Detector" }
    }
}

/*************************************************************************************
 * BONRAY Device Type Codes
 */
VARIABLE BONRAY_model_code    /* 0x614F */
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0xE537, "ACTUATOR" ,   "Valve         Actuator" }
    }
}

/*************************************************************************************
 * Table 1. Expanded Device Type Codes
 */
VARIABLE device_type_code
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0304,  Ametek_model_code(0x0304)                         }, // 0x0003

        { 0x0A01,  Brooks_Instrument_model_code(0x0A01)              }, // 0x000A
        { 0x0A02,  Brooks_Instrument_model_code(0x0A02)              }, // 0x000A
        { 0x0A03,  Brooks_Instrument_model_code(0x0A03)              }, // 0x000A
        { 0x0A04,  Brooks_Instrument_model_code(0x0A04)              }, // 0x000A

        { 0x0D14,  Daniel_Industries_model_code(0x0D14)              }, // 0x000D
        { 0x0D15,  Daniel_Industries_model_code(0x0D15)              }, // 0x000D
        { 0x0D28,  Daniel_Industries_model_code(0x0D28)              }, // 0x000D
        { 0x0D29,  Daniel_Industries_model_code(0x0D29)              }, // 0x000D

        { 0x0E01,  Delta_model_code(0x0E01)                          }, // 0x000E

        { 0x1103,  Endress_Hauser_model_code(0x1103)                 }, // 0x0011
        { 0x1104,  Endress_Hauser_model_code(0x1104)                 }, // 0x0011
        { 0x1105,  Endress_Hauser_model_code(0x1105)                 }, // 0x0011
        { 0x1106,  Endress_Hauser_model_code(0x1106)                 }, // 0x0011
        { 0x1107,  Endress_Hauser_model_code(0x1107)                 }, // 0x0011
        { 0x1108,  Endress_Hauser_model_code(0x1108)                 }, // 0x0011
        { 0x1109,  Endress_Hauser_model_code(0x1109)                 }, // 0x0011
        { 0x110A,  Endress_Hauser_model_code(0x110A)                 }, // 0x0011
        { 0x110B,  Endress_Hauser_model_code(0x110B)                 }, // 0x0011
        { 0x110C,  Endress_Hauser_model_code(0x110C)                 }, // 0x0011
        { 0x110D,  Endress_Hauser_model_code(0x110D)                 }, // 0x0011
        { 0x110E,  Endress_Hauser_model_code(0x110E)                 }, // 0x0011
        { 0x110F,  Endress_Hauser_model_code(0x110F)                 }, // 0x0011
        { 0x1110,  Endress_Hauser_model_code(0x1110)                 }, // 0x0011
        { 0x1111,  Endress_Hauser_model_code(0x1111)                 }, // 0x0011
        { 0x1112,  Endress_Hauser_model_code(0x1112)                 }, // 0x0011
        { 0x1113,  Endress_Hauser_model_code(0x1113)                 }, // 0x0011
        { 0x1114,  Endress_Hauser_model_code(0x1114)                 }, // 0x0011
        { 0x1117,  Endress_Hauser_model_code(0x1117)                 }, // 0x0011
        { 0x1118,  Endress_Hauser_model_code(0x1118)                 }, // 0x0011
        { 0x1119,  Endress_Hauser_model_code(0x1119)                 }, // 0x0011
        { 0x111A,  Endress_Hauser_model_code(0x111A)                 }, // 0x0011
        { 0x111B,  Endress_Hauser_model_code(0x111B)                 }, // 0x0011
        { 0x111D,  Endress_Hauser_model_code(0x111D)                 }, // 0x0011
        { 0x111E,  Endress_Hauser_model_code(0x111E)                 }, // 0x0011
        { 0x111F,  Endress_Hauser_model_code(0x111F)                 }, // 0x0011
        { 0x1120,  Endress_Hauser_model_code(0x1120)                 }, // 0x0011
        { 0x1121,  Endress_Hauser_model_code(0x1121)                 }, // 0x0011
        { 0x1122,  Endress_Hauser_model_code(0x1122)                 }, // 0x0011
        { 0x1123,  Endress_Hauser_model_code(0x1123)                 }, // 0x0011
        { 0x1124,  Endress_Hauser_model_code(0x1124)                 }, // 0x0011
        { 0x1127,  Endress_Hauser_model_code(0x1127)                 }, // 0x0011
        { 0x1128,  Endress_Hauser_model_code(0x1128)                 }, // 0x0011
        { 0x112A,  Endress_Hauser_model_code(0x112A)                 }, // 0x0011
        { 0x112B,  Endress_Hauser_model_code(0x112B)                 }, // 0x0011
        { 0x112D,  Endress_Hauser_model_code(0x112D)                 }, // 0x0011
        { 0x112E,  Endress_Hauser_model_code(0x112E)                 }, // 0x0011
        { 0x112F,  Endress_Hauser_model_code(0x112F)                 }, // 0x0011
        { 0x1130,  Endress_Hauser_model_code(0x1130)                 }, // 0x0011
        { 0x1131,  Endress_Hauser_model_code(0x1131)                 }, // 0x0011
        { 0x1132,  Endress_Hauser_model_code(0x1132)                 }, // 0x0011
        { 0x1133,  Endress_Hauser_model_code(0x1133)                 }, // 0x0011
        { 0x1134,  Endress_Hauser_model_code(0x1134)                 }, // 0x0011
        { 0x1135,  Endress_Hauser_model_code(0x1135)                 }, // 0x0011
        { 0x1136,  Endress_Hauser_model_code(0x1136)                 }, // 0x0011
        { 0x1137,  Endress_Hauser_model_code(0x1137)                 }, // 0x0011
        { 0x1138,  Endress_Hauser_model_code(0x1138)                 }, // 0x0011
        { 0x1139,  Endress_Hauser_model_code(0x1139)                 }, // 0x0011
        { 0x113A,  Endress_Hauser_model_code(0x113A)                 }, // 0x0011
        { 0x113B,  Endress_Hauser_model_code(0x113B)                 }, // 0x0011
        { 0x113C,  Endress_Hauser_model_code(0x113C)                 }, // 0x0011
        { 0x1140,  Endress_Hauser_model_code(0x1140)                 }, // 0x0011
        { 0x1141,  Endress_Hauser_model_code(0x1141)                 }, // 0x0011
        { 0x1142,  Endress_Hauser_model_code(0x1142)                 }, // 0x0011
        { 0x1143,  Endress_Hauser_model_code(0x1143)                 }, // 0x0011
        { 0x1144,  Endress_Hauser_model_code(0x1144)                 }, // 0x0011
        { 0x1145,  Endress_Hauser_model_code(0x1145)                 }, // 0x0011
        { 0x1146,  Endress_Hauser_model_code(0x1146)                 }, // 0x0011
        { 0x1148,  Endress_Hauser_model_code(0x1148)                 }, // 0x0011
        { 0x1149,  Endress_Hauser_model_code(0x1149)                 }, // 0x0011
        { 0x114A,  Endress_Hauser_model_code(0x114A)                 }, // 0x0011
        { 0x1150,  Endress_Hauser_model_code(0x1150)                 }, // 0x0011
        { 0x1151,  Endress_Hauser_model_code(0x1151)                 }, // 0x0011
        { 0x1152,  Endress_Hauser_model_code(0x1152)                 }, // 0x0011
        { 0x1153,  Endress_Hauser_model_code(0x1153)                 }, // 0x0011
        { 0x1154,  Endress_Hauser_model_code(0x1154)                 }, // 0x0011
        { 0x1155,  Endress_Hauser_model_code(0x1155)                 }, // 0x0011
        { 0x1156,  Endress_Hauser_model_code(0x1156)                 }, // 0x0011
        { 0x1157,  Endress_Hauser_model_code(0x1157)                 }, // 0x0011
        { 0x1158,  Endress_Hauser_model_code(0x1158)                 }, // 0x0011
        { 0x1159,  Endress_Hauser_model_code(0x1159)                 }, // 0x0011
        { 0x115A,  Endress_Hauser_model_code(0x115A)                 }, // 0x0011
        { 0x115C,  Endress_Hauser_model_code(0x115C)                 }, // 0x0011
        { 0x115D,  Endress_Hauser_model_code(0x115D)                 }, // 0x0011
        { 0x1160,  Endress_Hauser_model_code(0x1160)                 }, // 0x0011
        { 0x1161,  Endress_Hauser_model_code(0x1161)                 }, // 0x0011
        { 0x1162,  Endress_Hauser_model_code(0x1162)                 }, // 0x0011
        { 0x1164,  Endress_Hauser_model_code(0x1164)                 }, // 0x0011
        { 0x1165,  Endress_Hauser_model_code(0x1165)                 }, // 0x0011
        { 0x1166,  Endress_Hauser_model_code(0x1166)                 }, // 0x0011
        { 0x1167,  Endress_Hauser_model_code(0x1167)                 }, // 0x0011
        { 0x1168,  Endress_Hauser_model_code(0x1168)                 }, // 0x0011
        { 0x1169,  Endress_Hauser_model_code(0x1169)                 }, // 0x0011
        { 0x1171,  Endress_Hauser_model_code(0x1171)                 }, // 0x0011
        { 0x1172,  Endress_Hauser_model_code(0x1172)                 }, // 0x0011
        { 0x1178,  Endress_Hauser_model_code(0x1178)                 }, // 0x0011
        { 0x1179,  Endress_Hauser_model_code(0x1179)                 }, // 0x0011
        { 0x117C,  Endress_Hauser_model_code(0x117C)                 }, // 0x0011
        { 0x118C,  Endress_Hauser_model_code(0x118C)                 }, // 0x0011
        { 0x118D,  Endress_Hauser_model_code(0x118D)                 }, // 0x0011
        { 0x118E,  Endress_Hauser_model_code(0x118E)                 }, // 0x0011
        { 0x119C,  Endress_Hauser_model_code(0x119C)                 }, // 0x0011
        { 0x119D,  Endress_Hauser_model_code(0x119D)                 }, // 0x0011
        { 0x119F,  Endress_Hauser_model_code(0x119F)                 }, // 0x0011
        { 0x11A3,  Endress_Hauser_model_code(0x11A3)                 }, // 0x0011
        { 0x11B5,  Endress_Hauser_model_code(0x11B5)                 }, // 0x0011
        { 0x11B6,  Endress_Hauser_model_code(0x11B6)                 }, // 0x0011
        { 0x11C8,  Endress_Hauser_model_code(0x11C8)                 }, // 0x0011
        { 0x11C9,  Endress_Hauser_model_code(0x11C9)                 }, // 0x0011
        { 0x11CA,  Endress_Hauser_model_code(0x11CA)                 }, // 0x0011
        { 0x11CB,  Endress_Hauser_model_code(0x11CB)                 }, // 0x0011
        { 0x11CC,  Endress_Hauser_model_code(0x11CC)                 }, // 0x0011
        { 0x11CE,  Endress_Hauser_model_code(0x11CE)                 }, // 0x0011
        { 0x11CF,  Endress_Hauser_model_code(0x11CF)                 }, // 0x0011
        { 0x11D0,  Endress_Hauser_model_code(0x11D0)                 }, // 0x0011
        { 0x11D1,  Endress_Hauser_model_code(0x11D1)                 }, // 0x0011
        { 0x11EE,  Endress_Hauser_model_code(0x11EE)                 }, // 0x0011
        { 0x11EF,  Endress_Hauser_model_code(0x11EF)                 }, // 0x0011
        { 0x11F0,  Endress_Hauser_model_code(0x11F0)                 }, // 0x0011
        { 0x11F1,  Endress_Hauser_model_code(0x11F1)                 }, // 0x0011

        { 0x1207,  Fischer_Porter_model_code(0x1207)                 }, // 0x0012
        { 0x1208,  Fischer_Porter_model_code(0x1208)                 }, // 0x0012
        { 0x120E,  Fischer_Porter_model_code(0x120E)                 }, // 0x0012
        { 0x120F,  Fischer_Porter_model_code(0x120F)                 }, // 0x0012
        { 0x1219,  Fischer_Porter_model_code(0x1219)                 }, // 0x0012
        { 0x121A,  Fischer_Porter_model_code(0x121A)                 }, // 0x0012
        { 0x1242,  Fischer_Porter_model_code(0x1242)                 }, // 0x0012
        { 0x1250,  Fischer_Porter_model_code(0x1250)                 }, // 0x0012
        { 0x1251,  Fischer_Porter_model_code(0x1251)                 }, // 0x0012
        { 0x1252,  Fischer_Porter_model_code(0x1252)                 }, // 0x0012
        { 0x1253,  Fischer_Porter_model_code(0x1253)                 }, // 0x0012
        { 0x1254,  Fischer_Porter_model_code(0x1254)                 }, // 0x0012
        { 0x1255,  Fischer_Porter_model_code(0x1255)                 }, // 0x0012
        { 0x1256,  Fischer_Porter_model_code(0x1256)                 }, // 0x0012
        { 0x1257,  Fischer_Porter_model_code(0x1257)                 }, // 0x0012
        { 0x1258,  Fischer_Porter_model_code(0x1258)                 }, // 0x0012
        { 0x1259,  Fischer_Porter_model_code(0x1259)                 }, // 0x0012

        { 0x1300,  Fisher_Controls_model_code(0x1300)                }, // 0x0013
        { 0x1303,  Fisher_Controls_model_code(0x1303)                }, // 0x0013
        { 0x1304,  Fisher_Controls_model_code(0x1304)                }, // 0x0013
        { 0x1305,  Fisher_Controls_model_code(0x1305)                }, // 0x0013
        { 0x1308,  Fisher_Controls_model_code(0x1308)                }, // 0x0013
        { 0x1309,  Fisher_Controls_model_code(0x1309)                }, // 0x0013
        { 0x130A,  Fisher_Controls_model_code(0x130A)                }, // 0x0013
        { 0x130B,  Fisher_Controls_model_code(0x130B)                }, // 0x0013
        { 0x130F,  Fisher_Controls_model_code(0x130F)                }, // 0x0013

        { 0x1401,  SchneiderElectric_Foxboro_model_code(0x1401)      }, // 0x0014
        { 0x1402,  SchneiderElectric_Foxboro_model_code(0x1402)      }, // 0x0014
        { 0x1403,  SchneiderElectric_Foxboro_model_code(0x1403)      }, // 0x0014
        { 0x1404,  SchneiderElectric_Foxboro_model_code(0x1404)      }, // 0x0014
        { 0x1405,  SchneiderElectric_Foxboro_model_code(0x1405)      }, // 0x0014
        { 0x1407,  SchneiderElectric_Foxboro_model_code(0x1407)      }, // 0x0014
        { 0x1408,  SchneiderElectric_Foxboro_model_code(0x1408)      }, // 0x0014
        { 0x1409,  SchneiderElectric_Foxboro_model_code(0x1409)      }, // 0x0014
        { 0x140A,  SchneiderElectric_Foxboro_model_code(0x140A)      }, // 0x0014
        { 0x140B,  SchneiderElectric_Foxboro_model_code(0x140B)      }, // 0x0014
        { 0x1419,  SchneiderElectric_Foxboro_model_code(0x1419)      }, // 0x0014
        { 0x141B,  SchneiderElectric_Foxboro_model_code(0x141B)      }, // 0x0014
        { 0x141C,  SchneiderElectric_Foxboro_model_code(0x141C)      }, // 0x0014
        { 0x141D,  SchneiderElectric_Foxboro_model_code(0x141D)      }, // 0x0014
        { 0x141E,  SchneiderElectric_Foxboro_model_code(0x141E)      }, // 0x0014
        { 0x141F,  SchneiderElectric_Foxboro_model_code(0x141F)      }, // 0x0014
        { 0x1428,  SchneiderElectric_Foxboro_model_code(0x1428)      }, // 0x0014
        { 0x1429,  SchneiderElectric_Foxboro_model_code(0x1429)      }, // 0x0014
        { 0x142E,  SchneiderElectric_Foxboro_model_code(0x142E)      }, // 0x0014
        { 0x142F,  SchneiderElectric_Foxboro_model_code(0x142F)      }, // 0x0014
        { 0x1430,  SchneiderElectric_Foxboro_model_code(0x1430)      }, // 0x0014
        { 0x1433,  SchneiderElectric_Foxboro_model_code(0x1433)      }, // 0x0014
        { 0x1434,  SchneiderElectric_Foxboro_model_code(0x1434)      }, // 0x0014
        { 0x1435,  SchneiderElectric_Foxboro_model_code(0x1435)      }, // 0x0014
        { 0x1436,  SchneiderElectric_Foxboro_model_code(0x1436)      }, // 0x0014
        { 0x1438,  SchneiderElectric_Foxboro_model_code(0x1438)      }, // 0x0014
        { 0x1440,  SchneiderElectric_Foxboro_model_code(0x1440)      }, // 0x0014
        { 0x1441,  SchneiderElectric_Foxboro_model_code(0x1441)      }, // 0x0014
        { 0x1442,  SchneiderElectric_Foxboro_model_code(0x1442)      }, // 0x0014
        { 0x1443,  SchneiderElectric_Foxboro_model_code(0x1443)      }, // 0x0014
        { 0x1444,  SchneiderElectric_Foxboro_model_code(0x1444)      }, // 0x0014
        { 0x147D,  SchneiderElectric_Foxboro_model_code(0x147D)      }, // 0x0014

        { 0x1501,  Fuji_model_code(0x1501)                           }, // 0x0015
        { 0x1502,  Fuji_model_code(0x1502)                           }, // 0x0015
        { 0x1504,  Fuji_model_code(0x1504)                           }, // 0x0015
        { 0x1505,  Fuji_model_code(0x1505)                           }, // 0x0015
        { 0x1540,  Fuji_model_code(0x1540)                           }, // 0x0015
        { 0x1581,  Fuji_model_code(0x1581)                           }, // 0x0015

        { 0x1603,  Hartmann_Braun_model_code(0x1603)                 }, // 0x0016
        { 0x1604,  Hartmann_Braun_model_code(0x1604)                 }, // 0x0016
        { 0x1605,  Hartmann_Braun_model_code(0x1605)                 }, // 0x0016
        { 0x1606,  Hartmann_Braun_model_code(0x1606)                 }, // 0x0016
        { 0x1608,  Hartmann_Braun_model_code(0x1608)                 }, // 0x0016
        { 0x1640,  Hartmann_Braun_model_code(0x1640)                 }, // 0x0016
        { 0x1641,  Hartmann_Braun_model_code(0x1641)                 }, // 0x0016
        { 0x1685,  Hartmann_Braun_model_code(0x1685)                 }, // 0x0016
        { 0x1687,  Hartmann_Braun_model_code(0x1687)                 }, // 0x0016

        { 0x1701,  Honeywell_model_code(0x1701)                      }, // 0x0017
        { 0x1702,  Honeywell_model_code(0x1702)                      }, // 0x0017
        { 0x1703,  Honeywell_model_code(0x1703)                      }, // 0x0017
        { 0x1704,  Honeywell_model_code(0x1704)                      }, // 0x0017
        { 0x1705,  Honeywell_model_code(0x1705)                      }, // 0x0017
        { 0x1706,  Honeywell_model_code(0x1706)                      }, // 0x0017
        { 0x1707,  Honeywell_model_code(0x1707)                      }, // 0x0017
        { 0x1708,  Honeywell_model_code(0x1708)                      }, // 0x0017
        { 0x1709,  Honeywell_model_code(0x1709)                      }, // 0x0017
        { 0x170A,  Honeywell_model_code(0x170A)                      }, // 0x0017
        { 0x170B,  Honeywell_model_code(0x170B)                      }, // 0x0017
        { 0x170C,  Honeywell_model_code(0x170C)                      }, // 0x0017
        { 0x1720,  Honeywell_model_code(0x1720)                      }, // 0x0017
        { 0x1721,  Honeywell_model_code(0x1721)                      }, // 0x0017
        { 0x1722,  Honeywell_model_code(0x1722)                      }, // 0x0017
        { 0x1724,  Honeywell_model_code(0x1724)                      }, // 0x0017
        { 0x1726,  Honeywell_model_code(0x1726)                      }, // 0x0017
        { 0x1727,  Honeywell_model_code(0x1727)                      }, // 0x0017
        { 0x1729,  Honeywell_model_code(0x1729)                      }, // 0x0017
        { 0x172A,  Honeywell_model_code(0x172A)                      }, // 0x0017
        { 0x172B,  Honeywell_model_code(0x172B)                      }, // 0x0017
        { 0x172C,  Honeywell_model_code(0x172C)                      }, // 0x0017
        { 0x1740,  Honeywell_model_code(0x1740)                      }, // 0x0017
        { 0x1750,  Honeywell_model_code(0x1750)                      }, // 0x0017
        { 0x1751,  Honeywell_model_code(0x1751)                      }, // 0x0017
        { 0x1754,  Honeywell_model_code(0x1754)                      }, // 0x0017
        { 0x1755,  Honeywell_model_code(0x1755)                      }, // 0x0017
        { 0x1756,  Honeywell_model_code(0x1756)                      }, // 0x0017
        { 0x1757,  Honeywell_model_code(0x1757)                      }, // 0x0017
        { 0x1758,  Honeywell_model_code(0x1758)                      }, // 0x0017
        { 0x1759,  Honeywell_model_code(0x1759)                      }, // 0x0017
        { 0x175A,  Honeywell_model_code(0x175A)                      }, // 0x0017
        { 0x175B,  Honeywell_model_code(0x175B)                      }, // 0x0017

        { 0x1908,  kayray_model_code(0x1908)                         }, // 0x0019
        { 0x1909,  kayray_model_code(0x1909)                         }, // 0x0019
        { 0x190A,  kayray_model_code(0x190A)                         }, // 0x0019
        { 0x190B,  kayray_model_code(0x190B)                         }, // 0x0019
        { 0x190F,  kayray_model_code(0x190F)                         }, // 0x0019

        { 0x1A01,  ABB_model_code(0x1A01)                            }, // 0x001A
        { 0x1A02,  ABB_model_code(0x1A02)                            }, // 0x001A
        { 0x1A03,  ABB_model_code(0x1A03)                            }, // 0x001A
        { 0x1A04,  ABB_model_code(0x1A04)                            }, // 0x001A
        { 0x1A05,  ABB_model_code(0x1A05)                            }, // 0x001A
        { 0x1A07,  ABB_model_code(0x1A07)                            }, // 0x001A
        { 0x1A08,  ABB_model_code(0x1A08)                            }, // 0x001A
        { 0x1A09,  ABB_model_code(0x1A09)                            }, // 0x001A
        { 0x1A0A,  ABB_model_code(0x1A0A)                            }, // 0x001A
        { 0x1A0B,  ABB_model_code(0x1A0B)                            }, // 0x001A
        { 0x1A0C,  ABB_model_code(0x1A0C)                            }, // 0x001A
        { 0x1A0D,  ABB_model_code(0x1A0D)                            }, // 0x001A
        { 0x1A0F,  ABB_model_code(0x1A0F)                            }, // 0x001A
        { 0x1A10,  ABB_model_code(0x1A10)                            }, // 0x001A
        { 0x1A18,  ABB_model_code(0x1A18)                            }, // 0x001A
        { 0x1A1A,  ABB_model_code(0x1A1A)                            }, // 0x001A
        { 0x1A1B,  ABB_model_code(0x1A1B)                            }, // 0x001A
        { 0x1A1C,  ABB_model_code(0x1A1C)                            }, // 0x001A
        { 0x1A1D,  ABB_model_code(0x1A1D)                            }, // 0x001A
        { 0x1A1E,  ABB_model_code(0x1A1E)                            }, // 0x001A
        { 0x1A1F,  ABB_model_code(0x1A1F)                            }, // 0x001A
        { 0x1A21,  ABB_model_code(0x1A21)                            }, // 0x001A
        { 0x1A22,  ABB_model_code(0x1A22)                            }, // 0x001A
        { 0x1A23,  ABB_model_code(0x1A23)                            }, // 0x001A
        { 0x1A24,  ABB_model_code(0x1A24)                            }, // 0x001A
        { 0x1A25,  ABB_model_code(0x1A25)                            }, // 0x001A
        { 0x1A26,  ABB_model_code(0x1A26)                            }, // 0x001A
        { 0x1A29,  ABB_model_code(0x1A29)                            }, // 0x001A
        { 0x1A2A,  ABB_model_code(0x1A2A)                            }, // 0x001A
        { 0x1A30,  ABB_model_code(0x1A30)                            }, // 0x001A
        { 0x1A31,  ABB_model_code(0x1A31)                            }, // 0x001A
        { 0x1A32,  ABB_model_code(0x1A32)                            }, // 0x001A
        { 0x1A33,  ABB_model_code(0x1A33)                            }, // 0x001A
        { 0x1A34,  ABB_model_code(0x1A34)                            }, // 0x001A
        { 0x1A35,  ABB_model_code(0x1A35)                            }, // 0x001A
        { 0x1A63,  ABB_model_code(0x1A63)                            }, // 0x001A
        { 0x1A89,  ABB_model_code(0x1A89)                            }, // 0x001A
        { 0x1A8A,  ABB_model_code(0x1A8A)                            }, // 0x001A
        { 0x1A8C,  ABB_model_code(0x1A8C)                            }, // 0x001A
        { 0x1A8D,  ABB_model_code(0x1A8D)                            }, // 0x001A
        { 0x1A8E,  ABB_model_code(0x1A8E)                            }, // 0x001A
        { 0x1A91,  ABB_model_code(0x1A91)                            }, // 0x001A
        { 0x1A99,  ABB_model_code(0x1A99)                            }, // 0x001A
        { 0x1A9B,  ABB_model_code(0x1A9B)                            }, // 0x001A
        { 0x1A9C,  ABB_model_code(0x1A9C)                            }, // 0x001A
        { 0x1A9D,  ABB_model_code(0x1A9D)                            }, // 0x001A
        { 0x1A9E,  ABB_model_code(0x1A9E)                            }, // 0x001A
        { 0x1AA0,  ABB_model_code(0x1AA0)                            }, // 0x001A
        { 0x1AA1,  ABB_model_code(0x1AA1)                            }, // 0x001A
        { 0x1AA3,  ABB_model_code(0x1AA3)                            }, // 0x001A
        { 0x1AA4,  ABB_model_code(0x1AA4)                            }, // 0x001A

        { 0x1D01,  M_System_Co_model_code(0x1D01)                    }, // 0x001D
        { 0x1D02,  M_System_Co_model_code(0x1D02)                    }, // 0x001D
        { 0x1D03,  M_System_Co_model_code(0x1D03)                    }, // 0x001D
        { 0x1D04,  M_System_Co_model_code(0x1D04)                    }, // 0x001D

        { 0x1F07,  micro_motion_model_code(0x1F07)                   }, // 0x001F
        { 0x1F14,  micro_motion_model_code(0x1F14)                   }, // 0x001F
        { 0x1F15,  micro_motion_model_code(0x1F15)                   }, // 0x001F
        { 0x1F16,  micro_motion_model_code(0x1F16)                   }, // 0x001F
        { 0x1F17,  micro_motion_model_code(0x1F17)                   }, // 0x001F
        { 0x1F1E,  micro_motion_model_code(0x1F1E)                   }, // 0x001F
        { 0x1F24,  micro_motion_model_code(0x1F24)                   }, // 0x001F
        { 0x1F25,  micro_motion_model_code(0x1F25)                   }, // 0x001F
        { 0x1F26,  micro_motion_model_code(0x1F26)                   }, // 0x001F
        { 0x1F27,  micro_motion_model_code(0x1F27)                   }, // 0x001F
        { 0x1F29,  micro_motion_model_code(0x1F29)                   }, // 0x001F
        { 0x1F2A,  micro_motion_model_code(0x1F2A)                   }, // 0x001F
        { 0x1F34,  micro_motion_model_code(0x1F34)                   }, // 0x001F
        { 0x1F3A,  micro_motion_model_code(0x1F3A)                   }, // 0x001F
        { 0x1F3D,  micro_motion_model_code(0x1F3D)                   }, // 0x001F
        { 0x1F41,  micro_motion_model_code(0x1F41)                   }, // 0x001F
        { 0x1F46,  micro_motion_model_code(0x1F46)                   }, // 0x001F
        { 0x1F47,  micro_motion_model_code(0x1F47)                   }, // 0x001F
        { 0x1F4B,  micro_motion_model_code(0x1F4B)                   }, // 0x001F

        { 0x2001,  Moore_Industries_model_code(0x2001)               }, // 0x0020
        { 0x2003,  Moore_Industries_model_code(0x2003)               }, // 0x0020
        { 0x2004,  Moore_Industries_model_code(0x2004)               }, // 0x0020
        { 0x2005,  Moore_Industries_model_code(0x2005)               }, // 0x0020
        { 0x2006,  Moore_Industries_model_code(0x2006)               }, // 0x0020
        { 0x2007,  Moore_Industries_model_code(0x2007)               }, // 0x0020
        { 0x2008,  Moore_Industries_model_code(0x2008)               }, // 0x0020
        { 0x2009,  Moore_Industries_model_code(0x2009)               }, // 0x0020

        { 0x2101,  Moore_Products_model_code(0x2101)                 }, // 0x0021
        { 0x2102,  Moore_Products_model_code(0x2102)                 }, // 0x0021
        { 0x2103,  Moore_Products_model_code(0x2103)                 }, // 0x0021
        { 0x2105,  Moore_Products_model_code(0x2105)                 }, // 0x0021
        { 0x2106,  Moore_Products_model_code(0x2106)                 }, // 0x0021
        { 0x2108,  Moore_Products_model_code(0x2108)                 }, // 0x0021
        { 0x2109,  Moore_Products_model_code(0x2109)                 }, // 0x0021
        { 0x210A,  Moore_Products_model_code(0x210A)                 }, // 0x0021
        { 0x210B,  Moore_Products_model_code(0x210B)                 }, // 0x0021
        { 0x210C,  Moore_Products_model_code(0x210C)                 }, // 0x0021
        { 0x210D,  Moore_Products_model_code(0x210D)                 }, // 0x0021

        { 0x2502,  Ronan_model_code(0x2502)                          }, // 0x0025
        { 0x2503,  Ronan_model_code(0x2503)                          }, // 0x0025
        { 0x2504,  Ronan_model_code(0x2504)                          }, // 0x0025
        { 0x2505,  Ronan_model_code(0x2505)                          }, // 0x0025
        { 0x2506,  Ronan_model_code(0x2506)                          }, // 0x0025
        { 0x2513,  Ronan_model_code(0x2513)                          }, // 0x0025
        { 0x2515,  Ronan_model_code(0x2515)                          }, // 0x0025

        { 0x2601,  rosemount_model_code(0x2601)                      }, // 0x0026
        { 0x2602,  rosemount_model_code(0x2602)                      }, // 0x0026
        { 0x2603,  rosemount_model_code(0x2603)                      }, // 0x0026
        { 0x2604,  rosemount_model_code(0x2604)                      }, // 0x0026
        { 0x2605,  rosemount_model_code(0x2605)                      }, // 0x0026
        { 0x2606,  rosemount_model_code(0x2606)                      }, // 0x0026
        { 0x260B,  rosemount_model_code(0x260B)                      }, // 0x0026
        { 0x260C,  rosemount_model_code(0x260C)                      }, // 0x0026
        { 0x260D,  rosemount_model_code(0x260D)                      }, // 0x0026
        { 0x260E,  rosemount_model_code(0x260E)                      }, // 0x0026
        { 0x260F,  rosemount_model_code(0x260F)                      }, // 0x0026
        { 0x2610,  rosemount_model_code(0x2610)                      }, // 0x0026
        { 0x2611,  rosemount_model_code(0x2611)                      }, // 0x0026
        { 0x2612,  rosemount_model_code(0x2612)                      }, // 0x0026
        { 0x2613,  rosemount_model_code(0x2613)                      }, // 0x0026
        { 0x2614,  rosemount_model_code(0x2614)                      }, // 0x0026
        { 0x2615,  rosemount_model_code(0x2615)                      }, // 0x0026
        { 0x2616,  rosemount_model_code(0x2616)                      }, // 0x0026
        { 0x2617,  rosemount_model_code(0x2617)                      }, // 0x0026
        { 0x2618,  rosemount_model_code(0x2618)                      }, // 0x0026
        { 0x2619,  rosemount_model_code(0x2619)                      }, // 0x0026
        { 0x261A,  rosemount_model_code(0x261A)                      }, // 0x0026
        { 0x261B,  rosemount_model_code(0x261B)                      }, // 0x0026
        { 0x261C,  rosemount_model_code(0x261C)                      }, // 0x0026
        { 0x261D,  rosemount_model_code(0x261D)                      }, // 0x0026
        { 0x261E,  rosemount_model_code(0x261E)                      }, // 0x0026
        { 0x261F,  rosemount_model_code(0x261F)                      }, // 0x0026
        { 0x2620,  rosemount_model_code(0x2620)                      }, // 0x0026
        { 0x2621,  rosemount_model_code(0x2621)                      }, // 0x0026
        { 0x2622,  rosemount_model_code(0x2622)                      }, // 0x0026
        { 0x2623,  rosemount_model_code(0x2623)                      }, // 0x0026
        { 0x2624,  rosemount_model_code(0x2624)                      }, // 0x0026
        { 0x2625,  rosemount_model_code(0x2625)                      }, // 0x0026
        { 0x2626,  rosemount_model_code(0x2626)                      }, // 0x0026
        { 0x2627,  rosemount_model_code(0x2627)                      }, // 0x0026
        { 0x2628,  rosemount_model_code(0x2628)                      }, // 0x0026
        { 0x2629,  rosemount_model_code(0x2629)                      }, // 0x0026
        { 0x262A,  rosemount_model_code(0x262A)                      }, // 0x0026
        { 0x262D,  rosemount_model_code(0x262D)                      }, // 0x0026
        { 0x262F,  rosemount_model_code(0x262F)                      }, // 0x0026
        { 0x2636,  rosemount_model_code(0x2636)                      }, // 0x0026
        { 0x2637,  rosemount_model_code(0x2637)                      }, // 0x0026
        { 0x2638,  rosemount_model_code(0x2638)                      }, // 0x0026
        { 0x263A,  rosemount_model_code(0x263A)                      }, // 0x0026
        { 0x263B,  rosemount_model_code(0x263B)                      }, // 0x0026
        { 0x263C,  rosemount_model_code(0x263C)                      }, // 0x0026
        { 0x263D,  rosemount_model_code(0x263D)                      }, // 0x0026
        { 0x263F,  rosemount_model_code(0x263F)                      }, // 0x0026
        { 0x2640,  rosemount_model_code(0x2640)                      }, // 0x0026
        { 0x2641,  rosemount_model_code(0x2641)                      }, // 0x0026
        { 0x2642,  rosemount_model_code(0x2642)                      }, // 0x0026
        { 0x2643,  rosemount_model_code(0x2643)                      }, // 0x0026
        { 0x2644,  rosemount_model_code(0x2644)                      }, // 0x0026
        { 0x2645,  rosemount_model_code(0x2645)                      }, // 0x0026
        { 0x2647,  rosemount_model_code(0x2647)                      }, // 0x0026
        { 0x2649,  rosemount_model_code(0x2649)                      }, // 0x0026
        { 0x264A,  rosemount_model_code(0x264A)                      }, // 0x0026
        { 0x264B,  rosemount_model_code(0x264B)                      }, // 0x0026
        { 0x264E,  rosemount_model_code(0x264E)                      }, // 0x0026
        { 0x264F,  rosemount_model_code(0x264F)                      }, // 0x0026
        { 0x2650,  rosemount_model_code(0x2650)                      }, // 0x0026
        { 0x2651,  rosemount_model_code(0x2651)                      }, // 0x0026
        { 0x2653,  rosemount_model_code(0x2653)                      }, // 0x0026
        { 0x2655,  rosemount_model_code(0x2655)                      }, // 0x0026
        { 0x2656,  rosemount_model_code(0x2656)                      }, // 0x0026
        { 0x2657,  rosemount_model_code(0x2657)                      }, // 0x0026
        { 0x2658,  rosemount_model_code(0x2658)                      }, // 0x0026
        { 0x2659,  rosemount_model_code(0x2659)                      }, // 0x0026
        { 0x265A,  rosemount_model_code(0x265A)                      }, // 0x0026
        { 0x265D,  rosemount_model_code(0x265D)                      }, // 0x0026
        { 0x265E,  rosemount_model_code(0x265E)                      }, // 0x0026
        { 0x2662,  rosemount_model_code(0x2662)                      }, // 0x0026
        { 0x2664,  rosemount_model_code(0x2664)                      }, // 0x0026
        { 0x266A,  rosemount_model_code(0x266A)                      }, // 0x0026
        { 0x266B,  rosemount_model_code(0x266B)                      }, // 0x0026
        { 0x266D,  rosemount_model_code(0x266D)                      }, // 0x0026
        { 0x266E,  rosemount_model_code(0x266E)                      }, // 0x0026
        { 0x266F,  rosemount_model_code(0x266F)                      }, // 0x0026
        { 0x2670,  rosemount_model_code(0x2670)                      }, // 0x0026
        { 0x2673,  rosemount_model_code(0x2673)                      }, // 0x0026
        { 0x2676,  rosemount_model_code(0x2676)                      }, // 0x0026
        { 0x267A,  rosemount_model_code(0x267A)                      }, // 0x0026
        { 0x2680,  rosemount_model_code(0x2680)                      }, // 0x0026
        { 0x2683,  rosemount_model_code(0x2683)                      }, // 0x0026
        { 0x2685,  rosemount_model_code(0x2685)                      }, // 0x0026
        { 0x2687,  rosemount_model_code(0x2687)                      }, // 0x0026
        { 0x268A,  rosemount_model_code(0x268A)                      }, // 0x0026
        { 0x268D,  rosemount_model_code(0x268D)                      }, // 0x0026
        { 0x268E,  rosemount_model_code(0x268E)                      }, // 0x0026
        { 0x2691,  rosemount_model_code(0x2691)                      }, // 0x0026

        { 0x2700,  Peek_Measurement_model_code(0x2700)               }, // 0x0027

        { 0x2801,  Actaris_Neptune_model_code(0x2801)                }, // 0x0028

        { 0x2A01,  Siemens_model_code(0x2A01)                        }, // 0x002A
        { 0x2A02,  Siemens_model_code(0x2A02)                        }, // 0x002A
        { 0x2A03,  Siemens_model_code(0x2A03)                        }, // 0x002A
        { 0x2A04,  Siemens_model_code(0x2A04)                        }, // 0x002A
        { 0x2A05,  Siemens_model_code(0x2A05)                        }, // 0x002A
        { 0x2A06,  Siemens_model_code(0x2A06)                        }, // 0x002A
        { 0x2A07,  Siemens_model_code(0x2A07)                        }, // 0x002A
        { 0x2A08,  Siemens_model_code(0x2A08)                        }, // 0x002A
        { 0x2A0A,  Siemens_model_code(0x2A0A)                        }, // 0x002A
        { 0x2A0B,  Siemens_model_code(0x2A0B)                        }, // 0x002A
        { 0x2A0C,  Siemens_model_code(0x2A0C)                        }, // 0x002A
        { 0x2A0D,  Siemens_model_code(0x2A0D)                        }, // 0x002A
        { 0x2A10,  Siemens_model_code(0x2A10)                        }, // 0x002A
        { 0x2A12,  Siemens_model_code(0x2A12)                        }, // 0x002A
        { 0x2A13,  Siemens_model_code(0x2A13)                        }, // 0x002A
        { 0x2A15,  Siemens_model_code(0x2A15)                        }, // 0x002A
        { 0x2A16,  Siemens_model_code(0x2A16)                        }, // 0x002A
        { 0x2A17,  Siemens_model_code(0x2A17)                        }, // 0x002A
        { 0x2A18,  Siemens_model_code(0x2A18)                        }, // 0x002A
        { 0x2A19,  Siemens_model_code(0x2A19)                        }, // 0x002A
        { 0x2A1A,  Siemens_model_code(0x2A1A)                        }, // 0x002A
        { 0x2A1D,  Siemens_model_code(0x2A1D)                        }, // 0x002A
        { 0x2A1F,  Siemens_model_code(0x2A1F)                        }, // 0x002A
        { 0x2A20,  Siemens_model_code(0x2A20)                        }, // 0x002A
        { 0x2A22,  Siemens_model_code(0x2A22)                        }, // 0x002A
        { 0x2A23,  Siemens_model_code(0x2A23)                        }, // 0x002A
        { 0x2A24,  Siemens_model_code(0x2A24)                        }, // 0x002A
        { 0x2A28,  Siemens_model_code(0x2A28)                        }, // 0x002A
        { 0x2A2D,  Siemens_model_code(0x2A2D)                        }, // 0x002A
        { 0x2A34,  Siemens_model_code(0x2A34)                        }, // 0x002A
        { 0x2A37,  Siemens_model_code(0x2A37)                        }, // 0x002A
        { 0x2A3D,  Siemens_model_code(0x2A3D)                        }, // 0x002A
        { 0x2A3E,  Siemens_model_code(0x2A3E)                        }, // 0x002A

        { 0x2C02,  Toshiba_model_code(0x2C02)                        }, // 0x002C
        { 0x2C03,  Toshiba_model_code(0x2C03)                        }, // 0x002C
        { 0x2C04,  Toshiba_model_code(0x2C04)                        }, // 0x002C
        { 0x2C05,  Toshiba_model_code(0x2C05)                        }, // 0x002C
        { 0x2C06,  Toshiba_model_code(0x2C06)                        }, // 0x002C
        { 0x2C07,  Toshiba_model_code(0x2C07)                        }, // 0x002C
        { 0x2C08,  Toshiba_model_code(0x2C08)                        }, // 0x002C
        { 0x2C09,  Toshiba_model_code(0x2C09)                        }, // 0x002C
        { 0x2C0A,  Toshiba_model_code(0x2C0A)                        }, // 0x002C

        { 0x2E06,  rosemount_analytical_model_code(0x2E06)           }, // 0x002E
        { 0x2E07,  rosemount_analytical_model_code(0x2E07)           }, // 0x002E
        { 0x2E0A,  rosemount_analytical_model_code(0x2E0A)           }, // 0x002E
        { 0x2E0B,  rosemount_analytical_model_code(0x2E0B)           }, // 0x002E
        { 0x2E0C,  rosemount_analytical_model_code(0x2E0C)           }, // 0x002E
        { 0x2E0D,  rosemount_analytical_model_code(0x2E0D)           }, // 0x002E
        { 0x2E0E,  rosemount_analytical_model_code(0x2E0E)           }, // 0x002E
        { 0x2E0F,  rosemount_analytical_model_code(0x2E0F)           }, // 0x002E
        { 0x2E10,  rosemount_analytical_model_code(0x2E10)           }, // 0x002E
        { 0x2E14,  rosemount_analytical_model_code(0x2E14)           }, // 0x002E
        { 0x2E15,  rosemount_analytical_model_code(0x2E15)           }, // 0x002E
        { 0x2E16,  rosemount_analytical_model_code(0x2E16)           }, // 0x002E
        { 0x2E17,  rosemount_analytical_model_code(0x2E17)           }, // 0x002E
        { 0x2E18,  rosemount_analytical_model_code(0x2E18)           }, // 0x002E
        { 0x2E19,  rosemount_analytical_model_code(0x2E19)           }, // 0x002E
        { 0x2E1A,  rosemount_analytical_model_code(0x2E1A)           }, // 0x002E
        { 0x2E1B,  rosemount_analytical_model_code(0x2E1B)           }, // 0x002E
        { 0x2E1C,  rosemount_analytical_model_code(0x2E1C)           }, // 0x002E
        { 0x2E1D,  rosemount_analytical_model_code(0x2E1D)           }, // 0x002E
        { 0x2E1E,  rosemount_analytical_model_code(0x2E1E)           }, // 0x002E
        { 0x2E1F,  rosemount_analytical_model_code(0x2E1F)           }, // 0x002E
        { 0x2E20,  rosemount_analytical_model_code(0x2E20)           }, // 0x002E
        { 0x2E21,  rosemount_analytical_model_code(0x2E21)           }, // 0x002E
        { 0x2E2B,  rosemount_analytical_model_code(0x2E2B)           }, // 0x002E
        { 0x2E3A,  rosemount_analytical_model_code(0x2E3A)           }, // 0x002E
        { 0x2E50,  rosemount_analytical_model_code(0x2E50)           }, // 0x002E
        { 0x2E51,  rosemount_analytical_model_code(0x2E51)           }, // 0x002E
        { 0x2E52,  rosemount_analytical_model_code(0x2E52)           }, // 0x002E
        { 0x2E53,  rosemount_analytical_model_code(0x2E53)           }, // 0x002E
        { 0x2E55,  rosemount_analytical_model_code(0x2E55)           }, // 0x002E
        { 0x2E56,  rosemount_analytical_model_code(0x2E56)           }, // 0x002E

        { 0x2F01,  Valmet_Automation_model_code(0x2F01)              }, // 0x002F
        { 0x2F3C,  Valmet_Automation_model_code(0x2F3C)              }, // 0x002F
        { 0x2F3D,  Valmet_Automation_model_code(0x2F3D)              }, // 0x002F
        { 0x2F3E,  Valmet_Automation_model_code(0x2F3E)              }, // 0x002F
        { 0x2F3F,  Valmet_Automation_model_code(0x2F3F)              }, // 0x002F
        { 0x2F40,  Valmet_Automation_model_code(0x2F40)              }, // 0x002F

        { 0x3001,  Flowserve_model_code(0x3001)                      }, // 0x0030
        { 0x3002,  Flowserve_model_code(0x3002)                      }, // 0x0030
        { 0x3003,  Flowserve_model_code(0x3003)                      }, // 0x0030
        { 0x3004,  Flowserve_model_code(0x3004)                      }, // 0x0030
        { 0x3005,  Flowserve_model_code(0x3005)                      }, // 0x0030
        { 0x3006,  Flowserve_model_code(0x3006)                      }, // 0x0030
        { 0x3007,  Flowserve_model_code(0x3007)                      }, // 0x0030
        { 0x3008,  Flowserve_model_code(0x3008)                      }, // 0x0030
        { 0x3009,  Flowserve_model_code(0x3009)                      }, // 0x0030
        { 0x300A,  Flowserve_model_code(0x300A)                      }, // 0x0030
        { 0x300B,  Flowserve_model_code(0x300B)                      }, // 0x0030

        { 0x3201,  Viatran_model_code(0x3201)                        }, // 0x0032
        { 0x3202,  Viatran_model_code(0x3202)                        }, // 0x0032
        { 0x322E,  Viatran_model_code(0x322E)                        }, // 0x0032

        { 0x3601,  Azbil_Corporation_model_code(0x3601)              }, // 0x0036
        { 0x3602,  Azbil_Corporation_model_code(0x3602)              }, // 0x0036
        { 0x3603,  Azbil_Corporation_model_code(0x3603)              }, // 0x0036
        { 0x3604,  Azbil_Corporation_model_code(0x3604)              }, // 0x0036
        { 0x3605,  Azbil_Corporation_model_code(0x3605)              }, // 0x0036
        { 0x3608,  Azbil_Corporation_model_code(0x3608)              }, // 0x0036
        { 0x360C,  Azbil_Corporation_model_code(0x360C)              }, // 0x0036
        { 0x360D,  Azbil_Corporation_model_code(0x360D)              }, // 0x0036
        { 0x360E,  Azbil_Corporation_model_code(0x360E)              }, // 0x0036
        { 0x360F,  Azbil_Corporation_model_code(0x360F)              }, // 0x0036
        { 0x3610,  Azbil_Corporation_model_code(0x3610)              }, // 0x0036
        { 0x3611,  Azbil_Corporation_model_code(0x3611)              }, // 0x0036
        { 0x3614,  Azbil_Corporation_model_code(0x3614)              }, // 0x0036
        { 0x3615,  Azbil_Corporation_model_code(0x3615)              }, // 0x0036
        { 0x3616,  Azbil_Corporation_model_code(0x3616)              }, // 0x0036

        { 0x3701,  Yokogawa_model_code(0x3701)                       }, // 0x0037
        { 0x3702,  Yokogawa_model_code(0x3702)                       }, // 0x0037
        { 0x3703,  Yokogawa_model_code(0x3703)                       }, // 0x0037
        { 0x3704,  Yokogawa_model_code(0x3704)                       }, // 0x0037
        { 0x3705,  Yokogawa_model_code(0x3705)                       }, // 0x0037
        { 0x3706,  Yokogawa_model_code(0x3706)                       }, // 0x0037
        { 0x3708,  Yokogawa_model_code(0x3708)                       }, // 0x0037
        { 0x3709,  Yokogawa_model_code(0x3709)                       }, // 0x0037
        { 0x370A,  Yokogawa_model_code(0x370A)                       }, // 0x0037
        { 0x370B,  Yokogawa_model_code(0x370B)                       }, // 0x0037
        { 0x370C,  Yokogawa_model_code(0x370C)                       }, // 0x0037
        { 0x370D,  Yokogawa_model_code(0x370D)                       }, // 0x0037
        { 0x3711,  Yokogawa_model_code(0x3711)                       }, // 0x0037
        { 0x3712,  Yokogawa_model_code(0x3712)                       }, // 0x0037
        { 0x3714,  Yokogawa_model_code(0x3714)                       }, // 0x0037
        { 0x3715,  Yokogawa_model_code(0x3715)                       }, // 0x0037
        { 0x3716,  Yokogawa_model_code(0x3716)                       }, // 0x0037
        { 0x3718,  Yokogawa_model_code(0x3718)                       }, // 0x0037
        { 0x371A,  Yokogawa_model_code(0x371A)                       }, // 0x0037
        { 0x371B,  Yokogawa_model_code(0x371B)                       }, // 0x0037
        { 0x371C,  Yokogawa_model_code(0x371C)                       }, // 0x0037
        { 0x3720,  Yokogawa_model_code(0x3720)                       }, // 0x0037
        { 0x3721,  Yokogawa_model_code(0x3721)                       }, // 0x0037
        { 0x3727,  Yokogawa_model_code(0x3727)                       }, // 0x0037
        { 0x3728,  Yokogawa_model_code(0x3728)                       }, // 0x0037
        { 0x372B,  Yokogawa_model_code(0x372B)                       }, // 0x0037
        { 0x372C,  Yokogawa_model_code(0x372C)                       }, // 0x0037
        { 0x372D,  Yokogawa_model_code(0x372D)                       }, // 0x0037
        { 0x373C,  Yokogawa_model_code(0x373C)                       }, // 0x0037
        { 0x373D,  Yokogawa_model_code(0x373D)                       }, // 0x0037
        { 0x373E,  Yokogawa_model_code(0x373E)                       }, // 0x0037
        { 0x373F,  Yokogawa_model_code(0x373F)                       }, // 0x0037
        { 0x3740,  Yokogawa_model_code(0x3740)                       }, // 0x0037
        { 0x3741,  Yokogawa_model_code(0x3741)                       }, // 0x0037
        { 0x3742,  Yokogawa_model_code(0x3742)                       }, // 0x0037
        { 0x3750,  Yokogawa_model_code(0x3750)                       }, // 0x0037
        { 0x3751,  Yokogawa_model_code(0x3751)                       }, // 0x0037
        { 0x3752,  Yokogawa_model_code(0x3752)                       }, // 0x0037
        { 0x3753,  Yokogawa_model_code(0x3753)                       }, // 0x0037
        { 0x3754,  Yokogawa_model_code(0x3754)                       }, // 0x0037
        { 0x3755,  Yokogawa_model_code(0x3755)                       }, // 0x0037
        { 0x3757,  Yokogawa_model_code(0x3757)                       }, // 0x0037
        { 0x3758,  Yokogawa_model_code(0x3758)                       }, // 0x0037
        { 0x3759,  Yokogawa_model_code(0x3759)                       }, // 0x0037
        { 0x375A,  Yokogawa_model_code(0x375A)                       }, // 0x0037
        { 0x375B,  Yokogawa_model_code(0x375B)                       }, // 0x0037
        { 0x375C,  Yokogawa_model_code(0x375C)                       }, // 0x0037
        { 0x375D,  Yokogawa_model_code(0x375D)                       }, // 0x0037
        { 0x375F,  Yokogawa_model_code(0x375F)                       }, // 0x0037
        { 0x3760,  Yokogawa_model_code(0x3760)                       }, // 0x0037

        { 0x3B13,  Solartron_Mobrey_model_code(0x3B13)               }, // 0x003B
        { 0x3B14,  Solartron_Mobrey_model_code(0x3B14)               }, // 0x003B
        { 0x3B15,  Solartron_Mobrey_model_code(0x3B15)               }, // 0x003B
        { 0x3B21,  Solartron_Mobrey_model_code(0x3B21)               }, // 0x003B
        { 0x3B29,  Solartron_Mobrey_model_code(0x3B29)               }, // 0x003B
        { 0x3B2A,  Solartron_Mobrey_model_code(0x3B2A)               }, // 0x003B
        { 0x3B2B,  Solartron_Mobrey_model_code(0x3B2B)               }, // 0x003B
        { 0x3B2C,  Solartron_Mobrey_model_code(0x3B2C)               }, // 0x003B
        { 0x3B2D,  Solartron_Mobrey_model_code(0x3B2D)               }, // 0x003B
        { 0x3B2E,  Solartron_Mobrey_model_code(0x3B2E)               }, // 0x003B
        { 0x3B2F,  Solartron_Mobrey_model_code(0x3B2F)               }, // 0x003B
        { 0x3B30,  Solartron_Mobrey_model_code(0x3B30)               }, // 0x003B
        { 0x3B31,  Solartron_Mobrey_model_code(0x3B31)               }, // 0x003B

        { 0x3D01,  Princo_model_code(0x3D01)                         }, // 0x003D

        { 0x3E01,  Smar_model_code(0x3E01)                           }, // 0x003E
        { 0x3E02,  Smar_model_code(0x3E02)                           }, // 0x003E
        { 0x3E03,  Smar_model_code(0x3E03)                           }, // 0x003E
        { 0x3E04,  Smar_model_code(0x3E04)                           }, // 0x003E
        { 0x3E05,  Smar_model_code(0x3E05)                           }, // 0x003E
        { 0x3E06,  Smar_model_code(0x3E06)                           }, // 0x003E
        { 0x3E09,  Smar_model_code(0x3E09)                           }, // 0x003E
        { 0x3E0A,  Smar_model_code(0x3E0A)                           }, // 0x003E

        { 0x3F01,  SchneiderElectric_Eckardt_model_code(0x3F01)      }, // 0x003F
        { 0x3F02,  SchneiderElectric_Eckardt_model_code(0x3F02)      }, // 0x003F
        { 0x3F03,  SchneiderElectric_Eckardt_model_code(0x3F03)      }, // 0x003F
        { 0x3F04,  SchneiderElectric_Eckardt_model_code(0x3F04)      }, // 0x003F
        { 0x3F05,  SchneiderElectric_Eckardt_model_code(0x3F05)      }, // 0x003F
        { 0x3F06,  SchneiderElectric_Eckardt_model_code(0x3F06)      }, // 0x003F
        { 0x3F07,  SchneiderElectric_Eckardt_model_code(0x3F07)      }, // 0x003F
        { 0x3F08,  SchneiderElectric_Eckardt_model_code(0x3F08)      }, // 0x003F
        { 0x3FD0,  SchneiderElectric_Eckardt_model_code(0x3FD0)      }, // 0x003F
        { 0x3FD7,  SchneiderElectric_Eckardt_model_code(0x3FD7)      }, // 0x003F

        { 0x4015,  Measurement_Technology_model_code(0x4015)         }, // 0x0040
        { 0x4022,  Measurement_Technology_model_code(0x4022)         }, // 0x0040

        { 0x42EB,  Samson_model_code(0x42EB)                         }, // 0x0042
        { 0x42EC,  Samson_model_code(0x42EC)                         }, // 0x0042
        { 0x42ED,  Samson_model_code(0x42ED)                         }, // 0x0042
        { 0x42EE,  Samson_model_code(0x42EE)                         }, // 0x0042
        { 0x42EF,  Samson_model_code(0x42EF)                         }, // 0x0042
        { 0x42F9,  Samson_model_code(0x42F9)                         }, // 0x0042

        { 0x43EE,  Sparling_Instruments_model_code(0x43EE)           }, // 0x0043

        { 0x45AD,  Krohne_model_code(0x45AD)                         }, // 0x0045
        { 0x45AE,  Krohne_model_code(0x45AE)                         }, // 0x0045
        { 0x45AF,  Krohne_model_code(0x45AF)                         }, // 0x0045
        { 0x45B9,  Krohne_model_code(0x45B9)                         }, // 0x0045
        { 0x45BA,  Krohne_model_code(0x45BA)                         }, // 0x0045
        { 0x45BB,  Krohne_model_code(0x45BB)                         }, // 0x0045
        { 0x45BC,  Krohne_model_code(0x45BC)                         }, // 0x0045
        { 0x45BD,  Krohne_model_code(0x45BD)                         }, // 0x0045
        { 0x45BE,  Krohne_model_code(0x45BE)                         }, // 0x0045
        { 0x45BF,  Krohne_model_code(0x45BF)                         }, // 0x0045
        { 0x45C2,  Krohne_model_code(0x45C2)                         }, // 0x0045
        { 0x45C3,  Krohne_model_code(0x45C3)                         }, // 0x0045
        { 0x45C4,  Krohne_model_code(0x45C4)                         }, // 0x0045
        { 0x45C5,  Krohne_model_code(0x45C5)                         }, // 0x0045
        { 0x45C6,  Krohne_model_code(0x45C6)                         }, // 0x0045
        { 0x45C7,  Krohne_model_code(0x45C7)                         }, // 0x0045
        { 0x45C8,  Krohne_model_code(0x45C8)                         }, // 0x0045
        { 0x45C9,  Krohne_model_code(0x45C9)                         }, // 0x0045
        { 0x45CA,  Krohne_model_code(0x45CA)                         }, // 0x0045
        { 0x45CB,  Krohne_model_code(0x45CB)                         }, // 0x0045
        { 0x45CC,  Krohne_model_code(0x45CC)                         }, // 0x0045
        { 0x45CD,  Krohne_model_code(0x45CD)                         }, // 0x0045
        { 0x45CE,  Krohne_model_code(0x45CE)                         }, // 0x0045
        { 0x45CF,  Krohne_model_code(0x45CF)                         }, // 0x0045
        { 0x45D0,  Krohne_model_code(0x45D0)                         }, // 0x0045
        { 0x45D1,  Krohne_model_code(0x45D1)                         }, // 0x0045
        { 0x45D2,  Krohne_model_code(0x45D2)                         }, // 0x0045
        { 0x45D3,  Krohne_model_code(0x45D3)                         }, // 0x0045
        { 0x45D4,  Krohne_model_code(0x45D4)                         }, // 0x0045
        { 0x45D5,  Krohne_model_code(0x45D5)                         }, // 0x0045
        { 0x45D6,  Krohne_model_code(0x45D6)                         }, // 0x0045
        { 0x45D7,  Krohne_model_code(0x45D7)                         }, // 0x0045
        { 0x45D9,  Krohne_model_code(0x45D9)                         }, // 0x0045
        { 0x45DB,  Krohne_model_code(0x45DB)                         }, // 0x0045
        { 0x45DD,  Krohne_model_code(0x45DD)                         }, // 0x0045
        { 0x45DE,  Krohne_model_code(0x45DE)                         }, // 0x0045
        { 0x45E2,  Krohne_model_code(0x45E2)                         }, // 0x0045
        { 0x45E3,  Krohne_model_code(0x45E3)                         }, // 0x0045
        { 0x45E4,  Krohne_model_code(0x45E4)                         }, // 0x0045
        { 0x45E5,  Krohne_model_code(0x45E5)                         }, // 0x0045
        { 0x45E6,  Krohne_model_code(0x45E6)                         }, // 0x0045
        { 0x45E7,  Krohne_model_code(0x45E7)                         }, // 0x0045
        { 0x45E8,  Krohne_model_code(0x45E8)                         }, // 0x0045
        { 0x45E9,  Krohne_model_code(0x45E9)                         }, // 0x0045
        { 0x45EA,  Krohne_model_code(0x45EA)                         }, // 0x0045
        { 0x45EB,  Krohne_model_code(0x45EB)                         }, // 0x0045
        { 0x45EC,  Krohne_model_code(0x45EC)                         }, // 0x0045
        { 0x45ED,  Krohne_model_code(0x45ED)                         }, // 0x0045
        { 0x45EE,  Krohne_model_code(0x45EE)                         }, // 0x0045
        { 0x45EF,  Krohne_model_code(0x45EF)                         }, // 0x0045
        { 0x45F0,  Krohne_model_code(0x45F0)                         }, // 0x0045
        { 0x45F2,  Krohne_model_code(0x45F2)                         }, // 0x0045
        { 0x45F3,  Krohne_model_code(0x45F3)                         }, // 0x0045
        { 0x45F4,  Krohne_model_code(0x45F4)                         }, // 0x0045
        { 0x45F5,  Krohne_model_code(0x45F5)                         }, // 0x0045
        { 0x45F6,  Krohne_model_code(0x45F6)                         }, // 0x0045
        { 0x45F7,  Krohne_model_code(0x45F7)                         }, // 0x0045
        { 0x45F8,  Krohne_model_code(0x45F8)                         }, // 0x0045
        { 0x45F9,  Krohne_model_code(0x45F9)                         }, // 0x0045

        { 0x47E9,  Druck_model_code(0x47E9)                          }, // 0x0047
        { 0x47EC,  Druck_model_code(0x47EC)                          }, // 0x0047
        { 0x47EF,  Druck_model_code(0x47EF)                          }, // 0x0047

        { 0x4800,  SOR_model_code(0x4800)                            }, // 0x0048

        { 0x49EB,  Elcon_Instruments_model_code(0x49EB)              }, // 0x0049
        { 0x49EC,  Elcon_Instruments_model_code(0x49EC)              }, // 0x0049
        { 0x49ED,  Elcon_Instruments_model_code(0x49ED)              }, // 0x0049
        { 0x49EE,  Elcon_Instruments_model_code(0x49EE)              }, // 0x0049

        { 0x4D01,  Westlock_Controls_model_code(0x4D01)              }, // 0x004D
        { 0x4D02,  Westlock_Controls_model_code(0x4D02)              }, // 0x004D
        { 0x4D03,  Westlock_Controls_model_code(0x4D03)              }, // 0x004D
        { 0x4D04,  Westlock_Controls_model_code(0x4D04)              }, // 0x004D
        { 0x4D05,  Westlock_Controls_model_code(0x4D05)              }, // 0x004D
        { 0x4D06,  Westlock_Controls_model_code(0x4D06)              }, // 0x004D

        { 0x4E02,  Drexelbrook_model_code(0x4E02)                    }, // 0x004E
        { 0x4EEA,  Drexelbrook_model_code(0x4EEA)                    }, // 0x004E
        { 0x4EEB,  Drexelbrook_model_code(0x4EEB)                    }, // 0x004E
        { 0x4EEC,  Drexelbrook_model_code(0x4EEC)                    }, // 0x004E
        { 0x4EED,  Drexelbrook_model_code(0x4EED)                    }, // 0x004E

        { 0x4FED,  Rosemount_Tank_Radar_model_code(0x4FED)           }, // 0x004F
        { 0x4FEF,  Rosemount_Tank_Radar_model_code(0x4FEF)           }, // 0x004F

        { 0x5067,  KTEK_model_code(0x5067)                           }, // 0x0050
        { 0x5072,  KTEK_model_code(0x5072)                           }, // 0x0050
        { 0x5073,  KTEK_model_code(0x5073)                           }, // 0x0050
        { 0x5076,  KTEK_model_code(0x5076)                           }, // 0x0050
        { 0x5077,  KTEK_model_code(0x5077)                           }, // 0x0050
        { 0x5078,  KTEK_model_code(0x5078)                           }, // 0x0050
        { 0x507A,  KTEK_model_code(0x507A)                           }, // 0x0050
        { 0x507B,  KTEK_model_code(0x507B)                           }, // 0x0050
        { 0x507E,  KTEK_model_code(0x507E)                           }, // 0x0050
        { 0x507F,  KTEK_model_code(0x507F)                           }, // 0x0050
        { 0x5098,  KTEK_model_code(0x5098)                           }, // 0x0050
        { 0x50AA,  KTEK_model_code(0x50AA)                           }, // 0x0050
        { 0x50AB,  KTEK_model_code(0x50AB)                           }, // 0x0050
        { 0x50BA,  KTEK_model_code(0x50BA)                           }, // 0x0050
        { 0x50C2,  KTEK_model_code(0x50C2)                           }, // 0x0050
        { 0x50CB,  KTEK_model_code(0x50CB)                           }, // 0x0050

        { 0x52E8,  Draeger_model_code(0x52E8)                        }, // 0x0052
        { 0x52E9,  Draeger_model_code(0x52E9)                        }, // 0x0052
        { 0x52EB,  Draeger_model_code(0x52EB)                        }, // 0x0052
        { 0x52ED,  Draeger_model_code(0x52ED)                        }, // 0x0052
        { 0x52EE,  Draeger_model_code(0x52EE)                        }, // 0x0052
        { 0x52EF,  Draeger_model_code(0x52EF)                        }, // 0x0052
        { 0x52F1,  Draeger_model_code(0x52F1)                        }, // 0x0052
        { 0x52F7,  Draeger_model_code(0x52F7)                        }, // 0x0052

        { 0x54F8,  Meridian_model_code(0x54F8)                       }, // 0x0054
        { 0x54F9,  Meridian_model_code(0x54F9)                       }, // 0x0054

        { 0x55DE,  BTG_model_code(0x55DE)                            }, // 0x0055

        { 0x56DC,  Magnetrol_model_code(0x56DC)                      }, // 0x0056
        { 0x56DD,  Magnetrol_model_code(0x56DD)                      }, // 0x0056
        { 0x56DE,  Magnetrol_model_code(0x56DE)                      }, // 0x0056
        { 0x56DF,  Magnetrol_model_code(0x56DF)                      }, // 0x0056
        { 0x56E0,  Magnetrol_model_code(0x56E0)                      }, // 0x0056
        { 0x56E1,  Magnetrol_model_code(0x56E1)                      }, // 0x0056
        { 0x56E2,  Magnetrol_model_code(0x56E2)                      }, // 0x0056
        { 0x56E3,  Magnetrol_model_code(0x56E3)                      }, // 0x0056
        { 0x56E4,  Magnetrol_model_code(0x56E4)                      }, // 0x0056
        { 0x56E5,  Magnetrol_model_code(0x56E5)                      }, // 0x0056
        { 0x56E6,  Magnetrol_model_code(0x56E6)                      }, // 0x0056
        { 0x56E7,  Magnetrol_model_code(0x56E7)                      }, // 0x0056
        { 0x56E8,  Magnetrol_model_code(0x56E8)                      }, // 0x0056
        { 0x56E9,  Magnetrol_model_code(0x56E9)                      }, // 0x0056
        { 0x56EA,  Magnetrol_model_code(0x56EA)                      }, // 0x0056
        { 0x56EB,  Magnetrol_model_code(0x56EB)                      }, // 0x0056
        { 0x56EC,  Magnetrol_model_code(0x56EC)                      }, // 0x0056
        { 0x56ED,  Magnetrol_model_code(0x56ED)                      }, // 0x0056
        { 0x56EE,  Magnetrol_model_code(0x56EE)                      }, // 0x0056
        { 0x56EF,  Magnetrol_model_code(0x56EF)                      }, // 0x0056

        { 0x57A2,  Neles_Finland_Oy_model_code(0x57A2)               }, // 0x0057
        { 0x57A3,  Neles_Finland_Oy_model_code(0x57A3)               }, // 0x0057
        { 0x57D5,  Neles_Finland_Oy_model_code(0x57D5)               }, // 0x0057
        { 0x57D9,  Neles_Finland_Oy_model_code(0x57D9)               }, // 0x0057
        { 0x57DE,  Neles_Finland_Oy_model_code(0x57DE)               }, // 0x0057
        { 0x57E4,  Neles_Finland_Oy_model_code(0x57E4)               }, // 0x0057
        { 0x57E5,  Neles_Finland_Oy_model_code(0x57E5)               }, // 0x0057
        { 0x57EB,  Neles_Finland_Oy_model_code(0x57EB)               }, // 0x0057
        { 0x57EE,  Neles_Finland_Oy_model_code(0x57EE)               }, // 0x0057
        { 0x57EF,  Neles_Finland_Oy_model_code(0x57EF)               }, // 0x0057
        { 0x57F1,  Neles_Finland_Oy_model_code(0x57F1)               }, // 0x0057

        { 0x58C8,  Milltronics_model_code(0x58C8)                    }, // 0x0058
        { 0x58C9,  Milltronics_model_code(0x58C9)                    }, // 0x0058
        { 0x58CA,  Milltronics_model_code(0x58CA)                    }, // 0x0058
        { 0x58CC,  Milltronics_model_code(0x58CC)                    }, // 0x0058
        { 0x58CD,  Milltronics_model_code(0x58CD)                    }, // 0x0058
        { 0x58CE,  Milltronics_model_code(0x58CE)                    }, // 0x0058
        { 0x58CF,  Milltronics_model_code(0x58CF)                    }, // 0x0058
        { 0x58D1,  Milltronics_model_code(0x58D1)                    }, // 0x0058
        { 0x58D3,  Milltronics_model_code(0x58D3)                    }, // 0x0058
        { 0x58D4,  Milltronics_model_code(0x58D4)                    }, // 0x0058
        { 0x58D6,  Milltronics_model_code(0x58D6)                    }, // 0x0058

        { 0x5AC8,  Anderson_Instrument_Company_model_code(0x5AC8)    }, // 0x005A
        { 0x5ACC,  Anderson_Instrument_Company_model_code(0x5ACC)    }, // 0x005A

        { 0x5BEA,  Inor_model_code(0x5BEA)                           }, // 0x005B
        { 0x5BEB,  Inor_model_code(0x5BEB)                           }, // 0x005B
        { 0x5BEF,  Inor_model_code(0x5BEF)                           }, // 0x005B

        { 0x5CC8,  Robertshaw_model_code(0x5CC8)                     }, // 0x005C

        { 0x5DE4,  PEPPERL_FUCHS_model_code(0x5DE4)                  }, // 0x005D
        { 0x5DE5,  PEPPERL_FUCHS_model_code(0x5DE5)                  }, // 0x005D
        { 0x5DE6,  PEPPERL_FUCHS_model_code(0x5DE6)                  }, // 0x005D
        { 0x5DE7,  PEPPERL_FUCHS_model_code(0x5DE7)                  }, // 0x005D
        { 0x5DE8,  PEPPERL_FUCHS_model_code(0x5DE8)                  }, // 0x005D
        { 0x5DE9,  PEPPERL_FUCHS_model_code(0x5DE9)                  }, // 0x005D
        { 0x5DEA,  PEPPERL_FUCHS_model_code(0x5DEA)                  }, // 0x005D
        { 0x5DEB,  PEPPERL_FUCHS_model_code(0x5DEB)                  }, // 0x005D
        { 0x5DEC,  PEPPERL_FUCHS_model_code(0x5DEC)                  }, // 0x005D
        { 0x5DED,  PEPPERL_FUCHS_model_code(0x5DED)                  }, // 0x005D
        { 0x5DEE,  PEPPERL_FUCHS_model_code(0x5DEE)                  }, // 0x005D
        { 0x5DEF,  PEPPERL_FUCHS_model_code(0x5DEF)                  }, // 0x005D

        { 0x5EEE,  Accutech_model_code(0x5EEE)                       }, // 0x005E
        { 0x5EEF,  Accutech_model_code(0x5EEF)                       }, // 0x005E

        { 0x60EB,  Baumer_model_code(0x60EB)                         }, // 0x0060
        { 0x60EC,  Baumer_model_code(0x60EC)                         }, // 0x0060
        { 0x60ED,  Baumer_model_code(0x60ED)                         }, // 0x0060
        { 0x60EE,  Baumer_model_code(0x60EE)                         }, // 0x0060
        { 0x60EF,  Baumer_model_code(0x60EF)                         }, // 0x0060

        { 0x61CD,  Knick_model_code(0x61CD)                          }, // 0x0061
        { 0x61CE,  Knick_model_code(0x61CE)                          }, // 0x0061
        { 0x61E0,  Knick_model_code(0x61E0)                          }, // 0x0061
        { 0x61E1,  Knick_model_code(0x61E1)                          }, // 0x0061
        { 0x61E2,  Knick_model_code(0x61E2)                          }, // 0x0061
        { 0x61E3,  Knick_model_code(0x61E3)                          }, // 0x0061
        { 0x61E4,  Knick_model_code(0x61E4)                          }, // 0x0061
        { 0x61E5,  Knick_model_code(0x61E5)                          }, // 0x0061
        { 0x61E6,  Knick_model_code(0x61E6)                          }, // 0x0061
        { 0x61E7,  Knick_model_code(0x61E7)                          }, // 0x0061
        { 0x61E8,  Knick_model_code(0x61E8)                          }, // 0x0061
        { 0x61E9,  Knick_model_code(0x61E9)                          }, // 0x0061
        { 0x61EA,  Knick_model_code(0x61EA)                          }, // 0x0061
        { 0x61EB,  Knick_model_code(0x61EB)                          }, // 0x0061

        { 0x62BC,  VEGA_model_code(0x62BC)                           }, // 0x0062
        { 0x62BD,  VEGA_model_code(0x62BD)                           }, // 0x0062
        { 0x62BE,  VEGA_model_code(0x62BE)                           }, // 0x0062
        { 0x62BF,  VEGA_model_code(0x62BF)                           }, // 0x0062
        { 0x62C0,  VEGA_model_code(0x62C0)                           }, // 0x0062
        { 0x62C1,  VEGA_model_code(0x62C1)                           }, // 0x0062
        { 0x62C2,  VEGA_model_code(0x62C2)                           }, // 0x0062
        { 0x62C3,  VEGA_model_code(0x62C3)                           }, // 0x0062
        { 0x62C4,  VEGA_model_code(0x62C4)                           }, // 0x0062
        { 0x62C5,  VEGA_model_code(0x62C5)                           }, // 0x0062
        { 0x62C6,  VEGA_model_code(0x62C6)                           }, // 0x0062
        { 0x62C7,  VEGA_model_code(0x62C7)                           }, // 0x0062
        { 0x62C8,  VEGA_model_code(0x62C8)                           }, // 0x0062
        { 0x62C9,  VEGA_model_code(0x62C9)                           }, // 0x0062
        { 0x62CA,  VEGA_model_code(0x62CA)                           }, // 0x0062
        { 0x62CB,  VEGA_model_code(0x62CB)                           }, // 0x0062
        { 0x62CC,  VEGA_model_code(0x62CC)                           }, // 0x0062
        { 0x62CD,  VEGA_model_code(0x62CD)                           }, // 0x0062
        { 0x62CE,  VEGA_model_code(0x62CE)                           }, // 0x0062
        { 0x62CF,  VEGA_model_code(0x62CF)                           }, // 0x0062
        { 0x62D0,  VEGA_model_code(0x62D0)                           }, // 0x0062
        { 0x62D1,  VEGA_model_code(0x62D1)                           }, // 0x0062
        { 0x62D2,  VEGA_model_code(0x62D2)                           }, // 0x0062
        { 0x62D3,  VEGA_model_code(0x62D3)                           }, // 0x0062
        { 0x62D4,  VEGA_model_code(0x62D4)                           }, // 0x0062
        { 0x62D5,  VEGA_model_code(0x62D5)                           }, // 0x0062
        { 0x62D6,  VEGA_model_code(0x62D6)                           }, // 0x0062
        { 0x62D7,  VEGA_model_code(0x62D7)                           }, // 0x0062
        { 0x62D8,  VEGA_model_code(0x62D8)                           }, // 0x0062
        { 0x62D9,  VEGA_model_code(0x62D9)                           }, // 0x0062
        { 0x62DA,  VEGA_model_code(0x62DA)                           }, // 0x0062
        { 0x62DB,  VEGA_model_code(0x62DB)                           }, // 0x0062
        { 0x62DC,  VEGA_model_code(0x62DC)                           }, // 0x0062
        { 0x62DE,  VEGA_model_code(0x62DE)                           }, // 0x0062
        { 0x62DF,  VEGA_model_code(0x62DF)                           }, // 0x0062
        { 0x62E1,  VEGA_model_code(0x62E1)                           }, // 0x0062
        { 0x62E2,  VEGA_model_code(0x62E2)                           }, // 0x0062

        { 0x63ED,  MTS_SYS_CORPS_model_code(0x63ED)                  }, // 0x0063
        { 0x63EE,  MTS_SYS_CORPS_model_code(0x63EE)                  }, // 0x0063
        { 0x63EF,  MTS_SYS_CORPS_model_code(0x63EF)                  }, // 0x0063

        { 0x6464,  Oval_model_code(0x6464)                           }, // 0x0064
        { 0x6465,  Oval_model_code(0x6465)                           }, // 0x0064
        { 0x646E,  Oval_model_code(0x646E)                           }, // 0x0064
        { 0x646F,  Oval_model_code(0x646F)                           }, // 0x0064
        { 0x6470,  Oval_model_code(0x6470)                           }, // 0x0064
        { 0x647B,  Oval_model_code(0x647B)                           }, // 0x0064
        { 0x6482,  Oval_model_code(0x6482)                           }, // 0x0064
        { 0x6484,  Oval_model_code(0x6484)                           }, // 0x0064

        { 0x6564,  GE_Masoneilan_model_code(0x6564)                  }, // 0x0065
        { 0x6565,  GE_Masoneilan_model_code(0x6565)                  }, // 0x0065
        { 0x65AA,  GE_Masoneilan_model_code(0x65AA)                  }, // 0x0065
        { 0x65C8,  GE_Masoneilan_model_code(0x65C8)                  }, // 0x0065
        { 0x65C9,  GE_Masoneilan_model_code(0x65C9)                  }, // 0x0065
        { 0x65CA,  GE_Masoneilan_model_code(0x65CA)                  }, // 0x0065
        { 0x65CB,  GE_Masoneilan_model_code(0x65CB)                  }, // 0x0065
        { 0x65CC,  GE_Masoneilan_model_code(0x65CC)                  }, // 0x0065
        { 0x65CD,  GE_Masoneilan_model_code(0x65CD)                  }, // 0x0065
        { 0x65CE,  GE_Masoneilan_model_code(0x65CE)                  }, // 0x0065
        { 0x65CF,  GE_Masoneilan_model_code(0x65CF)                  }, // 0x0065
        { 0x65EE,  GE_Masoneilan_model_code(0x65EE)                  }, // 0x0065
        { 0x65EF,  GE_Masoneilan_model_code(0x65EF)                  }, // 0x0065

        { 0x66EF,  Besta_model_code(0x66EF)                          }, // 0x0066

        { 0x67E0,  Ohmart_model_code(0x67E0)                         }, // 0x0067
        { 0x67E1,  Ohmart_model_code(0x67E1)                         }, // 0x0067

        { 0x6801,  Harold_Beck_and_Sons_model_code(0x6801)           }, // 0x0068
        { 0x680A,  Harold_Beck_and_Sons_model_code(0x680A)           }, // 0x0068
        { 0x68EF,  Harold_Beck_and_Sons_model_code(0x68EF)           }, // 0x0068

        { 0x6BE4,  Wika_model_code(0x6BE4)                           }, // 0x006B
        { 0x6BE5,  Wika_model_code(0x6BE5)                           }, // 0x006B
        { 0x6BEB,  Wika_model_code(0x6BEB)                           }, // 0x006B
        { 0x6BEC,  Wika_model_code(0x6BEC)                           }, // 0x006B
        { 0x6BEE,  Wika_model_code(0x6BEE)                           }, // 0x006B
        { 0x6BEF,  Wika_model_code(0x6BEF)                           }, // 0x006B
        { 0x6BF0,  Wika_model_code(0x6BF0)                           }, // 0x006B
        { 0x6BF1,  Wika_model_code(0x6BF1)                           }, // 0x006B

        { 0x6CEB,  Bopp_Reuther_Heinrichs_model_code(0x6CEB)         }, // 0x006C
        { 0x6CEC,  Bopp_Reuther_Heinrichs_model_code(0x6CEC)         }, // 0x006C
        { 0x6CED,  Bopp_Reuther_Heinrichs_model_code(0x6CED)         }, // 0x006C
        { 0x6CEE,  Bopp_Reuther_Heinrichs_model_code(0x6CEE)         }, // 0x006C
        { 0x6CEF,  Bopp_Reuther_Heinrichs_model_code(0x6CEF)         }, // 0x006C

        { 0x6DDF,  PR_Electronics_model_code(0x6DDF)                 }, // 0x006D
        { 0x6DE6,  PR_Electronics_model_code(0x6DE6)                 }, // 0x006D
        { 0x6DE7,  PR_Electronics_model_code(0x6DE7)                 }, // 0x006D
        { 0x6DE8,  PR_Electronics_model_code(0x6DE8)                 }, // 0x006D
        { 0x6DE9,  PR_Electronics_model_code(0x6DE9)                 }, // 0x006D
        { 0x6DEA,  PR_Electronics_model_code(0x6DEA)                 }, // 0x006D
        { 0x6DEB,  PR_Electronics_model_code(0x6DEB)                 }, // 0x006D
        { 0x6DEE,  PR_Electronics_model_code(0x6DEE)                 }, // 0x006D
        { 0x6DEF,  PR_Electronics_model_code(0x6DEF)                 }, // 0x006D

        { 0x6EDD,  Rotork_Process_Controls_model_code(0x6EDD)        }, // 0x006E

        { 0x71EE,  Apparatebau_Hundsbach_model_code(0x71EE)          }, // 0x0071

        { 0x72DE,  Dynisco_model_code(0x72DE)                        }, // 0x0072
        { 0x72DF,  Dynisco_model_code(0x72DF)                        }, // 0x0072
        { 0x72E0,  Dynisco_model_code(0x72E0)                        }, // 0x0072

        { 0x74EF,  Direct_Measurement_model_code(0x74EF)             }, // 0x0074

        { 0x75EF,  Klay_model_code(0x75EF)                           }, // 0x0075

        { 0x76ED,  CiDRA_CORP_model_code(0x76ED)                     }, // 0x0076
        { 0x76EE,  CiDRA_CORP_model_code(0x76EE)                     }, // 0x0076
        { 0x76EF,  CiDRA_CORP_model_code(0x76EF)                     }, // 0x0076

        { 0x78EF,  Buerkert_Fluid_Control_Systems_model_code(0x78EF) }, // 0x0078

        { 0x79EF,  Venture_Measurement_model_code(0x79EF)            }, // 0x0079

        { 0x7CEE,  GLI_model_code(0x7CEE)                            }, // 0x007C
        { 0x7CEF,  GLI_model_code(0x7CEF)                            }, // 0x007C

        { 0x7EE0,  Paper_Machine_Components_model_code(0x7EE0)       }, // 0x007E
        { 0x7EE1,  Paper_Machine_Components_model_code(0x7EE1)       }, // 0x007E

        { 0x7F3C,  Labom_model_code(0x7F3C)                          }, // 0x007F
        { 0x7FE0,  Labom_model_code(0x7FE0)                          }, // 0x007F
        { 0x7FEC,  Labom_model_code(0x7FEC)                          }, // 0x007F
        { 0x7FEF,  Labom_model_code(0x7FEF)                          }, // 0x007F

        { 0x8003,  Danfoss_model_code(0x8003)                        }, // 0x0080

        { 0x817F,  Turbo_model_code(0x817F)                          }, // 0x0081

        { 0x8254,  Tokyo_Keiso_model_code(0x8254)                    }, // 0x0082
        { 0x8257,  Tokyo_Keiso_model_code(0x8257)                    }, // 0x0082
        { 0x8258,  Tokyo_Keiso_model_code(0x8258)                    }, // 0x0082
        { 0x8259,  Tokyo_Keiso_model_code(0x8259)                    }, // 0x0082
        { 0x825B,  Tokyo_Keiso_model_code(0x825B)                    }, // 0x0082
        { 0x825C,  Tokyo_Keiso_model_code(0x825C)                    }, // 0x0082
        { 0x8260,  Tokyo_Keiso_model_code(0x8260)                    }, // 0x0082
        { 0x8261,  Tokyo_Keiso_model_code(0x8261)                    }, // 0x0082
        { 0x8262,  Tokyo_Keiso_model_code(0x8262)                    }, // 0x0082
        { 0x8263,  Tokyo_Keiso_model_code(0x8263)                    }, // 0x0082
        { 0x8265,  Tokyo_Keiso_model_code(0x8265)                    }, // 0x0082
        { 0x8267,  Tokyo_Keiso_model_code(0x8267)                    }, // 0x0082
        { 0x8269,  Tokyo_Keiso_model_code(0x8269)                    }, // 0x0082
        { 0x827B,  Tokyo_Keiso_model_code(0x827B)                    }, // 0x0082
        { 0x827F,  Tokyo_Keiso_model_code(0x827F)                    }, // 0x0082

        { 0x837D,  SMC_model_code(0x837D)                            }, // 0x0083
        { 0x837E,  SMC_model_code(0x837E)                            }, // 0x0083
        { 0x837F,  SMC_model_code(0x837F)                            }, // 0x0083
        { 0x83EF,  SMC_model_code(0x83EF)                            }, // 0x0083

        { 0x84EF,  Status_Instruments_model_code(0x84EF)             }, // 0x0084

        { 0x857F,  Huakong_model_code(0x857F)                        }, // 0x0085

        { 0x8661,  Autrol_Duon_model_code(0x8661)                    }, // 0x0086
        { 0x8667,  Autrol_Duon_model_code(0x8667)                    }, // 0x0086
        { 0x866B,  Autrol_Duon_model_code(0x866B)                    }, // 0x0086
        { 0x8673,  Autrol_Duon_model_code(0x8673)                    }, // 0x0086
        { 0x8675,  Autrol_Duon_model_code(0x8675)                    }, // 0x0086
        { 0x8676,  Autrol_Duon_model_code(0x8676)                    }, // 0x0086
        { 0x8678,  Autrol_Duon_model_code(0x8678)                    }, // 0x0086
        { 0x867B,  Autrol_Duon_model_code(0x867B)                    }, // 0x0086
        { 0x867E,  Autrol_Duon_model_code(0x867E)                    }, // 0x0086

        { 0x8701,  Vortek_Instruments_model_code(0x8701)             }, // 0x0087

        { 0x89EF,  Action_Instruments_model_code(0x89EF)             }, // 0x0089

        { 0x8C88,  Magtech_model_code(0x8C88)                        }, // 0x008C
        { 0x8C89,  Magtech_model_code(0x8C89)                        }, // 0x008C
        { 0x8C8A,  Magtech_model_code(0x8C8A)                        }, // 0x008C

        { 0x8D01,  Rueger_model_code(0x8D01)                         }, // 0x008D
        { 0x8D7F,  Rueger_model_code(0x8D7F)                         }, // 0x008D

        { 0x8E70,  Mettler_Toledo_model_code(0x8E70)                 }, // 0x008E
        { 0x8E74,  Mettler_Toledo_model_code(0x8E74)                 }, // 0x008E
        { 0x8E75,  Mettler_Toledo_model_code(0x8E75)                 }, // 0x008E
        { 0x8E76,  Mettler_Toledo_model_code(0x8E76)                 }, // 0x008E
        { 0x8E77,  Mettler_Toledo_model_code(0x8E77)                 }, // 0x008E
        { 0x8E78,  Mettler_Toledo_model_code(0x8E78)                 }, // 0x008E
        { 0x8E79,  Mettler_Toledo_model_code(0x8E79)                 }, // 0x008E
        { 0x8E7A,  Mettler_Toledo_model_code(0x8E7A)                 }, // 0x008E
        { 0x8E7B,  Mettler_Toledo_model_code(0x8E7B)                 }, // 0x008E
        { 0x8E7C,  Mettler_Toledo_model_code(0x8E7C)                 }, // 0x008E
        { 0x8E7D,  Mettler_Toledo_model_code(0x8E7D)                 }, // 0x008E

        { 0x8F73,  Det_Tronics_model_code(0x8F73)                    }, // 0x008F
        { 0x8F74,  Det_Tronics_model_code(0x8F74)                    }, // 0x008F
        { 0x8F75,  Det_Tronics_model_code(0x8F75)                    }, // 0x008F
        { 0x8F76,  Det_Tronics_model_code(0x8F76)                    }, // 0x008F
        { 0x8F77,  Det_Tronics_model_code(0x8F77)                    }, // 0x008F
        { 0x8F78,  Det_Tronics_model_code(0x8F78)                    }, // 0x008F
        { 0x8F79,  Det_Tronics_model_code(0x8F79)                    }, // 0x008F
        { 0x8F7A,  Det_Tronics_model_code(0x8F7A)                    }, // 0x008F
        { 0x8F7B,  Det_Tronics_model_code(0x8F7B)                    }, // 0x008F
        { 0x8F7C,  Det_Tronics_model_code(0x8F7C)                    }, // 0x008F
        { 0x8F7E,  Det_Tronics_model_code(0x8F7E)                    }, // 0x008F
        { 0x8F7F,  Det_Tronics_model_code(0x8F7F)                    }, // 0x008F

        { 0x907B,  Thermo_MeasureTech_model_code(0x907B)             }, // 0x0090
        { 0x907C,  Thermo_MeasureTech_model_code(0x907C)             }, // 0x0090
        { 0x907D,  Thermo_MeasureTech_model_code(0x907D)             }, // 0x0090
        { 0x907E,  Thermo_MeasureTech_model_code(0x907E)             }, // 0x0090
        { 0x907F,  Thermo_MeasureTech_model_code(0x907F)             }, // 0x0090

        { 0x91EF,  Dezurik_model_code(0x91EF)                        }, // 0x0091

        { 0x927F,  Phase_Dynamics_model_code(0x927F)                 }, // 0x0092

        { 0x937A,  Welltech_model_code(0x937A)                       }, // 0x0093
        { 0x937B,  Welltech_model_code(0x937B)                       }, // 0x0093
        { 0x937C,  Welltech_model_code(0x937C)                       }, // 0x0093

        { 0x9480,  ENRAF_model_code(0x9480)                          }, // 0x0094

        { 0x9977,  Metran_model_code(0x9977)                         }, // 0x0099
        { 0x9979,  Metran_model_code(0x9979)                         }, // 0x0099
        { 0x997B,  Metran_model_code(0x997B)                         }, // 0x0099
        { 0x997C,  Metran_model_code(0x997C)                         }, // 0x0099
        { 0x997D,  Metran_model_code(0x997D)                         }, // 0x0099
        { 0x997E,  Metran_model_code(0x997E)                         }, // 0x0099
        { 0x997F,  Metran_model_code(0x997F)                         }, // 0x0099

        { 0x9A7F,  Milton_Roy_model_code(0x9A7F)                     }, // 0x009A

        { 0x9BD3,  PMV_model_code(0x9BD3)                            }, // 0x009B

        { 0x9C01,  Turck_model_code(0x9C01)                          }, // 0x009C

        { 0x9D70,  Panametrics_model_code(0x9D70)                    }, // 0x009D
        { 0x9D77,  Panametrics_model_code(0x9D77)                    }, // 0x009D
        { 0x9D79,  Panametrics_model_code(0x9D79)                    }, // 0x009D

        { 0x9E01,  Stahl_model_code(0x9E01)                          }, // 0x009E
        { 0x9E02,  Stahl_model_code(0x9E02)                          }, // 0x009E
        { 0x9E03,  Stahl_model_code(0x9E03)                          }, // 0x009E
        { 0x9E04,  Stahl_model_code(0x9E04)                          }, // 0x009E
        { 0x9E05,  Stahl_model_code(0x9E05)                          }, // 0x009E
        { 0x9E06,  Stahl_model_code(0x9E06)                          }, // 0x009E
        { 0x9E07,  Stahl_model_code(0x9E07)                          }, // 0x009E

        { 0x9F7F,  Analytical_Technology_Inc_model_code(0x9F7F)      }, // 0x009F

        { 0xA002,  FINT_model_code(0xA002)                           }, // 0x00A0
        { 0xA0ED,  FINT_model_code(0xA0ED)                           }, // 0x00A0

        { 0xA175,  BERTHOLD_model_code(0xA175)                       }, // 0x00A1
        { 0xA179,  BERTHOLD_model_code(0xA179)                       }, // 0x00A1
        { 0xA17A,  BERTHOLD_model_code(0xA17A)                       }, // 0x00A1
        { 0xA17B,  BERTHOLD_model_code(0xA17B)                       }, // 0x00A1
        { 0xA17C,  BERTHOLD_model_code(0xA17C)                       }, // 0x00A1
        { 0xA17D,  BERTHOLD_model_code(0xA17D)                       }, // 0x00A1
        { 0xA17F,  BERTHOLD_model_code(0xA17F)                       }, // 0x00A1

        { 0xA2EF,  InterCorr_model_code(0xA2EF)                      }, // 0x00A2

        { 0xA57F,  Sierra_Instruments_model_code(0xA57F)             }, // 0x00A5

        { 0xA677,  Fluid_Components_model_code(0xA677)               }, // 0x00A6
        { 0xA678,  Fluid_Components_model_code(0xA678)               }, // 0x00A6
        { 0xA679,  Fluid_Components_model_code(0xA679)               }, // 0x00A6
        { 0xA67E,  Fluid_Components_model_code(0xA67E)               }, // 0x00A6
        { 0xA67F,  Fluid_Components_model_code(0xA67F)               }, // 0x00A6

        { 0xA87F,  Meriam_Instrument_model_code(0xA87F)              }, // 0x00A8

        { 0xA92E,  Invensys_model_code(0xA92E)                       }, // 0x00A9
        { 0xA934,  Invensys_model_code(0xA934)                       }, // 0x00A9

        { 0xAB01,  CRANE_model_code(0xAB01)                          }, // 0x00AB

        { 0xAD00,  JTec_Associates_model_code(0xAD00)                }, // 0x00AD
        { 0xAD01,  JTec_Associates_model_code(0xAD01)                }, // 0x00AD

        { 0xAE7A,  Tracerco_model_code(0xAE7A)                       }, // 0x00AE
        { 0xAE7B,  Tracerco_model_code(0xAE7B)                       }, // 0x00AE
        { 0xAE7E,  Tracerco_model_code(0xAE7E)                       }, // 0x00AE
        { 0xAE7F,  Tracerco_model_code(0xAE7F)                       }, // 0x00AE
        { 0xAE9F,  Tracerco_model_code(0xAE9F)                       }, // 0x00AE
        { 0xAEA0,  Tracerco_model_code(0xAEA0)                       }, // 0x00AE

        { 0xAF7E,  AGAR_model_code(0xAF7E)                           }, // 0x00AF

        { 0xB001,  Phoenix_Contact_model_code(0xB001)                }, // 0x00B0
        { 0xB002,  Phoenix_Contact_model_code(0xB002)                }, // 0x00B0
        { 0xB003,  Phoenix_Contact_model_code(0xB003)                }, // 0x00B0
        { 0xB004,  Phoenix_Contact_model_code(0xB004)                }, // 0x00B0
        { 0xB005,  Phoenix_Contact_model_code(0xB005)                }, // 0x00B0
        { 0xB006,  Phoenix_Contact_model_code(0xB006)                }, // 0x00B0
        { 0xB010,  Phoenix_Contact_model_code(0xB010)                }, // 0x00B0
        { 0xB011,  Phoenix_Contact_model_code(0xB011)                }, // 0x00B0
        { 0xB012,  Phoenix_Contact_model_code(0xB012)                }, // 0x00B0
        { 0xB013,  Phoenix_Contact_model_code(0xB013)                }, // 0x00B0

        { 0xB27F,  Amer_Level_Instr_model_code(0xB27F)               }, // 0x00B2

        { 0xB301,  Hawk_model_code(0xB301)                           }, // 0x00B3
        { 0xB302,  Hawk_model_code(0xB302)                           }, // 0x00B3

        { 0xB401,  Rotork_YTC_model_code(0xB401)                     }, // 0x00B4
        { 0xB402,  Rotork_YTC_model_code(0xB402)                     }, // 0x00B4
        { 0xB403,  Rotork_YTC_model_code(0xB403)                     }, // 0x00B4
        { 0xB404,  Rotork_YTC_model_code(0xB404)                     }, // 0x00B4
        { 0xB405,  Rotork_YTC_model_code(0xB405)                     }, // 0x00B4
        { 0xB407,  Rotork_YTC_model_code(0xB407)                     }, // 0x00B4
        { 0xB409,  Rotork_YTC_model_code(0xB409)                     }, // 0x00B4
        { 0xB40A,  Rotork_YTC_model_code(0xB40A)                     }, // 0x00B4
        { 0xB40B,  Rotork_YTC_model_code(0xB40B)                     }, // 0x00B4
        { 0xB47F,  Rotork_YTC_model_code(0xB47F)                     }, // 0x00B4

        { 0xB57F,  Pyromation_Inc_model_code(0xB57F)                 }, // 0x00B5
        { 0xB580,  Pyromation_Inc_model_code(0xB580)                 }, // 0x00B5
        { 0xB581,  Pyromation_Inc_model_code(0xB581)                 }, // 0x00B5
        { 0xB582,  Pyromation_Inc_model_code(0xB582)                 }, // 0x00B5

        { 0xB67F,  Satron_Instruments_model_code(0xB67F)             }, // 0x00B6

        { 0xB77C,  BIFFI_model_code(0xB77C)                          }, // 0x00B7
        { 0xB77D,  BIFFI_model_code(0xB77D)                          }, // 0x00B7
        { 0xB77E,  BIFFI_model_code(0xB77E)                          }, // 0x00B7

        { 0xB97F,  BD_Sensors_model_code(0xB97F)                     }, // 0x00B9

        { 0xBC7A,  APLISENS_model_code(0xBC7A)                       }, // 0x00BC
        { 0xBC7B,  APLISENS_model_code(0xBC7B)                       }, // 0x00BC

        { 0xBD01,  Badger_Meter_model_code(0xBD01)                   }, // 0x00BD
        { 0xBD02,  Badger_Meter_model_code(0xBD02)                   }, // 0x00BD

        { 0xBE7E,  HIMA_model_code(0xBE7E)                           }, // 0x00BE
        { 0xBE7F,  HIMA_model_code(0xBE7F)                           }, // 0x00BE

        { 0xBF7C,  GP_50_model_code(0xBF7C)                          }, // 0x00BF

        { 0xC0EF,  Kongsberg_Maritime_model_code(0xC0EF)             }, // 0x00C0

        { 0xC101,  ASA_SpA_model_code(0xC101)                        }, // 0x00C1

        { 0xC221,  Hengesbach_model_code(0xC221)                     }, // 0x00C2

        { 0xC374,  Lanlian_Instruments_model_code(0xC374)            }, // 0x00C3

        { 0xC401,  Spectrum_Controls_model_code(0xC401)              }, // 0x00C4

        { 0xC580,  Kajaani_Process_Measurements_model_code(0xC580)   }, // 0x00C5
        { 0xC581,  Kajaani_Process_Measurements_model_code(0xC581)   }, // 0x00C5

        { 0xC680,  FAFNIR_model_code(0xC680)                         }, // 0x00C6

        { 0xC780,  SICK_model_code(0xC780)                           }, // 0x00C7
        { 0xC781,  SICK_model_code(0xC781)                           }, // 0x00C7
        { 0xC782,  SICK_model_code(0xC782)                           }, // 0x00C7
        { 0xC783,  SICK_model_code(0xC783)                           }, // 0x00C7
        { 0xC784,  SICK_model_code(0xC784)                           }, // 0x00C7
        { 0xC785,  SICK_model_code(0xC785)                           }, // 0x00C7

        { 0xCA80,  Spirax_Sarco_SRL_model_code(0xCA80)               }, // 0x00CA
        { 0xCA81,  Spirax_Sarco_SRL_model_code(0xCA81)               }, // 0x00CA
        { 0xCA82,  Spirax_Sarco_SRL_model_code(0xCA82)               }, // 0x00CA
        { 0xCA90,  Spirax_Sarco_SRL_model_code(0xCA90)               }, // 0x00CA

        { 0xCC80,  Tecfluid_SA_model_code(0xCC80)                    }, // 0x00CC
        { 0xCC81,  Tecfluid_SA_model_code(0xCC81)                    }, // 0x00CC

        { 0xCE80,  Roost_model_code(0xCE80)                          }, // 0x00CE

        { 0xCF80,  KOSO_model_code(0xCF80)                           }, // 0x00CF
        { 0xCF81,  KOSO_model_code(0xCF81)                           }, // 0x00CF
        { 0xCF83,  KOSO_model_code(0xCF83)                           }, // 0x00CF

        { 0xD280,  Honeywell_Analytics_model_code(0xD280)            }, // 0x00D2
        { 0xD281,  Honeywell_Analytics_model_code(0xD281)            }, // 0x00D2
        { 0xD282,  Honeywell_Analytics_model_code(0xD282)            }, // 0x00D2
        { 0xD283,  Honeywell_Analytics_model_code(0xD283)            }, // 0x00D2

        { 0xD380,  HEINRICHS_model_code(0xD380)                      }, // 0x00D3
        { 0xD381,  HEINRICHS_model_code(0xD381)                      }, // 0x00D3
        { 0xD382,  HEINRICHS_model_code(0xD382)                      }, // 0x00D3
        { 0xD3EA,  HEINRICHS_model_code(0xD3EA)                      }, // 0x00D3

        { 0xD480,  SIC_model_code(0xD480)                            }, // 0x00D4
        { 0xD481,  SIC_model_code(0xD481)                            }, // 0x00D4
        { 0xD482,  SIC_model_code(0xD482)                            }, // 0x00D4
        { 0xD485,  SIC_model_code(0xD485)                            }, // 0x00D4
        { 0xD486,  SIC_model_code(0xD486)                            }, // 0x00D4
        { 0xD487,  SIC_model_code(0xD487)                            }, // 0x00D4
        { 0xD490,  SIC_model_code(0xD490)                            }, // 0x00D4

        { 0xD580,  HACH_LANGE_model_code(0xD580)                     }, // 0x00D5
        { 0xD581,  HACH_LANGE_model_code(0xD581)                     }, // 0x00D5
        { 0xD582,  HACH_LANGE_model_code(0xD582)                     }, // 0x00D5
        { 0xD583,  HACH_LANGE_model_code(0xD583)                     }, // 0x00D5
        { 0xD584,  HACH_LANGE_model_code(0xD584)                     }, // 0x00D5
        { 0xD586,  HACH_LANGE_model_code(0xD586)                     }, // 0x00D5
        { 0xD587,  HACH_LANGE_model_code(0xD587)                     }, // 0x00D5
        { 0xD588,  HACH_LANGE_model_code(0xD588)                     }, // 0x00D5
        { 0xD589,  HACH_LANGE_model_code(0xD589)                     }, // 0x00D5
        { 0xD58A,  HACH_LANGE_model_code(0xD58A)                     }, // 0x00D5
        { 0xD58B,  HACH_LANGE_model_code(0xD58B)                     }, // 0x00D5
        { 0xD58C,  HACH_LANGE_model_code(0xD58C)                     }, // 0x00D5
        { 0xD58D,  HACH_LANGE_model_code(0xD58D)                     }, // 0x00D5
        { 0xD58E,  HACH_LANGE_model_code(0xD58E)                     }, // 0x00D5
        { 0xD58F,  HACH_LANGE_model_code(0xD58F)                     }, // 0x00D5
        { 0xD590,  HACH_LANGE_model_code(0xD590)                     }, // 0x00D5
        { 0xD591,  HACH_LANGE_model_code(0xD591)                     }, // 0x00D5
        { 0xD5AF,  HACH_LANGE_model_code(0xD5AF)                     }, // 0x00D5
        { 0xD5B0,  HACH_LANGE_model_code(0xD5B0)                     }, // 0x00D5
        { 0xD5B1,  HACH_LANGE_model_code(0xD5B1)                     }, // 0x00D5
        { 0xD5B2,  HACH_LANGE_model_code(0xD5B2)                     }, // 0x00D5
        { 0xD5B3,  HACH_LANGE_model_code(0xD5B3)                     }, // 0x00D5
        { 0xD5B4,  HACH_LANGE_model_code(0xD5B4)                     }, // 0x00D5
        { 0xD5B5,  HACH_LANGE_model_code(0xD5B5)                     }, // 0x00D5

        { 0xD980,  Manometr_Kharkiv_model_code(0xD980)               }, // 0x00D9

        { 0xDB80,  Spectrex_model_code(0xDB80)                       }, // 0x00DB
        { 0xDB81,  Spectrex_model_code(0xDB81)                       }, // 0x00DB
        { 0xDB83,  Spectrex_model_code(0xDB83)                       }, // 0x00DB

        { 0xDE81,  REXA_model_code(0xDE81)                           }, // 0x00DE

        { 0xDF80,  General_Monitors_MSA_model_code(0xDF80)           }, // 0x00DF
        { 0xDF81,  General_Monitors_MSA_model_code(0xDF81)           }, // 0x00DF
        { 0xDF82,  General_Monitors_MSA_model_code(0xDF82)           }, // 0x00DF
        { 0xDF83,  General_Monitors_MSA_model_code(0xDF83)           }, // 0x00DF
        { 0xDF84,  General_Monitors_MSA_model_code(0xDF84)           }, // 0x00DF
        { 0xDF85,  General_Monitors_MSA_model_code(0xDF85)           }, // 0x00DF
        { 0xDF87,  General_Monitors_MSA_model_code(0xDF87)           }, // 0x00DF
        { 0xDF88,  General_Monitors_MSA_model_code(0xDF88)           }, // 0x00DF
        { 0xDF89,  General_Monitors_MSA_model_code(0xDF89)           }, // 0x00DF
        { 0xDF90,  General_Monitors_MSA_model_code(0xDF90)           }, // 0x00DF
        { 0xDF93,  General_Monitors_MSA_model_code(0xDF93)           }, // 0x00DF
        { 0xDF95,  General_Monitors_MSA_model_code(0xDF95)           }, // 0x00DF
        { 0xDF96,  General_Monitors_MSA_model_code(0xDF96)           }, // 0x00DF

        { 0xF980,  FCG_model_code(0xF980)                            }, // 0x00F9
        { 0xF981,  FCG_model_code(0xF981)                            }, // 0x00F9
        { 0xF982,  FCG_model_code(0xF982)                            }, // 0x00F9
        { 0xF983,  FCG_model_code(0xF983)                            }, // 0x00F9
        { 0xF984,  FCG_model_code(0xF984)                            }, // 0x00F9

        { 0xE080,  ExSaf_model_code(0xE080)                          }, // 0x6000
        { 0xE0EC,  ExSaf_model_code(0xE0EC)                          }, // 0x6000
        { 0xE0ED,  ExSaf_model_code(0xE0ED)                          }, // 0x6000

        { 0xE082,  SEOJIN_INSTECH_model_code(0xE082)                 }, // 0x6001
        { 0xE0B4,  SEOJIN_INSTECH_model_code(0xE0B4)                 }, // 0x6001
        { 0xE0B5,  SEOJIN_INSTECH_model_code(0xE0B5)                 }, // 0x6001

        { 0xE083,  TASI_FLOW_model_code(0xE083)                      }, // 0x6002
        { 0xE1B4,  TASI_FLOW_model_code(0xE1B4)                      }, // 0x6002

        { 0xE2A1,  Power_Genex_Ltd_model_code(0xE2A1)                }, // 0x6003

        { 0xE085,  APM_model_code(0xE085)                            }, // 0x6004
        { 0xE086,  APM_model_code(0xE086)                            }, // 0x6004
        { 0xE087,  APM_model_code(0xE087)                            }, // 0x6004

        { 0xE088,  BIFOLD_ORANGE_model_code(0xE088)                  }, // 0x6005
        { 0xE089,  BIFOLD_ORANGE_model_code(0xE089)                  }, // 0x6005
        { 0xE1D2,  BIFOLD_ORANGE_model_code(0xE1D2)                  }, // 0x6005

        { 0xE08A,  BARTEC_model_code(0xE08A)                         }, // 0x6006

        { 0xE08B,  Detcon_model_code(0xE08B)                         }, // 0x6007

        { 0xE08C,  MSA_model_code(0xE08C)                            }, // 0x6008
        { 0xE09F,  MSA_model_code(0xE09F)                            }, // 0x6008
        { 0xE0BC,  MSA_model_code(0xE0BC)                            }, // 0x6008
        { 0xE0F2,  MSA_model_code(0xE0F2)                            }, // 0x6008
        { 0xE0F3,  MSA_model_code(0xE0F3)                            }, // 0x6008
        { 0xE0F6,  MSA_model_code(0xE0F6)                            }, // 0x6008

        { 0xE08D,  METROVAL_model_code(0xE08D)                       }, // 0x6009
        { 0xE0C8,  METROVAL_model_code(0xE0C8)                       }, // 0x6009
        { 0xE0C9,  METROVAL_model_code(0xE0C9)                       }, // 0x6009
        { 0xE196,  METROVAL_model_code(0xE196)                       }, // 0x6009

        { 0xE081,  Etalon_Rus_model_code(0xE081)                     }, // 0x600A

        { 0xE08E,  JOGLER_model_code(0xE08E)                         }, // 0x600B

        { 0xE08F,  KSB_model_code(0xE08F)                            }, // 0x600C

        { 0xE090,  Richter_CT_model_code(0xE090)                     }, // 0x600D

        { 0xE091,  NET_SAFETY_model_code(0xE091)                     }, // 0x600E
        { 0xE09E,  NET_SAFETY_model_code(0xE09E)                     }, // 0x600E
        { 0xE0DF,  NET_SAFETY_model_code(0xE0DF)                     }, // 0x600E
        { 0xE1E3,  NET_SAFETY_model_code(0xE1E3)                     }, // 0x600E

        { 0xE092,  SECanada_model_code(0xE092)                       }, // 0x600F

        { 0xE093,  SUPCON_model_code(0xE093)                         }, // 0x6010
        { 0xE094,  SUPCON_model_code(0xE094)                         }, // 0x6010
        { 0xE191,  SUPCON_model_code(0xE191)                         }, // 0x6010

        { 0xE095,  DKK_TOA_model_code(0xE095)                        }, // 0x6011
        { 0xE096,  DKK_TOA_model_code(0xE096)                        }, // 0x6011
        { 0xE097,  DKK_TOA_model_code(0xE097)                        }, // 0x6011
        { 0xE098,  DKK_TOA_model_code(0xE098)                        }, // 0x6011
        { 0xE099,  DKK_TOA_model_code(0xE099)                        }, // 0x6011
        { 0xE09A,  DKK_TOA_model_code(0xE09A)                        }, // 0x6011
        { 0xE09B,  DKK_TOA_model_code(0xE09B)                        }, // 0x6011

        { 0xE09C,  Dwyer_Instruments_model_code(0xE09C)              }, // 0x6012

        { 0xE09D,  FineTek_model_code(0xE09D)                        }, // 0x6013

        { 0xE0CE,  TopWorx_model_code(0xE0CE)                        }, // 0x6014
        { 0xE0EB,  TopWorx_model_code(0xE0EB)                        }, // 0x6014
        { 0xE194,  TopWorx_model_code(0xE194)                        }, // 0x6014
        { 0xE394,  TopWorx_model_code(0xE394)                        }, // 0x6014
        { 0xE435,  TopWorx_model_code(0xE435)                        }, // 0x6014

        { 0xE0A1,  HOFFER_FLOW_CNTRLS_model_code(0xE0A1)             }, // 0x6015
        { 0xE0F1,  HOFFER_FLOW_CNTRLS_model_code(0xE0F1)             }, // 0x6015

        { 0xE0A2,  DUST_NETWORKS_model_code(0xE0A2)                  }, // 0x6016
        { 0xE0A3,  DUST_NETWORKS_model_code(0xE0A3)                  }, // 0x6016
        { 0xE0A4,  DUST_NETWORKS_model_code(0xE0A4)                  }, // 0x6016
        { 0xE1D3,  DUST_NETWORKS_model_code(0xE1D3)                  }, // 0x6016

        { 0xE0A5,  Forbes_Marshall_model_code(0xE0A5)                }, // 0x6017
        { 0xE0A6,  Forbes_Marshall_model_code(0xE0A6)                }, // 0x6017
        { 0xE0A7,  Forbes_Marshall_model_code(0xE0A7)                }, // 0x6017
        { 0xE0A9,  Forbes_Marshall_model_code(0xE0A9)                }, // 0x6017
        { 0xE2B3,  Forbes_Marshall_model_code(0xE2B3)                }, // 0x6017

        { 0xE0AB,  All_Measures_Ltd_model_code(0xE0AB)               }, // 0x6018

        { 0xE0AC,  Pepperl_Fuchs_model_code(0xE0AC)                  }, // 0x6019

        { 0xE0AD,  CSI_model_code(0xE0AD)                            }, // 0x601A

        { 0xE0AE,  TC_Fluid_Control_model_code(0xE0AE)               }, // 0x601B

        { 0xE0AF,  Rohrback_Cosasco_model_code(0xE0AF)               }, // 0x601C
        { 0xE0B0,  Rohrback_Cosasco_model_code(0xE0B0)               }, // 0x601C
        { 0xE0B1,  Rohrback_Cosasco_model_code(0xE0B1)               }, // 0x601C
        { 0xE0F8,  Rohrback_Cosasco_model_code(0xE0F8)               }, // 0x601C
        { 0xE18E,  Rohrback_Cosasco_model_code(0xE18E)               }, // 0x601C

        { 0xE0B2,  AirSprite_model_code(0xE0B2)                      }, // 0x601D

        { 0xE0B3,  Microcyber_model_code(0xE0B3)                     }, // 0x601E
        { 0xE0CF,  Microcyber_model_code(0xE0CF)                     }, // 0x601E
        { 0xE0D0,  Microcyber_model_code(0xE0D0)                     }, // 0x601E
        { 0xE185,  Microcyber_model_code(0xE185)                     }, // 0x601E
        { 0xE186,  Microcyber_model_code(0xE186)                     }, // 0x601E
        { 0xE187,  Microcyber_model_code(0xE187)                     }, // 0x601E
        { 0xE188,  Microcyber_model_code(0xE188)                     }, // 0x601E
        { 0xE189,  Microcyber_model_code(0xE189)                     }, // 0x601E
        { 0xE2A6,  Microcyber_model_code(0xE2A6)                     }, // 0x601E
        { 0xE40A,  Microcyber_model_code(0xE40A)                     }, // 0x601E

        { 0xE0B6,  TIG_model_code(0xE0B6)                            }, // 0x601F
        { 0xE0B7,  TIG_model_code(0xE0B7)                            }, // 0x601F
        { 0xE0B8,  TIG_model_code(0xE0B8)                            }, // 0x601F
        { 0xE0B9,  TIG_model_code(0xE0B9)                            }, // 0x601F
        { 0xE0BA,  TIG_model_code(0xE0BA)                            }, // 0x601F

        { 0xE0BB,  ifm_prover_Gmbh_model_code(0xE0BB)                }, // 0x6020

        { 0xE0BD,  FLEXIM_model_code(0xE0BD)                         }, // 0x6021

        { 0xE0BE,  TOKYO_KEIKI_model_code(0xE0BE)                    }, // 0x6022
        { 0xE0BF,  TOKYO_KEIKI_model_code(0xE0BF)                    }, // 0x6022
        { 0xE1E9,  TOKYO_KEIKI_model_code(0xE1E9)                    }, // 0x6022

        { 0xE0C0,  SBEM_model_code(0xE0C0)                           }, // 0x6023
        { 0xE0D8,  SBEM_model_code(0xE0D8)                           }, // 0x6023
        { 0xE0D9,  SBEM_model_code(0xE0D9)                           }, // 0x6023
        { 0xE18F,  SBEM_model_code(0xE18F)                           }, // 0x6023

        { 0xE0C1,  SkoFlo_Industries_model_code(0xE0C1)              }, // 0x6024

        { 0xE0C2,  StoneL_Corporation_model_code(0xE0C2)             }, // 0x6025

        { 0xE0C3,  EUREKA_FLOW_model_code(0xE0C3)                    }, // 0x6026
        { 0xE0C5,  EUREKA_FLOW_model_code(0xE0C5)                    }, // 0x6026

        { 0xE0C4,  BEKA_associates_model_code(0xE0C4)                }, // 0x6027

        { 0xE0C6,  CAPSTAR_AUTOMATION_model_code(0xE0C6)             }, // 0x6028

        { 0xE0C7,  Pulsar_model_code(0xE0C7)                         }, // 0x6029
        { 0xE1C5,  Pulsar_model_code(0xE1C5)                         }, // 0x6029

        { 0xE0CB,  ELEMER_model_code(0xE0CB)                         }, // 0x602A
        { 0xE0E8,  ELEMER_model_code(0xE0E8)                         }, // 0x602A
        { 0xE1F1,  ELEMER_model_code(0xE1F1)                         }, // 0x602A
        { 0xE30E,  ELEMER_model_code(0xE30E)                         }, // 0x602A
        { 0xE330,  ELEMER_model_code(0xE330)                         }, // 0x602A
        { 0xE332,  ELEMER_model_code(0xE332)                         }, // 0x602A
        { 0xE47A,  ELEMER_model_code(0xE47A)                         }, // 0x602A
        { 0xE4B8,  ELEMER_model_code(0xE4B8)                         }, // 0x602A
        { 0xE4C2,  ELEMER_model_code(0xE4C2)                         }, // 0x602A
        { 0xE4D5,  ELEMER_model_code(0xE4D5)                         }, // 0x602A

        { 0xE0CD,  Soft_Tech_Group_model_code(0xE0CD)                }, // 0x602B

        { 0xE0D1,  FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code(0xE0D1)}, // 0x602C
        { 0xE0D2,  FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code(0xE0D2)}, // 0x602C
        { 0xE1D8,  FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code(0xE1D8)}, // 0x602C
        { 0xE1D9,  FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code(0xE1D9)}, // 0x602C
        { 0xE1DA,  FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code(0xE1DA)}, // 0x602C
        { 0xE1DB,  FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code(0xE1DB)}, // 0x602C

        { 0xE0D3,  ENOTEC_GmbH_model_code(0xE0D3)                    }, // 0x602D
        { 0xE0D4,  ENOTEC_GmbH_model_code(0xE0D4)                    }, // 0x602D

        { 0xE0D5,  Val_Controls_AS_model_code(0xE0D5)                }, // 0x602E
        { 0xE0D6,  Val_Controls_AS_model_code(0xE0D6)                }, // 0x602E
        { 0xE1CF,  Val_Controls_AS_model_code(0xE1CF)                }, // 0x602E
        { 0xE2FA,  Val_Controls_AS_model_code(0xE2FA)                }, // 0x602E

        { 0xE0D7,  Kurz_Instruments_model_code(0xE0D7)               }, // 0x602F

        { 0xE0DB,  E_Senza_Technologies_model_code(0xE0DB)           }, // 0x6030
        { 0xE195,  E_Senza_Technologies_model_code(0xE195)           }, // 0x6030

        { 0xE0DC,  Crowcon_model_code(0xE0DC)                        }, // 0x6031
        { 0xE0DD,  Crowcon_model_code(0xE0DD)                        }, // 0x6031
        { 0xE0FC,  Crowcon_model_code(0xE0FC)                        }, // 0x6031

        { 0xE0DE,  GEORGIN_model_code(0xE0DE)                        }, // 0x6032
        { 0xE1CB,  GEORGIN_model_code(0xE1CB)                        }, // 0x6032

        { 0xE0E0,  Eldridge_Products_model_code(0xE0E0)              }, // 0x6033

        { 0xE0E1,  DDTOP_model_code(0xE0E1)                          }, // 0x6034
        { 0xE1AC,  DDTOP_model_code(0xE1AC)                          }, // 0x6034
        { 0xE1AD,  DDTOP_model_code(0xE1AD)                          }, // 0x6034
        { 0xE1AE,  DDTOP_model_code(0xE1AE)                          }, // 0x6034
        { 0xE1AF,  DDTOP_model_code(0xE1AF)                          }, // 0x6034
        { 0xE1B0,  DDTOP_model_code(0xE1B0)                          }, // 0x6034
        { 0xE1B7,  DDTOP_model_code(0xE1B7)                          }, // 0x6034
        { 0xE1B8,  DDTOP_model_code(0xE1B8)                          }, // 0x6034
        { 0xE1B9,  DDTOP_model_code(0xE1B9)                          }, // 0x6034
        { 0xE4B6,  DDTOP_model_code(0xE4B6)                          }, // 0x6034

        { 0xE0E2,  KSR_Kuebler_model_code(0xE0E2)                    }, // 0x6035

        { 0xE0E3,  Armstrong_Intl_model_code(0xE0E3)                 }, // 0x6036
        { 0xE0E4,  Armstrong_Intl_model_code(0xE0E4)                 }, // 0x6036
        { 0xE0E5,  Armstrong_Intl_model_code(0xE0E5)                 }, // 0x6036

        { 0xE0E6,  Hitachi_High_Tech_model_code(0xE0E6)              }, // 0x6037
        { 0xE2E7,  Hitachi_High_Tech_model_code(0xE2E7)              }, // 0x6037

        { 0xE0E9,  AMFLOW_model_code(0xE0E9)                         }, // 0x6038
        { 0xE2C6,  AMFLOW_model_code(0xE2C6)                         }, // 0x6038

        { 0xE0EA,  Fluidwell_model_code(0xE0EA)                      }, // 0x6039
        { 0xE2CD,  Fluidwell_model_code(0xE2CD)                      }, // 0x6039

        { 0xE0EE,  Mecon_GmbH_model_code(0xE0EE)                     }, // 0x603A

        { 0xE0EF,  Fox_Thermal_Inst_model_code(0xE0EF)               }, // 0x603B

        { 0xE0F0,  EJMConsulting_model_code(0xE0F0)                  }, // 0x603C

        { 0xE0F4,  Anton_Paar_model_code(0xE0F4)                     }, // 0x603D

        { 0xE0F5,  Dinel_sro_model_code(0xE0F5)                      }, // 0x603E

        { 0xE0F7,  Simtronics_ASA_model_code(0xE0F7)                 }, // 0x603F
        { 0xE1A9,  Simtronics_ASA_model_code(0xE1A9)                 }, // 0x603F
        { 0xE1D5,  Simtronics_ASA_model_code(0xE1D5)                 }, // 0x603F

        { 0xE0F9,  INTEC_model_code(0xE0F9)                          }, // 0x6040
        { 0xE1A4,  INTEC_model_code(0xE1A4)                          }, // 0x6040
        { 0xE1A5,  INTEC_model_code(0xE1A5)                          }, // 0x6040
        { 0xE1A6,  INTEC_model_code(0xE1A6)                          }, // 0x6040

        { 0xE192,  Softing_model_code(0xE192)                        }, // 0x6041

        { 0xE180,  Fire_Sentry_Corp_model_code(0xE180)               }, // 0x6042

        { 0xE181,  FUTURE_INSTRUMENT_model_code(0xE181)              }, // 0x6043
        { 0xE182,  FUTURE_INSTRUMENT_model_code(0xE182)              }, // 0x6043
        { 0xE183,  FUTURE_INSTRUMENT_model_code(0xE183)              }, // 0x6043
        { 0xE1CC,  FUTURE_INSTRUMENT_model_code(0xE1CC)              }, // 0x6043
        { 0xE1CD,  FUTURE_INSTRUMENT_model_code(0xE1CD)              }, // 0x6043

        { 0xE184,  MOTOYAMA_model_code(0xE184)                       }, // 0x6044

        { 0xE1C7,  ifak_system_model_code(0xE1C7)                    }, // 0x6045

        { 0xE18A,  MORGAN_KOREA_model_code(0xE18A)                   }, // 0x6046

        { 0xE18B,  AWIATECH_model_code(0xE18B)                       }, // 0x6047

        { 0xE18C,  Chentronics_model_code(0xE18C)                    }, // 0x6048

        { 0xE18D,  United_Electric_model_code(0xE18D)                }, // 0x6049
        { 0xE308,  United_Electric_model_code(0xE308)                }, // 0x6049

        { 0xE190,  Shanghai_Sinoto_Instrument_Co_Ltd_model_code(0xE190)},   // 0x604A

        { 0xE193,  MATSUSHIMA_model_code(0xE193)                     }, // 0x604B

        { 0xE197,  Westcontrol_AS_model_code(0xE197)                 }, // 0x604C
        { 0xE1BE,  Westcontrol_AS_model_code(0xE1BE)                 }, // 0x604C

        { 0xE198,  Oldham_model_code(0xE198)                         }, // 0x604D
        { 0xE1EA,  Oldham_model_code(0xE1EA)                         }, // 0x604D

        { 0xE19D,  WooriTG_model_code(0xE19D)                        }, // 0x604E

        { 0xE19E,  trianglemicrosol_model_code(0xE19E)               }, // 0x604F

        { 0xE1A0,  Nivis_LLC_model_code(0xE1A0)                      }, // 0x6051
        { 0xE1A1,  Nivis_LLC_model_code(0xE1A1)                      }, // 0x6051

        { 0xE1A2,  SKF_USA_model_code(0xE1A2)                        }, // 0x6052

        { 0xE19C,  ESP_Safety_model_code(0xE19C)                     }, // 0x6053
        { 0xE1AB,  ESP_Safety_model_code(0xE1AB)                     }, // 0x6053
        { 0xE1DD,  ESP_Safety_model_code(0xE1DD)                     }, // 0x6053
        { 0xE44F,  ESP_Safety_model_code(0xE44F)                     }, // 0x6053
        { 0xE450,  ESP_Safety_model_code(0xE450)                     }, // 0x6053

        { 0xE1A3,  HollySys_model_code(0xE1A3)                       }, // 0x6054
        { 0xE1D6,  HollySys_model_code(0xE1D6)                       }, // 0x6054
        { 0xE1D7,  HollySys_model_code(0xE1D7)                       }, // 0x6054

        { 0xE1B3,  GF_Signet_model_code(0xE1B3)                      }, // 0x6055

        { 0xE1AA,  Sierra_Monitor_model_code(0xE1AA)                 }, // 0x6057

        { 0xE1B1,  Schneider_Electric_model_code(0xE1B1)             }, // 0x6058

        { 0xE1B2,  Emerson_model_code(0xE1B2)                        }, // 0x6059

        { 0xE1B6,  ABM_Sensor_Tech_model_code(0xE1B6)                }, // 0x605A

        { 0xE1B5,  RIKEN_KEIKI_model_code(0xE1B5)                    }, // 0x605B
        { 0xE2E8,  RIKEN_KEIKI_model_code(0xE2E8)                    }, // 0x605B
        { 0xE2E9,  RIKEN_KEIKI_model_code(0xE2E9)                    }, // 0x605B
        { 0xE306,  RIKEN_KEIKI_model_code(0xE306)                    }, // 0x605B
        { 0xE481,  RIKEN_KEIKI_model_code(0xE481)                    }, // 0x605B

        { 0xE1BA,  EEPL_India_model_code(0xE1BA)                     }, // 0x605C

        { 0xE1BB,  Thermal_Inst_model_code(0xE1BB)                   }, // 0x605D

        { 0xE1BC,  HYDAC_ELECTRONIC_model_code(0xE1BC)               }, // 0x605E
        { 0xE1E7,  HYDAC_ELECTRONIC_model_code(0xE1E7)               }, // 0x605E

        { 0xE1BD,  GNEUSS_model_code(0xE1BD)                         }, // 0x605F

        { 0xE1BF,  Actuation_Technologies_model_code(0xE1BF)         }, // 0x6060
        { 0xE1C1,  Actuation_Technologies_model_code(0xE1C1)         }, // 0x6060
        { 0xE1C2,  Actuation_Technologies_model_code(0xE1C2)         }, // 0x6060
        { 0xE28B,  Actuation_Technologies_model_code(0xE28B)         }, // 0x6060
        { 0xE28C,  Actuation_Technologies_model_code(0xE28C)         }, // 0x6060

        { 0xE1C0,  Roxar_model_code(0xE1C0)                          }, // 0x6061

        { 0xE1C3,  GASTRON_model_code(0xE1C3)                        }, // 0x6062
        { 0xE1C6,  GASTRON_model_code(0xE1C6)                        }, // 0x6062

        { 0xE1C4,  HAMILTON_Bonaduz_model_code(0xE1C4)               }, // 0x6063
        { 0xE44D,  HAMILTON_Bonaduz_model_code(0xE44D)               }, // 0x6063
        { 0xE44E,  HAMILTON_Bonaduz_model_code(0xE44E)               }, // 0x6063

        { 0xE1C8,  Nobel_Weighing_Systems_model_code(0xE1C8)         }, // 0x6064

        { 0xE1C9,  ECD_model_code(0xE1C9)                            }, // 0x6065

        { 0xE1CE,  Sensor_Electronics_model_code(0xE1CE)             }, // 0x6066

        { 0xE1D0,  Senscient_model_code(0xE1D0)                      }, // 0x6067

        { 0xE1D1,  Shanghai_Kent_Instrument_Co_Ltd_model_code(0xE1D1)}, // 0x6068

        { 0xE1D4,  Scott_Safety_model_code(0xE1D4)                   }, // 0x6069

        { 0xE1DC,  Welkin_Co_Ltd_model_code(0xE1DC)                  }, // 0x606A

        { 0xE1DE,  Teplopribor_model_code(0xE1DE)                    }, // 0x606B

        { 0xE1DF,  Auburn_Systems_model_code(0xE1DF)                 }, // 0x606C

        { 0xE1E0,  Therm_O_Disc_model_code(0xE1E0)                   }, // 0x606D

        { 0xE1E1,  SWISA_INSTRUMENT_model_code(0xE1E1)               }, // 0x606E

        { 0xE1E2,  Sage_Metering_model_code(0xE1E2)                  }, // 0x606F

        { 0xE1E4,  Cooper_Bussmann_model_code(0xE1E4)                }, // 0x6070

        { 0xE1E5,  FSUE_VNIIA_model_code(0xE1E5)                     }, // 0x6071

        { 0xE1E6,  Groveley_Detection_model_code(0xE1E6)             }, // 0x6072

        { 0xE1E8,  Hengyang_No_Photoelectric_Info_Tech_Co_model_code(0xE1E8)},  // 0x6073

        { 0xE1EC,  Imtex_Controls_Ltd_model_code(0xE1EC)             }, // 0x6075

        { 0xE1FD,  AUMA_model_code(0xE1FD)                           }, // 0x607C
        { 0xE2B5,  AUMA_model_code(0xE2B5)                           }, // 0x607C

        { 0xE2ED,  Novus_Automation_model_code(0xE2ED)               }, // 0x607E
        { 0xE313,  Novus_Automation_model_code(0xE313)               }, // 0x607E

        { 0xE0FF,  MISTRAS_Group_model_code(0xE0FF)                  }, // 0x6080

        { 0xE285,  Gefran_SpA_model_code(0xE285)                     }, // 0x6081

        { 0xE286,  Lumasense_Technologies_model_code(0xE286)         }, // 0x6082

        { 0xE28D,  Three_S_model_code(0xE28D)                        }, // 0x6084

        { 0xE29B,  Euromag_International_model_code(0xE29B)          }, // 0x6086

        { 0xE2AD,  Isoil_Hemina_model_code(0xE2AD)                   }, // 0x6088
        { 0xE39C,  Isoil_Hemina_model_code(0xE39C)                   }, // 0x6088

        { 0xE2A2,  Walsn_model_code(0xE2A2)                          }, // 0x6089
        { 0xE2A3,  Walsn_model_code(0xE2A3)                          }, // 0x6089

        { 0xE389,  JUMO_GmbH_Co_KG_model_code(0xE389)                }, // 0x608C
        { 0xE3E3,  JUMO_GmbH_Co_KG_model_code(0xE3E3)                }, // 0x608C
        { 0xE3E4,  JUMO_GmbH_Co_KG_model_code(0xE3E4)                }, // 0x608C

        { 0xE32F,  NEW_COSMOS_ELECTRIC_model_code(0xE32F)            }, // 0x608F

        { 0xE2B8,  Nuoan_model_code(0xE2B8)                          }, // 0x6092
        { 0xE4DE,  Nuoan_model_code(0xE4DE)                          }, // 0x6092

        { 0xE2BA,  SPA_ANALITPRIBOR_model_code(0xE2BA)               }, // 0x6093
        { 0xE2BB,  SPA_ANALITPRIBOR_model_code(0xE2BB)               }, // 0x6093

        { 0xE2BF,  Gasensor_model_code(0xE2BF)                       }, // 0x6099
        { 0xE2C0,  Gasensor_model_code(0xE2C0)                       }, // 0x6099
        { 0xE3A8,  Gasensor_model_code(0xE3A8)                       }, // 0x6099
        { 0xE3A9,  Gasensor_model_code(0xE3A9)                       }, // 0x6099

        { 0xE2C1,  Flowline_model_code(0xE2C1)                       }, // 0x609A

        { 0xE328,  SWAN_Analytical_Instruments_model_code(0xE328)    }, // 0x60B5

        { 0xE32C,  Sensia_model_code(0xE32C)                         }, // 0x60B7
        { 0xE33D,  Sensia_model_code(0xE33D)                         }, // 0x60B7
        { 0xE488,  Sensia_model_code(0xE488)                         }, // 0x60B7

        { 0xE334,  Hoerbiger_model_code(0xE334)                      }, // 0x60B8

        { 0xE333,  Shanghai_Yinuo_model_code(0xE333)                 }, // 0x60B9

        { 0xE35A,  LIMACO_model_code(0xE35A)                         }, // 0x60C1

        { 0xE35D,  Rhosonics_Analytical_BV_model_code(0xE35D)        }, // 0x60C2

        { 0xE38E,  WISE_Sensing_model_code(0xE38E)                   }, // 0x60C3

        { 0xE3E5,  EMIS_model_code(0xE3E5)                           }, // 0x60C5
        { 0xE429,  EMIS_model_code(0xE429)                           }, // 0x60C5
        { 0xE47B,  EMIS_model_code(0xE47B)                           }, // 0x60C5

        { 0xE38D,  VALTEK_SULAMERIC_model_code(0xE38D)               }, // 0x60CA

        { 0xE38F,  Elimko_Ltd_Sti_model_code(0xE38F)                 }, // 0x60CB

        { 0xE4BF,  GODA_Instruments_model_code(0xE4BF)               }, // 0x60CD

        { 0xE3BD,  DVG_Automation_model_code(0xE3BD)                 }, // 0x60D5

        { 0xE37C,  OTIS_Instruments_model_code (0xE37C)              }, // 0x60DB

        { 0xE363,  KANEKO_SANGYO_model_code(0xE363)                  }, // 0x60DC

        { 0xE3A6,  Barksdale_Control_Products_model_code(0xE3A6)     }, // 0x60E7

        { 0xE3B7,  NIVUS_model_code(0xE3B7)                          }, // 0x60EB

        { 0xE3BB,  Guangzhou_Xitai_model_code(0xE3BB)                }, // 0x60EE

        { 0xE405,  VZLJOT_model_code(0xE405)                         }, // 0x60F4
        { 0xE42C,  VZLJOT_model_code(0xE42C)                         }, // 0x60F4
        { 0xE4A9,  VZLJOT_model_code(0xE4A9)                         }, // 0x60F4

        { 0xE3C4,  ROCKSENSOR_model_code(0xE3C4)                     }, // 0x60F5

        { 0xE406,  DELTA_CONTROLS_model_code(0xE406)                 }, // 0x610A

        { 0xE408,  WISE_CONTROL_INC_model_code(0xE408)               }, // 0x610B

        { 0xE41B,  CURTO_model_code(0xE41B)                          }, // 0x610E
        { 0xE465,  CURTO_model_code(0xE465)                          }, // 0x610E
        { 0xE467,  CURTO_model_code(0xE467)                          }, // 0x610E

        { 0xE41A,  Uniphos_Envirotronics_model_code(0xE41A)          }, // 0x6110

        { 0xE457,  Three_Sci_model_code(0xE457)                      }, // 0x611A

        { 0xE468,  SHENYANG_LANSHEN_INSTRUMENT_CO_LTD_model_code(0xE468)},  // 0x611E

        { 0xE469,  SENKO_model_code(0xE469)                          }, // 0x6120

        { 0xE46B,  Toshniwal_Industries_Private_Limited_model_code(0xE46B)},    // 0x6124

        { 0xE46C,  JSC_EPF_SIBNA_model_code(0xE46C)                  }, // 0x6125

        { 0xE48A,  Diyuan_model_code(0xE48A)                         }, // 0x6130

        { 0xE494,  New_Power_model_code(0xE494)                      }, // 0x6131

        { 0xE305,  Hanwei_Electronics_Group_Corporation_model_code(0xE305)},    // 0x6135

        { 0xE49F,  SEG_model_code(0xE49F)                            }, // 0x6134

        { 0xE4A6,  BALDOTA_CONTROL_model_code(0xE4A6)                }, // 0x6137
        { 0xE4A7,  BALDOTA_CONTROL_model_code(0xE4A7)                }, // 0x6137
        { 0xE4A8,  BALDOTA_CONTROL_model_code(0xE4A8)                }, // 0x6137

        { 0xE4D8,  GASDNA_CO_LTD_model_code(0xE4D8)                  }, // 0x6143

        { 0xE537,  BONRAY_model_code(0xE537)                         }  // 0x614F
    }
}

ARRAY OF VARIABLE device_types
{
    LABEL  [device_types];
    HELP   [device_types_help];
    ELEMENTS
    {
        0x0000, Standard_model_code;
        0x0003, Ametek_model_code;
        0x000A, Brooks_Instrument_model_code;
        0x000D, Daniel_Industries_model_code;
        0x000E, Delta_model_code;
        0x0011, Endress_Hauser_model_code;
        0x0012, Fischer_Porter_model_code;
        0x0013, Fisher_Controls_model_code;
        0x0014, SchneiderElectric_Foxboro_model_code;
        0x0015, Fuji_model_code;
        0x0016, Hartmann_Braun_model_code;
        0x0017, Honeywell_model_code;
        0x0019, kayray_model_code;
        0x001A, ABB_model_code;
        0x001D, M_System_Co_model_code;
        0x001F, micro_motion_model_code;
        0x0020, Moore_Industries_model_code;
        0x0021, Moore_Products_model_code;
        0x0025, Ronan_model_code;
        0x0026, rosemount_model_code;
        0x0027, Peek_Measurement_model_code;
        0x0028, Actaris_Neptune_model_code;
        0x002A, Siemens_model_code;
        0x002C, Toshiba_model_code;
        0x002E, rosemount_analytical_model_code;
        0x002F, Valmet_Automation_model_code;
        0x0030, Flowserve_model_code;
        0x0032, Viatran_model_code;
        0x0036, Azbil_Corporation_model_code;
        0x0037, Yokogawa_model_code;
        0x003B, Solartron_Mobrey_model_code;
        0x003D, Princo_model_code;
        0x003E, Smar_model_code;
        0x003F, SchneiderElectric_Eckardt_model_code;
        0x0040, Measurement_Technology_model_code;
        0x0042, Samson_model_code;
        0x0043, Sparling_Instruments_model_code;
        0x0045, Krohne_model_code;
        0x0047, Druck_model_code;
        0x0048, SOR_model_code;
        0x0049, Elcon_Instruments_model_code;
        0x004D, Westlock_Controls_model_code;
        0x004E, Drexelbrook_model_code;
        0x004F, Rosemount_Tank_Radar_model_code;
        0x0050, KTEK_model_code;
        0x0052, Draeger_model_code;
        0x0054, Meridian_model_code;
        0x0055, BTG_model_code;
        0x0056, Magnetrol_model_code;
        0x0057, Neles_Finland_Oy_model_code;
        0x0058, Milltronics_model_code;
        0x005A, Anderson_Instrument_Company_model_code;
        0x005B, Inor_model_code;
        0x005C, Robertshaw_model_code;
        0x005D, PEPPERL_FUCHS_model_code;
        0x005E, Accutech_model_code;
        0x0060, Baumer_model_code;
        0x0061, Knick_model_code;
        0x0062, VEGA_model_code;
        0x0063, MTS_SYS_CORPS_model_code;
        0x0064, Oval_model_code;
        0x0065, GE_Masoneilan_model_code;
        0x0066, Besta_model_code;
        0x0067, Ohmart_model_code;
        0x0068, Harold_Beck_and_Sons_model_code;
        0x006B, Wika_model_code;
        0x006C, Bopp_Reuther_Heinrichs_model_code;
        0x006D, PR_Electronics_model_code;
        0x006E, Rotork_Process_Controls_model_code;
        0x0071, Apparatebau_Hundsbach_model_code;
        0x0072, Dynisco_model_code;
        0x0074, Direct_Measurement_model_code;
        0x0075, Klay_model_code;
        0x0076, CiDRA_CORP_model_code;
        0x0078, Buerkert_Fluid_Control_Systems_model_code;
        0x0079, Venture_Measurement_model_code;
        0x007C, GLI_model_code;
        0x007E, Paper_Machine_Components_model_code;
        0x007F, Labom_model_code;
        0x0080, Danfoss_model_code;
        0x0081, Turbo_model_code;
        0x0082, Tokyo_Keiso_model_code;
        0x0083, SMC_model_code;
        0x0084, Status_Instruments_model_code;
        0x0085, Huakong_model_code;
        0x0086, Autrol_Duon_model_code;
        0x0087, Vortek_Instruments_model_code;
        0x0089, Action_Instruments_model_code;
        0x008C, Magtech_model_code;
        0x008D, Rueger_model_code;
        0x008E, Mettler_Toledo_model_code;
        0x008F, Det_Tronics_model_code;
        0x0090, Thermo_MeasureTech_model_code;
        0x0091, Dezurik_model_code;
        0x0092, Phase_Dynamics_model_code;
        0x0093, Welltech_model_code;
        0x0094, ENRAF_model_code;
        0x0099, Metran_model_code;
        0x009A, Milton_Roy_model_code;
        0x009B, PMV_model_code;
        0x009C, Turck_model_code;
        0x009D, Panametrics_model_code;
        0x009E, Stahl_model_code;
        0x009F, Analytical_Technology_Inc_model_code;
        0x00A0, FINT_model_code;
        0x00A1, BERTHOLD_model_code;
        0x00A2, InterCorr_model_code;
        0x00A5, Sierra_Instruments_model_code;
        0x00A6, Fluid_Components_model_code;
        0x00A8, Meriam_Instrument_model_code;
        0x00A9, Invensys_model_code;
        0x00AB, CRANE_model_code;
        0x00AD, JTec_Associates_model_code;
        0x00AE, Tracerco_model_code;
        0x00AF, AGAR_model_code;
        0x00B0, Phoenix_Contact_model_code;
        0x00B2, Amer_Level_Instr_model_code;
        0x00B3, Hawk_model_code;
        0x00B4, Rotork_YTC_model_code;
        0x00B5, Pyromation_Inc_model_code;
        0x00B6, Satron_Instruments_model_code;
        0x00B7, BIFFI_model_code;
        0x00B9, BD_Sensors_model_code;
        0x00BC, APLISENS_model_code;
        0x00BD, Badger_Meter_model_code;
        0x00BE, HIMA_model_code;
        0x00BF, GP_50_model_code;
        0x00C0, Kongsberg_Maritime_model_code;
        0x00C1, ASA_SpA_model_code;
        0x00C2, Hengesbach_model_code;
        0x00C3, Lanlian_Instruments_model_code;
        0x00C4, Spectrum_Controls_model_code;
        0x00C5, Kajaani_Process_Measurements_model_code;
        0x00C6, FAFNIR_model_code;
        0x00C7, SICK_model_code;
        0x00CA, Spirax_Sarco_SRL_model_code;
        0x00CC, Tecfluid_SA_model_code;
        0x00CE, Roost_model_code;
        0x00CF, KOSO_model_code;
        0x00D2, Honeywell_Analytics_model_code;
        0x00D3, HEINRICHS_model_code;
        0x00D4, SIC_model_code;
        0x00D5, HACH_LANGE_model_code;
        0x00D9, Manometr_Kharkiv_model_code;
        0x00DB, Spectrex_model_code;
        0x00DE, REXA_model_code;
        0x00DF, General_Monitors_MSA_model_code;
        0x00E0, manufacturer_expansion_model_code;
        0x00E1, manufacturer_expansion2_model_code;
        0x00E2, manufacturer_expansion3_model_code;
        0x00E3, manufacturer_expansion4_model_code;
        0x00E4, manufacturer_expansion5_model_code;
        0x00E5, manufacturer_expansion6_model_code;
        0x00F9, FCG_model_code;
        0x6000, ExSaf_model_code;
        0x6001, SEOJIN_INSTECH_model_code;
        0x6002, TASI_FLOW_model_code;
        0x6003, Power_Genex_Ltd_model_code;
        0x6004, APM_model_code;
        0x6005, BIFOLD_ORANGE_model_code;
        0x6006, BARTEC_model_code;
        0x6007, Detcon_model_code;
        0x6008, MSA_model_code;
        0x6009, METROVAL_model_code;
        0x600A, Etalon_Rus_model_code;
        0x600B, JOGLER_model_code;
        0x600C, KSB_model_code;
        0x600D, Richter_CT_model_code;
        0x600E, NET_SAFETY_model_code;
        0x600F, SECanada_model_code;
        0x6010, SUPCON_model_code;
        0x6011, DKK_TOA_model_code;
        0x6012, Dwyer_Instruments_model_code;
        0x6013, FineTek_model_code;
        0x6014, TopWorx_model_code;
        0x6015, HOFFER_FLOW_CNTRLS_model_code;
        0x6016, DUST_NETWORKS_model_code;
        0x6017, Forbes_Marshall_model_code;
        0x6018, All_Measures_Ltd_model_code;
        0x6019, Pepperl_Fuchs_model_code;
        0x601A, CSI_model_code;
        0x601B, TC_Fluid_Control_model_code;
        0x601C, Rohrback_Cosasco_model_code;
        0x601D, AirSprite_model_code;
        0x601E, Microcyber_model_code;
        0x601F, TIG_model_code;
        0x6020, ifm_prover_Gmbh_model_code;
        0x6021, FLEXIM_model_code;
        0x6022, TOKYO_KEIKI_model_code;
        0x6023, SBEM_model_code;
        0x6024, SkoFlo_Industries_model_code;
        0x6025, StoneL_Corporation_model_code;
        0x6026, EUREKA_FLOW_model_code;
        0x6027, BEKA_associates_model_code;
        0x6028, CAPSTAR_AUTOMATION_model_code;
        0x6029, Pulsar_model_code;
        0x602A, ELEMER_model_code;
        0x602B, Soft_Tech_Group_model_code;
        0x602C, FUJIAN_WIDEPLUS_PRECISION_INSTR_model_code;
        0x602D, ENOTEC_GmbH_model_code;
        0x602E, Val_Controls_AS_model_code;
        0x602F, Kurz_Instruments_model_code;
        0x6030, E_Senza_Technologies_model_code;
        0x6031, Crowcon_model_code;
        0x6032, GEORGIN_model_code;
        0x6033, Eldridge_Products_model_code;
        0x6034, DDTOP_model_code;
        0x6035, KSR_Kuebler_model_code;
        0x6036, Armstrong_Intl_model_code;
        0x6037, Hitachi_High_Tech_model_code;
        0x6038, AMFLOW_model_code;
        0x6039, Fluidwell_model_code;
        0x603A, Mecon_GmbH_model_code;
        0x603B, Fox_Thermal_Inst_model_code;
        0x603C, EJMConsulting_model_code;
        0x603D, Anton_Paar_model_code;
        0x603E, Dinel_sro_model_code;
        0x603F, Simtronics_ASA_model_code;
        0x6040, INTEC_model_code;
        0x6041, Softing_model_code;
        0x6042, Fire_Sentry_Corp_model_code;
        0x6043, FUTURE_INSTRUMENT_model_code;
        0x6044, MOTOYAMA_model_code;
        0x6045, ifak_system_model_code;
        0x6046, MORGAN_KOREA_model_code;
        0x6047, AWIATECH_model_code;
        0x6048, Chentronics_model_code;
        0x6049, United_Electric_model_code;
        0x604A, Shanghai_Sinoto_Instrument_Co_Ltd_model_code;
        0x604B, MATSUSHIMA_model_code;
        0x604C, Westcontrol_AS_model_code;
        0x604D, Oldham_model_code;
        0x604E, WooriTG_model_code;
        0x604F, trianglemicrosol_model_code;
        0x6051, Nivis_LLC_model_code;
        0x6052, SKF_USA_model_code;
        0x6053, ESP_Safety_model_code;
        0x6054, HollySys_model_code;
        0x6055, GF_Signet_model_code;
        0x6057, Sierra_Monitor_model_code;
        0x6058, Schneider_Electric_model_code;
        0x6059, Emerson_model_code;
        0x605A, ABM_Sensor_Tech_model_code;
        0x605B, RIKEN_KEIKI_model_code;
        0x605C, EEPL_India_model_code;
        0x605D, Thermal_Inst_model_code;
        0x605E, HYDAC_ELECTRONIC_model_code;
        0x605F, GNEUSS_model_code;
        0x6060, Actuation_Technologies_model_code;
        0x6061, Roxar_model_code;
        0x6062, GASTRON_model_code;
        0x6063, HAMILTON_Bonaduz_model_code;
        0x6064, Nobel_Weighing_Systems_model_code;
        0x6065, ECD_model_code;
        0x6066, Sensor_Electronics_model_code;
        0x6067, Senscient_model_code;
        0x6068, Shanghai_Kent_Instrument_Co_Ltd_model_code;
        0x6069, Scott_Safety_model_code;
        0x606A, Welkin_Co_Ltd_model_code;
        0x606B, Teplopribor_model_code;
        0x606C, Auburn_Systems_model_code;
        0x606D, Therm_O_Disc_model_code;
        0x606E, SWISA_INSTRUMENT_model_code;
        0x606F, Sage_Metering_model_code;
        0x6070, Cooper_Bussmann_model_code;
        0x6071, FSUE_VNIIA_model_code;
        0x6072, Groveley_Detection_model_code;
        0x6073, Hengyang_No_Photoelectric_Info_Tech_Co_model_code;
        0x6075, Imtex_Controls_Ltd_model_code;
        0x607C, AUMA_model_code;
        0x607E, Novus_Automation_model_code;
        0x6080, MISTRAS_Group_model_code;
        0x6081, Gefran_SpA_model_code;
        0x6082, Lumasense_Technologies_model_code;
        0x6084, Three_S_model_code;
        0x6086, Euromag_International_model_code;
        0x6088, Isoil_Hemina_model_code;
        0x6089, Walsn_model_code;
        0x608C, JUMO_GmbH_Co_KG_model_code;
        0x608F, NEW_COSMOS_ELECTRIC_model_code;
        0x6092, Nuoan_model_code;
        0x6093, SPA_ANALITPRIBOR_model_code;
        0x6099, Gasensor_model_code;
        0x609A, Flowline_model_code;
        0x60B5, SWAN_Analytical_Instruments_model_code;
        0x60B7, Sensia_model_code;
        0x60B8, Hoerbiger_model_code;
        0x60B9, Shanghai_Yinuo_model_code;
        0x60C1, LIMACO_model_code;
        0x60C2, Rhosonics_Analytical_BV_model_code;
        0x60C3, WISE_Sensing_model_code;
        0x60C5, EMIS_model_code;
        0x60CA, VALTEK_SULAMERIC_model_code;
        0x60CB, Elimko_Ltd_Sti_model_code;
        0x60CD, GODA_Instruments_model_code;
        0x60D5, DVG_Automation_model_code;
        0x60DB, OTIS_Instruments_model_code ;
        0x60DC, KANEKO_SANGYO_model_code;
        0x60E7, Barksdale_Control_Products_model_code;
        0x60EB, NIVUS_model_code;
        0x60EE, Guangzhou_Xitai_model_code;
        0x60F4, VZLJOT_model_code;
        0x60F5, ROCKSENSOR_model_code;
        0x610A, DELTA_CONTROLS_model_code;
        0x610B, WISE_CONTROL_INC_model_code;
        0x610E, CURTO_model_code;
        0x6110, Uniphos_Envirotronics_model_code;
        0x611A, Three_Sci_model_code;
        0x611E, SHENYANG_LANSHEN_INSTRUMENT_CO_LTD_model_code;
        0x6120, SENKO_model_code;
        0x6124, Toshniwal_Industries_Private_Limited_model_code;
        0x6125, JSC_EPF_SIBNA_model_code;
        0x6130, Diyuan_model_code;
        0x6131, New_Power_model_code;
        0x6135, Hanwei_Electronics_Group_Corporation_model_code;
        0x6134, SEG_model_code;
        0x6137, BALDOTA_CONTROL_model_code;
        0x6143, GASDNA_CO_LTD_model_code;
        0x614F, BONRAY_model_code;
    }
}

/**********************************************************************
 * Table 8. Company (Manufacturer) Identification Codes
 * Note- Manufacturer is an improper word to use, since this set of
 * enumerations may be used in variables that describe the OEM company.
 */
VARIABLE company_identification_code
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
        { 0x0000, [Standard] },
        { 0x0001, [Acromag] },
        { 0x0002, [Allen_Bradley] },
        { 0x0003, [Ametek] },
        { 0x0004, [Analog_Devices] },
        { 0x0005, [ABB] },
        { 0x0006, [Beckman] },
        { 0x0007, [Bell_Microsensor] },
        { 0x0008, [Bourns] },
        { 0x0009, [Bristol_Babcock] },
        { 0x000A, [Brooks_Instrument] },
        { 0x000B, [Chessell] },
        { 0x000C, [Combustion_Engineering] },
        { 0x000D, [Daniel_Industries] },
        { 0x000E, [Delta] },
        { 0x000F, [Dieterich_Standard] },
        { 0x0010, [Dohrmann] },
        { 0x0011, [Endress_Hauser] },
        { 0x0012, [ABB] },
        { 0x0013, "Fisher Controls Intl LLC" },
        { 0x0014, [SchneiderElectric_Foxboro] },
        { 0x0015, [Fuji] },
        { 0x0016, [ABB] },
        { 0x0017, [Honeywell] },
        { 0x0018, [ITT_Barton] },
        { 0x0019, [Thermo_Measure_Tech] },
        { 0x001A, [ABB] },
        { 0x001B, [Leeds_and_Northrup] },
        { 0x001C, [Leslie] },
        { 0x001D, [M_System_Co] },
        { 0x001E, [Measurex] },
        { 0x001F, [Micro_Motion] },
        { 0x0020, [Moore_Industries] },
        { 0x0021, [Prime_Measurement_Products] },
        { 0x0022, [Ohkura_Electric] },
        { 0x0023, [Paine] },
        { 0x0024, [Rochester_Instrument_Systems] },
        { 0x0025, [Ronan] },
        { 0x0026, [Rosemount] },
        { 0x0027, [Peek_Measurement] },
        { 0x0028, [Actaris_Neptune] },
        { 0x0029, [Sensall] },
        { 0x002A, [Siemens] },
        { 0x002B, [Weed] },
        { 0x002C, [Toshiba] },
        { 0x002D, [Transmation] },
        { 0x002E, [Rosemount_Analytical] },
        { 0x002F, [Metso_Flow_Control_Inc] },
        { 0x0030, [Flowserve] },
        { 0x0031, [Varec] },
        { 0x0032, [Viatran] },
        { 0x0033, [Delta_Weed] },
        { 0x0034, [Westinghouse] },
        { 0x0035, [Xomox] },
        { 0x0036, [Azbil_Corporation] },
        { 0x0037, [Yokogawa] },
        { 0x0038, [Nuovo_Pignone] },
        { 0x0039, [Promac] },
        { 0x003A, [Exac_Corporation] },
        { 0x003B, [Mobrey] },
        { 0x003C, [Acrom_Control_System] },
        { 0x003D, [Princo] },
        { 0x003E, [Smar] },
        { 0x003F, [SchneiderElectric_Eckardt] },
        { 0x0040, [Measurement_Technology] },
        { 0x0041, [Applied_System_Technologies] },
        { 0x0042, [Samson] },
        { 0x0043, [Sparling_Instruments] },
        { 0x0044, [Fireye] },
        { 0x0045, [Krohne] },
        { 0x0046, [Betz_Equipment] },
        { 0x0047, [Druck] },
        { 0x0048, [SOR] },
        { 0x0049, [Elcon_Instruments] },
        { 0x004A, [Spirax_Sarco] },
        { 0x004B, [Termiflex] },
        { 0x004C, [VAF_Instruments] },
        { 0x004D, [Westlock_Controls] },
        { 0x004E, [AMETEK_Drexelbrook] },
        { 0x004F, [Rosemount_Tank_Radar] },
        { 0x0050, [K_TEK] },
        { 0x0051, [Sensidyne] },
        { 0x0052, [Draeger] },
        { 0x0053, [Raytek] },
        { 0x0054, [Siemens_Milltronics_PI] },
        { 0x0055, [BTG] },
        { 0x0056, [Magnetrol] },
        { 0x0057, "Neles Finland Oy" },
        { 0x0058, [Siemens_Milltronics_PI] },
        { 0x0059, [HELIOS] },
        { 0x005A, [Anderson_Instrument_Company] },
        { 0x005B, [INOR] },
        { 0x005C, [ROBERTSHAW] },
        { 0x005D, [Pepperl_Fuchs] },
        { 0x005E, [ACCUTECH] },
        { 0x005F, [Flow_Measurement] },
        { 0x0060, [Baumer] },
        { 0x0061, [Knick] },
        { 0x0062, [VEGA_Grieshaber_KG] },
        { 0x0063, [MTS_SYS_CORPS_SENSORS] },
        { 0x0064, [Oval] },
        { 0x0065, [GE_Masoneilan] },
        { 0x0066, [Besta] },
        { 0x0067, [Ohmart] },
        { 0x0068, [Harold_Beck_and_Sons] },
        { 0x0069, [rittmeyer_instrumentation] },
        { 0x006A, [rossel_messtechnik] },
        { 0x006B, [WIKA] },
        { 0x006C, [Bopp_and_Reuther_Messtechnik] },
        { 0x006D, [pr_eletronics] },
        { 0x006E, [Rotork_Process_Controls] },
        { 0x006F, [valcom] },
        { 0x0070, [us_electric_motors] },
        { 0x0071, [apparatebau_hundsbach] },
        { 0x0072, [Dynisco] },
        { 0x0074, [Direct_Mesaurement] },
        { 0x0075, [Klay] },
        { 0x0076, [CiDRA_CORP] },
        { 0x0077, [Mmg_Automatiky] },
        { 0x0078, [Buerkert] },
        { 0x0079, [Venture_Measurement] },
        { 0x007A, [Pondus] },
        { 0x007B, [Zap] },
        { 0x007C, [GLI] },
        { 0x007D, [FRPT] },
        { 0x007E, [Paper_Machine_Components] },
        { 0x007F, [Labom] },
        { 0x0080, [Danfoss] },
        { 0x0081, [Turbo] },
        { 0x0082, [Tokyo_Keiso] },
        { 0x0083, [SMC] },
        { 0x0084, [Status_Instruments] },
        { 0x0085, [Huakong] },
        { 0x0086, [Autrol_Duon] },
        { 0x0087, [Vortek_Instruments_LLC] },
        { 0x0088, [AG_Crosby] },
        { 0x0089, [Action_Instruments] },
        { 0x008A, [Keystone_Controls] },
        { 0x008B, [Thermo_Electric_Company] },
        { 0x008C, [ISE_Magtech] },
        { 0x008D, [Rueger] },
        { 0x008E, [Mettler_Toledo] },
        { 0x008F, [Det_Tronics] },
        { 0x0090, [Thermo_Measure_Tech] },
        { 0x0091, [DeZURIK] },
        { 0x0092, [Phase_Dynamics] },
        { 0x0093, [WELLTECH_SHANGHAI] },
        { 0x0094, [ENRAF] },
        { 0x0095, [Four_Tech_ASA] },
        { 0x0096, [Brandt_Instrument] },
        { 0x0097, [Nivelco] },
        { 0x0098, [Camille_Bauer] },
        { 0x0099, [Metran] },
        { 0x009A, [Milton_Roy] },
        { 0x009B, [PMV] },
        { 0x009C, [Turck] },
        { 0x009D, [Panametrics] },
        { 0x009E, [Stahl] },
        { 0x009F, [Analytical_Technology_Inc] },
        { 0x00A0, [FINT] },
        { 0x00A1, [BERTHOLD] },
        { 0x00A2, [InterCorr] },
        { 0x00A3, [China_BRICONTE_Co_Ltd] },
        { 0x00A4, [Electron_Machine] },
        { 0x00A5, [Sierra_Instruments] },
        { 0x00A6, [Fluid_Components] },
        { 0x00A7, [Solid_AT] },
        { 0x00A8, [Meriam_Instrument] },
        { 0x00A9, [Invensys] },
        { 0x00AA, [S_Products] },
        { 0x00AB, [CRANE] },
        { 0x00AC, [Micro_Matic_Instrument] },
        { 0x00AD, [JTec_Associates] },
        { 0x00AE, [TRACERCO] },
        { 0x00AF, [AGAR] },
        { 0x00B0, [Phoenix_Contact] },
        { 0x00B1, [Andean_Instruments] },
        { 0x00B2, [Amer_Level_Instr] },
        { 0x00B3, [Hawk] },
        { 0x00B4, [Rotork_YTC] },
        { 0x00B5, [Pyromation_Inc] },
        { 0x00B6, [Satron_Instruments] },
        { 0x00B7, [BIFFI] },
        { 0x00B8, [SAIC] },
        { 0x00B9, [BD_Sensors] },
        { 0x00BA, [Andean_Instruments] },
        { 0x00BB, [Kemotron] },
        { 0x00BC, [APLISENS] },
        { 0x00BD, [Badger_Meter] },
        { 0x00BE, [HIMA] },
        { 0x00BF, [GP_50] },
        { 0x00C0, [Kongsberg_Maritime] },
        { 0x00C1, [ASA] },
        { 0x00C2, [Hengesbach] },
        { 0x00C3, [Lanlian_Instruments] },
        { 0x00C4, [Spectrum_Controls] },
        { 0x00C5, [Kajaani_Process_Measurements] },
        { 0x00C6, [FAFNIR] },
        { 0x00C7, [SICK_Engineering_GmbH] },
        { 0x00C8, [JSP_Nova_Paka] },
        { 0x00C9, [MESACON] },
        { 0x00CA, [Spirax_Sarco_SRL] },
        { 0x00CB, [L_J_TECHNOLOGIES] },
        { 0x00CC, [Tecfluid_SA] },
        { 0x00CD, [Sailsors_Instruments] },
        { 0x00CE, [Roost] },
        { 0x00CF, [KOSO] },
        { 0x00D0, [MJK] },
        { 0x00D1, [GE_Energy] },
        { 0x00D2, [Honeywell_Analytics] },
        { 0x00D3, [HEINRICHS] },
        { 0x00D4, [SIC] },
        { 0x00D5, [HACH_LANGE] },
        { 0x00D6, [Exalon_Instruments] },
        { 0x00D7, [FAURE_HERMAN] },
        { 0x00D8, [STI] },
        { 0x00D9, [Manometr_Kharkiv] },
        { 0x00DA, [Dalian_Instruments] },
        { 0x00DB, [Spectrex] },
        { 0x00DC, [SIPAI_Instruments] },
        { 0x00DD, [Advanced_Flow] },
        { 0x00DE, "REXA" },
        { 0x00DF, [General_Monitors_MSA] },
        { 0x00E0, [Manufacturer_Expansion] },
        { 0x00E1, [Manufacturer_Expansion] },
        { 0x00E2, [Manufacturer_Expansion] },
        { 0x00E3, [Manufacturer_Expansion] },
        { 0x00E4, [Manufacturer_Expansion] },
        { 0x00E5, [Manufacturer_Expansion] },
        { 0x00F9, [FieldComm_Group] },
        { 0x00FA, [not_used] },
        { 0x00FB, [none] },
        { 0x00FC, [unknown] },
        { 0x00FD, [special] },
        { 0x6000, [ExSaf] },
        { 0x6001, [SEOJIN_INSTECH] },
        { 0x6002, [TASI_FLOW] },
        { 0x6003, [Power_Genex_Ltd] },
        { 0x6004, [APM] },
        { 0x6005, [BIFOLD_ORANGE] },
        { 0x6006, [BARTEC] },
        { 0x6007, [Detcon] },
        { 0x6008, [MSA] },
        { 0x6009, [METROVAL] },
        { 0x600A, [Etalon_Rus] },
        { 0x600B, [JOGLER] },
        { 0x600C, [KSB] },
        { 0x600D, [Richter_CT] },
        { 0x600E, [NET_SAFETY] },
        { 0x600F, [SECanada] },
        { 0x6010, [SUPCON] },
        { 0x6011, [DKK_TOA] },
        { 0x6012, [Dwyer_Instruments] },
        { 0x6013, [FineTek] },
        { 0x6014, [TopWorx] },
        { 0x6015, [HOFFER_FLOW_CNTRLS] },
        { 0x6016, [DUST_NETWORKS] },
        { 0x6017, [Forbes_Marshall] },
        { 0x6018, [All_Measures_Ltd] },
        { 0x6019, [Pepperl_Fuchs] },
        { 0x601A, [CSI] },
        { 0x601B, [TC_Fluid_Control] },
        { 0x601C, [Rohrback_Cosasco] },
        { 0x601D, [AirSprite] },
        { 0x601E, [Microcyber] },
        { 0x601F, [TIG] },
        { 0x6020, [ifm_prover_Gmbh] },
        { 0x6021, [FLEXIM] },
        { 0x6022, [TOKYO_KEIKI] },
        { 0x6023, [SBEM] },
        { 0x6024, [SkoFlo_Industries] },
        { 0x6025, [StoneL_Corporation] },
        { 0x6026, [EUREKA_FLOW] },
        { 0x6027, [BEKA_associates] },
        { 0x6028, [CAPSTAR_AUTOMATION] },
        { 0x6029, [Pulsar] },
        { 0x602A, [ELEMER] },
        { 0x602B, [Soft_Tech_Group] },
        { 0x602C, [FUJIAN_WIDEPLUS_PRECISION_INSTR] },
        { 0x602D, [ENOTEC_GmbH] },
        { 0x602E, [Val_Controls_AS] },
        { 0x602F, [Kurz_Instruments] },
        { 0x6030, [E_Senza_Technologies] },
        { 0x6031, [Crowcon] },
        { 0x6032, [GEORGIN] },
        { 0x6033, [Eldridge_Products] },
        { 0x6034, [DDTOP] },
        { 0x6035, [KSR_Kuebler] },
        { 0x6036, [Armstrong_Intl] },
        { 0x6037, [Hitachi_High_Tech] },
        { 0x6038, [AMFLOW] },
        { 0x6039, [Fluidwell] },
        { 0x603A, [Mecon_GmbH] },
        { 0x603B, [Fox_Thermal_Inst] },
        { 0x603C, [EJMConsulting] },
        { 0x603D, [Anton_Paar] },
        { 0x603E, [Dinel_sro] },
        { 0x603F, [Simtronics_ASA] },
        { 0x6040, [INTEC] },
        { 0x6041, [Softing] },
        { 0x6042, [Fire_Sentry_Corp] },
        { 0x6043, [FUTURE_INSTRUMENT] },
        { 0x6044, [MOTOYAMA] },
        { 0x6045, [ifak_system] },
        { 0x6046, [MORGAN_KOREA] },
        { 0x6047, [AWIATECH] },
        { 0x6048, [Chentronics] },
        { 0x6049, [United_Electric] },
        { 0x604A, [Shanghai_Sinoto_Instrument_Co_Ltd] },
        { 0x604B, [MATSUSHIMA] },
        { 0x604C, [Westcontrol_AS] },
        { 0x604D, [Oldham] },
        { 0x604E, [WooriTG] },
        { 0x604F, [trianglemicrosol] },
        { 0x6050, [Nivo_Controls_Pvt_Ltd] },
        { 0x6051, [Nivis_LLC] },
        { 0x6052, [SKF_USA] },
        { 0x6053, [ESP_Safety] },
        { 0x6054, [HollySys] },
        { 0x6055, [GF_Signet] },
        { 0x6056, [Korosi_Specindo] },
        { 0x6057, [Sierra_Monitor] },
        { 0x6058, [Schneider_Electric] },
        { 0x6059, [Emerson] },
        { 0x605A, [ABM_Sensor_Tech] },
        { 0x605B, [RIKEN_KEIKI] },
        { 0x605C, [EEPL_India] },
        { 0x605D, [Thermal_Inst] },
        { 0x605E, [HYDAC_ELECTRONIC] },
        { 0x605F, [GNEUSS] },
        { 0x6060, [Actuation_Technologies] },
        { 0x6061, [Roxar] },
        { 0x6062, [GASTRON] },
        { 0x6063, [HAMILTON_Bonaduz] },
        { 0x6064, [Nobel_Weighing_Systems] },
        { 0x6065, [ECD] },
        { 0x6066, [Sensor_Electronics] },
        { 0x6067, [Senscient] },
        { 0x6068, [Shanghai_Kent_Instrument_Co_Ltd] },
        { 0x6069, [Scott_Safety] },
        { 0x606A, [Welkin_Co_Ltd] },
        { 0x606B, [Teplopribor] },
        { 0x606C, [Auburn_Systems] },
        { 0x606D, [Therm_O_Disc] },
        { 0x606E, [SWISA_INSTRUMENT] },
        { 0x606F, [Sage_Metering] },
        { 0x6070, [Cooper_Bussmann] },
        { 0x6071, [FSUE_VNIIA] },
        { 0x6072, [Groveley_Detection] },
        { 0x6073, [Hengyang_No_Photoelectric_Info_Tech_Co] },
        { 0x6074, [Universal_Flow] },
        { 0x6075, [Imtex_Controls_Ltd] },
        { 0x607C, [AUMA] },
        { 0x607E, [Novus_Automation] },
        { 0x6080, [MISTRAS_Group] },
        { 0x6081, [Gefran_SpA] },
        { 0x6082, [Lumasense_Technologies] },
        { 0x6084, [Three_S] },
        { 0x6086, [Euromag_International] },
        { 0x6088, [Isoil_Hemina] },
        { 0x6089, [Walsn] },
        { 0x608C, [JUMO_GmbH_Co_KG] },
        { 0x608F, [NEW_COSMOS_ELECTRIC] },
        { 0x6092, "Nuoan" },
        { 0x6093, [SPA_ANALITPRIBOR] },
        { 0x6099, [Gasensor] },
        { 0x609A, [Flowline] },
        { 0x60B5, [SWAN_Analytical_Instruments] },
        { 0x60B7, "Sensia" },
        { 0x60B8, [Hoerbiger] },
        { 0x60B9, [Shanghai_Yinuo] },
        { 0x60C1, [LIMACO] },
        { 0x60C2, [Rhosonics_Analytical_BV] },
        { 0x60C3, [WISE_Sensing] },
        { 0x60C5, [EMIS] },
        { 0x60CA, [VALTEK_SULAMERIC] },
        { 0x60CB, [Elimko_Ltd_Sti] },
        { 0x60CD, "GODA Instruments" },
        { 0x60D5, [DVG_Automation] },
        { 0x60DB, [OTIS_Instruments] },
        { 0x60DC, [KANEKO_SANGYO] },
        { 0x60E7, [Barksdale_Control_Products] },
        { 0x60EB, [NIVUS] },
        { 0x60EE, [Guangzhou_Xitai] },
        { 0x60F4, [VZLJOT] },
        { 0x60F5, [ROCKSENSOR] },
        { 0x610A, [DELTA_CONTROLS] },
        { 0x610B, [WISE_CONTROL_INC] },
        { 0x610E, [CURTO] },
        { 0x6110, [Uniphos_Envirotronics] },
        { 0x611A, "3-Sci" },
        { 0x611E, [SHENYANG_LANSHEN_INSTRUMENT_CO_LTD] },
        { 0x6120, "SENKO" },
        { 0x6124, [Toshniwal_Industries_Private_Limited] },
        { 0x6125, [JSC_EPF_SIBNA] },
        { 0x6130, "Diyuan" },
        { 0x6131, "New Power" },
        { 0x6135, "Hanwei Electronics Group Corporation" },
        { 0x6134, "SEG" },
        { 0x6137, "BALDOTA CONTROL" }
        { 0x6143, "GASDNA CO., LTD" }
        { 0x614F, "BONRAY" }
    }
}

/**********************************************************************
 * Table 2. Unit Codes
 */
VARIABLE units_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        /* Design Note: Enumerations 240-249 are reserved for manufacturer
           specific definitions. Therefore, Host applications should not
           display meaningful information for these enumerations unless
           the Host has the Device Description for that specific Field
           Device. Some Host applications have used a string of the
           character '?' for this application (e.g. Pressure:10.5???? */
        /* Design Note: Any enumerations in the range from 250-255
           are 'Reserved' for a single definition if not defined. */
        /* Design Note: Any enumeration not covered by another "Design
           Note" is 'Undefined'. */

/* Temperature */
//Remove new dictionary references from legacy binary format
        { 32,  degreesC_temp,          [degrees_celsius_help] },
//Remove new dictionary references from legacy binary format
        { 33,  degreesF_temp,          [degrees_fahrenheit_help] },
//Remove new dictionary references from legacy binary format
        { 34,  degreesR_temp,          [degrees_rankine_help] },
//Remove new dictionary references from legacy binary format
        { 35,  kelvin_temp,            [degrees_kelvin_help] },

/*
 * Pressure
 *
 *  NOTE: Pressure is ambiguous with respect to absolute or gage
 */
        { 1,   [InH2O],             [inches_of_water_68_degrees_F_help] },
        { 2,   [InHg],              [inches_of_mercury_0_degrees_C_help] },
        { 3,   [FtH2O],             [feet_of_water_68_degrees_F_help] },
        { 4,   [mmH2O],             [millimeters_of_water_68_degrees_F_help] },
        { 5,   [mmHg],              [millimeters_of_mercury_0_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 6,   psi_temp,               [pounds_per_square_inch_help] },
        { 7,   [bar],               [bars_help] },
        { 8,   [mbar],              [millibars_help] },
//Remove new dictionary references from legacy binary format
        { 9,   grams_cm2_temp,         [grams_per_square_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 10,  kgrams_cm2_temp,        [kilograms_per_square_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 11,  pascals_temp,           [pascals_help] },
//Remove new dictionary references from legacy binary format
        { 12,  kpascals_temp,          [kilopascals_help] },
//Remove new dictionary references from legacy binary format
        { 13,  Torr_temp,              [torr_help] },
//Remove new dictionary references from legacy binary format
        { 14,  ATM_temp,               [atmospheres_help] },
//Remove new dictionary references from legacy binary format
        { 145, inH2O_60F_temp,         [inches_of_water_60_degrees_F_help] },
//Remove new dictionary references from legacy binary format
        { 237, Mpascals_temp,          [megapascals_help] },
//Remove new dictionary references from legacy binary format
        { 238, inH2O_4C_temp,          [inches_of_water_4_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 239, mmH2O_4C_temp,          [millimeters_of_water_4_degrees_C_help] },

/* Volumetric Flow */
//Remove new dictionary references from legacy binary format
        { 15,  ft3_min_temp,           [cubic_feet_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 16,  [gal_min],           Gallons_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 17,  L_min_temp,             [liters_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 18,  galUK_min_temp,         [imperial_gallons_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 19,  m3_hr_temp,             [cubic_meter_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 22,  [gal_s],             Gallons_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 23,  Mgal_day_temp,          Million_gallons_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 24,  L_s_temp,               [liters_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 25,  Mltr_day_temp,          [million_liters_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 26,  ft3_s_temp,             [cubic_feet_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 27,  ft3_day_temp,           [cubic_feet_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 28,  m3_s_temp,              cubic_meter_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 29,  m3_day_temp,            cubic_meter_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 30,  galUK_hr_temp,          [imperial_gallons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 31,  galUK_day_temp,         [imperial_gallons_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 121, Nm3_hr_temp,            Normal_cubic_meters_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 122, NL_hr_temp,             Normal_liters_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 123, SCF_min_temp,           Standard_cubic_feet_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 130, ft3_hr_temp,            [cubic_feet_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 131, m3_min_temp,            [cubic_meters_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 132, [bbl_s],             Oil_Barrels_per_second_help_temp },        /* 1 barrel equals 42 U.S. gallons */
//Remove new dictionary references from legacy binary format
        { 133, [bbl_min],           Oil_Barrels_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 134, [bbl_hr],            Oil_Barrels_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 135, [bbl_day],           Oil_Barrels_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 136, [gal_hr],            Gallons_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 137, galUK_s_temp,           [imperial_gallons_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 138, L_hr_temp,              [liters_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 235, gal_day_temp,           Gallons_per_day_help_temp },

/* Velocity */
        { 20,  [ft_s],              [feet_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 21,  m_s_temp,               [meters_per_second_help] },
        { 114, [in_s],              [inches_per_second_help] },
        { 115, [in_min],            [inches_per_minute_help] },
        { 116, [ft_min],            [feet_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 120, m_hr_temp,              [meters_per_hour_help] },

/* Volume */
        { 40,  [gal],               [gallons_help] },
//Remove new dictionary references from legacy binary format
        { 41,  Liter_temp,             [liters_help] },
//Remove new dictionary references from legacy binary format
        { 42,  galUK_temp,             [imperial_gallons_help] },
//Remove new dictionary references from legacy binary format
        { 43,  m3_temp,                [cubic_meters_help] },
//Remove new dictionary references from legacy binary format
        { 46,  [bbl],               Barrel_help_temp },        /* 1 barrel equals 42 U.S. gallons */
//Remove new dictionary references from legacy binary format
        { 110, bushels_temp,           [bushels_help] },
//Remove new dictionary references from legacy binary format
        { 111, Yd3_temp,               [cubic_yards_help] },
//Remove new dictionary references from legacy binary format
        { 112, ft3_temp,               [cubic_feet_help] },
//Remove new dictionary references from legacy binary format
        { 113, in3_temp,               [cubic_inches_help] },
//Remove new dictionary references from legacy binary format
        { 124, bblUS_temp,             Liquid_barrel_help_temp },       /* 1 liquid barrel equals 31.5 U.S. gallons. Note that */
//Remove new dictionary references from legacy binary format
        { 166, Nm3_temp,               Normal_cubic_meters_help_temp },
//Remove new dictionary references from legacy binary format
        { 167, NL_temp,                Normal_liters_help_temp },
//Remove new dictionary references from legacy binary format
        { 168, SCF_temp,               Standard_cubic_feet_help_temp },
//Remove new dictionary references from legacy binary format
        { 236, hL_temp,                [hectoliters_help] },

/* Length */
        { 44,  [ft],                [feet_help] },
//Remove new dictionary references from legacy binary format
        { 45,  meters_temp,            [meters_help] },
        { 47,  [in],                [inches_help] },
//Remove new dictionary references from legacy binary format
        { 48,  centimeters_temp,       [centimeters_help] },
//Remove new dictionary references from legacy binary format
        { 49,  millimeters_temp,       [millimeters_help] },
//Remove new dictionary references from legacy binary format
        { 151, ft16_temp,              feet_in_sixteenths_help_temp },
         /* APPHELP "There must be 6 digits to the left of the decimal"
                " point of the associated numeric value. The format"
                " of the most significant two of these digits"
                " indicate the number of 'feet'. The adjacent two"
                " lesser significant digits are indicate the number"
                " of additional sixteenths (i.e. 16 sixteenths = 1"
                " inch). If the numeric value is in the floating"
                " point format, any digits to the right of the"
                " decimal point are discarded by the host. " */
/* Time */
//Remove new dictionary references from legacy binary format
        { 50,  minutes_temp,           [minutes_help] },
//Remove new dictionary references from legacy binary format
        { 51,  seconds_temp,           [seconds_help] },
//Remove new dictionary references from legacy binary format
        { 52,  hours_temp,             [hours_help] },
//Remove new dictionary references from legacy binary format
        { 53,  days_temp,              [days_help] },

/* Mass */
//Remove new dictionary references from legacy binary format
        { 60,  grams_temp,             [grams_help] },
//Remove new dictionary references from legacy binary format
        { 61,  kgrams_temp,            [kilograms_help] },
//Remove new dictionary references from legacy binary format
        { 62,  metTon_temp,            [metric_tons_help] },
        { 63,  [lb],                [pounds_help] },
//Remove new dictionary references from legacy binary format
        { 64,  ton_temp,               [short_tons_help] },
//Remove new dictionary references from legacy binary format
        { 65,  tonUK_temp,             [long_tons_help] },
//Remove new dictionary references from legacy binary format
        { 125, oz_temp,                ounces_help_temp },

/* Mass Flow */
//Remove new dictionary references from legacy binary format
        { 70,  grams_s_temp,           [grams_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 71,  grams_min_temp,         [grams_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 72,  grams_hr_temp,          [grams_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 73,  kgrams_s_temp,          [kilograms_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 74,  kgrams_min_temp,        [kilograms_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 75,  kgrams_hr_temp,         [kilograms_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 76,  kgrams_day_temp,        [kilograms_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 77,  metTon_min_temp,        [metric_tons_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 78,  metTon_hr_temp,         [metric_tons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 79,  metTon_day_temp,        [metric_tons_per_day_help] },
        { 80,  [lb_s],              [pounds_per_second_help] },
        { 81,  [lb_min],            [pounds_per_minute_help] },
        { 82,  [lb_hr],             [pounds_per_hour_help] },
        { 83,  [lb_day],            [pounds_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 84,  ton_min_temp,           [short_tons_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 85,  ton_hr_temp,            [short_tons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 86,  ton_day_temp,           [short_tons_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 87,  tonUK_hr_temp,          [long_tons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 88,  tonUK_day_temp,         [long_tons_per_day_help] },

/* Mass per Volume */
//Remove new dictionary references from legacy binary format
        { 90,  spec_grav_temp,         [specific_gravity_units_help] },
//Remove new dictionary references from legacy binary format
        { 91,  grams_cm3_temp,         [grams_per_cubic_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 92,  kgrams_m3_temp,         [kilograms_per_cubic_meter_help] },
        { 93,  [lb_gal],            [pounds_per_gallon_help] },
//Remove new dictionary references from legacy binary format
        { 94,  lb_ft3_temp,            [pounds_per_cubic_foot_help] },
//Remove new dictionary references from legacy binary format
        { 95,  grams_ml_temp,          [grams_per_milliliter_help] },
//Remove new dictionary references from legacy binary format
        { 96,  kgrams_l_temp,          [kilograms_per_liter_help] },
//Remove new dictionary references from legacy binary format
        { 97,  grams_l_temp,           [grams_per_liter_help] },
//Remove new dictionary references from legacy binary format
        { 98,  lb_In3_temp,            [pounds_per_cubic_inch_help] },
//Remove new dictionary references from legacy binary format
        { 99,  ton_yd3_temp,           [short_tons_per_cubic_yard_help] },
//Remove new dictionary references from legacy binary format
        { 100, degreesTw_temp,         [degrees_twaddell_help] },
//Remove new dictionary references from legacy binary format
        { 102, degBaumeHv_temp,        [degrees_baume_heavy_help] },
//Remove new dictionary references from legacy binary format
        { 103, degBaumeLt_temp,        [degrees_baume_light_help] },
//Remove new dictionary references from legacy binary format
        { 104, degreesAPI_temp,        [degrees_API_help] },
//Remove new dictionary references from legacy binary format
        { 146, ugrams_l_temp,          micrograms_per_liter_help_temp },
//Remove new dictionary references from legacy binary format
        { 147, ugrams_m3_temp,         micrograms_per_cubic_meter_help_temp },
//Remove new dictionary references from legacy binary format
        { 148, Percent_temp,           [percent_consistency_help] },

/* Viscosity */
//Remove new dictionary references from legacy binary format
        { 54,  cSt_temp,               [centistokes_help] },
//Remove new dictionary references from legacy binary format
        { 55,  cP_temp,                [centipoise_help] },

/* Radial Velocity */
//Remove new dictionary references from legacy binary format
        { 117,  degrees_s_temp,        [degrees_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 118,  rps_temp,              [revolutions_per_second_help] },
        { 119,  [rpm],              [revolutions_per_minute_help] },

/* Energy (includes Work) */
//Remove new dictionary references from legacy binary format
        { 69,  joule_temp,             joule_help_temp },
//Remove new dictionary references from legacy binary format
        { 89,  dekatherm_temp,         dekatherm_help_temp },              /* or 1 MMBtu = 1055.056 megajoule (MJ) */
//Remove new dictionary references from legacy binary format
        { 126, ftLbf_temp,             [foot_pound_force_help] },
//Remove new dictionary references from legacy binary format
        { 128, kWatthr_temp,           kilowatt_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 162, MCal_temp,              megacalorie_help_temp },            /* 1 calorie = 4.184 Joules */
//Remove new dictionary references from legacy binary format
        { 164, MJ_temp,                megajoule_help_temp },
//Remove new dictionary references from legacy binary format
        { 165, btu_temp,               [british_thermal_unit_help] },   /* (1.0 BTU=0.2519958kcal) Energy */

/* Force */
//Remove new dictionary references from legacy binary format
        { 68,  Newton_temp,            [newton_help] },

/* Power */
//Remove new dictionary references from legacy binary format
        { 127, kWatt_temp,             kilowatt_help_temp },
//Remove new dictionary references from legacy binary format
        { 129, [horsepower],        Horsepower_help_temp },
//Remove new dictionary references from legacy binary format
        { 140, MCal_hr_temp,           [mega_calorie_per_hour_help] },          /* 1 calorie = 4.184 Joules */
//Remove new dictionary references from legacy binary format
        { 141, MJ_hr_temp,             megajoules_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 142, btu_hr_temp,            [british_thermal_unit_per_hour_help] }, /* (1.0 BTU=0.2519958kcal) Energy */

/* Frequency */
//Remove new dictionary references from legacy binary format
        { 38,  hertz_temp,             [hertz_help] },

/* Analytical */
//Remove new dictionary references from legacy binary format
        { 57,  Percent_temp,           [percent_help] },
//Remove new dictionary references from legacy binary format
        { 59,  [pH],                potential_of_hydrogen_help_temp },
//Remove new dictionary references from legacy binary format
        { 150, Percent_temp,           [percent_steam_quality_help] },
//Remove new dictionary references from legacy binary format
        { 160, percentPlato_temp,      [percent_plato_help] },
//Remove new dictionary references from legacy binary format
        { 161, percentLEL_temp,        [percent_lel_help] },

/* Capacitance */
//Remove new dictionary references from legacy binary format
        { 153, pF_temp,                [picofarads_help] },

/* Electromagnetic unit of Electric Potential */
//Remove new dictionary references from legacy binary format
        { 36,  miliVolts_temp,         [millivolts_help] },
//Remove new dictionary references from legacy binary format
        { 58,  volts_temp,             [volts_help] },

/* Electrostatic unit of Current */
//Remove new dictionary references from legacy binary format
        { 39,  milliamperes_temp,      [milliamperes_help] },

/* Electromagnetic unit of Resistance */
//Remove new dictionary references from legacy binary format
        { 37,  Ohms_temp,              [ohms_help] },
//Remove new dictionary references from legacy binary format
        { 163, kOhms_temp,             kiloohms_help_temp },

/* Angle */
//Remove new dictionary references from legacy binary format
        { 143, deg_temp,               Degrees_help_temp },
        { 144, [radian],            [radian_help] },

/* Conductance */
//Remove new dictionary references from legacy binary format
        { 56,  uS_temp,                [microsiemens_help] },
//Remove new dictionary references from legacy binary format
        { 66,  mS_cm_temp,             [milli_siemens_per_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 67,  uS_cm_temp,             [micro_siemens_per_centimeter_help] },

/* Volume per Volume */
//Remove new dictionary references from legacy binary format
        { 149, volPercent_temp,        [volume_percent_help] },
//Remove new dictionary references from legacy binary format
        { 154, mL_l_temp,              milliliters_per_liter_help_temp },
//Remove new dictionary references from legacy binary format
        { 155, uL_l_temp,              microliters_per_liter_help_temp },

/* Volume per Mass */
//Remove new dictionary references from legacy binary format
        { 107, degreesBg_temp,         [degrees_balling_help] },
//Remove new dictionary references from legacy binary format
        { 152, ft3_lb_temp,            [cubic_feet_per_pound_help] },

/* Concentration */
//Remove new dictionary references from legacy binary format
      /*{  57, Percent_temp,           [percent_help] }, 57-Percent is already included in the Analytical Section above. */ 
//Remove new dictionary references from legacy binary format
        { 101, degreesBx_temp,         [degrees_brix_help] },
//Remove new dictionary references from legacy binary format
        { 105, Percent_temp,           [percent_solids_per_weight_help] },
//Remove new dictionary references from legacy binary format
        { 106, Percent_temp,           [percent_solids_per_volume_help] },
        { 108, [proof_vol],         [proof_per_volume_help] },
        { 109, [proof_mass],        [proof_per_mass_help] },
//Remove new dictionary references from legacy binary format
        { 139, ppm_temp,               [parts_per_million_help] },
//Remove new dictionary references from legacy binary format
        { 169, ppb_temp,               [ppb_help] },

/* Miscellaneous */
//Remove new dictionary references from legacy binary format
        { 156, dB_temp,                decibel_help_temp },

        /* Generic */
        /*   240  Note- Enumeration may be used for manufacturer specific definitions */
        /*   241  Note- Enumeration may be used for manufacturer specific definitions */
        /*   242  Note- Enumeration may be used for manufacturer specific definitions */
        /*   243  Note- Enumeration may be used for manufacturer specific definitions */
        /*   244  Note- Enumeration may be used for manufacturer specific definitions */
        /*   245  Note- Enumeration may be used for manufacturer specific definitions */
        /*   246  Note- Enumeration may be used for manufacturer specific definitions */
        /*   247  Note- Enumeration may be used for manufacturer specific definitions */
        /*   248  Note- Enumeration may be used for manufacturer specific definitions */
        /*   249  Note- Enumeration may be used for manufacturer specific definitions */
//Remove new dictionary references from legacy binary format
        { 250, Not_Used_temp },
//Remove new dictionary references from legacy binary format
        { 251, none_temp },
//Remove new dictionary references from legacy binary format
        { 252, unknown_temp },
//Remove new dictionary references from legacy binary format
        { 253, Special_temp }
    }
}

/**********************************************************************
 * Expanded Units code tables
 *
 * this table allows the correct unit code table to be found given
 * the device variable classification code.  This table is not
 * normally used in field device DDs.
 */
ARRAY OF VARIABLE units_code_expanded
{
    ELEMENTS
    {
//Remove new dictionary references from legacy binary format
              0, units_code,                                   Units_temp;                                   // Standard Unit Codes  (non-expansion)
//Remove new dictionary references from legacy binary format
             64, units_code_temperature,                       temperature_classification_temp;              // Temperature
//Remove new dictionary references from legacy binary format
             65, units_code_pressure,                          pressure_classification_temp;                 // Pressure
//Remove new dictionary references from legacy binary format
             66, units_code_volumetric_flow,                   Volumetric_Flow_Classification_temp;          // Volumetric Flow
//Remove new dictionary references from legacy binary format
             67, units_code_velocity,                          velocity_classification_temp;                 // Velocity
//Remove new dictionary references from legacy binary format
             68, units_code_volume,                            volume_classification_temp;                   // Volume
//Remove new dictionary references from legacy binary format
             69, units_code_length,                            length_classification_temp;                   // Length
//Remove new dictionary references from legacy binary format
             70, units_code_time,                              time_classification_temp;                     // Time
//Remove new dictionary references from legacy binary format
             71, units_code_mass,                              mass_classification_temp;                     // Mass
//Remove new dictionary references from legacy binary format
             72, units_code_mass_flow,                         Mass_Flow_Classification_temp;                // Mass Flow
//Remove new dictionary references from legacy binary format
             73, units_code_mass_per_volume,                   Mass_Per_Volume_Classification_temp;          // Mass per Volume
//Remove new dictionary references from legacy binary format
             74, units_code_viscosity,                         viscosity_classification_temp;                // Viscosity
//Remove new dictionary references from legacy binary format
             75, units_code_angular_velocity,                  Angular_Velocity_Classification_temp;         // Angular Velocity
//Remove new dictionary references from legacy binary format
           //76, units_code_area,                              area_classification_temp;                     // Area
//Remove new dictionary references from legacy binary format
             77, units_code_energy,                            energy_classification_temp;                   // Energy
//Remove new dictionary references from legacy binary format
             78, units_code_force,                             force_classification_temp;                    // Force
//Remove new dictionary references from legacy binary format
             79, units_code_power,                             power_classification_temp;                    // Power
//Remove new dictionary references from legacy binary format
             80, units_code_frequency,                         frequency_classification_temp;                // Frequency
//Remove new dictionary references from legacy binary format
             81, units_code_analytical,                        analytical_classification_temp;               // Analytical
//Remove new dictionary references from legacy binary format
             82, units_code_capacitance,                       capacitance_classification_temp;              // Capacitance
//Remove new dictionary references from legacy binary format
             83, units_code_emf,                               Emf_Classification_temp;                      // Electromagnetic Force
//Remove new dictionary references from legacy binary format
             84, units_code_current,                           current_classification_temp;                  // Current
//Remove new dictionary references from legacy binary format
             85, units_code_resistance,                        resistance_classification_temp;               // Resistance
//Remove new dictionary references from legacy binary format
             86, units_code_angle,                             angle_classification_temp;                    // Angle
//Remove new dictionary references from legacy binary format
             87, units_code_conductance,                       conductance_classification_temp;              // Conductance
//Remove new dictionary references from legacy binary format
             88, units_code_volume_per_volume,                 Volume_Per_Volume_Classification_temp;        // Volume per Volume
//Remove new dictionary references from legacy binary format
             89, units_code_volume_per_mass,                   Volume_Per_Mass_Classification_temp;          // Volume per Mass
//Remove new dictionary references from legacy binary format
             90, units_code_concentration,                     concentration_classification_temp;            // Concentration
           //91, reserved
           //92, reserved
           //93, reserved
           //94, reserved
           //95, reserved
//Remove new dictionary references from legacy binary format
             96, units_code_acceleration,                      acceleration_classification_temp;             // Acceleration
//Remove new dictionary references from legacy binary format
             97, units_code_turbidity,                         turbidity_classification_temp;                // Turbidity
//Remove new dictionary references from legacy binary format
           //98, units_code_temperature difference,            temperature_diff_classification_temp;         // Temperature Difference
//Remove new dictionary references from legacy binary format
             99, units_code_volumetric_gas_per_second,         volumetric_gas_flow_per_second_class_temp;    // Volumetric Gas Flow per Second
//Remove new dictionary references from legacy binary format
            100, units_code_volumetric_gas_per_minute,         volumetric_gas_flow_per_minute_class_temp;    // Volumetric Gas Flow per Minute
//Remove new dictionary references from legacy binary format
            101, units_code_volumetric_gas_per_hour,           volumetric_gas_flow_per_hour_class_temp;      // Volumetric Gas Flow per Hour
//Remove new dictionary references from legacy binary format
            102, units_code_volumetric_gas_per_day,            volumetric_gas_flow_per_day_class_temp;       // Volumetric Gas Flow per Day
//Remove new dictionary references from legacy binary format
            103, units_code_volumetric_liquid_flow_per_second, volumetric_liquid_flow_per_second_class_temp; // Volumetric Liquid Flow per Second
//Remove new dictionary references from legacy binary format
            104, units_code_volumetric_liquid_flow_per_minute, volumetric_liquid_flow_per_minute_class_temp; // Volumetric Liquid Flow per Minute
//Remove new dictionary references from legacy binary format
            105, units_code_volumetric_liquid_flow_per_hour,   volumetric_liquid_flow_per_hour_class_temp;   // Volumetric Liquid Flow per Hour
//Remove new dictionary references from legacy binary format
            106, units_code_volumetric_liquid_flow_per_day,    volumetric_liquid_flow_per_day_class_temp;    // Volumetric Liquid Flow per Day
//Remove new dictionary references from legacy binary format
            107, units_code_thermal_expansion,                 thermal_expansion_classification_temp;        // Thermal Expansion
//Remove new dictionary references from legacy binary format
            108, units_code_volumetric_energy_density,         volumetric_energy_density_class_temp;         // Volumetric Energy Density
//Remove new dictionary references from legacy binary format
            109, units_code_mass_energy_density,               mass_energy_density_class_temp;               // Mass Energy Density
//Remove new dictionary references from legacy binary format
            110, units_code_torque,                            torque_classification_temp;                   // Torque
//Remove new dictionary references from legacy binary format
            111, units_code_miscellaneous,                     miscellaneous_classification_temp;            // Miscellaneous
//Remove new dictionary references from legacy binary format
            112, units_code_torsional_stiffness,               tortional_stiffness_temp;                     // Torsional Stiffness
//Remove new dictionary references from legacy binary format
            113, units_code_linear_stiffness,                  linear_stiffness_temp;                        // Linear Stiffness
    }
}

/**********************************************************************
 * Table 2.64 (0x40) Temperature
 */
VARIABLE units_code_temperature
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 32,  degreesC_temp,          [degrees_celsius_help] },
//Remove new dictionary references from legacy binary format
        { 33,  degreesF_temp,          [degrees_fahrenheit_help] },
//Remove new dictionary references from legacy binary format
        { 34,  degreesR_temp,          [degrees_rankine_help] },
//Remove new dictionary references from legacy binary format
        { 35,  kelvin_temp,            [degrees_kelvin_help] }
    }
}

/**********************************************************************
 * Table 2.65 (0x41) Pressure
 *
 *  NOTE: Pressure is ambiguous with respect to absolute or gage
 */
VARIABLE units_code_pressure
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        { 1,   [InH2O],             [inches_of_water_68_degrees_F_help] },
        { 2,   [InHg],              [inches_of_mercury_0_degrees_C_help] },
        { 3,   [FtH2O],             [feet_of_water_68_degrees_F_help] },
        { 4,   [mmH2O],             [millimeters_of_water_68_degrees_F_help] },
        { 5,   [mmHg],              [millimeters_of_mercury_0_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 6,   psi_temp,               [pounds_per_square_inch_help] },
        { 7,   [bar],               [bars_help] },
        { 8,   [mbar],              [millibars_help] },
//Remove new dictionary references from legacy binary format
        { 9,   grams_cm2_temp,         [grams_per_square_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 10,  kgrams_cm2_temp,        [kilograms_per_square_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 11,  pascals_temp,           [pascals_help] },
//Remove new dictionary references from legacy binary format
        { 12,  kpascals_temp,          [kilopascals_help] },
//Remove new dictionary references from legacy binary format
        { 13,  Torr_temp,              [torr_help] },
//Remove new dictionary references from legacy binary format
        { 14,  ATM_temp,               [atmospheres_help] },
//Remove new dictionary references from legacy binary format
        { 145, inH2O_60F_temp,         [inches_of_water_60_degrees_F_help] },
//Remove new dictionary references from legacy binary format
        { 170, cmH20_4C_temp,          [centimeters_of_water_4_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 171, mH20_4C_temp,           [meters_of_water_4_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 172, cmHg_temp,              [centimeters_of_mercury_0_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 173, lb_ft2_temp,            [pounds_square_foot_help] },
        { 174, [hPa],               [hPa_help] },
//Remove new dictionary references from legacy binary format
        { 175, psia_temp,              [pounds_square_inch_absolute_help] },
//Remove new dictionary references from legacy binary format
        { 176, kgrams_m2_temp,         [kilograms_per_square_meter_help] },
//Remove new dictionary references from legacy binary format
        { 177, ftH20_4C_temp,          [feet_of_water_4_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 178, ftH2O_60F_temp,         [feet_of_water_60_degrees_F_help] },
//Remove new dictionary references from legacy binary format
        { 179, mHg_temp,               [meters_of_mercury_0_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 180, Mpsi_temp,              mega_pounds_per_square_inch_help_temp },
//Remove new dictionary references from legacy binary format
        { 181, oz_in2_temp,            oz_in2_help_temp },
//Remove new dictionary references from legacy binary format
        { 237, Mpascals_temp,          [megapascals_help] },
//Remove new dictionary references from legacy binary format
        { 238, inH2O_4C_temp,          [inches_of_water_4_degrees_C_help] },
//Remove new dictionary references from legacy binary format
        { 239, mmH2O_4C_temp,          [millimeters_of_water_4_degrees_C_help] }
    }
}

/**********************************************************************
 * Table 2.66 (0x42) Volumetric Flow
 */
VARIABLE units_code_volumetric_flow
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 15,  ft3_min_temp,           [cubic_feet_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 16,  [gal_min],           Gallons_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 17,  L_min_temp,             [liters_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 18,  galUK_min_temp,         [imperial_gallons_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 19,  m3_hr_temp,             [cubic_meter_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 22,  [gal_s],             Gallons_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 23,  Mgal_day_temp,          Million_gallons_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 24,  L_s_temp,               [liters_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 25,  Mltr_day_temp,          [million_liters_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 26,  ft3_s_temp,             [cubic_feet_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 27,  ft3_day_temp,           [cubic_feet_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 28,  m3_s_temp,              cubic_meter_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 29,  m3_day_temp,            cubic_meter_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 30,  galUK_hr_temp,          [imperial_gallons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 31,  galUK_day_temp,         [imperial_gallons_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 121, Nm3_hr_temp,            Normal_cubic_meters_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 122, NL_hr_temp,             Normal_liters_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 123, SCF_min_temp,           Standard_cubic_feet_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 130, ft3_hr_temp,            [cubic_feet_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 131, m3_min_temp,            [cubic_meters_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 132, [bbl_s],             Oil_Barrels_per_second_help_temp },        /* 1 barrel equals 42 U.S. gallons */
//Remove new dictionary references from legacy binary format
        { 133, [bbl_min],           Oil_Barrels_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 134, [bbl_hr],            Oil_Barrels_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 135, [bbl_day],           Oil_Barrels_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 136, [gal_hr],            Gallons_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 137, galUK_s_temp,           [imperial_gallons_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 138, L_hr_temp,              [liters_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 170, bblBeer_s_temp,         [beer_barrels_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 171, bblBeer_min_temp,       [beer_barrels_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 172, bblBeer_hr_temp,        [beer_barrels_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 173, bblBeer_day_temp,       [beer_barrels_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 174, NL_day_temp,            Normal_liters_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, NL_min_temp,            Normal_liters_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, NL_s_temp,              Normal_liters_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 177, SL_d_temp,              [standard_liter_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 178, SL_hr_temp,             [standard_liter_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 179, SL_min_temp,            [standard_liter_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 180, SL_s_temp,              [standard_liter_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 181, Nm3_day_temp,           Normal_cubic_meters_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 182, Nm3_min_temp,           Normal_cubic_meters_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 183, Nm3_s_temp,             Normal_cubic_meters_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 184, SCF_day_temp,           Standard_cubic_feet_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 185, SCF_hr_temp,            Standard_cubic_feet_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 186, SCF_s_temp,             Standard_cubic_feet_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 187, Sm3_day_temp,           [standard_cubic_meter_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 188, Sm3_hr_temp,            [standard_cubic_meter_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 189, Sm3_min_temp,           [standard_cubic_meter_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 190, Sm3_s_temp,             [standard_cubic_meter_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 235, gal_day_temp,           Gallons_per_day_help_temp }
    }
}

/**********************************************************************
 * Table 2.67 (0x43) Velocity
 */
VARIABLE units_code_velocity
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        { 20,  [ft_s],              [feet_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 21,  m_s_temp,               [meters_per_second_help] },
        { 114, [in_s],              [inches_per_second_help] },
        { 115, [in_min],            [inches_per_minute_help] },
        { 116, [ft_min],            [feet_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 120, m_hr_temp,              [meters_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 170, mm_s_temp,              mm_s_help_temp }
    }
}

/**********************************************************************
 * Table 2.68 (0x44) Volume
 */
VARIABLE units_code_volume
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        { 40,  [gal],               [gallons_help] },
//Remove new dictionary references from legacy binary format
        { 41,  Liter_temp,             [liters_help] },
//Remove new dictionary references from legacy binary format
        { 42,  galUK_temp,             [imperial_gallons_help] },
//Remove new dictionary references from legacy binary format
        { 43,  m3_temp,                [cubic_meters_help] },
//Remove new dictionary references from legacy binary format
        { 46,  [bbl],               Barrel_help_temp },        /* 1 barrel equals 42 U.S. gallons */
//Remove new dictionary references from legacy binary format
        { 110, bushels_temp,           [bushels_help] },
//Remove new dictionary references from legacy binary format
        { 111, Yd3_temp,               [cubic_yards_help] },
//Remove new dictionary references from legacy binary format
        { 112, ft3_temp,               [cubic_feet_help] },
//Remove new dictionary references from legacy binary format
        { 113, in3_temp,               [cubic_inches_help] },
//Remove new dictionary references from legacy binary format
        { 124, bblUS_temp,             Liquid_barrel_help_temp },       /* 1 liquid barrel equals 31.5 U.S. gallons. Note that */
//Remove new dictionary references from legacy binary format
        { 166, Nm3_temp,               Normal_cubic_meters_help_temp },
//Remove new dictionary references from legacy binary format
        { 167, NL_temp,                Normal_liters_help_temp },
//Remove new dictionary references from legacy binary format
        { 168, SCF_temp,               Standard_cubic_feet_help_temp },
//Remove new dictionary references from legacy binary format
        { 170, bblBeer_temp,           [beer_bbl_help] },
//Remove new dictionary references from legacy binary format
        { 171, SL_temp,                [standard_liter_help] },
//Remove new dictionary references from legacy binary format
        { 172, Sm3_temp,               [standard_cubic_meter_help] },
//Remove new dictionary references from legacy binary format
        { 173, Mgal_temp,              Mgal_help_temp },
//Remove new dictionary references from legacy binary format
        { 236, hL_temp,                [hectoliters_help] }
    }
}

/**********************************************************************
 * Table 2.69 (0x45) Length
 */
VARIABLE units_code_length
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        { 44,  [ft],                [feet_help] },
//Remove new dictionary references from legacy binary format
        { 45,  meters_temp,            [meters_help] },
        { 47,  [in],                [inches_help] },
//Remove new dictionary references from legacy binary format
        { 48,  centimeters_temp,       [centimeters_help] },
//Remove new dictionary references from legacy binary format
        { 49,  millimeters_temp,       [millimeters_help] },
//Remove new dictionary references from legacy binary format
        { 151, ft16_temp,              feet_in_sixteenths_help_temp },
//Remove new dictionary references from legacy binary format
        { 170, micron_temp,            micron_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, uIn_temp,               microinch_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, nm_temp,                nanometer_help_temp }
    }
}

/*************************************************************************************
 * Table 2.70 (0x46) Time
 */
VARIABLE units_code_time
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 50,  minutes_temp,           [minutes_help] },
//Remove new dictionary references from legacy binary format
        { 51,  seconds_temp,           [seconds_help] },
//Remove new dictionary references from legacy binary format
        { 52,  hours_temp,             [hours_help] },
//Remove new dictionary references from legacy binary format
        { 53,  days_temp,              [days_help] },
//Remove new dictionary references from legacy binary format
        { 170, mSec_temp,              milliseconds_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, uSec_temp,              microseconds_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, nSec_temp,              nanoseconds_help_temp}
    }
}

/*************************************************************************************
 * Table 2.71 (0x47) Mass
 */
VARIABLE units_code_mass
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 60,  grams_temp,             [grams_help] },
//Remove new dictionary references from legacy binary format
        { 61,  kgrams_temp,            [kilograms_help] },
//Remove new dictionary references from legacy binary format
        { 62,  metTon_temp,            [metric_tons_help] },
        { 63,  [lb],                [pounds_help] },
//Remove new dictionary references from legacy binary format
        { 64,  ton_temp,               [short_tons_help] },
//Remove new dictionary references from legacy binary format
        { 65,  tonUK_temp,             [long_tons_help] },
//Remove new dictionary references from legacy binary format
        { 125, oz_temp,                Ounce_help_temp }
    }
}

/*************************************************************************************
 * Table 2.72 (0x48) Mass Flow
 */
VARIABLE units_code_mass_flow
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 70,  grams_s_temp,           [grams_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 71,  grams_min_temp,         [grams_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 72,  grams_hr_temp,          [grams_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 73,  kgrams_s_temp,          [kilograms_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 74,  kgrams_min_temp,        [kilograms_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 75,  kgrams_hr_temp,         [kilograms_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 76,  kgrams_day_temp,        [kilograms_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 77,  metTon_min_temp,        [metric_tons_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 78,  metTon_hr_temp,         [metric_tons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 79,  metTon_day_temp,        [metric_tons_per_day_help] },
        { 80,  [lb_s],              [pounds_per_second_help] },
        { 81,  [lb_min],            [pounds_per_minute_help] },
        { 82,  [lb_hr],             [pounds_per_hour_help] },
        { 83,  [lb_day],            [pounds_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 84,  ton_min_temp,           [short_tons_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 85,  ton_hr_temp,            [short_tons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 86,  ton_day_temp,           [short_tons_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 87,  tonUK_hr_temp,          [long_tons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 88,  tonUK_day_temp,         [long_tons_per_day_help] }
    }
}

/*************************************************************************************
 * Table 2.73 (0x49) Mass per Volume
 */
VARIABLE units_code_mass_per_volume
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 90,  spec_grav_temp,         [specific_gravity_units_help] },
//Remove new dictionary references from legacy binary format
        { 91,  grams_cm3_temp,         [grams_per_cubic_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 92,  kgrams_m3_temp,         [kilograms_per_cubic_meter_help] },
        { 93,  [lb_gal],            [pounds_per_gallon_help] },
//Remove new dictionary references from legacy binary format
        { 94,  lb_ft3_temp,            [pounds_per_cubic_foot_help] },
//Remove new dictionary references from legacy binary format
        { 95,  grams_ml_temp,          [grams_per_milliliter_help] },
//Remove new dictionary references from legacy binary format
        { 96,  kgrams_l_temp,          [kilograms_per_liter_help] },
//Remove new dictionary references from legacy binary format
        { 97,  grams_l_temp,           [grams_per_liter_help] },
//Remove new dictionary references from legacy binary format
        { 98,  lb_In3_temp,            [pounds_per_cubic_inch_help] },
//Remove new dictionary references from legacy binary format
        { 99,  ton_yd3_temp,           [short_tons_per_cubic_yard_help] },
//Remove new dictionary references from legacy binary format
        { 100, degreesTw_temp,         [degrees_twaddell_help] },
//Remove new dictionary references from legacy binary format
        { 102, degBaumeHv_temp,        [degrees_baume_heavy_help] },
//Remove new dictionary references from legacy binary format
        { 103, degBaumeLt_temp,        [degrees_baume_light_help] },
//Remove new dictionary references from legacy binary format
        { 104, degreesAPI_temp,        [degrees_API_help] },
//Remove new dictionary references from legacy binary format
        { 146, ugrams_l_temp,          micrograms_per_liter_help_temp },
//Remove new dictionary references from legacy binary format
        { 147, ugrams_m3_temp,         micrograms_per_cubic_meter_help_temp },
//Remove new dictionary references from legacy binary format
        { 148, Percent_temp,           [percent_consistency_help] },
//Remove new dictionary references from legacy binary format
        { 170, mgrams_l_temp,          milligrams_per_liter_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, mgrams_m3_temp,         milligrams_per_cubic_meter_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, t_m3_temp,              t_m3_help_temp }
    }
}

/*************************************************************************************
 * Table 2.74 (0x4A) Viscosity
 */
VARIABLE units_code_viscosity
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 54,  cSt_temp,               [centistokes_help] },
//Remove new dictionary references from legacy binary format
        { 55,  cP_temp,                [centipoise_help] },
//Remove new dictionary references from legacy binary format
        { 170, PaSec_temp,             pascal_second_help_temp }
    }
}

/*************************************************************************************
 * Table 2.75 (0x4B) Angular Velocity
 */
VARIABLE units_code_angular_velocity
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 117,  degrees_s_temp,        [degrees_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 118,  rps_temp,              [revolutions_per_second_help] },
        { 119,  [rpm],              [revolutions_per_minute_help] }
    }
}

/*************************************************************************************
 * Table 2.76 (0x4C) Area Unit Code
 */

/*************************************************************************************
 * Table 2.77 (0x4D) Energy (includes Work)
 */
VARIABLE units_code_energy
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 69,  joule_temp,             joule_help_temp },
//Remove new dictionary references from legacy binary format
        { 89,  dekatherm_temp,         dekatherm_help_temp },              /* or 1 MMBtu = 1055.056 megajoule (MJ) */
//Remove new dictionary references from legacy binary format
        { 126, ftLbf_temp,             [foot_pound_force_help] },
//Remove new dictionary references from legacy binary format
        { 128, kWatthr_temp,           kilowatt_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 162, MCal_temp,              megacalorie_help_temp },            /* 1 calorie = 4.184 Joules */
//Remove new dictionary references from legacy binary format
        { 164, MJ_temp,                megajoule_help_temp },
//Remove new dictionary references from legacy binary format
        { 165, btu_temp,               [british_thermal_unit_help] },   /* (1.0 BTU=0.2519958kcal) Energy */
//Remove new dictionary references from legacy binary format
        { 170, kVAHr_temp,             kilo_volt_ampere_hours_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, kVarHr_temp,            kilo_volt_ampere_reactive_hours_help_temp }
    }
}

/*************************************************************************************
 * Table 2.78 (0x4E) Force
 */
VARIABLE units_code_force
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 68,  Newton_temp,            [newton_help] },
//Remove new dictionary references from legacy binary format
        { 170, kN_temp,                kilo_newton_help_temp }
    }
}

/*************************************************************************************
 * Table 2.79 (0x4F) Power
 */
VARIABLE units_code_power
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 127, kWatt_temp,             kilowatt_help_temp },
//Remove new dictionary references from legacy binary format
        { 129, [horsepower],        Horsepower_help_temp },
//Remove new dictionary references from legacy binary format
        { 140, MCal_hr_temp,           [mega_calorie_per_hour_help] },          /* 1 calorie = 4.184 Joules */
//Remove new dictionary references from legacy binary format
        { 141, MJ_hr_temp,             megajoules_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 142, btu_hr_temp,            [british_thermal_unit_per_hour_help] }, /* (1.0 BTU=0.2519958kcal) Energy */
//Remove new dictionary references from legacy binary format
        { 170, MJ_s_temp,              megajoules_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, MJ_day_temp,            megajoules_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, MMBtu_s_temp,           million_british_thermal_units_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 173, MMBtu_hr_temp,          million_british_thermal_units_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 174, MMBtu_day_temp,         million_british_thermal_units_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, VA_temp,                volt_ampere_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, kVA_temp,               kilo_volt_ampere_help_temp },
//Remove new dictionary references from legacy binary format
        { 177, kVar_temp,              kilo_volt_ampere_reactive_help_temp },
//Remove new dictionary references from legacy binary format
        { 178, btu_day_temp,           british_thermal_units_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 179, btu_min_temp,           british_thermal_units_per_day_minute_temp},
//Remove new dictionary references from legacy binary format
        { 180, MJ_min_temp,            megajoules_per_minute_help_temp }
    }
}

/*************************************************************************************
 * Table 2.80 (0x50) Frequency
 */
VARIABLE units_code_frequency
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 38,  hertz_temp,             [hertz_help] }
    }
}


/*************************************************************************************
 * Table 2.81 (0x51) Analytical
 */
VARIABLE units_code_analytical
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 57,  Percent_temp,           [percent_help] },
//Remove new dictionary references from legacy binary format
        { 59,  [pH],                potential_of_hydrogen_help_temp },
//Remove new dictionary references from legacy binary format
        { 150, Percent_temp,           [percent_steam_quality_help] },
//Remove new dictionary references from legacy binary format
        { 160, percentPlato_temp,      [percent_plato_help] },
//Remove new dictionary references from legacy binary format
        { 161, percentLEL_temp,        [percent_lel_help] }
    }
}

/*************************************************************************************
 * Table 2.82 (0x52) Capacitance
 */
VARIABLE units_code_capacitance
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 153, pF_temp,                [picofarads_help] }
    }
}

/*************************************************************************************
 * Table 2.83 (0x53) Electromagnetic Force (EMF)
 */
VARIABLE units_code_emf
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 36,  miliVolts_temp,        [millivolts_help] },
//Remove new dictionary references from legacy binary format
        { 58,  volts_temp,             [volts_help] }
    }
}

/*************************************************************************************
 * Table 2.84 (0x54) Current
 */
VARIABLE units_code_current
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 39,  milliamperes_temp,      [milliamperes_help] },
//Remove new dictionary references from legacy binary format
        { 170, nA_temp,                nanoamperes_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, uA_temp,                microamperes_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, amperes_temp,           amperes_help_temp }
    }
}

/*************************************************************************************
 * Table 2.85 (0x55) Resistance
 */
VARIABLE units_code_resistance
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 37,  Ohms_temp,              [ohms_help] },
//Remove new dictionary references from legacy binary format
        { 163, kOhms_temp,             kiloohms_help_temp },
//Remove new dictionary references from legacy binary format
        { 170, MOhm_temp,              megaohms_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, ohmCm_temp,             ohm_centimeter_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, kohmCm_temp,            kiloohm_centimeter_help_temp },
//Remove new dictionary references from legacy binary format
        { 173, MOhmCm_temp,            megaohm_centimeter_help_temp },
//Remove new dictionary references from legacy binary format
        { 174, mOhm_temp,              miliohms_help_temp }
    }
}

/*************************************************************************************
 * Table 2.86 (0x56) Angle
 */
VARIABLE units_code_angle
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 143, deg_temp,               Degrees_help_temp },
        { 144, [radian],            [radian_help] }
    }
}

/*************************************************************************************
 * Table 2.87 (0x57) Conductance
 */
VARIABLE units_code_conductance
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 56,  uS_temp,                [microsiemens_help] },
//Remove new dictionary references from legacy binary format
        { 66,  mS_cm_temp,             [milli_siemens_per_centimeter_help] },
//Remove new dictionary references from legacy binary format
        { 67,  uS_cm_temp,             [micro_siemens_per_centimeter_help] }
    }
}

/*************************************************************************************
 * Table 2.88 (0x58) Volume per Volume
 */
VARIABLE units_code_volume_per_volume
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 149, volPercent_temp,        [volume_percent_help] },
//Remove new dictionary references from legacy binary format
        { 154, mL_l_temp,              milliliters_per_liter_help_temp  },
//Remove new dictionary references from legacy binary format
        { 155, uL_l_temp,              microliters_per_liter_help_temp }
    }
}

/*************************************************************************************
 * Table 2.89 (0x59) Volume per Mass
 */
 VARIABLE units_code_volume_per_mass
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 107, degreesBg_temp,         [degrees_balling_help] },
//Remove new dictionary references from legacy binary format
        { 152, ft3_lb_temp,            [cubic_feet_per_pound_help] }
    }
}

/*************************************************************************************
 * Table 2.90 (0x5A) Concentration
 */
VARIABLE units_code_concentration
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 57,  Percent_temp,           [percent_help] },
//Remove new dictionary references from legacy binary format
        { 101, degreesBx_temp,         [degrees_brix_help] },
//Remove new dictionary references from legacy binary format
        { 105, Percent_temp,           [percent_solids_per_weight_help] },
//Remove new dictionary references from legacy binary format
        { 106, Percent_temp,           [percent_solids_per_volume_help] },
        { 108, [proof_vol],         [proof_per_volume_help] },
        { 109, [proof_mass],        [proof_per_mass_help] },
//Remove new dictionary references from legacy binary format
        { 139, ppm_temp,               [parts_per_million_help] },
//Remove new dictionary references from legacy binary format
        { 169, ppb_temp,               [ppb_help] },
//Remove new dictionary references from legacy binary format
        { 170, [ppth],              ppth_help_temp }
    }
}

/*************************************************************************************
 * Table 2.91 (0x5B) Valve Actuator
 */

/*************************************************************************************
 * Table 2.92 (0x5C) Level
 */

/*************************************************************************************
 * Table 2.93 (0x5D) Vortex Flow
 */

/*************************************************************************************
 * Table 2.94 (0x5E) Mag Flow
 */

/*************************************************************************************
 * Table 2.95 (0x5F) Coriolis Flow
 */

/*************************************************************************************
 * Table 2.96 (0x60) Acceleration
 */
VARIABLE units_code_acceleration
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, gravAccel_temp,         gravitational_acceleration_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, ft_s2_temp,             feet_per_square_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, m_s2_temp,              meters_per_square_second_help_temp }
    }
}

/*************************************************************************************
 * Table 2.97 (0x61) Turbidity
 */
VARIABLE units_code_turbidity
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, fnu_temp,               formazin_nephelometric_units_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, ftu_temp,               formazin_turbidity_unit_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, ntu_temp,               nephelometric_turbidity_unit_help_temp }
    }
}

/*************************************************************************************
 * Table 2.98 (0x62)
 */

/*************************************************************************************
 * Table 2.99 (0x63) Volumetric Gas Flow per Second
 */
VARIABLE units_code_volumetric_gas_per_second
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 186, SCF_s_temp,             Standard_cubic_feet_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, NL_s_temp,              Normal_liters_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 180, SL_s_temp,              [standard_liter_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 183, Nm3_s_temp,             Normal_cubic_meters_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 190, Sm3_s_temp,             [standard_cubic_meter_per_second_help] }
    }
}

/*************************************************************************************
 * Table 2.100 (0x64) Volumetric Gas Flow per Minute
 */
VARIABLE units_code_volumetric_gas_per_minute
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 123, SCF_min_temp,           Standard_cubic_feet_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, NL_min_temp,            Normal_liters_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 179, SL_min_temp,            [standard_liter_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 182, Nm3_min_temp,           Normal_cubic_meters_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 189, Sm3_min_temp,           [standard_cubic_meter_per_minute_help] }
    }
}

/*************************************************************************************
 * Table 2.101 (0x65) Volumetric Gas Flow per Hour
 */
VARIABLE units_code_volumetric_gas_per_hour
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 185, SCF_hr_temp,            Standard_cubic_feet_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 122, NL_hr_temp,             Normal_liters_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 178, SL_hr_temp,             [standard_liter_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 121, Nm3_hr_temp,            Normal_cubic_meters_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 188, Sm3_hr_temp,            [standard_cubic_meter_per_hour_help] }
    }
}

/*************************************************************************************
 * Table 2.102 (0x66) Volumetric Gas Flow per Day
 */
VARIABLE units_code_volumetric_gas_per_day
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 184, SCF_day_temp,           Standard_cubic_feet_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 174, NL_day_temp,            Normal_liters_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 177, SL_d_temp,              [standard_liter_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 181, Nm3_day_temp,           Normal_cubic_meters_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 187, Sm3_day_temp,           [standard_cubic_meter_per_day_help] }
    }
}

/*************************************************************************************
 * Table 2.103 (0x67) Volumetric Liquid Flow per Second
 */
VARIABLE units_code_volumetric_liquid_flow_per_second
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 174, flOzUK_s_temp,          imperial_fluid_ounces_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, flOz_s_temp,            fluid_ounces_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, mL_s_temp,              milliliter_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 24,  L_s_temp,               [liters_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 178, hL_s_temp,              hectoliter_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 28,  m3_s_temp,              cubic_meter_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 180, ML_s_temp,              million_liters_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 22,  [gal_s],             Gallons_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 182, kgal_s_temp,            kilogallons_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 183, Mgal_s_temp,            million_gallons_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 137, galUK_s_temp,           [imperial_gallons_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 185, MgalUK_s_temp,          imperial_million_gallons_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 26,  ft3_s_temp,             [cubic_feet_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 187, acreFt_s_temp,          acre_feet_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 188, drum_s_temp,            drum_per_second_help_temp},
//Remove new dictionary references from legacy binary format
        { 189, bblUS_s_temp,           liquid_barrel_per_second_help_temp },
//Remove new dictionary references from legacy binary format
        { 170, bblBeer_s_temp,         [beer_barrels_per_second_help] },
//Remove new dictionary references from legacy binary format
        { 132, [bbl_s],             Oil_Barrels_per_second_help_temp },        /* 1 barrel equals 42 U.S. gallons */
//Remove new dictionary references from legacy binary format
        { 192, bblBeerUK_s_temp,       UK_beer_barrels_per_second_help_temp }
    }
}

/*************************************************************************************
 * Table 2.104 (0x68) Volumetric Liquid Flow per Minute
 */
VARIABLE units_code_volumetric_liquid_flow_per_minute
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 174, flOzUK_min_temp,        imperial_fluid_ounces_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, flOz_min_temp,          fluid_ounces_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, mL_min_temp,            milliliter_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 17,  L_min_temp,             [liters_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 178, hL_min_temp,            hectoliter_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 131, m3_min_temp,            [cubic_meters_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 180, ML_min_temp,            million_liters_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 16,  [gal_min],           Gallons_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 182, kgal_min_temp,          kilogallons_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 183, Mgal_min_temp,          million_gallons_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 18,  galUK_min_temp,         [imperial_gallons_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 185, MgalUK_min_temp,        imperial_million_gallons_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 15,  ft3_min_temp,           [cubic_feet_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 187, acreFt_min_temp,        acre_feet_per_minute_help_temp},
//Remove new dictionary references from legacy binary format
        { 188, drum_min_temp,          drum_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 189, bblUS_min_temp,         liquid_barrel_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, bblBeer_min_temp,       [beer_barrels_per_minute_help] },
//Remove new dictionary references from legacy binary format
        { 133, [bbl_min],           Oil_Barrels_per_minute_help_temp },
//Remove new dictionary references from legacy binary format
        { 192, bblBeerUK_min_temp,     UK_beer_barrels_per_minute_help_temp }
    }
}

/*************************************************************************************
 * Table 2.105 (0x69) Volumetric Liquid Flow per Hour
 */
VARIABLE units_code_volumetric_liquid_flow_per_hour
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 174, flOzUK_hr_temp,         imperial_fluid_ounces_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, flOz_hr_temp,           fluid_ounces_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, mL_hr_temp,             milliliter_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 138, L_hr_temp,              [liters_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 178, hL_hr_temp,             hectoliter_per_hour_help_temp},
//Remove new dictionary references from legacy binary format
        { 19,  m3_hr_temp,             [cubic_meter_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 180, ML_hr_temp,             million_liters_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 136, [gal_hr],            Gallons_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 182, kgal_hr_temp,           kilogallons_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 183, Mgal_hr_temp,           million_gallons_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 30,  galUK_hr_temp,          [imperial_gallons_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 185, MgalUK_hr_temp,         imperial_million_gallons_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 130, ft3_hr_temp,            [cubic_feet_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 187, acreFt_hr_temp,         acre_feet_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 188, drum_hr_temp,           drum_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 189, bblUS_hr_temp,          liquid_barrel_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, bblBeer_hr_temp,        [beer_barrels_per_hour_help] },
//Remove new dictionary references from legacy binary format
        { 134, [bbl_hr],            Oil_Barrels_per_hour_help_temp },
//Remove new dictionary references from legacy binary format
        { 192, bblBeerUK_hr_temp,      UK_beer_barrels_per_hour_help_temp }
    }
}

/*************************************************************************************
 * Table 2.106 (0x6A) Volumetric Liquid Flow per Day
 */
VARIABLE units_code_volumetric_liquid_flow_per_day
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 174, flOzUK_day_temp,        imperial_fluid_ounces_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, flOz_day_temp,          fluid_ounces_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, mL_day_temp,            milliliter_per_day_help_temp},
//Remove new dictionary references from legacy binary format
        { 177, L_day_temp,             liters_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 178, hL_day_temp,            hectoliter_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 29,  m3_day_temp,            cubic_meters_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 25,  Mltr_day_temp,          [million_liters_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 235, gal_day_temp,           Gallons_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 182, kgal_day_temp,          kilogallons_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 23,  Mgal_day_temp,          Million_gallons_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 31,  galUK_day_temp,         [imperial_gallons_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 185, MgalUK_day_temp,        imperial_million_gallons_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 27,  ft3_day_temp,           [cubic_feet_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 187, acreFt_day_temp,        acre_feet_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 188, drum_day_temp,          drum_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 189, bblUS_day_temp,         liquid_barrel_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 173, bblBeer_day_temp,       [beer_barrels_per_day_help] },
//Remove new dictionary references from legacy binary format
        { 135, [bbl_day],           Oil_Barrels_per_day_help_temp },
//Remove new dictionary references from legacy binary format
        { 192, bblBeerUK_day_temp,     UK_beer_barrels_per_day_help_temp }
    }
}

/*************************************************************************************
 * Table 2.107 (0x6B) Thermal Expansion
 */
VARIABLE units_code_thermal_expansion
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, _C_temp,                _C_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, _F_temp,                _F_help_temp }
    }
}

/*************************************************************************************
 * Table 2.108 (0x6C) Volumetric Energy Density
 */
VARIABLE units_code_volumetric_energy_density
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, kJ_L_temp,              kilojoules_per_liter_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, btu_ft3_temp,           british_thermal_units_per_cubic_foot_help_temp }
    }
}

/*************************************************************************************
 * Table 2.109 (0x6D) Mass Energy Density
 */
VARIABLE units_code_mass_energy_density
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, MJ_kg_temp,             megajoules_per_kilogram_help_temp},
//Remove new dictionary references from legacy binary format
        { 171, btu_lb_temp,            british_thermal_units_per_pound_help_temp }
    }
}

/*************************************************************************************
 * Table 2.110 (0x6E) Torque
 */
VARIABLE units_code_torque
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, Nm_temp,                Newton_meter_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, kNm_temp,               kilo_newton_meter_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, lbFt_temp,              pound_force_feet_help_temp },
//Remove new dictionary references from legacy binary format
        { 173, lbIn_temp,              lbIn_help_temp },
//Remove new dictionary references from legacy binary format
        { 174, mNm_temp,               mNm_help_temp }
    }
}

/*************************************************************************************
 * Table 2.111 (0x6F) Miscellaneous
 */
VARIABLE units_code_miscellaneous
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 57,  Percent_temp,           [percent_help] },
//Remove new dictionary references from legacy binary format
        { 156, dB_temp,                decibel_help_temp },
//Remove new dictionary references from legacy binary format
        { 170, grams_mol_temp,         grams_per_mol_help_temp }
    }
}

/*************************************************************************************
 * Table 2.112 (0x70) Torsional Stiffness Unit Codes
 */
VARIABLE units_code_torsional_stiffness
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, kNm_deg_temp,           kNm_deg_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, Nm_deg_temp,            Nm_deg_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, lbFt_deg_temp,          lbFt_deg_help_temp },
//Remove new dictionary references from legacy binary format
        { 173, lbIn_deg_temp,          lbIn_deg_help_temp },
//Remove new dictionary references from legacy binary format
        { 174, kNm_rad_temp,           kNm_rad_help_temp },
//Remove new dictionary references from legacy binary format
        { 175, Nm_rad_temp,            Nm_rad_help_temp },
//Remove new dictionary references from legacy binary format
        { 176, lbFt_rad_temp,          lbFt_rad_help_temp },
//Remove new dictionary references from legacy binary format
        { 177, lbIn_rad_temp,          lbIn_rad_help_temp }
    }
}

/*************************************************************************************
 * Table 2.113 (0x71) Linear Stiffness Unit Codes
 */
VARIABLE units_code_linear_stiffness
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 170, kNm_m_temp,             kNm_m_help_temp },
//Remove new dictionary references from legacy binary format
        { 171, N_m_temp,               N_m_help_temp },
//Remove new dictionary references from legacy binary format
        { 172, lb_ft_temp,             lb_ft_help_temp },
//Remove new dictionary references from legacy binary format
        { 173, lb_in_temp,             lb_in_help_temp }
    }
}

/*************************************************************************************
 * Table 3. Transfer Function Codes
 */
VARIABLE transfer_function_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
       /* Design Note: Enumerations 240-249 are reserved for manufacturer
          specific definitions. Therefore, Host applications should not
          display meaningful information for these enumerations unless
          the Host has the Device Description for that specific Field
          Device. Some Host applications have used a string of the
          character '?' for this application (e.g. Xfer fnctn:???? */
       /* Design Note: Any enumerations in the range from 250-255
          are 'Reserved' for a single definition if not defined. */
       /* Design Note: Any enumeration not covered by another "Design
          Note" is 'Undefined'. */

//Remove new dictionary references from legacy binary format
        { 0,   linear_temp,                                     [linear_help] },
         /* APPHELP "Equation y=mx+b" */
//Remove new dictionary references from legacy binary format
        { 1,   Square_Root_temp,                                [square_root_help] },
         /* APPHELP "Equation y=sqrt(x)" */
//Remove new dictionary references from legacy binary format
        { 2,   square_root_third_power_temp,                    [square_root_third_power_help] },
         /* APPHELP "Equation y=sqrt(x^3)" */
//Remove new dictionary references from legacy binary format
        { 3,   square_root_fifth_power_temp,                    [square_root_fifth_power_help] },
         /* APPHELP "Equation y=sqrt(x^5)" */
//Remove new dictionary references from legacy binary format
        { 4,   Special_Curve_temp,                              [special_curve_help] },
//Remove new dictionary references from legacy binary format
        { 5,   square_temp,                                     [square_help] },
//Remove new dictionary references from legacy binary format
        { 6,   square_root_with_cutoff_temp,                    square_root_cutoff_help_temp },
//Remove new dictionary references from legacy binary format
        {10,   equal_percentage_1_25_temp,                      equal_percentage_1_25_help_temp },
//Remove new dictionary references from legacy binary format
        {11,   equal_percentage_1_33_temp,                      equal_percentage_1_33_help_temp },
//Remove new dictionary references from legacy binary format
        {12,   equal_percentage_1_50_temp,                      equal_percentage_1_50_help_temp },
//Remove new dictionary references from legacy binary format
        {15,   quick_open_1_25_temp,                            quick_open_1_25_help_temp },
//Remove new dictionary references from legacy binary format
        {16,   quick_open_1_33_temp,                            quick_open_1_33_help_temp },
//Remove new dictionary references from legacy binary format
        {17,   quick_open_1_50_temp,                            quick_open_1_50_help_temp },
//Remove new dictionary references from legacy binary format
        {30,   hyperbolic_factor_0_10_temp,                     hyperbolic_factor_0_10_help_temp },
//Remove new dictionary references from legacy binary format
        {31,   hyperbolic_factor_0_20_temp,                     hyperbolic_factor_0_20_help_temp },
//Remove new dictionary references from legacy binary format
        {32,   hyperbolic_factor_0_30_temp,                     hyperbolic_factor_0_30_help_temp },
//Remove new dictionary references from legacy binary format
        {34,   hyperbolic_factor_0_50_temp,                     hyperbolic_factor_0_50_help_temp },
//Remove new dictionary references from legacy binary format
        {37,   hyperbolic_factor_0_70_temp,                     hyperbolic_factor_0_70_help_temp },
//Remove new dictionary references from legacy binary format
        {40,   hyperbolic_factor_1_00_temp,                     hyperbolic_factor_1_00_help_temp },
//Remove new dictionary references from legacy binary format
        {41,   hyperbolic_factor_1_50_temp,                     hyperbolic_factor_1_50_help_temp },
//Remove new dictionary references from legacy binary format
        {42,   hyperbolic_factor_2_00_temp,                     hyperbolic_factor_2_00_help_temp },
//Remove new dictionary references from legacy binary format
        {43,   hyperbolic_factor_3_00_temp,                     hyperbolic_factor_3_00_help_temp },
//Remove new dictionary references from legacy binary format
        {44,   hyperbolic_factor_4_00_temp,                     hyperbolic_factor_4_00_help_temp },
//Remove new dictionary references from legacy binary format
        {45,   hyperbolic_factor_5_00_temp,                     hyperbolic_factor_5_00_help_temp },
//Remove new dictionary references from legacy binary format
        {100,  flat_bottom_tank_temp,                           flat_bottom_tank_help_temp },
//Remove new dictionary references from legacy binary format
        {101,  conical_bottom_tank_temp,                        conical_bottom_tank_help_temp },
//Remove new dictionary references from legacy binary format
        {102,  parabolic_bottom_tank_temp,                      parabolic_bottom_tank_help_temp },
//Remove new dictionary references from legacy binary format
        {103,  spherical_bottom_tank_temp,                      spherical_bottom_tank_help_temp },
//Remove new dictionary references from legacy binary format
        {104,  angled_bottom_tank_temp,                         angled_bottom_tank_help_temp },
//Remove new dictionary references from legacy binary format
        {105,  flat_end_cylinder_tank_temp,                     flat_end_cylinder_tank_help_temp },
//Remove new dictionary references from legacy binary format
        {106,  parabolic_end_cylinder_tank_temp,                parabolic_end_cylinder_tank_help_temp },
//Remove new dictionary references from legacy binary format
        {107,  spherical_tank_temp,                             spherical_tank_help_temp },
         /* APPHELP "Equation y=x^2" */
//Remove new dictionary references from legacy binary format
        { 230, Discrete_Switch_temp,                            [discrete_switch_help] },  /* Discrete (switch) */
//Remove new dictionary references from legacy binary format
        { 231, Square_Root_plus_Special_Curve_temp,             [square_root_plus_special_curve_help] },
         /* APPHELP "THE FCG RECOMMENDS THAT ENUMERATION 231 NOT BE"
                "USED IN NEW IMPLEMENTATIONS !" */
//Remove new dictionary references from legacy binary format
        { 232, square_root_third_power_plus_special_Curve_temp, [square_root_third_power_plus_special_curve_help] },
         /* APPHELP "THE FCG RECOMMENDS THAT ENUMERATION 232 NOT BE"
                "USED IN NEW IMPLEMENTATIONS !" */
//Remove new dictionary references from legacy binary format
        { 233, square_root_fifth_power_plus_special_curve_temp, [square_root_fifth_power_plus_special_curve_help] },
         /* APPHELP "THE FCG RECOMMENDS THAT ENUMERATION 233 NOT BE"
                "USED IN NEW IMPLEMENTATIONS !" */
/*   240  Note- Enumeration may be used for manufacturer specific definitions */
/*   241  Note- Enumeration may be used for manufacturer specific definitions */
/*   242  Note- Enumeration may be used for manufacturer specific definitions */
/*   243  Note- Enumeration may be used for manufacturer specific definitions */
/*   244  Note- Enumeration may be used for manufacturer specific definitions */
/*   245  Note- Enumeration may be used for manufacturer specific definitions */
/*   246  Note- Enumeration may be used for manufacturer specific definitions */
/*   247  Note- Enumeration may be used for manufacturer specific definitions */
/*   248  Note- Enumeration may be used for manufacturer specific definitions */
/*   249  Note- Enumeration may be used for manufacturer specific definitions */
//Remove new dictionary references from legacy binary format
        { 250, Not_Used_temp },
//Remove new dictionary references from legacy binary format
        { 251, none_temp },
//Remove new dictionary references from legacy binary format
        { 252, unknown_temp },
//Remove new dictionary references from legacy binary format
        { 253, Special_temp }
    }
}

/*************************************************************************************
 * Table 4. Material Codes
 */
VARIABLE material_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        /* Design Note: The materials in this table should be
           solids at 72 Degrees Fahrenheit. */
        /* Design Note: Leave 240-249 Undefined for future use. */
        /* Design Note: Any enumerations in the range from 250-255
           are 'Reserved' for a single definition if not defined. */
        /* Design Note: Any enumeration not covered by another "Design
           Note" is 'Undefined'. */
        /* Note:
           Ryton is a registered trademark of Phillips Petroleum Company.
           Kynar is a trademark of Pennwalt Incorporated.
           Hastelloy C is a trademark of Cabot Corporation.
           Teflon and Kalrez are registered trademarks of E. I. DuPont De
           Nemours Company.
           Inconel is a trademark of International Nickel Company. */

//Remove new dictionary references from legacy binary format
        { 0,   carbon_steel_temp,               [carbon_steel_help] },
//Remove new dictionary references from legacy binary format
        { 1,   stainless_steel_304_temp,        [stainless_steel_304_help] },
//Remove new dictionary references from legacy binary format
        { 2,   stainless_steel_316_temp,        [stainless_steel_316_help] },
//Remove new dictionary references from legacy binary format
        { 3,   Alloy_C22_temp,                  Alloy_C22_help_temp },
//Remove new dictionary references from legacy binary format
        { 4,   Nickel_Copper_Alloy_temp,        Nickel_Copper_Alloy_help_temp },
//Remove new dictionary references from legacy binary format
        { 5,   tantalum_temp,                   [tantalum_help] },
//Remove new dictionary references from legacy binary format
        { 6,   titanium_temp,                   [titanium_help] },
//Remove new dictionary references from legacy binary format
        { 7,   pt_ir_temp,                      [platinum_iridium_help] },
//Remove new dictionary references from legacy binary format
        { 8,   alloy_20_temp,                   [alloy_20_help] },
//Remove new dictionary references from legacy binary format
        { 9,   co_cr_ni_temp,                   [cobalt_chromium_nickel_help] },
//Remove new dictionary references from legacy binary format
        { 10,  ptfe_temp,                       PTFE_help_temp },
//Remove new dictionary references from legacy binary format
        { 11,  vito_temp,                       viton_help_temp },
//Remove new dictionary references from legacy binary format
        { 12,  buna_n_temp,                     buna_n_help_temp },
//Remove new dictionary references from legacy binary format
        { 13,  ethyl_prop_temp,                 [ethyl_prop_help] },
//Remove new dictionary references from legacy binary format
        { 14,  urethane_temp,                   [urethane_help] },
//Remove new dictionary references from legacy binary format
        { 15,  gold_monel_temp,                 gold_alloy_400_help_temp },
//Remove new dictionary references from legacy binary format
        { 16,  tefzel_temp,                     tefzel_help_temp },
//Remove new dictionary references from legacy binary format
        { 17,  PPS_temp,                        PPS_help_temp },
//Remove new dictionary references from legacy binary format
        { 18,  ceramic_temp,                    [ceramic_help] },
//Remove new dictionary references from legacy binary format
        { 19,  stainless_steel_316l_sup_temp,   [stainless_steel_316l_help] },
//Remove new dictionary references from legacy binary format
        { 20,  pvc_temp,                        pvc_help_temp },
//Remove new dictionary references from legacy binary format
        { 21,  nitrile_rubber_temp,             [nitrile_rubber_help] },
//Remove new dictionary references from legacy binary format
        { 22,  FFKM_temp,                       FFKM_help_temp },
//Remove new dictionary references from legacy binary format
        { 23,  Nickel_Chromium_Alloy_temp,      Nickel_Chromium_Alloy_help_temp },
//Remove new dictionary references from legacy binary format
        { 24,  PVDF_temp,                       PVDF_help_temp },
//Remove new dictionary references from legacy binary format
        { 25,  aluminium_temp,                  [aluminum_help] },
//Remove new dictionary references from legacy binary format
        { 26,  nickel_temp,                     [nickel_help] },
//Remove new dictionary references from legacy binary format
        { 27,  fep_temp,                        fep_help_temp },
         /* APPHELP "Typically a sealing material for O-Ring" */
//Remove new dictionary references from legacy binary format
        { 28,  stainless_steel_316_ti_temp,     [stainless_steel_316_ti_help] },
//Remove new dictionary references from legacy binary format
        { 30,  hastelloy_c276_temp,             alloy_c_276_help_temp },
//Remove new dictionary references from legacy binary format
        { 31,  klinger_c4401_temp,              [klinger_c4401_help] },
//Remove new dictionary references from legacy binary format
        { 32,  thermotork_temp,                 [thermotork_help] },
//Remove new dictionary references from legacy binary format
        { 33,  grafoil_temp,                    [grafoil_help] },
//Remove new dictionary references from legacy binary format
        { 34,  ptfe_coated_316l_sst_temp,       PTFE_coated_sst_help_temp },
//Remove new dictionary references from legacy binary format
        { 35,  gold_plated_hastelloy_c276_temp, gold_plated_alloy_c276_help_temp },
//Remove new dictionary references from legacy binary format
        { 36,  ptfe_glass_temp,                 PTFE_glass_help_temp },
//Remove new dictionary references from legacy binary format
        { 37,  ptfe_graphite_temp,              PTFE_graphite_help_temp },
//Remove new dictionary references from legacy binary format
        { 38,  aflas_temp,                      aflas_help_temp }, /* Aflas */
//Remove new dictionary references from legacy binary format
        { 234, PTFE_C276_temp,                  PTFE_C276_help_temp },
//Remove new dictionary references from legacy binary format
        { 235, stainless_steel_CF_8M_temp,      [stainless_steel_CF_8M_help] },
//Remove new dictionary references from legacy binary format
        { 236, hastelloy_sst_temp,              alloy_c276_sst_help_temp },
//Remove new dictionary references from legacy binary format
        { 237, gold_plated_sst_temp,            [gold_plated_sst_help] },
//Remove new dictionary references from legacy binary format
        { 239, monel_400_temp,                  alloy_400_help_temp },
//Remove new dictionary references from legacy binary format
        { 250, Not_Used_temp },
//Remove new dictionary references from legacy binary format
        { 251, none_temp },
//Remove new dictionary references from legacy binary format
        { 252, unknown_temp },
//Remove new dictionary references from legacy binary format
        { 253, Special_temp }
    }
}

/*************************************************************************************
 * Table 5. NULL
 */


/*************************************************************************************
 * Table 6. Alarm Selection Codes
 */
VARIABLE alarm_selection_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        /* Design Note: These apply to the alarm state of the
           physical output of an Analog Output */
        /* Design Note: Leave 240-249 Undefined for future use. */
        /* Design Note: Any enumerations in the range from 250-255
           are 'Reserved' for a single definition if not defined. */
        /* Design Note: Any enumeration not covered by another "Design
           Note" is 'Undefined'. */

//Remove new dictionary references from legacy binary format
        { 0,   High_temp,                   High_help_temp },
//Remove new dictionary references from legacy binary format
        { 1,   Low_temp,                    Low_help_temp },
//Remove new dictionary references from legacy binary format
        { 239, Hold_Last_Output_Value_temp, Hold_Last_Output_Value_help_temp },
//Remove new dictionary references from legacy binary format
        { 250, Not_Used_temp },
//Remove new dictionary references from legacy binary format
        { 251, none_temp },
//Remove new dictionary references from legacy binary format
        { 252, unknown_temp },
//Remove new dictionary references from legacy binary format
        { 253, Special_temp }
    }
}

/*************************************************************************************
 * Table 7. Write Protect Codes
 */
VARIABLE write_protect_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        /* Design Note: Leave 240-249 Undefined for future use. */
        /* Design Note: Any enumerations in the range from 250-255
           are 'Reserved' for a single definition if not defined. */
        /* Design Note: Any enumeration not covered by another "Design
           Note" is 'Undefined'. */

//Remove new dictionary references from legacy binary format
        { 0,   no_temp,         not_write_protected_help_temp },
//Remove new dictionary references from legacy binary format
        { 1,   yes_temp,        write_protected_help_temp },

//Remove new dictionary references from legacy binary format
        { 250, Not_Used_temp },
//Remove new dictionary references from legacy binary format
        { 251, none_temp },
//Remove new dictionary references from legacy binary format
        { 252, unknown_temp },
//Remove new dictionary references from legacy binary format
        { 253, Special_temp }
    }
}

/*************************************************************************************
 * Table 9. Burst Mode Control Codes
 */
VARIABLE burst_mode_control_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        /* Design Note: The [on] labels for multiple enumerations are the same because 99%
                        of devices only implement one of the communication interfaces. This
                        allows end users to simply choose "On" or "Off" for the configuration.
                        In the rare event a field device supports multiple communication
                        channels, the HART standard dictionaries have more descriptive labels
                        that can be used (e.g. in a redefinition).  They are [on_token],
                        [on_tdma], and [on_hart_ip]. */

//Remove new dictionary references from legacy binary format
        { 0, off_temp,                burst_off_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, on_temp,                 burst_on_token_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, on_temp,                 burst_on_tdma_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, on_token_plus_tdma_temp, burst_on_token_plus_tdma_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, on_temp,                 on_hart_ip_help_temp }

      /*{ 250, [reserved] },
        { 251, [reserved] },
        { 252, [reserved] },
        { 253, [reserved] }*/
    }
}

/*************************************************************************************
 * Table 10. Physical Signaling Codes
 */
VARIABLE physical_signaling_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        /* Design Note: Enumeration 7 is 'Reserved',
           potentially for an expansion indication. */
        /* Design Note: Any enumerations in the range from 8-255
           are not allowable. */

//Remove new dictionary references from legacy binary format
        { 0, Bell_202_Current_temp, bell_202_current_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, Bell_202_Voltage_temp, bell_202_voltage_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, rs_485_temp,           rs_485_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, rs_232_temp,           RS_232_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, wireless_temp,         wireless_help_temp },
//Remove new dictionary references from legacy binary format
        { 6, Special_temp }

//Remove new dictionary references from legacy binary format
      /*{ 7, reserved_temp }*/
    }
}

/*************************************************************************************
 * Table 11. Flag Assignments
 */
VARIABLE std_flag_assignment
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, Multi_Sensor_Fld_Dev_temp,      multi_sensor_fld_dev_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x02, eeprom_ctrl_temp,               eeprom_ctrl_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x04, Protocol_Bridge_Device_temp,    protocol_bridge_device_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x08, ieee_802_15_4_temp,             ieee_802_15_4_help_temp },

//Remove new dictionary references from legacy binary format
        { 0x40, PSK_Capable_Device_temp,        psk_capable_device_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x80, PSK_Loop_Current_Disabled_temp, psk_loop_current_disabled_help_temp  }
    }
}

/*************************************************************************************
 * Table 12. Transfer Service Function Codes
 * (see Block Data Tranfer Specification)
 */

/*************************************************************************************
 * Table 13. Transfer Service Identifier Codes
 * (see Block Data Tranfer Specification)
 */

/*************************************************************************************
 * Table 14. Operating Mode Codes
 * (This Table is Reserved.  Field Devices must always return 0x00)
 */
VARIABLE std_operating_mode_code
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, reserved_temp },
//Remove new dictionary references from legacy binary format
        { 0x02, reserved_temp },
//Remove new dictionary references from legacy binary format
        { 0x04, reserved_temp },
//Remove new dictionary references from legacy binary format
        { 0x08, reserved_temp },
//Remove new dictionary references from legacy binary format
        { 0x10, reserved_temp },
//Remove new dictionary references from legacy binary format
        { 0x20, reserved_temp },
//Remove new dictionary references from legacy binary format
        { 0x40, reserved_temp },
//Remove new dictionary references from legacy binary format
        { 0x80, reserved_temp }
    }
}

/*************************************************************************************
 * Table 15. Analog Output Numbers Codes
 */
VARIABLE analog_output_numbers_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, analog_channel_0_temp, analog_channel_0_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, analog_channel_1_temp, analog_channel_1_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, analog_channel_2_temp, analog_channel_2_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, analog_channel_3_temp, analog_channel_3_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, analog_channel_4_temp, analog_channel_4_help_temp }
    }
}

/*************************************************************************************
 * Table 16. Loop Current Mode Codes;
 */
VARIABLE loop_current_mode_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, disable_temp, disable_loop_current_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, enable_temp,  enable_loop_current_mode_help_temp }
    }
}

/*************************************************************************************
 * Table 17. Extended Device Status Codes
 */
VARIABLE std_extended_device_status_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, Maintenance_Required_temp,   maintenance_required_help_temp,   HARDWARE },
//Remove new dictionary references from legacy binary format
        { 0x02, Device_Variable_Alert_temp,  device_variable_alert_help_temp,  SOFTWARE },
//Remove new dictionary references from legacy binary format
        { 0x04, critical_power_failure_temp, critical_power_failure_help_temp, HARDWARE },
//Remove new dictionary references from legacy binary format
        { 0x08, failure_temp,                failure_help_temp,                MORE },
//Remove new dictionary references from legacy binary format
        { 0x10, out_of_specification_temp,   out_of_specification_help_temp,   MORE },
//Remove new dictionary references from legacy binary format
        { 0x20, function_check_temp,         function_check_help_temp,         MORE }
    }
}

/*************************************************************************************
 * Table 18. Lock Device Codes
 */
VARIABLE lock_device_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, unlock_temp,         unlock_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, Temporary_Lock_temp, temporary_lock_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, Permanent_Lock_temp, permanent_lock_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, lock_all_temp,       lock_all_help_temp }
    }
}

/*************************************************************************************
 * Table 19. Write Device Variable Codes
 */
VARIABLE write_device_variable_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, normal_device_variable_temp, normal_device_variable_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, Fix_Value_temp,              fix_value_help_temp}
    }
}

/*************************************************************************************
 * Table 20. Device Variable Family Codes
 */
VARIABLE device_variable_family_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
        /* Codes 0-3 are reserved and must never be used */

//Remove new dictionary references from legacy binary format
        { 4,  temperature_family_temp   },
//Remove new dictionary references from legacy binary format
        { 5,  pressure_family_temp      },
//Remove new dictionary references from legacy binary format
        { 6,  actuator_family_temp      },
//Remove new dictionary references from legacy binary format
        { 7,  simple_pid_family_temp    },
//Remove new dictionary references from legacy binary format
        { 8,  ph_family_temp            },
//Remove new dictionary references from legacy binary format
        { 9,  conductivity_family_temp  },
//Remove new dictionary references from legacy binary format
        { 10, totalizer_family_temp     },
//Remove new dictionary references from legacy binary format
        { 11, level_family_temp         },
//Remove new dictionary references from legacy binary format
        { 12, vortex_flow_family_temp   },
//Remove new dictionary references from legacy binary format
        { 13, mag_flow_family_temp      },
//Remove new dictionary references from legacy binary format
        { 14, coriolis_flow_family_temp },

        /* Codes 132-249 are reserved and must never be used */

//Remove new dictionary references from legacy binary format
        { 250, Not_Used_temp }
    }
}

/*************************************************************************************
 * Table 21. Device Variable Classification Codes
 */
VARIABLE device_variable_classification_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0,  Device_Variable_Not_Classified_temp          },

        /* Codes 1-63 Reserved are reserved and must never be used */

//Remove new dictionary references from legacy binary format
        { 64, temperature_classification_temp              },    /* 0x40 */
//Remove new dictionary references from legacy binary format
        { 65, pressure_classification_temp                 },    /* 0x41 */
//Remove new dictionary references from legacy binary format
        { 66, Volumetric_Flow_Classification_temp          },    /* 0x42 */
//Remove new dictionary references from legacy binary format
        { 67, velocity_classification_temp                 },    /* 0x43 */
//Remove new dictionary references from legacy binary format
        { 68, volume_classification_temp                   },    /* 0x44 */
//Remove new dictionary references from legacy binary format
        { 69, length_classification_temp                   },    /* 0x45 */
//Remove new dictionary references from legacy binary format
        { 70, time_classification_temp                     },    /* 0x46 */
//Remove new dictionary references from legacy binary format
        { 71, mass_classification_temp                     },    /* 0x47 */
//Remove new dictionary references from legacy binary format
        { 72, Mass_Flow_Classification_temp                },    /* 0x48 */
//Remove new dictionary references from legacy binary format
        { 73, Mass_Per_Volume_Classification_temp          },    /* 0x49 */
//Remove new dictionary references from legacy binary format
        { 74, viscosity_classification_temp                },    /* 0x4A */
//Remove new dictionary references from legacy binary format
        { 75, Angular_Velocity_Classification_temp         },    /* 0x4B */
//Remove new dictionary references from legacy binary format
        { 76, area_classification_temp                     },    /* 0x4C */
//Remove new dictionary references from legacy binary format
        { 77, energy_classification_temp                   },    /* 0x4D */
//Remove new dictionary references from legacy binary format
        { 78, force_classification_temp                    },    /* 0x4E */
//Remove new dictionary references from legacy binary format
        { 79, power_classification_temp                    },    /* 0x4F */
//Remove new dictionary references from legacy binary format
        { 80, frequency_classification_temp                },    /* 0x50 */
//Remove new dictionary references from legacy binary format
        { 81, analytical_classification_temp               },    /* 0x51 */
//Remove new dictionary references from legacy binary format
        { 82, capacitance_classification_temp              },    /* 0x52 */
//Remove new dictionary references from legacy binary format
        { 83, Emf_Classification_temp                      },    /* 0x53 */
//Remove new dictionary references from legacy binary format
        { 84, current_classification_temp                  },    /* 0x54 */
//Remove new dictionary references from legacy binary format
        { 85, resistance_classification_temp               },    /* 0x55 */
//Remove new dictionary references from legacy binary format
        { 86, angle_classification_temp                    },    /* 0x56 */
//Remove new dictionary references from legacy binary format
        { 87, conductance_classification_temp              },    /* 0x57 */
//Remove new dictionary references from legacy binary format
        { 88, Volume_Per_Volume_Classification_temp        },    /* 0x58 */
//Remove new dictionary references from legacy binary format
        { 89, Volume_Per_Mass_Classification_temp          },    /* 0x59 */
//Remove new dictionary references from legacy binary format
        { 90, concentration_classification_temp            },    /* 0x5A */
//Remove new dictionary references from legacy binary format
      //{ 91, reserved_temp                                },    /* 0x5B */
//Remove new dictionary references from legacy binary format
      //{ 92, reserved_temp                                },    /* 0x5C */
//Remove new dictionary references from legacy binary format
      //{ 93, reserved_temp                                },    /* 0x5D */
//Remove new dictionary references from legacy binary format
      //{ 94, reserved_temp                                },    /* 0x5E */
//Remove new dictionary references from legacy binary format
      //{ 95, reserved_temp                                },    /* 0x5F */
//Remove new dictionary references from legacy binary format
        { 96, acceleration_classification_temp             },    /* 0x60 */
//Remove new dictionary references from legacy binary format
        { 97, turbidity_classification_temp                },    /* 0x61 */
//Remove new dictionary references from legacy binary format
        { 98, temperature_diff_classification_temp         },    /* 0x62 */
//Remove new dictionary references from legacy binary format
        { 99, volumetric_gas_flow_per_second_class_temp    },    /* 0x63 */
//Remove new dictionary references from legacy binary format
        { 100,volumetric_gas_flow_per_minute_class_temp    },    /* 0x64 */
//Remove new dictionary references from legacy binary format
        { 101,volumetric_gas_flow_per_hour_class_temp      },    /* 0x65 */
//Remove new dictionary references from legacy binary format
        { 102,volumetric_gas_flow_per_day_class_temp       },    /* 0x66 */
//Remove new dictionary references from legacy binary format
        { 103,volumetric_liquid_flow_per_second_class_temp },    /* 0x67 */
//Remove new dictionary references from legacy binary format
        { 104,volumetric_liquid_flow_per_minute_class_temp },    /* 0x68 */
//Remove new dictionary references from legacy binary format
        { 105,volumetric_liquid_flow_per_hour_class_temp   },    /* 0x69 */
//Remove new dictionary references from legacy binary format
        { 106,volumetric_liquid_flow_per_day_class_temp    },    /* 0x6A */
//Remove new dictionary references from legacy binary format
        { 107,thermal_expansion_classification_temp        },    /* 0x6B */
//Remove new dictionary references from legacy binary format
        { 108,volumetric_energy_density_class_temp         },    /* 0x6C */
//Remove new dictionary references from legacy binary format
        { 109,mass_energy_density_class_temp               },    /* 0x6D */
//Remove new dictionary references from legacy binary format
        { 110,torque_classification_temp                   },    /* 0x6E */
//Remove new dictionary references from legacy binary format
        { 111,miscellaneous_classification_temp            },    /* 0x6F */
//Remove new dictionary references from legacy binary format
        { 112,tortional_stiffness_temp                     },    /* 0x70 */
//Remove new dictionary references from legacy binary format
        { 113,linear_stiffness_temp                        }     /* 0x71 */
    }
}

/*************************************************************************************
 * Table 22. Trim Point Codes
 */
VARIABLE trim_point_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, trim_point_not_supported_temp,             trim_point_not_supported_help_temp  },
//Remove new dictionary references from legacy binary format
        { 1, Lower_Trim_Point_Supported_temp,           lower_trim_point_supported_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, Upper_Trim_Point_Supported_temp,           upper_trim_point_supported_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, Lower_and_Upper_Trim_Point_Supported_temp, lower_and_upper_trim_point_supported_help_temp }
    }
}

/*************************************************************************************
 * Table 23. Capture Mode Codes
 */
VARIABLE capture_mode_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, disable_temp,               disable_capture_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, enable_capture_device_temp, enable_capture_device_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, enable_capture_back_temp,   enable_capture_back_help_temp }
    }
}

/*************************************************************************************
 * Table 24. Physical Layer Type Codes
 */
VARIABLE physical_layer_type_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, asynchronous_fsk_temp, asynchronous_fsk_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, synchronous_psk_temp,  synchronous_psk_help_temp }

        /* 3 is reserved */
    }
}

/*************************************************************************************
 * Table 25. Lock Device Status Codes
 */
VARIABLE std_lock_device_status_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, Device_Locked_temp,        device_locked_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x02, lock_permanent_temp,       lock_permanent_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x04, lock_primary_master_temp,  lock_primary_master_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x08, configuration_locked_temp, configuration_locked_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x10, lock_gateway_temp,         lock_gateway_help_temp }
    }
}

/*************************************************************************************
 * Table 26. Analog Channel Flags
 */
VARIABLE std_analog_channel_flags
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, analog_input_channel_temp, analog_input_channel_help_temp }
    }
}

/*************************************************************************************
 * Table 27. Analog Channel Saturated Codes
 *
 * this variable is an extension of Bit #2 of the Device Status byte and
 * indicate any additional auxilliary analog channels that are Saturated
 */

VARIABLE std_analog_channel_saturated_codes    // Refer to table 15, 28
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, secondary_analog_channel_saturated_temp,  secondary_analog_channel_saturated_help_temp,  AO2(AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x02, tertiary_analog_channel_saturated_temp ,  tertiary_analog_channel_saturated_help_temp,   AO3(AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x04, quaternary_analog_channel_saturated_temp, quaternary_analog_channel_saturated_help_temp, AO4(AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x08, quinary_analog_channel_saturated_temp,    quinary_analog_channel_saturated_help_temp,    AO5(AUTO & BAD) }
    }
}

/*************************************************************************************
 * Table 28. Analog Channel Fixed Codes
 *
 * this variable is an extension of Bit #3 of the Device Status byte and
 * indicate any additional auxilliary analog channels that are fixed
 */

VARIABLE std_analog_channel_fixed_codes        // Refer to table 15, 27
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, secondary_analog_channel_fixed_temp,  secondary_analog_channel_fixed_help_temp,  AO2(AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x02, tertiary_analog_channel_fixed_temp,   tertiary_analog_channel_fixed_help_temp,   AO3(AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x04, quaternary_analog_channel_fixed_temp, quaternary_analog_channel_fixed_help_temp, AO4(AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x08, quinary_analog_channel_fixed_temp,    quinary_analog_channel_fixed_help_temp,    AO5(AUTO & BAD) }
    }
}


/*************************************************************************************
 * Table 29. Standardized Status 0 Codes
 */
VARIABLE std_standardized_status_0_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, dev_var_simulation_active_temp,   dev_var_simulation_active_help_temp,   MODE },
//Remove new dictionary references from legacy binary format
        { 0x02, non_volatile_memory_defect_temp,  non_volatile_memory_defect_help_temp,  HARDWARE },
//Remove new dictionary references from legacy binary format
        { 0x04, volatile_memory_defect_temp,      volatile_memory_defect_help_temp,      HARDWARE },
//Remove new dictionary references from legacy binary format
        { 0x08, watchdog_reset_executed_temp,     watchdog_reset_executed_help_temp,     HARDWARE & SOFTWARE },
//Remove new dictionary references from legacy binary format
        { 0x10, power_supply_out_of_range_temp,   power_supply_out_of_range_help_temp,   HARDWARE },
//Remove new dictionary references from legacy binary format
        { 0x20, environmental_out_of_range_temp,  environmental_out_of_range_help_temp,  MISC },
//Remove new dictionary references from legacy binary format
        { 0x40, electronic_defect_temp,           electronic_defect_help_temp,           HARDWARE },
//Remove new dictionary references from legacy binary format
        { 0x80, device_configuration_locked_temp, device_configuration_locked_help_temp, MODE }
    }
}

/*************************************************************************************
 * Table 30. Standardized Status 1 Codes
 */
VARIABLE std_standardized_status_1_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, status_simulation_active_temp,            status_simulation_active_help_temp,            MODE },
//Remove new dictionary references from legacy binary format
        { 0x02, discrete_variable_simulation_active_temp, discrete_variable_simulation_active_help_temp, MODE },
//Remove new dictionary references from legacy binary format
        { 0x04, event_notification_overflow_temp,         event_notification_overflow_help_temp,         MISC },
//Remove new dictionary references from legacy binary format
        { 0x08, power_supply_needs_maintenance_temp,      power_supply_needs_maintenance_help_temp,      HARDWARE },
//Remove new dictionary references from legacy binary format
        { 0x10, device_in_secure_C8PSK_mode_temp,         device_in_secure_C8PSK_mode_help_temp,         MODE }
    }
}

/*************************************************************************************
 * Table 31. Standardized Status 2 Codes
 */
VARIABLE std_standardized_status_2_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, sub_device_list_changed_temp,           sub_device_list_changed_help_temp,           PROCESS },
//Remove new dictionary references from legacy binary format
        { 0x02, duplicate_master_detected_temp,         duplicate_master_detected_help_temp,         MISC },
//Remove new dictionary references from legacy binary format
        { 0x04, sub_device_mismatch_temp,               sub_device_mismatch_help_temp,               PROCESS },
//Remove new dictionary references from legacy binary format
        { 0x08, sub_devices_w_duplicate_ids_found_temp, sub_devices_w_duplicate_ids_found_help_temp, MISC },
//Remove new dictionary references from legacy binary format
        { 0x10, stale_data_notice_temp,                 stale_data_notice_help_temp,                 MISC }
    }
}

/*************************************************************************************
 * Table 32. Standardized Status 3 Codes
 */
VARIABLE std_standardized_status_3_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
       { 0x01, capacity_denied_temp,              capacity_denied_help_temp,              DATA },
     /*  0x02,  0x02 is reserved */
//Remove new dictionary references from legacy binary format
       { 0x04, bandwidth_allocation_pending_temp, bandwidth_allocation_pending_help_temp, MISC },
//Remove new dictionary references from legacy binary format
       { 0x08, block_transfer_pending_temp,       block_transfer_pending_help_temp,       MISC },
//Remove new dictionary references from legacy binary format
       { 0x10, radio_failure_temp,                radio_failure_help_temp,                MISC }
    }
}

/*************************************************************************************
 * Table 33. Burst Message Trigger Mode Codes
 */
VARIABLE burst_message_trigger_mode_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, continuous_temp, continuous_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, window_temp,     window_help_temp     },
//Remove new dictionary references from legacy binary format
        { 2, rising_temp,     rising_help_temp     },
//Remove new dictionary references from legacy binary format
        { 3, falling_temp,    falling_help_temp    },
//Remove new dictionary references from legacy binary format
        { 4, on_change_temp,  on_change_help_temp  }
    }
}

/*************************************************************************************
 * Table 34. Device Variable Code Codes
 *
 * Additional codes added to supplement the device's array of Device
 * Variables. Codes 246-249 should not be used in DD as they are
 * intended for non-DD host applications.
 */
VARIABLE device_variable_code_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 242, battery_voltage_code_temp,     battery_voltage_code_help_temp     },
//Remove new dictionary references from legacy binary format
        { 243, battery_life_code_temp,        battery_life_code_help_temp        },
//Remove new dictionary references from legacy binary format
        { 244, percent_range_code_temp,       percent_range_code_help_temp       },
//Remove new dictionary references from legacy binary format
        { 245, loop_current_code_temp,        loop_current_code_help_temp        },
//Remove new dictionary references from legacy binary format
        { 246, primary_variable_code_temp ,   primary_variable_code_help_temp    },
//Remove new dictionary references from legacy binary format
        { 247, secondary_variable_code_temp,  secondary_variable_code_help_temp  },
//Remove new dictionary references from legacy binary format
        { 248, tertiary_variable_code_temp,   tertiary_variable_code_help_temp   },
//Remove new dictionary references from legacy binary format
        { 249, quaternary_variable_code_temp, quaternary_variable_code_help_temp },
//Remove new dictionary references from legacy binary format
        { 250, Not_Used_temp }
    }
}

/*************************************************************************************
 * Table 35. Event Notification Control Codes
 */
VARIABLE event_nofication_control_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, off_temp,                event_off_help_temp                },
//Remove new dictionary references from legacy binary format
        { 1, on_token_temp,           event_on_token_help_temp           },
//Remove new dictionary references from legacy binary format
        { 2, on_tdma_temp,            event_on_tdma_help_temp            },
//Remove new dictionary references from legacy binary format
        { 3, on_token_plus_tdma_temp, event_on_token_plus_tdma_help_temp }

    }
}

/*************************************************************************************
 * Table 36. Event Status Codes
 */
VARIABLE std_event_status_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, configuration_changed_event_temp, configuration_changed_event_help_temp, INFO },
//Remove new dictionary references from legacy binary format
        { 0x02, device_status_event_temp,         device_status_event_help_temp,         SUMMARY },
//Remove new dictionary references from legacy binary format
        { 0x04, more_status_available_event_temp, more_status_available_event_help_temp, SUMMARY }

    }
}

/*************************************************************************************
 * Table 37. Trend Control Codes
 */
VARIABLE trend_control_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, disable_temp,               disable_trend_help_temp},
//Remove new dictionary references from legacy binary format
        { 1, enable_single_trend_temp,   enable_single_trend_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, enable_filtered_trend_temp, enable_filtered_trend_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, enable_average_trend_temp,  enable_average_trend_help_temp }
    }
}

/*************************************************************************************
 * Table 38. Time-Set Codes
 */
VARIABLE time_set_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, read_receive_time_temp,read_receive_time_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, write_date_time_temp,  write_date_time_help_temp }

    }
}


// Table 39 is used by network managers - not modeled in the DD
// Table 40 is used by network managers - not modeled in the DD

/*************************************************************************************
 * Table 41. Synchronous Action Control Flags
 */
VARIABLE synchronous_action_control_flag_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, synchronous_action_command_temp,  synchronous_action_command_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x10, synchronous_action_one_shot_temp, synchronous_action_one_shot_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x80, synchronous_action_enabled_temp,  synchronous_action_enabled_help_temp }
    }
}

/*************************************************************************************
 * Table 42. Real Time Clock Flag Codes
 */
VARIABLE std_real_time_clock_flag_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, non_volatile_clock_temp,  non_volatile_clock_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x02, clock_uninitialized_temp, clock_uninitialized_help_temp }

    }
}

// Table 43 is used by network managers - not modeled in the DD

/*************************************************************************************
 * Table 44. Device Power Source Codes
 */
VARIABLE power_source_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, line_power_temp,         line_power_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, battery_power_temp,      battery_power_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, rechargeable_power_temp, rechargeable_power_help_temp }
    }
}

// Table 45 is used by network managers - not modeled in the DD
// Table 46 is used by network managers - not modeled in the DD
// Table 47 is used by network managers - not modeled in the DD
// Table 48 is used by network managers - not modeled in the DD
// Table 49 is used by network managers - not modeled in the DD
// Table 50 is used by network managers - not modeled in the DD

/*************************************************************************************
 * Table 51. Wireless Operation Mode Codes
 */
VARIABLE wireless_operation_mode_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, idle_mode_temp,           idle_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, active_search_mode_temp,  active_search_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, negotiating_mode_temp,    negotiating_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, quarantined_mode_temp,    quarantined_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, operational_mode_temp,    operational_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 5, suspended_mode_temp,      suspended_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 6, passive_search_mode_temp, passive_search_mode_help_temp }
    }
}

/*************************************************************************************
 * Table 52. Join Process Status Codes
 */
VARIABLE std_join_process_status_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED (2)
    {
//Remove new dictionary references from legacy binary format
        { 0x0001, wireless_network_found_temp,       wireless_network_found_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0002, network_time_captured_temp,        network_time_captured_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0004, wireless_timing_synchronized_temp, wireless_timing_synchronized_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0008, advertisement_received_temp,       advertisement_received_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0010, join_requested_temp,               join_requested_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0020, join_retrying_temp,                join_retrying_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0040, join_failed_temp,                  join_failed_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0080, network_authenticated_temp,        network_authenticated_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0100, network_joined_temp,               network_joined_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0200, negotiating_network_temp,          negotiating_network_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x0400, join_complete_temp,                join_complete_help_temp }
   }
}

// Table 53 is used by network managers - not modeled in the DD

/*************************************************************************************
 * Table 54. SI Control Codes
 */
VARIABLE si_control_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, no_temp,  no_restrictions_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, yes_temp, si_units_only_help_temp }

    }
}

// Table 55 is used by IO systems - not modeled in the DD

/*************************************************************************************
 * Table 56. Network Access Mode Codes
 */
VARIABLE network_access_mode_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, open_nework_temp,                     open_nework_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, whitelist_nework_temp,                whitelist_nework_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, blacklist_nework_temp,                blacklist_nework_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, whitelist_plus_blacklist_nework_temp, whitelist_plus_blacklist_nework_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, lockdown_nework_temp,                 lockdown_nework_help_temp },
//Remove new dictionary references from legacy binary format
        { 5, quarantine_nework_temp,               quarantine_nework_help_temp }
    }
}

/*************************************************************************************
 * Table 57. Device Profile Codes
 */
VARIABLE device_profile_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 1,   process_automation_device_temp,                     process_automation_device_help_temp },
//Remove new dictionary references from legacy binary format
        { 2,   discrete_device_temp,                               discrete_device_help_temp  },
//Remove new dictionary references from legacy binary format
        { 3,   process_automation_plus_discrete_temp,              process_automation_plus_discrete_help_temp },
//Remove new dictionary references from legacy binary format
        { 4,   io_system_temp,                                     io_system_help_temp },
//Remove new dictionary references from legacy binary format
        {14,   discrete_adapter_temp,                              discrete_adapter_help_temp },
//Remove new dictionary references from legacy binary format
        {65,   hart_ip_process_automation_device_temp,             hart_ip_process_automation_device_help_temp },
//Remove new dictionary references from legacy binary format
        {66,   hart_ip_discrete_device_temp,                       hart_ip_discrete_device_help_temp },
//Remove new dictionary references from legacy binary format
        {67,   hart_ip_process_automation_plus_discrete_temp,      hart_ip_process_automation_plus_discrete_help_temp },
//Remove new dictionary references from legacy binary format
        {68,   hart_ip_io_system_temp,                             hart_ip_io_system_help_temp },
//Remove new dictionary references from legacy binary format
        { 129, wirelessHART_process_automation_device_temp,        wirelessHART_process_automation_device_help_temp },
//Remove new dictionary references from legacy binary format
        { 130, wirelessHART_discrete_device_temp,                  wirelessHART_discrete_device_help_temp },
//Remove new dictionary references from legacy binary format
        { 131, wirelessHART_process_automation_plus_discrete_temp, wirelessHART_process_automation_plus_discrete_help_temp },
//Remove new dictionary references from legacy binary format
        { 132, wirelessHART_gateway_temp,                          wirelessHART_gateway_help_temp },
//Remove new dictionary references from legacy binary format
        { 140, wirelessHART_access_point_temp,                     wirelessHART_access_point_help_temp },
//Remove new dictionary references from legacy binary format
        { 141, wirelessHART_process_adapter_temp,                  wirelessHART_process_adapter_help_temp },
//Remove new dictionary references from legacy binary format
        { 142, wirelessHART_discrete_adapter_temp,                 wirelessHART_discrete_adapter_help_temp },
//Remove new dictionary references from legacy binary format
        { 144, wirelessHART_handheld_tool_temp,                    wirelessHART_handheld_tool_help_temp }
    }
}

/*************************************************************************************
 * Table 58. Device Power Status Codes
 */
VARIABLE device_power_status_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, nominal_power_temp,         nominal_power_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, low_power_temp,             low_power_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, critically_low_power_temp,  critically_low_power_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, recharging_Low_power_temp,  recharging_Low_power_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, recharging_high_power_temp, recharging_high_power_help_temp }
    }
}

// Table 59 is used by network managers - not modeled in the DD
// Table 60 is used with IO systems - not modeled in the DD

/*************************************************************************************
 * Table 61. Join Mode Codes
 */
VARIABLE join_mode_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, no_join_temp,        no_join_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, join_now_temp,       join_now_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, attemp_to_join_temp, attemp_to_join_help_temp  }
    }
}

// Table 62 is used by network managers - not modeled in the DD
// Table 63 is used by network managers - not modeled in the DD
// Table 64 is used by network managers - not modeled in the DD

/*************************************************************************************
 * Table 65. Device Variable Property Codes
 */
VARIABLE dev_var_property_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, variable_is_input_temp,       variable_is_input_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x80, variable_simulated_temp,      variable_simulated_help_temp }
    }
}

/*************************************************************************************
 * Table 66. Squawk Control Codes
 */
VARIABLE squawk_control_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, off_temp,         Squawk_off_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, on_temp,          Squawk_on_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, squawk_once_temp, Squawk_once_help_temp }
    }
}

/*************************************************************************************
 * Table 67. Event Manager Registration Control Codes
 */
VARIABLE event_manager_registration_control_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, event_mgr_register_temp,   event_mgr_register_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, event_mgr_deregister_temp, event_mgr_deregister_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, event_mgr_reset_temp,      event_mgr_reset_help_temp }
    }
}

/*************************************************************************************
 * Table 68. Event Manager Registration Status Codes
 */
VARIABLE event_manager_registration_status_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, event_mgr_stat_register_temp, event_mgr_stat_register_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x02, event_mgr_stat_client_temp,   event_mgr_stat_client_help_temp }
    }
}

/*************************************************************************************
 * Table 69. Location Method Codes
 */
VARIABLE location_method_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, no_fix_loc_temp,              no_fix_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, gps_loc_temp,                 gps_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, differential_gps_loc_temp,    differential_gps_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, pps_loc_temp,                 pps_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, rtk_fixed_loc_temp,           rtk_fixed_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 5, rtk_float_loc_temp,           rtk_float_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 6, estd_dead_reckoning_loc_temp, estd_dead_reckoning_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 7, manual_input_mode_loc_temp,   manual_input_mode_loc_help_temp },
//Remove new dictionary references from legacy binary format
        { 8, simulation_mode_loc_temp,     simulation_mode_loc_help_temp }
    }
}

/*************************************************************************************
 * Table 70. Condensed Status Mapping Codes
 */
VARIABLE statusmap_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 6, not_defined_temp,          not_defined_help_temp },
//Remove new dictionary references from legacy binary format
        { 0, no_effect_temp,            no_effect_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, Maintenance_Required_temp, maintenance_required_help_temp },
//Remove new dictionary references from legacy binary format
        { 4, out_of_specification_temp, out_of_specification_help_temp },
//Remove new dictionary references from legacy binary format
        { 5, function_check_temp,       function_check_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, failure_temp,              failure_help_temp }

        /* Design Note: Ordering of enumerations is based on priority order (least to most severe).
           Code 2 is reserved.
           Leave 7-15 Undefined for future use. */
    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE 6;        //refer to common table A.1, default mapping
#endif //__TOKVER__ >=  600
}

/*************************************************************************************
 * Table 71. Status Simulation Mode Codes
 */
VARIABLE status_simulate_mode_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, disable_temp, disable_status_simulation_mode_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, enable_temp,  enable_status_simulation_mode_help_temp }
    }
}

/*************************************************************************************
 * Table 72. Simulated Value Codes
 */
VARIABLE status_simulate_value_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, simulated_reset_temp, simulated_reset_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, simulated_set_temp, simulated_set_help_temp }
    }
}

/*************************************************************************************
 * Table 73. Sub-device Assignment Status Codes
 */
VARIABLE subdevice_assignment_status_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, device_not_found_unique_id_temp, device_not_found_unique_id_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x02, device_not_found_long_tag_temp,  device_not_found_long_tag_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x04, older_device_revision_temp,      older_device_revision_help_temp },
//Remove new dictionary references from legacy binary format
        { 0x08, newer_device_revision_temp,      newer_device_revision_help_temp }
    }
}

/*************************************************************************************
 * Table 74. Sub-device Assignment Transfer Codes
 */
VARIABLE subdevice_assignment_transfer_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, transfer_all_device_identity_temp,  transfer_all_device_identity_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, transfer_all_except_device_id_temp, transfer_all_except_device_id_help_temp }
    }
}

/*************************************************************************************
 * Table 75. Wireless Capability Flag Codes
 */
VARIABLE wireless_capability_flag_codes
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x08, saturating_counters_temp, saturating_counters_help_temp }
    }
}

// Table 76 is used by wireless gateways - not modeled in the DD
// Table 77 is used by wireless gateways - not modeled in the DD
// Table 78 is used by wireless gateways - not modeled in the DD
// Table 79 is used by wireless gateways - not modeled in the DD
// Table 80 is used by wireless gateways - not modeled in the DD
// Table 81 is used by wireless gateways - not modeled in the DD

/*************************************************************************************
 * Process Data Quality
 * not a table but probably should be one!
 */
VARIABLE process_data_status
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, bad_temp,           bad_status_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, Poor_Accuracy_temp, poor_accuracy_status_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, Manual_Fixed_temp,  manual_fixed_status_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, good_temp,          good_status_help_temp }
    }
}

/*************************************************************************************
 * Limit Status
 * not a table but probably should be one!
 */
VARIABLE limit_status
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, Not_Limited_temp,  not_limited_status_help_temp},
//Remove new dictionary references from legacy binary format
        { 1, Low_Limited_temp,  low_limited_status_help_temp },
//Remove new dictionary references from legacy binary format
        { 2, High_Limited_temp, high_limited_status_help_temp },
//Remove new dictionary references from legacy binary format
        { 3, constant_temp,     constant_status_help_temp }
    }
}

/*************************************************************************************
 * Device Family Status
 */
VARIABLE std_device_family_status
{
    CLASS LOCAL;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x08, more_device_family_status_available_temp, more_device_family_status_available_help_temp, MORE }
    }
}

/*************************************************************************************
 * Country codes and numbers from ISO-3166 defined at
 * http://www.davros.org/misc/iso3166.txt
 * The value for the country code is in ASCII
 * ie, #define US 0x5553, 0x55 is ASCII code for “U”, and 0x53 is
 * ASCII code for “S”
 */
#define AF 0x4146 // Afghanistan
#define AL 0x414C // Albania, People's Socialist Republic of
#define DZ 0X445A // Algeria, People's Democratic Republic of
#define AS 0x4153 // American Samoa
#define AD 0x4144 // Andorra, Principality of
#define AO 0x414f // Angola, Republic of
#define AI 0x4149 // Anguilla
#define AQ 0x4151 // Antarctica (the territory South of 60 deg S)
#define AG 0x4147 // Antigua and Barbuda
#define AR 0x4152 // Argentina, Argentine Republic
#define AM 0x414D // Armenia
#define AW 0x4157 // Aruba
#define AU 0x4155 // Australia, Commonwealth of
#define AT 0x4154 // Austria, Republic of
#define AZ 0x415A // Azerbaijan, Republic of
#define BS 0x4253 // Bahamas, Commonwealth of the
#define BH 0x4248 // Bahrain, Kingdom of
#define BD 0x4244 // Bangladesh, People's Republic of
#define BB 0x4242 // Barbados
#define BY 0x4259 // Belarus
#define BE 0x4245 // Belgium, Kingdom of
#define BZ 0x425A // Belize
#define BJ 0x424A // Benin, People's Republic of
#define BM 0x424D // Bermuda
#define BT 0x4254 // Bhutan, Kingdom of
#define BO 0x424F // Bolivia, Republic of
#define BA 0x4241 // Bosnia and Herzegovina
#define BW 0x4257 // Botswana, Republic of
#define BV 0x4256 // Bouvet Island (Bouvetoya)
#define BR 0x4252 // Brazil, Federative Republic of
#define IO 0x494F // British Indian Ocean Territory (Chagos Archipelago)
#define VG 0x5647 // British Virgin Islands
#define BN 0x424E // Brunei Darussalam
#define BG 0x4247 // Bulgaria, People's Republic of
#define BF 0x4246 // Burkina Faso
#define BI 0x4249 // Burundi, Republic of
#define KH 0x4B48 // Cambodia, Kingdom of
#define CM 0x434D // Cameroon, United Republic of
#define CA 0x4341 // Canada
#define CV 0x4356 // Cape Verde, Republic of
#define KY 0x4B59 // Cayman Islands
#define CF 0x4346 // Central African Republic
#define TD 0x5444 // Chad, Republic of
#define CL 0x434C // Chile, Republic of
#define CN 0x434E // China, People's Republic of
#define CX 0x4358 // Christmas Island
#define CC 0x4343 // Cocos (Keeling) Islands
#define CO 0x434F // Colombia, Republic of
#define KM 0x4B4D // Comoros, Union of the
#define CD 0x4344 // Congo, Democratic Republic of
#define CG 0x4347 // Congo, People's Republic of
#define CK 0x434B // Cook Islands
#define CR 0x4352 // Costa Rica, Republic of
#define CI 0x4349 // Cote D'Ivoire, Ivory Coast, Republic of the
#define CU 0x4355 // Cuba, Republic of
#define CY 0x4359 // Cyprus, Republic of
#define CZ 0x435A // Czech Republic
#define DK 0x444B // Denmark, Kingdom of
#define DJ 0x444A // Djibouti, Republic of
#define DM 0x444D // Dominica, Commonwealth of
#define DO 0x444F // Dominican Republic
#define EC 0x4543 // Ecuador, Republic of
#define EG 0x4547 // Egypt, Arab Republic of
#define/*SV*/ SLV 0x5356 // El Salvador, Republic of (SLV used because #define SV is already used as Secondary)
#define GQ 0x4751 // Equatorial Guinea, Republic of
#define ER 0x4552 // Eritrea
#define EE 0x4545 // Estonia
#define ET 0x4554 // Ethiopia
#define FO 0x464F // Faeroe Islands
#define FK 0x464B // Falkland Islands (Malvinas)
#define FJ 0x464A // Fiji, Republic of the Fiji Islands
#define FI 0x4649 // Finland, Republic of
#define FR 0x4652 // France, French Republic
#define GF 0x4746 // French Guiana
#define PF 0x5046 // French Polynesia
#define TF 0x5446 // French Southern Territories
#define GA 0x4741 // Gabon, Gabonese Republic
#define GM 0x474D // Gambia, Republic of the
#define GE 0x4745 // Georgia
#define DE 0x4445 // Germany
#define GH 0x4748 // Ghana, Republic of
#define GI 0x4749 // Gibraltar
#define GR 0x4752 // Greece, Hellenic Republic
#define GL 0x474C // Greenland
#define GD 0x4744 // Grenada
#define GP 0x4750 // Guadaloupe
#define GU 0x4755 // Guam
#define GT 0x4754 // Guatemala, Republic of
#define GN 0x474E // Guinea, Revolutionary People's Rep'c of
#define GW 0x4757 // Guinea-Bissau, Republic of
#define GY 0x4759 // Guyana, Republic of
#define HT 0x4854 // Haiti, Republic of
#define HM 0x484d // Heard and McDonald Islands
#define VA 0x5641 // Holy See (Vatican City State)
#define HN 0x484E // Honduras, Republic of
#define HK 0x484B // Hong Kong, Special Administrative Region of China
#define HR 0x4852 // Hrvatska (Croatia)
#define HU 0x4855 // Hungary, Hungarian People's Republic
#define IS 0x4953 // Iceland, Republic of
#define IN 0x494E // India, Republic of
#define ID 0x4944 // Indonesia, Republic of
#define IR 0x4952 // Iran, Islamic Republic of
#define IQ 0x4951 // Iraq, Republic of
#define IE 0x4945 // Ireland
#define IL 0x494C // Israel, State of
#define IT 0x4954 // Italy, Italian Republic
#define JM 0x4A4D // Jamaica
#define JP 0x4a50 // Japan
#define JO 0x4A4F // Jordan, Hashemite Kingdom of
#define KZ 0x4B5A // Kazakhstan, Republic of
#define KE 0x4B45 // Kenya, Republic of
#define KI 0x4B49 // Kiribati, Republic of
#define KP 0x4B50 // Korea, Democratic People's Republic of
#define KR 0x4B52 // Korea, Republic of
#define KW 0x4B57 // Kuwait, State of
#define KG 0x4B47 // Kyrgyz Republic
#define LA 0x4C41 // Lao People's Democratic Republic
#define LV 0x4C56 // Latvia
#define LB 0x4C42 // Lebanon, Lebanese Republic
#define LS 0x4C53 // Lesotho, Kingdom of
#define LR 0x4C52 // Liberia, Republic of
#define LY 0x4C59 // Libyan Arab Jamahiriya
#define LI 0x4C49 // Liechtenstein, Principality of
#define LT 0x4C54 // Lithuania
#define LU 0x4C55 // Luxembourg, Grand Duchy of
#define MO 0x4D4F // Macao, Special Administrative Region of China
#define MK 0x4D4B // Macedonia, the former Yugoslav Republic of
#define MG 0x4D47 // Madagascar, Republic of
#define MW 0x4d57 // Malawi, Republic of
#define MY 0x4D59 // Malaysia
#define MV 0x4D56 // Maldives, Republic of
#define ML 0x4D4C // Mali, Republic of
#define MT 0x4D54 // Malta, Republic of
#define MH 0x4D48 // Marshall Islands
#define MQ 0x4D51 // Martinique
#define MR 0x4D52 // Mauritania, Islamic Republic of
#define MU 0x4D55 // Mauritius
#define/*YT*/ MYT 0x5954 // Mayotte (MYT used because YT is a keyword)
#define MX 0x4D58 // Mexico, United Mexican States
#define FM 0x464D // Micronesia, Federated States of
#define MD 0x4D44 // Moldova, Republic of
#define MC 0x4D43 // Monaco, Principality of
#define MN 0x4D4E // Mongolia, Mongolian People's Republic
#define MS 0x4D53 // Montserrat
#define MA 0x4D41 // Morocco, Kingdom of
#define MZ 0x4D5A // Mozambique, People's Republic of
#define MM 0x4D4D // Myanmar
#define NA 0x4E41 // Namibia
#define NR 0x4E52 // Nauru, Republic of
#define NP 0x4E50 // Nepal, Kingdom of
#define AN 0x414E // Netherlands Antilles
#define NL 0x4E4C // Netherlands, Kingdom of the
#define NC 0x4E43 // New Caledonia
#define NZ 0x4E5A // New Zealand
#define NI 0x4E49 // Nicaragua, Republic of
#define NE 0x4E45 // Niger, Republic of the
#define NG 0x4E47 // Nigeria, Federal Republic of
#define NU 0x4E55 // Niue, Republic of
#define NF 0x4E46 // Norfolk Island
#define MP 0x4D50 // Northern Mariana Islands
#define NO 0x4E4F // Norway, Kingdom of
#define OM 0x4F4D // Oman, Sultanate of
#define PK 0x504B // Pakistan, Islamic Republic of
#define PW 0x5057 // Palau
#define PS 0x5053 // Palestinian Territory, Occupied
#define PA 0x5041 // Panama, Republic of
#define PG 0x5047 // Papua New Guinea
#define PY 0x5059 // Paraguay, Republic of
#define PE 0x5045 // Peru, Republic of
#define PH 0x5048 // Philippines, Republic of the
#define PN 0x504E // Pitcairn Island
#define PL 0x504C // Poland, Polish People's Republic
#define PT 0x5054 // Portugal, Portuguese Republic
#define PR 0x5052 // Puerto Rico
#define QA 0x5141 // Qatar, State of
#define RE 0x5245 // Reunion
#define RO 0x524F // Romania, Socialist Republic of
#define RU 0x5255 // Russian Federation
#define RW 0x5257 // Rwanda, Rwandese Republic
#define SH 0x5348 // St. Helena
#define KN 0x4B4E // St. Kitts and Nevis
#define LC 0x4C43 // St. Lucia
#define PM 0x504D // St. Pierre and Miquelon
#define VC 0x5643 // St. Vincent and the Grenadines
#define WS 0x5753 // Samoa, Independent State of
#define SM 0x534d // San Marino, Republic of
#define ST 0x5354 // Sao Tome and Principe, Democratic Republic of
#define SA 0x5341 // Saudi Arabia, Kingdom of
#define SN 0x534E // Senegal, Republic of
#define CS 0x4353 // Serbia and Montenegro
#define SC 0x5343 // Seychelles, Republic of
#define SL 0x534C // Sierra Leone, Republic of
#define SG 0x5347 // Singapore, Republic of
#define SK 0x534B // Slovakia (Slovak Republic)
#define SI 0x5349 // Slovenia
#define SB 0x5342 // Solomon Islands
#define SO 0x534F // Somalia, Somali Republic
#define ZA 0x5A41 // South Africa, Republic of
#define GS 0x4753 // South Georgia and the South Sandwich Islands
#define ES 0x4553 // Spain, Spanish State
#define LK 0x4C4B // Sri Lanka, Democratic Socialist Republic of
#define SD 0x5344 // Sudan, Democratic Republic of the
#define SR 0x5352 // Suriname, Republic of
#define SJ 0x534A // Svalbard & Jan Mayen Islands
#define SZ 0x535A // Swaziland, Kingdom of
#define SE 0x5345 // Sweden, Kingdom of
#define CH 0x4348 // Switzerland, Swiss Confederation
#define SY 0x5359 // Syrian Arab Republic
#define TW 0x5457 // Taiwan, Province of China
#define TJ 0x544A // Tajikistan
#define TZ 0x545A // Tanzania, United Republic of
#define TH 0x5448 // Thailand, Kingdom of
#define TL 0x544C // Timor-Leste, Democratic Republic of
#define TG 0x5447 // Togo, Togolese Republic
#define TK 0x544B // Tokelau (Tokelau Islands)
#define TO 0x544F // Tonga, Kingdom of
#define TT 0x5454 // Trinidad and Tobago, Republic of
#define TN 0x544E // Tunisia, Republic of
#define TR 0x5452 // Turkey, Republic of
#define TM 0x544D // Turkmenistan
#define TC 0x5443 // Turks and Caicos Islands
#define/*TV*/ TUV 0x5456 // Tuvalu (TUV used because #define TV is already used for Teritary )
#define VI 0x5649 // US Virgin Islands
#define UG 0x5547 // Uganda, Republic of
#define UA 0x5541 // Ukraine
#define AE 0x4145 // United Arab Emirates
#define GB 0x4742 // United Kingdom of Great Britain & N. Ireland
#define UM 0x554D // United States Minor Outlying Islands
#define US 0x5553 // United States of America
#define UY 0x5559 // Uruguay, Eastern Republic of
#define UZ 0x555A // Uzbekistan
#define VU 0x5655 // Vanuatu
#define VE 0x5645 // Venezuela, Bolivarian Republic of
#define VN 0x564E // Viet Nam, Socialist Republic of
#define WF 0x5746 // Wallis and Futuna Islands
#define EH 0x4548 // Western Sahara
#define YE 0x5945 // Yemen
#define ZM 0x5A4D // Zambia, Republic of
#define ZW 0x5A57 // Zimbabwe

VARIABLE country_code_codes
{
    CLASS LOCAL;
    TYPE ENUMERATED (2)
    {
//Remove new dictionary references from legacy binary format
        { AF,  AF_country_code_temp, AF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AL,  AL_country_code_temp, AL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { DZ,  DZ_country_code_temp, DZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AS,  AS_country_code_temp, AS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AD,  AD_country_code_temp, AD_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AO,  AO_country_code_temp, AO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AI,  AI_country_code_temp, AI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AQ,  AQ_country_code_temp, AQ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AG,  AG_country_code_temp, AG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AR,  AR_country_code_temp, AR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AM,  AM_country_code_temp, AM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AW,  AW_country_code_temp, AW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AU,  AU_country_code_temp, AU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AT,  AT_country_code_temp, AT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AZ,  AZ_country_code_temp, AZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BS,  BS_country_code_temp, BS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BH,  BH_country_code_temp, BH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BD,  BD_country_code_temp, BD_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BB,  BB_country_code_temp, BB_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BY,  BY_country_code_temp, BY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BE,  BE_country_code_temp, BE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BZ,  BZ_country_code_temp, BZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BJ,  BJ_country_code_temp, BJ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BM,  BM_country_code_temp, BM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BT,  BT_country_code_temp, BT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BO,  BO_country_code_temp, BO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BA,  BA_country_code_temp, BA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BW,  BW_country_code_temp, BW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BV,  BV_country_code_temp, BV_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BR,  BR_country_code_temp, BR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IO,  IO_country_code_temp, IO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { VG,  VG_country_code_temp, VG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BN,  BN_country_code_temp, BN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BG,  BG_country_code_temp, BG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BF,  BF_country_code_temp, BF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { BI,  BI_country_code_temp, BI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KH,  KH_country_code_temp, KH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CM,  CM_country_code_temp, CM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CA,  CA_country_code_temp, CA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CV,  CV_country_code_temp, CV_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KY,  KY_country_code_temp, KY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CF,  CF_country_code_temp, CF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TD,  TD_country_code_temp, TD_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CL,  CL_country_code_temp, CL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CN,  CN_country_code_temp, CN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CX,  CX_country_code_temp, CX_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CC,  CC_country_code_temp, CC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CO,  CO_country_code_temp, CO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KM,  KM_country_code_temp, KM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CD,  CD_country_code_temp, CD_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CG,  CG_country_code_temp, CG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CK,  CK_country_code_temp, CK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CR,  CR_country_code_temp, CR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CI,  CI_country_code_temp, CI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CU,  CU_country_code_temp, CU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CY,  CY_country_code_temp, CY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CZ,  CZ_country_code_temp, CZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { DK,  DK_country_code_temp, DK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { DJ,  DJ_country_code_temp, DJ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { DM,  DM_country_code_temp, DM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { DO,  DO_country_code_temp, DO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { EC,  EC_country_code_temp, EC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { EG,  EG_country_code_temp, EG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SLV, SV_country_code_temp, SV_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GQ,  GQ_country_code_temp, GQ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ER,  ER_country_code_temp, ER_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { EE,  EE_country_code_temp, EE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ET,  ET_country_code_temp, ET_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { FO,  FO_country_code_temp, FO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { FK,  FK_country_code_temp, FK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { FJ,  FJ_country_code_temp, FJ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { FI,  FI_country_code_temp, FI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { FR,  FR_country_code_temp, FR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GF,  GF_country_code_temp, GF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PF,  PF_country_code_temp, PF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TF,  TF_country_code_temp, TF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GA,  GA_country_code_temp, GA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GM,  GM_country_code_temp, GM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GE,  GE_country_code_temp, GE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { DE,  DE_country_code_temp, DE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GH,  GH_country_code_temp, GH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GI,  GI_country_code_temp, GI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GR,  GR_country_code_temp, GR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GL,  GL_country_code_temp, GL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GD,  GD_country_code_temp, GD_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GP,  GP_country_code_temp, GP_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GU,  GU_country_code_temp, GU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GT,  GT_country_code_temp, GT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GN,  GN_country_code_temp, GN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GW,  GW_country_code_temp, GW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GY,  GY_country_code_temp, GY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { HT,  HT_country_code_temp, HT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { HM,  HM_country_code_temp, HM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { VA,  VA_country_code_temp, VA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { HN,  HN_country_code_temp, HN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { HK,  HK_country_code_temp, HK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { HR,  HR_country_code_temp, HR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { HU,  HU_country_code_temp, HU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IS,  IS_country_code_temp, IS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IN,  IN_country_code_temp, IN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ID,  ID_country_code_temp, ID_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IR,  IR_country_code_temp, IR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IQ,  IQ_country_code_temp, IQ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IE,  IE_country_code_temp, IE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IL,  IL_country_code_temp, IL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { IT,  IT_country_code_temp, IT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { JM,  JM_country_code_temp, JM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { JP,  JP_country_code_temp, JP_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { JO,  JO_country_code_temp, JO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KZ,  KZ_country_code_temp, KZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KE,  KE_country_code_temp, KE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KI,  KI_country_code_temp, KI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KP,  KP_country_code_temp, KP_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KR,  KR_country_code_temp, KR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KW,  KW_country_code_temp, KW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KG,  KG_country_code_temp, KG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LA,  LA_country_code_temp, LA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LV,  LV_country_code_temp, LV_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LB,  LB_country_code_temp, LB_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LS,  LS_country_code_temp, LS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LR,  LR_country_code_temp, LR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LY,  LY_country_code_temp, LY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LI,  LI_country_code_temp, LI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LT,  LT_country_code_temp, LT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LU,  LU_country_code_temp, LU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MO,  MO_country_code_temp, MO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MK,  MK_country_code_temp, MK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MG,  MG_country_code_temp, MG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MW,  MW_country_code_temp, MW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MY,  MY_country_code_temp, MY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MV,  MV_country_code_temp, MV_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ML,  ML_country_code_temp, ML_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MT,  MT_country_code_temp, MT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MH,  MH_country_code_temp, MH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MQ,  MQ_country_code_temp, MQ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MR,  MR_country_code_temp, MR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MU,  MU_country_code_temp, MU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MYT, YT_country_code_temp, YT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MX,  MX_country_code_temp, MX_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { FM,  FM_country_code_temp, FM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MD,  MD_country_code_temp, MD_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MC,  MC_country_code_temp, MC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MN,  MN_country_code_temp, MN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MS,  MS_country_code_temp, MS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MA,  MA_country_code_temp, MA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MZ,  MZ_country_code_temp, MZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MM,  MM_country_code_temp, MM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NA,  NA_country_code_temp, NA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NR,  NR_country_code_temp, NR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NP,  NP_country_code_temp, NP_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AN,  AN_country_code_temp, AN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NL,  NL_country_code_temp, NL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NC,  NC_country_code_temp, NC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NZ,  NZ_country_code_temp, NZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NI,  NI_country_code_temp, NI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NE,  NE_country_code_temp, NE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NG,  NG_country_code_temp, NG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NU,  NU_country_code_temp, NU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NF,  NF_country_code_temp, NF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { MP,  MP_country_code_temp, MP_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { NO,  NO_country_code_temp, NO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { OM,  OM_country_code_temp, OM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PK,  PK_country_code_temp, PK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PW,  PW_country_code_temp, PW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PS,  PS_country_code_temp, PS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PA,  PA_country_code_temp, PA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PG,  PG_country_code_temp, PG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PY,  PY_country_code_temp, PY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PE,  PE_country_code_temp, PE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PH,  PH_country_code_temp, PH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PN,  PN_country_code_temp, PN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PL,  PL_country_code_temp, PL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PT,  PT_country_code_temp, PT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PR,  PR_country_code_temp, PR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { QA,  QA_country_code_temp, QA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { RE,  RE_country_code_temp, RE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { RO,  RO_country_code_temp, RO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { RU,  RU_country_code_temp, RU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { RW,  RW_country_code_temp, RW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SH,  SH_country_code_temp, SH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { KN,  KN_country_code_temp, KN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LC,  LC_country_code_temp, LC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { PM,  PM_country_code_temp, PM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { VC,  VC_country_code_temp, VC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { WS,  WS_country_code_temp, WS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SM,  SM_country_code_temp, SM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ST,  ST_country_code_temp, ST_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SA,  SA_country_code_temp, SA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SN,  SN_country_code_temp, SN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CS,  CS_country_code_temp, CS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SC,  SC_country_code_temp, SC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SL,  SL_country_code_temp, SL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SG,  SG_country_code_temp, SG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SK,  SK_country_code_temp, SK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SI,  SI_country_code_temp, SI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SB,  SB_country_code_temp, SB_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SO,  SO_country_code_temp, SO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ZA,  ZA_country_code_temp, ZA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GS,  GS_country_code_temp, GS_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ES,  ES_country_code_temp, ES_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { LK,  LK_country_code_temp, LK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SD,  SD_country_code_temp, SD_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SR,  SR_country_code_temp, SR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SJ,  SJ_country_code_temp, SJ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SZ,  SZ_country_code_temp, SZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SE,  SE_country_code_temp, SE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { CH,  CH_country_code_temp, CH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { SY,  SY_country_code_temp, SY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TW,  TW_country_code_temp, TW_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TJ,  TJ_country_code_temp, TJ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TZ,  TZ_country_code_temp, TZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TH,  TH_country_code_temp, TH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TL,  TL_country_code_temp, TL_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TG,  TG_country_code_temp, TG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TK,  TK_country_code_temp, TK_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TO,  TO_country_code_temp, TO_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TT,  TT_country_code_temp, TT_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TN,  TN_country_code_temp, TN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TR,  TR_country_code_temp, TR_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TM,  TM_country_code_temp, TM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TC,  TC_country_code_temp, TC_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { TUV, TV_country_code_temp, TV_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { VI,  VI_country_code_temp, VI_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { UG,  UG_country_code_temp, UG_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { UA,  UA_country_code_temp, UA_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { AE,  AE_country_code_temp, AE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { GB,  GB_country_code_temp, GB_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { UM,  UM_country_code_temp, UM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { US,  US_country_code_temp, US_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { UY,  UY_country_code_temp, UY_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { UZ,  UZ_country_code_temp, UZ_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { VU,  VU_country_code_temp, VU_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { VE,  VE_country_code_temp, VE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { VN,  VN_country_code_temp, VN_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { WF,  WF_country_code_temp, WF_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { EH,  EH_country_code_temp, EH_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { YE,  YE_country_code_temp, YE_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ZM,  ZM_country_code_temp, ZM_country_code_help_temp },
//Remove new dictionary references from legacy binary format
        { ZW,  ZW_country_code_temp, ZW_country_code_help_temp }

    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE US;
#endif //__TOKVER__ >=  600
}

VARIABLE master_mode_code
{
    CLASS LOCAL;
    TYPE ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0, secondary_master_mode_code_temp, secondary_master_mode_code_help_temp },
//Remove new dictionary references from legacy binary format
        { 1, primary_master_mode_code_temp,   primary_master_mode_code_help_temp   }

    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE 1;    //Primary Master (1) is default
#endif //__TOKVER__ >=  600
}

/**************************************************
 **                                              **
 **   The following variables model data         **
 **   contained in the field device in order to  **
 **   allow for easier distribution of table     **
 **   and enumeration item changes without       **
 **   having to retokenize the universal,        **
 **   common practice, and wireless DDs.         **
 **                                              **
 **   Care must be taken to all who import these **
 **   items such that the appropriate COMMANDS   **
 **   and supporting data are imported from the  **
 **   respective DDs.  To help facilitate this,  **
 **   MACROS are provided in macros.h to import  **
 **   the proper items based on the              **
 **   functionality of the device.               **
 **                                              **
 **************************************************/

/*************************************************************************************
 * All COMMAND REPLY declarations must include this VARIABLE as the second
 * parameter.
 *
 * This VARIABLE provides status on the health of the device as a whole. For
 * more information see "Command Summary Specification"
 *************************************************************************************
 */
VARIABLE device_status
{
//Remove new dictionary references from legacy binary format
    LABEL device_status_temp;
//Remove new dictionary references from legacy binary format
    HELP device_status_help_temp;
    CLASS HART & DYNAMIC & DIAGNOSTIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED
    {
//Remove new dictionary references from legacy binary format
        { 0x01, PV_Out_of_Limits_temp,         pv_out_of_limits_help_temp,        DV1 (AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x02, Non_PV_Out_of_Limits_temp,     non_pv_out_of_limits_help_temp,    MISC },
//Remove new dictionary references from legacy binary format
        { 0x04, loop_current_saturated_temp,   loop_current_saturated_help_temp,  AO1 (AUTO & BAD) },
//Remove new dictionary references from legacy binary format
        { 0x08, loop_current_fixed_temp,       loop_current_fixed_help_temp,      AO1 (MANUAL & GOOD) },
//Remove new dictionary references from legacy binary format
        { 0x10, More_Status_Available_temp,    more_status_available_help_temp,   MORE & IGNORE_IN_HANDHELD },
//Remove new dictionary references from legacy binary format
        { 0x20, Cold_Start_temp,               cold_start_help_temp,              MISC },
//Remove new dictionary references from legacy binary format
        { 0x40, Configuration_Changed_temp,    configuration_changed_help_temp,   MISC & IGNORE_IN_HANDHELD },
//Remove new dictionary references from legacy binary format
        { 0x80, Device_Malfunction_temp,       device_malfunction_help_temp,      MORE}
    }
}

/*************************************************************************************
 * Unique Identifier Variables
 *
 * Used in CMD 0
 */

/*************************************************************************************
 * This VARIABLE indicate the company actually made the field device.
 * the value of This VARIABLE may or may not be the same as the value
 * of the private_label_distributor.
 *
 * NOTE: the HART Protocol requires masters to display the
 *      private_label_distributor and not the manufacturer_id
 */
manufacturer_id LIKE VARIABLE company_identification_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL manufacturer_id_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP manufacturer_id_code_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * This VARIABLE indicate the company actually distributing the
 * field device.  The value of This VARIABLE may or may not be the same
 * manufacturer_id.
 */
private_label_distributor LIKE VARIABLE company_identification_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL private_label_distributor_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP Private_Label_Distributor_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * The combination of the one byte device_type variable along with the one byte
 * variable manufacturer_id" combines to form the expanded device type code. This
 * code will have only one unique Device Description.
 */
device_type LIKE VARIABLE device_type_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Device_Type_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP device_type_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}


physical_signaling_code LIKE VARIABLE physical_signaling_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL physical_signaling_code_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP physical_signaling_code_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

device_flags LIKE VARIABLE std_flag_assignment
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Device_Flags_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP device_flags_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Used in CMD 0 (HART 6 and later), 9 & 48
 */
extended_fld_device_status LIKE VARIABLE std_extended_device_status_codes    // Refer to table 17
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL extended_device_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP extended_device_status_help_temp;
    REDEFINE CLASS HART & DYNAMIC & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}


/*************************************************************************************
 * This variable describes the type of device the host is communicating with
 * so that a host can "know" what commands are supported in the device. Most
 * useful when a host doesn't have the DD for the device but must work
 * generically.
 *
 * Used in CMD 0 for HART7 and later
 *************************************************************************************
 */
device_profile LIKE VARIABLE device_profile_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL device_profile_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP device_profile_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Loop Current Mode Variable
 *
 * Used in CMDs 6-7
 */
loop_current_mode LIKE VARIABLE loop_current_mode_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Loop_Current_Mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_current_mode_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
}

/*************************************************************************************
 * 'Write protect' locks configuration changes from occurring in the
 * device.  Assume to be hardware jumper/switch. DD developer should
 * redefine handling to READ & WRITE if the device has software
 * configurable write protection.
 *
 * Used in CMD 15, and possibly device specific write command
 */
write_protect LIKE VARIABLE write_protect_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Write_Protect_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP write_protect_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

loop_alarm_code LIKE VARIABLE alarm_selection_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL loop_alarm_code_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_alarm_code_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

SVanalogChannelAlarmCode LIKE VARIABLE alarm_selection_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL sv_loop_alarm_code_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_alarm_code_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

TVanalogChannelAlarmCode LIKE VARIABLE alarm_selection_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL tv_loop_alarm_code_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_alarm_code_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

QVanalogChannelAlarmCode LIKE VARIABLE alarm_selection_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL qv_loop_alarm_code_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_alarm_code_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

loop_flags LIKE VARIABLE std_analog_channel_flags
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL loop_flags_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_flags_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

SVanalogChannelFlags LIKE VARIABLE std_analog_channel_flags
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL sv_loop_flags_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_flags_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

TVanalogChannelFlags LIKE VARIABLE std_analog_channel_flags
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL tv_loop_flags_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_flags_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

QVanalogChannelFlags LIKE VARIABLE std_analog_channel_flags
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL qv_loop_flags_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP loop_flags_help_temp;
    REDEFINE CLASS ANALOG_CHANNEL;
    REDEFINE HANDLING READ;
}

transfer_function LIKE VARIABLE transfer_function_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Transfer_Function_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP transfer_function_help_temp;
    REDEFINE CLASS RANGE;
    REDEFINE HANDLING READ;
}

SVtransferFunction LIKE VARIABLE transfer_function_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL sv_transfer_function_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP transfer_function_help_temp;
    REDEFINE CLASS RANGE;
    REDEFINE HANDLING READ;
}

TVtransferFunction LIKE VARIABLE transfer_function_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL tv_transfer_function_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP transfer_function_help_temp;
    REDEFINE CLASS RANGE;
    REDEFINE HANDLING READ;
}

QVtransferFunction LIKE VARIABLE transfer_function_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL qv_transfer_function_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP transfer_function_help_temp;
    REDEFINE CLASS RANGE;
    REDEFINE HANDLING READ;
}


/*************************************************************************************
 * More Status Available Variables
 *
 * Used in CMD 48
 */

standardized_status_0 LIKE VARIABLE std_standardized_status_0_codes          // Refer to table 29
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_0_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_0_help_temp;
    REDEFINE CLASS HART & DYNAMIC & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_1 LIKE VARIABLE std_standardized_status_1_codes          // Refer to table 30
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_1_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_1_help_temp;
    REDEFINE CLASS HART & DYNAMIC & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_2 LIKE VARIABLE std_standardized_status_2_codes          // Refer to table 31
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_2_help_temp;
    REDEFINE CLASS HART & DYNAMIC & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_3 LIKE VARIABLE std_standardized_status_3_codes          // Refer to table 32
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_3_help_temp;
    REDEFINE CLASS HART & DYNAMIC & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

analog_channel_saturated1 LIKE VARIABLE std_analog_channel_saturated_codes   // Refer to table 27
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_saturated_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_saturated_help_temp;
    REDEFINE CLASS DEVICE & DYNAMIC & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

analog_channel_fixed1 LIKE VARIABLE std_analog_channel_fixed_codes           // Refer to table 28
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_fixed_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_fixed_help_temp;
    REDEFINE CLASS DEVICE & DYNAMIC & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Lock Device Variables
 *
 * Used in CMDs 71 & 76
 */
 
/*************************************************************************************
 * Used in CMD 71
 */
lock_device_code LIKE VARIABLE lock_device_codes    // Refer to table 18
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Lock_Device_Code_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP lock_device_code_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING WRITE;
}

/*************************************************************************************
 * Used in CMD 76
 */
lock_device_status_code LIKE VARIABLE std_lock_device_status_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL lock_device_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP lock_device_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Squawk Variable
 *
 * Used in CMD 72
 */
squawk_control LIKE VARIABLE squawk_control_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL squawk_control_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP squawk_control_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING WRITE;
}

/*************************************************************************************
 * IO System Master Mode Variables
 *
 * Used in CMDs 74 & 87
 */
master_mode LIKE VARIABLE master_mode_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL master_mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  master_mode_help_temp;
    REDEFINE CLASS HART;
}

/*************************************************************************************
 * Sub-device Variables
 *
 * Used in CMD 84
 */

/*************************************************************************************
 * The sub-device type uses the same list of devices as the device type.
 */
subdev_expanded_device_type LIKE VARIABLE device_type_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL subdevice_device_type_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP subdevice_device_type_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * The sub-device manufacturer code and distributor uses the same list of manufacturers
 * as the device type.
 */
subdev_manufacturer_id LIKE VARIABLE company_identification_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL subdevice_manufacturer_id_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP subdevice_manufacturer_id_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

subdev_distributor LIKE VARIABLE company_identification_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL subdevice_distributor_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP subdevice_distributor_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

subdev_dev_profile LIKE VARIABLE device_profile_codes  //  Refer to table 57
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL subdevice_profile_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP subdevice_profile_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Real-Time Variables
 *
 * Used in CMDs 89-90
 */

/*************************************************************************************
 * Used in CMD 89
 */
time_set LIKE VARIABLE time_set_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL time_set_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP time_set_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING WRITE;
}

/*************************************************************************************
 * Used in CMD 90
 */
real_time_clock_flag LIKE VARIABLE std_real_time_clock_flag_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL real_time_clock_flag_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP real_time_clock_flag_help_temp;
    REDEFINE CLASS DEVICE & DYNAMIC;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Trend Variables
 *
 * Used in CMDs 91-93
 */

/*************************************************************************************
 * Used in CMDs 91-92
 */
trend_control LIKE VARIABLE trend_control_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_control_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP trend_control_help_temp;
    REDEFINE CLASS HART & SERVICE;
}

/*************************************************************************************
 * Used in CMD 93
 */
trend_digital_units LIKE VARIABLE units_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_units_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP trend_units_help_temp;
    REDEFINE CLASS DEVICE & SERVICE;
    REDEFINE HANDLING READ;
}

trend_classification LIKE VARIABLE device_variable_classification_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_classification_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP trend_classification_help_temp;
    REDEFINE CLASS DEVICE & SERVICE;
    REDEFINE HANDLING READ;
}

trend_value_data_quality LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_1 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_2 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_3 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_4_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_4 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_5_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_5 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_6_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_6 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_7_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_7 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_8_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_8 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_9_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_9 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_10_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_10 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_11_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_data_quality_11 LIKE VARIABLE process_data_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_data_quality_12_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_data_quality_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}


trend_value_limit_status LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_1 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_2 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_3 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_4_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_4 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_5_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_5 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_6_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_6 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_7_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_7 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_8_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_8 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_9_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_9 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_10_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_10 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_11_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_limit_status_11 LIKE VARIABLE limit_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_limit_status_12_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_limit_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_device_family_status LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}
std_trend_value_0_device_family_status_1 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_2 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_3 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_4_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_4 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_5_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_5 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_6_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_6 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_7_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_7 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_8_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_8 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_9_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_9 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_10_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_10 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_11_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

std_trend_value_0_device_family_status_11 LIKE VARIABLE std_device_family_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL trend_family_status_12_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  trend_family_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Country Code Variables
 *
 * Used in CMDs 512-513
 */
country_code LIKE VARIABLE country_code_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL country_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  country_help_temp;
    REDEFINE CLASS DEVICE;
}

/*************************************************************************************
 * Burst Message Variables
 *
 * Used in CMDs 104, 105 & 109
 */

/*************************************************************************************
 * Used in CMDs 104 & 105
 */
burst_trigger_units LIKE VARIABLE units_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_units_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_units_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_trigger_units_1 LIKE VARIABLE units_code
//Remove new dictionary references from legacy binary format
{   REDEFINE LABEL burst_trigger_units_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_units_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_trigger_units_2 LIKE VARIABLE units_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_units_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_units_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_trigger_units_3 LIKE VARIABLE units_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_units_4_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_units_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_trigger_classification LIKE VARIABLE device_variable_classification_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_class_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_class_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_trigger_class_1 LIKE VARIABLE device_variable_classification_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_class_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_class_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_trigger_class_2 LIKE VARIABLE device_variable_classification_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_class_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_class_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_trigger_class_3 LIKE VARIABLE device_variable_classification_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_class_4_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_class_help_temp;
    REDEFINE CLASS COMPUTATION;
}

burst_message_trigger_mode LIKE VARIABLE burst_message_trigger_mode_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_mode_help_temp;
    REDEFINE CLASS HART;
}

burst_trigger_mode_1 LIKE VARIABLE burst_message_trigger_mode_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_mode_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_mode_help_temp;
    REDEFINE CLASS HART;
}

burst_trigger_mode_2 LIKE VARIABLE burst_message_trigger_mode_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_mode_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_mode_help_temp;
    REDEFINE CLASS HART;
}

burst_trigger_mode_3 LIKE VARIABLE burst_message_trigger_mode_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_trigger_mode_4_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_trigger_mode_help_temp;
    REDEFINE CLASS HART;
}

/*************************************************************************************
 * Used in CMDs 105 & 109
 */
burst_mode_select LIKE VARIABLE burst_mode_control_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Burst_Mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_mode_help_temp;
    REDEFINE CLASS HART;
}

/* Some versions of some host systems overwrite the burst_mode_select value based on the
   burst mode bit in the Data Link Layer.  This is holdover from previous HART universal
   revisions as an optimization due to the fact that there was no read command to find out
   from the device if burst mode was enabled or not.  This does not work for wireless
   devices because the DLL burst mode bit on wired interface does not reflect the state of
   burst mode on the wireless interface. Define a new burst_mode_select bit to avoid
   this optimization from negatively affecting the DD. */
burst_mode_select_0 LIKE VARIABLE burst_mode_control_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL Burst_Mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_mode_help_temp;
    REDEFINE CLASS HART;
}

burst_mode_select_1 LIKE VARIABLE burst_mode_control_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_mode_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_mode_help_temp;
    REDEFINE CLASS HART;
}

burst_mode_select_2 LIKE VARIABLE burst_mode_control_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_mode_3_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_mode_help_temp;
    REDEFINE CLASS HART;
}

burst_mode_select_3 LIKE VARIABLE burst_mode_control_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL burst_mode_4_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP burst_mode_help_temp;
    REDEFINE CLASS HART;
}

/*************************************************************************************
 * Event Notification Variables
 *
 * Used in CMDs 115, 116 & 118
 */

/*************************************************************************************
 * Used in CMD 115
 */
event_status LIKE VARIABLE std_event_status_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL event_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP event_status_help_temp;
    REDEFINE CLASS HART & DYNAMIC;
    REDEFINE HANDLING READ;
}

event_status_1 LIKE VARIABLE std_event_status_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL event_status_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP event_status_help_temp;
    REDEFINE CLASS HART & DYNAMIC;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Event Masks
 *
 * Used in CMDs 115-116
 */

device_status_mask LIKE VARIABLE device_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL device_status_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP device_status_mask_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ & WRITE;
}

device_status_mask_1 LIKE VARIABLE device_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL device_status_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP device_status_mask_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ & WRITE;
}

extended_fld_device_status_mask LIKE VARIABLE std_extended_device_status_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL extended_device_status_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP extended_device_status_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

extended_fld_device_status_mask_1 LIKE VARIABLE std_extended_device_status_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL extended_device_status_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP extended_device_status_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

dev_operating_mode_mask LIKE VARIABLE std_operating_mode_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL operating_mode_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP operating_mode_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

dev_operating_mode_mask_1 LIKE VARIABLE std_operating_mode_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL operating_mode_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP operating_mode_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_0_mask LIKE VARIABLE std_standardized_status_0_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_0_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_0_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_0_mask_1 LIKE VARIABLE std_standardized_status_0_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_0_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_0_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_1_mask LIKE VARIABLE std_standardized_status_1_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_1_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_1_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_1_mask_1 LIKE VARIABLE std_standardized_status_1_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_1_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_1_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

analog_channel_saturated1_mask LIKE VARIABLE std_analog_channel_saturated_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_saturated_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_saturated_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

analog_channel_saturated1_mask_1 LIKE VARIABLE std_analog_channel_saturated_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_saturated_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_saturated_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_2_mask LIKE VARIABLE std_standardized_status_2_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_2_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_2_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_2_mask_1 LIKE VARIABLE std_standardized_status_2_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_2_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_2_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_3_mask LIKE VARIABLE std_standardized_status_3_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_3_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_3_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

standardized_status_3_mask_1 LIKE VARIABLE std_standardized_status_3_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_3_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_3_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

analog_channel_fixed1_mask LIKE VARIABLE std_analog_channel_fixed_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_fixed_mask_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_fixed_mask_help_temp;
    REDEFINE CLASS DEVICE;
}

analog_channel_fixed1_mask_1 LIKE VARIABLE std_analog_channel_fixed_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_fixed_mask_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_fixed_mask_help_temp;
    REDEFINE CLASS DEVICE;
}


/*************************************************************************************
 * Used in CMDs 115 & 118
 */
event_notification_control_n LIKE VARIABLE event_nofication_control_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL event_notification_control_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP event_notification_control_help_temp;
    REDEFINE CLASS HART;
}

event_notification_control_n_1 LIKE VARIABLE event_nofication_control_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL event_notification_control_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP event_notification_control_help_temp;
    REDEFINE CLASS HART;
}

/*************************************************************************************
 * Event Acknowledgement
 *
 * Used in CMD 119
 */

device_status_latched_value LIKE VARIABLE device_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL device_status_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP device_status_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

device_status_latched_value_1 LIKE VARIABLE device_status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL device_status_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP device_status_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

extended_fld_device_status_latched_value LIKE VARIABLE std_extended_device_status_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL extended_device_status_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP extended_device_status_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

extended_fld_device_status_latched_value_1  LIKE VARIABLE std_extended_device_status_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL extended_device_status_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP extended_device_status_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

std_operating_mode_latched_value LIKE VARIABLE std_operating_mode_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL operating_mode_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP operating_mode_latched_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

dev_operating_mode_latched_value_1 LIKE VARIABLE std_operating_mode_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL operating_mode_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP operating_mode_latched_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

standardized_status_0_latched_value LIKE VARIABLE std_standardized_status_0_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_0_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_0_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_0_latched_value_1 LIKE VARIABLE std_standardized_status_0_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_0_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_0_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_1_latched_value LIKE VARIABLE std_standardized_status_1_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_1_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_1_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_1_latched_value_1 LIKE VARIABLE std_standardized_status_1_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_1_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_1_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

analog_channel_saturated1_latched_value LIKE VARIABLE std_analog_channel_saturated_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_saturated_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_saturated_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

analog_channel_saturated1_latched_value_1   LIKE VARIABLE std_analog_channel_saturated_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_saturated_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_saturated_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_2_latched_value LIKE VARIABLE std_standardized_status_2_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_2_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_2_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_2_latched_value_1 LIKE VARIABLE std_standardized_status_2_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_2_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_2_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_3_latched_value LIKE VARIABLE std_standardized_status_3_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_3_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_3_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

standardized_status_3_latched_value_1 LIKE VARIABLE std_standardized_status_3_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL standardized_status_3_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP standardized_status_3_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

analog_channel_fixed1_latched_value LIKE VARIABLE std_analog_channel_fixed_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_fixed_latched_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_fixed_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}

analog_channel_fixed1_latched_value_1 LIKE VARIABLE std_analog_channel_fixed_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL analog_channel_fixed_latched_event_2_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP analog_channel_fixed_latched_help_temp;
    REDEFINE CLASS DEVICE & DIAGNOSTIC;
    REDEFINE HANDLING READ;
}


/*************************************************************************************
 * Country Code Variables
 *
 * Used in CMDs 512-513
 */
si_control LIKE VARIABLE si_control_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL si_unit_control_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP  si_unit_control_help_temp;
    REDEFINE CLASS DEVICE;
}

/*************************************************************************************
 * Event Manager Variables
 *
 * Used in CMDs 514-515
 */

/*************************************************************************************
 * Used in CMD 514
 */
event_manager_registration_control LIKE VARIABLE event_manager_registration_control_codes // Refer to table 67
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL event_manager_registration_control_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP event_manager_registration_control_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING WRITE;
}

/*************************************************************************************
 * Used in CMD 515
 */
event_manager_registration_status LIKE VARIABLE event_manager_registration_status_codes // Refer to table 68
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL event_manager_registration_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP event_manager_registration_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Device Location Variable
 *
 * Used in CMD 516-517
 */
location_mechanism LIKE VARIABLE location_method_codes //  Refer to table 69
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL location_mechanism_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP location_mechanism_help_temp;
    REDEFINE CLASS DEVICE;
}

/*************************************************************************************
 * Condensed Status Variables
 *
 * Used in CMDs 523, 524, 526 & 527
 */

/*************************************************************************************
 * Used in CMDs 523-524
 */

statusmap_variable_000 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_000_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_001 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_001_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_002 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_002_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_003 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_003_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_004 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_004_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_005 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_005_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_006 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_006_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_007 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_007_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table A.1, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_008 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_008_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_009 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_009_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_010 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_010_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_011 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_011_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_012 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_012_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_013 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_013_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_014 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_014_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_015 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_015_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_016 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_016_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_017 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_017_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_018 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_018_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_019 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_019_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_020 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_020_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_021 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_021_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_022 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_022_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_023 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_023_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_024 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_024_temp; //[fld_dev_stat2_1];
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_025 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_025_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_026 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_026_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_027 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_027_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_028 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_028_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    
}

statusmap_variable_029 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_029_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_030 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_030_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_031 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_031_temp; //[fld_dev_stat2_8];
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_032 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_032_temp; //[fld_dev_stat3_1];
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_033 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_033_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_034 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_034_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_035 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_035_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_036 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_036_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_037 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_037_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_038 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_038_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_039 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_039_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_040 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_040_temp; //[fld_dev_stat4_1];
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_041 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_041_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_042 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_042_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_043 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_043_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_044 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_044_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_045 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_045_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_046 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_046_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_047 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_047_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_048 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_048_temp; //[fld_dev_stat5_1];
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_049 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_049_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_050 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_050_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_051 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_051_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_052 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_052_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_053 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_053_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_054 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_054_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_055 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_055_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_056 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_056_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 17, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_057 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_057_temp; //[device_variable_alert];
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 17, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_058 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_058_temp; //"Critical Power Failure"
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 3;        //Refer to table 17, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_059 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_059_temp; //"Failure"
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 17, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_060 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_060_temp; //"Out of Specification"
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 17, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_061 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_061_temp; //"Function Check"
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 17, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_062 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_063 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_064 LIKE VARIABLE statusmap_codes //  Operating Mode Map - Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 0 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_065 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_066 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_067 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_068 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_069 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_070 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_071 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp; //  Operating Mode Bit 7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_072 LIKE VARIABLE statusmap_codes //  standardized Status 0 Map - Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_072_temp; // Device Variable Simulation Active Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 5;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_073 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_073_temp; //"Non-Volatile Memory Defect Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 3;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_074 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_074_temp; //"Volatile Memory Defect Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 3;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_075 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_075_temp; //"Watchdog Reset Executed Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 3;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_076 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_076_temp; //"Power Supply Conditions Out of Range Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_077 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_077_temp; //"Environmental Conditions Out of Range Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_078 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_078_temp; //"Electronic Defect Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 3;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_079 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_079_temp; // Device Configuration Locked Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 29, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_080 LIKE VARIABLE statusmap_codes //  standardized Status 1 Map - Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_080_temp; //"Status Simulation Active Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 30, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_081 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_081_temp; //"Discrete Variable Simulation Active Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 5;        //Refer to table 30, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_082 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_082_temp; //"Event Notification Overflow Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 30, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_083 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_084 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_085 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_086 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_087 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_088 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_088_temp; //"Secondary Analog Channel Saturated Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 27, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_089 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_089_temp;  //"Tertiary Analog Channel Saturated";
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 27, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_090 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_090_temp; //"Quaternary Analog Channel Saturated Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 27, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_091 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_091_temp; //"Quinary Analog Channel Saturated Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 27, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_092 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_093 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_094 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_095 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_096 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_096_temp; //"Sub-Device List Changed Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 31, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_097 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_097_temp; //"Duplicate Master Detected Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 1;        //Refer to table 31, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_098 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_098_temp; //"Sub-Device Mismatch Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 1;        //Refer to table 31, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_099 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_099_temp; //"Sub-Devices with Duplicate IDs Found Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 31, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_100 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_100_temp; //"Stale Data Notice Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 4;        //Refer to table 31, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_101 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_102 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_103 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_104 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_104_temp; //"Capacity Denied Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 1;        //Refer to table 32, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_105 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL reserved_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 32, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_106 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_106_temp; //"Bandwidth Allocation Pending Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 32, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_107 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_107_temp; //"Block Transfer Pending";
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 0;        //Refer to table 32, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_108 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_108_temp; //"Radio Failure Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 3;        //Refer to table 32, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_109 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_110 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_111 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_112 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_112_temp; //"Secondary Analog Channel Fixed Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 5;        //Refer to table 28, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_113 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_113_temp; //"Tertiary Analog Channel Fixed Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 5;        //Refer to table 28, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_114 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_114_temp; //"Quaternary Analog Channel Fixed Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 5;        //Refer to table 28, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_115 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_115_temp; //"Quinary Analog Channel Fixed Map;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

#if __TOKVER__ >= 800                // REDEFINE not supported in Tok Ver 6
    REDEFINE DEFAULT_VALUE 5;        //Refer to table 28, default mapping
#endif //__TOKVER__ >=  800

}

statusmap_variable_116 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_117 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_118 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_119 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL undefined_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_120 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_120_temp; // Device Specific Status 15-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_121 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_121_temp;  // Device Specific Status 15-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_122 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_122_temp;  // Device Specific Status 15-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_123 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_123_temp; //Device Specific Status 15-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_124 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_124_temp; // Device Specific Status 15-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_125 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_125_temp; // Device Specific Status 15-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_126 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_126_temp; // Device Specific Status 15-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_127 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_127_temp; // Device Specific Status 15-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_128 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_128_temp; // Device Specific Status 16-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_129 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_129_temp; //["Device Specific Status 16-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_130 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_130_temp; // Device Specific Status 16-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_131 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_131_temp; // Device Specific Status 16-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_132 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_132_temp; // Device Specific Status 16-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_133 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_133_temp; // Device Specific Status 16-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_134 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_134_temp; // Device Specific Status 16-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_135 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_135_temp; // Device Specific Status 16-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_136 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_136_temp; // Device Specific Status 17-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_137 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_137_temp; // Device Specific Status 17-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_138 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_138_temp; // Device Specific Status 17-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_139 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_139_temp; // Device Specific Status 17-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_140 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_140_temp; // Device Specific Status 17-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_141 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_141_temp; // Device Specific Status 17-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_142 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_142_temp; // Device Specific Status 17-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_143 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_143_temp; // Device Specific Status 17-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_144 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_144_temp; // Device Specific Status 18-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_145 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_145_temp; // Device Specific Status 18-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_146 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_146_temp; // Device Specific Status 18-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_147 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_147_temp; // Device Specific Status 18-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_148 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_148_temp; // Device Specific Status 18-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_149 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_149_temp; // Device Specific Status 18-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_150 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_150_temp; // Device Specific Status 18-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_151 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_151_temp; // Device Specific Status 18-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_152 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_152_temp; // Device Specific Status 19-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_153 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_153_temp; // Device Specific Status 19-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_154 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_154_temp; // Device Specific Status 19-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_155 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_155_temp; // Device Specific Status 19-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_156 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_156_temp; //[fld_dev_stat18_5];
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_157 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_157_temp; // Device Specific Status 19-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_158 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_158_temp; // Device Specific Status 19-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_159 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_159_temp; // Device Specific Status 19-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_160 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_160_temp; // Device Specific Status 20-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_161 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_161_temp; // Device Specific Status 20-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_162 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_162_temp; // Device Specific Status 20-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_163 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_163_temp; // Device Specific Status 20-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_164 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_164_temp; // Device Specific Status 20-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_165 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_165_temp; // Device Specific Status 20-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_166 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_166_temp; // Device Specific Status 20-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_167 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_167_temp; // Device Specific Status 20-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_168 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_168_temp; // Device Specific Status 21-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_169 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_169_temp; // Device Specific Status 21-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_170 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_170_temp; // Device Specific Status 21-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_171 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_171_temp; // Device Specific Status 21-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_172 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_172_temp; // Device Specific Status 21-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_173 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_173_temp; // Device Specific Status 21-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_174 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_174_temp; // Device Specific Status 21-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_175 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_175_temp; // Device Specific Status 21-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_176 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_176_temp; // Device Specific Status 22-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_177 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_177_temp; // Device Specific Status 22-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_178 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_178_temp; // Device Specific Status 22-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_179 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_179_temp; // Device Specific Status 22-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_180 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_180_temp; // Device Specific Status 22-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_181 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_181_temp; // Device Specific Status 22-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_182 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_182_temp; // Device Specific Status 22-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_183 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_183_temp; // Device Specific Status 22-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_184 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_184_temp; // Device Specific Status 23-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_185 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_185_temp; // Device Specific Status 23-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_186 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_186_temp; // Device Specific Status 23-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_187 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_187_temp; // Device Specific Status 23-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_188 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_188_temp; // Device Specific Status 23-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_189 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_189_temp; // Device Specific Status 23-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_190 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_190_temp; // Device Specific Status 23-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_191 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_191_temp; // Device Specific Status 23-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_192 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_192_temp; // Device Specific Status 24-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_193 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_193_temp; // Device Specific Status 24-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_194 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_194_temp; // Device Specific Status 24-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_195 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_195_temp; // Device Specific Status 24-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_196 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_196_temp; // Device Specific Status 24-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_197 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_197_temp; // Device Specific Status 24-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_198 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_198_temp; // Device Specific Status 24-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_199 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_199_temp; // Device Specific Status 24-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_200 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_200_temp; // Device Specific Status 25-1 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_201 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_201_temp; // Device Specific Status 25-2 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_202 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_202_temp; // Device Specific Status 25-3 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_203 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_203_temp; // Device Specific Status 25-4 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_204 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_204_temp; // Device Specific Status 25-5 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_205 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_205_temp; // Device Specific Status 25-6 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_206 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_206_temp; // Device Specific Status 25-7 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

statusmap_variable_207 LIKE VARIABLE statusmap_codes // Refer to table 70
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL statusmap_variable_207_temp; // Device Specific Status 25-8 Map
//Remove new dictionary references from legacy binary format
    REDEFINE HELP statusmap_variable_help_temp;
    REDEFINE CLASS DEVICE;

}

/*************************************************************************************
 * Used in CMD 526
 */
status_simulate_mode LIKE VARIABLE status_simulate_mode_codes  //  refer to table 71
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL status_simulate_mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP status_simulate_mode_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING WRITE;
}

/*************************************************************************************
 * Used in CMD 527
 */
simulate_bit_value LIKE VARIABLE status_simulate_value_codes  //  refer to table 72
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL status_simulate_value_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP status_simulate_value_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING WRITE;
}

/*************************************************************************************
 * Sub-Device Assignment Variables
 *
 * Used in CMDs 529 & 531
 */

/*************************************************************************************
 * Used in CMD 529
 */
subdev_assign_status LIKE VARIABLE subdevice_assignment_status_codes // Refer to table 73
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL subdevice_assignment_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP subdevice_assignment_status_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Used in CMD 531
 */
subdevice_transfer_code LIKE VARIABLE subdevice_assignment_transfer_codes // Refer to table 74
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL subdevice_assignment_transfer_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP subdevice_assignment_transfer_help_temp;
    REDEFINE CLASS DEVICE;
    REDEFINE HANDLING WRITE;
}

/*************************************************************************************
 * Join Status Variables
 *
 * Used in CMD 769
 */

wireless_mode LIKE VARIABLE wireless_operation_mode_codes // Refer to table 51
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL wireless_mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP wireless_mode_help_temp;
    REDEFINE CLASS DEVICE & DYNAMIC;
    REDEFINE HANDLING READ;
}

join_status LIKE VARIABLE std_join_process_status_codes // Refer to table 52
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL join_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP join_status_help_temp;
    REDEFINE CLASS DEVICE & DYNAMIC;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Join Mode Variable
 *
 * Used in CMDs 771-772
 */
join_mode LIKE VARIABLE join_mode_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL join_mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP join_mode_help_temp;
    REDEFINE CLASS HART;
}

/*************************************************************************************
 * Wireless Device Capabilities Variables
 *
 * Used in CMD 777
 */

wireless_capability LIKE VARIABLE wireless_capability_flag_codes //  refer to table 75
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL wireless_capability_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP wireless_capability_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

power_source LIKE VARIABLE power_source_codes // Refer to table 44 - Device Power Source
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL power_source_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP power_source_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Report Device Health Variable
 *
 * Used in CMD 779
 */
device_power_status LIKE VARIABLE device_power_status_codes // Refer to table 58 - Device Power Status
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL power_status_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP power_status_help_temp;
    REDEFINE CLASS DEVICE & DYNAMIC;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * Network Access Mode Variable
 *
 * Used in CMDs 821-822
 */
network_access_mode LIKE VARIABLE network_access_mode_codes
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL network_access_mode_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP network_access_mode_help_temp;
    REDEFINE CLASS HART;
}

/*************************************************************************************
 * Wireless Module Revision Variables
 *
 * Used in CMD 64,512
 */
 
/*************************************************************************************
 * The wireless device type uses the same list of devices as the device type.
 */
wireless_module_device_type LIKE VARIABLE device_type_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL wireless_module_device_type_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP wireless_module_device_type_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

/*************************************************************************************
 * The wireless manufacturer code uses the same list of manufacturers as the device type.
 */
wireless_module_manufacturer_id LIKE VARIABLE company_identification_code
{
//Remove new dictionary references from legacy binary format
    REDEFINE LABEL wireless_module_manufacturer_id_temp;
//Remove new dictionary references from legacy binary format
    REDEFINE HELP wireless_module_manufacturer_id_help_temp;
    REDEFINE CLASS HART;
    REDEFINE HANDLING READ;
}

